<?php
/**
 * stanislav.elbl, 27.08.2015
 */

namespace RonPortal\Dochazka;

use Ron\Http\HtmlHeaders;
use Ron\MVC\BrowserController;
use Ron\MVC\JsonOutputObject;
use Ron\TextManager;
use RonPortal\NeniPravoException;
use RonPortal\Obdobi;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\RonPortalDBRecord;
use RonPortal\RonPortalSettings;


class PuvodniDochazkaController extends BrowserController
{
    function __construct($database)
    {
        parent::__construct($database, 'dochazkazamestnance',
            function() { return new PuvodniDochazkaBrowser($this ->database); }
        );
    }


    private function nastavTerminal()
    {
        $result = new JsonOutputObject();
        try {
            if (RonPortalSettings::getIntValue('vkladani_zpusob_terminal') != 4) {
                // neni povolen rucni vyber terminalu
                throw new \Exception('Zmena terminalu neni povolena!');
            }

            $timestamp = $_POST['datumCasOperace'];

            if (Obdobi::jeObdobiUzavreno($this ->database, 'ADS', $timestamp, $timestamp)) {
                throw new \Exception('Obdobi je jiz uzavreno!');
            }

            if (!isset($_POST['terminal'])) {
                throw new \Exception('Chyba - kod terminalu musi byt zadan!');
            }

            if (!isset($_POST['osoba'])) {
                throw new \Exception('Chyba - neni zadana osoba');
            }

            $pracovniPomer = PracovniPomer::nactiPP($this ->database, $_POST['osoba']);

            Prava::nastavObdobiProPrava($timestamp);
            if (!Prava::lzeZobrazitPP('vkladani', $pracovniPomer ->idPP)) {
                throw new NeniPravoException();
            }

            $rec = new RonPortalDBRecord($this ->database, 'PuvodniDochazka');
            $rec ->logDescription = 'PuvodniDochazkaController';
            $rec ->addField('DatumCasOperace', 'timestamp', true, 0, $timestamp);

            $rec ->addField('RC', '', true, 15, $pracovniPomer ->rc);
            $rec ->addField('OsobniCislo', '', true, 15, $pracovniPomer ->osobniCislo);
            $rec ->addField('DatumNastupu', 'timestamp', true, 0, $pracovniPomer ->datumNastupu);
            $rec ->addField('', '', true, 0, '   ' . $pracovniPomer ->jmenoPrijmeniKratke);

            $rec ->addField('KodOperace', '', true, 10, $_POST['kodOperace']);
            $rec ->addField('VlozenoManualne', 'int', true, 0, 1);
            $rec ->addField('Terminal', '', false, 10, $_POST['terminal']);

            $params = $this ->database ->createParameters();
            $rec ->setWhere(
                'RC = ' . $params ->addParameter($pracovniPomer ->rc) .
                ' AND OsobniCislo = ' . $params ->addParameter($pracovniPomer ->osobniCislo) .
                ' AND DatumNastupu = ' . $params ->addParameter($this ->database ->dateToParam($pracovniPomer ->datumNastupu)) .
                ' AND DatumCasOperace = ' . $params ->addParameter($this ->database ->timestampToParam($timestamp)) .
                ' AND KodOperace = ' . $params ->addParameter($_POST['kodOperace']),
                $params
            );

            if (!$rec ->loadRecord()) {
                throw new \Exception('Chyba - zaznam nebyl nalezen!');
            }

            $rec ->updateRecord();
        } catch (\Exception $e) {
            $result ->addError($e);
        }

        return $result;
    }

    public function handleAction($action)
    {
        if (isset($_GET['action']) && $_GET['action'] == 'nastavterminal') {
            return $this ->nastavTerminal();
        }

        TextManager::getInstance() ->loadStrings('dochazka.ini');
        HtmlHeaders::getInstance()
            ->addScript('script/dochazka.js')
            ->addCss('css/dochazka.css')
            ->setAppTitle(TextManager::_('Dochazka_title'));

        return parent::handleAction($action);
    }

}


return new PuvodniDochazkaController($this ->database);
