<?php
/**
 * stanislav.elbl, 13.05.2020
 * Monitorovani vyroby
 */

namespace RonPortal\Vyroba;

use Ron\DateUtils;
use Ron\DB\Database;
use Ron\Http\HtmlHeaders;
use Ron\MessageManager;
use Ron\MVC\BaseViewObject;
use Ron\MVC\JsonController;
use Ron\MVC\JsonOutputObject;
use Ron\MVC\Model;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Controller\PritomnostController;
use RonPortal\NeniPravoException;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\System\FotoOutputObject;


class MonitorovaniVyrobyController extends PritomnostController
{
    /**
     * MonitorovaniVyrobyController constructor.
     * @param Database $database
     */
    public function __construct(Database $database)
    {
        parent::__construct($database, Prava::vratPrava('vyroba'));
        $this->database = $database;
    }


    /**
     * @param JsonOutputObject $output
     */
    function vlozPuvodni($output)
    {
        Prava::nastavObdobiProPrava(time());

        $f = new VkladaniVyrobyFormular($this->database);
        if (!$f ->handleAction()) {
            $output ->addMessages(MessageManager::getInstance());
        }
    }


    /**
     * @param JsonOutputObject $output
     * @throws NeniPravoException
     * @throws \Exception
     * @throws \Ron\DB\DBException
     */
    function loadDetailData($output)
    {
        $pp = PracovniPomer::nactiPP($this ->database, (int)$_GET['idpp']);
        if (!Prava::lzeZobrazitPP('vyroba', $pp ->idPP)) {
            throw new NeniPravoException();
        }

        $params = $this ->database ->createParameters();
        $cursor = $this ->database ->getSqlResult(
            'SELECT PuvodniVyroba.DatumCas, PuvodniVyroba.KodVyrobniCinnosti, VyrobniCinnost.Popis, PuvodniVyroba.Terminal' .
            ' FROM PuvodniVyroba, VyrobniCinnost' .
            ' WHERE VyrobniCinnost.Kod = PuvodniVyroba.KodVyrobniCinnosti' .
            ' AND PuvodniVyroba.RC = ' . $params ->addParameter($pp ->rc) .
            ' AND PuvodniVyroba.OsobniCislo = ' . $params ->addParameter($pp ->osobniCislo) .
            ' AND PuvodniVyroba.DatumNastupu = ' . $params ->addParameter($this ->database ->dateToParam($pp ->datumNastupu)) .
            ' AND PuvodniVyroba.DatumCas >= ' . $params ->addParameter($this ->database ->dateToParam(DateUtils::incMinute(time(), -12 * 60))) .
            ' ORDER BY PuvodniVyroba.DatumCas',
            $params
        );

        $data = [];
        while ($row = $cursor ->fetch()) {
            $time = $this ->database ->stringToTimestamp($row[0]);
            $data[] = [
                'datum' => StringUtils::format('date', $time),
                'cas' => StringUtils::format('time', $time),
                'kodVyrobniCinnosti' => StringUtils::jsonText($row[1]),
                'popis' => StringUtils::jsonText($row[2]),
                'terminal' => StringUtils::jsonText($row[3])
            ];
        }

        $output ->addData('osobniCislo', StringUtils::jsonText($pp ->osobniCislo));
        $output ->addData('idPP', $pp ->idPP);
        if ($pp ->idFotografie) {
            $output->addData('idFotografie', $pp->idFotografie);
        }
        $output ->addData('puvodniVyroba', $data);

        $cursor ->close();
    }


    function handleJsonAction($action, $output)
    {
        switch ($_GET['action']) {
            case 'vlozpuvodni' :
                $this ->vlozPuvodni($output);
                break;

            case 'detaildata' :
                $this ->loadDetailData($output);
                break;

            default :
                parent::handleJsonAction($action, $output); // pritomnost
                //throw new \Exception('Chyba v MonitorovaniVyrobyController: Neznama akce! (' . $_GET['action'] . ')');
        }
    }


    function foto()
    {
        $idpp = (int)$_GET['idpp'];

        Prava::nastavObdobiProPrava(time(), time());

        if (!Prava::lzeZobrazitPP('vyroba', $idpp)) {
            throw new NeniPravoException();
        }

        $result = new FotoOutputObject($this->database);
        $result->nactiIdFotkyPracovnihoPomeru($_GET['idpp']);
        return $result;
    }


    public function handleAction($action)
    {
        TextManager::getInstance() ->loadStrings('dochazka.ini');
        TextManager::getInstance() ->loadStrings('vyroba.ini');

        Prava::nastavObdobiProPrava(time());

        if (isset($_GET['action'])) {
            if ($_GET['action'] == "foto") {
                return $this ->foto();
            }

            return parent::handleAction($action);
        }

        HtmlHeaders::getInstance() ->addScript('script/vyroba.js');
        HtmlHeaders::getInstance() ->addCss('css/dochazka.css');

        $b = new MonitorovaniVyrobyBrowser($this ->database);
        $b ->load();

        if ($b ->zobrazitVkladani) {
            $f = new VkladaniVyrobyFormular($this->database);

            Model::addViewObject($f);
        }

        Model::addViewObject(new BaseViewObject('ronportal/vyroba/view_detail_monitorovani.php'));

        return $b;
    }
}

return new MonitorovaniVyrobyController($this ->database);
