<?php
/**
 * UTF8
 * pavel.rudol, 28.12.2017
 */

namespace RonPortal\Editace;

use Exception;
use Ron\Controls\ComboBox;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\Http\HtmlHeaders;
use Ron\MessageManager;
use Ron\MVC\BaseViewObject;
use Ron\MVC\HttpController;
use Ron\MVC\JsonOutputObject;
use Ron\MVC\Model;
use Ron\MVC\Redirect;
use Ron\StringUtils;
use Ron\TextManager;
use Ron\XmlLoader;
use RonPortal\Dochazka\SchvaleniDochazky;
use RonPortal\NeniPravoException;
use RonPortal\PracovniPomer;
use RonPortal\RonPortalSession;
use RonPortal\Obdobi;
use RonPortal\Prava;
use RonPortal\System\AgentManager;
use RonPortal\RonPortalDBRecord;
use RonPortal\RonPortalSettings;
use RonPortal\User;
use function Ron\ronText;

class RozdeleniMVController extends HttpController
{
    /** @var Database */
    private $database;

    /** @var PracovniPomer */
    private $pracovniPomer;
    /** @var Obdobi */
    private $obdobi;
    private $nastaveniEditace;

    /**
     * RozdeleniMVController constructor.
     * @param Database $database
     */
    public function __construct($database)
    {
        $this->database = $database;
    }

    /**
     * @param Database $database
     * @param Obdobi $obdobi
     * @param PracovniPomer $pracovniPomer
     * @throws Exception
     */
    public static function lzeEditovat($database, $obdobi, $pracovniPomer)
    {
        if ($obdobi->uzavreno) {
            throw new Exception(ronText('Obdobi_uzavreno'));
        }

        $schvaleniDochazky = new SchvaleniDochazky();
        $schvaleniDochazky->load($database, $pracovniPomer->rc, $pracovniPomer->osobniCislo, $pracovniPomer->datumNastupu, $obdobi->zacatek, $obdobi->konec);
        if (!$schvaleniDochazky->lzeEditovatPP($pracovniPomer->idPP, $obdobi->konec))
            throw new Exception('Nelze editovat, dochazka je jiz schvalena');
    }

    /**
     * @throws DBException
     * @throws Exception
     */
    private function nactiNastaveni()
    {
        $params = $this->database->createParameters();
        $sql = "SELECT Val FROM SYS_BVAR_GLOBAL WHERE Name = " . $params->addParameter('RonPortal_Rozdeleni_mv');

        $cursor = $this->database->getSqlResult($sql, $params);

        $row = $cursor->fetch();

        $text = $row[0];

        $loader = new XmlLoader();
        $loader->includeAttributes = true;
        $loader->prefixToSubarray = true;
        $loader->ignorePrefixes = true;
        $loader->convertFromUtf8 = true;
        $this->nastaveniEditace = $loader->readXml($text);
    }

    /**
     * @throws Exception
     */
    private function nactiOsobuObdobi()
    {
        $idpp = $_POST['osoba'] ?? $_GET['osoba'] ?? $_GET['idpp'] ?? 0; // oprava SE - umoznuje idpp zadat v URL parametru osoba nebo idpp
        $pp = PracovniPomer::nactiPP($this->database, $idpp);

        $pocatekObdobi = isset($_POST['obdobi']) ? DateUtils::urlStringToDate($_POST['obdobi']) :
            (isset($_GET['obdobi']) ? DateUtils::urlStringToDate($_GET['obdobi']) : DateUtils::firstDayOfMonth(time()));
        $obdobi = new Obdobi();
        $obdobi ->nactiObdobi($this ->database, 'ADS', $pocatekObdobi);
        
        $this->pracovniPomer = $pp;
        $this->obdobi = $obdobi;
    }

    /**
     * @throws Exception
     */
    private function zkontrolujPrava()
    {
        $prava = Prava::vratPrava('editace');

        $jePravoEditovat = $prava ->lzeZobrazitPP($this->pracovniPomer->idPP, $this->obdobi->zacatek, $this->obdobi->konec);
        if (!$jePravoEditovat) {
            // nemuze editovat, jeste zkontroluje, jestli ma pravo editovat rozdeleni vysledku
            $prava = Prava::vratPrava('editace_r');
            $jePravoEditovat = $prava ->lzeZobrazitPP($this->pracovniPomer->idPP, $this->obdobi->zacatek, $this->obdobi->konec);
        }
        
        if (!$jePravoEditovat) {
            throw new NeniPravoException();
        }
    }

    /**
     * @param bool $zobrazPost
     * @return BaseViewObject
     * @throws Exception
     * @throws DBException
     */
    private function zobrazFormular($zobrazPost = false)
    {
        $this->nactiNastaveni();

        $mesicniData = RozdeleniMV::nactiData($this->database, $this->pracovniPomer, $this->obdobi, $this->nastaveniEditace);

        if (!$zobrazPost) {
            try {
                $mesicniData->zkontrolujSoucty();
            } catch (Exception $exception) {
                MessageManager::addErrorMessage($exception->getMessage());
            }
        }

        $typRozdeleni = $this->nastaveniEditace['typrozdeleni'];

        $paramsCS = $this->database->createParameters();
        $sqlCasovaSlozka =
            "SELECT TypCasoveSlozkyMzdy.Kod, TypCasoveSlozkyMzdy.Popis " .
            "FROM TypCasoveSlozkyMzdy JOIN VlastnostCasoveSlozky ON TypCasoveSlozkyMzdy.Kod = VlastnostCasoveSlozky.Kod " .
            "WHERE VlastnostCasoveSlozky.PlatnostOd <= " . $paramsCS->addParameter($this->database->dateToParam($this->obdobi->konec)) .
            "  AND (VlastnostCasoveSlozky.PlatnostDo >= " . $paramsCS->addParameter($this->database->dateToParam($this->obdobi->zacatek)) . " OR VlastnostCasoveSlozky.PlatnostDo IS NULL) " .
            "  AND VlastnostCasoveSlozky.ManualniEditaceRozdelniMV = 1" .
            "  AND TypCasoveSlozkyMzdy.Kod NOT IN ('" . implode("', '", $mesicniData->kodyCS_R) .  "')";

        $comboCS = new ComboBox('rozdelenimv_casova_slozka');
        $comboCS->loadData($this->database, $sqlCasovaSlozka, $zobrazPost ? $_POST['rozdelenimv_casova_slozka'] : null, $paramsCS);
        $comboCS->htmlParams =
            [
                'class' => 'ronselect',
                'data-columns' => 'text'
            ];

        foreach ($mesicniData->mesicniDataRozdeleni as $mesicniVysledekRozdeleni) {
            $sqlCasoveSlozky =
                "SELECT Kod, Popis " .
                "FROM TypCasoveSlozkyMzdy " .
                "WHERE Kod IN ('" . implode("', '", $mesicniVysledekRozdeleni->kodyCS) . "') ";
            $comboCSRozdeleni = new ComboBox('rozdelenimv_casova_slozka_' . $mesicniVysledekRozdeleni->kodCS);
            $comboCSRozdeleni->loadData($this->database, $sqlCasoveSlozky, $zobrazPost ? $_POST['rozdelenimv_casova_slozka_' . $mesicniVysledekRozdeleni->kodCS] : null);
            $comboCSRozdeleni->htmlParams =
                [
                    'class' => 'ronselect',
                    'data-columns' => 'text'
                ];

            MessageManager::addDebugLog('Combo casove slozky pro souctovou CS ' . $mesicniVysledekRozdeleni->kodCS . ' : ' . $sqlCasoveSlozky);


            Model::addData('combo_casova_slozka_' . $mesicniVysledekRozdeleni->kodCS, $comboCSRozdeleni);
        }

        switch ((int)$this->nastaveniEditace['zvolitrozdeleni']) {
            case 3: //kdykoliv v prubehu obdobi
                $datumOd = $this->obdobi->zacatek;
                $datumDo = $this->obdobi->konec;
                break;
            case 1: // na zacatku obdobi
                $datumOd = $this->obdobi->zacatek;
                $datumDo = $this->obdobi->zacatek;
                break;
            case 2: // na konci obodbi
                $datumOd = $this->obdobi->konec;
                $datumDo = $this->obdobi->konec;
                break;
            case 0: //nevybirat (vychozi hodnota = nenastaveno)
            default:
                $datumOd = null;
                $datumDo = null;
                break;
        }

        $rozdeleni = null;

        if (!is_null($datumOd) && !is_null($datumDo)) {
            $params = $this->database->createParameters();

            $sql =
                "SELECT Rozdeleni FROM RozdeleniOsob
                WHERE RC = " . $params->addParameter($this->pracovniPomer->rc) .
                " AND OsobniCislo = " . $params->addParameter($this->pracovniPomer->osobniCislo) .
                " AND DatumNastupu = " . $params->addDateParameter($this->pracovniPomer->datumNastupu) .
                " AND KodTypuRozdeleni = " . $params->addParameter($typRozdeleni) .
                " AND Hlavni = " . $params->addParameter(1) .
                " AND PlatnostOd <= " . $params->addDateParameter($datumOd) .
                " AND (PlatnostDo >= " . $params->addDateParameter($datumDo) . " OR PlatnostDo IS NULL)";

            $cursor = $this->database->select($sql, $params);

            $row = $cursor->fetch();

            if (isset($row)) {
                $rozdeleni = $row[0];
            }
        }

        Model::addData('rozdeleni_mv', $mesicniData);
        Model::addData('combo_casova_slozka', $comboCS);
        Model::addData('typ_rozdeleni', $typRozdeleni);
        Model::addData('zobraz_post', $zobrazPost);
        Model::addData('rozdeleni_zamestnance', $rozdeleni);

        return new BaseViewObject('ronportal/editace/view_rozdeleni_mv.php');
    }

    /**
     * @return BaseViewObject|Redirect
     * @throws Exception
     * @throws DBException
     */
    private function zpracujPost()
    {
        self::lzeEditovat($this->database, $this->obdobi, $this->pracovniPomer);

        $this->nactiNastaveni();

        try {
            if (RozdeleniMV::zpracujPost($this->database, $this->pracovniPomer, $this->obdobi, $this->nastaveniEditace)) {
                if ((is_null($this->nastaveniEditace['prepocetdochazky'])) || ($this->nastaveniEditace['prepocetdochazky'] == 1)) {
                    AgentManager::prepocetDochazky($this->pracovniPomer->idPP, $this->obdobi->zacatek, $this->obdobi->konec);
                }

                return new Redirect(null, '?menuid=rozdelenimv&osoba=' . $this->pracovniPomer->idPP . '&obdobi=' . DateUtils::dateToUrlString($this->obdobi->zacatek));
            } else {
                return $this->zobrazFormular();
            }
        }
        catch (Exception $exception) {
            MessageManager::addErrorMessage($exception->getMessage());
            return $this->zobrazFormular(true);
        }
    }

    /**
     * @return BaseViewObject|Redirect
     * @throws Exception
     * @throws DBException
     */
    private function smazRadek()
    {
        self::lzeEditovat($this->database, $this->obdobi, $this->pracovniPomer);

        $record = new RonPortalDBRecord($this->database, 'MesicniVysledek_R_E', $this);

        $record->addField('RC', '', true);
        $record->addField('OsobniCislo', '', true);
        $record->addField('DatumNastupu', 'timestamp', true);
        $record->addField('Mesic', 'timestamp', true);
        $record->addField('TypCasoveSlozkyMzdy', '', true);
        $record->addField('KodTypuRozdeleni', 'int', true);
        $record->addField('Rozdeleni', '', true);
        $record->addField('Hodiny', 'float', true);
        $record->addField('Dny', 'float', true);
        $record->addField('KalendarniDny', 'float', true);
        $record->addField('Poznamka', '', true);

        $params = $this->database->createParameters();
        $where =
            "     RC = " . $params->addParameter($this->pracovniPomer->rc) .
            " AND OsobniCislo = " . $params->addParameter($this->pracovniPomer->osobniCislo) .
            " AND DatumNastupu = " . $params->addParameter($this->database->dateToParam($this->pracovniPomer->datumNastupu)) .
            " AND Mesic = " . $params->addParameter($this->database->dateToParam($this->obdobi->zacatek)) .
            " AND TypCasoveSlozkyMzdy = " . $params->addParameter($_GET['casovaslozka']) .
            " AND KodTypuRozdeleni = " . $params->addParameter($_GET['typrozdeleni']) .
            " AND Rozdeleni = " . $params->addParameter($_GET['rozdeleni']);

        $record->setWhere($where, $params);

        if ($record->deleteRecord()) {
            AgentManager::prepocetDochazky($this->pracovniPomer->idPP, $this->obdobi->zacatek, $this->obdobi->konec);
            return new Redirect(null, '?menuid=rozdelenimv&osoba=' . $this->pracovniPomer->idPP . '&obdobi=' . DateUtils::dateToUrlString($this->obdobi ->zacatek));
        }
        else {
            return $this->zobrazFormular();
        }
    }

    /** Vraci seznam rozdeleni pro ronselect
     * @return JsonOutputObject
     * @throws Exception
     */
    private function seznamRozdeleni()
    {
        $output = new JsonOutputObject();

        $this->nactiNastaveni();

        $typRozdeleni = $this->nastaveniEditace['typrozdeleni'];

        if ($this->nastaveniEditace['pouzevlastni'] == 1) {
            $params = $this->database->createParameters();

            $sqlRozdeleni =
                "SELECT Kod, Popis " .
                "FROM Rozdeleni JOIN RozdeleniOsob ON (RozdeleniOsob.KodTypuRozdeleni = Rozdeleni.KodTypuRozdeleni AND RozdeleniOsob.Rozdeleni = Rozdeleni.Kod) " .
                "WHERE RozdeleniOsob.KodTypuRozdeleni = " . $params->addParameter($typRozdeleni)  .
                "  AND RozdeleniOsob.RC = " . $params->addParameter($this->pracovniPomer->rc) .
                "  AND RozdeleniOsob.OsobniCislo = " . $params->addParameter($this->pracovniPomer->osobniCislo) .
                "  AND RozdeleniOsob.DatumNastupu = " . $params->addParameter($this->database->dateToParam($this->pracovniPomer->datumNastupu)) .
                "  AND RozdeleniOsob.PlatnostOd <= " . $params->addParameter($this->database->dateToParam($this->obdobi->konec)) .
                "  AND (RozdeleniOsob.PlatnostDo >= " . $params->addParameter($this->database->dateToParam($this->obdobi->zacatek)) . " OR RozdeleniOsob.PlatnostDo IS NULL)" .
                "  AND DatumVytvoreni <= " . $params->addParameter($this->database->dateToParam($this->obdobi->konec)) .
                "  AND (DatumZruseni >= " . $params->addParameter($this->database->dateToParam($this->obdobi->zacatek)) . " OR DatumZruseni IS NULL)";
        } else {
            $params = $this->database->createParameters();

            $sqlRozdeleni =
                "SELECT Kod, Popis 
                 FROM Rozdeleni 
                 WHERE KodTypuRozdeleni = " . $params->addParameter($typRozdeleni) . " 
                   AND DatumVytvoreni <= " . $params->addParameter($this->database->dateToParam($this->obdobi->konec)) .
                "  AND (DatumZruseni >= " . $params->addParameter($this->database->dateToParam($this->obdobi->zacatek)) . " OR DatumZruseni IS NULL)";
        }

        $sqlRozdeleni .= " ORDER BY Kod";

        MessageManager::addDebugLog('Combo rozdeleni: ' . $sqlRozdeleni);

        $cursor = $this->database->getSqlResult($sqlRozdeleni, $params);

        $seznamRozdeleni = [];
        while ($row = $cursor->fetch()) {
            $rozdeleni = [];
            $rozdeleni['kod'] = StringUtils::jsonText($row[0]);
            $rozdeleni['popis'] = StringUtils::jsonText($row[1]);
            $seznamRozdeleni[] = $rozdeleni;
        }

        $output->addData('items', $seznamRozdeleni);

        return $output;
    }

    /**
     * @param string $action
     * @return BaseViewObject|JsonOutputObject|Redirect
     * @throws Exception
     */
    public function handleAction($action)
    {
        if (!RonPortalSettings::getIntValue(RonPortalSettings::EDITACE_POVOLENA)) {
            throw new Exception('V nastaveni portalu neni povolena editace!');
        }
        TextManager::getInstance()->loadStrings('dochazka.ini');
        HtmlHeaders::getInstance()->addScript('script/dochazka.js');
        HtmlHeaders::getInstance()->addCss('css/dochazka.css');
        HtmlHeaders::getInstance()->setTitle(ronText('RON_Portal') . ' - ' . ronText('Rozdeleni_mesicnich_vysledku'));

        $this->nactiOsobuObdobi();

        Prava::nastavObdobiProPrava( $this->obdobi ->zacatek, $this->obdobi ->konec );

        $this->zkontrolujPrava();
        
        if (isset($_POST['ulozitRozdeleniMV']) && $_POST['ulozitRozdeleniMV']) {
            return $this->zpracujPost();
        }

        if (isset($_GET['action'])) {
            if ($_GET['action'] == 'smazat') {
                return $this->smazRadek();
            }

            if ($_GET['action'] == 'seznamrozdeleni') {
                return $this->seznamRozdeleni();
            }
        }

        return $this->zobrazFormular();

    }

}

return new RozdeleniMVController($this ->database);
