<?php /** @noinspection SqlResolve */

/**
 * User: pavel.rudol
 * Date: 24.8.2018
 */

namespace RonPortal\Vyroba;

use Exception;
use Ron\DB\Database;
use Ron\Http\HtmlHeaders;
use Ron\MVC\BaseViewObject;
use Ron\MVC\HttpController;
use Ron\MVC\InternalRedirect;
use Ron\MVC\OutputObject;
use Ron\MVC\Redirect;
use Ron\MVC\ViewObject;
use Ron\TextManager;
use RonPortal\NeniPravoException;
use RonPortal\Prava;
use RonPortal\PravaNaOsoby;
use RonPortal\RonPortalSettings;

class VyrobniCinnostiController extends HttpController
{
    /** @var Database */
    private $database;

    /** @var PravaNaOsoby */
    private $pravaZobrazit;


    /**
     * @param Database $database
     */
    public function __construct($database)
    {
        $this->database = $database;

        $this->pravaZobrazit = Prava::vratPrava('vyroba');
    }

    /**
     * @return VyrobniCinnostiBrowser
     * @throws Exception
     */
    private function cinnostiBrowser()
    {
        $b = new VyrobniCinnostiBrowser($this->database);

        $b->load();

        return $b;
    }

    /**
     * @param string $formaction
     * @return Redirect|VyrobniCinnostFormular
     * @throws Exception
     */
    private function cinnostFormular($formaction = 'new')
    {
        $f = new VyrobniCinnostFormular($this->database, $formaction);

        if ($f->handleAction()) {
            return new Redirect('', '?menuid=vyrobnicinnosti');
        }

        return $f;
    }

    /**
     * @param string $action
     * @return BaseViewObject|InternalRedirect|OutputObject|Redirect|ViewObject|string
     * @throws Exception
     */
    public function handleAction($action)
    {
        if (!RonPortalSettings::getIntValue('vyroba')) {
            throw new NeniPravoException();
        }

        if (!$this->pravaZobrazit->jsouNejakePrava()) {
            throw new NeniPravoException();
        }

        TextManager::getInstance()->loadStrings('vyroba.ini');
        HtmlHeaders::getInstance()->addCss('css/dochazka.css');
        HtmlHeaders::getInstance()->addScript('script/vyroba.js');

        $formaction = $_POST['formaction'] ?? $_GET['formaction'] ?? '';

        if ($formaction) {
            // formular (editace, insert)
            return $this ->cinnostFormular($formaction);
        }

        return $this->cinnostiBrowser();
    }

}

return new VyrobniCinnostiController($this->database);
