<?php
/**
 * stanislav.elbl, 31.10.2017
 */

namespace RonPortal;

use Ron\DateUtils;
use Ron\DB\Database;
use Ron\Http\Csrf;
use Ron\Http\HtmlHeaders;
use Ron\MessageManager;
use Ron\MVC\BaseViewObject;
use Ron\MVC\JsonController;
use Ron\MVC\JsonOutputObject;
use Ron\MVC\MessageManagerViewObject;
use Ron\MVC\Model;
use Ron\MVC\Redirect;
use Ron\MVC\TextViewObject;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Common\ZastupUzivateleBrowser;
use RonPortal\Common\ZastupUzivateleFormular;
use RonPortal\Controller\LoginAsController;


class RoleProZastup
{
    public $kod;
    public $popis;
    public $poznamka;
}


class ZastupUzivateleInfo
{
    public $id;
    /** @var int */
    public $platnostOd;
    /** @var int */
    public $platnostDo;


    /** @var string  RON cislo zastupujiciho */
    public $zastupujici;
    /** @var string  Jmeno zastupujiciho */
    public $zastupujiciInfo;

    /** @var string  RON cislo zastupovaneho */
    public $zastupovany;
    /** @var string  Jmeno zastupovaneho */
    public $zastupovanyInfo;

    /** @var int */
    public $potvrzeni;
    /** @var int */
    public $casPotvrzeni;

    public $poznamka;
    public $vyjadreniZastupujiciho;
    public $zaznamAktivity;

    /** @var RoleProZastup[] */
    public $role = [];

    /**
     * @param Database $database
     * @param array $row
     * @return ZastupUzivateleInfo
     */
    private static function nactiRadek($database, $row)
    {
        $zastup = new ZastupUzivateleInfo();
        $zastup ->id = $row[0];
        $zastup ->zastupovany = $row[1];
        $zastup ->zastupovanyInfo = $row[10] . ' ' . $row[11];
        $zastup ->zastupujici = $row[2];
        $zastup ->zastupujiciInfo = isset($row[13]) ? $row[12] . ' ' . $row[13] : null;
        $zastup ->platnostOd = $database ->stringToTimestamp($row[3]);
        $zastup ->platnostDo = $database ->stringToTimestamp($row[4]);
        $zastup ->poznamka = $row[5];
        $zastup ->potvrzeni = $row[6];
        $zastup ->casPotvrzeni = $database ->stringToTimestamp($row[7]);
        $zastup ->vyjadreniZastupujiciho = $row[8];
        $zastup ->zaznamAktivity = $row[9];

        return $zastup;
    }


    /**
     * @param Database $database
     * @param int $id
     * @return ZastupUzivateleInfo
     */
    public static function nactiZastup($database, $id)
    {
        $params = $database ->createParameters();
        $sql =
            'SELECT ZastupUzivatele.Id, Zastupovany, Zastupujici, PlatnostOd, PlatnostDo, ' . // 0..4
            $database ->castWideMemo('ZastupUzivatele.Poznamka') . ', ' . // 5
            'Potvrzeni, CasPotvrzeni, ' . $database ->castWideMemo('VyjadreniZastupujiciho') . ', ' . // 6..8
            $database ->castWideMemo('ZaznamAktivity') . ', ' . // 9
            'Zastupovany.Jmeno, Zastupovany.Prijmeni, ' . // 10,11
            'Zastupujici.Jmeno, Zastupujici.Prijmeni' .   // 12,13
            ' FROM ZastupUzivatele, Osoba Zastupovany, Osoba Zastupujici' .
            ' WHERE ZastupUzivatele.Zastupovany = Zastupovany.RC' .
            ' AND ZastupUzivatele.Zastupujici = Zastupujici.RC' .
            ' AND ZastupUzivatele.Id = ' . $params ->addParameter($id);

        $cursor = $database ->getSqlResult($sql, $params);
        if ($row = $cursor ->fetch()) {
            $result = self::nactiRadek($database, $row);
        } else {
            $result = null;
        }
        $cursor ->close();

        $result ->nactiRole($database);

        return $result;
    }


    /**
     * @param Database $database
     */
    public function nactiRole($database)
    {
        $this ->role = [];
        $params = $database ->createParameters();
        $sql =
            'SELECT Kod, Popis, ' . $database ->castWideMemo('Poznamka') .
            ' FROM RoleProZastupUzivatele, VybraneRoleProZastup' .
            ' WHERE RoleProZastupUzivatele.Kod = VybraneRoleProZastup.KodRole' .
            ' AND VybraneRoleProZastup.IdZastupu = ' . $params ->addParameter($this ->id) .
            ' ORDER BY Popis';

        $cursor = $database ->select($sql, $params);
        while ($row = $cursor ->fetch()) {
            $role = new RoleProZastup();
            $role ->kod = $row[0];
            $role ->popis = $row[1];
            $role ->poznamka = $row[2];

            $this ->role[] = $role;
        }
    }


    /**
     * @param Database $database
     * @param string $rcZastupujici
     * @param string $rcZastupovany
     * @return ZastupUzivateleInfo[]
     */
    public static function nactiAktualniZastupy($database, $rcZastupujici, $rcZastupovany)
    {
        /** @var ZastupUzivateleInfo[] $result */
        $result = [];

        $params = $database ->createParameters();
        $sql =
            'SELECT ZastupUzivatele.Id, Zastupovany, Zastupujici, PlatnostOd, PlatnostDo, ' . // 0..4
            $database ->castWideMemo('ZastupUzivatele.Poznamka') . ', ' . // 5
            'Potvrzeni, CasPotvrzeni, ' . $database ->castWideMemo('VyjadreniZastupujiciho') . ', ' . // 6..8
            $database ->castWideMemo('ZaznamAktivity') . ', ' . // 9
            'Zastupovany.Jmeno, Zastupovany.Prijmeni' . // 10,11
            ' FROM ZastupUzivatele, Osoba Zastupovany' .
            ' WHERE ZastupUzivatele.Zastupovany = Zastupovany.RC' .
            ' AND Zastupujici = ' . $params ->addParameter($rcZastupujici) .
            ' AND Zastupovany = ' . $params ->addParameter($rcZastupovany) .
            ' AND PlatnostOd <= ' . $params ->addParameter($database ->dateToParam(time())) .
            ' AND (PlatnostDo >= ' . $params ->addParameter($database ->dateToParam(time())) . ' OR PlatnostDo IS NULL)';

        $cursor = $database ->getSqlResult($sql, $params);
        while ($row = $cursor ->fetch()) {
            $result[] = self::nactiRadek($database, $row);
        }
        $cursor ->close();

        foreach ($result as $zastup) {
            $zastup ->nactiRole($database);
        }

        return $result;
    }
}


class ZastupyController extends JsonController
{
    /**
     * @var Database
     */
    private $database;

    /**
     * ZastupyController constructor.
     * @param Database $database
     */
    public function __construct(Database $database)
    {
        $this->database = $database;
    }


    public function novaZadost($formAction, $vlastni = true)
    {
        if (isset($_POST['cizi_zastup']) && (int)$_POST['cizi_zastup']) {
            $vlastni = false;
        }

        $f = new ZastupUzivateleFormular($this ->database, $vlastni, $formAction);
        if ($f ->handleAction()) {
            return new Redirect(null, '?menuid=zastupuzivatele');
        }

        return $f;
    }


    public static function getActivityLogText($text)
    {
        return date('j.n.Y G:i:s') . ' (' . User::getLoginName() . ') ' . $text . "\n";
    }


    /**
     * Prida zadany text do pole ZaznamAktivity k zaznamu ZastupUzivatele
     * @param int $idZastupu
     * @param string $text
     * @throws \Exception
     */
    function addActivityLog($idZastupu, $text)
    {
        $params = $this ->database ->createParameters();
        $sql =
            'SELECT ' . $this ->database  ->castWideMemo('ZaznamAktivity') .
            ' FROM ZastupUzivatele ' .
            ' WHERE Id = ' . $params ->addParameter($idZastupu);
        $cursor = $this ->database ->select($sql, $params);
        $row = $cursor ->fetch();
        $cursor ->close();
        if (!$row) {
            throw new \Exception("Chyba v metode addActivityLog: pozadovany zaznam nebyl nalezen!");
        }

        $newText = $row[0] . self::getActivityLogText($text);

        $params = $this ->database ->createParameters();
        $this ->database ->sql(
            'UPDATE ZastupUzivatele' .
            ' SET ZaznamAktivity = ' . $params ->addParameter($newText) .
            ' WHERE Id = ' . $params ->addParameter($idZastupu),
            $params);
    }

    /**
     * @param int $idZastupu
     * @return Redirect
     * @throws \Exception
     * @throws \Ron\DB\DBException
     */
    function loginAs($idZastupu)
    {
        try {
            Csrf::validateCSRFGetParam('zastup_login_' . (int)$idZastupu);

            $params = $this->database->createParameters();
            $sql =
                'SELECT Zastupovany, Zastupujici, PlatnostOd, PlatnostDo' .
                ' FROM ZastupUzivatele' .
                ' WHERE Id = ' . $params->addParameter($idZastupu);

            $cursor = $this->database->getSqlResult($sql, $params);
            $row = $cursor->fetch();
            $cursor->close();

            if (!$row) {
                throw new \Exception('Zadany zaznam zastupu nebyl nalezen!');
            }

            // kontrola, jestli je to zaznam pro aktualniho (skutecneho!) uzivatele
            if ($row[1] != (User::jeZastup() ? User::getRealUserRC() : User::getRC())) {
                throw new NeniPravoException();
            }

            // kontrola platnosti od - do
            $platnostDo = $this->database->stringToTimestamp($row[3]);

            if ($this->database->stringToTimestamp($row[2]) > time() || ($platnostDo && $platnostDo < DateUtils::today())) {
                throw new \Exception(\Ron\ronText('Zaznam_zastupu_neni_platny'));
            }
        } catch (\Exception $e) {
            MessageManager::addErrorMessage($e);
            return new Redirect(null, '?menuid=zastupuzivatele');
        }

        // zaloguje prihlaseni
        $this ->addActivityLog($idZastupu, \Ron\ronText('Prihlaseni_uzivatele'));

        // provede "prihlaseni jako" - nastaveni session atd.
        LoginAsController::loginAsUser($this ->database, null, $row[0]);
        User::setZastup($idZastupu, $platnostDo);

        Prava::nactiPravaUzivatele($this ->database);

        Log::addRecordToLog($this->database,
            sprintf(TextManager::_('Login_as_log'), User::getRealLoginName(), User::getLoginName()),
            TextManager::_('Login_log_category'),
            sprintf(TextManager::_('Login_as_log_comment'), User::getUserName())
        );


        return new Redirect(null, '?menuid=zastupuzivatele&action=loggedas&id=' . $idZastupu);
    }


    function loggedAs()
    {
        $zastupy = ZastupUzivateleInfo::nactiAktualniZastupy($this ->database, User::getRealUserRC(), User::getRC());
        Model::addData('aktualniZastupy', $zastupy);

        return new BaseViewObject('ronportal/common/view_zastup_uzivatele.php');
    }


    /**
     * Zobrazeni podrobnosti zastupu - aktualni uzivatel zastupuje, ale neni "prihlasen jako"
     * @param int $id
     * @return BaseViewObject
     * @throws \Exception
     */
    function zastupujiInfo($id)
    {
        $zastup = ZastupUzivateleInfo::nactiZastup($this ->database, $id);
        if (!$zastup) {
            throw new \Exception('Pozadovany zaznam zastupu nebyl nalezen!');
        }

        if (!strcasecmp(User::getRC(), $zastup ->zastupujici)) {
            // zaznam pro me (jsem zastupujici)
        } else if (!strcasecmp(User::getRC(), $zastup ->zastupovany)) {
            // jsem zastupovany
        } else if (!Prava::vratPrava('zast_uz_komu') ->lzeZobrazitRC($zastup ->zastupovany)) {
            // neni to zaznam "pro me" a nemam na nej pravo
            throw new NeniPravoException();
        }

        Model::addData('aktualniZastupy', [$zastup]);

        return new BaseViewObject('ronportal/common/view_zastup_uzivatele.php');
    }


    function zapisVyjadreni($id, $text)
    {
        $rec = new RonPortalDBRecord($this ->database, 'ZastupUzivatele');
        $rec ->addField('Id', 'int', true);
        $rec ->addField('Zastupovany', '', true);
        $rec ->addField('Zastupujici', '', true);
        $rec ->addField('PlatnostOd', 'date', true);
        $rec ->addField('PlatnostDo', 'date', true);
        $rec ->addField('VyjadreniZastupujiciho', 'memo', false, 0, $text);

        $rec->setXmlEvents(RonPortalXMLEvents::ZASTUP_UZIVATELE_BEFORE_POST, RonPortalXMLEvents::ZASTUP_UZIVATELE_AFTER_POST);

        $potvrzeni = 0;
        $logTextId = 'log_zastupu_editace_vyjadreni';

        if (isset($_POST['potvrdit'])  &&  $_POST['potvrdit']) {
            $potvrzeni = 1;
            $logTextId = 'log_zastupu_potvrzeni';
        } else if (isset($_POST['zamitnout'])  &&  $_POST['zamitnout']) {
            $potvrzeni = 2;
            $logTextId = 'log_zastupu_zamitnuti';
        }

        if ($potvrzeni) {
            $rec ->addField('Potvrzeni', 'int', false, 0, $potvrzeni);
            $rec ->addField('CasPotvrzeni', 'timestamp', false, 0, time()) ->setUpdateLog(true);
        }

        $fZaznamAktivity = $rec ->addField('ZaznamAktivity', 'memo', false);
        $fZaznamAktivity ->updateLog = true;

        $params = $this ->database ->createParameters();
        $rec ->setWhere('Id = ' . $params ->addParameter((int)$id), $params);

        if (!$rec ->loadRecord()) {
            throw new \Exception('Chyba v ZastupyController::zapisVyjadreni - zaznam nenalezen!');
        }

        if ($rec['#Potvrzeni'] == 3) {
            throw new \Exception('Chyba - nelze nastavit potvrzeni zruseneho zaznamu!');
        }

        $userRC = User::jeZastup() ? User::getRealUserRC() : User::getRC();
        if ($rec['Zastupujici'] != $userRC) {
            // aktualni skutecny uzivatel neni zastupujici - nemuze napsat vyjadreni
            throw new NeniPravoException();
        }

        Csrf::validateCSRFPostParam('zastup_vyjadreni_' . (int)$id);

        $fZaznamAktivity ->setNewValue($fZaznamAktivity ->originalValue . self::getActivityLogText(\Ron\ronText($logTextId)));

        $rec ->updateRecord();
    }


    function zrusitZastup($id)
    {
        try {
            $rec = new RonPortalDBRecord($this->database, 'ZastupUzivatele');
            $rec->addField('Id', 'int', true);
            $rec->addField('Zastupovany', '', true);
            $rec->addField('Zastupujici', '', true);
            $rec->addField('PlatnostOd', 'date', true);
            $rec->addField('PlatnostDo', 'date', true);

            $rec->addField('Potvrzeni', 'int', false, 0, 3); // 3 = zruseno
            $rec->addField('CasPotvrzeni', 'timestamp', false, 0, time())->setUpdateLog(true);

            $rec->addField('VyjadreniZastupujiciho', 'memo', true);
            $fZaznamAktivity = $rec->addField('ZaznamAktivity', 'memo', false);
            $fZaznamAktivity->updateLog = true;

            $rec->setXmlEvents(RonPortalXMLEvents::ZASTUP_UZIVATELE_BEFORE_POST, RonPortalXMLEvents::ZASTUP_UZIVATELE_AFTER_POST);

            $params = $this->database->createParameters();
            $rec->setWhere('Id = ' . $params->addParameter((int)$id), $params);

            if (!$rec->loadRecord()) {
                throw new \Exception('Chyba v ZastupyController::zapisVyjadreni - zaznam nenalezen!');
            }

            $prava = Prava::vratPrava('zast_uz_komu');
            if (!$prava->lzeZobrazitRC($rec['Zastupovany'])) {
                throw new NeniPravoException();
            }

            Csrf::validateCSRFGetParam('zrusit_zastup_' . $id);

            $fZaznamAktivity->setNewValue($fZaznamAktivity->originalValue . self::getActivityLogText(\Ron\ronText('log_zastupu_zruseni')));

            $rec->updateRecord();
        } catch (\Exception $e) {
            MessageManager::addErrorMessage($e);
        }

        return new Redirect(null, '?menuid=zastupuzivatele&record=' . $id);
    }


    /**
     * @param JsonOutputObject $output
     */
    public function mojeZastupy($output)
    {
        $htmlZastupuji = '';
        $htmlJsemZastupovan = '';

        $params = $this ->database ->createParameters();
        $sql =
            'SELECT ZastupUzivatele.Id, ' . //  ZastupUzivatele.Role, RoleProZastupUzivatele.Popis, ' . // 0
            '  ZastupUzivatele.PlatnostOd, ZastupUzivatele.PlatnostDo, ZastupUzivatele.Potvrzeni, ZastupUzivatele.CasPotvrzeni, ' . // 1..4
            '  Zastupovany.RC, Zastupovany.Jmeno, Zastupovany.Prijmeni, ' . // 5..7
            '  Zastupujici.RC, Zastupujici.Jmeno, Zastupujici.Prijmeni ' . // 8..10
            ' FROM ZastupUzivatele' .
            ' INNER JOIN Osoba Zastupovany ON Zastupovany.RC = ZastupUzivatele.Zastupovany' .
            ' INNER JOIN Osoba Zastupujici ON Zastupujici.RC = ZastupUzivatele.Zastupujici' .
            ' WHERE (ZastupUzivatele.PlatnostDo >= ' . $params ->addParameter($this ->database ->dateToParam(time())) . ' OR ZastupUzivatele.PlatnostDo IS NULL) ' .
            '  AND (ZastupUzivatele.Zastupovany = ' . $params ->addParameter(User::getRC()) .
                '  OR  ZastupUzivatele.Zastupujici = ' . $params ->addParameter(User::getRC()) . ')' .
            ' ORDER BY PlatnostOd DESC';

        $cursor = $this ->database ->getSqlResult($sql, $params);
        while ($row = $cursor ->fetch()) {
            if (User::hasRC($row[8])) {
                // ja zastupuji
                $htmlZastupuji .=
                    '<tr>' .
                    '<td>' . StringUtils::format('finedate', $cursor ->stringToTimestamp($row[1])) . ' - ' . StringUtils::format('finedate', $cursor ->stringToTimestamp($row[2])) . '</td>' . // platnost od-do
                    '<td>' . StringUtils::htmlTextConvert($row[6]) . ' ' . StringUtils::htmlTextConvert($row[7]) . '</td>' . // jmeno
                    "<td><a href='?menuid=zastupuzivatele&record={$row[0]}'>" . \Ron\ronText('Podrobnosti').'</a></td>' .
                    '</tr>';
            } else if (User::hasRC($row[5])) {
                // jsem zastupovan
                $htmlJsemZastupovan .=
                    '<tr>' .
                    '<td>' . StringUtils::format('finedate', $cursor ->stringToTimestamp($row[1])) . ' - ' . StringUtils::format('finedate', $cursor ->stringToTimestamp($row[2])) . '</td>' . // platnost od-do
                    '<td>' . StringUtils::htmlTextConvert($row[9]) . ' ' . StringUtils::htmlTextConvert($row[10]) . '</td>' . // jmeno
                    "<td><a href='?menuid=zastupuzivatele&record={$row[0]}'>" . \Ron\ronText('Podrobnosti').'</a></td>' .
                    '</tr>';
            }
        }

        if ($htmlZastupuji  ||  $htmlJsemZastupovan) {
            $html =
                '<table>';

            if ($htmlZastupuji) {
                $html .= '<tr><td colspan="3">' . \Ron\ronText('Vy_zastupujete_uzivatele') . '</td></tr>' . $htmlZastupuji;
            }

            if ($htmlJsemZastupovan) {
                $html .= '<tr><td colspan="3">' . \Ron\ronText('Zastupuje_vas_uzivatel') . '</td></tr>' . $htmlJsemZastupovan;
            }

            $html .= '</table>';
        } else {
            $html = \Ron\ronText('Nemate_naplanovan_zadny_zastup');
        }

        $output ->addText('html', $html);
    }


    function handleJsonAction($action, $output)
    {
        switch ($_GET['action']) {
            case 'mojezastupy' : // json - dashboard
                $this ->mojeZastupy($output);
                break;
        }
    }


    function seznamHeader($mojeZastupy)
    {
        Model::addViewObject( new TextViewObject('<h1>' . \Ron\ronText('Seznam_zastupu_uzivatelu') . '</h1>'));

        if (!Prava::vratPrava('zast_uz_komu') ->pouzeSebe()) {
            Model::addViewObject(new TextViewObject(
                '<ul class="linkslist">' .
                '<li' . ($mojeZastupy ? ' class="selectedlink"' : '') . '><a href="?menuid=zastupuzivatele">' . \Ron\ronText('Moje_zastupy') . '</a></li>' .
                '<li' . (!$mojeZastupy ? ' class="selectedlink"' : '') . '><a href="?menuid=zastupuzivatele&action=seznam">' . \Ron\ronText('Zastupy_vsech_uzivatelu') . '</a></li>' .
                '</ul>'
            ));
        }

        Model::addViewObject(new MessageManagerViewObject());
    }


    function seznamZastupu()
    {
        $this ->seznamHeader(false);

        $prava = Prava::vratPrava('zast_uz_komu');

        if (!$prava ->jsouNejakePrava()) {
            throw new NeniPravoException();
        }

        if ($prava ->pouzeSebe()) {
            // ma pravo jen na sebe => redirect na vlastni
            return new Redirect(null, '?menuid=zastupuzivatele');
        }

        $b = new ZastupUzivateleBrowser($this ->database, ZastupUzivateleBrowser::VSECHNY_ZASTUPY);
        $b ->load();
        return $b;
    }


    public function handleAction($action)
    {
        TextManager::getInstance() ->loadStrings('users.ini');
        HtmlHeaders::getInstance() ->addCss('css/zastupuzivatele.css');

        $this ->addTranslationsScript('zastupuzivatele');

        Prava::nastavObdobiProPrava(time());

        if (isset($_GET['action']) && ($_GET['action'] === 'zapisvyjadreni')) {
            // editace vyjadreni - bud v rezimu "prihlaseny jako", nebo z prehledu zastupu
            try {
                $this->zapisVyjadreni((int)$_POST['idZastupu'], $_POST['vyjadreni']);
            } catch (\Exception $e) {
                MessageManager::addErrorMessage($e);
                return new Redirect(null, '?menuid=zastupuzivatele&action=zastupuji&record=' . (int)$_POST['idZastupu']);
            }

            if (User::jeZastup()) {
                return new Redirect(null, '?menuid=zastupuzivatele');
            } else {
                return new Redirect(null, '?menuid=zastupuzivatele&action=zastupuji&record=' . $_POST['idZastupu']);
            }
        }

        if (User::jeZastup()) {
            return $this ->loggedAs();
        }

        if (isset($_GET['action'])) {
            switch ($_GET['action']) {
                case 'translations' :
                    return $this ->translations();

                case 'novazadost' :
                    return $this ->novaZadost('new');

                case 'novyzastup' :
                    return $this ->novaZadost('new', false);

                case 'loggedas' :
                    return $this ->loggedAs();

                case 'loginas' :
                    return $this ->loginAs((int)$_GET['id']);

                case 'mojezastupy' : // json - dashboard
                    return parent::handleAction($action);

                case 'zrusitzastup' :
                    return $this ->zrusitZastup((int)$_GET['id']);

                case 'seznam' :
                    return $this ->seznamZastupu();
            }
        }

        if (isset($_GET['formaction'])  &&  $_GET['formaction'] === 'edit') {
            return $this ->novaZadost('edit');
        }

        if (isset($_GET['record'])) {
            return $this ->zastupujiInfo((int)$_GET['record']);
        }

        $this ->seznamHeader(true);
        //Model::addViewObject( new TextViewObject('<h1>' . \Ron\ronText('Seznam_mych_zastupu') . '</h1>'));

        $b = new ZastupUzivateleBrowser($this ->database, ZastupUzivateleBrowser::JA_ZASTUPUJI);
        $b ->load();
        Model::addViewObject($b);

        $b = new ZastupUzivateleBrowser($this ->database, ZastupUzivateleBrowser::JSEM_ZASTUPOVAN);
        $b ->load();
        return $b;
    }

}

return new ZastupyController($this ->database);
