<?php
/**
 * stanislav.elbl, 15.03.2015
 */


namespace Ron\DB;


class MysqliDatabase extends Database
{

    /**
     * @var \mysqli $connection
     */
    private $connection;

    private $connectionString;
    private $dbName;

    private $codePage;


    public function __construct($connectionString, $dbName)
    {
        $this->connectionString = $connectionString;
        $this->dbName = $dbName;
    }


    /**
     * @return mixed
     */
    public function getConnectionString()
    {
        return $this->connectionString;
    }

    /**
     * @return mixed
     */
    public function getDbName()
    {
        return $this->dbName;
    }

    /**
     * @return mixed
     */
    public function getCodePage()
    {
        return $this->codePage;
    }

    /**
     * @param mixed $codePage
     */
    public function setCodePage($codePage)
    {
        $this->codePage = $codePage;
    }



    /**
     * @return string
     */
    public function getDbType()
    {
        return 'mysql';
    }

    /**
     * @param string $user
     * @param string $password
     * @throws DBException
     */
    public function connect($user, $password)
    {
        $this ->connection = new \mysqli($this ->connectionString, $user, $password, $this ->dbName);
        if ($this ->connection ->connect_error) {
            throw new DBException($this, '', \Ron\ronText('Connection_fail'));
        }

        if ($this ->codePage) {
            $this ->connection ->query("SET NAMES '{$this ->codePage}'");
        }
    }


    /**
     * @return bool
     */
    public function close()
    {
        return $this ->connection ->close();
    }


    public function createParameters()
    {
        return new PlaceholderParameters($this);
    }


    /**
     * @param string $sqlText
     * @param PlaceholderParameters $params
     * @return Cursor
     * @throws DBException
     */
    public function executeSelect($sqlText, $params = null)
    {
        if ($params) {
            $stmt = $this ->connection ->prepare($sqlText);

            $paramArray = [ $stmt, str_repeat('s', count($params ->data)) ];
            foreach($params ->data as $index => $param) {
                $paramArray[] = &$params ->data[$index]; // by reference!
            }

            if (!call_user_func_array('mysqli_stmt_bind_param', $paramArray)) {
                throw new DBException($this, $sqlText);
            }

            if (!$stmt ->execute()) {
                throw new DBException($this, $sqlText);
            }

            $data = $stmt->get_result();
        } else {
            $data = $this->connection->query($sqlText);
        }

        if (!$data) {
            throw new DBException($this, $sqlText);
        }

        return new MysqliCursor($this, $data);
    }

    /**
     * @param string $sqlText
     * @param PlaceholderParameters $params
     * @throws DBException
     */
    public function executeSql($sqlText, $params = null)
    {
        if ($params) {
            $stmt = $this ->connection ->prepare($sqlText);

            $paramArray = [ $stmt, str_repeat('s', count($params ->data)) ];
            foreach($params ->data as $index => $param) {
                $paramArray[] = &$params ->data[$index]; // by reference!
            }

            $result = call_user_func_array('mysqli_stmt_bind_param', $paramArray);

            if ($result) $result = $stmt ->execute();
        } else {
            $result = $this->connection->query($sqlText);
        }

        if (!$result) {
            throw new DBException($this, $sqlText);
        }
    }


    function lastAutoincrementValue() {
        return $this ->connection ->insert_id;
    }


    /**
     * @return string
     */
    public function lastErrorText()
    {
        return $this ->connection ->error;
    }

    /**
     * @return bool
     */
    public function beginTransaction()
    {
        // TODO: Implement beginTransaction() method.
    }

    /**
     * @return bool
     */
    public function commitTransaction()
    {
        // TODO: Implement commitTransaction() method.
    }

    /**
     * @return bool
     */
    public function rollbackTransaction()
    {
        // TODO: Implement rollbackTransaction() method.
    }

    /**
     * @param string $str
     * @return string
     */
    public function addSlashes($str)
    {
        return $this ->connection ->real_escape_string($str);
    }

    /**
     * @param string $text
     * @return string;
     */
    public function castWideMemo($text)
    {
        return $text;
    }

    /**
     * @param string $str
     * @return int|null
     */
    public function stringToTimestamp($str)
    {
        if ( !isset($str) ) return null;
        if ( $str === '' ) return null;

        //echo "prevadim: $str <BR> ";
        // Maximalni hodnota pro cas je 19.1.1938 - vetsi hodnoty PHP nezvlada
        if (preg_match ("/2(\d\d\d)/", $str, $m) && ($m[1] >= 38))
        {
            return 0x7fffffff;
        }

        // Pokusi se prevest datum primo. Pokud se to nepovede tak funkce
        // vraci -1 a musi se to prechrchlat
        $result = strtotime($str);
        return $result;
    }

    /**
     * @param int $timestamp
     * @return string
     */
    public function timestampToString($timestamp)
    {
        return "'" . date("Y-m-d H:i:s", $timestamp) . "'";
    }

    /**
     * Prevede timestamp na sql konstantu
     * @param int $timestamp
     * @return string
     */
    public function dateToString($timestamp)
    {
        return "'" . date("Y-m-d", $timestamp) . "'";
    }

    /**
     * prevede cas na sql konstantu
     * @param int $timestamp
     * @return string
     */
    public function timeToString($timestamp)
    {
        throw new \Exception('MysqliDatabase::timeToString is not implemented!');
        // TODO: Implement timeToString() method.
    }


    public function timestampToParam($timestamp)
    {
        return date('Y-m-d H:i:s', $timestamp);
    }

    public function dateToParam($timestamp)
    {
        return date('Y-m-d', $timestamp);
    }

    public function timeToParam($timestamp)
    {
        throw new \Exception('MysqliDatabase::timeToParam is not implemented!');
        // TODO: Implement timeToParam() method.
    }


    /**
     * prevede retezec z databaze na denni cas - vraci pocet hodin (realne cislo)\
     * napr "... 7:30" => 7.5
     * @param string $str
     * @return integer
     */
    public function stringToTimeOfDay($str)
    {
        throw new \Exception('MysqliDatabase::stringToTimeOfDay is not implemented!');
        // TODO: Implement stringToTimeOfDay() method.
    }


}


class MysqliCursor extends Cursor
{
    /**
     * @var \mysqli_result
     */
    protected $data;


    /**
     * @param MysqliDatabase $database
     * @param \mysqli_result $data
     */
    function __construct($database, $data)
    {
        parent::__construct($database);
        $this->data = $data;
    }


    /**
     * @return resource
     */
    public function getData()
    {
        return $this->data;
    }

    /**
     * @return array|bool
     */
    public function fetch()
    {
        return $this ->data ->fetch_array(MYSQLI_NUM);
    }

    /**
     * Vraci pocet nactenych zaznamu
     * @return int
     */
    public function numRows()
    {
        return $this ->data ->num_rows;
    }

    /**
     * @param int $rowNumber
     * @return bool
     */
    public function seek($rowNumber)
    {
        return $this ->data ->data_seek($rowNumber);
    }

    /**
     * @return bool
     */
    public function last()
    {
        $cnt = $this ->numRows();

        if ($cnt > 0) {
            return $this ->seek($cnt - 1);
        }

        return false;
    }
}




/**
 * @param MysqliDatabase $database
 * @return mixed
 */
function db_lastautoincrementvalue($database) {
    return $database ->lastAutoincrementValue();
}







require('_appdbset.php');

function createDefaultDatabase() {
    global $db_connectionstring;
    global $db_name;

    $database = new MysqliDatabase($db_connectionstring, $db_name);

    global $db_codepage;
    $database ->setCodePage( isset($db_codepage) ? $db_codepage : 'cp1250' );

    return $database;
}

