<?php

namespace Ron\Controls;

use Exception;
use Ron\DateUtils;
use Ron\StringUtils;
use Ron\TextManager;

class EditField
{
    public $name; // jmeno pole v HTML formulari

    public $format;
    public $readOnly;     // priznak jestli je jen pro cteni

    public $styl;         // zakladni css styl
    public $skrytNuly;    // priznak, jestli se maji skryt nulove hodnoty
    public $sirkaPole;

    public $hodnota;      // datova hodnota (cislo)
    public $text;         // datova hodnota (text)
    public $puvodniText;  // puvodni datova hodnota (napr. nactena z databaze)
    public $chyba;        // priznak chyby (napr. spatny format cisla)

    //var $f_Limit;       // limity hodnot pro jednotlive sloupce

    public function __construct()
    {
        $this->skrytNuly = true;
    }


    public function isTimeFormat()
    {
        return $this->format === 'hodiny';
    }


    public function isFloatFormat()
    {
        switch ($this->format) {
            case 'float' :
            case 'number0' :
            case 'number1' :
            case 'number_1' :
            case 'number2' :
            case 'number_2' :
            case 'curr' :
            return true;
        }

        return false;
    }


    public function vratTextProHodnotu($hodnota)
    {
        if (!isset($hodnota)) {
            return ''; // hodnota vubec neni nastavena
        }

        if (!$hodnota && $this->skrytNuly) {
            return ''; // nulova hodnota se nema zobrazovat => prazdny text
        }

        return StringUtils::format($this->format, $hodnota);
    }



    /**
     * Prevede text zadany uzivatelem na hodnotu
     * @param string $text
     * @return false|float|int|string
     * @throws Exception
     */
    public function vratHodnotuProText($text)
    {
        if ($text == '') {
            if ($this ->isTimeFormat()  ||  $this ->isFloatFormat()) {
                return 0;
            }

            return '';
        }

        if ($this->isTimeFormat()) {
            return DateUtils::timeStringToFloat($text, true); // povolitZkraceny = true
        }

        if ($this ->isFloatFormat()) {
            if (StringUtils::isFloat($text)) {
                return StringUtils::stringToFloat($text);
            }
            throw new Exception( TextManager::_('Bad_number_format'));
        }

        return false;
    }


    public function setValue($hodnota)
    {
        $this->hodnota = $hodnota;

        $this->text = $this->vratTextProHodnotu($hodnota);
        $this->puvodniText = $this->text;

        $this->chyba = false;
    }


    /**
     * Vraci HTML pro zobrazeni
     * @return string
     */
    public function getFieldHTML()
    {
        $text = $this->text;

        if ($this->readOnly) {
            return StringUtils::htmlTextConvert($text); // jen zobrazit
        }

        // editace
        $styleStr =
            'text-align:right;' .
            ($this->sirkaPole ? " width:{$this -> sirkaPole}px;" : '') .
            // pokud je hodnota editovana (puvodni hodnota z DB je jina nez hodnota zadana uzivatelem), zobrazi ji tucne
            (($this->puvodniText !== $this->text) ? ' font-weight:bold;' : '');

        $className =
            ($this->isTimeFormat() ? 'inputTime' : 'inputFloat') .
            ($this->chyba ? ' error' : '');

        $readonlyStr = $this->readOnly ? "readonly='readonly'" : '';

        return
            "<input type='text' name='{$this -> name}' value='" . StringUtils::htmlTextConvert($text) . "' $readonlyStr" .
            " class='$className' style='$styleStr' maxlength='10' />";
    }


    /**
     * Nacte text zadany uzivatelem. Zaroven se jej pokusi prevest na hodnotu.
     * @param bool $throwException
     * @return bool
     * @throws Exception
     */
    public function nactiPost($throwException = false)
    {
        if ($this->readOnly) {
            return true;
        }

        if (isset($this->name, $_POST[$this->name])) {
            $this->text = trim($_POST[$this->name]);
            try {
                $this ->hodnota = $this->vratHodnotuProText($this->text);
            } catch (Exception $e) {
                $this ->chyba = true;
                if ($throwException) {
                    throw $e;
                }

                return false;
            }
        } else {
            $this->text = '';
        }

        return true;
    }
}
