<?php
/**
 * stanislav.elbl, 21.02.2021
 */

namespace Ron\Controls;

use Ron\DB\Database;
use Ron\DB\Parameters;
use Ron\DB\DBException;
use Ron\DB\SqlStatement;
use Ron\StringUtils;

class Option
{
    /** @var mixed      Hodnota polozky (pri renderovani <option> se pouzije pro value) */
    public $key;

    /** @var string     Zobrazeny text */
    public $caption;

    /** @var string     Vygeneruje se jako HTML atribut title */
    public $title;

    /** @var string     Vygeneruje se jako HTML atribut class */
    public $cssClass;

    /**
     * Asociativni pole dalsich atributu (napr. data)
     * @var array
     */
    var $params = [];

    function __construct($key, $caption)
    {
        $this ->key = $key;
        $this ->caption = $caption;
    }

    /**
     * @param mixed $key
     * @return $this
     */
    public function setKey($key)
    {
        $this->key = $key;
        return $this;
    }

    /**
     * @param string $caption
     * @return $this
     */
    public function setCaption(string $caption)
    {
        $this->caption = $caption;
        return $this;
    }

    /**
     * @param string|null $title
     * @return $this
     */
    public function setTitle(?string $title)
    {
        $this->title = $title;
        return $this;
    }

    /**
     * @param string|null $cssClass
     * @return $this
     */
    public function setCssClass(?string $cssClass)
    {
        $this->cssClass = $cssClass;
        return $this;
    }

    /**
     * @param array $params
     * @return $this
     */
    public function setParams($params)
    {
        $this->params = $params;
        return $this;
    }


    public function loadParams($attrIndexes, $data)
    {
        if ($attrIndexes) {
            if (!$this ->params) {
                $this ->params = [];
            }

            foreach ($attrIndexes as $attrName => $index) {
                $this ->params[$attrName] = $data[$index] ?? '';
            }
        }
    }


    function getJson()
    {
        $result = [
            'key' => StringUtils::jsonText($this ->key),
            'value' => StringUtils::jsonText($this ->caption)
        ];

        if ($this ->cssClass) {
            $result['cssClass'] = $this ->cssClass;
        }

        if ($this ->title) {
            $result['title'] = StringUtils::jsonText($this ->title);
        }

        foreach ($this ->params as $paramName => $paramValue) {
            $result[$paramName] = StringUtils::jsonText($paramValue);
        }

        return $result;
    }


    public static function createItem($row, $keyIndex = 0, $captionIndex = 1, $titleIndex = null, $cssClassIndex = null)
    {
        $option = new static($row[$keyIndex], $row[$captionIndex]);

        if (isset($titleIndex)) {
            $option ->title = $row[$titleIndex];
        }

        if (isset($cssClassIndex)) {
            $option ->cssClass = $row[$cssClassIndex];
        }

        return $option;
    }


    /**
     * Nacte polozky ze zadaneho dotazu.
     * @param Database $database
     * @param string|SqlStatement $sql
     * @param Parameters $sqlParams
     * @param int $keyIndex Index pole s klicem v sql dotazu
     * @param int $captionIndex Index pole s hodnotou v sql dotazu
     * @param null $emptyValue
     * @return Option[]
     * @throws DBException
     */
    public static function loadSQL($database, $sql, $sqlParams, $keyIndex = 0, $captionIndex = 1, $emptyValue = null) //, $titleIndex = null, $cssClassIndex = null, $attrIndexes = null, $preferredValues = null, $filterFunction = null)
    {
        $result = [];

        if (isset($emptyValue)) {
            // prida polozku pro "emptyValue"
            $result[] = new Option('', $emptyValue);
        }

        $cursor = $database ->getSqlResult($sql, $sqlParams);

        while ($row = $cursor ->fetch()) {
                $result[] = static::createItem($row, $keyIndex, $captionIndex);
        }

        $cursor ->close();

        return $result;
    }


    /**
     * @param Database $database
     * @param string|SqlStatement   $sql
     * @param Parameters    $sqlParams
     * @param ICreateItem   $createItemInterface
     * @return array
     */
    public static function loadSQLCustomCreate($database, $sql, $sqlParams, $createItemInterface) // , $emptyValue = false
    {
        $result = [];

        $cursor = $database ->getSqlResult($sql, $sqlParams);

        while ($row = $cursor ->fetch()) {
            $result[] = $createItemInterface ->createItem($row);
        }

        $cursor ->close();

        return $result;
    }
}
