<?php
/**
 * stanislav.elbl, 02.07.2021 17:22
 */

namespace Ron\Controls;

use Ron\DB\Database;
use Ron\StringUtils;
use Ron\TextManager;

class Preferences
{
    const FORMAT_INI = 'ini';
    const FORMAT_TEXT = 'text';

    var $caption;
    /** @var string     slouzi pro rozliseni jednotlivych stranek nastaveni */
    var $name;

    /** @var string     kategorie */
    var $category;

    /** @var string     nazev souboru s texty, ktery se ma nacist */
    var $strings;

    var $index;

    /** @var string     kde se maji ukladat - resi si to aplikace */
    var $storage;
    /** @var string     ini (default) / text */
    var $format;

    /**
     * @var Preference[]
     */
    var $items = [];

    public function __construct()
    {
        $this ->format = 'ini';
    }

    /**
     * Nacte polozky ze zadaneho pole.
     * @param $initArray
     */
    public function initItems($initArray)
    {
        foreach ($initArray as $itemArray) {
            if (!isset($itemArray['disabled']) || !$itemArray['disabled']) {
                $item = new Preference();
                $item->init($itemArray);

                $this->items[] = $item;
            }
        }
    }


    /**
     * Nacte nastaveni ze zadaneho pole
     * @param $initArray
     */
    public function init($initArray)
    {
        foreach ($initArray as $key => $value) {
            switch ($key) {
                case 'items' :
                    $this ->initItems($value);
                    break;

                default :
                    $this ->{$key} = $value;
            }
        }
    }


    /**
     * Vraci polozku zadaneho jmena
     * @param string $name
     * @return null|Preference
     */
    public function findItem($name)
    {
        foreach ($this ->items as $item) {
            if ($item ->name == $name) {
                return $item;
            }
        }

        return null;
    }


    public function setItemValue($itemName, $value, $convertMemoLines = true, $storage = null)
    {
        $item = $this ->findItem($itemName);

        if ($item  &&  (!isset($storage)  ||  ($storage == $this ->storage  &&  !$item->storage)  || $storage == $item ->storage) ) {
            if ($convertMemoLines  &&  $item ->type == Preference::MEMO) {
                $item->value = str_replace('\n', "\n", $value);
            } else {
                $item->value = $value;
            }
        }
    }


    /**
     * Nacte hodnoty polozek z retezce s radky ve tvaru item_name=hodnota
     * @param string|string[] $preferencesText
     * @param string $storage
     */
    public function loadItemValues($preferencesText, $storage)
    {
        if (is_array($preferencesText)) {
            foreach ($preferencesText as $key => $value) {
                $this->setItemValue($key, $value, false, $storage);
            }
        } else {
            $str_settings = explode("\n", $preferencesText);

            foreach ($str_settings as $setStr) {
                $pom = sscanf($setStr, '%[^=]=%[^$]'); // , &$key, &$value
                $key = strtolower(trim(isset($pom[0]) ? $pom[0] : ''));
                $value = trim(isset($pom[1]) ? $pom[1] : '');
                $this->setItemValue($key, $value);
            }
        }
    }


    /**
     * Nacte potrebna data z databaze (volby pro SQL_COMBOBOX)
     * @param Database $database
     */
    public function loadData($database)
    {
        foreach ($this ->items as $item) {
            if ($item ->type == Preference::SQL_COMBOBOX  &&  $item ->sql) {
                $item ->options = [];
                if ($item ->enableEmpty) {
                    $item ->options[''] = ' ';
                }

                $cursor = $database ->getSqlResult($item ->sql);
                while ($row = $cursor ->fetch()) {
                    $item ->options[$row[0]] = $row[1];
                }
                $cursor ->close();
            }
        }
    }


    public function loadPost()
    {
        foreach ($this ->items as $item) {
            if ($item ->name  &&  !$item ->readOnly) {
                switch ($item->type) {
                    case Preference::CHECKBOX :
                        $item->value = isset($_POST[$item->name]) ? 1 : 0;
                        break;

                    default :
                        $item ->value = isset($_POST[$item ->name]) ? $_POST[$item ->name] : null;
                        break;
                }
            }
        }
    }


    /**
     * Nacte text nastaveni do pole
     * @param string $preferencesText
     * @return array
     */
    private static function loadItemsToArray($preferencesText)
    {
        $str_settings = explode("\n", $preferencesText);

        $resultArray = [];

        foreach ($str_settings as $setStr) {
            $pom = sscanf($setStr, '%[^=]=%[^$]'); // , &$key, &$value
            $key = isset($pom[0]) ? strtolower(trim($pom[0])) : '';
            $value = isset($pom[1]) ? trim($pom[1]) : '';

            if ($key != '') {   // &&  $key[0] != "\\"    - toto slouzi pro opravu
                $resultArray[$key] = $value;
            }
        }

        return $resultArray;
    }


    private static function arrayToText($preferencesArray)
    {
        $result = '';

        foreach($preferencesArray as $key => $value) {
            $result .= $key . '=' . $value . "\n";
        }

        return $result;
    }


    public function storeItemValues($preferencesText, &$logText, $storage = null)
    {
        $storage = $storage ?: $this ->storage;

        $logText = '';

        // nacte puvodni hodnoty
        $resultArray = self::loadItemsToArray($preferencesText);

        // zapise zmenene hodnoty
        foreach ($this ->items as $item) {
            if ($item  &&  $item ->name  &&  !$item ->readOnly  &&  ($item ->storage == $storage  ||  (!$item ->storage  &&  $storage == $this ->storage)) ) {
                if ($item ->type == Preference::DELETE) {
                    unset($resultArray[$item->name]);
                } else {
                    $newValue = ($item->type == Preference::MEMO)
                        ? str_replace("\n", '\n', str_replace("\r\n", '\n', $item->value))
                        : $item->value;

                    $isDefault = isset($item->defaultValue) && $newValue === $item->defaultValue;

                    if ($isDefault  &&  !isset($resultArray[$item->name])) {
                        // defaultni hodnota se nezmenila
                    } else if ( !isset($resultArray[$item->name]) || $item->value != $resultArray[$item->name]) {
                        // doslo ke zmene
                        $logText .=
                            $item->name . ":\n" .
                            '   puvodni: ' . (isset($resultArray[$item->name]) ? $resultArray[$item->name] : '') . "\n" .
                            '      nova: ' . $newValue . ($isDefault ? ' (default)' : '') . "\n";

                        if ($isDefault) {
                            unset($resultArray[$item->name]);
                        } else {
                            $resultArray[$item->name] = $newValue;
                        }
                    } else if ($isDefault) {
                        // nedoslo ke zmene, ale jedna se o defaultni hodnotu
                        unset($resultArray[$item->name]);
                    }
                }
            }
        }

        return self::arrayToText($resultArray);
//        $result = '';
//
//        foreach($resultArray as $key => $value) {
//            $result .= $key . '=' . $value . "\n";
//        }
//
//        return $result;
    }


    public static function removeItem($preferencesText, $itemName)
    {
        $resultArray = self::loadItemsToArray($preferencesText);
        unset($resultArray[$itemName]);
        return self::arrayToText($resultArray);
    }


    public function renderItems()
    {
        foreach ($this ->items as $item) {
            if ($item ->type == Preference::DELETE) {

            }
            else if ($item ->type == Preference::CATEGORY) {
                echo
                    '<tr>' .
                    '<td class="category" colspan="2">' .
                    TextManager::_($item ->title) .
                    '</td>' .
                    '</tr>';
            } else {
                echo
                    '<tr>' .
                    ($item ->noTitle
                        ? '<td colspan="2">'
                        : '<td class="title">' . TextManager::_($item ->title) . '</td><td>'
                    );

                switch($item ->type) {
                    case Preference::TEXT :
                        if ($item ->readOnly) {
                            echo '<div class="preferencevalue">' . StringUtils::htmlTextConvert($item ->value) . '</div>';
                        } else {
                            echo '<input type="text" name="' . $item ->name . '" value="' . StringUtils::htmlTextConvert($item ->value) . '" ' . $item ->getHtmlParams() . '/>';
                        }
                        break;

                    case Preference::SQL_COMBOBOX : // POZOR! Volby uz musi byt nacteny!
                    case Preference::COMBOBOX :
                        echo '<select name="' . $item ->name . '" ' . $item ->getHtmlParams() . '>';
                        foreach ($item ->options as $key =>$text) {
                            echo '<option value="' . $key . '" ' . ($item ->value == $key ? 'selected="selected"' : '') . '>' . TextManager::_($text) . '</option>';
                        }
                        echo '</select>';
                        break;

                    case Preference::CHECKBOX :
                        if ($item ->readOnly) {
                            echo '<div class="preferencevalue">' . TextManager::_($item ->value ? 'Str_Yes' : 'Str_No') . '</div>';
                        } else {
                            echo '<input type="checkbox" name="' . $item ->name . '" value="1" ' . ($item ->value ? ' checked="checked"' : '') . $item ->getHtmlParams() . '/>';
                        }
                        break;

                    case Preference::MEMO :
                        echo '<textarea name="' . $item ->name . '" ' . $item ->getHtmlParams() . '>' . StringUtils::htmlTextConvert($item ->value) . '</textarea>';
                        break;

                    case Preference::COLOR :
                        echo '<input type="text" class="color-input" data-indicator="#color-indicator-' . $item ->name . '" name="' . $item ->name . '" value="' . StringUtils::htmlTextConvert($item ->value) . '" ' . $item ->getHtmlParams() . '/>';
                        echo '<div class="color-indicator ' . ($item ->indicatorClass ?: '') . '" ';
                        if ($item ->value != '') {
                            echo ' style="background-color: ' . StringUtils::htmlTextConvert($item ->value) . ' !important"';
                        }
                        echo ' id="color-indicator-' . $item ->name . '">&nbsp;</div>';
                        break;
                }

                if ($item ->infoText) {
                    echo '<div class="infoText">' . TextManager::_($item ->infoText) . '</div>';
                }

                echo
                    '</td>' .
                    '</tr>';
            }
        }
    }
}
