<?php

namespace Ron\Controls;

use Ron\DB\Cursor;
use Ron\DB\Database;
use Ron\StringUtils;


interface ShiftFormItem
{
    /**
     * @param string|string[] $columns
     * @return array
     */
    public function getJson($columns);
}

class BaseShiftFormItem implements ShiftFormItem
{
    public function getJson($columns)
    {
        $cols = is_array($columns) ? $columns : explode(';', $columns);

        $result = [];
        foreach ($cols as $columnName) {
            $result[$columnName] = StringUtils::jsonText($this ->{$columnName});
        }

        return $result;
    }
}



class ShiftForm
{
    /** @var  Database */
    protected $database;

    /** @var ShiftFormItem[]    Pole polozek pro levy seznam */
    protected $leftItems;
    /** @var ShiftFormItem[]    Pole polozek pro pravy seznam */
    protected $rightItems;

    /** @var string|string[]    Seznam sloupcu pro levy seznam */
    public $leftColumns;
    /** @var string|string[]    Seznam sloupcu pro pravy seznam */
    public $rightColumns;

    /** @var string  Url pro javascripty  */
    public $url;

    /**
     * ShiftForm constructor.
     * @param Database $database
     */
    public function __construct(Database $database)
    {
        $this->database = $database;
    }


    /**
     * Nacte pole polozek ze zadaneho kurzoru
     * @param Cursor $cursor
     * @param string|string[] $columns
     * @return array
     */
    public static function loadItems($cursor, $columns)
    {
        $cols = is_array($columns) ? $columns : explode(';', $columns);

        $result = [];

        while ($row = $cursor ->fetch()) {
            $item = new BaseShiftFormItem();

            foreach ($cols as $i => $columnName) {
                if ($columnName) {
                    $item->{$columnName} = $row[$i];
                }
            }

            $result[] = $item;
        }

        return $result;
    }


    /**
     * Prevede pole polozek na json pole
     * @param ShiftFormItem[] $items
     * @param string|string[] $cols
     * @return array|null
     */
    public static function getJsonItems($items, $cols)
    {
        if (!$items) {
            return null;
        }

        $cols = is_array($cols) ? $cols : explode(';', $cols);
        $result = [];
        foreach ($items as $item) {
            $result[] = $item ->getJson($cols);
        }

        return $result;
    }


    public function getJsonData()
    {
        $cols = is_array($this ->leftColumns)
            ? $this ->leftColumns
            : explode(';', $this ->leftColumns);

        $leftItems = [];
        foreach ($this ->leftItems as $item) {
            $leftItems[] = $item ->getJson($cols);
        }

        $cols = is_array($this ->rightColumns)
            ? $this ->rightColumns
            : explode(';', $this ->rightColumns);

        $rightItems = [];
        foreach ($this ->rightItems as $item) {
            $rightItems[] = $item ->getJson($cols);
        }

        return [
            'leftColumns' => $this ->leftColumns,
            'leftItems' => $leftItems,

            'rightColumns' => $this ->rightColumns,
            'rightItems' => $rightItems,

            'url' => $this ->url
        ];
    }


    public function loadData()
    {
        return $this ->getJsonData();
    }


    public function handleMoveLeft()
    {
        // vratit: leftaction=add  rightaction=remove
        // mozne hodnoty: add, remove, reload (cely seznam), none
        return [];
    }

    public function handleMoveRight()
    {
        return [];
    }


    public function handleAction()
    {
        switch ($_GET['shiftformaction']) {
            case 'loaddata' :
                return $this ->loadData();

            case 'moveleft' :
                return $this ->handleMoveLeft();
                break;

            case 'moveright' :
                return $this ->handleMoveRight();
                break;
        }

        return null;
    }
}
