<?php
/**
 * stanislav.elbl, 19.02.2021
 */

namespace Ron\Http;

class HttpResponse
{
    /** @var HttpHeaders */
    private $headers;

    private $htmlHead;

    /** @var Cookie[] */
    private $cookies;

    /** @var HttpResponse $httpResponse   Globalni instance */
    protected static $httpResponse;


    public function __construct() {
        $this ->headers = new HttpHeaders();
        $this ->cookies = [];
    }

    /**
     * @return HttpHeaders
     */
    public function getHeaders()
    {
        return $this->headers;
    }

    /**
     * @return Cookie[]
     */
    public function getCookies()
    {
        return $this->cookies;
    }


    public function render() {
        $this ->getHeaders() ->render();

        foreach ($this ->cookies as $cookie) {
            $cookie ->send();
        }
    }


    /**
     * Vraci globalni instanci objektu HttpResponse
     * @return HttpResponse
     */
    public static function getInstance()
    {
        if (!self::$httpResponse) {
            self::$httpResponse = new HttpResponse();
        };

        return self::$httpResponse;
    }


    /**
     * Nastavi header zadaneho jmena na zadanou hodnotu
     * @param string $name
     * @param string $value
     */
    public static function setHeader($name, $value)
    {
        self::getInstance() ->getHeaders() ->setHeader($name, $value);
    }


    /**
     * Prida header zadaneho jmena.
     * @param $name
     * @param $value
     */
    public static function addHeader($name, $value)
    {
        self::getInstance() ->getHeaders() ->addHeader($name, $value);
    }


    public static function renderHeaders()
    {
        self::getInstance() ->render();
    }


    /**
     * Nastavi cookie
     * @param string $name
     * @param string $value
     * @param int    $expire
     * @param string $path
     */
    public static function setCookie($name, $value, $expire = 0, $path = '')
    {
        $cookie = new Cookie($name, $value, $expire, $path);
        self::getInstance() ->cookies[$name] = $cookie;
    }


    /**
     * Nastavi cookie zadaneho jmena a zapise hodnotu i do session. Pokud neni hodnota v GET ani POST zadana, nastavi promennou v session podle existujici cookie.
     * @param string $cookieName    Jmeno cookie
     * @param string $htmlName      Jmeno v GET nebo POST
     * @param bool $post        Priznak, jestli ma hodnotu hledat v POST
     * @param bool $get         Priznak, jestli ma hodnotu hledat v GET
     * @return bool             Vraci true, pokud byla cookie nastavena.
     */
    public static function setCookieAndSession($cookieName, $htmlName, $post, $get)
    {
        if ($post && isset($_POST[$htmlName])) {
            self::setCookie($cookieName, $_POST[$htmlName], time() + 30000000);
            $_SESSION[$cookieName] = $_POST[$htmlName];
            return true;
        } else if ($get && isset($_GET[$htmlName])) {
            self::setCookie($cookieName, $_GET[$htmlName], time() + 30000000);
            $_SESSION[$cookieName] = $_GET[$htmlName];
            return true;
        }

        if (!isset($_SESSION[$cookieName])  &&  isset($_COOKIE[$cookieName])) {
            $_SESSION[$cookieName] = $_COOKIE[$cookieName];
        }

        return false;
    }

}
