<?php
/**
 * stanislav.elbl, 19.02.2021
 */

namespace Ron\MVC;

use Ron\Http\HttpHeaders;
use Ron\Http\HttpResponse;
use Ron\HttpUtils;
use Ron\MessageManager;
use Ron\TextManager;

class Redirect
{
    public $url;
    public $params;


    public function __construct($url, $params = null)
    {
        $this ->url = $url;
        $this ->params = $params;
    }


    public function getRedirectUrl()
    {
        // 24.8.2018 - SE: doplnuji i puvodni kod s $_SERVER['SCRIPT_NAME'], protoze kod s getAbsoluteURL nefunguje,
        // pokud je HTTP server bez HTTP a je za nejakym firewallem, ktery HTTPS zprostredkovava
        // (generuje url s http://, protoze nema jak poznat, ze bezi pres https)
        // Pro kompatibilitu s proxy je potreba do _appdbset.php pridat $proxy_compatible_redirect = true
        global $proxy_compatible_redirect;
        if ($proxy_compatible_redirect) {
            $url = $this->url ? $this->url : $_SERVER['SCRIPT_NAME'];

            if ($this->params) {
                $url .= (($this->params[0] != '?') ? '?' : '') . $this->params;
            }

            return $url;
        }

        $params = $this ->params;
        if ($params && $params[0] != '?') $params = '?' . $params; // doplneno 26.6.2017

        /*
         * 17.3.2016 RON-PR pokud neni zadana URL adresa, pouzije se pro presmerovani absolutni adresa
         * (z duvodu kompatibility s IIS6)
         * v absolutni adrese se sklada ze serveru a nazvu skriptu, ktery se spousti
        */
        if (!$this->url) {
            return HttpUtils::getAbsoluteURL(null, $params);
        }

        return $this ->url . ($params ?: '');

    }


    public function render()
    {
        MessageManager::getInstance() ->storeToSession();

        global $debug_redirect;
        if ($debug_redirect) {
            require('ron/redirect_debug.php');
        } else {
            HttpResponse::setHeader('Content-Type', 'text/html; charset=' . TextManager::getInstance() ->getCharSet());
            HttpResponse::setHeader(HttpHeaders::LOCATION, $this ->getRedirectUrl());
        }
    }
}
