<?php
/**
 * stanislav.elbl, 19.02.2021
 */

namespace Ron\MVC;

/**
 * Zajistuje zobrazeni textu - muze obsahovat i HTML znacky
 */
class TextViewObject implements ViewObject
{
    const ERROR_CLASS = 'errortext';

    /** @var string         Text, ktery se zobrazi. Muze obsahovat HTML znacky. */
    protected $text;

    /** @var null|string    CSS class obalujiciho elementu */
    protected $className;

    /** @var string         Urcuje, do jakeho elementu se obsah obali (muze byt i prazdny) */
    protected $wrapElement;

    /**
     * @param string $text          Text, ktery se ma renderovat
     * @param string $className     CSS trida obalujiciho elementu
     * @param string $wrapElement   Obalujici element
     */
    public function __construct($text, $className = '', $wrapElement = 'div')
    {
        $this ->text = $text;
        $this ->className = $className;
        $this ->wrapElement = $wrapElement;
    }

    /**
     * Vraci zobrazovany text.
     * @return mixed
     */
    public function getText()
    {
        return $this->text;
    }

    /**
     * Nastavi text pro zobrazeni
     * @param mixed $text
     * @return TextViewObject
     */
    public function setText($text)
    {
        $this->text = $text;
        return $this;
    }

    /**
     * Nastavi css class obalujiciho elementu
     * @return null|string
     */
    public function getClassName()
    {
        return $this->className;
    }

    /**
     * @param null|string $className
     * @return TextViewObject
     */
    public function setClassName($className)
    {
        $this->className = $className;
        return $this;
    }

    /**
     * Vraci obalujici HTML element
     * @return mixed|string
     */
    public function getWrapElement()
    {
        return $this->wrapElement;
    }

    /**
     * Nastavi obalujici HTML element
     * @param null|string $wrapElement
     * @return TextViewObject
     */
    public function setWrapElement($wrapElement)
    {
        $this->wrapElement = $wrapElement;
        return $this;
    }


    /**
     * Zobrazi obsah - $text, ktery je pripadne obaleny HTML elementem $wrapElement
     * @return void
     */
    public function render()
    {
        echo
            ($this ->wrapElement
                ? '<' . $this ->wrapElement . ($this ->className ? ' class="' . $this ->className . '"' : '') . '>'
                : '') .
            $this ->text .
            ($this ->wrapElement
                ? '</' . $this ->wrapElement . '>'
                : '');
    }
}
