<?php
/**
 * stanislav.elbl, 24.02.2021
 */

namespace RonPortal\Admin;

use Ron\MessageManager;
use Ron\MVC\BaseViewObject;
use Ron\MVC\HttpController;
use Ron\MVC\Model;
use Ron\StringUtils;
use RonPortal\NeniPravoException;
use RonPortal\Prava;


class LDAPTestController extends HttpController
{
    const LDAP_OPT_DIAGNOSTIC_MESSAGE = LDAP_OPT_ERROR_STRING; // 0x0032;

    public static function getOptionName($option)
    {
        switch ($option) {
            case LDAP_OPT_DEREF :          return 'LDAP_OPT_DEREF';
            case LDAP_OPT_SIZELIMIT :      return 'LDAP_OPT_SIZELIMIT';
            case LDAP_OPT_TIMELIMIT :      return 'LDAP_OPT_TIMELIMIT';
            case LDAP_OPT_NETWORK_TIMEOUT :     return 'LDAP_OPT_NETWORK_TIMEOUT';
            case LDAP_OPT_PROTOCOL_VERSION :    return 'LDAP_OPT_PROTOCOL_VERSION';
            case LDAP_OPT_ERROR_NUMBER :   return 'LDAP_OPT_ERROR_NUMBER';
            case LDAP_OPT_REFERRALS :      return 'LDAP_OPT_REFERRALS';
            case LDAP_OPT_RESTART :        return 'LDAP_OPT_RESTART';
            case LDAP_OPT_HOST_NAME :      return 'LDAP_OPT_HOST_NAME';
            case LDAP_OPT_ERROR_STRING :   return 'LDAP_OPT_ERROR_STRING';
            //case self::LDAP_OPT_DIAGNOSTIC_MESSAGE :  return 'LDAP_OPT_DIAGNOSTIC_MESSAGE'; - totez jako LDAP_OPT_ERROR_STRING
            case LDAP_OPT_MATCHED_DN :     return 'LDAP_OPT_MATCHED_DN';
            case LDAP_OPT_SERVER_CONTROLS :     return 'LDAP_OPT_SERVER_CONTROLS';
            case LDAP_OPT_CLIENT_CONTROLS :     return 'LDAP_OPT_CLIENT_CONTROLS';
            case LDAP_OPT_DEBUG_LEVEL :    return 'LDAP_OPT_DEBUG_LEVEL';
            case LDAP_OPT_X_KEEPALIVE_IDLE :    return 'LDAP_OPT_X_KEEPALIVE_IDLE';
            case LDAP_OPT_X_KEEPALIVE_PROBES :  return 'LDAP_OPT_X_KEEPALIVE_PROBES';
            case LDAP_OPT_X_KEEPALIVE_INTERVAL :  return 'LDAP_OPT_X_KEEPALIVE_INTERVAL';
            case LDAP_OPT_X_TLS_CACERTDIR :     return 'LDAP_OPT_X_TLS_CACERTDIR';
            case LDAP_OPT_X_TLS_CACERTFILE :    return 'LDAP_OPT_X_TLS_CACERTFILE';
            case LDAP_OPT_X_TLS_CERTFILE :      return 'LDAP_OPT_X_TLS_CERTFILE';
            case LDAP_OPT_X_TLS_CIPHER_SUITE :  return 'LDAP_OPT_X_TLS_CIPHER_SUITE';
            case LDAP_OPT_X_TLS_CRLCHECK : return 'LDAP_OPT_X_TLS_CRLCHECK';
            case LDAP_OPT_X_TLS_CRLFILE :  return 'LDAP_OPT_X_TLS_CRLFILE';
            case LDAP_OPT_X_TLS_DHFILE :   return 'LDAP_OPT_X_TLS_DHFILE';
            case LDAP_OPT_X_TLS_KEYFILE :  return 'LDAP_OPT_X_TLS_KEYFILE';
            case LDAP_OPT_X_TLS_PROTOCOL_MIN :  return 'LDAP_OPT_X_TLS_PROTOCOL_MIN';
            case LDAP_OPT_X_TLS_RANDOM_FILE :   return 'LDAP_OPT_X_TLS_RANDOM_FILE';
            case LDAP_OPT_X_TLS_REQUIRE_CERT :  return 'LDAP_OPT_X_TLS_REQUIRE_CERT';

            default : return $option;
        }
    }


    function processLdapData($entry)
    {
        if (!$entry  ||  !$entry['count']) {
            return null;
        }

        $items = [];
        $data = [];
        for ($objectIndex = 0;  $objectIndex < $entry['count'];  $objectIndex++) {
            // projede vsechny nalezene objekty (uzivatele)
            $ldapObject = $entry[$objectIndex];

            $object = [];

            // projede vsechny polozky objektu
            for ($itemIndex = 0;  $itemIndex < $ldapObject['count'];  $itemIndex++) {
                $name = $ldapObject[$itemIndex];
                $value = $name ? $ldapObject[$name] : null;

                if (!isset($items[$name])  ||  !$items[$name]) {
                    $items[$name] = $name;
                }

                $object[$name] = $value;
            }

            $data[] = $object;
        }

        return ['items' => $items, 'data' => $data];
    }

    function testLdap()
    {
        $ds = ldap_connect($_POST['server']);
        if ($ds) {
            MessageManager::addInfoMessage( StringUtils::fromUTF8('Funkce ldap_connect proběhla úspěšně ...') );
            global $ldap_options;
            if (isset($ldap_options))
                foreach ($ldap_options as $option => $value) {
                    if (ldap_set_option($ds, $option, $value)) {
                        MessageManager::addInfoMessage(StringUtils::fromUTF8('Nastavení volby ') . self::getOptionName($option) . ' = ' . $value . StringUtils::fromUTF8(' proběhlo úspěšně.'));
                    } else {
                        MessageManager::addErrorMessage(StringUtils::fromUTF8('Chyba při nastavování volby ') . self::getOptionName($option) . ' = ' . $value . '!');
                        MessageManager::addErrorMessage(StringUtils::fromUTF8('Chyba ') . ldap_errno($ds) . ': ' . ldap_error($ds));
                    }
                }


            if (isset($_POST['tls']) && $_POST['tls']) {
                ldap_start_tls($ds);
            }

            $name = $_POST['prefix'] . $_POST['username'] . $_POST['suffix'];
            MessageManager::addInfoMessage('Testuji uzivatele ' . $name);
            $name = StringUtils::convertToEncoding($name, $GLOBALS['ldap_encoding'] ?? 'UTF-8');
            //ldap_bind($ldap, htmlspecialchars_decode($config['ldap_user']), htmlspecialchars_decode($config['ldap_password']))
            //if (ldap_bind($ds, $name, $_POST['password'])) {
            //if (ldap_bind($ds, $name, StringUtils::toUTF8(htmlspecialchars_decode($_POST['password'])))) {
            //$password = $_POST['password'];
            //$password = StringUtils::toUTF8($_POST['password']);
            //$password = iconv('UTF-8', 'windows-1250', $password);
            $password = StringUtils::convertToEncoding($_POST['password'], $GLOBALS['ldap_encoding'] ?? 'UTF-8');
            if (ldap_bind($ds, $name, $password)) {
                MessageManager::addInfoMessage(StringUtils::fromUTF8('Funkce ldap_bind proběhla úspěšně.'));

//                if ($_POST['dn']) {
//                    $filter = "(objectclass=*)";
//                    MessageManager::addInfoMessage('Načítám data pro DN ' . $_POST['dn']);
//
//                    //$sr=ldap_read($ds, "cn=users,dc=ron,dc=local", $filter);
//                    //$sr=ldap_read($ds, "ou=stanislav.elbl,cn=users,dc=ron,dc=local", $filter);

//                    $sr = ldap_read($ds, $_POST['dn'], $filter); // , $justthese);   ou=stanislav.elbl,
//                    $entry = ldap_get_entries($ds, $sr);
//
//                    Model::addData('ldapEntry', $entry);
//                }

                if ($_POST['dn']) {
                    MessageManager::addInfoMessage(StringUtils::fromUTF8('Načítám data pro DN ') . $_POST['dn']);

                    //$filter = "(objectclass=*)";
                    $filter = $_POST['filter'];

                    $attrs = $_POST['attributes'] ? explode(',', $_POST['attributes']) : null;

                    if ($attrs) {
                        $sr = ldap_search($ds, $_POST['dn'], $filter, $attrs);
                    } else {
                        $sr = ldap_search($ds, $_POST['dn'], $filter); // hodnota $attributes=null vyrabi warning, proto je to samostatne!
                    }

                    if ($sr) {
                        $entry = ldap_get_entries($ds, $sr);
                        Model::addData('ldapEntry', $entry);
                        Model::addData('ldapData', $this ->processLdapData($entry));
                    } else {
                        MessageManager::addErrorMessage('ldap_search - nenalezeno!');
                    }

                }

                $optResult = ldap_get_option($ds, LDAP_OPT_ERROR_STRING, $extended_error);

//                $sr=ldap_search($ds, '', $name, []);
//                var_dump($sr);
//                $info = ldap_get_entries($ds, $sr);
//                var_dump($info);


                if($extended_error) {
                    MessageManager::addErrorMessage('Chyba ' . ldap_errno($ds) . ': ' . ldap_error($ds));
                    MessageManager::addErrorMessage($extended_error);
                }
            } else {
                ldap_get_option($ds, LDAP_OPT_ERROR_STRING, $extended_error);

                if ($_POST['password'] == '') {
                    MessageManager::addErrorMessage(StringUtils::fromUTF8('Uživatel nebyl nalezen!'));
                } else {
                    MessageManager::addErrorMessage(StringUtils::fromUTF8('Špatné jméno nebo heslo uživatele'));
                }

                MessageManager::addErrorMessage(StringUtils::fromUTF8('Chyba ') . ldap_errno($ds) . ': ' . ldap_error($ds));
                if ($extended_error) MessageManager::addErrorMessage($extended_error);
            }
        } else {
            MessageManager::addErrorMessage('Nelze se pripojit k ldap serveru');
            MessageManager::addErrorMessage('Chyba ' . ldap_errno($ds) . ': ' . ldap_error($ds));
        }
    }


    public function handleAction($action)
    {
        if (!Prava::vratPrava('admin') ->jsouNejakePrava()) {
            throw new NeniPravoException();
        }

        try {
            if (isset($_POST['server'])) {
                $this->testLdap();
            }
        } catch (\Exception $e) {
            MessageManager::addErrorMessage($e);
        }

        return new BaseViewObject('view/ldaptestform.php');
    }
}
