<?php
/**
 * Roman.Szablatura RON Software 6.4.2018
 */

namespace RonPortal\Benefity;
namespace RonPortal\Benefity;

use Ron\Controls\BrowserField;
use Ron\Controls\BrowserRowArray;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\SqlStatement;
use Ron\GuiUtils;
use Ron\Http\Csrf;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Controls\FilterFormFilter;
use RonPortal\Controls\FilterFormFilterDatum;
use RonPortal\Controls\FilterFormFilterItem;
use RonPortal\Controls\FilterFormFilterOsoba;
use RonPortal\Controls\JmenoPrijmeniBrowserField;
use RonPortal\Controls\RonPortalBrowser;
use RonPortal\Controls\RonPortalFilterForm;
use RonPortal\Prava;
use RonPortal\PravaNaOsoby;
use RonPortal\Security;
use function Ron\ronText;



/**
 * Class DatumPlatnostiRezervaceField
 * @package RonPortal\Benefity
 */
class DatumPlatnostiRezervaceField extends BrowserField
{
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if ($rowData['col-datumrezervacedo']) { // DatumRezervaceDo
            return ronText('rezervace_do') . ': <span class="rezervace-do">' . parent::getViewText($rowData['col-datumrezervacedo'], $rowNumber, $recordId, $rowData) . '</span>';
        } else if ($rowData['col-datumomezeniplatnosti']) { // DatumOmezeniPlatnosti
            return ronText('platnost') . ': <span class="omezeni-platnosti-do">' . parent::getViewText($rowData['col-datumomezeniplatnosti'], $rowNumber, $recordId, $rowData) . '</span>';
        }

        return '';
    }
}


/**
 * Class DatumSchvaleniField
 * @package RonPortal\Benefity
 */
class DatumSchvaleniField extends BrowserField
{
    /** @var PravaNaOsoby $pravo_schvalit */
    public $pravo_schvalit;

    /** @var PravaNaOsoby $pravo_vznik_bodu */
    public $pravo_vznik_bodu;


    /**
     * DatumSchvaleniField constructor.
     * @param Database $database
     * @param PravaNaOsoby $pravo_schvalit
     * @param PravaNaOsoby $pravo_vznik_bodu
     */
    function __construct($database, $pravo_schvalit, $pravo_vznik_bodu)
    {
        parent::__construct($database);

        $this->pravo_schvalit = $pravo_schvalit;
        $this->pravo_vznik_bodu = $pravo_vznik_bodu;
    }

    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param BrowserRowArray $rowData
     * @return string
     * @throws \Exception
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if ($rowData['col-datumschvaleni']) {
            return parent::getViewText($value, $rowNumber, $recordId, $rowData);
        }

        if ($rowData[2] != 31 && $rowData[2] != 51) { // druh pohybu
            $zobrazitOdkaz = true;
            //Odkaz pro splneni jen pokud je platnostDo nebo RezervaceDo vetsi nez akt. datum
            if ($rowData['col-datumrezervacedo']) { // DatumRezervaceDo
                $zobrazitOdkaz = $this->database->stringToTimestamp($rowData['col-datumrezervacedo']) >= DateUtils::today();
            } else if ($rowData['col-datumomezeniplatnosti']) { // DatumOmezeniPlatnosti
                $zobrazitOdkaz = $this->database->stringToTimestamp($rowData['col-datumomezeniplatnosti']) >= DateUtils::today();
            }

            if ($zobrazitOdkaz) {
                if ($this->pravo_schvalit->lzeZobrazitRC($rowData[1]) || ($this->pravo_vznik_bodu->lzeZobrazitRC($rowData[1]) && $rowData[2] == 11)) {
                    return GuiUtils::changeLocationQuest(ronText('schvalit_benefit'), ronText('Opravdu_schvalit_benefit'),
                        '?menuid=benefity&action=schvalenirezervace&record=' . $rowData[0] . Csrf::csrfUrlParameter('schvalenibenefitu_' . $rowData[0]));
                }
            }

        }

        return '';
    }
}


/**
 * Class DatumCerpaniField
 * @package RonPortal\Benefity
 */
class DatumCerpaniField extends BrowserField
{

    /** @var PravaNaOsoby $pravo_vydat */
    public $pravo_vydat;

    /**
     * DatumCerpaniField constructor.
     * @param null|Database $database
     * @param PravaNaOsoby $pravo
     */
    function __construct($database, $pravo)
    {
        parent::__construct($database);
        $this->pravo_vydat = $pravo;
    }

    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param BrowserRowArray $rowData
     * @return string
     * @throws \Exception
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if ($rowData[2] != 31 && $rowData[2] != 51 && $rowData[2] != 11) {
            if ($rowData['col-datumcerpani']) { // DatumCerpani => jiz cerpano
                return parent::getViewText($value, $rowNumber, $recordId, $rowData);
            }

            if ($this->pravo_vydat->lzeZobrazitRC($rowData[1])) {
                $zobrazitOdkaz = true;
                // Odkaz pro splneni pokud je platnostDo nebo RezervaceDo vetsi nez akt. datum
                if ($rowData['col-datumrezervacedo']) { // DatumRezervaceDo
                    $zobrazitOdkaz = $this->database->stringToTimestamp($rowData['col-datumrezervacedo']) >= DateUtils::today();
                } elseif ($rowData['col-datumomezeniplatnosti']) { // DatumOmezeniPlatnosti
                    $zobrazitOdkaz = $this->database->stringToTimestamp($rowData['col-datumomezeniplatnosti']) >= DateUtils::today();
                }

                if ($zobrazitOdkaz) {
                    $zobrazitDanyTyp = true;
                    if (trim($this->pravo_vydat->priznak3 ?? '') !== '') {
                        $povoleneTypyBenefitu = explode(';', $this->pravo_vydat->priznak3);
                        $zobrazitDanyTyp = in_array($rowData['col-typ'], $povoleneTypyBenefitu);
                    }

                    if ($zobrazitDanyTyp) {
                        return GuiUtils::changeLocationQuest(ronText('vydat_benefit'), ronText('opravdu_vlozit_cerpani'),
                            '?menuid=benefity&action=cerpanirezervace&record=' . $rowData[0] . Csrf::csrfUrlParameter('cerpanibenefitu_' . $rowData[0]));
                    }
                }
            }
        }

        return '';
    }
}


/**
 * Class StavBenefituField
 * @package RonPortal\Benefity
 */
class StavBenefituField extends BrowserField
{
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = '';

        switch ($rowData[2]) {
            case PohybBenefitu::DRUH_POHYBU_VZNIK_BODU:
                $result .= $rowData['col-datumschvaleni'] // je schvaleno
                    ? ronText('vznik_bodu') . " (" . $rowData['col-pocetbodu'] . ")" // pocet bodu
                    : ronText('vznik') . " (" . ronText('neschvaleno') . ")";
                break;

            case PohybBenefitu::DRUH_POHYBU_ZANIK_BODU;
                $result .= ronText('zanik_bodu') . " (" . $rowData['col-pocetbodu'] .  ")"; // pocet bodu
                break;

            case PohybBenefitu::DRUH_POHYBU_CERPANI_BODU:
            case PohybBenefitu::DRUH_POHYBU_REZERVACE_BODU:
                if ($rowData['col-datumschvaleni']) {
                    $result .=
                        TextManager::_($rowData['col-datumcerpani'] ? 'cerpano' : 'necerpano') . // datum cerpani
                        ' - ' . StringUtils::htmlTextConvert($rowData['col-benefit-popis']) . ' (' . StringUtils::htmlTextConvert($rowData['col-typ-popis']) . ')';  // popis benefitu (typ benefitu)
                } else {
                    $result .= StringUtils::htmlTextConvert($rowData['col-benefit-popis']) . " (" . ronText('neschvaleno') . ")"; // popis benefitu
                }
                break;

            case PohybBenefitu::DRUH_POHYBU_ZANIK_REZERVACE_BODU:
                $title = StringUtils::htmlTextConvert($rowData['col-benefit-popis'])  . ' (' . StringUtils::htmlTextConvert($rowData['col-typ-popis']) . ')';
                $result .= "<div title='$title' >" . ronText('zanik_rezervace') . ' (' . $rowData['col-pocetbodu'] .  ')' . "</div>" ;
                break;
        }

        return $result
            ? "<a href='?menuid=benefity&action=pohyb&record=$rowData[0]'>" . $result . '</a>'
            : '';
    }
}


/**
 * Class BenefityBrowser
 * @package RonPortal\Benefity
 */
class BenefityBrowser extends RonPortalBrowser
{

    /** @var  PravaNaOsoby $prava_rezervovat */
    public $pravo_rezervovat;

    /** @var  PravaNaOsoby $prava_schvalit */
    public $pravo_schvalit;

    /** @var  PravaNaOsoby $pravo_vydat */
    public $pravo_vydat;

    /** @var  PravaNaOsoby $pravo_vznik */
    public $pravo_vznik;

    /**
     * BenefityBrowser constructor.
     * @param Database $database
     * @throws \Exception
     */
    public function __construct($database)
    {
        parent::__construct($database, 'benefity');

        Prava::nastavObdobiProPrava(time(), time());
        $this->pravo_rezervovat = Prava::vratPrava('benefit_rezervace');
        $this->pravo_schvalit = Prava::vratPrava('benefit_schvaleni');
        $this->pravo_vydat = Prava::vratPrava('benefit_vydani');
        $this->pravo_vznik = Prava::vratPrava('benefit_vznik_bodu');


        /*if ($this->pravo_schvalit->prava == 1 || $this->pravo_rezervovat->prava == 1 || $this->pravo_vznik->prava == 1 || $this->pravo_vydat->prava == 1) {
            $this->title = ronText('prehled_pohybu_benefitu');
        } else {
            $this->title = ronText('prehled_pohybu_vlastnich_benefitu');
        }*/

        $this->code = 'browserbenefity';
        $this->tableId = 'browserbenefity';
        $this->browserUrl = "?menuid=benefity";

        $this->sqlSelectPart =
            'PB.ID, PB.RC, DruhPohybu';    //0..2

        $this->sqlFromPart =
            'PohybBenefitu PB' .
            ' LEFT JOIN S_DruhPohybuBenefitu ON (S_DruhPohybuBenefitu.Kod = PB.DruhPohybu)' .
            ' LEFT JOIN Osoba ON (PB.RC = Osoba.RC)' .
            // ' LEFT JOIN PracovniPomer ON (PracovniPomer.RC = Osoba.RC)' .
            ' LEFT JOIN Benefit ON (Benefit.ID = PB.IdBenefitu)' .
            ' LEFT JOIN TypBenefitu ON  (TypBenefitu.Kod = Benefit.TypBenefitu)';


        $this->addField()
            ->setSqlFieldName('PB.ID', 0) ->setHeaderText('ID') ->setOrdering(true)
            ->setCode('col-id');

        $this->addField()
            ->setSqlFieldName('PB.DatumVlozeni') ->setHeaderText('datum_vlozeni')
            ->setOrdering(true) ->setFormat('date')
            ->setCode('col-datumvlozeni');

        $this ->defaultSort = '-col-datumvlozeni';

        // osoba
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Osoba.Prijmeni')
            ->setCode('col-prijmeni');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Osoba.Jmeno')
            ->setCode('col-jmeno');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Osoba.Titul')
            ->setCode('col-titul');

        $this ->addField(new JmenoPrijmeniBrowserField($this ->database, 'col-jmeno', 'col-prijmeni', 'col-titul'))
            ->setCode('col-osoba')
            ->setHeaderText('Zamestnanec') ->setOrdering(true)
            //Pokud ma pravo pouze na sebe, pak se mu zobrazuji pouze vlastni a pole osoba benefitu nema smysl
            ->setVisible($this->pravo_rezervovat->prava > 1 || $this->pravo_vydat->prava > 1 || $this->pravo_vznik->prava > 1 || $this->pravo_schvalit->prava > 1)
            ;

        $this->addField(new StavBenefituField($this->database))
            ->setHeaderText('stav_benefitu')
            ->setCode('col-stav');

        $this->addField()
            ->setSqlFieldName('S_DruhPohybuBenefitu.Popis')
            ->setCode('col-druhpohybu')
            ->setHeaderText('druh_pohybu_benefitu')
            ->setOrdering(true);

        // omezeni platnosti
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('PB.DatumOmezeniPlatnosti')
            ->setCode('col-datumomezeniplatnosti');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('PB.DatumRezervaceDo')
            ->setCode('col-datumrezervacedo');

        $this->addField(new DatumPlatnostiRezervaceField($this->database))
            ->setSortSQL('DatumOmezeniPlatnosti, DatumRezervaceDo') ->setSortSQLDesc('DatumOmezeniPlatnosti DESC, DatumRezervaceDo DESC')
            ->setCode('col-platnost') ->setFormat('date')
            ->setHeaderText('datum_platnosti_rezervace')
            ->setAlign('center')
            ->setOrdering(true);

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('PB.Insert_Username')
            ->setCode('col-insert_username');

        $this->addField()
            ->setSqlFieldName('PB.Insert_Timestamp') ->setSqlFieldIndex2('col-insert_username') // jmeno v zavorce
            ->setCode('col-insert_timestamp')
            ->setFormat('date') ->setHeaderText('datum_osoba_vlozeni') ->setAlign('center')
            ->setOrdering(true);

        // schvaleni
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('PB.Schvalil')
            ->setCode('col-schvalil');

        $this->addField(new DatumSchvaleniField($this->database, $this->pravo_schvalit, $this->pravo_vznik))
            ->setSqlFieldName('PB.DatumSchvaleni') ->setSqlFieldIndex2('col-schvalil')
            ->setOrdering(true)
            ->setFormat('date')
            ->setCode('col-datumschvaleni')
            ->setHeaderText('datum_osoba_schvaleni') ->setAlign('center');


        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('PB.Cerpani_Username')
            ->setCode('col-cerpani_username');

        $this->addField(new DatumCerpaniField($this->database, $this->pravo_vydat))
            ->setSqlFieldName('PB.DatumCerpani') ->setSqlFieldIndex2('col-cerpani_username')
            ->setFormat('date')
            ->setCode('col-datumcerpani')
            ->setAlign('center') ->setHeaderText('datum_osoba_cerpani')
            ->setOrdering(true);

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('PB.Update_Username')
            ->setCode('col-update_username');

        $this->addField()
            ->setSqlFieldName('PB.Update_Timestamp') ->setSqlFieldIndex2('col-update_username') // jmeno v zavorce
            ->setCode('col-update_timestamp')
            ->setFormat('date') ->setHeaderText('Posledni_aktualizace') ->setAlign('center')
            ->setOrdering(true);

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Benefit.Popis')
            ->setCode('col-benefit-popis');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('PB.PocetBodu')
            ->setCode('col-pocetbodu');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('TypBenefitu.Kod')
            ->setCode('col-typ');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('TypBenefitu.Popis')
            ->setCode('col-typ-popis');
    }


    function setFilterForm()
    {
        $this->filterForm = new RonPortalFilterForm($this->database, 'benefity', true);

        $this ->filterForm ->addFilter(
            (new FilterFormFilter('omezeni'))
                ->addItem('0', TextManager::_('Vse'))
                ->addItem('1', TextManager::_('jenRezervaci'))
                ->addItem('2', TextManager::_('jenSchvaleni'))
                ->addItem('3', TextManager::_('jenCerpani'))
                ->setSessionName('filtr_benefity_omezeni')
                ->setDefaultValue('0')
        );
//        echo
//            '<span class="filtercaption">' .
//            "<label><input type='radio' name='omezeni' value='0' " . ($this ->omezeniBenefitu == 0 ? 'checked="checked"' : "") . " class='clickAutoSubmit'> " . TextManager::_('Vse') . '</label>' .
//            '</span>';
//
//        echo
//            '<span class="margin-right-2 padding-right-2">' .
//            '<label class="margin-right-2"><input type="radio" name="omezeni" value="1" ' . ($this ->omezeniBenefitu == 1 ? 'checked="checked"' : '') . ' class="clickAutoSubmit"> ' . TextManager::_('jenRezervaci') . '</label>' .
//            '<label class="margin-right-2"><input type="radio" name="omezeni" value="2" ' . ($this ->omezeniBenefitu == 2 ? 'checked="checked"' : '') . ' class="clickAutoSubmit"> ' . TextManager::_('jenSchvaleni') . '</label>' .
//            '<label class="margin-right-2"><input type="radio" name="omezeni" value="3" ' . ($this ->omezeniBenefitu == 3 ? 'checked="checked"' : '') . ' class="clickAutoSubmit"> ' . TextManager::_('jenCerpani') . '</label>' .
//            '</span>';


        $this ->filterForm ->addFilter(
            (new FilterFormFilterDatum('kdatu'))
                ->setCaption(TextManager::_('benefity_k_datu'))
                ->setSessionName('filter_benefity_k_datu')
        );

        // osoba
        $this ->filterForm ->addFilter(
            (new FilterFormFilterOsoba($this ->filterForm, 'osoba'))
                ->addItem('', TextManager::_('_vsichni_'))
                ->setSessionName('filter_osobabenefitu')
                ->setCaption(TextManager::_('Zamestnanec'))
                ->setOptionsSqlFilter('EXISTS (SELECT PohybBenefitu.RC FROM PohybBenefitu WHERE PohybBenefitu.RC = Osoba.RC)')
                ->setPrava([Prava::vratPrava('benefit_rezervace'), Prava::vratPrava('benefit_schvaleni'), Prava::vratPrava('benefit_vydani'), Prava::vratPrava('benefit_vznik_bodu')])
                ->setDefaultValue('')
        );

        // typ benefitu
        // puvodni SQL dotaz:
        //        $sql =
        //            'SELECT DISTINCT TypBenefitu.Kod, TypBenefitu.Popis' .
        //            ' FROM TypBenefitu Left JOIN Benefit ON (TypBenefitu.Kod = Benefit.TypBenefitu)' .
        //            ' ORDER BY TypBenefitu.Popis';
        $this ->filterForm ->addFilter(
            (new FilterFormFilter('typbenefitu', FilterFormFilter::TYPE_OPTIONS))
                ->setSessionName('filterform_typbenefitu')
                ->addItem('', TextManager::_('_vse_'))
                ->setOptionsSql(
                    'SELECT Kod, Popis FROM TypBenefitu WHERE %FILTER% ORDER BY %ORDER%',
                    '1 = 1',
                    'Popis'
                )
                ->setCaption(TextManager::_('typ_benefitu'))
                ->setSqlFilter('Benefit.TypBenefitu = ')
        );

        // benefit
        // puvodni dotaz:
        //         $prava = array($this->pravo_rezervovat, $this->pravo_schvalit, $this->pravo_vydat, $this->pravo_vznik);
        //        $this->filtrDlePrava = Prava::sqlFiltrPodlePrav($prava, 'PracovniPomer', time(), time());
        //
        //        $sql =
        //            'SELECT DISTINCT Benefit.ID, Benefit.Popis' .
        //            ' FROM Benefit' .
        //            ' LEFT JOIN PohybBenefitu ON (Benefit.ID = PohybBenefitu.IdBenefitu)' .
        //            ' LEFT JOIN PracovniPomer ON (PracovniPomer.RC = PohybBenefitu.RC)' .
        //            ' WHERE PohybBenefitu.IdBenefitu IS NOT NULL' .
        //            ($this ->filtrDlePrava ? ' AND ' . $this->filtrDlePrava : '') .
        //            ' ORDER BY Benefit.Popis';
        $this ->filterForm ->addFilter(
            (new FilterFormFilter('benefit', FilterFormFilter::TYPE_OPTIONS))
                ->setSessionName('filterform_benefit')
                ->addItem('', TextManager::_('_vse_'))
                ->setOptionsSql(
                    'SELECT Id, Popis FROM Benefit WHERE %FILTER% ORDER BY %ORDER%',
                    '1 = 1',
                    'Popis'
                )
                ->setCaption(TextManager::_('benefit'))
                ->setSqlFilter('PB.IdBenefitu = ')
        );

        // druh pohybu
        $this ->filterForm ->addFilter(
            (new FilterFormFilter('druh', FilterFormFilter::TYPE_OPTIONS))
                ->setSessionName('filterform_benefity_druh')
                ->addItem('', TextManager::_('_vse_'))
                ->setOptionsSql(
                    'SELECT Kod, Popis FROM S_DruhPohybuBenefitu WHERE %FILTER% ORDER BY %ORDER%',
                    '1 = 1',
                    'Popis'
                )
                ->setCaption(TextManager::_('druh_pohybu'))
                ->setSqlFilter('PB.DruhPohybu = ')
        );

        // stav benefitu
        $this ->filterForm ->addFilter(
            (new FilterFormFilter('stav'))
                ->setSessionName('filterform_benefity_stav')
                ->addItem('', TextManager::_('Vse'))
                ->addItem(
                    (new FilterFormFilterItem(1, TextManager::_('neschvalene_benefity')))
                        ->setSqlFilter('(PB.DatumSchvaleni IS NULL AND PB.DruhPohybu IN (11, 21, 41))')
                )
                ->addItem(
                    (new FilterFormFilterItem(2, TextManager::_('necerpane_benefity')))
                        ->setSqlFilter('(PB.DatumSchvaleni IS NOT NULL AND PB.DatumCerpani IS NULL AND PB.DruhPohybu IN (21, 41))')
                )
                ->setDefaultValue('')
        );


        // odkazy na vlozeni zaznamu
        $this ->filterForm ->addFilter(
            (new FilterFormFilter('odkazy', FilterFormFilter::TYPE_TEXT))
                ->setPosition(FilterFormFilter::POSITION_BELOW_FORM)
                ->setRowClass('margin-top-2')
                ->setDefaultValue(
                    '<ul class="linkslist">' .
                    ($this->pravo_rezervovat->jsouNejakePrava()
                        ? "<li><a href='?menuid=benefity&action=rezervacebodu'>" . TextManager::_('rezervace_benefitu') . "</a></li>"
                        : ''
                    ) .
                    ($this->pravo_vznik->jsouNejakePrava()
                        ? "<li><a href='?menuid=benefity&action=vznikbodu'>" . TextManager::_('vznik') . "</a></li>"
                        : ''
                    ) .
                    "</ul>")
        );
    }


//    function renderFilter()
//    {
//        parent::renderFilter();
//        $this->filtrForm->render();
//    }


    /**
     * @param array $values
     * @return bool
     * @throws \Exception
     */
    function enableLine($values)
    {
        if ($this->pravo_rezervovat->lzeZobrazitRC($values[1]) || $this->pravo_schvalit->lzeZobrazitRC($values[1])
            || $this->pravo_vydat->lzeZobrazitRC($values[1]) || $this->pravo_vznik->lzeZobrazitRC($values[1])) {

            return true;
        }

        return false;
    }



    /**
     * @return string
     */
    function getSQLFilter()
    {
        $params = $this ->database ->createParameters();
        $result = $this ->filterForm ->buildSqlFilter($params);

        //$datum = ($_SESSION['datumFiltruBenefitu']);
//        $datum = ($this ->filterForm instanceof BenefityFiltrForm)
//            ? $this ->filterForm ->kDatu
//            : time();

        $datum = $this ->filterForm ->getFilterValue('kdatu');

//        $result .=
//            ($result ? ' AND ' : '') .
//            'PracovniPomer.ID = (SELECT Max(PP.ID) FROM PracovniPomer PP Where PP.RC = PB.RC)';
        $rc = $this ->filterForm ->getFilterValue('osoba', 'rc');
        if (isset($rc)) {
            $result .=
                ($result ? ' AND ' : '') .
                'PB.RC = ' . $params ->addParameter($rc);
        }

//        if ($this ->filterForm instanceof BenefityFiltrForm) {
//            if (isset($this ->filterForm ->typBenefitu)) {
//                $result .=
//                    ($result ? ' AND ' : '') .
//                    'TypBenefitu.Kod = ' . $params->addParameter($this->filterForm->typBenefitu);
//            }

//            if (isset($this ->filterForm ->benefit)) {
//                $result .=
//                    ($result ? ' AND ' : '') .
//                    ' PB.idBenefitu = ' . $params->addParameter((int)$this ->filterForm ->benefit);
//            }

//            if (isset($this ->filterForm ->druhPohybu)) {
//                $result .=
//                    ($result ? ' AND ' : '') .
//                    'PB.DruhPohybu = ' . $params->addParameter((int)$this ->filterForm ->druhPohybu);
//            }

            $omezeni = $this ->filterForm ->getFilterValue('omezeni');
            if ($omezeni == 1) {
                // Jen rezervace
                $result .=
                    ($result ? ' AND ' : '') .
                    "( PB.DruhPohybu IN (41))" .
                    " AND ((PB.DatumRezervaceDo IS NOT NULL) AND (PB.DatumRezervaceDo IS NOT NULL AND PB.DatumRezervaceDo <= " . $params->addDateParameter($datum) . "))";


            }  else if ($omezeni == 2) {
                // Jen schvalene
                $result .=
                    ($result ? ' AND ' : '') .
                    " (PB.DatumSchvaleni IS NOT NULL AND PB.DatumSchvaleni <= " . $params->addDateParameter($datum) . "" .
                    " AND PB.DruhPohybu NOT IN (51) )";


            } else if ($omezeni) {
                //Jen cerpani
                $result .=
                    ($result ? ' AND ' : '') .
                    " (PB.DatumCerpani <= " . $params->addDateParameter($datum) . " )";
            }

//        }

//        if ($this ->filterForm ->osobaBenefitu) {
//            $result .=
//                ($result ? ' AND ' : '') .
//                'PracovniPomer.ID = ' . $params->addParameter($this ->filterForm ->osobaBenefitu);
//        }

//        if ($this ->filterForm ->jenNeschvalene){
//            $result .=
//                ($result ? ' AND ' : '') .
//                '(PB.DatumSchvaleni IS NULL AND PB.DruhPohybu IN (11, 21, 41))';
//        }
//
//        if ($this->filterForm ->jenNecerpane){
//            $result .=
//                ($result ? ' AND ' : '') .
//                '(PB.DatumSchvaleni IS NOT NULL AND PB.DatumCerpani IS NULL AND PB.DruhPohybu IN (21, 41))';
//        }


        return new SqlStatement($result, $params);
    }
}