<?php
/**
 * pavel.rudol, 6.5.2022 15:46
 */

namespace RonPortal\Benefity;

use Exception;
use Ron\Controls\FormField;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Controls\JmenoPrijmeniFormField;
use RonPortal\Controls\RCOsobyFormField;
use RonPortal\Controls\RonPortalForm;
use RonPortal\NeniPravoException;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\RonPortalUtils;
use RonPortal\System\AgentManager;
use RonPortal\User;

class PohybBenefituForm extends RonPortalForm
{
    /** @var string Stav formulare pri vkladani noveho pohybu: osoba/benefit/all */
    private $stav;

    /** @var bool   True, pokud se jedna o rezervaci bodu, false pro vznik bodu */
    private $rezervaceBodu;

    private $omezitPolePoSchvaleni;

    private $pp;

    /**
     * @param Database $database
     * @param bool $rezervaceBodu
     * @param PohybBenefitu $pohyb
     * @throws \Ron\DB\DBException
     */
    public function __construct($database, $rezervaceBodu = false, $pohyb = null)
    {
        parent::__construct($database, isset($_GET['record']) ? 'edit' : 'new');
        $this ->rezervaceBodu = $rezervaceBodu;

        if ($this->isInsert()) {
            if ($rezervaceBodu) {
                $this->title = TextManager::_('rezervace_benefitu');
            } else {
                $this->title = TextManager::_('vznik');
            }
        } else {
            $this->title = TextManager::_('pohyb_benefitu');
        }

        $this ->code = 'formpohybbenefitu';

        $this ->stornoAction = '?menuid=benefity';

        $this->sqlTableName = 'PohybBenefitu';
        $this ->setXmlEventsName('POHYB_BENEFITU');

        $this ->sqlSelectPart =
            'PohybBenefitu.Id, PohybBenefitu.RC,' .   // 0,1
            ' Osoba.Prijmeni, Osoba.Jmeno, Osoba.Titul, Osoba.TitulZa,' .   // 2..5
            ' Benefit.VariabilniHodnotaVBodech, Benefit.DoplatekCastka, Benefit.HodnotaVBodech, PohybBenefitu.DatumSchvaleni, PohybBenefitu.IdBenefitu';    // 6..10

        $this ->htmlFormId = 'pohyb_benefitu_form';

        $zobrazitVse = true;
        $zobrazitVyberOsoby = false;

        $prava_vydani = Prava::vratPrava('benefit_vydani');

        //editace je mozna pokud neni zadano datum schvaleni a datum konce rezervace neni zadan, nebo je v budoucnosti
        //schvaleni je mozne dokud nebylo schvaleno a uzivatel ma pravo schvalit danou osobu
        //cerpani je mozne dokud nebylo schvaleno a uzivatel ma pravo zadat cerpani dane osobe
        if (isset($pohyb) && (($pohyb->druhPohybu == PohybBenefitu::DRUH_POHYBU_VZNIK_BODU && $pohyb->datumSchvaleni) ||
            ($pohyb->druhPohybu == PohybBenefitu::DRUH_POHYBU_REZERVACE_BODU && (isset($pohyb->datumRezervaceDo) && DateUtils::today() > $pohyb->datumRezervaceDo)) ||
            ($pohyb->druhPohybu == PohybBenefitu::DRUH_POHYBU_CERPANI_BODU && $pohyb->datumCerpani)
            )) {
            $this->readOnly = true;
        }

        $this->pp = null;

        if ($this ->isInsert()) {
            if (!isset($_POST['osoba']) || !$_POST['osoba']) {
                $zobrazitVyberOsoby = true;
                $zobrazitVse = false;
                $this ->stav = 'osoba';
            } else {
                $this ->stav = 'all';
            }
        } else {
            $this->pp = PracovniPomer::nactiPP($this->database, $pohyb->idPP);
        }

        // osoba
        $this ->addField(FormField::FIELD_TYPE_GROUP)
            ->setCode('group-osoba');

        $this ->addField(FormField::FIELD_TYPE_SECTION_HEADER)
            ->setCode('header-osoba') ->setCaption('Zamestnanec');

        if ($zobrazitVyberOsoby) {
            $this ->addField(new RCOsobyFormField('benefit_rezervace', DateUtils::today()))
                ->setFieldName('osoba');
        } else if ($this ->isInsert()) {
            $this ->addField(FormField::FIELD_TYPE_HIDDEN)
                ->setFieldName('osoba') ->setSqlFieldName(null) ->setDefaultValue($_POST['osoba']);

            $this->pp = PracovniPomer::nactiPP($this ->database, (int)$_POST['osoba']);
            $this ->addField(FormField::FIELD_TYPE_TEXT)
                ->setText(StringUtils::htmlTextConvert(($this->pp ->titul ? $this->pp ->titul . ' ' : '') . $this->pp ->jmeno . ' ' . $this->pp ->prijmeni));
        } else {
            $this->addField(new JmenoPrijmeniFormField(2, 3, 4, 5))
                ->setCode('field-osoba');
        }

        $this ->addField(FormField::FIELD_TYPE_ID)
            ->setSqlFieldName('Id', 'PohybBenefitu')->setPrimary(true)->setSqlFieldIndex(0)
            ->setFieldName('ID')->setCode('field-id');

        if ($zobrazitVse) {
            // aktualni stav bodu
            $this->addField(FormField::FIELD_TYPE_GROUP)
                ->setCode('group-stav');

            $this->addField(FormField::FIELD_TYPE_SECTION_HEADER)
                ->setCode('header-stav')->setCaption('aktualni_stav_bodu');

            $bodyOsoby = BenefityOsoby::nactiStavBoduOsoby($this->database, $this->pp->rc);

            // texty pro zobrazeni aktualniho stavu bodu osoby
            $this->addField(FormField::FIELD_TYPE_TEXT)
                ->setReadOnly()->setDefaultValue($bodyOsoby->vznikBoduCelkem)
                ->setCode('field-body-vznik')->setCaption('vznik_bodu')->setRowId('row_vznik_bodu')
                ->setFieldId('pohyb_vznik_bodu')->addFieldCssClass('podrobnosti-stavu');
            $this->addField(FormField::FIELD_TYPE_TEXT)
                ->setReadOnly()->setDefaultValue($bodyOsoby->rezervovanoBodu)
                ->setCode('field-body-rezervace')->setCaption('rezervovano_bodu')->setRowId('row_rezervovano_bodu')
                ->setFieldId('pohyb_rezervace_bodu')->addFieldCssClass('podrobnosti-stavu');
            $this->addField(FormField::FIELD_TYPE_TEXT)
                ->setReadOnly()->setDefaultValue($bodyOsoby->cerpanoBodu)
                ->setCode('field-body-cerpani')->setCaption('vycerpano_bodu')->setRowId('row_vycerpano_bodu')
                ->setFieldId('pohyb_cerpani_bodu')->addFieldCssClass('podrobnosti-stavu');
            $this->addField(FormField::FIELD_TYPE_TEXT)
                ->setReadOnly()->setDefaultValue($bodyOsoby->zanikRezervaceBodu)
                ->setCode('field-body-rezervace-zanik')->setCaption('zanik_rezervace')->setRowId('row_zanik_rezervace')
                ->setFieldId('pohyb_zanik_rezervace_bodu')->addFieldCssClass('podrobnosti-stavu');
            $this->addField(FormField::FIELD_TYPE_TEXT)
                ->setReadOnly()->setDefaultValue($bodyOsoby->zanikBodu)
                ->setCode('field-body-zanik')->setCaption('zanik_bodu')->setRowId('row_zanik_bodu')
                ->setFieldId('pohyb_zanik_bodu')->addFieldCssClass('podrobnosti-stavu');
            $this->addField(FormField::FIELD_TYPE_TEXT)
                ->setReadOnly()->setDefaultValue($bodyOsoby->zustatekBodu)
                ->setCode('field-body-zustatek')->setCaption('zustatek_bodu')->setRowId('row_zustatek_bodu')
                ->setFieldId('pohyb_zustatek_bodu')->addFieldCssClass('podrobnosti-stavu');
            if ($this->rezervaceBodu) {
                // podrobnosti benefitu
                $this->addField(FormField::FIELD_TYPE_GROUP)
                    ->setCode('group-benefit');

                $this->addField(FormField::FIELD_TYPE_SECTION_HEADER)
                    ->setCode('header-benefit')->setCaption('detail_benefitu');

                // vyber benefitu
                if ($this->isInsert()) {
                    $this->addField(new BenefitFormField($this->database, $this->pp->rc))
                        ->setFieldName('IdBenefitu')->setCaption('benefit')->setRequired(true)
                        ->setCode('field-vyberbenefitu')->setDefaultValue(isset($_POST['IdBenefitu']) ? $_POST['IdBenefitu'] : null)
                        ->setFieldId('pohyb_idbenefitu');
                } else {
                    $this->addField()
                        ->setSqlFieldName('Popis', 'Benefit')->setReadOnly(true)->setRequired(true)
                        ->setCode('field-benefit')->setCaption('benefit');
                    $this->addField(FormField::FIELD_TYPE_HIDDEN)
                        ->setSqlFieldName('IdBenefitu', 'PohybBenefitu')->setFieldName('IdBenefitu')->setSqlFieldIndex(10)
                        ->setRequired(true);
                }

                // texty pro zobrazeni podrobnosti o vybranem benefitu
                $this->addField($this->isInsert() ? FormField::FIELD_TYPE_TEXT : FormField::FIELD_TYPE_EDIT) // pri insert pouze text
                ->setSqlFieldName('TypBenefitu', 'Benefit')->setReadOnly()
                    ->setCode('field-benefit-typ')->setCaption('typ_benefitu')->setRowId('row_typ_benefitu')
                    ->setFieldId('pohyb_benefit_typ')->addFieldCssClass('podrobnosti-benefitu')
                    ->setOtherHtmlParams(['data-nazev-pole' => 'typBenefitu']);

                $this->addField($this->isInsert() ? FormField::FIELD_TYPE_TEXT : FormField::FIELD_TYPE_DATE) // pri insert pouze text
                ->setSqlFieldName('PlatnostOd', 'Benefit')->setReadOnly()
                    ->setCode('field-benefit-platnostod')->setCaption('Platnost_od')->setRowId('row_platnost_benefitu')
                    ->setFieldId('pohyb_benefit_platnostod')->addFieldCssClass('podrobnosti-benefitu')
                    ->setDataColumnClass('col-6 col-sm-2')
                    ->setOtherHtmlParams(['data-nazev-pole' => 'platnostOd']);

                $this->addField($this->isInsert() ? FormField::FIELD_TYPE_TEXT : FormField::FIELD_TYPE_DATE) // pri insert pouze text
                ->setSqlFieldName('PlatnostDo', 'Benefit')->setReadOnly()
                    ->setCode('field-benefit-platnostdo')->setCaption('Platnost_do')->setNewRow(false)
                    ->setFieldId('pohyb_benefit_platnostdo')->addFieldCssClass('podrobnosti-benefitu')
                    ->setDataColumnClass('col-6 col-sm-2')->setCaptionColumnClass('col-6 col-sm-2')
                    ->setOtherHtmlParams(['data-nazev-pole' => 'platnostDo']);

                $this->addField($this->isInsert() ? FormField::FIELD_TYPE_TEXT : FormField::FIELD_TYPE_FLOAT) // pri insert pouze text
                ->setSqlFieldName('DoplatekCastka', 'Benefit')->setSqlFieldIndex(7)->setReadOnly()
                    ->setCode('field-benefit-doplatekcastka')->setCaption('doplatek')
                    ->setFieldId('pohyb_benefit_doplatekcastka')->addFieldCssClass('podrobnosti-benefitu')
                    ->setOtherHtmlParams(['data-nazev-pole' => 'doplatekCastka']);

                $this->addField($this->isInsert() ? FormField::FIELD_TYPE_TEXT : FormField::FIELD_TYPE_INT) // pri insert pouze text
                ->setSqlFieldName('HodnotaVBodech', 'Benefit')->setSqlFieldIndex(8)->setReadOnly()
                    ->setCode('field-benefit-hodnotavbodech')->setCaption('hodnota_v_bodech')
                    ->setFieldId('pohyb_benefit_hodnotavbodech')->addFieldCssClass('podrobnosti-benefitu')
                    ->setOtherHtmlParams(['data-nazev-pole' => 'hodnotaVBodech']);

                $this->addField($this->isInsert() ? FormField::FIELD_TYPE_TEXT : FormField::FIELD_TYPE_INT) // pri insert pouze text
                ->setSqlFieldName('MaxPocetJedenZam', 'Benefit')->setReadOnly()
                    ->setCode('field-benefit-maxpocetjedenzam')->setCaption('max_na_osobu')
                    ->setFieldId('pohyb_benefit_maxpocetjedenzam')->addFieldCssClass('podrobnosti-benefitu')
                    ->setOtherHtmlParams(['data-nazev-pole' => 'maxPocetJedenZam']);

                $this->addField($this->isInsert() ? FormField::FIELD_TYPE_TEXT : FormField::FIELD_TYPE_CHECK) // pri insert pouze text
                    ->setSqlFieldName('AutomatickeSchvaleni', 'Benefit')->setReadOnly()
                    ->setCode('field-benefit-automatickeschvaleni')->setCaption('automaticke_schvaleni')
                    ->setFieldId('pohyb_benefit_automatickeschvaleni')->addFieldCssClass('podrobnosti-benefitu')
                    ->setOtherHtmlParams(['data-nazev-pole' => 'automatickeSchvaleni']);

                $this->addField($this->isInsert() ? FormField::FIELD_TYPE_TEXT : FormField::FIELD_TYPE_MEMO) // pri insert pouze text
                ->setSqlFieldName('PopisDetail', 'Benefit')->setReadOnly()
                    ->setCode('field-benefit-popisdetail')->setCaption('detailni_popis')
                    ->setFieldId('pohyb_benefit_popisdetail')->addDataColumnClass('poznamka podrobnosti-benefitu')
                    ->setOtherHtmlParams(['data-nazev-pole' => 'popisDetail']);

                // podrobnosti benefitu
                $this->addField(FormField::FIELD_TYPE_GROUP)
                    ->setCode('group-pohyb');

                $this->addField(FormField::FIELD_TYPE_SECTION_HEADER)
                    ->setCode('header-pohyb')->setCaption('hodnoty_rezervace');

                $this->addField($this->isInsert() ? FormField::FIELD_TYPE_TEXT : FormField::FIELD_TYPE_DATE) // pri insert pouze text
                    ->setSqlFieldName('DatumVlozeni', 'PohybBenefitu')->setReadOnly()
                    ->setCode('field-benefit-pohyb-datumvlozeni')->setCaption('datum_vlozeni')
                    ->setFieldId('pohyb_benefitu_datumvlozeni')->setDefaultValue(StringUtils::format('date', DateUtils::today()));

                //TODO doplnit podminky pro editaci

                //editace je mozna pokud neni zadano datum schvaleni a datum konce rezervace neni zadan, nebo je v budoucnosti
                //schvaleni je mozne dokud nebylo schvaleno a uzivatel ma pravo schvalit danou osobu
                //cerpani je mozne dokud nebylo schvaleno a uzivatel ma pravo zadat cerpani dane osobe



                $this->addField(FormField::FIELD_TYPE_DATE)
                    ->setSqlFieldName('DatumRezervaceDo', 'PohybBenefitu')->setFieldName('DatumRezervaceDo')
                    ->setCode('field-benefit-pohyb-datumkoncerezervace')->setCaption('platnost_rezervace')
                    ->setFieldId('pohyb_benefitu_datumkoncerezervace');

                $this->omezitPolePoSchvaleni[] = 'field-benefit-pohyb-datumkoncerezervace';

                $this->addField(FormField::FIELD_TYPE_INT)
                    ->setSqlFieldName('PocetBenefitu', 'PohybBenefitu')->setFieldName('PocetBenefitu')->setRequired(true)
                    ->setCode('field-benefit-pohyb-pocetbenefitu')->setCaption('pocet_benefitu')
                    ->setFieldId('pohyb_benefitu_pocetbenefitu')->setFieldCssClass('benefit_pocet_benefitu margin-right-2')
                    ->setDataAttributes(['data-doplatekcastka' => 7, 'data-hodnotavbodech' => 8])
                    ->setOtherHtmlParams(['type' => 'number', 'min' => 0,
                        'data-doplatek-selector' => "#pohyb_benefitu_doplatekcastka", 'data-pocetbodu-selector' => "#pohyb_benefitu_pocetbodu"]);

                $this->omezitPolePoSchvaleni[] = 'field-benefit-pohyb-pocetbenefitu';

                $this->addField(FormField::FIELD_TYPE_INT)
                    ->setSqlFieldName('PocetBodu', 'PohybBenefitu')->setFieldName('PocetBodu')
                    ->setCode('field-benefit-pohyb-pocetbodu')->setCaption('pocet_bodu')->setFieldCssClass('benefit_pocet_bodu margin-right-2')
                    ->setFieldId('pohyb_benefitu_pocetbodu')->setOtherHtmlParams(['type' => 'number', 'readonly' => true]);

                $this->omezitPolePoSchvaleni[] = 'field-benefit-pohyb-pocetbodu';

                $this->addField(FormField::FIELD_TYPE_INT)
                    ->setSqlFieldName('DoplatekCastka', 'PohybBenefitu')->setFieldName('DoplatekCastka')
                    ->setCode('field-benefit-pohyb-doplatekcastka')->setCaption('doplatek')->setFieldCssClass('margin-right-2')
                    ->setFieldId('pohyb_benefitu_doplatekcastka')->setOtherHtmlParams(['type' => 'number']);

                $this->omezitPolePoSchvaleni[] = 'field-benefit-pohyb-doplatekcastka';

                $lzeSchvalit = Prava::vratPrava('benefit_schvaleni')->lzeZobrazitRC($this->pp->rc);
                $this->addField(FormField::FIELD_TYPE_NOT_NULL_FLAG)
                    ->setSqlFieldName('DatumSchvaleni', 'PohybBenefitu')->setFieldName('DatumSchvaleni')
                    ->setReadOnly(!$lzeSchvalit)
                    ->setVisible(!$this ->isInsert()  ||  $lzeSchvalit)  // pri insert nezobrazuje, pokud uzivatel nemuze schvalt
                    ->setCode('field-benefit-pohyb-datumschvaleni')->setCaption('schvaleni')
                    ->setFieldId('pohyb_benefitu_datumschvaleni');

                $lzeVydat = PohybBenefituDBRecord::jePravoNaVydaniBenefitu($database, $this ->pp ->rc, $pohyb->typBenefitu ?? null);

                $this->addField(FormField::FIELD_TYPE_NOT_NULL_FLAG)
                    ->setSqlFieldName('DatumCerpani', 'PohybBenefitu')->setFieldName('DatumCerpani')
                    ->setReadOnly(!$lzeVydat)
                    ->setVisible(!$this ->isInsert() || $lzeVydat) // pri insert nezobrazuje, pokud uzivatel nema pravo
                    ->setCode('field-benefit-pohyb-datumcerpani')->setCaption('cerpani')
                    ->setFieldId('pohyb_benefitu_datumcerpani');

                $this->addField(FormField::FIELD_TYPE_HIDDEN)
                    ->setSqlFieldName('TypDoplatku', 'PohybBenefitu')->setFieldName('TypDoplatku')
                    ->setCode('field-benefit-pohyb-typdoplatku')->setFieldId('pohyb_benefitu_typdoplatku')
                    ->setDefaultValue(0); //bez doplatku - na portale zatim vzdy jen tato moznost (13.5.2022)
            } else {
                // podrobnosti o vzniku/zaniku bodu
                $this->addField(FormField::FIELD_TYPE_GROUP)
                    ->setCode('group-podrobnosti');

                $this->addField(FormField::FIELD_TYPE_SECTION_HEADER)
                    ->setCode('header-podrobnosti')->setCaption('data_vlozeni_a_platnosti');

                $this->addField($this->isInsert() ? FormField::FIELD_TYPE_TEXT : FormField::FIELD_TYPE_DATE) // pri insert pouze text
                ->setSqlFieldName('DatumVlozeni', 'PohybBenefitu')->setReadOnly()
                    ->setCode('field-benefit-pohyb-datumvlozeni')->setCaption('datum_vlozeni')
                    ->setFieldId('pohyb_benefitu_datumvlozeni')->setDefaultValue(StringUtils::format('date', DateUtils::today()));

                $this->addField(FormField::FIELD_TYPE_DATE)
                    ->setFieldName('DatumOmezeniPlatnosti')->setCaption('omezeni_platnosti_vzniku')
                    ->setCode('field-datumomezeniplatnosti');

                $this->addField(FormField::FIELD_TYPE_INT)
                    ->setFieldName('PocetBodu')->setCaption('pocet_bodu')->setRequired(true)
                    ->setCode('field-pocetbodu');

                $this->addField(FormField::FIELD_TYPE_NOT_NULL_FLAG)
                    ->setSqlFieldName('DatumSchvaleni', 'PohybBenefitu')
                    ->setFieldName('DatumSchvaleni')->setCaption('schvaleni')
                    ->setCode('field-schvaleni');
            }
            $this->addField(FormField::FIELD_TYPE_HIDDEN)
                ->setSqlFieldName('DruhPohybu', 'PohybBenefitu')->setFieldName('DruhPohybu')
                ->setCode('field-benefit-pohyb-druhpohybu')->setFieldId('pohyb_benefitu_druhpohybu')
                ->setDefaultValue($this->rezervaceBodu ? PohybBenefitu::DRUH_POHYBU_REZERVACE_BODU : PohybBenefitu::DRUH_POHYBU_VZNIK_BODU);

            $this->addField(FormField::FIELD_TYPE_MEMO)
                ->setSqlFieldName('Poznamka', 'PohybBenefitu')->setFieldName('Poznamka')
                ->setCode('field-benefit-pohyb-poznamka')->setCaption('Poznamka')
                ->setFieldId('pohyb_benefitu_poznamka');

        }
    }

    public function getSqlFromPart()
    {
        return
            'PohybBenefitu ' .
            ' INNER JOIN Osoba ON Osoba.RC = PohybBenefitu.RC ' .
            ' LEFT OUTER JOIN Benefit ON Benefit.Id = PohybBenefitu.IdBenefitu';
    }

    public function loadRecord()
    {
        $result = parent::loadRecord();
        if ($result  && !$this->jsouPrava($result[1])) {
            throw new NeniPravoException();
        }

        // pohyb benefitu je schvaleny  => takze ten nelze editovat
        if ($result  && isset($result[9]) && isset($this->omezitPolePoSchvaleni)) {
            foreach ($this->omezitPolePoSchvaleni as $kodPole) {
                $fieldIndex = $this ->findFieldIndexByCode($kodPole);
                if (isset($fieldIndex)) {
                    $this ->fields[$fieldIndex] ->otherHtmlParams['readonly'] = 'readonly';
                }
            }
        }

        return $result;
    }

    public function getButtons($insert)
    {
        if (!$this->readOnly) {
            $result = parent::getButtons($insert);

            if ($this->stav === 'osoba') {
                $result['submit']['value'] = TextManager::_('Pokracovat');
            }
        } else {
            $result['cancel'] = [
                'type' => 'button',
                'class' => 'button clickHref',
                'value' => TextManager::_('Zpet_na_seznam'),
                'data-ron-href' => $this->stornoAction
            ];
        }

        return $result;
    }

    public function renderCommonFields()
    {
        parent::renderCommonFields();

        if ($this ->isInsert()) {
            echo '<INPUT type="hidden" name="stav" value="' . $this->stav . '">';
        }
    }

    public function render()
    {
        parent::render();

        if ($this ->rezervaceBodu  &&  (!$this ->isInsert() ||  $this ->stav == 'all')) {
            echo "\n<h2>" . TextManager::_('Dokumenty') . "</h2>\n";
            echo "<div class='seznam-dokumentu margin-top-2' id='dokumenty_sablony_benefitu'></div>\n";
            if (isset($this->data)) {
                echo "<input type='hidden' id='pohyb_id_benefitu' value='{$this->data[10]}'>\n";
            }
        }
    }

    /**
     * @param PohybBenefituDBRecord $rec
     * @param $insert
     * @return void
     * @throws NeniPravoException
     * @throws \Ron\DB\DBException
     */
    public function createDBRecordFields($rec, $insert)
    {
        if ($insert) {
            $idPP = (int)$_POST['osoba'];

            $this->pp = PracovniPomer::nactiPP($this ->database, $idPP);
        }
        if (!$this->jsouPrava($this->pp ->rc)) {
            throw new NeniPravoException();
        }

        $rec->findFieldByName('RC')->setNewValue($this->pp ? $this->pp ->rc : null);

        parent::createDBRecordFields($rec, $insert);

        if (isset($_POST['DatumSchvaleni'])) {
            $rec->setSchvaleni();
        }
        if (isset($_POST['DatumCerpani'])) {
            $rec->setVydani();
        }

    }


    function handleAction()
    {
        if ($this ->isInsert()) {
            if (isset($_POST['stav'])) {
                if ($_POST['stav'] !== 'all') {
                    // formular jeste nebyl kompletne vyplnen (v prvnim kroku se vybira jen osoba
                    return false;
                }
            } else {
                // nove zobrazeni formulare
                return false;
            }
        }

        return parent::handleAction();
    }

    public function insertRecord()
    {
        $rec = new PohybBenefituDBRecord($this ->database, $this);
        $this ->createDBRecordFields($rec, true);

        if (!$this->beforeInsertRecord($rec)) {
            return false;
        }

        $rec->runAgent();

        return $this->afterInsertRecord($rec);
    }

    public function editRecord()
    {
        $rec = new PohybBenefituDBRecord($this ->database, $this);
        $this ->setDBRecord($rec);

        if (!$rec->loadRecord()) {
            throw new Exception('Chyba pri editaci zaznamu - puvodni zaznam nebyl nalezen!');
        }

        if (!$this->beforeUpdateRecord($rec)) {
            return false;
        }

        $rec->runAgent();

        return $this->afterUpdateRecord($rec);
    }

    /**
     * @param string $rc RC osoby
     * @return bool
     * @throws Exception
     */
    public function jsouPrava($rc)
    {
        return (Prava::vratPrava('benefit_rezervace')->lzeZobrazitRC($rc) || Prava::vratPrava('benefit_vznik_bodu')->lzeZobrazitRC($rc)
            || Prava::vratPrava('benefit_schvaleni')->lzeZobrazitRC($rc) || Prava::vratPrava('benefit_vydani')->lzeZobrazitRC($rc));
    }
}