<?php
/**
 * stanislav.elbl, 08.04.2022 12:35
 */

namespace RonPortal\Benefity;

use Ron\Controls\FormField;
use Ron\DateUtils;
use Ron\DB\DBUtils;
use Ron\DB\SQLUtils;
use Ron\MessageManager;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Controls\JmenoPrijmeniFormField;
use RonPortal\Controls\RCOsobyFormField;
use RonPortal\Controls\RonPortalForm;
use RonPortal\NeniPravoException;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\RonPortalUtils;

class SablonaBenefituForm extends RonPortalForm
{
    /** @var bool   Priznak urcuje, jestli je nainstalovana dochazka */
    public $jeDochazka;
    /** @var bool   Priznak urcuje, jestli jsou nainstalovany mzdy */
    public $jsouMzdy;

    /** @var string Stav formulare pri vkladani nove sablony: osoba/all */
    private $stav;

    function __construct($database)
    {
        parent::__construct($database, isset($_GET['record']) ? 'edit' : 'new');

        $this ->title = TextManager::_($this ->isInsert()
            ? 'sablona_rezervace_benefitu_vznik'
            : 'sablona_rezervace_benefitu_detail');

        $this ->code = 'formsablonabenefitu';

        $this ->stornoAction = '?menuid=benefity&action=sablony';

        $this->sqlTableName = 'SablonaRezervaceBenefitu';
        $this ->setXmlEventsName('SABLONA_REZERVACE_BENEFITU');

        $this ->sqlSelectPart =
            'SablonaRezervaceBenefitu.Id, SablonaRezervaceBenefitu.RC,' .   // 0,1
            ' Osoba.Prijmeni, Osoba.Jmeno, Osoba.Titul, Osoba.TitulZa,' .   // 2..5
            ' Benefit.VariabilniHodnotaVBodech, Benefit.DoplatekCastka, Benefit.HodnotaVBodech, Benefit.ID';    // 6..9

        $this ->htmlFormId = 'sablona_benefitu_form';

        $pravo_sablony = Prava::vratPrava('benefit_sablona');

        $zobrazitVse = true;
        $zobrazitVyberOsoby = false;
        //$zobrazitVyberBenefitu = false;

        if ($this ->isInsert()) {
            if (!isset($_POST['osoba'])  ||  !$_POST['osoba']) {
                $zobrazitVyberOsoby = true;
                $zobrazitVse = false;
                $this ->stav = 'osoba';
//            } else if (!isset($_POST['benefit'])  ||  !$_POST['benefit']) {
//                $zobrazitVyberBenefitu = true;
//                $zobrazitVse = false;
//                $this ->stav = 'benefit';
            } else {
                $this ->stav = 'all';
            }
        }

        // osoba
        $this ->addField(FormField::FIELD_TYPE_GROUP)
            ->setCode('group-osoba');

        $this ->addField(FormField::FIELD_TYPE_SECTION_HEADER)
            ->setCode('header-osoba') ->setCaption('Zamestnanec');

        $pp = null;

        if ($zobrazitVyberOsoby) {
            $this ->addField(new RCOsobyFormField('benefit_sablona', DateUtils::today()))
                ->setFieldName('osoba');
        } else if ($this ->isInsert()){
            $this ->addField(FormField::FIELD_TYPE_HIDDEN)
                ->setFieldName('osoba') ->setSqlFieldName(null) ->setDefaultValue($_POST['osoba']);

            $pp = PracovniPomer::nactiPP($this ->database, (int)$_POST['osoba']);
            $this ->addField(FormField::FIELD_TYPE_TEXT)
                ->setText(StringUtils::htmlTextConvert(($pp ->titul ? $pp ->titul . ' ' : '') . $pp ->jmeno . ' ' . $pp ->prijmeni));
        } else {
            $this->addField(new JmenoPrijmeniFormField(2, 3, 4, 5))
                ->setCode('field-osoba');
        }

        if ($zobrazitVse ) { // } ||  $zobrazitVyberBenefitu) {
            // popis benefitu
            $this->addField(FormField::FIELD_TYPE_GROUP)
                ->setCode('group-benefit');

            $this->addField(FormField::FIELD_TYPE_SECTION_HEADER)
                ->setCode('header-benefit')->setCaption('benefit');

            if ($this->isInsert()) {
                $this->addField(new BenefitFormField($this ->database, $pp ->rc))
                    ->setFieldName('IdBenefitu') ->setCaption('benefit') ->setRequired(true)
                    ->setCode('field-vyberbenefitu')
                    ->setFieldId('sablona_idbenefitu');
            } else {
                $this->addField()
                    ->setSqlFieldName('Popis', 'Benefit')->setReadOnly(true)->setRequired(true)
                    ->setCode('field-benefit')->setCaption('benefit');
            }

            // texty pro zobrazeni podrobnosti o vybranem benefitu
            $this ->addField($this ->isInsert() ? FormField::FIELD_TYPE_TEXT : FormField::FIELD_TYPE_EDIT) // pri insert pouze text
                ->setSqlFieldName('TypBenefitu', 'Benefit') ->setReadOnly()
                ->setCode('field-benefit-typ') ->setCaption('typ_benefitu') ->setRowId('row_typ_benefitu')
                ->setFieldId('sablona_benefit_typ') ->addFieldCssClass('podrobnosti-benefitu')
                ->setOtherHtmlParams(['data-nazev-pole' => 'typBenefitu']);

            $this ->addField($this ->isInsert() ? FormField::FIELD_TYPE_TEXT : FormField::FIELD_TYPE_DATE) // pri insert pouze text
                ->setSqlFieldName('PlatnostOd', 'Benefit') ->setReadOnly()
                ->setCode('field-benefit-platnostod') ->setCaption('Platnost_od') ->setRowId('row_platnost_benefitu')
                ->setFieldId('sablona_benefit_platnostod') ->addFieldCssClass('podrobnosti-benefitu')
                ->setDataColumnClass('col-6 col-sm-2')
                ->setOtherHtmlParams(['data-nazev-pole' => 'platnostOd']);

            $this ->addField($this ->isInsert() ? FormField::FIELD_TYPE_TEXT : FormField::FIELD_TYPE_DATE) // pri insert pouze text
                ->setSqlFieldName('PlatnostDo', 'Benefit') ->setReadOnly()
                ->setCode('field-benefit-platnostdo') ->setCaption('Platnost_do') ->setNewRow(false)
                ->setFieldId('sablona_benefit_platnostdo') ->addFieldCssClass('podrobnosti-benefitu')
                ->setDataColumnClass('col-6 col-sm-2') ->setCaptionColumnClass('col-6 col-sm-2')
                ->setOtherHtmlParams(['data-nazev-pole' => 'platnostDo']);

            $this ->addField($this ->isInsert() ? FormField::FIELD_TYPE_TEXT : FormField::FIELD_TYPE_FLOAT) // pri insert pouze text
                ->setSqlFieldName('DoplatekCastka', 'Benefit') ->setSqlFieldIndex(7) ->setReadOnly()
                ->setCode('field-benefit-doplatekcastka') ->setCaption('doplatek')
                ->setFieldId('sablona_benefit_doplatekcastka') ->addFieldCssClass('podrobnosti-benefitu')
                ->setOtherHtmlParams(['data-nazev-pole' => 'doplatekCastka']);

            $this ->addField($this ->isInsert() ? FormField::FIELD_TYPE_TEXT : FormField::FIELD_TYPE_INT) // pri insert pouze text
                ->setSqlFieldName('HodnotaVBodech', 'Benefit') ->setSqlFieldIndex(8) ->setReadOnly()
                ->setCode('field-benefit-hodnotavbodech') ->setCaption('hodnota_v_bodech')
                ->setFieldId('sablona_benefit_hodnotavbodech') ->addFieldCssClass('podrobnosti-benefitu')
                ->setOtherHtmlParams(['data-nazev-pole' => 'hodnotaVBodech']);

            $this ->addField($this ->isInsert() ? FormField::FIELD_TYPE_TEXT : FormField::FIELD_TYPE_INT) // pri insert pouze text
                ->setSqlFieldName('MaxPocetJedenZam', 'Benefit') ->setReadOnly()
                ->setCode('field-benefit-maxpocetjedenzam') ->setCaption('max_na_osobu')
                ->setFieldId('sablona_benefit_maxpocetjedenzam') ->addFieldCssClass('podrobnosti-benefitu')
                ->setOtherHtmlParams(['data-nazev-pole' => 'maxPocetJedenZam']);

            $this ->addField($this ->isInsert() ? FormField::FIELD_TYPE_TEXT : FormField::FIELD_TYPE_CHECK) // pri insert pouze text
                ->setSqlFieldName('AutomatickeSchvaleni', 'Benefit') ->setReadOnly()
                ->setCode('field-benefit-automatickeschvaleni') ->setCaption('automaticke_schvaleni')
                ->setFieldId('sablona_benefit_automatickeschvaleni') ->addFieldCssClass('podrobnosti-benefitu')
                ->setOtherHtmlParams(['data-nazev-pole' => 'automatickeSchvaleni']);

            $this ->addField($this ->isInsert() ? FormField::FIELD_TYPE_TEXT : FormField::FIELD_TYPE_MEMO) // pri insert pouze text
                ->setSqlFieldName('PopisDetail', 'Benefit') ->setReadOnly()
                ->setCode('field-benefit-popisdetail') ->setCaption('detailni_popis')
                ->setFieldId('sablona_benefit_popisdetail') ->addFieldCssClass('poznamka podrobnosti-benefitu')
                ->setOtherHtmlParams(['data-nazev-pole' => 'popisDetail']);
        }

        if ($zobrazitVse) {
            $this->addField(FormField::FIELD_TYPE_ID)
                ->setFieldName('Id', 'SablonaRezervaceBenefitu')->setSqlFieldIndex(0)->setPrimary(true)
                ->setCode('field-id');

            $newPoradi = null;
            if ($this ->isInsert()  &&  $pp) {
                $params = $this ->database ->createParameters();
                $newPoradi = SQLUtils::newSerialNumber($this ->database, 'SablonaRezervaceBenefitu', 'Poradi',
                    'RC = ' . $params ->addParameter($pp ->rc), $params);
            }

            // editovatelne hodnoty
            $this->addField(FormField::FIELD_TYPE_INT)
                ->setFieldName('Poradi')->setRequired(true)->setCaption('poradi_benefitu')
                ->setFieldCssClass('inputInt margin-right-2')
                ->setCode('field-poradi')
                ->setDefaultValue($newPoradi); // ->setUseDefaultValueToEditNull(true);

            $this->jeDochazka = RonPortalUtils::jeNainstalovanaAplikace($this->database, 'ADS');
            $this->jsouMzdy = RonPortalUtils::jeNainstalovanaAplikace($this->database, 'MZDY');

            if ($this->jeDochazka || $this->jsouMzdy) {
                $this->addField(FormField::FIELD_TYPE_RADIO)
                    ->setFieldName('ZdrojPoctuBenefitu')->setDataType('int')->setRequired(1)->setDefaultValue(0)
                    ->setCode('field-zdrojpoctubenefitu')
                    ->setCaption('benefit_zdroj_poctu')->setOtherHtmlParams2(['class' => 'form-check margin-bottom-2'])
                    //->setAppendFormClass('')
                    ->setOptions([
                        0 => TextManager::_('benefit_zadany_pocet_benefitu'),
                        1 => TextManager::_('benefit_mesicni_vysledek_mzdy'),
                        2 => TextManager::_('benefit_mesicni_vysledek_dochazka')
                    ]);

                $params = $this->database->createParameters();
                if (trim($pravo_sablony->priznak3 ?? '') != '') {
                    $povoleneTypyCS = explode(';', $pravo_sablony->priznak3);
                    $sqlWhere = ' AND ' . SQLUtils::getINCondition($this->database, 'Kod', $povoleneTypyCS, $params);
                } else {
                    $sqlWhere = '';
                }


                $this->addField(FormField::FIELD_TYPE_SELECT)
                    ->setFieldName('TypCasoveSlozkyMzdy')->setRowId('row_sablona_benefitu_cs')
                    ->setCode('field-typcasoveslozkymzdy')
                    ->setCaption('benefit_typ_casove_slozky')
                    ->setSelectSql(
                        'SELECT Kod, Popis FROM TypCasoveSlozkyMzdy' .
                        ' WHERE (%FILTER%)' . $sqlWhere . ' ORDER BY %ORDER%', '1 = 1', 'Popis');
            } else {
                $this->addField(FormField::FIELD_TYPE_CONSTANT)
                    ->setFieldName('ZdrojPoctuBenefitu')
                    ->setCode('field-zdrojpoctubenefitu')
                    ->setRequired(1)->setDefaultValue(0);
            }

            $this->addField(FormField::FIELD_TYPE_INT)
                ->setFieldName('PocetBenefitu')->setRowId('row_sablona_benefitu_pocet')->setFieldCssClass('inputInt margin-right-2')
                ->setCode('field-pocetbenefitu')
                ->setFieldId('sablona_pocetbenefitu') ->setCaption('pocet_benefitu')->setRequired(true)->setDefaultValue(1)
                ->setDataAttributes(['data-doplatekcastka' => 7, 'data-hodnotavbodech' => 8])->setFieldCssClass('benefit_pocet_benefitu')
                ->setOtherHtmlParams(['data-doplatek-selector' => "#sablona_doplatekcastka", 'data-pocetbodu-selector' => "#sablona_benefit_pocetbodu"]);

            $this->addField(FormField::FIELD_TYPE_INT)
                ->setFieldName('PocetBodu')->setFieldCssClass('inputInt margin-right-2 benefit_pocet_bodu')
                ->setCode('field-pocetbodu')
                ->setFieldId('sablona_benefit_pocetbodu') ->setCaption('pocet_bodu')
                ->setCaption2('benefity_info_doplatek');

            $this->addField(FormField::FIELD_TYPE_FLOAT)
                ->setFieldName('DoplatekCastka', 'SablonaRezervaceBenefitu')->setFieldCssClass('inputFloat margin-right-2')
                ->setCode('field-doplatekcastka')
                ->setFieldId('sablona_doplatekcastka') ->setCaption('doplatek_benefitu')->setCaption2('benefity_info_vysledny_pocet');

            $this->addField(FormField::FIELD_TYPE_RADIO)
                ->setFieldName('TypDoplatku')->setDataType('int')
                ->setCode('field-typdoplatku')
                ->setCaption('typ_doplatku')//->setOtherHtmlParams2(['class' => 'margin-right-2'])
                ->setOptions([
                    0 => TextManager::_('bez_doplatku'),
                    1 => TextManager::_('hotovost'),
                    2 => TextManager::_('benefit_srazka_ze_mzdy')
                ]);

            $this->addField(FormField::FIELD_TYPE_CHECK)
                ->setFieldName('TypKontrolySablony')
                ->setCode('field-typkontrolysablony')
                ->setCaption('povolit_mensi_pocet')
                ->setCaption2('povolit_mensi_pocet_info');

            // frekvence (kalendar) vkladani rezervaci
            $this->addField(FormField::FIELD_TYPE_GROUP)
                ->setCode('group-kalendar');

            $this->addField(FormField::FIELD_TYPE_SECTION_HEADER)
                ->setCode('header-kalendar')->setCaption('benefit_kalendar_rezervovani');

            $this->addField(FormField::FIELD_TYPE_RADIO)
                ->setFieldName('CasovaJednotka')->setDataType('int') ->setRequired(true)
                ->setCode('field-casovajednotka')
                ->setCaption('Obdobi')->setOtherHtmlParams2(['class' => 'margin-right-2'])
                ->setOptions([
                    1 => TextManager::_('casova_jednotka_benefitu_1'),
                    2 => TextManager::_('casova_jednotka_benefitu_2'),
                    3 => TextManager::_('casova_jednotka_benefitu_3'),
                    4 => TextManager::_('casova_jednotka_benefitu_4'),
                    5 => TextManager::_('casova_jednotka_benefitu_5'),
                    6 => TextManager::_('casova_jednotka_benefitu_6')
                ]);

            $this->addField(FormField::FIELD_TYPE_INT)
                ->setFieldName('PocetCasJednotek')->setFieldCssClass('inputInt margin-right-2')
                ->setCode('field-pocetcasjednotek')
                ->setCaption('opakovani')->setRequired(true)->setDefaultValue(1);

            $this->addField(FormField::FIELD_TYPE_TEXT)
                ->setCode('field-pocetcasjednotek-info')->setFieldCssClass('poznamka')
                ->setText(
                    '<div id="pocet_cas_jednotek_info1">' . TextManager::_('pocet_cas_jednotek_info1') . '</div>' .
                    '<div>' . TextManager::_('pocet_cas_jednotek_info2') . '</div>' .
                    '<div>' . TextManager::_('pocet_cas_jednotek_info3') . '</div>');
        }

        $this ->addLogFields(true, null, 'SablonaRezervaceBenefitu');
    }



    public function getButtons($insert)
    {
        $result = parent::getButtons($insert);

        if ($this ->stav === 'osoba') {
            $result['submit']['value'] = TextManager::_('Pokracovat');
        }
        return $result;
    }


    public function getSqlFromPart()
    {
        return
            'SablonaRezervaceBenefitu' .
            ' INNER JOIN Benefit ON Benefit.Id = SablonaRezervaceBenefitu.IdBenefitu' .
            ' INNER JOIN Osoba ON Osoba.RC = SablonaRezervaceBenefitu.RC';
    }


    public function loadRecord()
    {
        $result = parent::loadRecord();
        if ($result  &&  !Prava::vratPrava('benefit_sablona') ->lzeZobrazitRC($result[1])) {
            throw new NeniPravoException();
        }

        if ($result  &&  !(int)$result[6]) {
            // benefit ma pevne dany pocet bodu => takze ten nelze editovat
            $fieldIndex = $this ->findFieldIndexByCode('field-pocetbodu');
            if (isset($fieldIndex)) {
                $this ->fields[$fieldIndex] ->otherHtmlParams['readonly'] = 'readonly';
            }
        }

        return $result;
    }


    public function checkRequiredFields($edit)
    {
        if (!parent::checkRequiredFields($edit)) {
            return false;
        };

        if (isset($_POST['ZdrojPoctuBenefitu'])  &&  (int)$_POST['ZdrojPoctuBenefitu']) {
            // zdroj poctu benefitu je mesicni vysledek => musi byt zadan typ casove slozky mzdy
            $fieldIndex = $this ->findFieldIndexByCode('field-typcasoveslozkymzdy');
            $field = $this ->fields[$fieldIndex];
            if (!$_POST[$field ->fieldName]) {
                MessageManager::addErrorMessage(TextManager::_('form.Missing_value') . $field->caption, $field->fieldName);
                return false;
            }
        }

        return true;
    }

    public function renderCommonFields()
    {
        parent::renderCommonFields();

        if ($this ->isInsert()) {
            echo '<INPUT type="hidden" name="stav" value="' . $this->stav . '">';
        }
    }

    public function render()
    {
        parent::render();

        if (($this->data[9] ?? null)  &&  (!$this ->isInsert() ||  $this ->stav == 'all')) {    // Benefit.ID
            echo "\n<h2>" . TextManager::_('Dokumenty') . "</h2>\n";
            echo "<div class='seznam-dokumentu margin-top-2' id='dokumenty_sablony_benefitu'></div>\n";
            echo "<input type='hidden' id='sablona_id_benefitu' value='{$this->data[9]}'>\n";
        }
    }


    public function createDBRecordFields($rec, $insert)
    {
        $pp = null;

        if ($insert) {
            $idPP = (int)$_POST['osoba'];

            $pp = PracovniPomer::nactiPP($this ->database, $idPP);

            if (!Prava::vratPrava('benefit_sablona') ->lzeZobrazitRC($pp ->rc)) {
                throw new NeniPravoException();
            }
        }

        $rec ->addStringField('RC', true, 15, $pp ? $pp ->rc : null); // pri vkladani nastavi hodnotu, pri editaci je readonly

        parent::createDBRecordFields($rec, $insert);
    }


    function handleAction()
    {
        if ($this ->isInsert()) {
            if (isset($_POST['stav'])) {
                if ($_POST['stav'] !== 'all') {
                    // formular jeste nebyl kompletne vyplnen (v prvnim kroku se vybira jen osoba
                    return false;
                }
            } else {
                // nove zobrazeni formulare
                return false;
            }
        }

        return parent::handleAction();
    }
}
