<?php

namespace RonPortal\Common;

use Ron\Controls\FormField;
use Ron\Exception\NotImplementedException;
use Ron\RonException;
use Ron\TextManager;use RonPortal\Controls\RonPortalForm;
use RonPortal\Log;
use RonPortal\Security;
use RonPortal\User;

class ChangePasswordForm extends RonPortalForm
{
    public function __construct($database)
    {
        parent::__construct($database, 'new');

        // $this ->code = 'formchangepassword'; - z bezpecnostnich duvodu neumoznit konfiguraci?

        $this ->sqlTableName = 'Osoba';

        $this ->title = TextManager::_('Zmena_hesla');

        $this ->code = 'formchangepassword';

        $this ->addField()
            ->setFieldName('*oldpass')
            ->setCaption('Stare_heslo')
            ->setOtherHtmlParams(['type' => 'password']);

        $this ->addField()
            ->setFieldName('*newpass1')
            ->setCaption('Nove_heslo')
            ->setOtherHtmlParams(['type' => 'password']);

        $this ->addField()
            ->setFieldName('*newpass2')
            ->setCaption('Potvrzeni_hesla')
            ->setOtherHtmlParams(['type' => 'password']);
    }

    public function getButtons($insert)
    {
        $result = parent::getButtons($insert);

        if ($result['submit'] ?? null) {
            $result['submit']['value'] = TextManager::_('Nastavit_heslo');
        }

        return $result;
    }


    public function renderHeader()
    {
        parent::renderHeader();

        echo TextManager::_('Zmena_hesla_info');
    }


    public function insertRecord()
    {
        $oldPassword = $this ->findFieldByName('oldpass') ->getPostValue($this ->utf8) ?? '';

        $newPassword1 = $this ->findFieldByName('newpass1') ->getPostValue($this ->utf8) ?? '';
        $newPassword2 = $this ->findFieldByName('newpass2') ->getPostValue($this ->utf8) ?? '';

        if ($newPassword1 != $newPassword2) {
            throw new \Exception( TextManager::_('Hesla_se_neshoduji') );
        }

        if (User::getVynutitZmenuHesla() && $oldPassword == $newPassword1) {
            // pokud je povinna zmena hesla, tak nesmi byt stare heslo stejne jako nove
            throw new RonException(TextManager::_('Nove_heslo_je_stejne'), 'newpass1');
        }

        $policy = Security::getPasswordPolicy($this ->database);

        Security::checkPasswordPolicy($newPassword1, $policy);

        if (!Security::checkUserPassword($this ->database, User::getRC(), $oldPassword)) {
            Log::addRecordToLog($this->database,
                TextManager::_('Chybny_pokus_o_prihlaseni'),
                TextManager::_('Login_log_category'),
                'Chybny pokus o zmenu hesla uzivatele ' . User::getLoginName() . ' - spatne zadane puvodni heslo!',
                User::getLoginName());

            throw new \Exception(TextManager::_('Spatne_puvodni_heslo'));
        }

        $passwordType = Security::getPasswordType($this ->database);
        $newHash = Security::encodePassword($passwordType, $newPassword1);

        $params = $this ->database ->createParameters();
        $this ->database ->sql(
            'UPDATE Osoba SET IntrawebPassword = ' . $params->addParameter($newHash) .
            ', PocetSpatnychPrihlaseni = 0, VynutitZmenuHesla = 0,  PosledniZmenaHesla = ' . $params ->addTimestampParameter(time()) .
            ' WHERE RC = ' . $params->addParameter(User::getRC()),
            $params
        );

        Log::addRecordToLog($this->database,
            'Probehla zmena hesla uzivatele: ' . User::getLoginName(),
            TextManager::_('Login_log_category')
        );

        User::setVynutitZmenuHesla(0);

        return true;
    }


}
