<?php
/**
 * Roman.Szablatura RON Software 28.3.2017
 */

namespace RonPortal\Common;

use Exception;
use Ron\Controls\FormField;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\Controls\ComboBox;
use Ron\MessageManager;
use Ron\RonException;
use Ron\TextManager;
use function Ron\ronText;
use Ron\StringUtils;
use Ron\DB\DBUtils;
use RonPortal\Controls\RonPortalForm;
use RonPortal\Controls\RozdeleniFormField;
use RonPortal\Controls\UserLoginFormField;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\PravaNaOsoby;
use RonPortal\RonPortalSession;
use RonPortal\RonPortalSettings;
use RonPortal\RozdeleniOsob;
use RonPortal\TypRozdeleni;
use RonPortal\User;


class NavstevaFormField extends FormField
{
    public $editovatPoPrichodu = true;
    public $editovatPoOdchodu = false;

    /**
     * NavstevaFormField constructor.
     * @param string|mixed $fieldType
     */
    public function __construct($fieldType = FormField::FIELD_TYPE_EDIT)
    {
        parent::__construct($fieldType);
    }

    /**
     * @param boolean $editovatPoPrichodu
     * @return NavstevaFormField
     */
    public function setEditovatPoPrichodu($editovatPoPrichodu)
    {
        $this->editovatPoPrichodu = $editovatPoPrichodu;
        return $this;
    }

    /**
     * @param boolean $editovatPoOdchodu
     * @return NavstevaFormField
     */
    public function setEditovatPoOdchodu($editovatPoOdchodu)
    {
        $this->editovatPoOdchodu = $editovatPoOdchodu;
        return $this;
    }



    function loadConfigurationItem($key, $value)
    {
        switch ($key) {
            case 'editovatPoOdchodu' :
            case 'editovatPoPrichodu' :
                $this ->{$key} = (int)$value;
                break;

            default :
                parent::loadConfigurationItem($key, $value);
        }
    }
}


/**
 * Class VratniceField
 */
class VratniceField extends FormField
{
    /** @var Database $database */
    public $database;
    private $rozdeleniVratnice;


    /**
     * VratniceField constructor.
     * @param Database $database
     * @param $rozdeleniVratnice
     * @internal param null $options
     */
    public function __construct($database, $rozdeleniVratnice)
    {
        parent::__construct();
        $this->caption = '<strong>' . ronText('vratnice') . '</strong>';
        $this->rozdeleniVratnice = $rozdeleniVratnice;
        $this->database = $database;
    }

    /**
     * @param Database $database
     * @param mixed $value
     * @param bool $canEdit
     * @param bool $insert
     * @param array $rowData
     * @throws DBException
     */
    function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {
        $kod = $this->rozdeleniVratnice;
        $typ = RonPortalSettings::getIntValue('typrozdelenivratnice');

        if ($rowData  &&  $rowData[$this ->sqlFieldIndex]) {
            $kod = $rowData[$this ->sqlFieldIndex];
            $typ = $rowData[$this ->sqlFieldIndex - 1];
        } else if ($insert  &&  $typ) {
            $kod = RonPortalSession::getValue('rozdelenivratnice');
        }

        if ($kod  &&  $typ) {
            $params = $database->createParameters();
            $sql =
                'SELECT Popis FROM Rozdeleni' .
                ' WHERE KodTypuRozdeleni = ' . $params->addParameter($typ) .
                ' AND Kod = ' . $params->addParameter($kod);

            $cursor = $database->getSqlResult($sql, $params);
            $row = $cursor->fetch();
            $cursor->close();

            $text = $row[0];
        } else {
            $text = '';
        }

        echo '<span class="float-right"><button type="button" id="tlacitko_storno" class="button clickHref" data-ron-href="?menuid=navstevy">' . ronText('Zpet_na_seznam') . '</button></span>';

        echo '<div class="font-weight-bold">' . StringUtils::htmlTextConvert($text) . '</div>';

    }
}

///**
// * Class NavstivenaOsobaRozdeleniField
// */
//class NavstivenaOsobaRozdeleniField extends FormField
//{
//    private $url;
//    private $popis;
//    private $database;
//
//    /**
//     * NavstivenaOsobaRozdeleniField constructor.
//     * @param Database $database
//     * @param $edit
//     * @param $sqlIndex
//     * @param string $url
//     * @throws DBException
//     */
//    public function __construct($database, $edit, $sqlIndex, $url)
//    {
//        parent::__construct();
//        $this->url = $url;
//        $this->editable = $edit;
//        $this->database = $database;
//        $this->popis = DBUtils::getSqlValue($this->database,'SELECT Popis FROM TypRozdeleni WHERE Kod = '. (int)RonPortalSettings::getIntValue('typrozdelenievidencenavstev'));
//        $this->caption = $this->popis;
//        $this->sqlFieldIndex = $sqlIndex;
//    }
//
//    /**
//     * @param Database $database
//     * @param mixed $value
//     * @param bool $canEdit
//     * @param bool $insert
//     * @param array $rowData
//     * @throws DBException
//     */
//    function showFieldContent($database, $value, $canEdit, $insert, $rowData)
//    {
//        $sql =
//            "Select Rozdeleni.Kod, Rozdeleni.Popis " .
//            "FROM Rozdeleni " .
//            "WHERE Rozdeleni.KodTypuRozdeleni = '" . RonPortalSettings::getValue('typrozdelenievidencenavstev') . "'" .
//            "ORDER BY Rozdeleni.Popis";
//
//        if ($insert) {
//            $combo = new ComboBox('kodrozdeleni');
//            $combo->emptyRow = ronText('nevybrano');
//            $combo->loadData($database, $sql, $value ? $value : $_GET['kodrozdeleni']);
//            $combo->className = 'app_combo';
//            $combo->htmlParams = 'data-ron-select2="kodrozdeleni" onChange="window.location.href = &quot;' . $this->url . '&amp;kodrozdeleni=&quot; + encodeURIComponent(this.value);"';
//            $combo->render();
//        } else {
//            $popisRozdeleni = DBUtils::getSqlValue($this->database,"SELECT Popis FROM Rozdeleni WHERE Kod = '". (int)$rowData[26] . "' AND KodTypuRozdeleni= '". (int)$rowData[25] . "'");
//            echo '<input class="inputPole" type="text" disabled="disabled" name="kodrozdeleni" value="' . $popisRozdeleni . '">';
//            echo '<input class="inputPole" type="hidden" name="kodrozdeleni" value="' . $rowData[26] . '">';
//        }
//    }
//}

/**
 * Pouzito pro: cizi nebo zamestnanec
 * Class TypNavstevyField
 */
class TypNavstevyField extends FormField
{
    /**
     * TypNavstevyField constructor.
     */
    public function __construct()
    {
        parent::__construct(FormField::FIELD_TYPE_INT);
    }

    /**
     * @param Database $database
     * @param mixed $value
     * @param bool $canEdit
     * @param bool $insert
     * @param array $rowData
     */
    function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {
        echo
            '<div class="typNavstevy font-weight-bold margin-top">';

        if($insert) {
            echo
                '<label class="margin-right-2"><input  tabindex="2" type="radio" name="TypNavstevy" value="0" ' . ($value == 0 ? 'checked' : '') . '>' . ronText('cizi') . '</label>' .
                '<label><input  tabindex="3" type="radio" name="TypNavstevy" value="1" ' . ($value == 1 ? 'checked' : '') . '>' . ronText('vlastni') . '</label>';
        } else {
            echo
                ronText($value ? 'vlastni' : 'cizi') .
                '<input class="d-none" disabled="disabled" value="' . $value . '" type="radio" checked="checked" name="' . $this->fieldName . '" >'; // pro javascript
        }

        echo
            '</div>';
    }

    /**
     * @param $database
     * @param $tableName
     * @param $insert
     * @return null|DBRecordField
     */
//    function createDBRecordField($database, $tableName, $insert)
//    {
//        $recordField = new DBRecordField($this->sqlFieldName, 'int', true);
//        $recordField->setValue($_POST['TypNavstevy']);
//        return $recordField;
//    }
}


/**
 * Class NavstivenaOsobaField
 */
class NavstivenaOsobaField extends NavstevaFormField
{
    /** @var  Database $database */
    private $database;

    /**
     * NavstivenaOsoba constructor.
     * @param Database $database
     * @param $sqlIndex
     */
    public function __construct($database, $sqlIndex)
    {
        parent::__construct();

        $this ->setCaption('Jmeno');
        $this->database = $database;
        $this
            ->setFieldName('navstivenaOsoba')
            ->setSqlFieldName(null)
            ->setSqlFieldIndex($sqlIndex)
            ->setFieldId('navstivenaOsoba')
            ->setRequired(true)
            ->setReadOnly(true)
            ->setFieldCssClass('form-control ronselect')
            ->setOtherHtmlParams([
                'data-title' => ronText('navstivena_osoba'),
                'data-columns' => 'text'
            ]);
    }

    /**
     * @param Database $database
     * @param mixed $value
     * @param bool $canEdit
     * @param bool $insert
     * @param array $rowData
     * @throws DBException
     */
    function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {
        if ($insert) {
            $prava = Prava::vratPrava('navstevy');

            $params = $database->createParameters();
            $sql =
                'SELECT MIN(PracovniPomer.Id), Osoba.Prijmeni, Osoba.Jmeno, Osoba.Titul, Osoba.Telefon, MIN(PracovniPomer.OsobniCislo) ' .
                ' FROM PracovniPomer JOIN Osoba ON (PracovniPomer.RC = Osoba.RC) ' .
                "\nWHERE PracovniPomer.DatumNastupu <= " . $params->addParameter($database ->dateToParam(time())) .
                ' AND ((PracovniPomer.DatumUkonceni >= ' . $params->addParameter($database ->dateToParam(time())) . ' OR PracovniPomer.DatumUkonceni IS NULL)' .
                " AND PracovniPomer.Zamestnanec > 0)\n" .
                "\nAND " . $prava ->sqlFiltr('PracovniPomer', time(), time(), $params) .
                "\nGROUP BY Osoba.Prijmeni, Osoba.Jmeno, Osoba.Titul, Osoba.Telefon";

            $cursor = $database->getSqlResult($sql, $params);
// TODO: ZKONTROLOVAT A OPRAVIT ! ! !
            echo "<select name='{$this ->fieldName}' " . $this ->getHtmlParams() . ">";
                if (!$prava ->pouzeSebe()) {
                    echo '<option value=""></option>';
                }
                while ($osoba = $cursor->fetch()) {
                    $selectedStr = $value == $osoba[0]
                        ? 'selected="selected"'
                        : '';
                    echo
                        "<option value='$osoba[0]'" .
                        ' data-ron-telefon="' . StringUtils::htmlTextConvert($osoba[4]) . '"' .
                        ' data-jmeno="' . StringUtils::htmlTextConvert($osoba[2]) . '"' .
                        ' data-prijmeni="' . StringUtils::htmlTextConvert($osoba[1]) . '"' .
                        ' data-titul="' . StringUtils::htmlTextConvert($osoba[3]) . '"' .
                        ' data-osobnicislo="' . StringUtils::htmlTextConvert($osoba[5]) . '"' .
                        " $selectedStr>" .
                        StringUtils::htmlTextConvert($osoba[1] . ' ' . $osoba[2] . ($osoba[3] ? ', ' . $osoba[3] : '')) .
                        "</option>";
                }
            echo '</select>';

            $cursor ->close();

        } else {
            $params = $database->createParameters();
            $sql =
                'SELECT MIN(ID) FROM PracovniPomer WHERE PracovniPomer.RC = ' . $params->addParameter($rowData[$this ->sqlFieldIndex]);
            $idNavstiveneOsoby = DBUtils::getSqlValue($database, $sql, $params);

            echo StringUtils::htmlTextConvert($rowData[22]);
            echo '<input class="inputPole" type="hidden" name="navstivenaOsoba" value="' . $idNavstiveneOsoby . '">';
        }
    }
}


/**
 * Class ZamestnanecField
 */
class ZamestnanecField extends NavstevaFormField
{
    /** @var  Database $database */
    private $database;

    /**
     * ZamestnanecField constructor.
     * @param Database $database
     * @param null $nazevPole
     */
    public function __construct($database, $nazevPole)
    {
        parent::__construct();
        $this->database = $database;

        $this
            ->setFieldName($nazevPole)
            ->setCaption('Jmeno')
            ->setSqlFieldName(null)
            ->setSqlFieldIndex(6)
            ->setFieldId('zamestnanecNavsteva')
            ->setFieldCssClass('form-control ronselect')
            ->setOtherHtmlParams([
                'data-columns' => 'text'
            ]);
    }

    /**
     * @param Database $database
     * @param mixed $value
     * @param bool $canEdit
     * @param bool $insert
     * @param array $rowData
     * @throws DBException
     */
    function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {
        if ($insert) {
//            if ($value) {
//                $params = $database->createParameters();
//                $sql = 'SELECT MIN(ID) FROM PracovniPomer WHERE PracovniPomer.RC = ' . $params->addParameter($value);
//                $idZamestnance = DBUtils::getSqlValue($database, $sql, $params);
//            } else {
//                $idZamestnance = null;
//            }
//echo $idZamestnance;

            $params = $database->createParameters();
            $sql =
                "SELECT MIN(ID), COALESCE(Osoba.Prijmeni,'') + ' ' + COALESCE(Osoba.Jmeno,'') + COALESCE(', ' + Osoba.Titul,''), Osoba.Prijmeni, Osoba.Jmeno, Osoba.Titul" .
                ' FROM PracovniPomer JOIN Osoba ON (PracovniPomer.RC = Osoba.RC)' .
                ' WHERE PracovniPomer.DatumNastupu <= ' . $params->addDateParameter(time()) .
                ' AND ((PracovniPomer.DatumUkonceni >= ' . $params->addDateParameter(time()) . ' OR PracovniPomer.DatumUkonceni IS NULL)' .
                ' AND PracovniPomer.Zamestnanec > 0)' .
                ' GROUP BY Osoba.Prijmeni, Osoba.Jmeno, Osoba.Titul, Osoba.Telefon';

            $combo = new ComboBox($this ->fieldName);
            $combo->className = $this ->fieldCssClass;
            $combo->emptyRow = ' ';
            $combo->htmlParams = $this ->getHtmlParams();
            $combo->attrIndexes = ['data-prijmeni' => 2, 'data-jmeno' => 3, 'data-titul' => 4];
            $combo->showCode = 0;
            $combo->loadData($this->database, $sql, $value, $params);
            $combo->render();

        } else {
            if ($value) {
                $params = $database->createParameters();
                $cursor = $database->getSqlResult(
                    'SELECT Prijmeni, Jmeno, Titul FROM Osoba WHERE RC = ' . $params->addParameter($value),
                    $params);
                $rowOsoba = $cursor->fetch();
                $cursor->close();

                echo StringUtils::htmlTextConvert(
                    $rowOsoba[0] . ' ' . $rowOsoba[1] . ($rowOsoba[2] ? ', ' . $rowOsoba[2] : '')
                );
            }
        }
    }
}

/**
 * Class CasovePoleNavstevyField
 */
class CasovePoleNavstevyField extends NavstevaFormField
{
    function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {
        parent::showFieldContent($database, $value, $canEdit, $insert, $rowData);

        if ($canEdit  &&  $this ->currentTimeButton) {
            echo
                "<input type='button' class='button actual-time-button clickAutoSubmit'" .
                " data-date-input='{$this ->fieldId}' data-time-input='{$this ->fieldId}_time'" .
                ' value="' . ronText("Vlozit_aktualni_cas_a_odeslat") . '"' .
                '/>';
        }
    }
}





/**
 * Pouzito pro: kod, titul, prijmeni, jmeno, doklad, vozidlo
 * Class PoleNavstevyField
 */
class PoleNavstevyField extends NavstevaFormField
{
    private $nabizetPredchozi;
    //private $popis;
    //private $hodnota;

    /** @var Database $database */
    public $database;

    /**
     * PoleNavstevyField constructor.
     * @param Database $database
     * @param string $nazevPole
     * @param int $delka
     * @param int $sqlIndex
     */
    public function __construct($database, $nazevPole, $sqlIndex, $delka)
    {
        parent::__construct(FormField::FIELD_TYPE_EDIT);

        $this->database = $database;

        $this
            ->setFieldName($nazevPole) ->setSqlFieldIndex($sqlIndex)
            ->setFieldId($nazevPole)
            ->setReadOnly(false) ->setRequired(false)
            ->setFieldCssClass('margin-right-2')
            ->setMaxLength($delka)
            ->setOtherHtmlParams(['maxlength' => '']); // aby mely ve formulari vsechny inputy stejnou delku


//        $this->popis = $popis;

//        if ($required) {
//            $this->caption = "<div class='navstevy-povinne-pole $this->popis'>" . ronText($this->popis) . "</div>";
//        } else {
//            $this->caption = "<div class='$this->popis'>" . ronText($this->popis) . "</div>";
//        }
        $this->nabizetPredchozi = RonPortalSettings::getIntValue('navstevy_nabizet_existujici_hodnoty');
    }


    function getFieldCssClass()
    {
        return $this ->fieldCssClass . ' ' . MessageManager::getClass($this ->fieldName);
    }


    function loadConfigurationItem($key, $value)
    {
        switch ($key) {
            case 'nabizetPredchozi' :
                $this ->nabizetPredchozi = (int)$value;
                break;

            default :
                parent::loadConfigurationItem($key, $value);
        }
    }

    function getSqlDrivejsiHodnoty()
    {
        return
            "SELECT DISTINCT Navsteva.{$this->fieldName}".
            ' FROM Navsteva' .
            " WHERE Navsteva.{$this->fieldName} IS NOT NULL" .
            " ORDER BY Navsteva.{$this->fieldName}";
    }


    /**
     * @param Database $database
     * @param mixed $value
     * @param bool $canEdit
     * @param bool $insert
     * @param array $rowData
     * @throws DBException
     */
    function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {
        parent::showFieldContent($database, $value, $canEdit, $insert, $rowData);

        //$this->hodnota = $rowData[$this->sqlFieldIndex];
//        if ($canEdit) {
//            echo
//                '<input class="margin-right-2" type="text" id="' . $this->fieldId . '" name="' . $this->fieldName . '"' .
//                //' value="' . StringUtils::htmlTextConvert($rowData[$this->sqlFieldIndex]) . '"' .
//                ' value="' . StringUtils::htmlTextConvert($value) . '"' .
//                $this ->getHtmlParams() .
//                '>';

            if ($canEdit  &&  $this->nabizetPredchozi) {
                $sql = $this ->getSqlDrivejsiHodnoty();

                if ($sql) {
                    $combo = new ComboBox(''); // ($this->fieldName . '_combo');
                    $combo->emptyRow = ronText('vlozit_novou');
                    $combo->keyIndex = 0;
                    $combo->captionIndex = 0;
                    $combo->loadData($database, $sql, null);
                    $combo->className = 'adsfilterforminput_popis drivejsiHodnoty';
                    $combo->htmlParams = ['data-input-selector' => '#' . $this->fieldId];

                    $combo->render();
                }
            }
//        } else {
//            echo StringUtils::htmlTextConvert($rowData[$this->sqlFieldIndex]);
//            echo '<input class="inputPole" type="hidden" name="' . $this->fieldName . '" value="' . StringUtils::htmlTextConvert($rowData[$this->sqlFieldIndex]) . '">';
//        }
    }

//    function createDBRecordField($database, $tableName, $insert)
//    {
//        $field = new DBRecordField($this->sqlFieldName, '', !$this->editable, $this->maxLength);
//
//        if ($_POST[$this->fieldName] != '') {
//            $field->setValue($_POST[$this->fieldName]);
//        }
//
//        return $field;
//    }
}


/**
 * Class FirmaField
 */
class FirmaField extends PoleNavstevyField
{
    /**
     * FirmaField constructor.
     * @param Database $database
     * @param string $nazevPole
     * @param $sqlIndex
     */
    public function __construct($database, $nazevPole, $sqlIndex)
    {
        parent::__construct($database, $nazevPole, $sqlIndex, 50);
    }

    function getSqlDrivejsiHodnoty()
    {
        // Nabizi hodnoty z nastaveneho typu rozdeleni
        if (RonPortalSettings::getIntValue('typrozdelenifirma')) {
            return
                'SELECT Popis FROM Rozdeleni' .
                ' WHERE KodTypuRozdeleni = ' . RonPortalSettings::getIntValue('typrozdelenifirma') .
                ' ORDER BY Popis';
        }

        return parent::getSqlDrivejsiHodnoty();
    }
}


/**
 * Class PrichodZapsalField
 */
class PrichodZapsalField extends FormField
{
    /** @var Database $database */
    public $database;

    /**
     * PrichodZapsalField constructor.
     * @param Database $database
     * @internal param null $options
     */
    public function __construct($database)
    {
        parent::__construct();
        $this->database = $database;
    }

    /**
     * @param Database $database
     * @param mixed $value
     * @param bool $canEdit
     * @param bool $insert
     * @param array $rowData
     * @throws DBException
     */
    function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {
        if ($rowData  &&  $rowData[31]) {
            $params = $database->createParameters();
            $sql =
                'SELECT Osoba.Prijmeni, Osoba.Jmeno, Osoba.Titul' .
                ' FROM Osoba' .
                ' WHERE RC = ' . $params->addParameter($rowData[31]);

            $cursor = $database->getSqlResult($sql, $params);
            $row = $cursor->fetch();
            $cursor->close();

            echo StringUtils::htmlTextConvert(($row[2] ? $row[2] . ' ' : '') . $row[0] . ' ' . $row[1]);
        }
    }
}


///**
// * Class UzivatelskaPoleNavstev
// */
//class UzivatelskaPoleNavstev extends FormField
//{
//    /** @var Database $database */
//    public $database;
//
//    /**
//     * UzivatelskaPoleNavstev constructor.
//     * @param Database $database
//     * @param $nadpis
//     * @param $nazevPole
//     * @param $indexUzPole
//     * @param $navsteva
//     */
//    public function __construct($database, $nadpis, $nazevPole, $indexUzPole, $navsteva)
//    {
//        parent::__construct();
//        $this->caption = $nadpis;
//        $this->maxLength = 100;
//        $this->sqlFieldName = $nazevPole;
//        $this->fieldName = $nazevPole;
//        $this->sqlFieldIndex = $indexUzPole;
//        $this->database = $database;
//        $this->editable = (!$navsteva->odchod);
//    }
//
//    /**
//     * @param Database $database
//     * @param mixed $value
//     * @param bool $canEdit
//     * @param bool $insert
//     * @param array $rowData
//     */
//    function showFieldContent($database, $value, $canEdit, $insert, $rowData)
//    {
//        $odchod = $database->stringToTimestamp($rowData[15]);
//        if ($odchod == "") {
//            echo '<input class="form-input" type="text" title="' . ronText($this->caption) . '"  name="' . $this->fieldName . '" value="' . StringUtils::htmlTextConvert($value) . '" >';
//        } else {
//
//            echo StringUtils::htmlTextConvert($rowData[$this->sqlFieldIndex]);
//            echo '<input class="inputPole" type="hidden" name="' . $this->fieldName . '" value="' . StringUtils::htmlTextConvert($rowData[$this->sqlFieldIndex]) . '">';
//        }
//    }
//
//    /**
//     * @param $database
//     * @param $tableName
//     * @param $insert
//     * @return null|DBRecordField
//     */
//    function createDBRecordField($database, $tableName, $insert)
//    {
//        $field = new DBRecordField($this->sqlFieldName, '', false, $this->maxLength);
//        if ($_POST[$this->fieldName] != '') {
//            $field->setValue($_POST[$this->fieldName]);
//        }
//        return $field;
//    }
//}

///**
// * Class UzivatelJmenoField
// */
//class UzivatelJmenoField extends FormField
//{
//    /**
//     * UzivatelJmenoField constructor.
//     * @param $fieldname
//     * @param $edit
//     */
//    function __construct($fieldname, $edit)
//    {
//        parent::__construct();
//        $this->fieldName = $fieldname;
//        $this->sqlFieldName = $fieldname;
//        $this->maxLength = 25;
//        $this->editable = $edit;
//    }
//
//    function createDBRecordField($database, $tableName, $insert)
//    {
//        $field = new DBRecordField($this->sqlFieldName, '', !$this->editable, $this->maxLength);
//        $field->setValue(User::getUserName());
//        return $field;
//    }
//
//    function showFieldContent($database, $value, $canEdit, $insert, $rowData)
//    {
//        echo StringUtils::htmlTextConvert($value);
//    }
//}

/**
 * Class VydaniKartyNavsteveField
 */
class VydaniKartyNavsteveField extends NavstevaFormField
{
    /** @var  Database $database */
    private $database;

    /**
     * VydaniKartyNavsteveField constructor.
     * @param null $database
     */
    public function __construct($database)
    {
        parent::__construct();
        $this->caption = ronText('vyhledani_nacteni_id_medium');
        $this->database = $database;
        $this->sqlFieldIndex = 16;
    }


//    function createDBRecordField($database, $tableName, $insert)
//    {
//        return null; // vytvoreni poli reseno primo ve formulari
//    }


    /**
     * @param Database $database
     * @param mixed $value
     * @param bool $canEdit
     * @param bool $insert
     * @param array $rowData
     * @throws DBException
     */
    function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {
        // zobrazi externi PP (zamestnanec = 0)
        // pokud neni nastaven typ rozdeleni vratnice, zobrazuje vsechny
        // pokud je nastaven, zobrazuje jen PP prirazene do nektereho rozdeleni daneho typu
        // a pokud je v SESSION nastavena vratnice, tak zobrazuje jen PP z tohoto konkretniho rozdeleni
        if ($canEdit) {
            $params = $database->createParameters();
            $sql =
                'SELECT PracovniPomer.Id, Osoba.Prijmeni, Karta.OznaceniMedia, Karta.Kod ' .
                ' FROM PrirazeniKarty, PracovniPomer, Osoba, Karta' .
                "\nWHERE PracovniPomer.RC = PrirazeniKarty.RC" .
                ' AND PracovniPomer.OsobniCislo = PrirazeniKarty.OsobniCislo' .
                ' AND PracovniPomer.DatumNastupu = PrirazeniKarty.DatumNastupu' .
                ' AND Osoba.RC = PrirazeniKarty.RC' .
                ' AND Karta.Kod = PrirazeniKarty.KodKarty' .
                // jen externi osoby
                "\nAND PracovniPomer.Zamestnanec = 0" .
                // jen platne PP (jine stejne nejsou v terminalech)
                ' AND PracovniPomer.DatumNastupu <= ' . $params ->addParameter($database ->dateToParam(time())) .
                ' AND (PracovniPomer.DatumUkonceni >= ' . $params ->addParameter($database ->dateToParam(time())) . ' OR PracovniPomer.DatumUkonceni IS NULL)' .
                // jen platne prirazeni karty k aktualnimu datu
                "\nAND PrirazeniKarty.DatumPrideleni <= " . $params ->addParameter($database ->dateToParam(time())) .
                ' AND (PrirazeniKarty.DatumVraceni >= ' . $params ->addParameter($database ->dateToParam(time())) . ' OR PrirazeniKarty.DatumVraceni IS NULL)' .
                // jen platne karty
                "\nAND (Karta.DatumVyrazeni > " . $params ->addParameter($database ->dateToParam(time())) . ' OR Karta.DatumVyrazeni IS NULL)' .
                ' AND (Karta.Blokovana = 0  OR  Karta.Blokovana IS NULL)';

            if (RonPortalSettings::getIntValue('typrozdelenivratnice')) {
                // je zadan typ rozdeleni pro vratnice - externi osoba (karta) musi byt zarazena v nekterem rozdeleni tohoto typu
                $sql .=
                    "\nAND EXISTS (" .
                    '   SELECT * FROM RozdeleniOsob' .
                    '   WHERE RozdeleniOsob.RC = PracovniPomer.RC' .
                    '   AND RozdeleniOsob.OsobniCislo = PracovniPomer.OsobniCislo' .
                    '   AND RozdeleniOsob.DatumNastupu = PracovniPomer.DatumNastupu' .
                    '   AND RozdeleniOsob.KodTypuRozdeleni = ' . $params->addParameter(RonPortalSettings::getIntValue('typrozdelenivratnice')) .
                    // pokud je v session vybrana konkretni vratnice, zobrazi pouze karty z teto vratnice
                    (RonPortalSession::getValue('rozdelenivratnice')
                        ? ' AND RozdeleniOsob.Rozdeleni = ' . $params->addParameter(RonPortalSession::getValue('rozdelenivratnice'))
                        : ''
                    ) .
                    '   AND RozdeleniOsob.PlatnostOd <= ' . $params->addParameter($database ->dateToParam(time())) .
                    '   AND (RozdeleniOsob.PlatnostDo >= ' . $params->addParameter($database ->dateToParam(time())) . ' OR RozdeleniOsob.PlatnostDo IS NULL)' .
                    ')';
            }

            // pouze aktualne neprirazene karty
            $sql .=
                "\nAND NOT EXISTS (" .
                '  SELECT * FROM Navsteva N WHERE (N.osobaRC = PrirazeniKarty.RC AND N.OsobniCislo = PrirazeniKarty.OsobniCislo AND N.DatumNastupu = PrirazeniKarty.DatumNastupu)' .
                '  AND N.Prichod IS NOT NULL' .
                '  AND N.Odchod IS NULL' .
                ($rowData
                    ? '  AND N.Id <> ' . $params ->addParameter((int)$rowData[0])  // kartu prirazenou editovane navsteve zobrazi vzdy
                    : '') .
                ')';

            $sql .=
                ' ORDER BY Osoba.Prijmeni';

            echo "<button id='nacteniID' class='button' type='button' name='filtrovani_osob' >" . ronText("nacist_idMedium") . "</button>";

            $combo = new ComboBox('externiZamestnanec');
            $combo->emptyRow = ronText('nevybrano');
            $combo->className = 'adsfilterforminput_popis';
            $combo->titleIndex = 2;
            $combo->attrIndexes = ['data-oznaceni-media' => 2, 'data-kod-karty' => 3];
            $combo->loadData($database, $sql, $rowData ? $rowData[19] : null, $params); // idpp prirazeneho media
            $combo->render();
        } else {
            if ($rowData[16]) {
                $params = $this->database->createParameters();
                $sql =
                    'SELECT Osoba.Prijmeni, Osoba.Jmeno FROM Osoba' .
                    ' WHERE RC = ' . $params->addParameter($rowData[16]); // OsobaRC
                $cursor = $this->database->getSqlResult($sql, $params);
                $row = $cursor->fetch();
                $cursor->close();

                if ($row) {
                    echo $row[0] . ' ' . ($row[1] ?: '');
                }
            }
        }

        /*
        if ($rowData[31] == "" or $rowData[31] == null) {
            $params = $database->createParameters();
            $sql =
                'SELECT PrirazeniKarty.KodKarty, Osoba.Prijmeni ' .                     //0 - 1
                ' FROM PrirazeniKarty, Osoba, Karta' .
                ' WHERE (PrirazeniKarty.KodKarty = Karta.Kod)' .
                ' AND (Osoba.RC = PrirazeniKarty.RC) AND (Osoba.RC = PrirazeniKarty.RC)' .

                ' AND EXISTS(' .
                '   SELECT P.RC FROM PracovniPomer P' .
                '   WHERE P.RC = Osoba.RC' .
                '   AND P.DatumNastupu <= ' . $params->addParameter($database ->dateToParam(DateUtils::today())) .
                '   AND (P.DatumUkonceni IS NULL OR P.DatumUkonceni >= ' . $params->addParameter($database ->dateToParam(DateUtils::today())) . ')' .
                '   AND (P.Zamestnanec = 0)' .

                ' AND EXISTS (' .
                '   SELECT * FROM RozdeleniOsob' .
                '   WHERE RozdeleniOsob.RC = P.RC AND RozdeleniOsob.osobniCislo = P.osobniCislo AND RozdeleniOsob.datumNastupu = P.datumNastupu' .
                '   AND RozdeleniOsob.KodTypuRozdeleni = ' . $params->addParameter(RonPortalSettings::getIntValue('typrozdelenievidencenavstev')) .
                '   AND RozdeleniOsob.Rozdeleni = ' . $params->addParameter($_SESSION['rozdelenivratnice']) .
                '   AND RozdeleniOsob.PlatnostOd <= ' . $params->addParameter($database ->dateToParam(DateUtils::today())) .
                '   AND (RozdeleniOsob.PlatnostDo >= ' . $params->addParameter($database ->dateToParam(DateUtils::today())) . ' OR RozdeleniOsob.PlatnostDo IS NULL)' .
                ' )' .
                ')' .

                ' AND NOT EXISTS (SELECT * FROM Navsteva N WHERE (N.osobaRC=PrirazeniKarty.RC AND N.osobniCislo = PrirazeniKarty.osobniCislo AND N.datumNastupu = PrirazeniKarty.datumNastupu)' .
                '   AND (prichod is NOT NULL) AND odchod is NULL)' .
                ' ORDER BY Osoba.Prijmeni';

            echo "<button id='nacteniID' class='button' type='button' name='filtrovani_osob' >" . ronText("nacist_idMedium") . "</button>";

            $combo = new ComboBox('externiZamestnanec');
            $combo->emptyRow = ronText('nevybrano');
            $combo->className = 'adsfilterforminput_popis';
            $combo->loadData($database, $sql, $rowData[19] != "" ? $rowData[19] : null, $params);
            $combo->render();

        } else {
            //V poli "Vyhledat dle ID media" se zobrazuje jmeno navstevy pokud se jedna o navstevu zamestnancem firmy (napr. zapomenl medium) momentalne skryji
//            $sql =
//                "SELECT COALESCE(Osoba.Prijmeni,'') + ' ' + COALESCE(Osoba.Jmeno,'') + COALESCE(', ' + Osoba.Titul,'') FROM Osoba WHERE RC = '" . $this->database->addSlashes($rowData[6]) . "'";
//            $celeJmenoZamestnance = DBUtils::getSqlValue($database, $sql);
//            echo $celeJmenoZamestnance;

            $params = $database->createParameters();
            $sql = "SELECT MIN(ID) FROM PracovniPomer WHERE PracovniPomer.RC = " . $params->addParameter($rowData[6]);
            $idZamestnance = DBUtils::getSqlValue($database, $sql, $params);
            echo '<input class="inputPole" type="hidden" name="navstivenaOsoba" value="' . $idZamestnance . '">';
        }
        */
    }
}


/**
 * Class NacteniDokladuNavstevy
 */
class NacteniDokladuNavstevy extends FormField
{
    /**
     * @param Database $database
     * @param mixed $value
     * @param bool $canEdit
     * @param bool $insert
     * @param array $rowData
     */
    function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {
        if (!$rowData  ||  !$rowData[14]) {
            echo
                "<button id='nacteniDokladuPomociDialogu' class='button ctecka-dokladu' type='button' name='nacteniDokladuPomociDialogu'" .
                " data-prijmeni-input='input[name=prijmeni]'" .
                " data-jmeno-input='input[name=jmeno]'" .
                " data-doklad-input='input[name=doklad]'" .
                ">" .
                ronText("nacteni_dokladu") .
                "</button>";
        }
    }
}


/**
 * Class NavstevaForm
 */
class NavstevaForm extends RonPortalForm
{
    /** @var PravaNaOsoby $pravo_schvalit */
    private $pravo_schvalit;

    //private $typnavstevy;

    /** @var Navsteva $navsteva */
    private $navsteva;

    /** @var VydaniKartyNavsteveField */
    private $fVydaniKarty;

    /**
     * NavstevaForm constructor.
     * @param Database  $database
     * @param string    $formAction
     * @param bool      $ocekavana
     * @param Navsteva  $navsteva
     * @throws Exception
     */
    public function __construct($database, $formAction, $ocekavana, $navsteva)
    {
        parent::__construct($database, $formAction);

        $this ->code = 'formnavsteva';
        $this ->htmlParams = ['id' => 'form_navsteva'];

        $prava = Prava::vratPrava('navstevy');;
        $this->pravo_schvalit = Prava::vratPrava('navstevy_schvaleni');

        //$this->htmlAction = '?menuid=navstevy' . (($ocekavana) ? ("&ocekavana=" . $ocekavana) : "");
        $rozdelenivratnice = (isset($_SESSION['rozdelenivratnice']) && $_SESSION['rozdelenivratnice'] != "") ? ("&rozdelenivratnice=" . $_SESSION['rozdelenivratnice']) : "";
        $this->stornoAction = "?menuid=navstevy$rozdelenivratnice";
        //$this->typnavstevy = $typnavstevy;
        $this->navsteva = $navsteva;

        $this->title = $this->isInsert()
            ? ronText($ocekavana ? 'zaznam_ocekavane_navstevy' : 'zaznam_navstevy')
            : ronText('editace_navstevy');

        $this->setXmlEventsName('NAVSTEVA');

        $this->sqlSelectPart =
            'Id, TypNavstevy, Navsteva.kod, Navsteva.titul, Navsteva.prijmeni, Navsteva.jmeno, ZamestnanecRC, ZamestnanecOsobniCislo, ' .   //0 - 7
            ' ZamestnanecDatumNastupu, ZamestnanecTypRozdeleni, ZamestnanecRozdeleni, Firma,' .     // 8 - 11
            ' KodTypuRozdeleni_Firma, Rozdeleni_Firma, Prichod, Odchod,' .                          // 12..15
            ' OsobaRC, OsobniCislo, DatumNastupu, ' .                                               // 16..18 - pp prirazene karty
            // id pracovniho pomeru prirazene externi osoby (karty)
            '(SELECT P.ID FROM PracovniPomer P' .                                                   // 19 - idpp prirazene karty
            ' WHERE P.RC = Navsteva.OsobaRC' .
            ' AND P.OsobniCislo = Navsteva.OsobniCislo' .
            ' AND P.DatumNastupu = Navsteva.DatumNastupu), ' .

            'Vozidlo, NavstivenaOsobaRC, JmenoNavstiveneOsoby, Doklad, ' .                          // 20 - 23
            'NULL, KodTypuRozdeleni, Rozdeleni, ' .                                                 // 24..26
            'PredemOcekavanaNavsteva, NULL, KodTypuRozdeleni_Vratnice, Rozdeleni_Vratnice, ' .                         //27..30
            'PrichodZapsalRC';                                                                      // 31

//            'Navsteva.Uzivatelska1, Navsteva.Uzivatelska2, Navsteva.Uzivatelska3, Navsteva.Uzivatelska4, Navsteva.Uzivatelska5, ' .         //31 - 35
//            'Navsteva.Uzivatelska6, Navsteva.Uzivatelska7, Navsteva.Uzivatelska8, Navsteva.Uzivatelska9, Navsteva.Uzivatelska10, '.         //36 - 40
//            'NULL, NULL, NULL, NULL, PrichodZapsalRC';                                        //41 - 45


           // 'Navsteva.Schvalil, Navsteva.DT_Schvaleni, Navsteva.DuvodNavstevy2, Navsteva.Rozdeleni2, Navsteva.KodTypuRozdeleni2 ';          //47..51

        $this->sqlTableName = 'Navsteva';

        //Hidden
        // ID navstevy
        $this->addField(FormField::FIELD_TYPE_ID)
            ->setFieldName('id') ->setSqlFieldIndex(0) ->setPrimary(true)
            ->setCode('field-id');

//        // Rozdeleni_Vratnice - automaticke vlozeni
//        $this->addField(FORM_FIELD_TYPE_HIDDEN)
//            ->setFieldName('KodTypuRozdeleni_Vratnice') ->setSqlFieldIndex(29)
//            ->setCode('field-kodtypurozdeleni_vratnice')
//            ->setDefaultValue($_SESSION['rozdelenivratnice'] && RonPortalSettings::getIntValue('typrozdelenivratnice') ? RonPortalSettings::getIntValue('typrozdelenivratnice') : null);
//
//        $this->addField(FORM_FIELD_TYPE_HIDDEN)
//            ->setFieldName('Rozdeleni_Vratnice') ->setSqlFieldIndex(30)
//            ->setCode('field-rozdeleni_vratnice')
//            ->setDefaultValue($_SESSION['rozdelenivratnice'] && RonPortalSettings::getIntValue('typrozdelenivratnice') ? $_SESSION['rozdelenivratnice'] : null);


//        //Kod KodTypuRozdeleni_Firma - automaticke vlozeni
//        $this->addField(FORM_FIELD_TYPE_HIDDEN)
//            ->setFieldName('KodTypuRozdeleni_Firma') ->setSqlFieldIndex(12)
//            ->setCode('field-kodtypurozdeleni_firma')
//            ->setDefaultValue(RonPortalSettings::getIntValue('typrozdelenifirma'));

        // vratnice
        $this->addField(new VratniceField($database, RonPortalSession::getValue('rozdelenivratnice')))
            ->setRowId('navsteva-row-vratnice') ->setReadOnly(true) ->setSqlFieldIndex(30);

        if (RonPortalSettings::getIntValue('navstevyupozorneniknavsteve')) {
            $this ->addField(FormField::FIELD_TYPE_TEXT)
                ->setCode('field-upozorneni')
                ->setCaption('upozorneni') ->setRowId('navsteva-row-upozorneni')
                ->setText(
                    '<div class="navstevy-upozorneni-klicove font-weight-bold">' .
                    RonPortalSettings::getValue('navstevy_text_upozorneni_klicovy') .
                    "</div>" .
                    '<div>' .
                    RonPortalSettings::getValue('navstevy_text_upozorneni') .
                    "</div>"
                );
        }

        // navstivena osoba
        $this->addField(FormField::FIELD_TYPE_SECTION_HEADER)
            ->setCode('sekce-navstivenaosoba') ->setCaption('navstivena_osoba');

            // $this->addField(new NavstivenaOsobaRozdeleniField($database, true, 26, $this->htmlAction . '&formaction=new' . '&ocekavana=' .$ocekavana));

        $this->addField(new NavstivenaOsobaField($database, 21))
            ->setCode('field-navstivenaosoba');

        if ($this->isInsert()) {
            //$this->addField(new TelefonField());
            $this->addField(FormField::FIELD_TYPE_TEXT)
                ->setCaption('telefon') ->setFieldId('telefonNavstiveneOsoby')
                ->setCode('field-telefon') ->setRowId('navsteva-row-telefon');
        }

        // typ navstevy
        $this->addField(FormField::FIELD_TYPE_SECTION_HEADER)
            ->setCode('sekce-typnavstevy') ->setCaption('typ_navstevy');

        $this->addField(new TypNavstevyField())
            ->setFieldName('TypNavstevy') ->setDataType('int')
            ->setSqlFieldIndex(1) ->setReadOnly(true) ->setRequired(true);

        // zamestnanec
//        $this->addField(FORM_FIELD_TYPE_SECTION)
//            ->setCode('sekce-navstevazamestnanec') ->setCaption('navsteva_zamestnanec_jako_navsteva');

        $this->addField(new ZamestnanecField($database, 'zamestnanecNavsteva'));

        if(!$this->navsteva  ||  !$this->navsteva->odchod) {
            //Vydani karty navsteve
            $this->addField(FormField::FIELD_TYPE_SECTION_HEADER)
                ->setCode('sekce-vydanikarty') ->setCaption('vydani_karty_navsteve')
                ->setVisible(!$prava ->priznak); // nezobrazi, pokud muze zadavat jen ocekavane

            $this->addField($this ->fVydaniKarty = new VydaniKartyNavsteveField($database))
                ->setCode('field-karta')
                ->setRowId('navsteva-row-karta')
                ->setVisible(!$prava ->priznak); // nezobrazi, pokud muze zadavat jen ocekavane
        }

        // informace o navsteve
        $this->addField(FormField::FIELD_TYPE_SECTION_HEADER)
            ->setCode('sekce-navsteva') ->setCaption('navsteva');

        // Ocekavany prichod
        $this->addField($f = new CasovePoleNavstevyField(FormField::FIELD_TYPE_DATEANDTIME))
            ->setFieldName('OcekavanyPrichod')
            ->setCode('field-ocekavanyprichod')
            ->setCaption('ocekavany_prichod');
        $f ->editovatPoPrichodu = false; // kdyz je zadan prichod, tak uz nelze editovat ocekavany prichod

        // Prichod
        $this->addField($f = new CasovePoleNavstevyField(FormField::FIELD_TYPE_DATEANDTIME))
            ->setFieldName('Prichod')->setSqlFieldIndex(14)
            ->setCode('field-prichod')
            ->setCaption('prichod')->setFieldId('field-prichod')->setCurrentTimeButton(true)
            ->setVisible(!$prava ->priznak); // nezobrazi prichod a odchod, pokud muze zadavat jen ocekavane navstevy

        // Odchod
        $this->addField($f = new CasovePoleNavstevyField(FormField::FIELD_TYPE_DATEANDTIME))
            ->setFieldName('Odchod')->setSqlFieldIndex(15)
            ->setCode('field-odchod')
            ->setCaption('odchod')->setFieldId('field-odchod')->setCurrentTimeButton(true)
            ->setVisible(!$prava ->priznak);
        $f->editovatPoOdchodu = true;

        // Nacteni dokladu navstevy
        $this->addField(new NacteniDokladuNavstevy($database))
            ->setVisible(!$prava ->priznak); // nezobrazi, pokud muze zadavat jen ocekavane

        // Kod
        $this->addField(new PoleNavstevyField($database, 'kod', 2, 20))
            ->setCode('field-kod') ->setCaption('Kod');

        // Titul
        $this->addField(new PoleNavstevyField($database, 'titul', 3, 20))
            ->setCode('field-titul') ->setCaption('Titul');

        // Prijmeni
        $this->addField(new PoleNavstevyField($database, 'prijmeni', 4, 30))
            ->setCode('field-prijmeni') ->setCaption('Prijmeni') ->setRequired(true)
            ->setOtherHtmlParams(['data-title' => ronText('Prijmeni'), 'maxlength' => '']);

        // Jmeno
        $this->addField(new PoleNavstevyField($database, 'jmeno', 5, 30))
            ->setCode('field-jmeno') ->setCaption('Jmeno');

        // Doklad
        $this->addField(new PoleNavstevyField($database, 'doklad', 23, 20))
            ->setCode('field-doklad') ->setCaption('doklad')
            ->setVisible(!$prava ->priznak); // nezobrazi, pokud muze zadavat jen ocekavane

        // Vozidlo
        $this->addField(new PoleNavstevyField($database, 'vozidlo', 20, 10))
            ->setCode('field-vozidlo') ->setCaption('vozidlo');

        // Firma
        $this->addField(new FirmaField($database, 'firma', 11))
            ->setCode('field-firma') ->setCaption('firma');

        $this ->addField(FormField::FIELD_TYPE_DATE)
            ->setFieldName('DatumAbsolvovaniSkoleni') ->setCaption('Datum_absolvovani_skoleni_navstevy')
            ->setCode('field-datumabsolvovaninavstevy')
            ->setVisible(RonPortalSettings::getValue('platnost_skoleni_navstevy', '') !== '')
            ->setFieldId('datum_absolvovani_skoleni')
            ->setOtherHtmlParams(['data-maximalni-platnost' => RonPortalSettings::getValue('platnost_skoleni_navstevy', '')]);

        if (RonPortalSettings::getIntValue('platnost_skoleni_navstevy')) {
            $this ->addField(FormField::FIELD_TYPE_TEXT)
                ->setText(TextManager::_('Platnost_skoleni_navstevy_vyprsela'))
                ->setFieldCssClass('errormsg')
                ->setRowId('Platnost_skoleni_vyprsela_row')
                ->setRowClass('row d-none'); // zobrazi se javascriptem az po nacteni dokladu, v pripade, ze uz skoleni neni platne
        }

        // Rozdeleni 2
        $kodTypuRozdeleni2 = RonPortalSettings::getIntValue('navstevy_typrozdeleni2');
        $typRozdeleni2 = ($kodTypuRozdeleni2)
            ? TypRozdeleni::getTypRozdeleni($database, $kodTypuRozdeleni2)
            : null;

        if ($typRozdeleni2) {
            $this ->addField($f = new RozdeleniFormField($typRozdeleni2->kod))
                ->setFieldName('Rozdeleni2')
                ->setCaption($typRozdeleni2->popis);
            $f ->setTypRozdeleniFieldName('KodTypuRozdeleni2');

//            $this->addField(new NavstevaFormField(FormField::FIELD_TYPE_HIDDEN))
//                ->setFieldName('KodTypuRozdeleni2') ->setSqlFieldIndex(51)
//                ->setCode('field-kodtypurozdeleni2')
//                ->setDefaultValue($typRozdeleni2->kod);
//
//            $params = $database ->createParameters();
//
//            $this->addField(new NavstevaFormField(FormField::FIELD_TYPE_SELECT))
//                ->setFieldName('Rozdeleni2') ->setSqlFieldIndex(50) ->setRequired(false)
//                ->setCode('field-rozdeleni2')
//                ->setCaption($typRozdeleni2->popis)      //FORM_FIELD_ROW_ID => 'rozdeleni2',
//                ->setShowEmptyValue(true)
//                ->setSelectSql(new SqlStatement(
//                    'SELECT Kod, Popis, cssClass FROM Rozdeleni' .
//                    ' WHERE KodTypuRozdeleni = ' . $params ->addParameter((int)$typRozdeleni2->kod) .
//                    '   AND (DatumZruseni >= ' . $params ->addParameter($this->database->dateToParam(time())) . ' OR DatumZruseni IS NULL)' .
//                    " AND (cssClass IS NULL  OR  cssClass <> 'hidden')",
//                    $params));
        }

        // Prichod zapsal
        $this->addField(new PrichodZapsalField($database))
            ->setCode('field-prichodzapsal') ->setRowId('navstava-row-prichod-zapsal')
            ->setCaption("prichod_zapsal");


        // Duvod navstevy 2
        $this->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('DuvodNavstevy2') //->setSqlFieldIndex(49)
            ->setSelectSql('SELECT Kod, Popis FROM DuvodNavstevy ORDER BY Popis') ->setShowEmptyValue(true)
            ->setCaption('duvod_navstevy_ciselnik')
            ->setCode('field-duvodnavstevy2');

        // Duvod navstevy
        $this->addField(FormField::FIELD_TYPE_MEMO)
            ->setFieldName('DuvodNavstevy') ->setCaption('duvod_navstevy')
            ->setCode('field-duvodnavstevy');


        if ($this->pravo_schvalit->jsouNejakePrava()){
            $this->addField(FormField::FIELD_TYPE_SECTION_HEADER)
                ->setCode('sekce-schvaleni') ->setCaption('schvaleni_navstevy');

            $this->addField(FormField::FIELD_TYPE_TEXT)
                ->setCode('field-schvalil') ->setCaption('Schvalil') ->setText($navsteva ? $navsteva->navstevuSchvalil : '');

            $this ->addField(FormField::FIELD_TYPE_TEXT)
                ->setCode('field-datumschvaleni') ->setCaption('datum_schvaleni') ->setText($navsteva ? StringUtils::format('date',$navsteva->navstevuSchvalilDatum) : '');
        }


        // ostatni udaje
        $jeSekceOstatni = false;
        for ($i = 1; $i <= 10; $i++) {
            if (RonPortalSettings::getValue('navstevy_uzivatelska'.$i)) {
                if (!$jeSekceOstatni) {
                    $this->addField(FormField::FIELD_TYPE_SECTION_HEADER)
                        ->setCode('sekce-ostatni') ->setCaption('ostatni_udaje');

                    $jeSekceOstatni = true;
                }

                $this->addField(new NavstevaFormField())
                    ->setFieldName('Uzivatelska' . $i) ->setMaxLength(100)
                    ->setCaption(RonPortalSettings::getValue('navstevy_uzivatelska' . $i))
                    ->setCode('field-uzivatelska' . $i)
                    ->setReadOnly($navsteva  &&  $navsteva->odchod);

//                $this->addField(new UzivatelskaPoleNavstev($this->database, RonPortalSettings::getValue('navstevy_uzivatelska' . $i),'Uzivatelska' . $i,30+$i, $navsteva));
//                $this->database = $database;
//                $this->editable = (!$navsteva->odchod);
            }
        }

        // editace navstevy - uzivatel a datum
        $this->addField(new UserLoginFormField(false, 'Update_UserName')) ->setUpdateLog(true)
            ->setCode('field-update_username');

        $this->addField(FormField::FIELD_TYPE_ACTUAL_TIME)
            ->setFieldName('Update_Timestamp') ->setUpdateLog(true)
            ->setCode('field-update_timestamp');

        // vlozeni navstevy - uzivatel a datum
        $this->addField(new UserLoginFormField(true, 'Insert_UserName'))
            ->setCode('field-insert_username');

        $this->addField(FormField::FIELD_TYPE_ACTUAL_TIME)
            ->setFieldName('Insert_Timestamp') ->setReadOnly(true)
            ->setCode('field-insert_timestamp');
    }


    private function zkontrolujZaznam($rec)
    {
        if (!$rec['OcekavanyPrichod']  &&  !$rec['Prichod']) {
            throw new Exception(ronText('prichod_nebo_OcekavanyPrichod'));
        }

        if (RonPortalSettings::getIntValue('platnost_skoleni_navstevy')) {
            if ($rec['Prichod'] && $rec['Prichod'] !== $rec['#Prichod']) {
                // doslo k zadani nebo editaci Prichodu => zkontroluje datum skoleni
                if (!$rec['DatumAbsolvovaniSkoleni']  ||  DateUtils::incDay($rec['DatumAbsolvovaniSkoleni'], RonPortalSettings::getIntValue('platnost_skoleni_navstevy')) < DateUtils::today()) {
                    throw new RonException(TextManager::_('Platnost_skoleni_navstevy_vyprsela'), 'DatumAbsolvovaniSkoleni');
                }
            }
        }
    }

    /**
     * @param $rec
     * @return bool
     * @throws Exception
     */
    function beforeInsertRecord($rec)
    {
//        if (!isset($_POST['prijmeni'])  ||   !$_POST['prijmeni']) {
//            throw new Exception( ronText('prijmeni') . ' ' . ronText('nevybrano'));
//        }

        $this ->zkontrolujZaznam($rec);

//        if(($_POST['Prichod'] == '' || $_POST['Prichod_time'] == '') and ($_POST['OcekavanyPrichod'] == '' || $_POST['OcekavanyPrichod_time'] == '')){
//            throw new Exception( ronText('prichod_nebo_OcekavanyPrichod'));
//        }

        return parent::beforeInsertRecord($rec);
    }


    function beforeUpdateRecord($rec)
    {
        $this ->zkontrolujZaznam($rec);
        return parent::beforeUpdateRecord($rec); // TODO: Change the autogenerated stub
    }


    private function nastavEditaciPoPrichoduAOdchodu($prichod, $odchod)
    {
        if ($prichod  ||  $odchod) {
            foreach ($this->fields as $field) {
                if ($field instanceof NavstevaFormField) {
                    if (!$field->editovatPoPrichodu && $prichod) {
                        $field->readOnly = true;
                    } else if (!$field->editovatPoOdchodu && $odchod) {
                        $field->readOnly = true;
                    }
                }
            }
        }
    }


    function loadRecord()
    {
        $row = parent::loadRecord();
        $this ->nastavEditaciPoPrichoduAOdchodu($row[14], $row[15]);

        return $row;
    }


    function editRecord()
    {
        // zjisti, jestli je v puvodnim zaznamu vyplnen prichod a odchod
        $params = $this ->database ->createParameters();
        $cursor = $this ->database ->getSqlResult(
            'SELECT Prichod, Odchod FROM Navsteva WHERE Id = ' . $params ->addParameter((int)$_POST['id']),
            $params);

        if ($row = $cursor ->fetch()) {
            $this ->nastavEditaciPoPrichoduAOdchodu($row[0], $row[1]);
        }

        $cursor ->close();

        return parent::editRecord();
    }


    function createDBRecordFields($rec, $insert)
    {
        parent::createDBRecordFields($rec, $insert);

        $rozdeleniVratnice = (RonPortalSettings::getIntValue('typrozdelenivratnice')  &&  RonPortalSession::getValue('rozdelenivratnice'))
            ? RonPortalSession::getValue('rozdelenivratnice')
            : null;

        $rec ->addIntField('KodTypuRozdeleni_Vratnice', true, isset($rozdeleniVratnice) ? RonPortalSettings::getIntValue('typrozdelenivratnice') : null);
        $rec ->addStringField('Rozdeleni_Vratnice', true, 30, $rozdeleniVratnice);

        if ($this ->isInsert()) {
            $pracPomerNavstiveneOsoby = PracovniPomer::nactiPP($this->database, (int)$_POST['navstivenaOsoba']);

            $rec->addStringField('navstivenaOsobaRC', true, 15, $pracPomerNavstiveneOsoby->rc);
            $rec->addStringField('jmenoNavstiveneOsoby', true, 60,
                ($pracPomerNavstiveneOsoby->titul ? $pracPomerNavstiveneOsoby->titul . ' ' : '') .
                $pracPomerNavstiveneOsoby->jmeno . ' ' . $pracPomerNavstiveneOsoby->prijmeni);

            if (RonPortalSettings::getValue('typrozdelenievidencenavstev')) {
                // Nacte rozdeleni navstivene osoby
                $rozdeleniOsoby = RozdeleniOsob::nactiRozdeleniOsoby($this->database, $pracPomerNavstiveneOsoby, RonPortalSettings::getValue('typrozdelenievidencenavstev'), time());

                if ($rozdeleniOsoby) {
                    //Kod typu rozdeleni navstivene osoby = za kym.
                    $rec->addField('KodTypuRozdeleni', 'int', true, 0, $rozdeleniOsoby[0] ->kodTypuRozdeleni);
                    //Kod rozdeleni navstivene osoby = za kym.
                    $rec->addField('Rozdeleni', '', true, 30, $rozdeleniOsoby[0] ->kod);
                }
            }
        } else {
            $rec->addStringField('navstivenaOsobaRC', true, 15);
        }

        // navsteva zamestnance - doplni zamestnance a jeho rozdeleni
        if (($_POST['TypNavstevy'] ?? 0) == 1) { // neni vzdy zadano, protoze nekdy je editace nekterych poli omezena
            $zamRCNavsteva = PracovniPomer::nactiPP($this->database, (int)$_POST['zamestnanecNavsteva']);
            $rec->addStringField('zamestnanecRC', true, 15, $zamRCNavsteva->rc);
            $rec->addStringField('zamestnanecOsobniCislo', true, 15, $zamRCNavsteva->osobniCislo);
            $rec->addDateField('zamestnanecDatumNastupu', true, $zamRCNavsteva->datumNastupu);

            if (RonPortalSettings::getIntValue('typrozdelenievidencenavstev')) {
                $r = RozdeleniOsob::nactiRozdeleniOsoby($this->database, $zamRCNavsteva, RonPortalSettings::getIntValue('typrozdelenievidencenavstev'), time());

                if ($r) {
                    $rec->addIntField('zamestnanecTypRozdeleni', true, $r[0] ->kodTypuRozdeleni);
                    $rec->addStringField('zamestnanecRozdeleni', true, 30, $r[0] ->kod);
                }
            }
        }

        // Prirazeni externi osoby (karty)
        if ($this ->fVydaniKarty  &&  $this ->fVydaniKarty ->visible  &&  !$this ->fVydaniKarty ->readOnly  &&  isset($_POST['externiZamestnanec'])) {
            $externiOsoba = null;

            if ($_POST['externiZamestnanec']) {
                $params = $this->database->createParameters();
                $sql =
                    'SELECT RC, OsobniCislo, DatumNastupu' .
                    ' FROM PracovniPomer' .
                    ' WHERE Id = ' . $params->addParameter((int)$_POST["externiZamestnanec"]);

                $cursor = $this->database->getSqlResult($sql, $params);
                $externiOsoba = $cursor->fetch();
                $cursor->close();
            }

            $rec->addField('OsobaRC', '', false, 15, $externiOsoba ? $externiOsoba[0] : null);
            $rec->addField('OsobniCislo', '', false, 15, $externiOsoba ? $externiOsoba[1] : null);
            $rec->addField('DatumNastupu', 'timestamp', false, 0, $this->database->stringToTimestamp($externiOsoba ? $externiOsoba[2] : null));
        }

        //Pokud je zadan cas ocekavane navstevy je automaticky nastaven priznak na PredemOcekavanaNavsteva
        $rec->addField('PredemOcekavanaNavsteva', 'int', true, 0,  (($_POST['OcekavanyPrichod'] ?? false) ? 1 : 0 ));

        if (RonPortalSettings::getIntValue('typrozdelenifirma')) {
            $params = $this->database->createParameters();
            $rozdeleniFirma = DBUtils::getSqlValue($this->database,
                'SELECT Kod FROM Rozdeleni' .
                ' WHERE KodTypuRozdeleni = ' . $params ->addParameter(RonPortalSettings::getIntValue('typrozdelenifirma')) .
                ' AND Popis  = ' . $params->addParameter($_POST['firma']), $params);

            $rec->addIntField('KodTypuRozdeleni_Firma', false, $rozdeleniFirma ? RonPortalSettings::getIntValue('typrozdelenifirma') : null);
            $rec->addStringField('Rozdeleni_Firma', false, 30, $rozdeleniFirma ?: null);
        }

        //Prichod zapsal
        if (($_POST['Prichod'] ?? '') != '') { // neni vzdy zadano, protoze nekdy je editace nekterych poli omezena
            $rec->addField('PrichodZapsalRC', '', false, 30, User::getRC());
        }
    }
}

