<?php
/**
 * Roman.Szablatura RON Software 27.3.2017
 */

namespace RonPortal\Common;

use Exception;
use Ron\Controls\BrowserField;
use Ron\DB\DBException;
use Ron\DB\SqlStatement;
use Ron\DB\Database;
use Ron\DateUtils;
use Ron\GuiUtils;
use Ron\Http\Csrf;
use Ron\TextManager;
use RonPortal\Controls\FilterFormFilter;
use RonPortal\Controls\FilterFormFilterDatumOdDo;
use RonPortal\Controls\FilterFormFilterItem;
use RonPortal\Controls\RonPortalBrowser;
use RonPortal\Controls\RonPortalFilterForm;
use RonPortal\Prava;
use RonPortal\PravaNaOsoby;
use RonPortal\RonPortalSession;
use RonPortal\RonPortalSettings;


/**
 * Class NavstevyFilter
 */
class NavstevyFilter extends RonPortalFilterForm
{
    /** @var  PravaNaOsoby $prava */
    public $prava;

    public $omezeniVratnice = 1; // 0=vsechny, 1=pouze moje

    /**
     * NavstevyFilter constructor.
     * @param Database $database
     */
    public function __construct(Database $database)
    {
        parent::__construct($database, 'navstevy', true);

        $this->prava = Prava::vratPrava('navstevy');
        Prava::nastavObdobiProPrava(time(), time());

        $this ->addFilter(
            (new FilterFormFilterDatumOdDo($this))
                ->setPovolitPrazdneDatumDo(true)
                ->setRequired(true)
                ->setDefaultValue(DateUtils::today())
                ->setSessionName('navstevy_datum')
                ->addLink([
                        'datumOd' => DateUtils::today(),
                        'datumDo' => DateUtils::today(),
                        'text' => TextManager::_('Navstevy_pro_dnesni_den')
                    ]
                )
                ->addLink([
                        'datumOd' => DateUtils::today(),
                        'datumDo' => null,
                        'text' => TextManager::_('Navstevy_aktualni_a_budouci')
                    ]
                )
                ->setCaption(TextManager::_('Vybrane_obdobi_od'))
        );

//        $this ->nastavDatumOdDo(true, true, TextManager::_('Vybrane_obdobi_od'));
//        $this ->datumOdDoSessionPrefix = 'navstevy_';
//        $this ->defaultDatumOdDo = DateUtils::today();
//        $this ->povolitPrazdneDatumDo = true;
//        $this ->datumOdDoLinks = [
//            [
//                'datumOd' => DateUtils::today(),
//                'datumDo' => DateUtils::today(),
//                'text' => TextManager::_('Navstevy_pro_dnesni_den')
//            ],
//            [
//                'datumOd' => DateUtils::today(),
//                'datumDo' => null,
//                'text' => TextManager::_('Navstevy_aktualni_a_budouci')
//            ],
//        ];

        // Vytvori jednotlive filtry
        // vsechny vratnice / pouze moje vratnice
        $filtrVratnice = new FilterFormFilter('omezeniVratnice');
        $filtrVratnice
            ->setSessionName('navstevy_omezenivratnice')
            ->setRowId('filterrow_omezenivratnice')
            ->addItem('0', TextManager::_('vsechnyVratnice'))
            ->addItem( (new FilterFormFilterItem('1', TextManager::_('pouzeMojeVratnice'))) );
        $this ->addFilter($filtrVratnice, true);
        $filtrVratnice ->load($this ->database);


        $omezeniNavstevFilter = new FilterFormFilter('omezeniNavstev');
        $omezeniNavstevFilter
            //->setSessionName('omezeniNavstev')
            ->setRowId('filterrow_omezeninavstev')
            ->addItem('0', TextManager::_('zobrazit_navstevy_za_obdobi'))
            ->addItem('1', TextManager::_('jenOcekavany'))
            ->addItem('2', TextManager::_('pritomni'))
            ->setDefaultValue('0');
        $this ->addFilter($omezeniNavstevFilter, true);

//        //Combo volby dni a datumu
//        $zvolenaDoba =
//            TextManager::_('zvolena_doba') . ': ' .

        //$this->f_formaction = '?menuid=navstevy&';
//        $this->f_classname = 'filtercontainer';

        // jmeno navstivene osoby
        $filter = new FilterFormFilter('osoba', FilterFormFilter::TYPE_EDIT);
        $filter
            ->setCaption(TextManager::_('podle_navstivene_osoby'))
            ->setSqlFilter('Navsteva.JmenoNavstiveneOsoby LIKE ')
            ->setSessionName('navstivenaOsoba')
            ->setRowId('filterrow_osobanavstevy');
        $this ->addFilter($filter);

        // prijmeni navstevy
        $filterPrijmeni = new FilterFormFilter('prijmeni', FilterFormFilter::TYPE_EDIT);
        $filterPrijmeni
            ->setCaption(TextManager::_('podle_prijmeni'))
            ->setSqlFilter('Navsteva.Prijmeni LIKE ')
            ->setSessionName('prijmeniNavstevy')
            ->setRowId('filterrow_prijmeninavstevy');
        $this ->addFilter($filterPrijmeni);

        // typ navstevy (zamestnanec/cizi)
        $filtrTyp = new FilterFormFilter('typ', FilterFormFilter::TYPE_OPTIONS);
        $filtrTyp
            ->setCaption(TextManager::_('podle_typu_navstevy'))
            ->setItems(['' => TextManager::_('_vse_'), 0 => TextManager::_('cizi'), 1 => TextManager::_('vlastni')])
            ->setSqlFilter('Navsteva.TypNavstevy = ')
            ->setSessionName('typNavstevy')
            ->setRowId('filterrow_typnavstevy');
        $this ->addFilter($filtrTyp);

        // firma
        $filterFirma = new FilterFormFilter('firma', FilterFormFilter::TYPE_EDIT);
        $filterFirma
            ->setCaption(TextManager::_('podle_firmy'))
            ->setSqlFilter('Navsteva.Firma LIKE ')
            ->setSessionName('firmaNavstevy')
            ->setRowId('filterrow_firmanavstevy');
        $this ->addFilter($filterFirma);

        // schvaleni navstevy (vse/neschvalene/schvalene)
        $filtrSchvaleni = new FilterFormFilter('schvaleni', FilterFormFilter::TYPE_OPTIONS);
        $filtrSchvaleni
            ->setCaption(TextManager::_('podle_schvaleni'))
            ->setItems([
                '' => new FilterFormFilterItem('', TextManager::_('_vse_')),
                '0' => (new FilterFormFilterItem('0', TextManager::_('navsteva_neschvalena'))) ->setSqlFilter('Navsteva.Schvalil IS NULL'),
                '1' => (new FilterFormFilterItem('1', TextManager::_('navsteva_schvalena'))) ->setSqlFilter('Navsteva.Schvalil IS NOT NULL')
            ])
            // Navsteva.Schvalil IS NOT NULL
            ->setSessionName('schvaleniNavstevy')
            ->setRowId('filterrow_schvaleninavstevy');
        $this ->addFilter($filtrSchvaleni);


        // vyhledani podle ID media
        $params = $this->database->createParameters();
        $sql =
            'SELECT DISTINCT Karta.Kod, Karta.OznaceniMedia' .
            ' FROM Navsteva LEFT JOIN PrirazeniKarty' .
            ' ON (Navsteva.osobaRC = PrirazeniKarty.RC' .
            '  AND Navsteva.osobniCislo = PrirazeniKarty.osobniCislo' .
            '  AND Navsteva.datumNastupu = PrirazeniKarty.datumNastupu)' .
            ' LEFT JOIN Karta ON (Karta.Kod = PrirazeniKarty.kodKarty)' .
            ' WHERE Navsteva.Prichod IS NOT NULL' .
            ' AND Navsteva.Odchod IS NULL AND Karta.Kod IS NOT NULL';

        if ($filtrVratnice ->getValue()  &&  RonPortalSession::getValue('rozdelenivratnice')) {
            $sql .= ' AND Navsteva.Rozdeleni_Vratnice = ' . $params->addParameter(RonPortalSession::getValue('rozdelenivratnice'));
        }

        $sql .= ' ORDER BY Karta.OznaceniMedia';

        $filtrKarta = new FilterFormFilter('medium', FilterFormFilter::TYPE_OPTIONS);
        $filtrKarta
            ->setCaption(TextManager::_('vyhledat_zapujcena_media'))
            ->setItems(['' => new FilterFormFilterItem('', TextManager::_('_vse_'))])
            ->setOptionsSql(new SqlStatement($sql, $params))
            ->setSessionName('mediumNavstevy')
            ->setRowId('filterrow_mediumnavstevy')
            ->setCaptionHtmlAttributes(['title' => TextManager::_('filtr_zapujceneho_media')]);
        $this ->addFilter($filtrKarta);

        $filtrDuvod = new FilterFormFilter('duvod', FilterFormFilter::TYPE_OPTIONS);
        $filtrDuvod
            ->setCaption(TextManager::_('podle_duvodu_navstevy'))
            ->setItems(['' => new FilterFormFilterItem('', TextManager::_('_vse_'))])
            ->setOptionsSql('SELECT Kod, Popis FROM DuvodNavstevy WHERE %FILTER% ORDER BY %ORDER%', '1 = 1', 'Popis')
            ->setSessionName('duvodNavstevy')
            ->setRowId('filterrow_duvodnavstevy')
            ->setSqlFilter('Navsteva.duvodNavstevy2 = ');
        $this ->addFilter($filtrDuvod);

        $filtrSchvaleni = new FilterFormFilter('svozidlem', FilterFormFilter::TYPE_OPTIONS);
        $filtrSchvaleni
            ->setCaption(TextManager::_('podle_typu_vozidla'))
            ->setItems([
                '' => new FilterFormFilterItem('', TextManager::_('_vse_')),
                '0' => (new FilterFormFilterItem('0', TextManager::_('bez_vozidla'))) ->setSqlFilter('Navsteva.Vozidlo IS NULL'),
                '1' => (new FilterFormFilterItem('1', TextManager::_('s_vozidlem'))) ->setSqlFilter('Navsteva.Vozidlo IS NOT NULL')
            ])
            // Navsteva.Schvalil IS NOT NULL
            ->setSessionName('FiltrNavstevySVozidlem')
            ->setRowId('filterrow_vozidlonavstevy]');
        $this ->addFilter($filtrSchvaleni);

        $filtrZobrazitPoznamky = new FilterFormFilter('poznamky', FilterFormFilter::TYPE_CHECKBOX);
        $filtrZobrazitPoznamky
            ->setSessionName('uzivatelskePoznamkyNavstev')
            ->setCaption2(TextManager::_('navstevyUzivatelskePoznamky'));
        $this ->addFilter($filtrZobrazitPoznamky);

    }

    function loadData()
    {
        parent::loadData();
        $this ->omezeniVratnice = (int)$this ->getFilterValue('omezeniVratnice');
    }
}


/**
 * Class TypNavstevy
 */
class TypNavstevy extends BrowserField
{

    /**
     * TypNavstevy constructor.
     * @param Database $database
     * @param $options
     */
    public function __construct($database, $options)
    {
        parent::__construct($database, $options);
        $this->header = TextManager::_('typ_navstevy');
        $this->ordering = 1;
        $this->align = 'center';
        $this->sortSQL = 'TypNavstevy, Navsteva.OcekavanyPrichod, prichod';
        $this->sortSQLDesc = 'TypNavstevy DESC, Navsteva.OcekavanyPrichod DESC, prichod DESC';
        $this->sqlFieldIndex = 1;
        $this->sqlFieldName = 'TypNavstevy';
    }

    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        return
            '<a href="?menuid=navstevy&formaction=edit&record=' . (int)$rowData[0] . '" title="' . TextManager::_($rowData[1] ? 'vlastni' : 'cizi') . '">' .
            ($rowData[1] ? '<strong>Z</strong>' : '-->') .
            '</a>';
    }
}

/**
 * Class CasNavstevy
 */
class CasNavstevyBrowserField extends BrowserField
{
    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return int|string
     * @throws DBException
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = parent::getViewText($value, $rowNumber, $recordId, $rowData);

        $time = $this->database->stringToTimestamp($value);
        if (DateUtils::startOfTheDay($time) == DateUtils::today()) {
            $result =
                '<strong>' . $result . '</strong>';
        }

        return $result;
    }
}


/**
 * Class PotvrzeniOdchoduNavstevyField
 */
class PotvrzeniOdchoduNavstevyField extends BrowserField
{
    var $lzeZapsatOdchod;

    /**
     * PotvrzeniPrijetiField constructor.
     * @param Database $database
     * @param array|null $options
     */
    public function __construct($database, $options)
    {
        parent::__construct($database, $options);
        $this->lzeZapsatOdchod = !Prava::vratPrava('navstevy') ->priznak;
    }

    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        // Jestlize je zadan prichod a odchod jeste ne
        if (!$rowData[$this ->sqlFieldIndex]  &&  $this ->lzeZapsatOdchod  &&  $rowData['col-prichod']) {
            return GuiUtils::changeLocationQuest(
                TextManager::_('vlozit_odchod_navstevy'), TextManager::_('opravdu_ukoncit_navstevu'),
                '?menuid=navstevy&action=potvrzeniodchodu&id=' . $rowData[0]);
        }

        return parent::getViewText($value, $rowNumber, $recordId, $rowData);
    }
}


/**
 * Class NavstevuSchvalilaOsoba
 */
class NavstevuSchvalilaOsoba extends BrowserField
{
    /** @var PravaNaOsoby $pravo_schvaleni */
    private $pravo_schvaleni;


    /**
     * NavstevuSchvalilaOsoba constructor.
     * @param Database $database
     */
    public function __construct($database)
    {
        parent::__construct($database);
        $this->pravo_schvaleni = Prava::vratPrava('navstevy_schvaleni');
    }


    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     * @throws Exception
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if (!$value) {
            // jeste neschvaleno
            $rcNavstiveneOsoby = $rowData[2]; // Navsteva.NavstivenaOsobaRC
            $maPravoNaOsobu = $this->pravo_schvaleni->lzeZobrazitRC($rcNavstiveneOsoby);
            if ($maPravoNaOsobu && $rowData['col-ocekavany']) {
                return GuiUtils::changeLocationQuest(TextManager::_('Schvalit'), TextManager::_('opravdu_schvalujete_navstevu'),
                    ' ?menuid=navstevy&action=schvaleninavstevy&id=' . $rowData[0] .
                    '&browser=1' . Csrf::csrfUrlParameter());
            }

            // neni pravo schvalovat
            return '';
        }

        return parent::getViewText($value, $rowNumber, $recordId, $rowData);
    }
}



/**
 * Class NavstevyBrowser
 */
class NavstevyBrowser extends RonPortalBrowser
{

    /** @var  PravaNaOsoby $prava */
    private $prava;

    /**
     * NavstevyBrowser constructor.
     * @param Database $database
     * @throws Exception
     */
    public function __construct(Database $database)
    {
        parent::__construct($database, 'navstevy');

        Prava::nastavObdobiProPrava(time(),time());
        $this->prava = Prava::vratPrava('navstevy');

        $this->code = 'browsernavstevy';
        $this->tableId = 'browsernavstevy';

        $this ->maxPageRows = 200;

        $omezeniVratnice = (int)($_GET['omezeniVratnice'] ?? $_SESSION['navstevy_omezenivratnice'] ?? 0);
        $this->title = TextManager::_('navstevy') . ' ' .
            ($omezeniVratnice == 1
                ? $this->rozdeleniVratnice($database)
               : (!$omezeniVratnice ? TextManager::_('vsechnyVratnice') : '')
            );

        $this->sqlSelectPart =
            'Navsteva.Id, TypNavstevy, Navsteva.NavstivenaOsobaRC';   //0 - 2

        $this->primaryField = 0;

        $this->sqlFromPart =
            'Navsteva LEFT JOIN Osoba NavstivenaOsoba ON (Navsteva.NavstivenaOsobaRC = NavstivenaOsoba.RC) ' .
            ' LEFT JOIN Osoba Zapsal ON (Navsteva.prichodZapsalRC = Zapsal.RC)' .
            ' LEFT JOIN PrirazeniKarty' .
            '  ON (Navsteva.OsobaRC = PrirazeniKarty.RC) AND (Navsteva.OsobniCislo = PrirazeniKarty.OsobniCislo) AND (Navsteva.DatumNastupu = PrirazeniKarty.DatumNastupu)' .
            '  AND PrirazeniKarty.DatumPrideleni <= Navsteva.Prichod' .
            '  AND (PrirazeniKarty.DatumVraceni >= Navsteva.Prichod  OR  PrirazeniKarty.DatumVraceni IS NULL)' .
            ' LEFT JOIN Karta ON (Karta.Kod = PrirazeniKarty.kodKarty)' .
            ' LEFT JOIN DuvodNavstevy ON ( (DuvodNavstevy.Kod = Navsteva.duvodNavstevy2))';

        // Typ navstevy
        $this->addField( new TypNavstevy($database, null))
            ->setCode('col-typnavstevy');

        // Vratnice
        $this->addField()
            ->setSqlFieldName(
                '(SELECT Rozdeleni.Popis FROM Rozdeleni' .
                ' WHERE Rozdeleni.KodTypuRozdeleni = Navsteva.KodTypuRozdeleni_Vratnice AND Rozdeleni.Kod = Rozdeleni_Vratnice) AS Rozdeleni_vratnice_popis')
            ->setSortSQL('Rozdeleni_vratnice_popis') ->setOrdering(true)
            ->setHeaderText('vratnice')
            ->setCode('col-vratnice')
            ->setVisible(!$omezeniVratnice);

        // Duvod navstevy
        $this->addField()
            ->setSqlFieldName('DuvodNavstevy.Popis') ->setHeaderText('duvod_navstevy')
            ->setCode('col-duvodnavstevy2');

        //Ocekavany?
        $this->addField(BrowserField::FIELD_TYPE_SELECT)
            ->setSqlFieldName('Navsteva.PredemOcekavanaNavsteva') ->setHeaderText('ocekavany') ->setOrdering(true)
            ->setCode('col-ocekavany')
            ->setOptions([0 => TextManager::_('Str_No'), 1 => TextManager::_('Str_Yes')])
            ->setAlign('center');

        //Navsteva.prijmeni
        $this->addField()
            ->setSqlFieldName('Navsteva.Prijmeni') ->setHeaderText('Prijmeni')
            ->setCode('col-prijmeni')
            ->setOrdering(true) ->setClassName('font-weight-bold')
            ->setLink('?menuid=navstevy&action=zaznam');

        //Navsteva.jmeno
        $this->addField()
            ->setSqlFieldName('Navsteva.Jmeno') ->setHeaderText('Jmeno')
            ->setCode('col-jmeno')
            ->setOrdering(true)
            ->setLink('?menuid=navstevy&action=zaznam');

        //Ocekavany prichod
        $this->addField( new CasNavstevyBrowserField($database))
            ->setSqlFieldName('OcekavanyPrichod') ->setHeaderText('ocekavany_datum') ->setFormat('weektimestamp')
            ->setCode('col-ocekavanyprichod')
            ->setOrdering(true)
            ->setLink('?menuid=navstevy&formaction=edit');

        //doklad
        $this->addField()
            ->setSqlFieldName('Navsteva.Doklad') ->setHeaderText('doklad')
            ->setCode('col-doklad');

        //vozidlo
        $this->addField()
            ->setSqlFieldName('Vozidlo') ->setHeaderText('vozidlo')
            ->setCode('col-vozidlo');

        //firma
        $this->addField()
            ->setSqlFieldName('firma') ->setHeaderText('firma') ->setOrdering(true)
            ->setCode('col-firma');

        //OznaceniMedia
        $this->addField()
            ->setSqlFieldName('OznaceniMedia') ->setHeaderText('oznaceni_media') ->setOrdering(true)
            ->setCode('col-oznacenimedia');

        //Prichod
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName("COALESCE(Zapsal.Jmeno,'') {fn_concat()} ' ' {fn_concat()} COALESCE(Zapsal.Prijmeni,'')")
            ->setCode('col-prichodzapsal');

        $this->addField( new CasNavstevyBrowserField($database))
            ->setSqlFieldName('Prichod') ->setHeaderText('prichod') ->setFormat('weektimestamp') ->setOrdering(true)
            ->setCode('col-prichod')
            ->setTitleFieldIndex('col-prichodzapsal', TextManager::_('prichod_zapsal') . ': ') ->setLink('?menuid=navstevy&formaction=edit');

        $this->defaultSort = '-col-prichod'; // -count($this ->fields);

        //Odchod
        $this->addField(new PotvrzeniOdchoduNavstevyField($database, null))
            ->setSqlFieldName('Odchod') ->setHeaderText('Odchod') ->setFormat('weektimestamp') ->setOrdering(true)
            ->setCode('col-odchod');

        $this ->addField(BrowserField::FIELD_TYPE_COMMENT)
            ->setSqlFieldName('{fn_castwidememo(duvodNavstevy)}') ->setHeader(TextManager::_('duvod_navstevy') . ': ')
            ->setCode('col-duvodnavstevy')
            ->setClassName('browserpoznamka') ->setHeaderColSpan(-1);

        //NavstivenaOsoba
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('NavstivenaOsoba.Telefon')
            ->setCode('col-telefon');

        $this->addField()
            ->setSqlFieldName('JmenoNavstiveneOsoby') ->setHeaderText('navstivena_osoba')
            ->setCode('col-navstivenaosoba')
            ->setTitleFieldIndex('col-telefon', TextManager::_('telefon') . ': ');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Navsteva.DT_Schvaleni') ->setFormat('datetime')
            ->setCode('col-dt_schvaleni');

        $this->addField( new NavstevuSchvalilaOsoba($database))
            ->setSqlFieldName('Navsteva.Schvalil') ->setSqlFieldIndex2('@col-dt_schvaleni') // vola getViewText hidden pole s kodem col-dt_schvaleni
            ->setCode('col-schvalil')
            ->setHeaderText('navstevu_schvalil');

        $zobrazitPoznamky = (int)($_GET['poznamky'] ?? ($_SESSION['uzivatelskePoznamkyNavstev'] ?? 0));

        for ($x = 1; $x <= 10; $x++) {
            $this->addField(BrowserField::FIELD_TYPE_COMMENT)
                ->setSqlFieldName("Navsteva.Uzivatelska$x")
                ->setVisible($zobrazitPoznamky  &&  RonPortalSettings::getValue("navstevy_uzivatelska$x"))
                ->setHeader(RonPortalSettings::getValue('navstevy_uzivatelska' . $x) . ': ')
                ->setCode("col-uzivatelska$x")
                ->setClassName('browserpoznamka')    // puvodne mel navic cely radek class uzivatelska-pole-navstev
                ->setHeaderColSpan(-1);
        }
    }


    function setFilterForm()
    {
        $this->filterForm = new NavstevyFilter($this ->database);
        //$this->filterForm->loadData();
        // nedela nic - filter nastaven v constructoru
    }

    function renderFilter()
    {
        parent::renderFilter();

        echo
            '<ul class="linkslist margin-top-3">' .
            "<li><a href='?menuid=navstevy&formaction=new&ocekavana=1'>" . TextManager::_('ocekavana_navsteva') . "</a></li>" .
            (!$this ->prava ->priznak // zapis nove nasvtevy se nezobrazuje, pokud ma pravo jen na ocekavane navstevy
                ? "<li><a href='?menuid=navstevy&formaction=new&ocekavana=0'>" . TextManager::_('zapis_nove_navstevy') . "</a></li>"
                : ''
            ) .
            "<li><a id='najit_navstevu_pomoci_id_medium'>" . TextManager::_('najit_navstevu_pomoci_id_medium') . "</a></li>" .
            "</ul>";
    }


    /**
     * @param array $values
     * @return bool
     * @throws Exception
     */
    function enableLine($values)
    {
        //Zobrazi pokud ma na RC navstivene osoby pravo
        return $this->prava->lzeZobrazitRC($values[2]);
    }

    /**
     * @param Database $database
     * @return mixed
     * @throws DBException
     */
    private function rozdeleniVratnice($database)
    {
        $kod = RonPortalSession::getValue('rozdelenivratnice');
        if (!$kod) {
            return '';
        }

        $params = $database->createParameters();
        $sql =
            'SELECT Rozdeleni.Popis' .
            ' FROM Rozdeleni' .
            ' WHERE KodTypuRozdeleni = ' . $params ->addParameter(RonPortalSettings::getIntValue('typrozdelenivratnice')) .
            ' AND Rozdeleni.Kod = ' . $params->addParameter($kod);

        $cursor = $database->getSqlResult($sql, $params);
        $row = $cursor->fetch();
        $cursor ->close();
        return $row[0] ?? '';
    }


    /**
     * @return SqlStatement|string
     */
    function getSQLFilter()
    {
        $params = $this->database->createParameters();

        $result = '';

        if ($this ->filterForm ->omezeniVratnice  &&  RonPortalSession::getValue('rozdelenivratnice') != '') {
            $result =
                'Navsteva.Rozdeleni_Vratnice = ' . $params->addParameter(RonPortalSession::getValue('rozdelenivratnice'));
        }

//        if ($this ->filterForm ->jenMedium) {
//            $result .=
//                ($result ? ' AND ' : '') .
//                ' Karta.Kod = ' . $params->addParameter($this ->filterForm ->kodMedia) .
//                ' AND Prichod IS NOT NULL AND Odchod IS NULL';
        if ($medium = $this ->filterForm ->getFilterValue('medium')) {
            $result .=
                ($result ? ' AND ' : '') .
                ' Karta.Kod = ' . $params->addParameter($medium) .
                ' AND Prichod IS NOT NULL AND Odchod IS NULL';
        } else {
            $formFilter = $this ->filterForm ->buildSqlFilter($params);
            if ($formFilter) {
                $result .=
                    ($result ? ' AND ' : '') .
                    $formFilter;
            }

            //timestamp -> string
            if ($this ->filterForm ->getFilterValue('omezeniNavstev') == 1) {
                // ocekavane prichody
                $result .=
                    ($result ? ' AND ' : '') .
                    '(Navsteva.OcekavanyPrichod >= ' . $params->addDateParameter(time()) .
                    ' AND Navsteva.Prichod IS NULL AND Navsteva.Odchod IS NULL)';
            } else if ($this ->filterForm ->getFilterValue('omezeniNavstev') == 2) {
                // pritomni
                $result .=
                    ($result ? ' AND ' : '') .
                    ' (Navsteva.Prichod <= ' . $params->addTimestampParameter(time()) . // uz prisla a jeste neodesla
                    ' AND Navsteva.Odchod IS NULL';
                $result .= ")";

            } else {
                $result .=
                    ($result ? ' AND ' : '') .
                    '((Navsteva.Prichod IS NULL AND Navsteva.OcekavanyPrichod >= ' . $params->addDateParameter($this ->filterForm ->getFilterValue('datumod')) .
                    ($this ->filterForm ->getFilterValue('datumod', 'datumDo')
                        ? ' AND Navsteva.OcekavanyPrichod < ' . $params->addDateParameter(DateUtils::incDayStart($this ->filterForm ->getFilterValue('datumod', 'datumDo')))
                        : ''
                    ) .
                    ') OR (' .
                    'Navsteva.Prichod >= ' . $params->addDateParameter($this ->filterForm ->getFilterValue('datumod')) .
                    ($this ->filterForm ->getFilterValue('datumod', 'datumDo')
                        ? ' AND Navsteva.Prichod < ' . $params->addDateParameter(DateUtils::incDayStart($this ->filterForm ->getFilterValue('datumod', 'datumDo')))
                        : ''
                    ) .
                    '))';
            }
        }

        return new SqlStatement($result, $params);
    }
}
