<?php

/**
 * Formular pro zmenu osobniho hesla
 */

namespace RonPortal\Common;

use PHPMailer\PHPMailer\Exception;
use Ron\Controls\FormField;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\Http\App;
use Ron\MessageManager;
use Ron\RonException;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Controls\RCOsobyFormField;
use RonPortal\Controls\RonPortalForm;
use RonPortal\NeniPravoException;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\System\AgentManager;
use RonPortal\User;

class PersonalPasswordForm extends RonPortalForm
{
    /** @var FormField|null  */
    private $fieldStareHeslo = null;

    /** @var FormField  */
    private $fieldNoveHeslo = null;
    /** @var FormField  */
    private $fieldNoveHeslo2 = null;

    /** @var string */
    private $rc;

    private $enableDebug = false;

    /** @var bool Urcuje, jestli se jedna o zmenu hesla jine osoby */
    private $jinaOsoba = false;

    public function __construct($database)
    {
        parent::__construct($database);

        $this
            ->setTitle('Nastaveni_osobniho_hesla')
            ->setCode('formosobniheslo')
            ->setCssClass("form form-30-70 middle-form")
            ->setXmlEventsName('OSOBNI_HESLO');

        $this ->sqlTableName = 'Osoba';

        $pravo = Prava::vratPrava('osobni_heslo');

        // implicitne zmena vlastniho hesla
        $this ->rc = User::getRC();
        $this ->jinaOsoba = false;

        if ($_GET['jinaosoba'] ?? false) {
            if ($pravo ->pouzeSebe()) {
                throw new NeniPravoException();
            }

            $f = new RCOsobyFormField($pravo, time());
            $f
                ->setDefaultValue('')
                ->setCaption('Osoba')
                ->setRequired(true);

            if ((int)($_POST['osoba'] ?? $_GET['osoba'] ?? 0)) {
                $pp = PracovniPomer::nactiPP($this ->database, (int)($_POST['osoba'] ?? $_GET['osoba']));
                $this ->rc = $pp ->rc;
                $f ->setDefaultValue($pp ->idPP);
            } else {
                $this ->rc = null;
            }

            // $f ->filtr =   TODO: nastavit filtr, aby nemohl vybrat sam sebe?
            $this ->addField($f);
            $this ->jinaOsoba = true;
        }

        if (isset($this ->rc)  &&  !$pravo ->lzeZobrazitRC($this ->rc)) {
            throw new NeniPravoException();
        }

        if (!$this ->jinaOsoba) { // User::hasRC($this ->rc)) {
            // zmena vlastniho hesla a uzivatel nema povoleno menit heslo bez zadani stareho hesla
            $this ->fieldStareHeslo = $this ->addField(FormField::FIELD_TYPE_PASSWORD)
                ->setFieldName('*OldPassword')
                ->setCaption('Stare_heslo')
                ->setCode('field-stare-heslo')
                ->setFieldId('field-stare-heslo');
        }

        $this ->fieldNoveHeslo = $this ->addField(FormField::FIELD_TYPE_PASSWORD)
            ->setFieldName('*NewPassword')
            ->setCaption('Nove_heslo')
            ->setFieldId('field-nove-heslo')
            ->setCode('field-nove-heslo')
            ->setRowClass('row margin-top-3')
            ->setRequired(true);

        $this ->fieldNoveHeslo2 = $this ->addField(FormField::FIELD_TYPE_PASSWORD)
            ->setFieldName('*NewPassword2')
            ->setCaption('Potvrzeni_hesla')
            ->setFieldId('field-potvrzeni-hesla')
            ->setCode('field-potvrzeni-hesla')
            ->setRequired(true);
    }


    public function loadConfigurationItem($key, $value, $valueText)
    {
        if ($key == 'enabledebug') {
            $this ->enableDebug = (int)$value;
        } else {
            parent::loadConfigurationItem($key, $value, $valueText);
        }
    }


    public function getButtons($insert)
    {
        $result = parent::getButtons($insert);

        if (isset($result['submit'])) {
            $result['submit']['value'] = TextManager::_('Nastavit_heslo');
        }

        return $result;
    }


    /**
     * Nacte z databaze puvodni heslo (zasifrovane)
     * @param Database $database
     * @param string $rc
     * @return string
     * @throws DBException
     * @throws \Exception
     */
    private static function getOriginalPassword(Database $database, $rc)
    {
        $params = $database ->createParameters();
        $sql =
            'SELECT OsobniHeslo FROM Osoba WHERE RC = ' . $params->addParameter($rc);
        $cursor = $database->getSqlResult($sql, $params);
        $row = $cursor->fetch();
        $cursor ->close();

        if (!$row) {
            // nemelo by nastat
            throw new \Exception('Chyba pri overovani puvodniho hesla - zaznam v databazi nebyl nalezen');
        }

        return $row[0];
    }



    /**
     * @param $heslo
     * @return string
     * @throws Exception
     */
    private function zakodujHeslo($heslo)
    {
//        $heslo = str_replace(
//            ['"'],
//            ['"""'],
//            $heslo
//        );

        // zdvoji uvozovky - pozor, v aplikaci funguje az od 09-2024
        $heslo = str_replace('"', '""', $heslo);

        try {
            $agentResult = AgentManager::executePersonAgent('100 -vstup:"' . $heslo . '"', $this ->enableDebug && App::getDebugMode()); // v debug modu prida   -nolog
        } catch (\Exception $e) {
            $agentResult = null;
        }

        if (is_null($agentResult)) {
            throw new Exception(TextManager::_('Heslo_nezakodovano'));
        }

        return $agentResult->resultParams['heslo'];
    }



    public function validate($edit)
    {
        if (!parent::validate($edit)) {
            return false;
        }

        $noveHeslo = $this ->fieldNoveHeslo ->getPostValue($this ->utf8);

        // overi, ze se nove heslo shoduje s potvrzenim
        if ($this ->fieldNoveHeslo2 ->isVisible()) {
            if ($noveHeslo  !==  $this ->fieldNoveHeslo2 ->getPostValue($this ->utf8)) {
                throw new RonException( TextManager::_('Hesla_se_neshoduji'), 'NewPassword' );
            }
        }

        // zkontroluje stare heslo
        if ($this ->fieldStareHeslo  &&  $this ->fieldStareHeslo ->isVisible()) {
            // overit stare heslo
            $oldPassDb = self::getOriginalPassword($this ->database, $this ->rc); // nacte stare zakodovane heslo z DB

            $oldPass = $this ->fieldStareHeslo ->getPostValue($this ->utf8);
            if ($oldPassDb === null  &&  $oldPass !== null) {
                // puvodni heslo jeste neni zadane
                throw new RonException( TextManager::_('Spatne_puvodni_heslo'), 'OldPassword' );
            }

            $zakodovane = $this ->zakodujHeslo($oldPass ?? '');
            if ($zakodovane !== $oldPassDb) {
                throw new RonException( TextManager::_('Spatne_puvodni_heslo'), 'OldPassword' );
            }


//            if (is_null($heslo)) {
//                throw new Exception( ronText('Spatne_puvodni_heslo') );
//            }
//
//            $heslo = $this->zakodujHeslo($heslo);
//
//            $params = $this->database->createParameters();
//
//            $sql =
//                "SELECT OsobniHeslo FROM Osoba WHERE RC = " . $params->addParameter(User::getRC()); // . " AND OsobniHeslo = " . $params->addParameter($this ->database ->getBlobParam($heslo, 'OsobniHeslo'));
//            $cursor = $this->database->getSqlResult($sql, $params);
//            $row = $cursor->fetch();
//            $cursor ->close();
//
//            if (!$row) {
//                // nemelo by nastat
//                throw new Exception('Chyba pri overovani puvodniho hesla - zaznam v databazi nebyl nalezen');
//            }
//
//            if ($heslo != $row[0]) {
//                throw new Exception( ronText('Spatne_puvodni_heslo') );
//            }

        }

        return true;
    }


    public function updateRecord()
    {
        // tady nema co delat
        throw new \Exception('Chyba ve formulari PersonalPasswordForm!');
    }




    /**
     * @return bool
     * @throws DBException
     */
    public function insertRecord()
    {
        if (!$this->validate(false)) {
            return false;
        }

        $heslo = $this ->zakodujHeslo($this ->fieldNoveHeslo ->getPostValue($this ->utf8));

        $rec = $this->createDBRecord();
        $rec ->addStringField('RC', true);
        $rec ->addStringField('Jmeno', true);
        $rec ->addStringField('Prijmeni', true);
        ($rec ->addField('OsobniHeslo', 'binary', false, 0, $heslo))
            ->setMaxLogLength(0);

        $params = $this ->database ->createParameters();
        $rec ->setWhere('RC = ' . $params ->addParameter($this ->rc), $params);

        if (!$rec ->loadRecord()) {
            // nemelo by nastat
            throw new \Exception('Chyba pri nastavovani osobniho hesla - puvodni zaznam nebyl nalezen');
        }

        if (!$this->beforeUpdateRecord($rec)) {
            return false;
        }

        $rec->updateRecord();

        MessageManager::addSuccessMessage(TextManager::_('Heslo_zmeneno'));

        return $this->afterUpdateRecord($rec);
    }


    public function renderHeader()
    {
        parent::renderHeader();

        if (!$this ->jinaOsoba) {
            $prava = Prava::vratPrava('osobni_heslo');
            if (!$prava->pouzeSebe()) {
                $backaction = StringUtils::secureString($_GET['backaction'] ?? '');
                echo
                    '<div class="margin-top-2 margin-bottom-2"><a href="?menuid=personalpassword&jinaosoba=1&backaction=' . $backaction . '">' .
                    TextManager::_('Nastavit_heslo_jine_osobe') .
                    '</a></div>';
            }
        }
    }

}
