<?php
/**
 * stanislav.elbl, 01.11.2017
 */

namespace RonPortal\Common;

use Ron\Controls\BrowserField;
use Ron\DateUtils;
use Ron\DB\ArrayCursor;
use Ron\DB\SqlStatement;
use Ron\Http\Csrf;
use Ron\TextManager;
use RonPortal\Controls\JmenoPrijmeniBrowserField;
use RonPortal\Controls\RonPortalBrowser;
use RonPortal\Prava;
use RonPortal\User;


class LoginAsLinkField extends BrowserField
{
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if ($rowData['col-potvrzeni'] < 2) { // nezobrazuje link pro zrusene a zamitnute
            if ($this->database->stringToTimestamp($rowData['col-platnostod']) <= time() && (!$rowData['col-platnostdo'] || $this->database->stringToTimestamp($rowData['col-platnostdo']) >= DateUtils::today())) {
                return
                    '<a class="confirmQuestion" href="?menuid=zastupuzivatele&action=loginas&id=' . $rowData[0] .
                    Csrf::csrfUrlParameter('zastup_login_' . (int)$rowData[0]) .
                    '" data-ron-confirm-id="Opravdu_prihlasit_jako_zastup">' .
                    TextManager::_('Prihlasit_se_jako_zastup') .
                    '</a>';
            }
        }

        return '';
    }
}


class ZastupUzivateleBrowser extends RonPortalBrowser
{
    const JA_ZASTUPUJI = 1;
    const JSEM_ZASTUPOVAN = 2;
    const VSECHNY_ZASTUPY = 3;

    public $coZobrazit;

    /**
     * @param \Ron\DB\Database $database
     * @param int $coZobrazit  konstanty JA_ZASTUPUJI/JSEM_ZASTUPOVAN/VSECHNY_ZASTUPY
     */
    function __construct($database, $coZobrazit)
    {
        parent::__construct($database, 'zastupuzivatele');
        $this ->coZobrazit = $coZobrazit;

        $this->rowClickLink = '?menuid=zastupuzivatele';

        $this->code = 'browserzastupyuzivatelu';

        if ($coZobrazit == self::JA_ZASTUPUJI) {
            $this->title = TextManager::_('Vy_zastupujete_uzivatele');
        } else if ($coZobrazit == self::JSEM_ZASTUPOVAN) {
            $this->title = TextManager::_('Zastupuje_vas_uzivatel');
        } else {
            $this->title = '';
        }

        $this->sqlFromPart =
            'ZastupUzivatele' .
            //' INNER JOIN RoleProZastupUzivatele ON RoleProZastupUzivatele.Kod = ZastupUzivatele.Role' .
            ' INNER JOIN Osoba Zastupovany ON Zastupovany.RC = ZastupUzivatele.Zastupovany' .
            ' INNER JOIN Osoba Zastupujici ON Zastupujici.RC = ZastupUzivatele.Zastupujici';

        $this ->sqlSelectPart =
            'ZastupUzivatele.Id, Zastupovany.RC'; // 0,1

        $this ->primaryField = 0;

        $params = $database ->createParameters();
        $where =
            '(ZastupUzivatele.PlatnostDo >= ' . $params ->addParameter($database ->dateToParam(time())) .
            ' OR ZastupUzivatele.PlatnostDo IS NULL)';

        if ($coZobrazit == self::JA_ZASTUPUJI) {
            $where .=
                ' AND ZastupUzivatele.Zastupujici = ' . $params ->addParameter(User::getRC());
        } else if ($coZobrazit == self::JSEM_ZASTUPOVAN) {
            $where .=
                ' AND ZastupUzivatele.Zastupovany = ' . $params ->addParameter(User::getRC());
        }

        $this ->sqlWherePart = new SqlStatement($where, $params);

        $this ->addField()
            ->setSqlFieldName('ZastupUzivatele.PlatnostOd') ->setFormat('date') ->setHeaderText('Platnost_od')
            ->setCode('col-platnostod');

        $this ->orderFieldIndex = 'col-platnostod';

        $this ->addField()
            ->setSqlFieldName('ZastupUzivatele.PlatnostDo') ->setFormat('date') ->setHeaderText('Platnost_do')
            ->setCode('col-platnostdo');

        // zastupovany
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Zastupovany.Jmeno')
            ->setCode('col-zastupovany-jmeno');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Zastupovany.Prijmeni')
            ->setCode('col-zastupovany-prijmeni');

        $this->addField( new JmenoPrijmeniBrowserField($this ->database, 'col-zastupovany-jmeno', 'col-zastupovany-prijmeni'))
            ->setHeaderText('Zastupovany_uzivatel')
            ->setVisible($coZobrazit == self::JA_ZASTUPUJI  ||  $coZobrazit == self::VSECHNY_ZASTUPY)
            ->setCode('col-zastupovany');

        // zastupujici
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Zastupujici.Jmeno')
            ->setCode('col-zastupujici-jmeno');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Zastupujici.Prijmeni')
            ->setCode('col-zastupujici-prijmeni');

        $this->addField( new JmenoPrijmeniBrowserField($this ->database, 'col-zastupujici-jmeno', 'col-zastupujici-prijmeni') )
            ->setHeaderText('Zastupujici_uzivatel')
            ->setVisible($coZobrazit == self::JSEM_ZASTUPOVAN  ||  $coZobrazit == self::VSECHNY_ZASTUPY)
            ->setCode('col-zastupujici');

        // potvrzeni
        $this ->addField(BrowserField::FIELD_TYPE_SELECT)
            ->setSqlFieldName('ZastupUzivatele.Potvrzeni')
            ->setOptions([
                0 => TextManager::_('Zastup_potvrzeni_0'),
                1 => TextManager::_('Zastup_potvrzeni_1'),
                2 => TextManager::_('Zastup_potvrzeni_2'),
                3 => TextManager::_('Zastup_potvrzeni_3')
            ])
            ->setCode('col-potvrzeni');

        $this->addField(BrowserField::FIELD_TYPE_TEXT)
            ->setText(TextManager::_('Podrobnosti'))
            ->setLink('?menuid=zastupuzivatele')
            ->setCode('col-link-podrobnosti');

        if ($coZobrazit == self::JA_ZASTUPUJI) {
            $this->addField( new LoginAsLinkField($this ->database));
        } else {
            $this->addField(BrowserField::FIELD_TYPE_TEXT)
                ->setText(TextManager::_('Upravit'))
                ->setLink('?menuid=zastupuzivatele&formaction=edit');
        }

        // poznamka
        $this ->addField(BrowserField::FIELD_TYPE_COMMENT)
            ->setSqlFieldName('{fn_castwidememo(ZastupUzivatele.Poznamka)}') ->setClassName('poznamka browserpoznamka')
            ->setCode('col-poznamka')
            ->setHeaderColSpan(1);
    }

    function setFilterForm()
    {
        // parent::setFilterForm(); -- nedela nic, filtr se nezobrazuje
    }


    function load($sqlFilter = '')
    {
        parent::load($sqlFilter);
        $this ->cursor = new ArrayCursor($this ->cursor);
    }


    function enableLine($values)
    {
        return
            $this ->coZobrazit == self::JA_ZASTUPUJI  ||
            $this ->coZobrazit == self::JSEM_ZASTUPOVAN ||
            Prava::vratPrava('zast_uz_komu') ->lzeZobrazitRC($values[1], time(), time());
    }


    function render()
    {
        if ($this ->cursor ->numRows()) {
            parent::render();
        } else {
            if ($this->coZobrazit == self::JSEM_ZASTUPOVAN) {
                echo '<div class="infomsg">' . TextManager::_('Nemate_naplanovan_zadny_zastup') . '</div>';
            } else if ($this->coZobrazit == self::JA_ZASTUPUJI) {
                echo '<div class="infomsg">' . TextManager::_('Nikoho_nezastupujete') . '</div>';
            }
        }

        if ($this->coZobrazit == self::JSEM_ZASTUPOVAN) {
            echo '<div><a href="?menuid=zastupuzivatele&action=novazadost">' . TextManager::_('Pozadat_o_zastup') . '</a></div>';
        }

        if ($this->coZobrazit == self::VSECHNY_ZASTUPY) {
            echo '<div><a href="?menuid=zastupuzivatele&action=novyzastup">' . TextManager::_('Naplanovat_zastup_uzivatele') . '</a></div>';
        }
    }


}
