<?php
/**
 * stanislav.elbl, 31.07.2015
 * roman.szablatura, 4.5.2016
 */

namespace RonPortal\Controller;

use Exception;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\DBUtils;
use Ron\DB\Parameters;
use Ron\Http\HtmlHeaders;
use Ron\MVC;
use Ron\MVC\JsonOutputObject;
use Ron\MVC\ViewObject;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Common\PotvrzeniPrecteniForm;
use RonPortal\Dokumenty\DokumentOutputObject;
use RonPortal\Dokumenty\DokumentyUtils;
use RonPortal\NeniPravoException;
use RonPortal\Personalistika\DetailDistribuovanehoDokumentu;
use RonPortal\Personalistika\DokumentyOsobyBrowser;
use RonPortal\Personalistika\VyplatniListkyBrowser;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\RonPortalDBRecord;
use RonPortal\RonPortalSettings;
use RonPortal\User;


class DokumentyController extends MVC\JsonController
{
    /** @var Database */
    var $database;

    public function __construct($database)
    {
        $this->database = $database;

        TextManager::getInstance()->loadStrings('person.ini');
        HtmlHeaders::getInstance()->addCss('css/person.css');
        //HtmlHeaders::getInstance()->addScript('script/personscripts.js');
    }

    public static function vratObdobiDoProPrava()
    {
        $pocetMesicu = RonPortalSettings::getIntValue('person_pocet_mesicu_dopredu');

        return $pocetMesicu ? DateUtils::incMonth(DateUtils::today(), $pocetMesicu) : DateUtils::today();
    }

    /**
     * @param int $idDokumentu
     * @param string $rcOsoby
     * @param string $vychoziTypPrava
     * @throws Exception
     * @throws NeniPravoException
     */
    private function overPravoNaTypDokumentu($idDokumentu, $rcOsoby, $vychoziTypPrava = 'dokumenty_seznam')
    {
        $params = $this->database->createParameters();
        $typPrava = DBUtils::getSqlValue($this->database,
            'SELECT DefinicePrava' .
            ' FROM TypDokumentu JOIN Dokument ON (Dokument.Typ = TypDokumentu.Kod)' .
            ' WHERE Dokument.Id = ' . $params->addParameter($idDokumentu),
            $params);

        $typPrava = $typPrava ?: $vychoziTypPrava;
        $prava = Prava::vratPrava($typPrava);

        if (!$prava->lzeZobrazitRC($rcOsoby, DateUtils::today(), $this->vratObdobiDoProPrava())) {
            throw new NeniPravoException();
        }
    }

//    /**
//     * @param Database $database
//     * @return bool
//     * @throws Exception
//     */
//    public static function lzeZobrazitVyplatniListky($database)
//    {
//        $params = $database->createParameters();
//
//        $sql =
//            "SELECT zobrazitNaIntrawebu FROM TypDokumentu WHERE kod = " . $params->addParameter(-1);
//
//        $cursor = $database->getSqlResult($sql, $params);
//
//        $row = $cursor->fetch();
//
//        $cursor->close();
//
//        return !!$row[0];
//    }

    /**
     * @throws Exception
     */
    public function uploadDokument()
    {
//        $result = new DokumentUploadOutput();
//
//        try {
            if (!RonPortalSettings::getIntValue('povolituploaddokumentu', 1)) {
                throw new Exception('Upload dokumentu neni povolen!');
            }

            $pp = PracovniPomer::nactiPP($this ->database, (int)$_POST['idosoby']);

            if (!Prava::vratPrava('prohl_osobni')->lzeZobrazitRC($pp ->rc)) {
                throw new NeniPravoException();
            }

            // Vlozi dokument do databaze
            $id = DokumentyUtils::vlozDokument($this ->database, 'soubor', (int)$_POST['typ'], $_POST['popis'], $this, true);

            // Vlozi zaznam do vazebni tabulky
            $rec = new RonPortalDBRecord($this ->database, 'DokumentyOsoby', $this);
            $rec ->addField('RC', '', false, 15, $pp ->rc);
            $rec ->addField('Id', 'int', false, 0, $id);
            $rec ->insertRecord();
//        } catch (Exception $e) {
//            $result ->setError($e ->getMessage());
//        }
//
//        return $result;
    }

    /**
     * @param int $idDokumentu  Id dokumentu
     * @param string $rc        Id osoby, ktere je dokument prirazen
     * @return DokumentOutputObject
     * @throws DBException
     * @throws Exception
     * @throws NeniPravoException
     */
    public function downloadDokumentOsoby($idDokumentu, $rc)
    {
        $params = $this->database->createParameters();

        $sql =
            'SELECT Dokument.Id' .
            ' FROM Dokument, TypDokumentu, DokumentyOsoby' .
            ' WHERE Dokument.Id = ' . $params->addParameter($idDokumentu) .
            '   AND Dokument.Id = DokumentyOsoby.Id' .
            '   AND TypDokumentu.Kod = Dokument.Typ' .
            '   AND (Dokument.ZobrazitNaIntrawebu = 1  OR  (Dokument.ZobrazitNaIntrawebu = 2  AND  TypDokumentu.ZobrazitNaIntrawebu = 1))' .
            '   AND (DokumentyOsoby.RC = ' . $params->addParameter($rc) . ')';

        $cursor = $this ->database ->getSqlResult($sql, $params);

        if (!$row = $cursor ->fetch()) {
            // dokument neexistuje, nebo neni prirazen osobe
            throw new Exception('Chyba! Zadany dokument nebyl nalezen!');
        }

        return new DokumentOutputObject($this ->database, $idDokumentu);
    }


    /**
     * @return DokumentOutputObject|VyplatniListkyBrowser
     * @throws Exception
     */
    private function handleVyplatniListky()
    {
        if (User::isLoggedAs()) {
            // pri "prihlaseni jako" vyplatni listky nezobrazovat!
            throw new NeniPravoException();
        }
        if (!DokumentyUtils::lzeZobrazitVyplatniListky($this->database)) {
            throw new NeniPravoException();
        }

        // stahnuti vyplatniho listku osoby
        if (isset($_GET['record'])) {
            return $this ->downloadDokumentOsoby((int) $_GET['record'], User::getRC());
        }

        TextManager::getInstance()->loadStrings('users.ini');
        HtmlHeaders::getInstance()
            ->setAppTitle(TextManager::_('Vyplatni_listky'));

        $browser = new VyplatniListkyBrowser($this->database);
        $browser->load();
        return $browser;
    }


    /**
     * Stahnuti dokumentu pripojeneho primo k osobe
     * @param $idDokumentu
     * @return DokumentOutputObject
     * @throws DBException
     * @throws Exception
     * @throws NeniPravoException
     */
    private function handleDownloadDokumentuOsoby($idDokumentu)
    {
        if (isset($_GET['osoba'])) {
            $pp = PracovniPomer::nactiPP($this->database, (int)$_GET['osoba']);
            $rc = $pp ->rc;
        } else {
            $rc = User::getRC();
        }

        // zkontroluje pravo na zobrazeni daneho dokumentu (typ prava muze byt zadan u typu dokumentu)
        $this->overPravoNaTypDokumentu($idDokumentu, $rc);

        // zkontroluje, jestli je zadany dokument prirazen uzivateli a stahne jej
        return $this ->downloadDokumentOsoby($idDokumentu, $rc);
    }


    /**
     * @param int $idDokumentu
     * @param string $filtrTypuRozdeleni
     * @return DokumentOutputObject
     * @throws Exception
     * @throws NeniPravoException
     */
    function downloadDokumentRozdeleni($idDokumentu, $filtrTypuRozdeleni)
    {
        if (isset($_GET['osoba'])) {
            $pp = PracovniPomer::nactiPP($this->database, (int)$_GET['osoba']);
            $rc = $pp ->rc;
        } else {
            $rc = User::getRC();
        }

        $this->overPravoNaTypDokumentu($idDokumentu, $rc);

        $params = $this->database->createParameters();

        $sql =
            'SELECT Dokument.Id' .
            ' FROM Dokument, TypDokumentu, DokumentyRozdeleni, RozdeleniOsob' .
            ' WHERE Dokument.Id = ' . $params->addParameter($idDokumentu) .
            ' AND Dokument.Id = DokumentyRozdeleni.Id' .
            ' AND TypDokumentu.Kod = Dokument.Typ' .
            ' AND (Dokument.ZobrazitNaIntrawebu = 1  OR  (Dokument.ZobrazitNaIntrawebu = 2  AND  TypDokumentu.ZobrazitNaIntrawebu = 1))' .
            ' AND RozdeleniOsob.KodTypuRozdeleni = DokumentyRozdeleni.KodTypuRozdeleni' .
            ($filtrTypuRozdeleni ? ' AND ' . $filtrTypuRozdeleni : '') .
            ' AND RozdeleniOsob.Rozdeleni = DokumentyRozdeleni.Kod' .
            ' AND (RozdeleniOsob.RC = ' . $params->addParameter($rc) . ')' .
            ' AND RozdeleniOsob.PlatnostOd <= ' . $params->addDateParameter(time()) .
            ' AND (RozdeleniOsob.PlatnostDo >= ' . $params->addDateParameter(time()) . ' OR  RozdeleniOsob.PlatnostDo IS NULL)';

        $cursor = $this ->database ->getSqlResult($sql, $params);

        if (!$row = $cursor ->fetch()) {
            // dokument neexistuje, nebo neni prirazen osobe
            throw new Exception('Chyba! Zadany dokument nebyl nalezen!');
        }

        return new DokumentOutputObject($this ->database, $idDokumentu);
    }


//    /**
//     * @param int $idDokumentu
//     * @return DokumentOutputObject
//     * @throws DBException
//     * @throws Exception
//     */
//    function downloadDokumentPracMista($idDokumentu)
//    {
//        if (!$this->dokumentPrihlaseneho) {
//            $this->overPravoNaTypDokumentu($idDokumentu, $this->osobaRC);
//        }
//
//        $params = $this->database->createParameters();
//
//        $sql =
//            'SELECT Dokument.Id ' .
//            ' FROM Dokument, TypDokumentu, DokumentyRozdeleni, RozdeleniOsob ' .
//            ' WHERE Dokument.Id = ' . $params->addParameter($idDokumentu) .
//            ' AND Dokument.Id = DokumentyRozdeleni.Id ' .
//            ' AND TypDokumentu.Kod = Dokument.Typ ' .
//            ' AND (Dokument.ZobrazitNaIntrawebu = 1  OR  (Dokument.ZobrazitNaIntrawebu = 2  AND  TypDokumentu.ZobrazitNaIntrawebu = 1))' .
//            ' AND RozdeleniOsob.KodTypuRozdeleni = DokumentyRozdeleni.KodTypuRozdeleni ' .
//            ' AND RozdeleniOsob.KodTypuRozdeleni = -1 ' . // jen dokumenty prac. mist
//            ' AND RozdeleniOsob.Rozdeleni = DokumentyRozdeleni.Kod ' .
//            " AND (RozdeleniOsob.RC = " . $params->addParameter($this->osobaRC) . ")" .
//            ' AND RozdeleniOsob.PlatnostOd <= ' . $params->addParameter($this->database->timestampToParam(time())) .
//            ' AND (RozdeleniOsob.PlatnostDo >= ' . $params->addParameter($this->database->timestampToParam(time())) . '  OR  RozdeleniOsob.PlatnostDo IS NULL)';
//
//
//        if (!$cursor = $this ->database ->getSqlResult($sql, $params)) {
//            throw new DBException($this ->database, $sql);
//        }
//
//        if (!$row = $cursor ->fetch()) {
//            // dokument neexistuje, nebo neni prirazen osobe
//            throw new Exception('Chyba! Zadany dokument nebyl nalezen!');
//        }
//
//        return new DokumentOutputObject($this ->database, $idDokumentu);
//    }


    private function overPravoNaDistribuovanyDokument($idDokumentu, $platnostOd, $idVzkazu)
    {
        $params = $this->database->createParameters();

        $sql =
            'SELECT Vzkaz.RCPrijemce' .
            ' FROM VerzeDokumentu, DokumentVzkazu, Vzkaz' .
            ' WHERE VerzeDokumentu.Id = DokumentVzkazu.IdDokumentu' .
            ' AND VerzeDokumentu.PlatnostOd = DokumentVzkazu.PlatnostOd' .
            ' AND Vzkaz.Id = DokumentVzkazu.IdVzkazu' .
            ' AND VerzeDokumentu.Id = ' . $params->addParameter($idDokumentu) .
            ' AND VerzeDokumentu.PlatnostOd = ' . $params->addTimestampParameter($platnostOd) .
            ' AND DokumentVzkazu.IdVzkazu = ' . $params->addParameter($idVzkazu);

        $cursor = $this ->database ->getSqlResult($sql, $params);
        $row = $cursor ->fetch();
        $cursor ->close();

        if (!$row) {
            throw new NeniPravoException();
        }

        if (User::hasRC($row[0])) {
            // Je to vzkaz pro me => muzu videt dokument
            return;
        }

        $this->overPravoNaTypDokumentu($idDokumentu, $row[0], 'dokumenty_prohl');   // defaultne "Dokumenty - prehled distribuovanych dokumentu"
    }


    /**
     * @param int $idDokumentu
     * @param int $platnostOd
     * @param int $idVzkazu
     * @return DokumentOutputObject
     * @throws DBException
     * @throws Exception
     */
    function downloadDistribuovanyDokument($idDokumentu, $platnostOd, $idVzkazu)
    {
        // overi, jestli je dokument pripojen k zadanemu vzkadu a nacte RC prijemce vzkazu
        $this ->overPravoNaDistribuovanyDokument($idDokumentu, $platnostOd, $idVzkazu);

//        $params = $this->database->createParameters();
//
//        $sql =
//            'SELECT VerzeDokumentu.Id, Vzkaz.RCPrijemce ' .
//            'FROM VerzeDokumentu JOIN DokumentVzkazu ON (VerzeDokumentu.id = DokumentVzkazu.idDokumentu AND VerzeDokumentu.PlatnostOd = DokumentVzkazu.PlatnostOd)' .
//            'JOIN Vzkaz ON (Vzkaz.Id = DokumentVzkazu.IdVzkazu) ' .
//            'WHERE VerzeDokumentu.id = ' . $params->addParameter($idDokumentu) .
//            ' AND VerzeDokumentu.PlatnostOd = ' . $params->addParameter($this->database->timestampToParam($platnostOd)) .
//            //' AND Vzkaz.RCPrijemce = ' . $params->addParameter($this->osobaRC) .
//            ' AND DokumentVzkazu.IdVzkazu = ' . $params->addParameter($idVzkazu);
//
//        $cursor = $this ->database ->getSqlResult($sql, $params);
//
//        if (!$row = $cursor ->fetch()) {
//            // dokument neexistuje, nebo neni prirazen osobe
//            throw new NeniPravoException();
//            //throw new Exception('Chyba! Zadany dokument nebyl nalezen!');
//        }
//        $cursor ->close();
//
//        $this->overPravoNaTypDokumentu($idDokumentu, $row[1], 'dokumenty_prohl');   // defaultne "Dokumenty - prehled distribuovanych dokumentu"

        return new DokumentOutputObject($this ->database, $idDokumentu, true, $platnostOd);
    }


    /**
     * @return MVC\Redirect|PotvrzeniPrecteniForm
     * @throws Exception
     */
    private function potvrditPrecteni()
    {
        $idDokumentu = isset($_POST['dokumentid']) ? (int)$_POST['dokumentid'] : (int)$_GET['dokumentid'];
        $platnostOd = isset($_POST['platnostod']) ? $_POST['platnostod'] : $_GET['platnostod'];
        $idVzkazu = isset($_POST['vzkazid']) ? (int)$_POST['vzkazid'] : (int)$_GET['vzkazid'];

        $platnostOd = DateUtils::dateStringToTimestamp($platnostOd);

        //kontrola jestli je vzkaz pro prihlasenou osobu a jestli uz vzkaz nebyl potvrzen
        $this->kontrolaPrihlaseneOsoby($idDokumentu, $platnostOd, $idVzkazu);

        $f = new PotvrzeniPrecteniForm($this->database, $idDokumentu, $platnostOd, $idVzkazu);

        if ($f->handleAction()) {
            return new MVC\Redirect(null,'?menuid=dokumenty');
        }
        return $f;
    }

    /**
     * @param int $idDokumentu
     * @param int $platnostOd
     * @param int $idVzkazu
     * @throws DBException
     * @throws Exception
     */
    private function kontrolaPrihlaseneOsoby($idDokumentu, $platnostOd, $idVzkazu)
    {
        $params = $this->database->createParameters();

        $sql =
            'SELECT RCPrijemce, Precteno, TypDokumentu.Distribuovany ' .
            'FROM Vzkaz JOIN DokumentVzkazu ON (DokumentVzkazu.IdVzkazu = Vzkaz.Id) ' .
            ' JOIN Osoba ON (Vzkaz.RCPrijemce = Osoba.RC) ' .
            ' JOIN Dokument ON (Dokument.Id = DokumentVzkazu.IdDokumentu) ' .
            ' JOIN TypDokumentu ON (TypDokumentu.kod = Dokument.typ) ' .
            'WHERE IdVzkazu = ' . $params->addParameter($idVzkazu) .
            ' AND IdDokumentu = ' . $params->addParameter($idDokumentu) .
            ' AND DokumentVzkazu.PlatnostOd = ' . $params->addTimestampParameter($platnostOd);

        $cursor = $this->database->getSqlResult($sql, $params);
        $row = $cursor->fetch();
        $cursor ->close();

        if (!User::hasRC($row[0])) {
            throw new NeniPravoException();
        }

        if ($row[1] == 1) {
            throw new Exception(TextManager::_('precteni_potvrzeno'));
        }

        if (!((int)$row[2] == 1 || (int)$row[2] == 3)) { //12.6.2024 pridana varianta distribuovany = 3 = dokument distribuovany skriptem vyzadujici precteni
            throw new Exception(TextManager::_('Dokument_bez_potvrzeni'));
        }
    }

    /**
     * @param int $idDokumentu
     * @param int $platnostOd
     * @param int $idVzkazu
     * @return DetailDistribuovanehoDokumentu
     * @throws Exception
     */
    private function detailDokumentu($idDokumentu, $platnostOd, $idVzkazu)
    {
        $this ->overPravoNaDistribuovanyDokument($idDokumentu, $platnostOd, $idVzkazu);

        $b = new DetailDistribuovanehoDokumentu($this->database, $idDokumentu, $platnostOd, $idVzkazu);
        $b->load();

        return $b;
    }

    /**
     * @param Parameters $params
     * @return string
     */
    private function mojeNeprecteneDokumenty($params)
    {
        $sql = " AND RCPrijemce= " . $params->addParameter(User::getRC());

        return $sql;
    }

    /**
     * @param Parameters $params
     * @throws Exception
     */
    private function neprecteneDokumentyPodrizenych($params)
    {
        $prava = Prava::vratPrava('dokumenty_prohl');
        $sql =
            " AND EXISTS(SELECT ID FROM PracovniPomer WHERE PracovniPomer.RC = Vzkaz.RCPrijemce 
                AND PracovniPomer.DatumNastupu <= " . $params->addDateParameter(time()) .
            "   AND (PracovniPomer.DatumUkonceni >= " . $params->addDateParameter(time()) . " OR PracovniPomer.DatumUkonceni IS NULL)" .
            "   AND " . $prava->sqlFiltr('PracovniPomer', time(), time(), $params) . ")";

        return $sql;
    }

    /**
     * @param JsonOutputObject $output
     * @throws Exception
     */
    private function distribuovaneDokumentyDASHBOARD($output, $moje = true)
    {
        $html =
            '<table class="dashboardTable" data-ron-href="?menuid=dokumenty">' .
            '<tr>' .
            ($moje ? '' : '<th>' . TextManager::_('Zamestnanec') . '</th>')  .
            '<th>' . TextManager::_('Date') . '</th><th>' . TextManager::_('Typ_dokumentu') . '</th><th>' . TextManager::_('Popis') . '</th>' .
            '<th>' . TextManager::_('identifikace_podpisu') .  '</th></tr>';

        $params = $this->database->createParameters();

        $sql = 'SELECT Dokument.Id, Dokument.Datum, TypDokumentu.Popis, Dokument.Popis, Dokument.JmenoSouboru, TypDokumentu.DefinicePrava, 
            DokumentVzkazu.IdVzkazu, DokumentVzkazu.PlatnostOd, TypDokumentu.Distribuovany, DokumentVzkazu.Precteno, Vzkaz.DatumCas, ' . //6 - 10
            $this->database->castWideMemo('DokumentVzkazu.IdentifikacePodpisu') . ', ' . $this->database->castWideMemo('DokumentVzkazu.Vyjadreni') . ', ' . // 11 - 12
            'Vzkaz.RCPrijemce, VerzeDokumentu.PotvrditDo, Osoba.Jmeno, Osoba.Prijmeni '; //13 - 16

        $sql .=
            'FROM Dokument JOIN TypDokumentu ON (Dokument.Typ = TypDokumentu.Kod) 
            JOIN VerzeDokumentu ON (Dokument.id = VerzeDokumentu.id)
            JOIN DokumentVzkazu ON (VerzeDokumentu.id = DokumentVzkazu.idDokumentu AND VerzeDokumentu.PlatnostOd = DokumentVzkazu.PlatnostOd)
            JOIN Vzkaz ON (Vzkaz.id=DokumentVzkazu.idVzkazu)
            JOIN Osoba ON (Osoba.RC = Vzkaz.RCPrijemce)';

        //13.4.2023 RON-PaRu doplnena podminka TypDokumentu.Distribuovany = 1, budou se nacitat jen distribuovane vyzadujici precteni, na zadost MaKr
        $sql .=
            " WHERE TypDokumentu.Distribuovany = 1 AND DokumentVzkazu.Precteno = 0 ";

       $sql .= $moje ? $this->mojeNeprecteneDokumenty($params) : $this->neprecteneDokumentyPodrizenych($params);
        
        $cursor = $this ->database ->getSqlResult($sql, $params);
        while ($row = $cursor ->fetch()) {
            $class = '';
            if (DateUtils::startOfTheDay(time()) >= $this->database->stringToTimestamp($row[14])) {
                $class = 'class="distribuovany_po_terminu"';
            }

            $html .=
                '<tr>' .
                ($moje ? '' : '<td>' . StringUtils::htmlTextConvert($row[15]) . ' ' . StringUtils::htmlTextConvert($row[16]) . '</td>')  .
                '<td>' . StringUtils::format('date', $this->database->stringToTimestamp($row[1])) . '</td>' .
                '<td>' . StringUtils::htmlTextConvert($row[2]) . '</td>' .
//                '<td>' . StringUtils::htmlTextConvert($row[3]) . '</td>' .
                "<td><a href='?menuid=dokumenty&distribuovanedokumenty=1&dokumentid=$row[0]&vzkazid=$row[6]&platnostod=" . StringUtils::format('date', $this->database->stringToTimestamp($row[7])) ."' target='_blank'>" .
                    StringUtils::htmlTextConvert($row[3]) . '</a></td>' .
                "<td>";

            if ($row[8] == 1) {
                $action = $moje ? 'potvrzeniprecteniform=1&formaction=edit' : 'distribuovanedetail=1';
                $html .=
                    "<a href='?menuid=dokumenty&$action&vzkazid=$row[6]&dokumentid=$row[0]&platnostod=" . StringUtils::format('date', $this->database->stringToTimestamp($row[7])) . "' target='_blank'>" .
                    ($moje ? TextManager::_('potvrzeni_precteni') : TextManager::_('detail')) . '</a>' .
                    ($row[14] ? ', <span ' . ($class ?: '') . '>' . TextManager::_('dokumenty_potvrdit_do') . ': ' . DBUtils::getFormatedValue($this->database, 'finedate', $row[14]) . '</span>' : '');
            }
            $html .=
                '</td>' .
                '</tr>';
        }

        $html .= '</table>';

        $output ->addText('html', $html);
        $output->addDebugMessage($sql);
    }

    function handleJsonAction($action, $output)
    {
        switch ($action) {
            case 'mojedisdokumenty' :
                $this ->distribuovaneDokumentyDASHBOARD($output);
                break;
            case 'disdokumentypodrizenych' :
                $this ->distribuovaneDokumentyDASHBOARD($output, false);
                break;

            case 'dokumentupload' :
                $this->uploadDokument();
                break;
        }
    }

    /**
     * @param string $action
     * @return ViewObject|DokumentOutputObject|JsonOutputObject
     * @throws Exception
     */
    public function handleAction($action)
    {
        if (!RonPortalSettings::getIntValue(RonPortalSettings::ZOBRAZIT_DOKUMENTY)) {
            throw new Exception('Zvolena funkce neni povolena!');
        }


        Prava::nastavObdobiProPrava(time(), $this->vratObdobiDoProPrava());

        if ($action == 'vyplatnilistky') {
            return $this->handleVyplatniListky();
        }

        if (isset($_GET['action'])) {
            switch ($_GET['action']) {
                case 'mojedisdokumenty' :
                case 'disdokumentypodrizenych' :
                case 'dokumentupload' :
                    return parent::handleAction($_GET['action']);
            }
        }
//        if (isset($_GET['action']) && $_GET['action'] == 'mojedisdokumenty') {
//            return parent::handleAction($_GET['action']);
//        }

        $idDokumentu = isset($_GET['dokumentid']) ? (int)$_GET['dokumentid'] : null;

        // stahnuti dokumentu osoby
        if (isset($_GET['dokumentosoby'])  &&  $_GET['dokumentosoby']  &&  isset($idDokumentu)) {
            return $this ->handleDownloadDokumentuOsoby($idDokumentu);
        }

        // stahnuti dokumentu rozdeleni
        if (isset($_GET['dokumentrozdeleni'])  &&  $_GET['dokumentrozdeleni']  &&  isset($idDokumentu)) {
            return $this ->downloadDokumentRozdeleni((int)$idDokumentu,
                'RozdeleniOsob.KodTypuRozdeleni <> -1' // dokumenty prac. mist jsou zvlast
            );
        }

        // stahnuti dokumentu pracovniho mista
        if (isset($_GET['dokumentpmista'])  &&  $_GET['dokumentpmista']  &&  isset($idDokumentu)) {
            return $this ->downloadDokumentRozdeleni((int)$idDokumentu,
                'RozdeleniOsob.KodTypuRozdeleni = -1'
            );
        }



        $idVzkazu = isset($_POST['vzkazid']) ? (int)$_POST['vzkazid'] : (isset($_GET['vzkazid']) ? (int)$_GET['vzkazid'] : null);
        $platnostOd = isset($_POST['platnostod']) ? $_POST['platnostod'] : (isset($_GET['platnostod']) ? $_GET['platnostod'] : null);
        if ($platnostOd) {
            $platnostOd = DateUtils::dateStringToTimestamp($platnostOd);
        }

        // stahnuti distribuovaneho dokumentu
        if(isset($_GET['distribuovanedokumenty'])  &&  $_GET['distribuovanedokumenty'] && $idDokumentu && $idVzkazu && $platnostOd){
            return $this->downloadDistribuovanyDokument($idDokumentu, $platnostOd, $idVzkazu);
        }

        //Detail distribuovaneho dokumentu
        if(isset($_GET['distribuovanedetail']) && $_GET['distribuovanedetail'] && $idDokumentu && $idVzkazu && $platnostOd){
            return $this->detailDokumentu($idDokumentu, $platnostOd, $idVzkazu);
        }

        //Potvrzeni precteni
        if(isset($_GET['potvrzeniprecteniform'])  &&  $_GET['potvrzeniprecteniform']){
            return $this->potvrditPrecteni();
        }

        // Seznam dokumentu
        $result = new DokumentyOsobyBrowser($this ->database);
        $result ->load();

        return $result;
    }
}
