<?php
/**
 * stanislav.elbl, 20.07.2015
 */

namespace RonPortal\Controller;

use Ron\DB\Database;
use Ron\MVC;
use Ron\TextManager;
use RonPortal\Controls\ComboBoxZamestnanec;
use RonPortal\Log;
use RonPortal\NeniPravoException;
use RonPortal\Prava;
use RonPortal\User;


class LoginAsController implements MVC\Controller
{
    /** @var \Ron\DB\Database */
    var $database;


    public function __construct($database)
    {
        $this ->database = $database;
    }


    /**
     * Provede prihlaseni aktualniho uzivatele "jako" - musi byt zadano $idPP nebo $rc (prave jedno z nich)
     * @param Database $database
     * @param int $idPP
     * @param ?string $rc
     * @throws \Exception
     */
    public static function loginAsUser(Database $database, $idPP, $rc = null)
    {
        $params = $database ->createParameters();

        if ($idPP) {
            if ($rc) {
                throw new \Exception('Chyba v metode loginAsUser - pokud je zadano idPP, nesmi byt zadano rc!');
            }

            $sql =
                'SELECT Osoba.RC, Osoba.Jmeno, Osoba.Prijmeni, Osoba.SkupinaIW, Osoba.IntrawebLogin, Osoba.IdFotografie' .
                ' FROM Osoba INNER JOIN PracovniPomer' .
                '  ON Osoba.RC = PracovniPomer.RC' .
                ' LEFT OUTER JOIN SkupinaUzivateluIW' .
                '  ON SkupinaUzivateluIW.Kod = Osoba.SkupinaIW' .
                ' WHERE PracovniPomer.Id = ' . $params ->addParameter((int)$idPP);
        } else {
            if (!$rc) {
                throw new \Exception('Chyba v metode loginAsUser - neni zadano idPP ani rc!');
            }

            $sql =
                'SELECT Osoba.RC, Jmeno, Prijmeni, SkupinaIW, IntrawebLogin, Osoba.IdFotografie' .
                ' FROM Osoba ' .
                ' LEFT OUTER JOIN SkupinaUzivateluIW' .
                '  ON SkupinaUzivateluIW.Kod = Osoba.SkupinaIW' .
                ' WHERE RC = ' . $params ->addParameter($rc);
        }

        $cursor = $database ->getSqlResult($sql, $params);
        $row = $cursor ->fetch();
        $cursor ->close();

        if (!$row) {
            throw new \Exception('Chyba! Zadany uzivatel nebyl nalezen!');
        }

        // nastavi hodnoty do session
        User::setRealLoginName( User::getLoginName() );
        User::setRealUserRC( User::getRC() );
        User::setRealUserName( User::getUserName() );
        LoginController::nastavSessionLogin($row[0],
            User::getUserName() . ' ' . TextManager::_('Prihlasen_jako') . ' ' . $row[1] . ' ' . $row[2],
            $row[5]); // fotoId
        LoginController::nastavSkupinuUzivatele($database, $row[3]);
        User::setLoginName($row[4]);

        Prava::clear();
        //Prava::nactiPravaUzivatele($database);

        unset($_SESSION['jevpredavacimprocesu']);
    }


    /**
     * @param int $idPP
     * @return null
     * @throws \Exception
     */
    public function handleLoginAs($idPP)
    {
        $prava = Prava::vratPrava('loginas');
        if ( !$prava ->lzeZobrazitPP($idPP, time(), time()) ) {
            throw new NeniPravoException();
        }

        self::loginAsUser($this ->database, $idPP);

//        $params = $this ->database ->createParameters();
//        $sql =
//            'SELECT Osoba.RC, Jmeno, Prijmeni, SkupinaIW, IntrawebLogin ' .
//            ' FROM Osoba, PracovniPomer ' .
//            ' WHERE Osoba.RC = PracovniPomer.RC AND PracovniPomer.Id = ' . $params ->addParameter((int)$idPP);
//
//        $cursor = $this ->database ->getSqlResult($sql, $params);
//        $row = $cursor ->fetch();
//        $cursor ->close();
//
//        if (!$row) {
//            throw new \Exception('Chyba! Zadany uzivatel nebyl nalezen!');
//        }
//
//        // nastavi hodnoty do session
//        User::setRealLoginName( User::getLoginName() );
//        User::setRealUserRC( User::getRC() );
//        LoginController::nastavSessionLogin($row[0], User::getUserName() . ' ' . \Ron\ronText('Prihlasen_jako') . ' ' . $row[1] . ' ' . $row[2], $row[3]); // rc, jmeno, prijmeni, skupinaIW
//
//        Prava::clear();
//        Prava::nactiPravaUzivatele($this ->database);
//
//        unset($_SESSION['jevpredavacimprocesu']);

        Prava::nactiPravaUzivatele($this ->database);

        Log::addRecordToLog($this->database,
            sprintf(TextManager::_('Login_as_log'), User::getRealLoginName(), User::getLoginName()),
            TextManager::_('Login_log_category'),
            sprintf(TextManager::_('Login_as_log_comment'), User::getUserName())
        );

        //User::setLoginName($row[4]);

        return new MVC\Redirect(null, '?menuid=afterloginas');
    }


    /**
     * @param string $action
     * @return MVC\InternalRedirect|MVC\Redirect|string
     * @throws \Exception
     */
    public function handleAction($action)
    {
        global $enable_login_as;
        if (!$enable_login_as) {
            throw new \Exception('Tato funkce neni povolena!');
        }

        if ( !User::userAuthenticated() ) {
            throw new NeniPravoException();
        }

        TextManager::getInstance() ->loadStrings('users.ini');

        if (User::isLoggedAs()) {
            throw new \Exception( TextManager::_('Jiz_jste_prihlaseno_jako') );
        }


        Prava::nastavObdobiProPrava(time(), time());

        if ( isset($_POST['osoba'])  &&  $_POST['osoba'] ) {
            return $this ->handleLoginAs($_POST['osoba']);
        }

        $prava = Prava::vratPrava('loginas');

        if (!$prava ->jsouNejakePrava()) {
            throw new NeniPravoException();
        }

        $combo = new ComboBoxZamestnanec('osoba');
        $combo ->filtr = "(IntrawebLogin IS NOT NULL  AND  IntrawebLogin >= ' ')";  // AND  IntrawebLogin <> '' - s timhle to nefunguje na oracle!
        $combo ->loadDataPP($this ->database, time(), time(), $prava, null, null, true);
        $combo->className .= ($combo->className ? ' ' : '') . 'form-control ronselect';
        $combo->htmlParams['data-max-visible-rows'] = '200';

        MVC\Model::addData('comboOsoba', $combo);

        return new MVC\BaseViewObject('view/login_as.php');
    }


}
