<?php
/**
 * stanislav.elbl, 15.07.2015
 */

namespace RonPortal\Controller;

use Exception;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\DBUtils;
use Ron\MessageManager;
use Ron\MVC\BaseViewObject;
use Ron\MVC\InternalRedirect;
use Ron\MVC\Model;
use Ron\MVC\Redirect;
use Ron\MVC\TextViewObject;
use Ron\MVC\HttpController;
use Ron\MVC\ViewObject;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Common\ChangePasswordForm;
use RonPortal\Log;
use RonPortal\Prava;
use RonPortal\RonPortalDBRecord;
use RonPortal\RonPortalSession;
use RonPortal\RonPortalXMLEvents;
use RonPortal\SamlUtils;
use RonPortal\Security;
use RonPortal\SoftipUtils;
use RonPortal\User;
use RonPortal\XML\RonPortalXMLForm;

class LoginController extends HttpController
{
    /** @var Database */
    var $database;


    /**
     * @param Database $database
     */
    public function __construct($database)
    {
        $this->database = $database;
    }



    /**
     * Pri logout vycisti session
     */
    private static function clearSession()
    {
        $terminalVkladani = isset($_SESSION['terminalprovkladani']) ? $_SESSION['terminalprovkladani'] : null;

        $lang = RonPortalSession::getLanguage();

        session_unset();
        ////session_destroy();
        //session_start();

        $_SESSION['language'] = $lang;

        User::setLoginName('');
        User::setUserAuthenticated(false);

        $_SESSION['terminalprovkladani'] = $terminalVkladani;
    }


    public static function logoutUser()
    {
        self::clearSession();
    }



    public static function nastavSessionLogin($rc, $jmenoPrijmeni, $fotoId)
    {
        User::setUserName($jmenoPrijmeni);
        User::setRC($rc);
        User::setUserFotoId($fotoId);
    }


    public function loginPage()
    {
        global $login_xml_id;

        if ($login_xml_id) {
            return new InternalRedirect('xmllogin');
        }

        return 'view/login.php';
    }


    /**
     * @return Redirect|string
     * @throws Exception
     */
    public function handleLogout()
    {
        if (User::isAuthenticated()) {
            RonPortalXMLForm::xmlEvents($this->database, RonPortalXMLEvents::LOGOUT_BEFORE, [], '');
        }

        global $simplesaml_sp;

        if ($simplesaml_sp) {
            if (!User::isLoggedAs()) {
                SamlUtils::samlLogout();
//            global $simplesaml_path;
//
//            require_once ( ($simplesaml_path ?: '../simplesamlphp') .  '/lib/_autoload.php');
//            $as = new \SimpleSAML\Auth\Simple($simplesaml_sp);
//            if ($as ->isAuthenticated()) {
//                $as ->logout();
//            }
//
//            \SimpleSAML\Session::getSessionFromRequest()->cleanup();
                self::clearSession();
                MessageManager::addSuccessMessage(TextManager::_('Succesfully_logged_out'));
                return 'view/loggedoutview.php';
            } else {
                self::clearSession();
                return new Redirect(null, '');
            }

            // pokud je "prihlasen jako", tak jen vycisti session Portalu a prejde na prihlasovaci stranku, cimz dojde k prihlaseni skutecneho uzivatele
        }

        self::clearSession();

        if (RonPortalXMLForm::$eventRedirect) {
            return RonPortalXMLForm::$eventRedirect;
        }

        return $this ->loginPage();
    }


    /**
     * Vraci false, pokud je uzivatel povinen zmenit si heslo.
     * @param bool $vynutitZmenu
     * @param int $posledniZmena
     * @return bool
     * @throws Exception
     */
    protected function overPlatnostHesla($vynutitZmenu, $posledniZmena)
    {
        // kontrola, jestli je vyzadovana zmena hesla (vyprsela platnost nebo zaskrtnuto "vynutit zmenu")
        if ($vynutitZmenu) {
            return false;
        }

        $policy = Security::getPasswordPolicy($this ->database);

        if ($policy ->platnost > 0) {
            // kontrola, jestli nevyprselo heslo
            if (!$posledniZmena) {
                // jeste neni nastaveno "posledni zmena"
                return false;
            }

            if (DateUtils::incDay($posledniZmena, $policy ->platnost) < time()) {
                return false;
            }
        }

        return true;
    }


    /**
     * @param Database $database
     * @param string $skupinaIW
     * @throws DBException
     */
    public static function nastavSkupinuUzivatele(Database $database, $skupinaIW)
    {
        global $profilUzivatele;
        if ($profilUzivatele) {
            $skupinaIW = $profilUzivatele;
        }

        $params = $database ->createParameters();
        $sql = 'SELECT CssSoubor, Popis FROM SkupinaUzivateluIW WHERE Kod = ' . $params ->addParameter($skupinaIW);
        $row = DBUtils::getSqlValue($database, $sql, $params, true);

        if ($row) {
            User::setUserCss($row[0]);
            User::setSkupinaUzivatele($skupinaIW, $row[1]);
        }
    }


    private function jePlatnyPP($rc)
    {
        $params = $this ->database ->createParameters();
        $sql =
            'SELECT Id FROM PracovniPomer ' .
            ' WHERE PracovniPomer.RC = ' . $params->addParameter($rc) .
            '   AND PracovniPomer.DatumNastupu <= ' . $params ->addParameter($this ->database ->dateToParam(time())) .
            '   AND (PracovniPomer.DatumUkonceni IS NULL  OR  PracovniPomer.DatumUkonceni >= ' . $params ->addParameter($this ->database ->dateToParam(time())) . ')';

        $cursor = $this ->database ->getSqlResult($sql, $params, 'Chyba pri nacitani pracovnich pomeru!');
        $row = $cursor ->fetch();
        $cursor ->close();

        return !!$row;
    }


    /**
     * Volano po prihlaseni uzivatele - nastavi session a pripadne zaloguje prihlaseni
     * @param $loginName
     * @param $rc
     * @param $jmeno
     * @param $prijmeni
     * @param $skupinaIW
     * @param bool|string $autoLogin
     * @param bool $writeLog
     * @throws Exception
     */
    protected function userLoggedIn($loginName, $rc, $jmeno, $prijmeni, $skupinaIW, $fotoId, $autoLogin, $writeLog = true)
    {
        if ($autoLogin === true) {
            $autoLogin = 'ANO';
        }

        if (!$this ->jePlatnyPP($rc)) {
            $this ->zalogujChybnePrihlaseni(
                sprintf(StringUtils::fromUTF8(TextManager::_('Invalid_login_log')), $loginName),
                $loginName,
                'Nemate_platny_pp'
            );
        }

        global $profilUzivatele;
        if ($profilUzivatele) {
            $skupinaIW = $profilUzivatele;
        }

        session_regenerate_id(true);    // po kazdem prihlaseni zmeni session ID

        User::setLoginName($loginName);
        self::nastavSessionLogin($rc, $jmeno . ' ' . $prijmeni, $fotoId);
        self::nastavSkupinuUzivatele($this ->database, $skupinaIW);
        User::setUserAuthenticated(true);

        RonPortalSession::initSessionTimestamp();

        if ($writeLog) {
            global $db_utils;
            global $db_connectionstring;
            global $appVersion;

            Log::addRecordToLog($this->database, StringUtils::fromUTF8('Přihlášení uživatele: ') . $loginName,
                TextManager::_('Login_log_category'),
                StringUtils::fromUTF8('Uživatel: ') . $loginName . ' - ' . User::getUserName() . ' (profil ' . User::getSkupinaUzivatele() . ')' .
                "\nVerze Intrawebu: " . $appVersion .
                "\nVerze PHP: " . phpversion() .
                "\nURL: " . (($_SERVER['HTTPS'] ?? false) === 'on' ? 'https' : 'http') . "://$_SERVER[HTTP_HOST]$_SERVER[REQUEST_URI]" .
                "\nCharset: " . TextManager::getInstance() ->getCharSet() .
                "\nmbstring: " . (function_exists('mb_substr') ? 'ON' : 'OFF') .
                StringUtils::fromUTF8("\nPřipojení k databázi: ") . $db_utils . ', ' . substr($db_connectionstring, 0, 20) . // nevypisuje cele, aby tam nebylo heslo
                "\n  odbc.defaultlrl: " . ini_get('odbc.defaultlrl') .
                "\n  mssql.textlimit: " . ini_get('mssql.textlimit') .
                "\n  mssql.textsize: " . ini_get('mssql.textsize') .
                StringUtils::fromUTF8("\nAutomatické přihlášení: ") . ($autoLogin ?: 'NE') .
                (!$autoLogin && isset($_POST['adblock']) && $_POST['adblock'] ? StringUtils::fromUTF8("\nPravděpodobně je zapnutý ADBLOCK nebo vypnutý Javascript!") : '') .
                "\nSession\n  save_path: " . ini_get('session.save_path') .
                "\n  gc_maxlifetime: " . ini_get('session.gc_maxlifetime') .
                "\n  gc_probability: " . ini_get('session.gc_probability') .
                "\n  gc_divisor: " . ini_get('session.gc_divisor') .
                "\nTimezone: " . ini_get('date.timezone') .
                "\ndisplay_errors: " . ini_get('display_errors') .
                "\nlog_errors: " . ini_get('log_errors') .
                "\nerror_log: " . ini_get('error_log') .
                "\nerror_reporting: " . sprintf('%X', error_reporting()) .
                "\nmax_execution_time: " . ini_get('max_execution_time') .
                "\npost_max_size: " . ini_get('post_max_size') .
                "\nupload_max_filesize: " . ini_get('upload_max_filesize') .
                "\nupload_tmp_dir: " . ini_get('upload_tmp_dir') .
                "\nSCRIPT_FILENAME: " . $_SERVER['SCRIPT_FILENAME'] .
                "\nREMOTE_USER: " . $_SERVER['REMOTE_USER'] .
                "\nREMOTE_HOST: " . $_SERVER['REMOTE_HOST'] .
                "\nHTTP_USER_AGENT: " . $_SERVER['HTTP_USER_AGENT'] .
                "\nUser CSS: " . User::getUserCss()
            );
        }
    }


    /**
     * @param $text
     * @param $userName
     * @param $exceptionCode
     * @return void
     * @throws Exception
     */
    private function zalogujChybnePrihlaseni($text, $userName = null, $exceptionCode = 'Authentication_error')
    {
        global $appVersion;

        Log::addRecordToLog($this->database,
            StringUtils::fromUTF8('Chybný pokus o přihlášení uživatele') . ($userName ? ' ' . $userName : ''),
            TextManager::_('Login_log_category'),
            $text .
            "\nVerze Intrawebu: " . $appVersion .
            "\nVerze PHP: " . phpversion() .
            "\nURL: http://$_SERVER[HTTP_HOST]$_SERVER[REQUEST_URI]",
            $userName);

        if ($exceptionCode) {
            sleep(1);
            throw new Exception(TextManager::_($exceptionCode));
        }
    }

    /**
     * @param string $id
     * @throws Exception
     * @throws DBException
     */
    private function checkIDKartyLogin($id)
    {
        global $idlogin_kod2;

        $params = $this ->database ->createParameters();
        $sql =
            'SELECT Osoba.RC, Osoba.Jmeno, Osoba.Prijmeni, Osoba.IntrawebLogin, Osoba.SkupinaIW, Karta.Blokovana, Karta.DatumVyrazeni, Osoba.IdFotografie ' .
            ' FROM Osoba, PrirazeniKarty, Karta ' .
            ' WHERE Karta.Kod = PrirazeniKarty.KodKarty ' .
            ' AND ' .( $idlogin_kod2 ? 'Karta.Kod2' : 'PrirazeniKarty.KodKarty') . ' = ' . $params->addParameter($id) .
            ' AND PrirazeniKarty.RC = Osoba.RC ' .
            ' AND PrirazeniKarty.DatumPrideleni <= ' . $params ->addParameter($this ->database ->timestampToParam(time())) .
            ' AND (PrirazeniKarty.DatumVraceni >= ' . $params ->addParameter($this ->database ->dateToParam(DateUtils::startOfTheDay(time()))) .
            ' OR  PrirazeniKarty.DatumVraceni IS NULL) ';

        $cursor = $this ->database ->select($sql, $params);
        $row = $cursor ->fetch();
        $cursor ->close();

        if (!$row) {
            // Nenasel prirazeni karty
            $this ->zalogujChybnePrihlaseni(
                StringUtils::fromUTF8('Chybný pokus o přihlášení uživatele kartou - neplatná nebo nepřiřazená karta!'),
                $id,
                'Bad_id_medium');
        }

        if ($row[5]) {
            $this ->zalogujChybnePrihlaseni(
                StringUtils::fromUTF8('Chybný pokus o přihlášení uživatele kartou - karta je zablokovaná!'),
                $id,
                'Karta_je_zablokovana');
        }

        if ($row[6]) if ($this ->database ->stringToTimestamp($row[6]) <= time()) {
            $this ->zalogujChybnePrihlaseni(
                StringUtils::fromUTF8('Chybný pokus o přihlášení uživatele kartou - karta je vyřazená!'),
                $id,
                'Karta_je_vyrazena');
        }

        User::setLoginName($id);

        $this ->userLoggedIn($row[3] ?: $id, $row[0], $row[1], $row[2], $row[4], $row[7], false);  // loginName, rc, jmeno, prijmeni, skupinaIW
    }


    /**
     * @param string $id
     * @throws Exception
     * @throws DBException
     */
    private function checkOsobaLogin($jmenoPole, $id)
    {
        $params = $this ->database ->createParameters();
        $sql =
            'SELECT Osoba.RC, Osoba.Jmeno, Osoba.Prijmeni, Osoba.IntrawebLogin, Osoba.SkupinaIW, Osoba.IdFotografie' .
            ' FROM Osoba' .
            " WHERE Osoba.{$jmenoPole} = " . $params ->addParameter($id);

        $cursor = $this ->database ->select($sql, $params);
        $row = $cursor ->fetch();
        if ($row) {
            $row2 = $cursor ->fetch();
            if ($row2) {
                throw new Exception('Nelze přihlásit - existuje vice osob se stejným identifikátorem!');
            }
        }
        $cursor ->close();

        if (!$row) {
            // Nenasel osobu
            $this ->zalogujChybnePrihlaseni(
                StringUtils::fromUTF8('Chybný pokus o přihlášení uživatele pomocí URL - osoba nenalezena!'),
                $id,
                'Osoba_k_tokenu_nenalezena');
        }

        User::setLoginName($id);

        $this ->userLoggedIn($row[3] ?: $id, $row[0], $row[1], $row[2], $row[4], $row[5], false);  // loginName, rc, jmeno, prijmeni, skupinaIW
    }


    /**
     * @param string $id
     * @throws Exception
     * @throws DBException
     */
    private function checkPPLogin($jmenoPole, $id)
    {
        $params = $this ->database ->createParameters();
        $sql =
            'SELECT Osoba.RC, Osoba.Jmeno, Osoba.Prijmeni, Osoba.IntrawebLogin, Osoba.SkupinaIW, Osoba.IdFotografie' .
            ' FROM Osoba, PracovniPomer' .
            ' WHERE PracovniPomer.RC = Osoba.RC' .
            "   AND PracovniPomer.{$jmenoPole} = " . $params ->addParameter($id) .
            '   AND PracovniPomer.DatumNastupu <= ' . $params ->addDateParameter(time()) .
            '   AND (PracovniPomer.DatumUkonceni >= ' . $params ->addDateParameter(time()) . ' OR PracovniPomer.DatumUkonceni IS NULL)';

        $cursor = $this ->database ->select($sql, $params);
        $row = $cursor ->fetch();
        if ($row) {
            $row2 = $cursor ->fetch();
            if ($row2) {
                throw new Exception('Nelze přihlásit - existuje vice osob se stejným identifikátorem!');
            }
        }
        $cursor ->close();

        if (!$row) {
            // Nenasel osobu
            $this ->zalogujChybnePrihlaseni(
                StringUtils::fromUTF8('Chybný pokus o přihlášení uživatele pomocí URL - osoba nenalezena!'),
                $id,
                'Osoba_k_tokenu_nenalezena');
        }

        User::setLoginName($id);

        $this ->userLoggedIn($row[3] ?: $id, $row[0], $row[1], $row[2], $row[4], $row[5], false);  // loginName, rc, jmeno, prijmeni, skupinaIW
    }


    /**
     * @param string $id
     * @throws Exception
     * @throws DBException
     */
    function checkIDLogin($id)
    {
        switch ($GLOBALS['idloginOptions']['id'] ?? 'karta')
        {
            case 'rc' :
                $this ->checkOsobaLogin('RC', $id);
                break;

            case 'email' :
                $this ->checkOsobaLogin('EMail', $id);
                break;

            case 'login' :
                $this ->checkOsobaLogin('IntrawebLogin', $id);
                break;

            case 'osobni_cislo' :
                $this ->checkPPLogin('OsobniCislo', $id);
                break;

            case 'idpp' :
                $this ->checkPPLogin('Id', $id);
                break;

            case 'karta' :
            default :
                $this ->checkIDKartyLogin($id);
        }
    }


    /**
     * @param $loginName
     * @throws Exception
     */
    private function badPassword($loginName)
    {
        $this ->zalogujChybnePrihlaseni(
            TextManager::_('Bad_password'),
            $loginName,
            'Bad_password'
        );
    }


    /**
     * @param Database $database
     * @param string $loginName
     * @param string $password
     * @param string $dbPassword    Pokud neni zadano, nacte se z databaze
     * @return bool|string          Pri spravnem overeni hesla pomoci LDAP vraci "ldap", jinak vraci true/false
     * @throws Exception
     */
    public static function overHeslo($database, $loginName, $password, $dbPassword = null)
    {
        global $ldap_server;
        global $ldap_only_existing_users; // priznak, jestli ma pres ldap prihlasovat pouze existujici uzivatele

        if ($ldap_server) {
            //$result = Security::checkLDAPLogin($loginName, $password . '+');
            $result = Security::checkLDAPLogin($loginName, htmlspecialchars_decode($password));
            if (!$ldap_only_existing_users  ||  isset($result)) {
                User::setPasswordChangeEnabled(false);
                return $result ? "ldap" : false;
                // pokud uzivatel neexistuje a je nastaveno $ldap_only_existing_users, pokracuje dale (na standardni overeni dochazkou)
            }
        }

        if (!isset($dbPassword)) {
            $params = $database ->createParameters();
            $sql =
                'SELECT IntrawebPassword FROM Osoba ' .
                ' WHERE UPPER(IntrawebLogin) = ' . $params->addParameter(strtoupper($loginName));
            $cursor = $database ->getSqlResult($sql, $params);
            $row = $cursor ->fetch();
            $cursor ->close();

            if (!$row) {
                // neni nalezen zadany uzivatel
                return false;
            }

            $dbPassword = $row[0] ?? '';
        }

        $result = Security::checkPassword( Security::getPasswordType($database), $password, $dbPassword);
        if ($result) {
            // probehlo overeni pomoci dochazky => muze si zmenit heslo
            User::setPasswordChangeEnabled(true);
        }

        return $result;
    }


    private function updateBadLoginCount($rc, $clear)
    {
        $rec = new RonPortalDBRecord($this ->database, 'Osoba', $this);

        $rec ->addStringField('Prijmeni', true);
        $rec ->addStringField('Jmeno', true);
        $rec ->addStringField('IntrawebLogin', true);

        $rec ->addIntField('PocetSpatnychPrihlaseni', true);
        $rec ->addTimestampField('PosledniSpatnePrihlaseni', $clear, time()); // pri clear je readonly

        $params = $this ->database ->createParameters();
        $rec ->setWhere(
            'RC = ' . $params ->addParameter($rc),
            $params
        );

        if ($rec ->loadRecord()) {
            $rec['PocetSpatnychPrihlaseni'] = $clear
                ? 0
                : ($rec['PocetSpatnychPrihlaseni'] ?? 0) + 1;
            $rec ->updateRecord();
        }
    }


    /**
     * Overi prihlaseni pomoci jmena a hesla. V pripade chyby (i spatne heslo) vyhodi vyjimku.
     *
     * @param string $loginName
     * @param string $password
     * @param bool   $autoLogin
     * @throws Exception
     */
    private function checkUserLogin($loginName, $password, $autoLogin)
    {
        if (trim($loginName) == '') {
            // jmeno nesmi byt prazdne!
            throw new Exception( TextManager::_('Bad_password') );
        }

        $row = null;
        $row2 = null;

        if (StringUtils::strlen($loginName) <= 40) {
            $params = $this->database->createParameters();
            $sql =
                'SELECT RC, Jmeno, Prijmeni, SkupinaIW, IntrawebPassword,' .
                ' VynutitZmenuHesla, PosledniZmenaHesla, IntrawebLogin, IdFotografie,' .
                ' PocetSpatnychPrihlaseni, PosledniSpatnePrihlaseni' .
                ' FROM Osoba' .
                ' WHERE UPPER(IntrawebLogin) = ' . $params->addParameter(strtoupper($loginName));

            $cursor = $this ->database ->select($sql, $params);
            $row = $cursor ->fetch();
            $row2 = $row ? $cursor ->fetch() : null;
            $cursor ->close();
        }

        if (!$row  &&  $GLOBALS['enable_email_login']  &&  StringUtils::strlen($loginName) <= 60) {
            // je povoleno i prihlasovani emailem
            $params = $this->database->createParameters();
            $sql =
                'SELECT RC, Jmeno, Prijmeni, SkupinaIW, IntrawebPassword,' . // 0..4
                ' VynutitZmenuHesla, PosledniZmenaHesla, IntrawebLogin, IdFotografie,' . // 5..8
                ' PocetSpatnychPrihlaseni, PosledniSpatnePrihlaseni' . //9,10
                ' FROM Osoba' .
                ' WHERE eMail = ' . $params->addParameter($loginName);

            $cursor = $this ->database ->select($sql, $params);
            $row = $cursor ->fetch();
            $row2 = $row ? $cursor ->fetch() : null;
            $cursor ->close();
        }

        if (!$row) {
            // nanasel uzivatele
            $this ->badPassword($loginName); // vyhodi vyjimku
        }


        if ($row[9] && $row[10]) {
            // kontrola zablokovani uctu z duvodu spatne zadaneho hesla
            $pocet = min($row[9], 15); // max 15, aby nedoslo k preteceni
            $sec = 1 << $pocet;

            $sec = min($sec, $GLOBALS['maxaccountblocktime'] ?? 600); // vychozi maximalni doba blokovani uctu 10 minut
            $time = $this ->database ->stringToTimestamp($row[10]) + $sec;

            if ($time >= time()) {
                throw new Exception(sprintf(TextManager::_('Uzivatelsky_ucet_je_docasne_zablokovan'), StringUtils::format('fulltime', $time)));
            }
        }

        //global $ldap_server;
        global $simplesaml_sp;

        $overeno = null;

        if (!$autoLogin) {
            // kontrola hesla
            $overeno = self::overHeslo($this ->database, $loginName, $password, $row[4]);
            $this ->updateBadLoginCount($row[0], $overeno);
            if (!$overeno) {
                $this ->badPassword($loginName); // vyhodi vyjimku
            }
        }

        if ($row2) {
            // existuje jeste jiny uzivatel se stejnym jmenem
            MessageManager::addErrorMessage(StringUtils::fromUTF8('V databázi existuje ještě další uživatel se stejným přihlašovacím jménem! To může způsobovat problémy!'));
        }

        $this ->userLoggedIn($row[7], $row[0], $row[1], $row[2], $row[3], $row[8], $autoLogin);

        // heslo je zadano spravne, nebo je automaticke prihlasovani
        User::setVynutitZmenuHesla(false);

        if ( $overeno !== 'ldap'  &&  !$simplesaml_sp ) {
            User::setPasswordChangeEnabled(true);
            // kontrola, jestli heslo nevyprselo
            if (!$this->overPlatnostHesla($row[5], $this->database->stringToTimestamp($row[6]))) {
                if (Prava::vratPrava('zmenahesla')->lzeZobrazitRC($row[0])) {
                    User::setVynutitZmenuHesla(true);
                }
            }
        }
    }


    /**
     * @param $id
     * @return void
     * @throws DBException
     * @throws Exception
     */
    private function secureIdLogin($id)
    {
        global $idloginOptions;

        if ($idloginOptions['passphrase'] ?? false) {
            // data jsou zasifrovana, tj. url parametr id obsahuje base64 retezec ve formatu <id>;<time>
            $data = openssl_decrypt($id,
                $idloginOptions['cipher_algo'] ?? 'AES-128-CBC',
                $idloginOptions['passphrase'],
                0,
                $idloginOptions['iv'] ?? ''
            );

            $items = explode($idloginOptions['separator'] ?? ';', $data);

            $id = $items[0] ?? '';
            $timeStr = $items[1] ?? '';
            $salt = $items[2] ?? '';
            $key = $items[3] ?? '';
        } else {
            $timeStr = $_GET['time'] ?? '';
            $salt = $_GET['salt'] ?? '';
            $key = $_GET['key'] ?? '';
        }

        if (!$id) {
            $this ->zalogujChybnePrihlaseni('Chybný pokus o přihlášení uživatele pomocí URL - není zadáno ID (id osoby nebo karty)!');
        }

        if ($validity = $idloginOptions['validity'] ?? 60) {
            // je zadana maximalni doba platnosti pozadavku
            if (!$timeStr) {
                $this->zalogujChybnePrihlaseni(
                    StringUtils::fromUTF8('Chybný pokus o přihlášení uživatele pomocí URL - není zadáno časové razítko!'),
                    $id);
            }

            try {
                $time = strtotime($timeStr);
            } catch (Exception $e) {
                $this->zalogujChybnePrihlaseni(
                    StringUtils::fromUTF8('Chybný pokus o přihlášení uživatele pomocí URL - neplatné časové razítko!'),
                    $id);
            }

            if ($time < time() - $validity  ||  $time > time() + $validity) {
                $this->zalogujChybnePrihlaseni(
                    StringUtils::fromUTF8('Chybný pokus o přihlášení uživatele pomocí URL - požadavek již není platný!'),
                    $id);
            }
        }

        if ($idloginOptions['hash'] ?? 'sha1') {
            $hashText = str_replace(
                ['%ID%', '%TIME%', '%SALT%'],
                [$id, $timeStr, $salt],
                $idloginOptions['hashFormat'] ?? '%ID%;%TIME%;%SALT%'
            );

            switch ($idloginOptions['hash'] ?? '') {
                case 'md5' :
                    $hash = md5($hashText);
                    break;
                case 'sha1' :
                    $hash = sha1($hashText);
                    break;
                default :
                    $hash = '';
            }

            if ($hash != $key) {
                $this->zalogujChybnePrihlaseni(
                    StringUtils::fromUTF8('Chybný pokus o přihlášení uživatele kartou - špatný hash!'),
                    $id);
            }
        }

        $this ->checkIDLogin($id);
    }

    private function redirect()
    {
        $urlParams = [];
        parse_str($_SERVER['QUERY_STRING'], $urlParams);

        $newParams = [];

        if (isset($_GET['idlogin'])) {
            // pro idlogin prenasi jen pripadne menuid
            if (isset($_GET['menuid'])  &&  $_GET['menuid'] != 'login') {
                $newParams['menuid'] = $_GET['menuid'];
            }
        } else {
            foreach ($urlParams as $key => $value) {
                if (strpos($key, 'session@') !== 0 && $key != 'loginpassword' && $key != 'loginname' && $key != 'token') {  //  && $key != 'idlogin'
                    if ($key != 'menuid' || $value != 'login') { // menuid=login dela problem, pokud existuje uzivatelsky login controller
                        $newParams[$key] = $value;
                    }
                }
            }
        }

        return new Redirect(null, '?' .http_build_query($newParams));
    }


    /**
     * @return bool|Redirect|string
     */
    protected function handleLogin()
    {
        global $activedirectorylogin; // nastaveno v _appdbset.php
        global $enableurlidlogin;     // => zapnuto nepovinne prihlaseni pomoci id v url ( ?idlogin=...)

        try {
            global $simplesaml_sp;
            global $enableidlogin;

            if ($simplesaml_sp) {
                $userName = SamlUtils::samlLogin();
                self::clearSession();
                try {
                    $this->checkUserLogin($userName, null, 'SAML');
                } catch (Exception $e) {
                    if (TextManager::_('SAML_login_error')) {
                        MessageManager::addErrorMessage(TextManager::_('SAML_login_error'));
                    }
                    MessageManager::addErrorMessage($e);
                    return 'view/loggedoutview.php';
                }

            } else if ($enableidlogin  &&  isset($_POST['loginname'])) {
                // povinne prihlaseni pomoci ID media
                self::clearSession();
                $this ->checkIDLogin($_POST['loginname']);

            } else if (isset($_POST['loginname'])) {
                // prihlaseni z logovaci stranky
                self::clearSession();
                $this ->checkUserLogin($_POST['loginname'], $_POST['loginpassword'], false);

            } else if (isset($_GET['loginname'])) {
                // prihlaseni primo v url
                self::clearSession();
                $this->checkUserLogin($_GET['loginname'], $_GET['loginpassword'], false);

            } else if ( $enableurlidlogin === 'secure'  &&  isset($_GET['idlogin']) ) {
                self::clearSession();
                $this->secureIdLogin($_GET['idlogin']);
            } else if ( $enableurlidlogin  &&  isset($_GET['idlogin']) ) {
                self::clearSession();
                $this->checkIDLogin($_GET['idlogin']);
            } else if ($activedirectorylogin) {
                // Prihlaseni pomoci active directory
                $remote_user = $_SERVER['REMOTE_USER'];

                if (!empty($remote_user)) {
                    self::clearSession();
                    // uzivatelske jmeno je za poslednim lomitkem
                    $actual_username = substr(strrchr($remote_user, '\\'), 1);
                    if (!$actual_username) {
                        // zadne lomitko v nazvu uzivatele (muze byt napriklad emailova adresa)
                        $actual_username = $remote_user;
                    }
                    if ($GLOBALS['activedirectorysuffix'] && !str_ends_with($actual_username, $GLOBALS['activedirectorysuffix'])) {
                        $actual_username = $actual_username . $GLOBALS['activedirectorysuffix'];
                    }
                    $this ->checkUserLogin($actual_username, '', true);
                }

                if (!User::userAuthenticated()) {
                    throw new Exception( TextManager::_('Nelze_automaticky_prihlasit') );
                }
            }
        } catch (Exception $e) {
            MessageManager::addErrorMessage($e ->getMessage());
            return $this ->loginPage();
        }

        if (User::userAuthenticated()) {
            $_SESSION['AfterLogin'] = 1;
            return $this ->redirect();
        } else {
            return $this ->loginPage();
        }
    }


    /**
     * Overi prihlaseni osoby se zadanym osobnim cislem. V pripade chyby (osoba neexistuje nebo nema platny PP) vyhodi vyjimku.
     * Pouziva se i z uzivatelskych kontroleru!
     * @param string $osobniCislo
     * @throws Exception
     */
    protected function checkPersonLogin($osobniCislo)
    {
        $params = $this ->database ->createParameters();
        $sql =
            'SELECT Osoba.RC, Jmeno, Prijmeni, SkupinaIW, IntrawebPassword, VynutitZmenuHesla, PosledniZmenaHesla, IntrawebLogin, Osoba.IdFotografie ' .
            '  FROM Osoba, PracovniPomer ' .
            ' WHERE Osoba.RC = PracovniPomer.RC ' .
            '   AND PracovniPomer.OsobniCislo = ' . $params->addParameter($osobniCislo) .
            '   AND PracovniPomer.DatumNastupu <= ' . $params ->addParameter($this ->database ->dateToParam(time())) .
            '   AND (PracovniPomer.DatumUkonceni IS NULL  OR  PracovniPomer.DatumUkonceni >= ' . $params ->addParameter($this ->database ->dateToParam(time())) . ')';

        $cursor = $this ->database ->getSqlResult($sql, $params, 'Chyba pri nacitani osoby!');
        $row = $cursor ->fetch();
        $cursor ->close();

        if (!$row) {
            throw new Exception( TextManager::_('Osoba_k_tokenu_nenalezena') );
        }

        // osoba se zadanym osobnim cislem nalezena
        User::setVynutitZmenuHesla(false);

        $this ->userLoggedIn($row[7], $row[0], $row[1], $row[2], $row[3], $row[8], false);  // loginName, rc, jmeno, prijmeni, skupinaIW
    }


    function handleTokenLogin()
    {
        try {
            global $softipLoginUrl;

            if (isset($softipLoginUrl)) {
                // login pomoci Softip tokenu
                $osobniCislo = SoftipUtils::tokenUserAuthentication( $_GET['token'] );

                if (!$osobniCislo) {
                    throw new Exception( TextManager::_('Neplatny_token') );
                }

                $this ->checkPersonLogin($osobniCislo);
            } else {
                throw new Exception('Chyba - neni zadana url pro overeni tokenu!');
            }

        } catch (Exception $e) {
            MessageManager::addErrorMessage($e->getMessage());
            return $this ->loginPage();
        }

        if (User::userAuthenticated()) {
            return $this->redirect();
        } else {
            return $this ->loginPage();
        }

    }


//    /**
//     * @param string $oldpass
//     * @param string $newpass1
//     * @param string $newpass2
//     * @return bool
//     * @throws Exception
//     */
//    function setPassword($oldpass, $newpass1, $newpass2)
//    {
//        if ($newpass1 != $newpass2) {
//            throw new Exception( TextManager::_('Hesla_se_neshoduji') );
//        }
//
//        if (User::getVynutitZmenuHesla() && $oldpass == $newpass1) {
//            // pokud je povinna zmena hesla, tak nesmi byt stare heslo stejne jako nove
//            throw new Exception( TextManager::_('Nove_heslo_je_stejne') );
//        }
//
//        $policy = Security::getPasswordPolicy($this ->database);
//
//        Security::checkPasswordPolicy($newpass1, $policy);
//
//        if (!Security::checkUserPassword($this ->database, User::getRC(), $oldpass)) {
//            $this ->zalogujChybnePrihlaseni(
//                StringUtils::fromUTF8('Chybný pokus o změnu hesla uživatele - špatně zadané původní heslo!'),
//                User::getLoginName(),
//                'Spatne_puvodni_heslo'
//            );
//        }
//
//        $passwordType = Security::getPasswordType($this ->database);
//        $newpass1 = Security::encodePassword($passwordType, $newpass1);
//
//        $params = $this ->database ->createParameters();
//        $this ->database ->sql(
//                'UPDATE Osoba SET IntrawebPassword = ' . $params->addParameter($newpass1) .
//                ', VynutitZmenuHesla = 0,  PosledniZmenaHesla = ' . $params ->addParameter($this ->database ->timestampToParam(time())) .
//                ' WHERE RC = ' . $params->addParameter(User::getRC()), $params);
//
//        Log::addRecordToLog($this->database,
//            StringUtils::fromUTF8('Proběhla změna hesla uživatele: ') . User::getLoginName(),
//            TextManager::_('Login_log_category'));
//
//        User::setVynutitZmenuHesla(0);
//
//        return true;
//    }


    /**
     * @return bool|Redirect|string|ViewObject
     */
    function handleChangePassword()
    {
        if (isset($_GET['changed'])  &&  $_GET['changed']) {
            Model::addViewObject(new TextViewObject(TextManager::_('Zmena_hesla'), '', 'h1'));

            return new TextViewObject(
                '<div class="successmsg">' . TextManager::_('Heslo_zmeneno') . '</div>' .
                "<p><a href='?'>" . TextManager::_('Pokracovat') . "</a></p>\n"
            );
        }

        if (User::getVynutitZmenuHesla()) {
            MessageManager::addErrorText( 'Platnost_hesla_vyprsela' );
        }

        $frm = new ChangePasswordForm($this ->database);
        if ($frm ->handleAction()) {
            return new Redirect(null, '?menuid=changepassword&changed=1');
        }

        return $frm;
    }


    public function handleUserLogin()
    {
        return false;
    }


    /**
     * @param string $action
     * @return string|Redirect|InternalRedirect
     * @throws Exception
     */
    public function handleAction($action)
    {
        TextManager::getInstance() ->loadStrings('users.ini');

        if ($action === 'logout') {
            return $this->handleLogout();
        }

        // uzivatelsky definovane prihlaseni - z pripadneho uzivatelskeho controlleru
        $result = $this ->handleUserLogin();
        if ($result) return $result;

        global $tokenLogin;
        if (isset($tokenLogin)  &&  $tokenLogin  &&  isset($_GET['token'])) {
            return $this ->handleTokenLogin();
        }

        global $enableurlidlogin;
        if ($enableurlidlogin  &&  isset($_GET['idlogin'])) {
            // prihlaseni pomoci ID v URL
            return $this ->handleLogin();
        }

        if ( isset($_GET['loginname']) ) {
            // prihlaseni v url - i kdyz je uzivatel prihlaseny (aby se mohl prihlasit jiny)
            return $this ->handleLogin();
        }

        if (!User::userAuthenticated()) {
            return $this ->handleLogin();
        }

        if ($action == 'changepassword'  ||  User::getVynutitZmenuHesla() ) {
            return $this ->handleChangePassword();
        }

        return null;
    }


}
