<?php
/**
 * stanislav.elbl, 02.07.2021 17:14
 */

namespace RonPortal\Controller;

use Ron\Controls\Preferences;
use Ron\DB\Database;
use Ron\Http\HtmlHeaders;
use Ron\MessageManager;
use Ron\MVC\BaseViewObject;
use Ron\MVC\Controller;
use Ron\MVC\Model;
use Ron\MVC\Redirect;
use Ron\TextManager;
use RonPortal\Prava;
use RonPortal\RonPortalDBRecord;
use RonPortal\RonPortalSession;
use RonPortal\RonPortalSettings;

class NastaveniPortaluController implements Controller
{
    /** @var Database */
    var $database;

    /**
     * NastaveniPortaluController constructor.
     * @param Database $database
     */
    public function __construct(Database $database)
    {
        $this->database = $database;
    }


    /**
     * @param Preferences $preferences
     * @param bool $binary
     * @param string $storage
     * @return mixed
     * @throws \Ron\DB\DBException
     */
    private function getPreferencesSysVar($preferences, $binary, $storage = null)
    {
        $storage = $storage ?: $preferences ->storage;

        $params = $this ->database ->createParameters();

        $sql =
            'SELECT Name, Val FROM ' . $storage .
            //'SELECT Name, {fn_CASTWIDEMEMO(CAST(Val AS VARCHAR(MAX)))} FROM ' . $storage .
            ' WHERE 0 = 1';

        foreach ($preferences ->items as $preference) {
            $preferenceStorage = $preference ->storage ?: $preferences ->storage;
            if ( !strcasecmp($preferenceStorage, $storage) ) {
                $sql .= ' OR Name = ' . $params->addParameter($preference->name);
            }
        }

        $result = [];
        $cursor = $this ->database ->getSqlResult($sql, $params);
        while ($row = $cursor ->fetch()) {
            $value = $binary ? $cursor ->stringToBinary($row[1]) : $row[1];

            if ($binary) {
                // v pripade potreby provede konverzi znakove sady, protoze u blob pole neprobiha automaticky
                global $db_target_charset;
                if ($db_target_charset) {
                    $charset = TextManager::getInstance() ->getCharSet();
                    if ($db_target_charset != $charset) {
                        $value = iconv($db_target_charset, $charset, $value);
                    }
                }
            }

            $result[$row[0]] = $value;
        }

        $cursor ->close();

        return $result;
    }


    private function getPreferencesNastaveniProIntraweb($storage)
    {
        $sql = "SELECT {fn_CASTWIDEMEMO($storage)} FROM NastaveniProIntraweb";

        MessageManager::addDebugLog($sql);

        $cursor = $this ->database ->getSqlResult($sql);
        $row = $cursor ->fetch();
        $cursor ->close();

        return $row ? $row[0] : null;
    }


    public function getPreferencesText($preferences, $storage)
    {
        switch (strtolower($storage)) {
            case 'nastavenispolecne' :
            case 'nastaveni' :
            case 'nastavenijidelny' :
            case 'nastavenipersonalistiky' :
                return $this ->getPreferencesNastaveniProIntraweb($storage);

            case 'sys_bvar_global' :
                return $this ->getPreferencesSysVar($preferences, true, $storage);

            case 'sys_var_global' :
                return $this ->getPreferencesSysVar($preferences, false, $storage); // binary = false
        }

        throw new \Exception('Neznamy zpusob ulozeni nastaveni!');
    }


    private function getOldPreference($name, $storage = 'nastaveni')
    {
        $name = strtolower($name);
        $preferencesText = $this ->getPreferencesNastaveniProIntraweb($storage);
        $str_settings = explode("\n", $preferencesText);

        foreach ($str_settings as $setStr) {
            $pom = sscanf($setStr, '%[^=]=%[^$]'); // , &$key, &$value
            $key = strtolower(trim($pom[0] ?? ''));
            if ($key === $name) {
                return str_replace('\n', "\n", trim($pom[1]));
            }
        }

        return null;
    }


    /**
     * @param Preferences $preferences
     * @throws \Ron\DB\DBException
     */
    public function loadValues($preferences)
    {
        $preferences ->loadItemValues($this ->getPreferencesText($preferences, $preferences ->storage), $preferences ->storage);

        foreach ($preferences ->items as $preference) {
            // nacte preference z jineho umisteni (pokud existuji)
            if ($preference ->storage  &&  strcasecmp($preference ->storage, $preferences ->storage)) {
                $preferences ->loadItemValues($this ->getPreferencesText($preferences, $preference ->storage), $preference ->storage);
            }

            // zkontroluje jeste, jestli neni nejaka polozka presunuta do jineho umisteni (z tabulky NastaveniProIntraweb do napr. SYS_BVAR_GLOBAL)
            // pokud ano, tak nacte starou hodnotu (jestli uz neni nastavena v novem umisteni)
            // slouzi pro presun polozky z NastaveniProIntraweb.nastaveni do (napr.) SYS_BVAR_GLOBAL
            if ($preference ->oldPrefName  &&  !$preference ->value) {
                // prazdna hodnota, ale je nastaveno oldPrefName => hodnota muze byt ve starem umisteni
                $preference ->value = $this->getOldPreference($preference ->oldPrefName);
            }
        }
    }


    /**
     * @param string $dirName
     * @return Preferences[]
     */
    public function loadPreferencesDir($dirName)
    {
        $dh = opendir($dirName);
        if (!$dh) {
            // zadany adresar pravdepodobne neexistuje
            return [];
        }

        $fileList = [];
        while (false !== ($fileName = readdir($dh))) {
            if (!is_dir($dirName . '/' . $fileName)) $fileList[] = $fileName;
        }

        $result_loadPreferencesDir = [];

        foreach ($fileList as $fileName) {
            /** @noinspection PhpIncludeInspection */
            $initArray = require $dirName . '/' . $fileName;

            if ($initArray) {
                if ($initArray instanceof Preferences) {
                    //var_dump($initArray);
                    $result_loadPreferencesDir[$initArray ->index] = $initArray;
                } else if (is_array($initArray)) {
//                    var_dump($initArray);
                    $pref = new Preferences();
                    $pref->init($initArray);
                    //$result_loadPreferencesDir[$initArray['index']] = $pref;
                    $result_loadPreferencesDir[] = $pref;
                }

            }
        }

        return $result_loadPreferencesDir;
    }


    /**
     * @return Preferences[]
     */
    public function loadPreferences()
    {
        $preferencesList = $this ->loadPreferencesDir('nastaveni');

        // nacte uzivatelske
        $userPreferences = $this ->loadPreferencesDir('usersettings');
        foreach ($userPreferences as $pref) {
            $preferencesList[] = $pref;
        }

        // seradi podle indexu
        usort($preferencesList,
            function($item1, $item2) {
                return $item1 ->index > $item2 ->index ? 1 : ($item1 ->index < $item2 ->index ? -1 : 0);
            }
        );

        return $preferencesList;
    }

    /**
     * @param Preferences[] $preferencesList
     * @param Preferences   $prefs
     * @return BaseViewObject
     */
    public function showPreferences($preferencesList, $prefs)
    {
        if ($prefs ->strings) {
            TextManager::getInstance() ->loadStrings($prefs ->strings);
        }

        $prefs ->loadData($this ->database);
        $this ->loadValues($prefs);

        Model::addData('preferencesList', $preferencesList);
        Model::addData('preferences', $prefs);
        return new BaseViewObject('ron/preferences_view.php');
    }


    /**
     * @param Preferences[] $preferencesList
     * @param string $name
     * @return Preferences
     */
    public function findPreferences($preferencesList, $name)
    {
        foreach ($preferencesList as $preferences) {
            if ($preferences ->name == $name) {
                return $preferences;
            }
        }

        return null;
    }


    /**
     * Ulozi polozky v ini formatu do zadaneho pole tabulky NastaveniProIntraweb
     * @param Preferences $preferences
     * @param string $storage
     * @throws \Exception
     * @throws \Ron\DB\DBException
     */
    private function storePreferencesNastaveni($preferences, $storage)
    {
        $storage = $storage ?: $preferences ->storage;

        $rec = new RonPortalDBRecord($this->database, 'NastaveniProIntraweb');
        $rec ->checkMultiple = false;   // aby fungoval update bez where podmiky
        $f = $rec->addField($storage, 'memo', false, 0);
        $f ->maxLogLength = 20000;
        //$rec ->where = 'Id = 1';

        $logText = '';

        if ($rec->loadRecord()) {
            $text = $f->originalValue;
            $newText = $preferences->storeItemValues($text, $logText, $storage);

            $rec ->logText = $logText;

            $f->setNewValue($newText);

            $rec->updateRecord();
        } else {
            $newText = $preferences->storeItemValues('', $logText, $storage);

            $rec ->logText = $logText;

            $f->setNewValue($newText);
            $rec->insertRecord();
        }
    }


    /**
     * Ulozi polozky do sys_(b)var_global. Uklada jen polozky patrici do zadane storage a pokud neni zadana, tak do storage objektu $preferences
     * @param Preferences $preferences
     * @param string $storage
     */
    private function storePreferencesSysVar($preferences, $storage = null)
    {
        $storage = strtolower($storage ?: $preferences ->storage);

        $blob = strtolower($storage) == 'sys_bvar_global';

        foreach ($preferences ->items as $preference) {
            $preferenceStorage = strtolower($preference ->storage ?: $preferences ->storage);

            if ($preferenceStorage == $storage) {
                $dbValue = $preference ->value;

                $params = $this->database->createParameters();
                $record = new RonPortalDBRecord($this->database, $storage);

                $record->addField('Name', '', true, 40, $preference->name); // readonly
                $record->addField('Val', $blob ? 'blob' : '', false, $blob ? 0 : 100, $dbValue)
                    ->setConvertTargetCharset($blob);   // pokud je to blob, konvertuje se znakova sada pred zapsanim do DB, protoze to nedela driver automaticky
                $record->setWhere('Name = ' . $params->addParameter($preference->name), $params);

                if (is_null($preference->value) || ($preference->value === $preference->defaultValue)) {
                    $record->deleteRecord();
                } else {
                    $record->updateOrInsert();
                }
            }
        }
    }


    /**
     * @param Preferences $preferences
     * @param string $storage
     */
    public function storePreferencesToStorage($preferences, $storage)
    {
        switch (strtolower($storage)) {
            case 'nastavenispolecne' :
            case 'nastaveni' :
            case 'nastavenijidelny' :
            case 'nastavenipersonalistiky' :
                $this ->storePreferencesNastaveni($preferences, $storage);
                break;

            case 'sys_var_global' :
            case 'sys_bvar_global' :
                $this ->storePreferencesSysVar($preferences, $storage);
                break;
        }
    }


    /**
     * @param Preferences $preferences
     */
    public function storePreferences($preferences)
    {
        // ulozi polozky, ktere patri do "hlavni" storage
        $preferences ->loadPost();
        $this ->storePreferencesToStorage($preferences, $preferences ->storage);

        // jeste ulozi polozky, ktere maji jinou storage
        foreach ($preferences ->items as $preferenceItem) {
            if ($preferenceItem ->storage  &&  $preferenceItem ->storage != $preferences ->storage) {
                $this ->storePreferencesToStorage($preferences, $preferenceItem ->storage);
            }
        }

        RonPortalSession::initSessionTimestamp(); // aby se napriklad nacetlo nove CSS
    }


    public function handleAction($action)
    {
        if (Prava::vratHodnotuPrava('admin') != 3) {
            throw new \Exception('Pro zvolenou akci nemate pravo! Pravo pro administraci musi byt nastaveno na hodnotu "Vse".');
        }

        TextManager::getInstance() ->loadStrings('preferences.ini');
        $preferencesList = $this ->loadPreferences();

        HtmlHeaders::getInstance() ->setTitle(\Ron\ronText('RON_Portal') . ' - ' . \Ron\ronText('nastaveni_portalu'));

        $prefs = null;

        if (isset($_POST['preferencesName'])) {
            $prefs = $this->findPreferences($preferencesList, $_POST['preferencesName']);

            if ($prefs) {
                $this ->storePreferences($prefs);
                RonPortalSettings::getInstance()->loadFromDB($this ->database);
                return new Redirect(null, '?menuid=nastaveniportalu&tab=' . $prefs ->name);
            }
        }

        if (!$prefs  &&  isset($_GET['tab'])) {
            $prefs = $this ->findPreferences($preferencesList, $_GET['tab']);
        }

        if (!$prefs) {
            reset($preferencesList);
            $prefs = $preferencesList[key($preferencesList)];
        }

        return $this ->showPreferences($preferencesList, $prefs);
    }

}
