<?php
/**
 * stanislav.elbl, 21.02.2021
 */

namespace RonPortal\Controls;

use Ron\DateUtils;
use Ron\DB\SqlStatement;
use Ron\StringUtils;
use Ron\MessageManager;
use Ron\TextManager;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\DBUtils;
use Ron\Controls\ComboBox;
use Ron\Controls\Option;
use RonPortal\PracovniPomer;
use RonPortal\PredavaciProcesyUtils;
use RonPortal\Prava;
use RonPortal\PravaNaOsoby;
use RonPortal\RonPortalSession;
use RonPortal\RonPortalSettings;
use RonPortal\User;

/**
 * Pomocna trida - urcuje filtrovani seznamu osob podle rozdeleni
 */
class ComboBoxZamestnanecFiltrRozdeleni
{
    /** @var int    Kod typu rozdeleni */
    public $typRozdeleni;
    /** @var string Kod rozdeleni */
    public $kodRozdeleni;
    /** @var int    Jestli se maji nacitat i podrizena rozdeleni */
    public $vcetnePodrizenych;

    /**
     * @param int $typRozdeleni
     * @param string $kodRozdeleni
     * @param int $vcetnePodrizenych
     */
    public function __construct(int $typRozdeleni, string $kodRozdeleni, int $vcetnePodrizenych)
    {
        $this->typRozdeleni = $typRozdeleni;
        $this->kodRozdeleni = $kodRozdeleni;
        $this->vcetnePodrizenych = $vcetnePodrizenych;
    }
}


/**
 * Class ComboBoxZamestnanec
 */
class ComboBoxZamestnanec extends ComboBox
{
    /** @var string     Umoznuje zmenit vychozi nastaveni zobrazenych sloupcu */
    public $ronSelectColumns = null;
    /** @var string     Umoznuje zmenit vychozi nastaveni zobrazenych sloupcu (titulky) */
    public $ronSelectCaptions = null;

    // udaje aktualne zvolene osoby - nastavi se pri nacitani dat
    public $idPP;
    public $rc;
    public $osobniCislo;
    public $datumNastupu;
    public $datumUkonceni;
    public $jmenoPrijmeni;
    public $idFotografie;

    /** @var bool       Priznak, jestli nactena data obsahuji vlastni PP uzivatele */
    public $obsahujeVlastniPP;
    /** @var string     Id vlastniho PP (pokud jej obsahuje) */
    public $idVlastnihoPP;

    public $filtrTypRozdeleni;
    public $filtrRozdeleni;
    public $filtrVcetnePodrizenych;

    public $filtrTypRozdeleni2;
    public $filtrRozdeleni2;
    public $filtrVcetnePodrizenych2;

    /** @var ComboBoxZamestnanecFiltrRozdeleni[]    seznam rozdeleni, podle kterych se maji odfiltrovat nactene pracovni pomery */
    protected $filtryRozdeleni = [];

    public $vcetneExternich = false;

    /** @var string[]   Radky s preddefinovanymi zaznamy - jako emptyRow v comboboxu, ale nacitaji se do options  */
    public $defaultRows = null;

    /** @var string     obecny sql filtr */
    public $filtr;

    public $nactenoPodlePrav = '';


    function __construct($varName = 'osoba')
    {
        parent::__construct($varName);

        $this ->ronSelectColumns = 'jmeno;osobni-cislo;datum-nastupu';
        $this ->ronSelectCaptions = TextManager::_('Jmeno') . ';' . TextManager::_('Osobni_cislo') . ';' . TextManager::_('Datum_nastupu');

        if (RonPortalSettings::getValue(RonPortalSettings::VYBER_OSOB_POLE1)) {
            $this ->ronSelectColumns = 'pole1;' . $this ->ronSelectColumns;
            $this ->ronSelectCaptions = ';' . $this ->ronSelectCaptions;
        }

        if (RonPortalSettings::getValue(RonPortalSettings::VYBER_OSOB_POLE2)) {
            $this ->ronSelectColumns .= ';pole2';
            $this ->ronSelectCaptions .= ';';
        }

        $this ->className = 'form-control ronselect';

        $this ->noHtmlConvert = true; // htmlTextConvert popisu zajistuje primo tato trida
    }


    /**
     * Prida rozdeleni do seznamu filtru
     * @param ComboBoxZamestnanecFiltrRozdeleni|int $typRozdeleni   Bud primo objekt s hodnotami pro filtr, nebo kod typu rozdeleni (v tom pripade musi byt zadane i dalsi hodnoty)
     * @param string $kodRozdeleni
     * @param int $vcetnePodrizenych
     * @return void
     */
    public function addFiltrRozdeleni(int $typRozdeleni, $kodRozdeleni = null, $vcetnePodrizenych = null)
    {
        $filtr = ($typRozdeleni instanceof ComboBoxZamestnanecFiltrRozdeleni)
            ? $typRozdeleni
            : new ComboBoxZamestnanecFiltrRozdeleni($typRozdeleni, $kodRozdeleni, $vcetnePodrizenych ?? 0);

        $this ->filtryRozdeleni[] = $filtr;
    }


    /**
     * @param \string[] $defaultRows
     * @return $this
     */
    public function setDefaultRows($defaultRows)
    {
        $this->defaultRows = $defaultRows;
        return $this;
    }


    function renderHtmlParams($params)
    {
        parent::renderHtmlParams($params);

        if (!isset($this ->htmlParams['data-columns'])  &&  $this ->ronSelectColumns) {
            echo " data-columns='{$this ->ronSelectColumns}'";
        }

        if (!isset($this ->htmlParams['data-captions'])  &&  $this ->ronSelectCaptions) {
            echo " data-captions='{$this ->ronSelectCaptions}'";
        }

        if ($this ->nactenoPodlePrav) {
            echo " data-prava='{$this ->nactenoPodlePrav}'";
        }
    }


    function nastavFiltrRozdeleni($typ, $kod, $vcetnePodrizenych)
    {
        $this->filtrTypRozdeleni = $typ;
        $this->filtrRozdeleni = (isset($kod) &&  $kod !== '') ? $kod : null;
        $this->filtrVcetnePodrizenych = $vcetnePodrizenych;
    }


    function nastavFiltrRozdeleni2($typ, $kod, $vcetnePodrizenych)
    {
        $this->filtrTypRozdeleni2 = $typ;
        $this->filtrRozdeleni2 = (isset($kod) &&  $kod !== '') ? $kod : null;;
        $this->filtrVcetnePodrizenych2 = $vcetnePodrizenych;
    }



    /**
     * @param Database $database
     * @param array $row
     * @param bool  $rozlisitPP     Urcuje, jestli se maji rozlisit jednotlive PP
     * @param bool  $nastavit       Pokud je true, tak se udaje osoby nastavi do vlastnosti idPP, RC atd.
     * @return void                 Prida zaznam pro osobu ze zadaneho DB radku
     */
    protected function pridejOsobu($database, $row, $rozlisitPP, $nastavit)
    {
        $kod = $row[7]; // id pracovniho pomeru

        // jmeno osoby a pokud se ma rozlisovat PP tak i Os.c. a nastup
        $jmeno = $row[1] . ' ' . $row[2]; // prijmeni a jmeno
        if ($row[3]) {
            // prida titul
            $jmeno = $jmeno . ', ' . $row[3];
        }

        $popis = $jmeno;
        if ($rozlisitPP) {
            $popis = $popis . ' - ' .
                TextManager::_('Osobni_cislo') . ' ' . $row[5] . ', ' .
                TextManager::_('Datum_nastupu') . ' ' . DBUtils::getFormatedValue($database, 'date', $row[6]);
        }

        $popis = StringUtils::htmlTextConvert($popis);

        // pripadne jeste uzivatelsky vybrane pole - nepouzivat htmlTextConvert, muze obsahovat HTML znacky
        $pomval = 0;
        if (RonPortalSettings::getValue(RonPortalSettings::VYBER_OSOB_POLE1)) {
            $popis = $row[9] . ' ' . $popis;
            $pomval = 1;
        }

        if (RonPortalSettings::getValue(RonPortalSettings::VYBER_OSOB_POLE2)) {
            $popis .= ' ' . $row[9 + $pomval];
        }

        $this->data[] = $option = new Option($kod, $popis);
        $option ->params = [
            'data-jmeno' => $jmeno,
            'data-osobni-cislo' => $row[5],
            'data-datum-nastupu' => DBUtils::getFormatedValue($database, 'date', $row[6])
        ];

        if (RonPortalSettings::getValue(RonPortalSettings::VYBER_OSOB_POLE1)) {
            $option ->params['data-pole1'] = $row[9];
        }

        if (RonPortalSettings::getValue(RonPortalSettings::VYBER_OSOB_POLE2)) {
            $option ->params['data-pole2'] = $row[9 + $pomval];
        }


        if (!$this ->obsahujeVlastniPP  &&  $row[0] == User::getRC()) {
            $this ->obsahujeVlastniPP = true;
            $this ->idVlastnihoPP = $kod;
            $option ->cssClass = 'vlastni-pp';
        }

        if ($nastavit) {
            $this->selectedOption = $option;

            $this->idPP = $row[7];

            $this->rc = $row[0];
            $this->idFotografie = $row[4];
            $this->osobniCislo = $row[5];
            $this->datumNastupu = $database ->stringToTimestamp($row[6]);
            $this->datumUkonceni = $row[8] ? $database ->stringToTimestamp($row[8]) : null;

            $this->jmenoPrijmeni = $popis;

            //$this -> f_DalsiPole = ...;
        }
    }


    /**
     *  Nastavi v session promenne osoby, ktera je vybrana
     */
    function nastavOsobuDoSession()
    {
        RonPortalSession::setVybranaOsoba($this->idPP, $this->rc, $this->osobniCislo, $this->datumNastupu);
    }


    /**
     * Vraci hodnotu, ktera se ma pouzit pro zobrazeni
     * @param Database $database
     * @param string|int $defaultValue
     * @return string|int
     */
    function getPostValue($database, $defaultValue)
    {
        if (isset($_POST[$this->varName])) {
            return $_POST[$this->varName];
        }

        if (isset($_GET[$this->varName])) {
            return $_GET[$this->varName];
        }

        return $defaultValue;
    }


    /**
     * Pomocna funkce - vraci true, pokud je aktualni zaznam ($row) vybrany.
     * @param array  $row
     * @param string $selectedValue
     * @return bool
     */
    protected function compareSelectedValue($row, $selectedValue)
    {
        return $selectedValue == $row[7]; // zaznam ma stejne IdPP jako vybrana osoba
    }


    /**
     * Nacte seznam osob ze zadaneho dotazu
     * dale nacte promenne pro vybrany zaznam - tj. zaznam z $_POST nebo $selectedValue (prednost ma hodnota v $_POST)
     * @param Database $database
     * @param string|SqlStatement   $sqlText
     * @param PravaNaOsoby|PravaNaOsoby[] $prava
     * @param bool     $emptyRow
     * @param int      $selectedValue
     * @param int      $datumOd
     * @param int      $datumDo
     * @return bool
     * @throws DBException
     */
    function loadSQLPP($database, $sqlText, $prava, $emptyRow, $selectedValue, $datumOd, $datumDo)
    {
        $this->data = [];

        $selectedValue = $this->getPostValue($database, $selectedValue);
        if ($prava  &&  !Prava::jsouNejakePrava($prava)) {
            // uzivatel nema zadna prava
            return false;
        }

        if ($emptyRow) {
            // v seznamu ma byt i prazdny zaznam
            $emptyOption = new Option('', $emptyRow === true ? '' : $emptyRow);
            $emptyOption ->setParams([
                'data-jmeno' => $emptyOption ->caption
            ]);
            $this ->data[] = $emptyOption;

            if ($selectedValue === '') {
                $this ->selectedOption = $emptyOption;
            }

            $prvniOsoba = false;
        } else {
            $prvniOsoba = true;
        }

        if ($this ->defaultRows) {
            foreach ($this ->defaultRows as $key => $value) {
                $defOption = new Option($key, $value);
                $this ->data[] = $defOption;

                if ($selectedValue === $key) {
                    $this ->selectedOption = $defOption;
                }

                $prvniOsoba = false;
            }
        }

        $cursor = $database ->getSqlResult($sqlText); // , null, 'Spatny SQL vyraz pro vyber osob!'

        $rozlisovat = false;
        $jeOsobaVybrana = ($emptyRow || $this ->defaultRows)  && $this ->selectedOption;
        // priznak, jestli uz je vybrana osoba (nestaci kontrolovat, jestli uz jsou nastavene promenne,
        //  protoze ty jsou nastavene hned pri prvni osobe, ale prednost ma "muj" vlastni zaznam)

        $row = $cursor ->fetch();

        while ($row) {
            $newRow = $cursor ->fetch(); // nacte i dalsi radek - aby mohl zjistit, jestli ma rozlisovat PP

            $zobrazitOsobu = true;
            if ($prava) {
                if (!Prava::lzeZobrazitPP($prava, $row[7], $datumOd, $datumDo)) {
                    $zobrazitOsobu = false;
                }
            }

            if ($zobrazitOsobu) {
                if ($newRow  &&  $row[0] === $newRow[0]) {
                    // pokud je nasledujici osoba stejna, musim rozlisovat PP
                    $rozlisovat = true;
                }

                $nastavit = false;

                if ($prvniOsoba || $this ->compareSelectedValue($row, $selectedValue) || (!$jeOsobaVybrana && $row[0] == User::getRC())) {
                    $nastavit = true;
                    $prvniOsoba = false;

                    if ($this ->compareSelectedValue($row, $selectedValue) || $row[0] == User::getRC()) {
                        $jeOsobaVybrana = true;
                    }
                }

                $this->pridejOsobu($database, $row, $rozlisovat, $nastavit);
            }

            if ($newRow  &&  $row[0] !== $newRow[0]) {
                // pokud je nasledujici osoba jina, nemusim u ni rozlisovat PP
                $rozlisovat = false;
            }

            $row = $newRow;
        }

        $cursor ->close();

        return true;
    }


    /**
     * @param Database $database
     * @param int $datumOd
     * @param int $datumDo
     * @param int $typRozdeleni
     * @param string $rozdeleni
     * @param int $vcetnePodrizenych    0=jen zadane, 1=vcetne podrizenych, 2=jen podrizene, 3=primo podrizene
     * @return string
     */
    function vratFiltrRozdeleni($database, $datumOd, $datumDo, $typRozdeleni, $rozdeleni, $vcetnePodrizenych)
    {
        if (!$typRozdeleni) {
            return '';
        }

        if (!isset($rozdeleni)) {
            return '';
        }

        if (!$datumOd) {
            $datumOd = time();
        }

        if (!$datumDo) {
            $datumDo = $datumOd;
        }

        if ($vcetnePodrizenych == 1  ||  $vcetnePodrizenych == 2) { // vcetne podrizenych, jen podrizene
            return
                'EXISTS (' .
                '  SELECT * FROM RozdeleniOsob, HierarchieRozdeleni ' .
                '  WHERE HierarchieRozdeleni.TypNadrazene = ' . (int)$typRozdeleni .
                "    AND HierarchieRozdeleni.KodNadrazene = '" . $database ->addSlashes($rozdeleni) . "'" .

                '    AND RozdeleniOsob.KodTypuRozdeleni = HierarchieRozdeleni.typPodrizene' .
                '    AND RozdeleniOsob.Rozdeleni = HierarchieRozdeleni.kodPodrizene' .

                '    AND RozdeleniOsob.RC = PracovniPomer.RC ' .
                '    AND RozdeleniOsob.OsobniCislo = PracovniPomer.OsobniCislo ' .
                '    AND RozdeleniOsob.DatumNastupu = PracovniPomer.DatumNastupu ' .
                '    AND RozdeleniOsob.PlatnostOd <= ' . $database ->dateToString($datumDo) .
                '    AND (RozdeleniOsob.PlatnostDo >= ' . $database ->dateToString($datumOd) . '  OR  RozdeleniOsob.PlatnostDo IS NULL)' .
                (($vcetnePodrizenych == 2) ? 'AND HierarchieRozdeleni.KodNadrazene <> HierarchieRozdeleni.KodPodrizene' : '') .
                ')';
        } else if ($vcetnePodrizenych == 3) { // jen primo podrizene
            return
                'EXISTS (' .
                '  SELECT * FROM RozdeleniOsob, Rozdeleni ' .
                '  WHERE Rozdeleni.KodTypuRozdeleni = ' . (int)$typRozdeleni .
                "    AND Rozdeleni.Rodic = '" . $database ->addSlashes($rozdeleni) . "'" .
                '    AND RozdeleniOsob.KodTypuRozdeleni = Rozdeleni.KodTypuRozdeleni' .
                '    AND RozdeleniOsob.Rozdeleni = Rozdeleni.Kod' .
                '    AND RozdeleniOsob.RC = PracovniPomer.RC ' .
                '    AND RozdeleniOsob.OsobniCislo = PracovniPomer.OsobniCislo ' .
                '    AND RozdeleniOsob.DatumNastupu = PracovniPomer.DatumNastupu ' .
                '    AND RozdeleniOsob.PlatnostOd <= ' . $database ->dateToString($datumDo) .
                '    AND (RozdeleniOsob.PlatnostDo >= ' . $database ->dateToString($datumOd) . '  OR  RozdeleniOsob.PlatnostDo IS NULL)' .
                ')';
        } else {
            return
                'EXISTS (' .
                '  SELECT * FROM RozdeleniOsob ' .
                '  WHERE RozdeleniOsob.KodTypuRozdeleni = ' . (int)$typRozdeleni .
                "    AND RozdeleniOsob.Rozdeleni = '" . $database ->addSlashes($rozdeleni) . "'" .
                '    AND RozdeleniOsob.RC = PracovniPomer.RC ' .
                '    AND RozdeleniOsob.OsobniCislo = PracovniPomer.OsobniCislo ' .
                '    AND RozdeleniOsob.DatumNastupu = PracovniPomer.DatumNastupu ' .
                '    AND RozdeleniOsob.PlatnostOd <= ' . $database ->dateToString($datumDo) .
                '    AND (RozdeleniOsob.PlatnostDo >= ' . $database ->dateToString($datumOd) . '  OR  RozdeleniOsob.PlatnostDo IS NULL)' .
                ')';
        }
    }


    /**
     * @param Database $database
     * @param int  $datumOd
     * @param int  $datumDo
     * @return string
     */
    private function getBasicSQL($database, $datumOd, $datumDo)
    {
        // POZOR! V pripade zmeny poradi muze byt potreba upravit ORDER BY!
        $sqlText =
            'SELECT Osoba.RC, Osoba.Prijmeni, Osoba.Jmeno, Osoba.Titul, Osoba.IdFotografie, ' . // 0 - 4
            ' PracovniPomer.OsobniCislo, PracovniPomer.DatumNastupu, PracovniPomer.ID, ' . // 5 - 7
            ' PracovniPomer.DatumUkonceni '; // 8

        // prida uzivatelske pole z nastaveni intrawebu
        if (RonPortalSettings::getValue(RonPortalSettings::VYBER_OSOB_POLE1)) {
            $sqlText .= ', ' . RonPortalSettings::getValue(RonPortalSettings::VYBER_OSOB_POLE1); // (9)
        }

        if (RonPortalSettings::getValue(RonPortalSettings::VYBER_OSOB_POLE2)) {
            $sqlText .= ', ' . RonPortalSettings::getValue(RonPortalSettings::VYBER_OSOB_POLE2); // (9 nebo 10)
        }

        $sqlText .=
            "\nFROM Osoba, PracovniPomer\n" .
            ' WHERE Osoba.RC = PracovniPomer.RC ';

        if (!$this->vcetneExternich) {
            $sqlText .= ' AND (PracovniPomer.Zamestnanec = 1  OR  PracovniPomer.Zamestnanec = 2)';
        }

        $sqlText = $sqlText .
            ' AND PracovniPomer.DatumNastupu <= ' . $database ->dateToString($datumDo) .
            ' AND (PracovniPomer.DatumUkonceni >= ' . $database ->dateToString($datumOd) . ' OR PracovniPomer.DatumUkonceni IS NULL)';

        $filtr = $this->vratFiltrRozdeleni($database, $datumOd, $datumDo,
            (int)$this->filtrTypRozdeleni, $this->filtrRozdeleni, $this->filtrVcetnePodrizenych);

        $filtr2 = $this->vratFiltrRozdeleni($database, $datumOd, $datumDo,
            (int)$this->filtrTypRozdeleni2, $this->filtrRozdeleni2, $this->filtrVcetnePodrizenych2);

        if ($filtr && $filtr2) {
            $filtr = '(' . $filtr . ")\nAND  (" . $filtr2 . ')';
        } else if ($filtr2) {
            $filtr = $filtr2;
        }

        if ($filtr) {

            $sqlText .= "\nAND (" . $filtr . ')';
        }

        foreach ($this ->filtryRozdeleni as $filtrRozdeleni) {
            $filtrRozdeleniStr = $this ->vratFiltrRozdeleni($database, $datumOd, $datumDo, $filtrRozdeleni ->typRozdeleni, $filtrRozdeleni ->kodRozdeleni, $filtrRozdeleni ->vcetnePodrizenych);
            if ($filtrRozdeleniStr) {
                $sqlText .=
                    ' AND (' . $filtrRozdeleniStr . ')';
            }
        }

        if ($this->filtr) {
            $sqlText .= "\nAND (" . $this->filtr . ')';
        }

        return $sqlText;
    }


    /**
     * @param Database $database
     * @param int  $datumOd
     * @param int  $datumDo
     * @param PravaNaOsoby|PravaNaOsoby[] $prava
     * @return string
     */
    function getSQLPodlePrav($database, $datumOd, $datumDo, $prava)
    {
        // TODO: predelat na SqlStatement
        // filtr podle zadanych datumu nebo podle zacatku a konce obdobi
//        if (!$datumDo) {  - odstraneno 01-2023 - je nesmysl, aby to bylo podle session i na strankach, na kterych to nelze zmenit
//            $datumDo = RonPortalSession::konecObdobi();
//        }
//
//        if (!$datumOd) {
//            $datumOd = RonPortalSession::obdobi();
//        }

        //16.10.2015 RON-PR datum nemusi byt nastavene v session, pouzije se aktualni datum
        if (!$datumDo) {
            $datumDo = DateUtils::lastDayOfMonth(time());
        }

        if (!$datumOd) {
            $datumOd = DateUtils::firstDayOfMonth(time());
        }

        $sqlText = $this ->getBasicSQL($database, $datumOd, $datumDo);

        $vynechatVcetneSebe = true;
        $filtrPrava = Prava::sqlFiltrPodlePrav($prava, 'PracovniPomer', $datumOd, $datumDo, $vynechatVcetneSebe); // vynechat "vcetne sebe"

        if ($filtrPrava) {
            $sqlText = $sqlText . ' AND (' . $filtrPrava . ')';
            $this ->nactenoPodlePrav = Prava::getPopisPrav($prava);
        }

        $jeUnion = false;
        if ($vynechatVcetneSebe  && Prava::pravaVcetneSebe($prava)) {
            $sqlText .=
                "\nUNION\n" .
                $this ->getBasicSQL($database, $datumOd, $datumDo) .
                " AND PracovniPomer.RC = '" . $database ->addSlashes(User::getRC()) . "'\n";

            $jeUnion = true;
        }

        if ($jeUnion  &&  $database ->getDbType() == 'interbase') {
            $sqlText .=
                '  ORDER BY 2, 3, 6, 8'; // na
        } else {
            $sqlText .=
                '  ORDER BY Osoba.Prijmeni, Osoba.Jmeno, ' . PracovniPomer::getRazeniPP() . ', PracovniPomer.Id';
        }

        MessageManager::addDebugLog("ComboBoxZamestnanec: $sqlText");
        return $sqlText;
    }


    /**
     * Nacte seznam pracovnich pomeru platnych v zadanem obdobi (kdyz neni obdobi zadano, pouzije aktualni zvolene obdobi)
     * Pokud jsou zadana prava, tak nacita jen osoby, na ktere ma uzivatel prava
     * @param Database $database
     * @param int    $datumOd
     * @param int    $datumDo
     * @param PravaNaOsoby|PravaNaOsoby[] $prava
     * @param string $emptyRow
     * @param mixed  $selectedValue
     * @param bool   $vcetneExternich
     * @return bool
     */
    function loadDataPP($database, $datumOd, $datumDo, $prava, $emptyRow, $selectedValue, $vcetneExternich = false)
    {
        $this->vcetneExternich = $vcetneExternich;
        $sqlText = $this->getSQLPodlePrav($database, $datumOd, $datumDo, $prava);
        return $this->loadSQLPP($database, $sqlText, $prava, $emptyRow, $selectedValue, $datumOd, $datumDo);
    }


    function loadDataPredavaciProces($database, $kodProcesu, $typRozdeleniProces, $emptyRow, $selectedValue)
    {
        $sqlText = PredavaciProcesyUtils::sqlVyberPPProPredavaciProces($database, $kodProcesu, $typRozdeleniProces);
        $this->loadSQLPP($database, $sqlText, null, $emptyRow, $selectedValue, 0, 0); // $prava = null
    }
}
