<?php

namespace RonPortal\Controls;

use Ron\DB\Database;
use Ron\MessageManager;
use Ron\TextManager;
use RonPortal\Prava;
use RonPortal\PravaNaOsoby;
use RonPortal\User;

class FilterFormFilterPP extends FilterFormFilter
{
    const SPEC_RC = 'rc';
    const SPEC_OSOBNI_CISLO = 'osobniCislo';
    const SPEC_DATUM_NASTUPU = 'datumNastupu';
    const SPEC_ID_FOTOGRAFIE = 'idFotografie';

    /** @var ComboBoxZamestnanec */
    protected $comboOsoby;

    protected $vcetneExternich = false;

    /** @var PravaNaOsoby|PravaNaOsoby[] */
    protected $prava;

    /** @var string     Kod predavaciho procesu - pokud je nastaven, tak se nenacitaji data podle prav, ale podle tohoto procesu */
    protected $predavaciProces = null;

    /** @var int|null   Kod typu rozdeleni pro predavaci proces  */
    protected $typRozdeleniProces = null;

    /** @var int|null   Zacatek obdobi, ve kterem kontroluje datum platnosti PP - pokud neni zadano, urci obdobi podle filterformu */
    protected $datumOd = null;

    /** @var int|null   Konec obdobi, ve kterem kontroluje datum platnosti PP - pokud neni zadano, urci obdobi podle filterformu */
    protected $datumDo = null;

    /** @var bool   Urcuje, jestli lze vybrat volbu "vsichni" - pokud ano, tak ji prida na zacatek seznamu */
    protected $povolitVsechnyOsoby = false;

    /** @var string|null    RC vybraneho PP */
    protected $vybranaOsobaRC = null;
    /** @var string|null    Osobni cislo vybraneho PP */
    protected $vybranaOsobaOsobniCislo = null;
    /** @var int|null       Datum nastupu vybraneho PP */
    protected $vybranaOsobaDatumNastupu = null;


    public function __construct(RonPortalFilterForm $filterForm, string $name = 'osoba')
    {
        parent::__construct($name, FilterFormFilter::TYPE_OPTIONS);

        $this ->filterForm = $filterForm;

        $this
            ->setPrava(Prava::vratPrava('prohlizeni'))
            ->setCaption(TextManager::_('Osoba'))
            ->setSessionName('vybranaosoba');

        $this ->defaultCssClass = 'adsfilterforminput_full form-control ronselect';

        $this ->setHtmlAttributes([
            'id' => 'adsfilterform_osoba',
            //'data-input-class' => 'ronselect adsfilterforminput_full',
            'data-max-visible-rows' => '200'
        ]);
    }


    /**
     * @param PravaNaOsoby|PravaNaOsoby[] $prava
     * @return FilterFormFilterPP
     */
    public function setPrava($prava): FilterFormFilterPP
    {
        $this->prava = $prava;
        return $this;
    }


    /**
     * @param string|null $predavaciProces
     * @param int|null $typRozdeleniProces
     * @return FilterFormFilterPP
     */
    public function setPredavaciProces(?string $predavaciProces, ?int $typRozdeleniProces): FilterFormFilterPP
    {
        $this->predavaciProces = $predavaciProces;
        $this->typRozdeleniProces = $typRozdeleniProces;
        return $this;
    }


    /**
     * @param int|null $typRozdeleniProces
     * @return FilterFormFilterPP
     */
    public function setTypRozdeleniProces(?int $typRozdeleniProces): FilterFormFilterPP
    {
        $this->typRozdeleniProces = $typRozdeleniProces;
        return $this;
    }



    /**
     * @param bool $vcetneExternich
     * @return FilterFormFilterPP
     */
    public function setVcetneExternich(bool $vcetneExternich): FilterFormFilterPP
    {
        $this->vcetneExternich = $vcetneExternich;
        return $this;
    }


    /**
     * @return bool
     */
    public function isPovolitVsechnyOsoby(): bool
    {
        return $this->povolitVsechnyOsoby;
    }

    /**
     * @param bool $povolitVsechnyOsoby
     * @return FilterFormFilterPP
     */
    public function setPovolitVsechnyOsoby(bool $povolitVsechnyOsoby): FilterFormFilterPP
    {
        $this->povolitVsechnyOsoby = $povolitVsechnyOsoby;
        return $this;
    }


    /**
     * @return int|null
     */
    public function getDatumOd()
    {
        return $this->datumOd;
    }

    /**
     * Nastavi pocatek obdobi, ve kterem se kontroluje platnost pracovnich pomeru
     * @param int|null $datumOd
     * @return FilterFormFilterPP
     */
    public function setDatumOd($datumOd): FilterFormFilterPP
    {
        $this->datumOd = $datumOd;
        return $this;
    }

    /**
     * @return int|null
     */
    public function getDatumDo()
    {
        return $this->datumDo;
    }

    /**
     * Nastavi konec obdobi, ve kterem se kontroluje platnost pracovnich pomeru
     * @param int|null $datumDo
     * @return FilterFormFilterPP
     */
    public function setDatumDo($datumDo): FilterFormFilterPP
    {
        $this->datumDo = $datumDo;
        return $this;
    }


    public function loadConfigurationItem($key, $value, $valueText)
    {
        switch ($key) {
            case 'prava' :
                $this ->setPrava(Prava::vratPrava($value));
                break;

            case 'vcetneExternich' :
                $this ->setVcetneExternich((int)$value);
                break;

            case 'povolitVsechnyOsoby' :
                $this->setPovolitVsechnyOsoby((int)$value);
                break;

            default :
                parent::loadConfigurationItem($key, $value, $valueText);
        }
    }


    public function getValue(?string $specification = null)
    {
        if ($this ->value  &&  $specification) {
            switch ($specification) {
                case 'rc' :
                    return $this ->comboOsoby ->rc;

                case 'osobniCislo' :
                    return $this ->comboOsoby ->osobniCislo;

                case 'datumNastupu' :
                    return $this ->comboOsoby ->datumNastupu;

                case 'idFotografie' :
                    return $this ->comboOsoby ->idFotografie;
            }
        }

        return parent::getValue($specification);
    }


    private function createComboOsoby()
    {
        $this ->comboOsoby = new ComboBoxZamestnanec($this ->name);
        //$this ->comboOsoby ->htmlParams['class'] = 'ronselect form-control adsfilterforminput_full';
        $this ->comboOsoby ->htmlParams['data-ron-autosubmit'] = $this ->filterForm ->autoSubmit ? '1' : '0';

        if ($this ->htmlAttributes) {
            $this ->comboOsoby ->addHtmlParams($this ->htmlAttributes);
        }

        if ($this ->prava) {
            $str = '';
            if (is_array($this ->prava)) {
                foreach ($this ->prava as $pravo) {
                    $str .= ($str ? ';' : '') . $pravo ->kod;
                }
            } else if ($this ->prava instanceof PravaNaOsoby) {
                $str = $this ->prava ->kod;
            }

            if ($str) {
                $this ->comboOsoby ->addHtmlParam('data-prava', $str);
            }
        }

        if ($this ->sessionName) {
            $this ->comboOsoby ->addHtmlParam('data-session-name', $this ->sessionName);
        }

        $defaultRows = [];
        if ($this ->items) {
            foreach ($this ->items as $item) {
                $defaultRows[$item ->key] = $item ->caption;
            }
        }

        $this ->comboOsoby
            ->setClassName($this ->getCssClass())
            ->setDefaultRows($defaultRows);
            //->setDataTypeComparison(false);
    }


    private function loadOsobyPodlePrav(Database $database)
    {
        if (!Prava::jsouNejakePrava($this->prava)) {
            //RonPortalSession::smazudajezamestnance();

            MessageManager::addErrorMessage( TextManager::_('Nemate_pravo') );
        }

        $datumOd = $this ->datumOd ?? $this ->filterForm ->getOmezitObdobiOd();
        $datumDo = $this ->datumDo ?? $this ->filterForm ->getOmezitObdobiDo() ?? $datumOd;
        // pokud je zaroven zobrazeno rozdeleni, tak osoby profiltruje
        $this ->createComboOsoby();

        foreach ($this ->filterForm ->getRozdeleniFilters() as $rozdeleniFilter) {
            if ($rozdeleniFilter ->getValue() !== null) {
                $this ->comboOsoby->addFiltrRozdeleni(
                    $rozdeleniFilter ->getTypRozdeleni(),
                    $rozdeleniFilter ->getValue(),
                    $rozdeleniFilter ->getValue('vcetnePodrizenych'));
            }
        }

        $this ->comboOsoby->filtr = $this ->optionsSqlFilter; // obecny filtr, napr. jen osoby, kterym se zpracovava dochazka

        // TODO: session[vsechnyOsoby] ? - asi neni potreba
//        if ($this ->povolitVsechyOsoby) {
//            if (isset($_POST['osoba'])) {
//                $this ->vsechnyOsoby = $_POST['osoba'] == '';
//            } else if (isset($_GET['osoba'])) {
//                $this ->vsechnyOsoby = $_GET['osoba'] == '';
//            } else if ( !RonPortalSession::vybranaOsoba() ) {
//                $this ->vsechnyOsoby = true;
//            }
//
//            $_SESSION['vsechnyOsoby'] = $this ->vsechnyOsoby; // RonPortalSession::setVsechnyOsoby($this ->vsechnyOsoby);
//        }


        $this ->loadDefaultValue($database); // nastavi $this ->value

//        $selectedValue = ($this ->povolitVsechyOsoby  &&  $this ->vsechnyOsoby)
//            ? ''
//            : RonPortalSession::vybranaOsoba();
//
//        if ($_GET['idpp'] ?? null) {
//            $selectedValue = (int)$_GET['idpp']; // pro kompatibilitu se starymi odkazy, ktere muzou obsahovat idpp=... misto osoba=...
//        }

        $this ->comboOsoby->loadDataPP($database,
            $datumOd, $datumDo, $this->prava,
            $this ->povolitVsechnyOsoby ? TextManager::_('_vsichni_'): false,  // $emptyRow
            $this ->value,
            $this->vcetneExternich); // vcetneExternich

//        if ($this ->povolitVsechnyOsoby) {
//            if (!$this ->comboOsoby ->selectedOption  ||  !$this ->comboOsoby ->selectedOption ->key) {
//                // Pokud jsou povoleny vsechny osoby a v selectu je vybrana volba "vsichni", tak nastavi setVsechnyOsoby
//                // Je to potreba v situaci, kdy byla puvodne v session osoba, na kterou nema uzivatel pravo
//                $this->vsechnyOsoby = true;
//                $_SESSION['vsechnyOsoby'] = true; // RonPortalSession::setVsechnyOsoby(true);
//            }
//        }
    }


    public function loadOsobyPodleProcesu(Database $database)
    {
        // TODO: otestovat - ale pouziva se to?
        $this ->createComboOsoby();

        $this ->loadDefaultValue($database); // nastavi $this ->value

        $this ->comboOsoby ->loadDataPredavaciProces($database,
            $this->predavaciProces, $this->typRozdeleniProces,
            false,
            $this ->value);

//        $this ->comboOsoby ->nastavOsobuDoSession();
//        $this ->vybranaOsoba = $this ->comboOsoba ->idPP;
//        $this ->vybranaOsobaRC = $this ->comboOsoba ->rc;
//        $this ->vybranaOsobaOsobniCislo = $this ->comboOsoba ->osobniCislo;
//        $this ->vybranaOsobaDatumNastupu = $this ->comboOsoba ->datumNastupu;

//        if (!$this ->vybranaOsobaRC || !$this ->vybranaOsobaOsobniCislo) {
//            MessageManager::addErrorMessage( TextManager::_('Neni_pracovni_pomer') );
//        }
        if (!$this ->value) {
            MessageManager::addErrorMessage( TextManager::_('Neni_pracovni_pomer') );
        }
    }


    /**
     * Ulozi hodnotu do session a/nebo do uzivatelskeho nastaveni (podle konfigurace)
     * @param Database $database
     * @return void
     * @throws \Exception
     */
    public function storeValue(Database $database)
    {
        if ($this ->value  ||  isset($_GET[$this ->name])) {
            $value = $this->value ?? ($this->povolitVsechnyOsoby && isset($_GET[$this->name]) ? '' : null);
            $this->storeValueByName($database, $value);
        }
    }


    /**
     * @param Database $database
     * @throws \Exception
     */
    public function load(Database $database)
    {
        if ($this ->predavaciProces) {
            $this ->loadOsobyPodleProcesu($database);
        } else {
            $this ->loadOsobyPodlePrav($database);
        }

        $this ->value = $this ->comboOsoby ->selectedOption ->key ?? null;
        if ($this ->value === '') {
            $this ->value = null;
         }

        $this ->storeValue($database);

        if ($this ->comboOsoby ->idPP) {
            // $this ->comboOsoba->nastavOsobuDoSession();  - nee
            $this ->vybranaOsobaRC = $this ->comboOsoby ->rc;
            $this ->vybranaOsobaOsobniCislo = $this ->comboOsoby ->osobniCislo;
            $this ->vybranaOsobaDatumNastupu = $this ->comboOsoby ->datumNastupu;
        } else {
            $this ->vybranaOsobaRC = null;
            $this ->vybranaOsobaOsobniCislo = null;
            $this ->vybranaOsobaDatumNastupu = null;
        }

        if (!$this ->povolitVsechnyOsoby && !$this ->value) {
            MessageManager::addErrorMessage( TextManager::_('Neni_pracovni_pomer') );
        }


//        $this ->comboOsoby = new ComboBoxZamestnanec($this ->name);
//        $this ->comboOsoby
//            ->setClassName($this ->getCssClass())
//            ->addHtmlParam('data-ron-autosubmit', '1')
//            ->setDefaultRows($defaultRows)
//            ->setDataTypeComparison(false);
//
//        $this ->loadDefaultValue($database);
//        $value = $this ->comboOsoby ->getPostValue($database, $this ->value);
//
//        $this ->comboOsoby ->filtr = $this->optionsSqlFilter;
//        $this ->comboOsoby ->defaultRows = $defaultRows;
//
//        $this ->comboOsoby->loadDataPP($database,
//            $this ->filterForm ->osobyDatumOd, $this ->filterForm ->osobyDatumDo, $this->prava,
//            null,
//            $value,
//            $this ->vcetneExternich);
//
//        $this ->value = $this ->comboOsoby ->selectedOption ->key ?? null;
//        if ($this ->value === '') {
//            $this ->value = null;
//        }
//
//        $this ->storeValue($database);
    }


    public function render()
    {
        $this ->comboOsoby ->render();

        if ($htmlId = $this ->comboOsoby ->htmlParams['id']) {
            if ($this->prava) {
                if ($this->comboOsoby->data && count($this->comboOsoby->data) > 1) {
                    // tlacitka pro posun na dalsi a predchozi PP
                    echo '<span id="adsfilterform_pp_fast" class="desktop-only">';
                    echo '<i class="inline-glyph glyph-button fa glyph-left comboBoxMove" data-step="-1" data-submit="1" data-selector="#' . $htmlId . '" data-trigger-change="1"></i>';
                    echo '<i class="inline-glyph glyph-button fa glyph-right comboBoxMove" data-submit="1" data-selector="#' . $htmlId . '" data-trigger-change="1"></i>';
                    echo '</span>';
                }
            }

            if ($this->comboOsoby->obsahujeVlastniPP && $this->comboOsoby->rc != User::getRC()) {
                // zobrazi ikonu pro prechod na vlastni PP
                echo
                    '<i id="vybratVlastniBtn" class="inline-glyph glyph-button fa glyph-own-data comboBoxMove"' .
                    " data-value='{$this ->comboOsoby ->idVlastnihoPP}'" .
                    ' title="' . TextManager::_('Vybrat_vlastni_zaznamy') . '"' .
                    ' data-submit="1" data-selector="#' . $htmlId . '" data-trigger-change="1"></i>';
            }

            if ($this->povolitVsechnyOsoby  &&  $this->value !== null) {
                // zobrazi ikonu pro prechod na vsechny osoby
                echo
                    '<i id="vybratVsechnyOsobyBtn" class="inline-glyph glyph-button fa glyph-all-persons comboBoxMove"' .
                    " data-value=''" .
                    ' title="' . TextManager::_('Vse') . '"' .
                    ' data-submit="1" data-selector="#' . $htmlId . '" data-trigger-change="1"></i>';
            }
        }
    }


    public function getVariables()
    {
        if (!$this ->value) {
            return [];
        }

        $variables = [
            $this ->name => $this ->value,
            $this ->name . '_rc' => $this ->getValue('rc'),
            $this ->name . '_osobniCislo' => $this ->getValue('osobniCislo'),
            $this ->name . '_datumNastupu' => $this ->getValue('datumNastupu')
        ];

        if ($this->name === 'osoba') {
            $variables['idPP'] = $this->value;
            $variables['RC'] = $variables[$this ->name . '_rc'];
            $variables['osobniCislo'] = $variables[$this ->name . '_osobniCislo'];
            $variables['datumNastupu'] = $variables[$this ->name . '_datumNastupu'];
        }

        return $variables;
    }


}
