<?php
/**
 * stanislav.elbl, 05.03.2021
 */

namespace RonPortal\Controls;

use Ron\Controls\FormField;
use Ron\DB\DBRecordField;
use Ron\StringUtils;
use RonPortal\NeniPravoException;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\RonPortalSession;
use RonPortal\User;

/**
 * Class OsobaFormField
 * @package RonPortal\Controls
 * Formularove pole pro vyber osoby
 */
class OsobaFormField extends FormField
{
    public $prava;
    public $datumPlatnostiPP;

    /** @var bool Priznak, jestli se ma zobrazit button na vyber vlastniho PP */
    public $zobrazitButtonVlastni = true;

    public $filtr;

    /** @var PracovniPomer  Pouzije se pro zobrazeni readonly hodnoty. Pokud neni zadan, tak se nacte dodatecne. */
    public $pracovniPomer;

    public $maxVisibleRows = 200;

    /**
     * OsobaFormField constructor.
     * @param string $prava kod prava
     * @param int $datumPlatnostiPP datum ke kteremu se bere platnost prac. pomeru
     * @param PracovniPomer $pracovniPomer      V pripade, ze je zadan, pouzije se pro zobrazeni readonly hodnoty. Nepovinny
     */
    function __construct($prava, $datumPlatnostiPP = null, $pracovniPomer = null)
    {
        parent::__construct();

        $this->fieldType = 'osoba';
        $this->fieldName = 'osoba';
        $this->prava = $prava;
        $this->required = true;

        $this->fieldCssClass = 'form-control ronselect';

        $this->includeToDBRecord = false;

        $this->datumPlatnostiPP = $datumPlatnostiPP ?: time();
        $this->pracovniPomer = $pracovniPomer;
        $this ->defaultValue = '#USER';
    }


    protected function loadConfigurationItem($key, $value)
    {
        switch ($key) {
            case 'maxVisibleRows' :
                $this ->maxVisibleRows = (int)$value;
                break;

            case 'prava' :
                $this ->prava = $value;
                break;

            default :
                parent::loadConfigurationItem($key, $value);
        }
    }


    public function getFieldAttributes($defaultAttrs = null, $rowData = null)
    {
        $result = parent::getFieldAttributes($defaultAttrs, $rowData);

        if (!isset($result['data-max-visible-rows'])  &&  $this ->maxVisibleRows) {
            $result['data-max-visible-rows'] = $this ->maxVisibleRows;
        }

        return $result;
    }


    /**
     * @param \Ron\DB\Database $database
     * @param mixed $value
     * @param bool $canEdit
     * @param bool $insert
     * @param array $rowData
     * @throws \Ron\DB\DBException
     */
    function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {
        if ($canEdit) {
            echo '<div class="no-margin-bottom no-flex-wrap input-group">';

            if ($value === '#USER') {
                $pp = PracovniPomer::najdiPPOsoby($database, User::getRC());
                $value = $pp ->idPP;
            } else if ($value === '#SESSION') {
                $value = RonPortalSession::vybranaOsoba();
            }

            $combo = new ComboBoxZamestnanec($this->fieldName); // jmeno promenne
            $combo->filtr = $this->filtr;
            $combo->htmlParams = $this ->getFieldAttributes();
//            $combo->htmlParams = $this->otherHtmlParams ?: [];
//            if ($this ->fieldId && is_array($combo->htmlParams) ) {
//                $combo->htmlParams['id'] = $this ->fieldId;
//            }
            $combo->loadDataPP($database,
                $this->datumPlatnostiPP, $this->datumPlatnostiPP,
                Prava::vratPrava($this->prava), ' ', $value, false); // $emptyRow = '', $selectedValue, $vcetneExternich = false
            $combo->render();

            $this ->value = ($combo ->selectedOption  &&  $combo ->selectedOption ->key)
                ? (int)$combo ->selectedOption ->key
                : null;

            //$combo->nastavOsobuDoSession(); - nedelat, nema to smysl

            if ($combo ->obsahujeVlastniPP  &&  $this ->fieldId) {
                echo
                    "<span class='input-group-text comboBoxMove' data-value='{$combo ->idVlastnihoPP}' data-selector='#{$this ->fieldId}' data-trigger-change='1'>" .
                    "<i class='inline-glyph glyph-button fa glyph-own-data'></i>" .
                    '</span>';
            }

            echo '</div>';
        } else {
            if ($this ->pracovniPomer || $value) {
                $pp = $this ->pracovniPomer ?: PracovniPomer::nactiPP($database, $value);
                echo
                    '<input type="hidden" name="' . $this->fieldName . '" value="' . ($pp ? $pp ->idPP : '') . '" />' .
                    $pp ->jmenoPrijmeni .
                    "\n";
                //echo StringUtils::htmlTextConvert($pp ->jmenoPrijmeni) . "\n";
//            } else if ($this ->required  &&  !$insert) {
//                // Puvodni kod, ale je nesmyslny, protoze zobrazuje osobu z session. Nepouzivat! Pouze pro kompatibilitu
//                echo '<div class="warning">POZOR! Chyba pri pouziti OsobaFormField - neni zadana hodnota (idpp)!</div>';
//                echo '<input type="hidden" name="' . $this->fieldName . '" value="' . RonPortalSession::vybranaOsoba() . '" />';
//                echo 'OSOBA ID ' . RonPortalSession::vybranaOsoba() . "\n";
            }
        }
    }


    /**
     * Zkontroluje pravo na editaci vybraneho pracovniho pomeru
     * @param PracovniPomer $pp
     * @return void
     * @throws NeniPravoException
     */
    public function zkontrolujPravo($pp)
    {
        if ($this ->prava) {
            if (!Prava::lzeZobrazitPP($this ->prava, $pp ->idPP, $this ->datumPlatnostiPP)) {
                throw new NeniPravoException();
            }
        }
    }

    /**
     * @param $database
     * @return array
     * @throws NeniPravoException
     * @throws \Ron\DB\DBException
     */
    public function createPPFields($database)
    {
        if (isset($_POST[$this ->fieldName])) {
            $this ->pracovniPomer = PracovniPomer::nactiPP($database, (int)$_POST[$this ->fieldName]);
            $this ->zkontrolujPravo($this ->pracovniPomer);
        }

        $fRC = new DBRecordField('RC', '', $this ->readOnly, 15);
        $fRC
            ->setChangeable($this ->changeable)
            ->setUpdateLog($this ->updateLog)
            ->setValue($this ->pracovniPomer ? $this ->pracovniPomer ->rc : null);

        $fOsobniCislo = new DBRecordField('OsobniCislo', '', $this ->readOnly, 15);
        $fOsobniCislo
            ->setChangeable($this ->changeable)
            ->setUpdateLog($this ->updateLog)
            ->setValue($this ->pracovniPomer ? $this ->pracovniPomer ->osobniCislo : null);

        $fDatumNastupu = new DBRecordField('DatumNastupu', 'date', $this ->readOnly);
        $fDatumNastupu
            ->setChangeable($this ->changeable)
            ->setUpdateLog($this ->updateLog)
            ->setValue($this ->pracovniPomer ? $this ->pracovniPomer ->datumNastupu : null);

        $fJmenoPrijmeni = new DBRecordField('', '', true, 0);
        if ($this ->pracovniPomer  &&  $this ->pracovniPomer ->jmenoPrijmeni) {
            $fJmenoPrijmeni ->setValue('   ' . $this ->pracovniPomer ->jmenoPrijmeni);
        }

        return [$fRC, $fOsobniCislo, $fDatumNastupu, $fJmenoPrijmeni];
    }


    public function createDBRecordField($database, $tableName, $insert, $utf8 = false)
    {
        if (!$this ->includeToDBRecord) {
            return null;
        }

        return $this ->createPPFields($database);
    }

}
