<?php
/**
 * stanislav.elbl, 11.06.2021 16:10
 */

namespace RonPortal\Dashboard;


use Ron\DB\Database;
use Ron\MessageManager;
use Ron\MVC\ViewObject;
use Ron\StringUtils;
use RonPortal\Prava;
use RonPortal\RonPortalSettings;

class DashboardView implements ViewObject
{
    /** @var Database */
    public $database;

    /** @var DashboardFrames */
    public $frames;


    /**
     * DashboardView constructor.
     * @param Database $database
     * @param DashboardFrames $frames
     */
    public function __construct(Database $database, $frames)
    {
        $this->database = $database;
        $this ->frames = $frames;
    }


    public function render()
    {
        MessageManager::getInstance() ->showMessages();

        echo '<div class="dashboard" id="dashboard">';

        $hiddenFrameExists = false;

        foreach ($this->frames ->frames as $frame) {
            if (!Prava::existujePravo('dashboard_' . $frame ->id) || Prava::jsouNejakePrava('dashboard_' . $frame ->id)) {
                if (!$frame->hidden) {
                    $style = '';

                    if ($frame->height > 0) {
                        //$style = 'width:' . (int)$frame ->width . 'px; height:' . (int)$frame ->height . 'px';
                        $style = 'height:' . (int)$frame->height . 'px';
                    }

                    $styleStr = $style ? 'style="' . $style . '"' : '';
                    $cols = ($frame->cols <= 12 && (int)$frame->cols > 0)
                        ? (int)$frame->cols
                        : RonPortalSettings::getIntValue('default_dashboard_width', 6);

                    echo
                        "<div class='dashboard-item col{$cols}' $styleStr data-dashboard-id='{$frame->id}'>" .
                        "<div class='dashboard-frame'>" .
                        '  <div class="dashboard-content">' .
                        '  <div class="dashboard-caption">' .
                        '<span class="dashboardbuttons">' .
                        '<div class="button-loading loading-indicator margin-right fa fa-spin glyph-spinner"></div>' .
                        '<div class="dashboardbutton refresh fa glyph-refresh"></div>' .
                        '<div class="dashboardbutton close fa glyph-close"></div>' .
                        '</span>' .
                        "<span class='caption'>{$frame->caption}</span>" .
                        '  </div>' .
                        '  <div class="dashboard-data" data-data-url="' . StringUtils::htmlTextConvert($frame->url) . '"></div>' .
                        '  </div>' .
                        '</div>' .
                        '</div>';
                } else {
                    $hiddenFrameExists = true;
                }
            }
        }

        echo '</div>';

        if ($hiddenFrameExists) {
            echo
                '<div id="addDashboardFrame" class="js">' . \Ron\ronText('Pridat_panel') . '</div>';

            echo
                '<div id="addFrameDialog" style="display:none" title="' . \Ron\ronText('Pridat_panel') . '">';

            foreach ($this->frames ->frames as $frame) {
                if ($frame->hidden) {
                    if (!Prava::existujePravo('dashboard_' . $frame ->id) || Prava::jsouNejakePrava('dashboard_' . $frame ->id)) {
                        echo
                            '<div>' .
                            "<input type='checkbox' class='addFrameCheckbox' data-dashboard-id='{$frame ->id}'/>" .
                            StringUtils::htmlTextConvert($frame->caption) .
                            '</div>';
                    }
                }
            }

            echo
                '<div class="buttons align-center"><input type="button" class="button" value="' . \Ron\ronText('OK') . '" id="addFrameOK"/></div>';

            echo '</div>';
        }
    }
}

