<?php
/**
 * Formular pro planovani absence
 */
namespace RonPortal\Dochazka;

use Ron\Controls\FormField;
use Ron\Controls\LinksList;
use Ron\DateUtils;
use Ron\DB\SqlStatement;
use Ron\MessageManager;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Controls\OsobaFormField;
use RonPortal\Controls\RonPortalForm;
use RonPortal\NeniPravoException;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\RonPortalScriptEvents;
use RonPortal\RonPortalSession;
use RonPortal\RonPortalSettings;
use RonPortal\RonPortalUtils;
use RonPortal\TypRozdeleni;
use RonPortal\User;
use Tracy\Dumper;


class AbsenceDatumOdField extends FormField
{
    function __construct($sqlFieldIndex)
    {
        parent::__construct(FormField::FIELD_TYPE_DATE);

        $this
            ->setFieldName('DatumOd')
            ->setSqlFieldIndex($sqlFieldIndex)
            ->setCode('field-datumod')
            ->setCaption('Datum_od')
            ->setRequired(true)
            ->setFieldId('absence_datum_od')
            ->setOtherHtmlParams(['data-roncalendar-next' => 'DatumDo']); // pro popup kalendar

        $value = date('j.n.Y');
        if (isset($_GET['datum'])) {
            try {
                $value = date('j.n.Y', DateUtils::urlStringToDate($_GET['datum']));
            } catch (\Exception $e) {
                MessageManager::addErrorMessage($e ->getMessage());
            }
        }

        $this ->setDefaultValue($value);
    }


    function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {
        // datum
        parent::showFieldContent($database, $value, $canEdit, $insert, $rowData);

        echo ' ';

        // cas
        if (isset($_POST['caskontinualniod'])) {
            $valueCas = $_POST['caskontinualniod'];
        } else if ($rowData[6] ?? null) {
            $valueCas = date('H:i', $database ->stringToTimestamp($rowData[6]));
        } else {
            $valueCas = '';
        }

        if (isset($_POST['Delka'])) {
            $delka = $_POST['Delka'];
        } else if (isset($_GET['delka'])) {
            $delka = $_GET['delka'];
        } else if ($rowData) {
            $delka = $rowData[5];
        } else {
            $delka = 0;
        }

        if ($canEdit) {
            // pole pro zadani casu u kontinualni absence
            echo
                '<span id="absence_kontinualni_casod" ' . ($delka == 4 ? '' : ' style="display:none"') . '>' .
                '<input class="app_input inputTime" type="text" name="caskontinualniod" maxlength="5" size="5" value="' . StringUtils::htmlTextConvert($valueCas) . '" />' .
                '</span>';
        } else {
            if ($delka == 4) {
                echo StringUtils::htmlTextConvert($valueCas);
            }
        }
    }
}



class AbsenceDatumDoField extends FormField
{
    function __construct($sqlFieldIndex)
    {
        parent::__construct(FormField::FIELD_TYPE_DATE);

        $this
            ->setFieldName('DatumDo')
            ->setSqlFieldIndex($sqlFieldIndex)
            ->setCode('field-datumdo')
            ->setCaption('<span style="white-space:nowrap">' . TextManager::_('Datum_do') . '</span>', false)
            ->setFieldId('absence_datum_do')
            ->setOtherHtmlParams(['data-roncalendar-prev' => 'DatumOd']) // pro popup kalendar
            ->setRequired(1);

        $value = date('j.n.Y');
        if (isset($_GET['datum'])) {
            try {
                $value = date('j.n.Y', DateUtils::urlStringToDate($_GET['datum']));
            } catch (\Exception $e) {
                // nedela nic - hlaska uz je zobrazena u pole datum od
            }
        }

        $this ->setDefaultValue($value);
    }



    function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {
        // datum
        parent::showFieldContent($database, $value, $canEdit, $insert, $rowData);

        echo ' ';

        // case
        if (isset($_POST['caskontinualnido'])) {
            $valueCas = $_POST['caskontinualnido'];
        } else if ($rowData[8] ?? null) {
            $valueCas = date('H:i', $database ->stringToTimestamp($rowData[8]));
        } else {
            $valueCas = '';
        }

        if (isset($_POST['Delka'])) {
            $delka = (int)$_POST['Delka'];
        } else if (isset($_GET['delka'])) {
            $delka = $_GET['delka'];
        } else if ($rowData) {
            $delka = $rowData[5];
        } else {
            $delka = 0;
        }

        if ($canEdit) {
            // pole pro zadani casu u kontinualni absence
            echo
                '<span id="absence_kontinualni_casdo" '.($delka == 4 ? '' : 'style="display:none"').'>'.
                '<input class="app_input inputTime" type="text" name="caskontinualnido" maxlength="5" size="5" value="' . StringUtils::htmlTextConvert($valueCas) . '" />'.
                '</span>';
        } else {
            if ($delka == 4) {
                echo StringUtils::htmlTextConvert($valueCas);
            }
        }
    }
}



/**
 * Zobrazeni nastavenych mesicnich vysledku (napr. odpracovana doba, zbyvajici dovolena)
 * Class CasovaSlozkaField
 * @package Ron\RonPortal
 */
class CasovaSlozkaField extends FormField
{
    /** @var string */
    var $kod;
    /** @var int */
    var $index;

    /**
     * @param int $index
     * @param string $kod
     * @param string $nadpis
     */
    function __construct($index, $kod, $nadpis)
    {
        parent::__construct(self::FIELD_TYPE_TEXT);

        if (!$nadpis) {
            $pomA = explode(':', $kod);
            $kodCasoveSlozky = $pomA[0];
            $nadpis = "db_typcasoveslozkymzdy_popis_$kodCasoveSlozky";
        }

        $this
            ->setCaption($nadpis)
            ->setCode("field-casovaslozka$index")
            ->setCaptionColumnClass('caption col-6 col-sm-3')
            ->setDataColumnClass('col-6 col-sm-9')
            ->setFieldId("casovaslozka$index")
            ->setText(''); // doplni se pozdeji - az po zobrazeni vyberu osoby

        $this ->kod = $kod;
        $this ->index = $index;
    }


    function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {
        $hodnota = ($rowData  &&  $rowData['field-osoba'])
            ? AbsenceController::vratMesicniVysledekProAbsenci($database, $rowData['field-osoba'], $this ->kod)
            : $rowData['field-osoba'];

        $this ->text = $hodnota;

        parent::showFieldContent($database, $value, $canEdit, $insert, $rowData);
    }
}


class PocetSmenField extends FormField
{
    function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {
        $pocetSmen = null;

        if ($rowData  &&  $rowData['field-osoba']  &&  $rowData['field-datumod']  &&  $rowData['field-datumdo']) {
            $pp = PracovniPomer::nactiPP($database, $rowData['field-osoba']);
            try {
                $datumOd = DateUtils::dateStringToTimestamp($rowData['field-datumod']);
                $datumDo = DateUtils::dateStringToTimestamp($rowData['field-datumdo']);

                $pocetSmen = SmenaProDen::pocetSmenOsoby($database, $pp, $datumOd, $datumDo);
            } catch (\Exception $e) {
                // nedela nic (
            }
        }

        $this ->text = $pocetSmen ?? '';

        parent::showFieldContent($database, $value, $canEdit, $insert, $rowData);
    }
}


class CasOdDoField extends FormField
{
    function __construct()
    {
        parent::__construct();

        //if ( isset($_POST['Delka']) ) $delka = $_POST['Delka'];
        //else $delka = 0; // vychozi hodnota = cela smena

        $this ->caption =
            '<span id="absence_caption_od" style="display:none">' .
            TextManager::_('Cas_absence_od') .
            '</span>';

        $this ->fieldType = 'casoddo';
    }


    function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {
        // pokud je delka absence = 3, tak zobrazi cas od-do, jinak ne
        if ($canEdit && isset($_POST['Delka'])) {
            $delka = (int)$_POST['Delka'];
        } else if ($canEdit && isset($_GET['delka'])) {
            $delka = (int)$_GET['delka'];
        } else if ($rowData) {
            $delka = $rowData[5];
        }  else {
            // vychozi hodnota = cela smena
            $delka = 0;
        }

        if ($canEdit && isset($_POST['casabsenceod'])) {
            $valueOd = $_POST['casabsenceod'];
        } else if ($rowData[6] ?? null) {    // pri insert necha prazdne - nema smysl tam davat nejakou hodnotu
            $valueOd = date('H:i', $database ->stringToTimestamp($rowData[6]));
        } else {
            $valueOd = '';
        }


        if (isset($_POST['casabsencedo'])) {
            $valueDo = $_POST['casabsencedo'];
        } else if ($rowData[8] ?? null) {
            $valueDo = date('H:i', $database ->stringToTimestamp($rowData[8]));
        } else {
            $valueDo = '';
        }

        if ($canEdit) {
            echo
                '<div id="absence_od_do"' . (($delka == 3) ? '' : ' style="display:none"') . '>' .
                '<input class="app_input inputTime" type="text" name="casabsenceod" maxlength="5" size="5" value="' . StringUtils::htmlTextConvert($valueOd) . '" />' .
                '&nbsp;&nbsp;' . TextManager::_('Do') . '&nbsp;&nbsp;' .
                '<input class="app_input inputTime" type="text" name="casabsencedo" maxlength="5" size="5" value="' . StringUtils::htmlTextConvert($valueDo) . '" />' .
                '</div>';
        } else if ($delka == 3) {
            echo
                $valueOd . ' ' .
                TextManager::_('Do') . ' ' . $valueDo;
        }

    }
}


class PrekryvaneAbsenceField extends FormField
{
    /** @var AbsenceFormular */
    private $absenceFormular;


    /**
     * @param AbsenceFormular $absenceFormular
     */
    public function __construct($absenceFormular)
    {
        parent::__construct(FormField::FIELD_TYPE_TEXT);

        $this ->absenceFormular = $absenceFormular;
    }


    protected function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {
        if (!$this ->absenceFormular ->prekryvaneAbsence) {
            return;
        }

        echo '<div class="errortext">';
        echo '<div>' . TextManager::_('Prekryvani_absenci') . '</div>';

        foreach ($this ->absenceFormular ->prekryvaneAbsence as $prekryvana) {
            $link = $prekryvana ->f_PP
                ? '?menuid=absence&action=zobrazabsenci&idpp=' . $prekryvana ->f_PP ->idPP .
                    '&datum=' . DateUtils::dateToUrlString($prekryvana ->datumOd) . '&poradi=' . $prekryvana ->poradi
                : null;


            echo
                '<div class="font-weight-normal padding-bottom padding-left-2">' .
                StringUtils::htmlTextConvert($prekryvana ->getPopisObdobiAbsence()) .
                ' ' .
                ($link ? '<a href="' . $link . '">' : '') .
                    StringUtils::htmlTextConvert($prekryvana ->popisAbsence) .
                ($link ? '</a>' : '') .
                '</div>';
        }

        echo
            '<input name="zkratit_prekryvane_absence" type="checkbox" value="1"> ' .
            TextManager::_('Presto_naplanovat_a_zkratit_prekryvane');

        echo '</div>';
    }
}


class AbsenceFormular extends RonPortalForm
{
    const SETTINGS_VKLADAT_NESCHVALENE = 'absencevkladatneschvalene';

    private $backAction;
    /** @var int */
    public $idPP;
    /** @var int    Po pripadne editaci se upravi, protoze se pouziva pro redirect v AbsenceController::form */
    public $datumOd;
    /** @var int    Poradi editovane absence (tj. pole Absence.Poradi z databaze) */
    public $poradi;

    /** @var [] */
    var $delky;


    /**
     * @var Absence[]   Seznam absenci, ktere se prekryvaji s touto absenci a uzivatel ma pravo je zrusit/zkratit
     *                  Nastavi se pri kontrole vlozenych dat
     */
    public $prekryvaneAbsence = null;

    /**
     * @param \Ron\DB\Database $database
     * @param bool $dialogMode
     * @throws \Exception
     * @throws \Ron\RonException
     */
    function __construct($database, $dialogMode = false)
    {
        parent::__construct($database, 'new', $dialogMode);

        $this ->code = 'formabsence';
        $this ->htmlFormId = "absence_formular";

        $this ->idPP = isset($_POST['osoba'])
            ? (int)$_POST['osoba']
            : ( isset($_GET['idpp'])
                ? (int)$_GET['idpp']
                : RonPortalSession::vybranaOsoba()
              );

        if (isset($_POST['Original_DatumOd'])) {
            $this ->datumOd = (int)$_POST['Original_DatumOd'];
        } else {
            if (isset($_GET['datum'])) {
                if (!$this ->isInsert()) {
                    // nova verze, datum ve formatu "j.n.Y"
                    $this->datumOd = DateUtils::dateStringToTimestamp($_GET['datum']);
                }
            } else {
                // puvodni verze s timestampem (vznika problem pri spatnem nastaveni timezone!)
                $this->datumOd = isset($_GET['datumod']) ? (int)$_GET['datumod'] : 0;
            }
        }

        $this ->poradi = isset($_POST['Poradi'])
            ? (int)$_POST['Poradi']
            : (isset($_GET['poradi']) ? (int)$_GET['poradi'] : 0);

        Prava::nastavObdobiProPrava(time(), time()); // prava podle aktualniho casu

        $this ->defaultRowClass = "row";
        $this ->defaultCaptionColClass = "caption col-12 col-sm-3";
        $this ->defaultDataColClass = "col-12 col-sm-9";

        $this ->title = TextManager::_('Planovani_absenci');
        $this ->sqlTableName = 'Absence';
        //$this ->htmlAction = '?menuid=absence';

        $this ->sqlSelectPart =
            'PracovniPomer.Id, Absence.RC, Absence.OsobniCislo, Absence.DatumNastupu, ' . // 0 - 3
            ' Absence.DatumOd, Absence.Delka, Absence.CasOd, Absence.DatumDo, Absence.CasDo, Absence.OperaceDochazky, ' . // 4 - 9
            ' Absence.Schvaleni, Absence.Schvalil, Absence.DatumSchvaleni'; // 10 - 12

        $this ->backAction = null;
        if (isset($_GET['backaction'])) {
            $this ->backAction = $_GET['backaction'];
        } else if (isset($_POST['backaction'])) {
            $this ->backAction = $_POST['backaction'];
        }

        if (isset($this ->backAction)) {
            // odstrani vsechny znaky krome pismen, cislic, podtrzitek
            $this ->backAction = StringUtils::secureString($this ->backAction);
        }

        if ($this ->dialogMode) {
            $this ->stornoAction = '#';
        } else if ($this ->formAction === self::NEW_RECORD  ||  $this ->formAction === self::INSERT_RECORD) {
            if ($this->backAction) {
                $this->stornoAction .= '?menuid=' . $this->backAction;
            } else if (isset($_GET['menuid'])) {
                $this ->stornoAction = '?menuid=' . $_GET['menuid'];
            } else {
                $this ->stornoAction = '?menuid=absence';
            }
        } else {
            $this->stornoAction =
                '?menuid=absence&action=zobrazabsenci&idpp=' . $this->idPP . '&datum=' . DateUtils::dateToUrlString($this->datumOd) . '&poradi=' . $this->poradi;

            if ($this->backAction) {
                $this->stornoAction .= '&backaction=' . $this->backAction;
            }
        }

        $this->delky =
            [
                '0' => TextManager::_('Absence_cela_smena'),
                '1' => TextManager::_('Absence_pulka_smeny1'),
                '2' => TextManager::_('Absence_pulka_smeny2'),
                '3' => TextManager::_('Absence_od_do'),
                '4' => TextManager::_('Absence_kontinualne')
            ];

        // zapamatuje si hodnotu puvodniho primarniho klice - pouzije se pri editaci
        $this ->addField(FormField::FIELD_TYPE_HIDDEN)
            ->setFieldName('Original_DatumOd') ->setSqlFieldIndex(4) ->setDataType('datetime');

        $this ->addField(FormField::FIELD_TYPE_HIDDEN)
            ->setFieldName('Poradi') ->setTableName('Absence') ->setDataType('int') ->setDefaultValue(1);

        $this->povoleneDelkyOperaci();

        if ($this ->backAction) {
            $this ->addField(FormField::FIELD_TYPE_HIDDEN)
                ->setFieldName('backaction') ->setSqlFieldName(null) ->setDefaultValue($this ->backAction);
        }

        $field = new OsobaFormField('absence', 0); // datum platnosti PP = 0, lze editovat = 0
        $field
            ->setSqlFieldIndex(0) // PracovniPomer.Id
            ->setCode('field-osoba') ->setFieldId('absence_osoba')
            ->setDefaultValue($this ->idPP) ->setCaption('Osoba')
            ->setReadOnly(true);
        $field ->filtr = 'PracovniPomer.ZpracovatDochazku = 1';
        $field ->otherHtmlParams = [
            //'class' => 'form-control ronselect',
            'data-columns' => 'text;osobni-cislo',
            'data-captions' => TextManager::_('Jmeno') . ';' . TextManager::_('Osobni_cislo')
        ];
        $this ->addField($field);

        if (RonPortalSettings::getValue(AbsenceController::SETTINGS_CASOVE_SLOZKY)) {
            // doplni casove slozky
            $kody = explode(';', RonPortalSettings::getValue(AbsenceController::SETTINGS_CASOVE_SLOZKY) );
            $nadpisy = explode(';', RonPortalSettings::getValue(AbsenceController::SETTINGS_CASOVE_SLOZKY_NADPISY));
            foreach ($kody as $index => $kod) {
                // zobrazeni vybrane casove slozky pri vyberu osoby (napr. zustatek dovolene)
                $this ->addField(new CasovaSlozkaField($index, $kod, $nadpisy[$index] ?? ''));
            }
        }

        $operace = null;
        if (isset($_GET['operace'])) {
            $operace = urldecode($_GET['operace']);
        }

        $this ->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('OperaceDochazky') ->setSqlFieldIndex(9)
            ->setCode('field-operace')
            ->setCaption('Absence_udalost') ->setDefaultValue($operace) ->setShowEmptyValue(true)
            ->setRequired(true) ->setFieldId('absence_operace')
            ->setSelectSql(
                'SELECT Kod, Popis FROM OperaceDochazky WHERE (%FILTER%) ORDER BY %ORDER%',
                'Absence_Intraweb = 1', 'Popis')
            ->setTranslationPrefix('db_operacedochazky_popis_', 0) // preklad dat z databaze
            ->setSelectFilterFunction(function($row) {
                return self::filterOperace($row);
            });

        $this ->addField(new AbsenceDatumOdField(4));
        $this ->addField(new AbsenceDatumDoField(7));

        $this ->addField(new PocetSmenField(FormField::FIELD_TYPE_TEXT))
            ->setCaption('Pocet_smen')
            ->setFieldId('absence_pocet_smen');

        $delka = '0';
        if (isset($_GET['delka'])) {
            $delka = $_GET['delka'];
        }

        $this ->addField()
            ->setFieldType('combo') ->setOptions($this->delky, true)
            ->setFieldName('Delka') ->setSqlFieldIndex(5)
            ->setCode('field-delka') ->setSelectDataTypeTest(false)
            ->setRequired(true) ->setDefaultValue($delka)
            ->setCaption('Delka_absence') ->setFieldId('absence_delka');

        $this ->addField(new CasOdDoField());

        $this ->addField(new PrekryvaneAbsenceField($this));


        if (RonPortalSettings::getIntValue('typrozdeleniabsence')) {
            $typRozdeleni = TypRozdeleni::getTypRozdeleni($database, RonPortalSettings::getIntValue('typrozdeleniabsence'));

            if ($typRozdeleni) {
                $params = $this ->database ->createParameters();

                $this->addField(FormField::FIELD_TYPE_SELECT)
                    ->setFieldName('Rozdeleni')->setTableName('Absence')
                    ->setCode('field-rozdeleni')
                    ->setCaption(StringUtils::htmlTextConvert($typRozdeleni ->popis))->setShowEmptyValue(true)
                    ->setFieldId('absence_rozdeleni')
                    ->setFieldCssClass('form-control ronselect')
                    ->setOtherHtmlParams('data-columns="text;value"')
                    ->setSelectSql(
                        new SqlStatement(
                            'SELECT Kod, Popis FROM Rozdeleni' .
                            ' WHERE KodTypuRozdeleni = ' . $typRozdeleni ->kod .
                            ' AND DatumVytvoreni <= ' . $params ->addDateParameter(time()) .
                            ' AND (DatumZruseni IS NULL OR DatumZruseni >= ' .$params ->addDateParameter(time()) . ')' .
                            ' AND (%FILTER%)' .
                            ' ORDER BY %ORDER%',
                            $params
                        ),
                        '1 = 1', 'Popis'
                    );
            }
        }

        $prava = Prava::vratPrava('absence_schval');

//        // Doporuceni
//        if (S_GetDoporucovaniAbsenci()) {
//            $field = new DoporuceniField();
//            $field ->f_dbrow_fieldindex = 14;
//            $this ->AddField($field);
//        }

        // Schvalovani
        if (RonPortalSettings::getIntValue(RonPortalSettings::ABSENCE_SCHVALOVANI)) {
            // Muze i schvalovat - zobrazi checkbox nebo odkaz pro schvaleni
//            if ($_GET['action'] == 'zobrazabsenci') {
//                // zobrazeni (bez editace)
//                $this ->AddField( ['type' => 'text', 'text' => '&nbsp;'] );
//                $this ->AddField(new SchvaleniField());
//                $this ->AddField( ['type' => 'text', 'text' => '&nbsp;'] );
//            } else
            if ($this ->formAction == 'save'  ||  $this ->formAction == 'edit') {
                // zobrazit neco ?
            } else {
                // vkladani - zobrazi zaskrtavatko (pokud neni v nastaveni natvrdo vypnute)
                if (!RonPortalSettings::getIntValue(self::SETTINGS_VKLADAT_NESCHVALENE) && $prava ->jsouNejakePrava()) {
                    // pri vkladani musi byt policko viditelne porad, protoze nevime, koho vybere
                    // pokud ale muze planovat jen sebe a zaroven se nesmi schvalovat (typicka situace), tak zaskrtavatko nezobrazi
                    $pravaplan = Prava::vratPrava('absence');
                    if (!$pravaplan ->pouzeSebe() || $prava ->lzeZobrazitRC(User::getRC())) {
                        $this ->addField(FormField::FIELD_TYPE_CHECK)
                            ->setFieldName('Schvaleni') ->setRequired(true) ->setDefaultValue(1)
                            ->setCode('field-schvaleni')
                            ->setCaption('Schvaleno');
                        //$this ->addField( [FORM_FIELD_TYPE => 'check', FORM_FIELD_NAME => 'Schvaleni', FORM_FIELD_CAPTION_TEXT => 'Schvaleno', FORM_FIELD_REQUIRED => 1, FORM_FIELD_DEFAULT_VALUE => 1] );
                    }
                }
            }
        }

        if ($this ->formAction == 'edit' || $this ->formAction == 'save') {
            $this ->addField(FormField::FIELD_TYPE_MEMO)
                ->setFieldName('HistorieSchvalovani') ->setTableName('Absence') ->setReadOnly(true)
                ->setCode('field-historieschvalovani');
        }

        for ($i = 1;  $i <= 5;  $i++) {
            $this ->addField()
                ->setFieldName("Uzivatelska$i") ->setTableName('Absence') ->setCaption("Absence_uzivatelska_$i")
                ->setCode("field-uzivatelska$i") ->setVisible(false);
        }

        $this ->addField(FormField::FIELD_TYPE_MEMO)
            ->setFieldName('Poznamka') ->setTableName('Absence') ->setCaption('Poznamka')
            ->setCode('field-poznamka')
            ->setFieldId('absence_poznamka');

//        $this->addField(new UserLoginField('Insert_UserName', false)) ->setCode('field-insert_username');
//        $this->addField(FORM_FIELD_TYPE_ACTUAL_TIME)
//            ->setFieldName('Insert_Timestamp') ->setCode('field-insert_timestamp') ->setEditable(false);
//
//        $this->addField(new UserLoginField('Update_UserName', true)) ->setCode('field-update_username') ->setUpdateLog(true);
//        $this->addField(FORM_FIELD_TYPE_ACTUAL_TIME)
//            ->setFieldName('Update_Timestamp') ->setCode('field-update_timestamp') ->setEditable(true) ->setUpdateLog(true);

    }


    private static function filterOperace($row)
    {
        if (Prava::existujePravo('absence_' . $row[0])) {
            return Prava::jsouNejakePrava('absence_' . $row[0]);
        }

        return true;
    }


    function povoleneDelkyOperaci()
    {
        $sql =
            'SELECT Kod, Delka 
            FROM OperaceDochazky JOIN PovolenaDelkaAbsence ON (PovolenaDelkaAbsence.OperaceDochazky = OperaceDochazky.Kod) 
            WHERE Absence_Intraweb = 1 ORDER BY Kod';

        $cursor = $this->database->getSqlResult($sql);

        $result = [];

        while ($row = $cursor->fetch()) {
            $result[$row[0]][] = $row[1];
        }

        foreach ($result as $operace => $povoleneDelky) {
            $this->addField(FormField::FIELD_TYPE_HIDDEN)
                ->setFieldName('*Delky_operace_' . $operace) ->setDefaultValue(implode(';', $povoleneDelky));
        }

        $delkyJSON = [];
        $delky = [];

        foreach ($this->delky as $item => $value) {
            $delky[] =
                [
                    'kod' => $item,
                    'popis' => StringUtils::jsonText($value)
                ];
        }

        $delkyJSON['delky'] = $delky;

        $this->addField(FormField::FIELD_TYPE_HIDDEN)
            ->setFieldName('*Delky_popis') ->setDefaultValue(json_encode($delkyJSON));
    }


    public function getCsrfRequestId()
    {
        return
            $this ->code .
            ($this ->isInsert()
                ? ''
                : '_' . $this ->idPP . '-' . date('j.n.Y', $this ->datumOd) . '-' . $this ->poradi
            );
    }


    /**
     * Sestavi dotaz pro nacteni zaznamu z databaze
     * @return string
     */
    function buildSelectSQL()
    {
        $params = $this ->database ->createParameters();

        $sql =
            'SELECT ' . $this ->getSqlSelectPart() .
            "  FROM Absence, PracovniPomer\n" .
            ' WHERE PracovniPomer.RC = Absence.RC ' .
            '   AND PracovniPomer.OsobniCislo = Absence.OsobniCislo ' .
            "   AND PracovniPomer.DatumNastupu = Absence.DatumNastupu \n" .
            '   AND PracovniPomer.Id = ' . $params ->addParameter((int)$this ->idPP) .
            '   AND Absence.DatumOd = ' . $params ->addDateParameter($this ->datumOd) .
            '   AND Absence.Poradi = ' . $params ->addParameter((int)$this ->poradi);

        return new SqlStatement($sql, $params);
    }


    /**
     * @param \ArrayAccess $dbrow
     * @return bool
     */
    function absenceLzeEditovat($dbrow)
    {
        $pravo = Prava::vratPrava('absence', 'absence_' . $dbrow[9]); // operace dochazky

        if (!$pravo ->lzeZobrazitPP($dbrow[0])) {
            // nema pravo na planovani pro danou osobu
            return false;
        }

        if ($dbrow[10] == 4) {
            // zruseno - editace nema smysl
            return false;
        }

        if ($dbrow[10] == 1 || $dbrow[10] == 3) {
            // jiz schvaleno - editovat muze pouze kdyz muze zaroven i schvalovat

            $pravo = Prava::vratPrava('absence_schval', 'absence_schval_' . $dbrow[9]);

            if (!$pravo ->lzeZobrazitPP($dbrow[0])) {
                return false;
            }
        }

        return true;
    }


    function canEditRecord($dbrow)
    {
        if (!$this ->absenceLzeEditovat($dbrow)) {
            return false;
        }

        return parent::canEditRecord($dbrow);
    }


    /**
     * Nacte post hodnoty do AbsenceDBRecord
     * @param bool $edit    Priznak, ze se jedna o editaci existujiciho zaznamu
     * @return AbsenceDBRecord
     * @throws NeniPravoException
     * @throws \Exception
     */
    function createRecordFromPost($edit)
    {
        // zkontroluje, jestli ma vybrana osoba platny PP
        if (!$_POST['osoba']) {
            throw new \Exception( TextManager::_('Nutno_vybrat_osobu') );
        }

        $pp = PracovniPomer::nactiPP($this ->database, (int)$_POST['osoba']);

        $rec = new AbsenceDBRecord($this ->database, $pp, 'absence', null, null, $this);

        // operace
        $operaceDochazky = $_POST['OperaceDochazky'];
        if (!$operaceDochazky) {
            throw new \Exception(TextManager::_('Musite_vybrat_udalost'));
        }
        $rec ->fOperace ->setNewValue($operaceDochazky);
        // TODO: tady by se dalo zkontrolovat jestli se pouziva rezim uprav a puvodni plan uz je schvalen (ale asi by musela byt nastavena uroven)
        $rec ->fOperacePuvodni ->setNewValue($operaceDochazky); // doplneno 05/2018

        Prava::nastavObdobiProPrava(time(), time()); // prava podle aktualniho casu
        // poznamka: proc to bylo zapoznamkovane? nefungovalo pak vkladani!

        // zkontroluje spravnost zadanych datumu
        $rec ->fDatumOd ->setNewValue( DateUtils::dateStringToTimestamp($_POST['DatumOd']) );
        $rec ->fDatumDo ->setNewValue( DateUtils::dateStringToTimestamp($_POST['DatumDo']) );

        if ($rec ->fDatumDo ->value  <  $rec ->fDatumOd ->value) {
            throw new \Exception( TextManager::_('Pocatecni_datum_vetsi_nez_koncove') );
        }

        // ostatni udaje
        Prava::nastavObdobiProPrava(time(), time()); // prava podle aktualniho casu
        // poznamka: proc to bylo zapoznamkovane? nefungovalo pak vkladani!

        $prava = Prava::vratPrava('absence', 'absence_' . $operaceDochazky);
        if (!$prava ->lzeZobrazitPP($pp ->idPP)) {
            throw new NeniPravoException();
        }
//        if (!$this ->f_PP ->vyberNactenyPP($database, $prava)) {
//            RonPortalUtils::neniPravoException();
//        }

        if (!isset($_POST['Delka']) || $_POST['Delka'] == '') {
            throw new \Exception(TextManager::_('Neni_zadana_delka_udalosti'));
        }
        $rec ->fDelka ->setNewValue((int)$_POST['Delka']);

        if ($rec ->fDelka ->value == 3) {
            // cas od - do
            if (!$_POST['casabsenceod'] || !$_POST['casabsencedo']) {
                throw new \Exception(TextManager::_('Musite_zadat_cas_udalosti'));
            }

            //$basedate = $rec ->fDatumOd ->value;
            //$basedate = mktime(0, 0, 0, 12, 30, 1899); - tohle nefunguje
//            $rec ->fCasOd ->setNewValue( DateUtils::timeStringToTimestamp($_POST['casabsenceod'], $basedate) ); // ! POZOR ! Cas obsahuje i datum! (musi to tak byt kvuli letnimu/zimnimu casu)
//            $rec ->fCasDo ->setNewValue( DateUtils::timeStringToTimestamp($_POST['casabsencedo'], $basedate) );
            $rec ->fCasOd ->setNewValue( DateUtils::timeStringToTimestamp($_POST['casabsenceod']) );
            $rec ->fCasDo ->setNewValue( DateUtils::timeStringToTimestamp($_POST['casabsencedo']) );
        } else if ($rec ->fDelka ->value == 4) {
            // cas od - do (kontinualne)
            if (!$_POST['caskontinualniod'] || !$_POST['caskontinualnido']) {
                throw new \Exception(TextManager::_('Musite_zadat_cas_udalosti'));
            }

            //$basedate = mktime(0, 0, 0, 12, 30, 1899); - tohle nefunguje
//            $rec ->fCasOd ->setNewValue( DateUtils::timeStringToTimestamp($_POST['caskontinualniod'], $rec ->fDatumOd ->value) ); // ! POZOR ! Cas obsahuje i datum !
//            $rec ->fCasDo ->setNewValue( DateUtils::timeStringToTimestamp($_POST['caskontinualnido'], $rec ->fDatumDo ->value) );
            $rec ->fCasOd ->setNewValue( DateUtils::timeStringToTimestamp($_POST['caskontinualniod']) );
            $rec ->fCasDo ->setNewValue( DateUtils::timeStringToTimestamp($_POST['caskontinualnido']) );
        }

        $rec ->fPoznamka ->setNewValue($_POST['Poznamka']);

        if (RonPortalSettings::getIntValue('typrozdeleniabsence')) {
            if ($_POST['Rozdeleni'] != '') {
                $rec->fKodTypuRozdeleni->setNewValue(RonPortalSettings::getIntValue('typrozdeleniabsence'));
                $rec->fRozdeleni->setNewValue($_POST['Rozdeleni']);
            } else {
                $rec->fKodTypuRozdeleni->setNewValue(null);
                $rec->fRozdeleni->setNewValue(null);
            }
        }

        for ($i = 1;  $i <= 5;  $i++) {
            $fieldIndex = $this ->findFieldIndexByCode("field-uzivatelska$i");
            if ($this ->fields[$fieldIndex] ->isVisible()) {
                $recField = $rec ->findFieldByName("Uzivatelska$i");
                $recField ->setNewValue($this ->fields[$fieldIndex] ->getPostValue(false));
            }
        }

        //$rec ->datumOdPuvodni = $_POST['Original_DatumOd']; // pri insertu je 0!
        //$rec ->fPoradi = $_POST['Poradi'];

        return $rec;
    }


//    /**
//     *
//     * @param AbsenceDBRecord $rec
//     * @param bool $editace
//     * @throws \Exception
//     */
//    function zkontrolujData($rec, $editace)
//    {
//        AbsenceDBRecord::zkontrolujPovoleneDelkyOperace($this->database, $rec->fOperace->value, $rec->fDelka->value);
//
//        AbsenceDBRecord::zkontrolujObdobiAbsence($this ->database, $rec->fDatumOd->value, $rec->fDatumDo->value, $rec->pp);
//
//        AbsenceDBRecord::zkontrolujPrekryvaniAbsenci($this ->database, $rec->pp, $rec->fOperace->value,
//            $rec->fDatumOd->value, $rec->fDatumDo->value, $rec->fDelka->value, $rec->fCasOd->value, $rec->fCasDo->value,
//            $editace ? $rec->fDatumOd->originalValue : null, $editace ? $rec->fPoradi->originalValue : null);
//
//        if (RonPortalSettings::getIntValue('zakazatabsencepoukonceni')) {
//            if ($rec->pp->datumUkonceni  &&  $rec->fDatumDo->value > $rec->pp->datumUkonceni) {
//                throw new \Exception( sprintf(TextManager::_('Nelze_vkladat_absence_po_ukonceni_pp'), StringUtils::format('date', $rec->pp ->datumUkonceni)) );
//            }
//        }
//    }




    function editRecord()
    {
        if (!Prava::vratPrava('absence') ->jsouNejakePrava()) {
            throw new NeniPravoException();
        }

        if (!$this ->validate(true)) {
            return false;
        }

        $rec = $this ->createRecordFromPost(true);

        $operace = OperaceDochazky::nactiOperaci($this->database, $rec ->fOperace ->value);
        $rec ->popisOperace = $operace ->popis;

        $params = $this->database->createParameters();
        $filter =
            'DatumOd = ' . $params->addParameter($this->database->dateToParam($_POST['Original_DatumOd'])) .
            ' AND Poradi = ' . $params->addParameter((int)$_POST['Poradi']);

        $rec ->loadRecord($filter, $params); // nacte puvodni hodnoty

        if (!$rec->editujAbsenci($filter, $params)) {
            throw new \Exception(TextManager::_('Chyba_pri_vkladani_udalosti'));
        }

        $this ->datumOd = $rec ->fDatumOd ->value; // aby fungoval redirect po editaci absence
        MessageManager::addInfoMessage( TextManager::_('Vlozeni_udalosti_OK') );

        return true;
    }


    function insertRecord()
    {
        if (!Prava::vratPrava('absence') ->jsouNejakePrava()) {
            throw new NeniPravoException();
        }

        if (!$this->validate(false)) {
            return false;
        }

        $rec = $this ->createRecordFromPost(true);

        $rec ->fHistorie ->setNewValue(
            date('j.n.Y G:i', time()) . ', ' . RonPortalUtils::getLogUserName(false) . ': ' .
            TextManager::_('Log_naplanovani_absence') . ' ' .
            date('j.n.Y', $rec ->fDatumOd ->value) . ' - ' . date('j.n.Y', $rec->fDatumDo->value) . "\n");

        $operace = OperaceDochazky::nactiOperaci($this->database, $rec ->fOperace ->value);
        $rec ->popisOperace = $operace ->popis;

        if ($operace ->autoSchvalovat) {
            $rec->fSchvaleni->setNewValue(1);
        } else {
            $prava = Prava::vratPrava('absence_schval', 'absence_schval_' . $rec ->fOperace ->value);
            if (RonPortalSettings::getIntValue(RonPortalSettings::ABSENCE_SCHVALOVANI)  &&  $rec->pp ->lzeZobrazitPP($prava)) {
                switch (RonPortalSettings::getIntValue(self::SETTINGS_VKLADAT_NESCHVALENE)) {
                    case 1 :
                        $schvaleno = 0;
                        break;

                    case 2 :
                        $schvaleno = 1;
                        break;

                    case 0 :
                    default :
                        $schvaleno = isset($_POST['Schvaleni'])  &&  $_POST['Schvaleni'];
                }

                // muze i schvalovat
                // nastavi schvaleni (f_updatelog = true -> provede pouze v pripade zmeny neceho jineho)
                $rec ->fSchvaleni ->setNewValue($schvaleno);
                $rec ->fSchvalil ->setNewValue($schvaleno ? RonPortalUtils::getLogUserName() : null);
                $rec ->fDatumSchvaleni ->setNewValue($schvaleno ? time() : null);
            } else {
                $rec ->fSchvaleni ->setNewValue(0);
            }
        }

        //$this ->zkontrolujData($rec, false);
        $rec ->zkontrolujDataPredZapisem(); // editace = false

        $this ->prekryvaneAbsence = $rec ->nactiAbsenceKPreruseni(!RonPortalSettings::getIntValue('prekryvaniabsenci'));

        if ($this ->prekryvaneAbsence) {
            if (isset($_POST['zkratit_prekryvane_absence'])  &&  (int)$_POST['zkratit_prekryvane_absence']) {
                $rec ->zkratPrekryvaneAbsence($this ->prekryvaneAbsence);
            } else {
                throw new \Exception(TextManager::_('Prekryvani_absenci'));
            }
        }

        $rec->absenceScript(RonPortalScriptEvents::VLOZENI_ABSENCE_BEFORE);

        $rec ->provedVlozeni();

        $this ->idPP = $rec ->pp ->idPP;
        $this ->datumOd = $rec ->fDatumOd ->value;
        $this ->poradi = $rec ->fPoradi ->value;

        MessageManager::addInfoMessage( TextManager::_('Vlozeni_udalosti_OK') );

        if (!$operace ->autoSchvalovat) {
            // hodnotaschvaleni = 0 (nova absence)
            $rec ->planovani_Email($this ->database, 0);
        }

        $rec ->absenceScript(RonPortalScriptEvents::VLOZENI_ABSENCE_AFTER);

        return true;

    }

    function renderHeader()
    {
        parent::renderHeader();
        if (!$this ->dialogMode) {
            echo '<div id="absence_prepinani">';

            LinksList::renderLinks([
                'planovani' => TextManager::_('Zobrazit_seznam_absenci'),
                'rocniprehled' => TextManager::_('Zobrazit_rocni_prehled')
            ], 'planovani', '?menuid=', '', false);

            echo "</div>\n";

            echo TextManager::_('Planovani_absence_info');
        }
    }
}
