<?php

/**
 * Seznam eNeschopenek
 */

namespace RonPortal\Dochazka;


use Ron\Controls\BrowserField;
use Ron\DB\Database;
use Ron\DB\SqlStatement;
use Ron\TextManager;
use RonPortal\Controls\FilterFormFilter;
use RonPortal\Controls\FilterFormFilterItem;
use RonPortal\Controls\RonPortalBrowser;
use RonPortal\PravaNaOsoby;
use RonPortal\RonPortalUtils;
use RonPortal\RozdeleniUtils;

class EditaceNeschopenky extends BrowserField
{
    function __construct()
    {
        parent::__construct();
        $this ->code = 'col-edit';
        $this ->align = 'left';
    }

    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     * @throws Exception
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        return
            '<i class="inline-glyph glyph-button fa glyph-edit open-dialog"' .
            ' data-dialog="#eneschopenka_dialog" data-ronform-url="?menuid=eneschopenka&action=dialog&record="' .
            ' data-ronform-record="' . $rowData['col-cislodokladu'] . '"' . // id radku
            ' title="' . TextManager::_('Editovat_zaznam') . '">';
    }
}

class ENeschopenkaBrowser extends RonPortalBrowser
{
    /** @var PravaNaOsoby */
    private $prava;
    
    /**
     * ENeschopenkaBrowser constructor.
     * @param Database $database
     * @param PravaNaOsoby $prava
     * @throws \Exception
     */
    function __construct($database, $prava)
    {
        parent::__construct($database, 'eneschopenka');

        $jsouMzdy = RonPortalUtils::jeNainstalovanaAplikace($this->database, 'MZDY');

        $this->prava = $prava;

        $this
            ->setCode('browsereneschopenka')
            ->setTitle('E_neschopenka');

        $this->sqlFromPart =
            'ENeschopenka JOIN Osoba ON ENeschopenka.RC = Osoba.RC' .
            ' JOIN DruhDavkyNZ ON DruhDavkyNZ.Kod = ENeschopenka.DruhDavky' .
            ($jsouMzdy
                ? ' LEFT OUTER JOIN DavkyNZ ON DavkyNZ.CisloDokladu = ENeschopenka.CisloDokladu'
                : ''
            );

        $this ->addField()
            ->setSqlFieldName('Osoba.Prijmeni') ->setHeaderText('Prijmeni')
            ->setOrdering(true) ->setClassName('font-weight-bold')
            ->setCode('col-prijmeni');       // ->setWrap(1)

        $this ->addField()
            ->setSqlFieldName('Osoba.Jmeno') ->setHeaderText('Jmeno')
            ->setOrdering(true) ->setClassName('font-weight-bold')
            ->setCode('col-prijmeni');       // ->setWrap(1)

        $this->addField()
            ->setSqlFieldName('DruhDavkyNZ.Popis') ->setHeaderText('E_neschopenka_druh')
            ->setOrdering(true) ->setAlign('center')
            ->setCode('col-druhdavky');

        $this->addField()
            ->setSqlFieldName('ENeschopenka.DatumPocatkuDavky') ->setHeaderText('E_neschopenka_datum_pocatku')
            ->setOrdering(true) ->setAlign('center') ->setFormat('date')
            ->setCode('col-datumpocatkudavky');

        $this ->defaultSort = 'col-datumpocatkudavky';

        $this->addField()
            ->setSqlFieldName('ENeschopenka.DatumUkonceniDavky') ->setHeaderText('E_neschopenka_datum_ukonceni')
            ->setOrdering(true) ->setAlign('center') ->setFormat('date')
            ->setCode('col-datumukoncenidavky');

        $this->addField()
            ->setSqlFieldName('ENeschopenka.UkoncenaDavka') ->setHeaderText('E_neschopenka_ukoncena_davka')
            ->setOrdering(true) ->setAlign('center') ->setFormat('anone')
            ->setCode('col-ukoncenadavka');

        $this->addField()
            ->setSqlFieldName('ENeschopenka.PricinaNemoci') ->setHeaderText('E_neschopenka_pricina_nemoci') ->setOrdering(true) ->setAlign('center')
            ->setTranslationPrefix('E_neschopenka_pricina_')
            ->setCode('col-pricinanemoci');

        $this->addField()
            ->setSqlFieldName('ENeschopenka.CisloDokladu')->setHeaderText('E_neschopenka_cislo_dokladu')
            ->setOrdering(true)->setAlign('right')->setVisible($jsouMzdy)
            ->setCode('col-cislodokladu');

        //pokud je nainstalovana aplikace MZDY, zobrazi se i o sloupce
        if ($jsouMzdy) {
            $this->addField()
                ->setSqlFieldName('DavkyNZ.DatumVychazkyOd') ->setHeaderText('E_neschopenka_vychazky_od')
                ->setOrdering(true) ->setAlign('center') ->setFormat('date')
                ->setCode('col-vychazkyod');

            $this->addField()
                ->setSqlFieldName('DavkyNZ.DatumVychazkyDo') ->setHeaderText('E_neschopenka_vychazky_do')
                ->setOrdering(true) ->setAlign('center') ->setFormat('date')
                ->setCode('col-vychazkydo');

            $this->addField(BrowserField::FIELD_TYPE_COMMENT)
                ->setSqlFieldName('{fn_CASTWIDEMEMO(DavkyNZ.IntervalVychazek)}') //->setHeader(TextManager::_('E_neschopenka_vychazky_interval') . ': ')
                ->setHeaderColSpan(-1)->setClassName('poznamka browserpoznamka')
                ->setCode('col-vychazkyinterval');

            $this->addField(BrowserField::FIELD_TYPE_COMMENT)
                ->setSqlFieldName('{fn_CASTWIDEMEMO(DavkyNZ.AdresaPobytu)}') ->setHeader('<strong>' . TextManager::_('E_neschopenka_adresa_pobytu') . '</strong>: ')
                ->setHeaderColSpan(-1)->setClassName('poznamka browserpoznamka')
                ->setCode('col-adresapobytu');
        }

        if ($this->prava->priznak) {
            $this->addField(new EditaceNeschopenky());
        }

        $this->addField(BrowserField::FIELD_TYPE_COMMENT)
            ->setSqlFieldName('{fn_CASTWIDEMEMO(ENeschopenka.Poznamka)}') ->setHeader('<strong>' . TextManager::_('Poznamka') . '</strong>: ')
            ->setHeaderColSpan(-1)->setClassName('poznamka browserpoznamka')
            ->setCode('col-poznamka');
    }


    function setFilterForm()
    {
        $this ->filterForm = $this->createFilterForm(
            'ADS', 'eneschopenka_prohl',
            true, true);

        $this ->filterForm ->addFilter((new FilterFormFilter('ukoncene'))
            ->addItem(0, TextManager::_('Vse'))
            ->addItem((new FilterFormFilterItem(1, TextManager::_('E_neschopenka_ukoncene')))
                ->setSqlFilter('ENeschopenka.UkoncenaDavka = 1')
            )
            ->addItem((new FilterFormFilterItem(2, TextManager::_('E_neschopenka_neukoncene')))
                ->setSqlFilter('ENeschopenka.UkoncenaDavka = 0')
            )
            ->setDefaultValue(0)
        );
    }

    function getSQLFilter()
    {
        $params = $this->database->createParameters();
        $sql = $this ->filterForm ->buildSqlFilter($params);

        $sql .=
            ($sql ? ' AND ' : '') .
            'ENeschopenka.DatumPocatkuDavky <= ' . $params->addDateParameter($this->filterForm->getOmezitObdobiDo()) .
            ' AND (ENeschopenka.DatumUkonceniDavky >= ' . $params->addDateParameter($this->filterForm->getOmezitObdobiOd()) .
            ' OR ENeschopenka.DatumUkonceniDavky IS NULL OR ENeschopenka.UkoncenaDavka = 0)';

        if ($this ->filterForm ->getFilterValue('osoba')) {
            $sql .=
                ' AND ENeschopenka.RC = ' . $params->addParameter($this->filterForm->getFilterValue('osoba', 'rc'));
        } else {
            $sql .=
                ' AND EXISTS(SELECT ID FROM PracovniPomer' .
                '  WHERE PracovniPomer.RC = Osoba.RC AND PracovniPomer.DatumNastupu <= ' . $params->addDateParameter($this->filterForm->getOmezitObdobiDo()) .
                '    AND (PracovniPomer.DatumUkonceni >= ' . $params->addDateParameter($this->filterForm->getOmezitObdobiOd()) .
                '    OR PracovniPomer.DatumUkonceni IS NULL' .
                ' )' .
                ' AND ' . $this->prava->sqlFiltr('PracovniPomer', $this->filterForm->getOmezitObdobiOd(), $this->filterForm->getOmezitObdobiDo(), $params);

            foreach ($this ->filterForm ->getRozdeleniFilters() as $rozdeleniFilter) {
                $value = $rozdeleniFilter ->getValue();
                if (isset($value)) {
                    $filtrR = RozdeleniUtils::sqlFilterRozdeleniPP($this->database,
                        'PracovniPomer',
                        $rozdeleniFilter ->getTypRozdeleni(), $rozdeleniFilter ->getValue(),
                        $this->filterForm->getOmezitObdobiOd(), $this->filterForm->getOmezitObdobiDo(),
                        $rozdeleniFilter ->getValue('vcetnePodrizenych'),
                        $params);

                    if ($filtrR) {
                        $sql .=
                            ($sql ? ' AND ' : '') .
                            '(' . $filtrR . ')';
                    }
                }
            }

            $sql .=
                ')';
        }

//        switch ($this->filterForm->ukoncene) {
//            case 0 :
//            default :
//                break;
//            case 1 :
//                $sql .= ' AND ENeschopenka.UkoncenaDavka = 1';
//                break;
//            case 2 :
//                $sql .= ' AND ENeschopenka.UkoncenaDavka = 0';
//                break;
//        }

        return new SqlStatement($sql, $params);
    }

    public function render()
    {
        parent::render();

        $dialog = new ENeschopenkaDialog($this->database, '');

        $dialog->render();
    }

}
