<?php
/**
 * stanislav.elbl, 24.02.2021
 */

namespace RonPortal\Dochazka;

use Ron\DateUtils;
use Ron\DB\SqlStatement;
use Ron\StringUtils;
use RonPortal\Controls\FilterFormFilterObdobi;
use RonPortal\Controls\FilterFormFilterPP;
use RonPortal\Controls\RonPortalBrowser;
use RonPortal\Controls\RonPortalFilterForm;

class MesicniVysledkyRBrowser extends VysledkyBrowser
{
    function __construct($database)
    {
        parent::__construct($database, 'mesicnivysledkyr');

        $this
            ->setCode('browsermesicnivysledky_r')
            ->setTitle('Mesicni_vysledky_podle_rozdeleni')
            ->setAction('mesicnivysledkyr');

        $this->sqlFromPart = 'MesicniVysledek_R, TypCasoveSlozkyMzdy, Rozdeleni';

        $this->sqlWherePart =
            'TypCasoveSlozkyMzdy.Kod = MesicniVysledek_R.TypCasoveSlozkyMzdy ' .
            ' AND Rozdeleni.KodTypuRozdeleni = MesicniVysledek_R.KodTypuRozdeleni ' .
            ' AND Rozdeleni.Kod = MesicniVysledek_R.Rozdeleni ';

        $this->cssClass .= ' mesicnivysledkyr';

        $this->addField()
            ->setSqlFieldName('MesicniVysledek_R.TypCasoveSlozkyMzdy')
            ->setCode('col-typcasoveslozkymzdy')
            ->setHeaderText('Kod_casove_slozky')
            ->setOrdering(true);

        $this->addField()
            ->setSqlFieldName('TypCasoveSlozkyMzdy.Popis')
            ->setCode('col-typcasoveslozky-popis')
            ->setHeaderText('Casova_slozka')
            ->setOrdering(true);

        $this->addField()
            ->setSqlFieldName('MesicniVysledek_R.Hodiny')
            ->setCode('col-hodiny')
            ->setHeaderText('Vysledek_Hodiny')
            ->setFormat('hodiny') ->setAlign('right')
            ->setOrdering(true)
            ->setClassName('hodiny');

        $this->addField()
            ->setSqlFieldName('MesicniVysledek_R.Dny')
            ->setCode('col-dny')
            ->setHeaderText('Vysledek_Pracovni_dny')
            ->setFormat('number_2') ->setAlign('right')
            ->setOrdering(true)
            ->setClassName('dny');

        $this->addField()
            ->setSqlFieldName('MesicniVysledek_R.KalendarniDny')
            ->setCode('col-kalendarnidny')
            ->setHeaderText('Vysledek_Kalendarni_dny')
            ->setFormat('number_2') ->setAlign('right')
            ->setOrdering(true)
            ->setClassName('kalendarnidny');

        $this->addField()
            ->setSqlFieldName('Rozdeleni.Popis')
            ->setCode('col-rozdeleni-popis')
            ->setHeaderText('Rozdeleni')
            ->setOrdering(true);

        $this->addField()
            ->setSqlFieldName('MesicniVysledek_R.Rozdeleni')
            ->setCode('col-rozdeleni')
            ->setOrdering(true);

        $this->defaultSort = 'col-typcasoveslozkymzdy';
        $this->primaryField = 'col-typcasoveslozkymzdy';
    }


    function setFilterForm()
    {
        $this ->filterForm = $this->createFilterForm(
            'ADS', 'prohlizeni',
            false, RonPortalBrowser::OSOBY_JEN_S_DOCHAZKOU);

        $this ->filterForm ->downloadFotoOsobyUrl = '?menuid=editace&action=foto&idpp=';
    }


    function load($sqlFilter = '')
    {
        parent::load($sqlFilter);

        $this ->browserUrl =
            '?menuid=mesicnivysledkyr' .
            '&idpp=' . $this ->filterForm ->getFilterValue('osoba') .
            '&obdobi=' . DateUtils::dateToUrlString($this ->filterForm ->getFilterValue('obdobi'));
    }


    function renderFilter()
    {
        $this->odkazyMesicniVysledky('mesicnivysledkyr');
        parent::renderFilter();
    }


    function getSQLFilter()
    {
        $params = $this ->database ->createParameters();
        $sql = $this ->filterForm ->buildSqlFilter($params);

        return new SqlStatement(
            $sql . ($sql ? ' AND ' : '') .
            'MesicniVysledek_R.Mesic = ' . $params ->addDateParameter($this ->filterForm ->getOmezitObdobiOd()) .
            ' AND MesicniVysledek_R.RC = ' . $params->addParameter($this ->filterForm ->getFilterValue('osoba', 'rc')) .
            ' AND MesicniVysledek_R.OsobniCislo = ' . $params->addParameter($this ->filterForm ->getFilterValue('osoba', 'osobniCislo')) .
            ' AND MesicniVysledek_R.DatumNastupu = ' . $params ->addDateParameter($this ->filterForm ->getFilterValue('osoba', 'datumNastupu')),
            $params);
    }

    function getRowAttributes($values, $className, $rowNumber)
    {
        // prida class mv_<kod_casove_slozky>
        $mvClass = 'mv_' . StringUtils::htmlTextConvert(str_replace(' ', '_', $values[0]));
        $className = ($className ? $className . ' ' : '') . $mvClass;

        return parent::getRowAttributes($values, $className, $rowNumber);
    }

}
