<?php
/**
 * stanislav.elbl, 26.02.2021
 */

namespace RonPortal\Dochazka;

use Ron\DB\Database;
use Ron\DB\SQLUtils;
use Ron\StringUtils;

class PrirazeniKalendare
{
    public $kod;
    public $popis;

    public $idpp;
    public $rc;

    public $platnostOd;
    public $platnostDo;

    /** @var int    Nacte se z Kalendar.DochazkaPodlePlanKapacit */
    public $dochazkaPodlePlanKapacit;
    /** @var int    Nacte se z Kalendar.NabehovaDobaProUrceniSmen */
    public $nabehovaDobaProUrceniSmen;


    /**
     * @param Database $database
     * @param int|int[] $idPP
     * @param $datumOd
     * @param $datumDo
     * @return PrirazeniKalendare[]
     */
    public static function nactiKalendarProObdobi($database, $idPP, $datumOd, $datumDo = null)
    {
        if (!isset($datumDo)) {
            $datumDo = $datumOd;
        }

        $params = $database->createParameters();
        $sql =
            'SELECT PrirazeniKalendare.Kalendar, Kalendar.Popis, Kalendar.DochazkaPodlePlanKapacit, ' .
            '       PrirazeniKalendare.PlatnostOd, PrirazeniKalendare.PlatnostDo, ' . // 3,4
            '       PracovniPomer.Id, PracovniPomer.RC, Kalendar.NabehovaDobaProUrceniSmen' . // 5..7
            '  FROM PrirazeniKalendare, Kalendar, PracovniPomer ' .
            ' WHERE PrirazeniKalendare.Kalendar = Kalendar.Kod ' .
            '   AND PrirazeniKalendare.PlatnostOd <= ' . $params->addDateParameter($datumDo) .
            '   AND (PrirazeniKalendare.PlatnostDo IS NULL  OR  PrirazeniKalendare.PlatnostDo >= ' . $params->addDateParameter($datumOd) . ')' .
            '   AND PrirazeniKalendare.RC = PracovniPomer.RC' .
            '   AND PrirazeniKalendare.OsobniCislo = PracovniPomer.OsobniCislo' .
            '   AND PrirazeniKalendare.DatumNastupu = PracovniPomer.DatumNastupu';

        if ($idPP  &&  is_array($idPP)) {
            // neprazdny seznam idpp
            $sql .=
                ' AND PracovniPomer.Id IN (';

            $first = true;
            foreach ($idPP as $id) {
                $sql .=
                    ($first ? '' : ', ') .
                    $params ->addParameter($id);

                $first = false;
            }
            $sql .= ')';
        } else {
            $sql .=
                '  AND PracovniPomer.Id = ' . $params ->addParameter((int)$idPP);
        }

        $sql .=
            ' ORDER BY PracovniPomer.Id, PrirazeniKalendare.PlatnostOd ';

        $cursor = $database->getSqlResult($sql, $params);

        $result = [];
        while ($row = $cursor->fetch()) {
            $kal = new static();

            $kal ->kod = $row[0];
            $kal ->popis = $row[1];
            $kal ->dochazkaPodlePlanKapacit = (int)$row[2];

            $kal ->platnostOd = $cursor ->stringToTimestamp($row[3]);
            $kal ->platnostDo = $cursor ->stringToTimestamp($row[4]);

            $kal ->idpp = $row[5];
            $kal ->rc = $row[6];

            $kal ->nabehovaDobaProUrceniSmen = $database ->stringToSecondsOfDay($row[7]);

            $result[] = $kal;
        }

        $cursor ->close();

        return $result;
    }

    /**
     * @param Database $database
     * @param int $idPP
     * @param int $datum
     * @return PrirazeniKalendare
     */
    public static function nactiKalendarProDen($database, $idPP, $datum)
    {
        $kal = static::nactiKalendarProObdobi($database, $idPP, $datum, $datum);
        return count($kal)
            ? $kal[0]
            : null;
    }


    public function format($format)
    {
        $result = str_replace('%KOD%', StringUtils::htmlTextConvert($this ->kod), $format);
        return str_replace('%POPIS%', StringUtils::htmlTextConvert($this ->popis), $result);
    }
}
