<?php
/**
 * stanislav.elbl, 02.03.2022 14:56
 */

namespace RonPortal\Dochazka;

use Ron\DateUtils;
use Ron\DB\Database;
use Ron\GuiUtils;
use Ron\Http\HtmlHeaders;
use Ron\MVC\JsonController;
use Ron\MVC\JsonOutputObject;
use Ron\MVC\Redirect;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Kapacity\PlanovaniKapacit_PlanovanaAbsence;
use RonPortal\NeniPravoException;
use RonPortal\Prava;


class RocniPrehledController extends JsonController
{
    /** @var Database */
    private $database;

    /**
     * RocniPrehledController constructor.
     * @param Database $database
     */
    public function __construct(Database $database)
    {
        $this->database = $database;
    }


    /**
     * @param JsonOutputObject $output
     * @throws NeniPravoException
     * @throws \Exception
     */
    private function loadData($output)
    {
        $idPP = (int)$_GET['idpp'] ?? null;
        $rok = (int)$_GET['rok'] ?? null;

        if (!$idPP  ||  !$rok) {
            throw new \Exception('Nelze nacist data pro zadane parametry!');
        }

        $datumOd = mktime(0, 0, 0, 1, 1, $rok);
        $datumDo = mktime(0, 0, 0, 12, 31, $rok);

        Prava::nastavObdobiProPrava($datumOd, $datumDo);

        if (!Prava::lzeZobrazitPP('absence_prehled', $idPP)  &&  !Prava::lzeZobrazitPP('prohlizeni', $idPP)  &&  !Prava::lzeZobrazitPP('absence', $idPP)) {
            throw new NeniPravoException();
        }

        $seznamAbsenci = PlanovaniKapacit_PlanovanaAbsence::nactiAbsence(
            $this ->database,
            $idPP, $datumOd, $datumDo, true, false);

        $absenceJson = [];
        $legendaJson = [];
        foreach ($seznamAbsenci as $absence) {
            if ($absence ->kod != null) {   // nenacita absence z puvodniho planu, ktere byly pri upravach smazany
                $a = $absence->getJSON();
                $a['url'] = '?menuid=absence&action=zobrazabsenci&idpp=' . $absence->idPP . '&datum=' . DateUtils::dateToUrlString($absence->datumOd) . '&poradi=' . $absence->poradi;
                $absenceJson[] = $a;

                if (!isset($legendaJson[$absence ->kod])) {
                    $legendaItem = [
                        'kod' => StringUtils::jsonText($absence ->kod),
                        'zkratka' => StringUtils::jsonText($absence ->zkratka),
                        'popis' => StringUtils::jsonText($absence ->popis)
                    ];

                    if ($absence ->barva) {
                        $legendaItem['barva'] = GuiUtils::getHtmlColor($absence ->barva);
                    }

                    $legendaJson[$absence ->kod] = $legendaItem;
                }
            }
        }
        $output ->addData('idpp', $idPP);
        $output ->addData('absence', $absenceJson);
        $output ->addData('legenda', array_values($legendaJson));
    }


    private function zobrazPrehled()
    {
        $view = new RocniPrehledView($this ->database);
        $view ->loadData();

        HtmlHeaders::getInstance() ->setTitle( TextManager::_('RON_Portal') . ' - ' . $view ->title);
        return $view;
    }


    /**
     * Zpracuje vlozeni nove absence
     * @return Redirect|AbsenceFormular
     */
    function absenceFormular()
    {
        $form = new AbsenceFormular($this ->database);
        if ($form ->handleAction()) {
            // zapis OK, redirect na detail absence
            return new Redirect(null, '?menuid=rocniprehled&idpp=' . $form ->idPP);
        } else {
            return $form;
        }
    }


    function handleJsonAction($action, $output)
    {
        switch ($_GET['action']) {
            case 'loaddata' :
                $this ->loadData($output);
                break;

            default :
                throw new \Exception('Neznama akce!');
        }
    }


    public function handleAction($action)
    {
        TextManager::getInstance() ->loadStrings('dochazka.ini');

        HtmlHeaders::getInstance() ->addCss('css/dochazka.css');
        HtmlHeaders::getInstance() ->addScript('script/dochazka.js');

        if ((isset($_GET['formaction']) && $_GET['formaction'])  ||  (isset($_POST['formaction']) && $_POST['formaction'])) {
            return $this ->absenceFormular();
        }

        if (isset($_GET['action'])) {
            return parent::handleAction($action);
        }

        return $this ->zobrazPrehled();
    }
}
