<?php
/**
 * stanislav.elbl, 28.02.2021
 */

namespace RonPortal\Dochazka;

use Ron\Controls\Dialog;
use Ron\Controls\DialogButton;
use Ron\DB\Database;
use Ron\MVC\Model;
use Ron\MVC\ViewObject;
use RonPortal\Obdobi;
use RonPortal\RonPortalXMLEvents;
use RonPortal\XML\RonPortalXMLForm;

/**
 * Dialog pro schvaleni dochazky
 */
class SchvaleniView implements ViewObject
{
    var $xmlEventsText = '';
    var $showDialog;

    var $redirectUrl = null;

    /** @var Obdobi     Schvalovane obdobi */
    protected $obdobi;

    /** @var ?int       Schvalovany pracovni pomer - muze byt null (napr. v seznamu osob pro schvalovani - doplni se az pomoci javascriptu)  */
    protected $idPP;

    /**
     * SchvaleniView constructor.
     * @param Database $database
     * @param int $obdobiOd
     * @param ?int $idPP
     * @param bool $showDialog
     * @throws \Exception
     */
    public function __construct($database, $obdobiOd, $idPP = null, $showDialog = false)
    {
        $this ->showDialog = $showDialog;

        $this ->obdobi = Obdobi::vratObdobi($database, 'ADS', $obdobiOd);
        $this ->idPP = $idPP;

        Model::setData('obdobi', $this ->obdobi);
        Model::setData('datumzpracovanidochazky', DochazkaUtils::datumCasZpracovaniDochazky($database));

        // nacte pripadne xml formulare
        $this ->xmlEventsText = RonPortalXMLForm::xmlEvents($database, RonPortalXMLEvents::SCHVALOVANI_SHOW, null, '', true); // returnContent = true
    }


    /**
     * @param ?string $redirectUrl
     * @return SchvaleniView
     */
    public function setRedirectUrl($redirectUrl)
    {
        $this->redirectUrl = $redirectUrl;
        return $this;
    }



    public function render()
    {
        //parent::render();
        $dialog = (new Dialog())
            ->setView('ronportal/dochazka/view_schvaleni_dochazky.php')
            ->setTitle('Schvalovani')
            ->setId('schvaleni_dialog');

        if ($this ->showDialog) {
            $dialog
                ->addClass('zobrazit-po-nacteni')
                ->setHtmlParam('data-idpp', $this ->idPP ?? '')
                ->setHtmlParam('data-obdobi', date('j.n.Y', $this ->obdobi ->zacatek))
                ->setHtmlParam('data-redirect-url', $this ->redirectUrl);
        }

        if (!$this ->obdobi ->uzavreno) {
            $dialog -> addButton(
                (new DialogButton())
                    ->setValue('Provest')
                    ->setId('btnSchvaleniOK')
                    ->setClass('submitbutton')
            );
        }

        $dialog ->addButton(
            DialogButton::cancelButton('btnStornoSchvaleni')
        );

        $dialog ->render();

        echo $this ->xmlEventsText;
    }
}
