<?php
/**
 * stanislav.elbl, 16.03.2021
 */

namespace RonPortal\Dochazka;

use Ron\Controls\LinksList;
use Ron\DB\Parameters;
use Ron\TextManager;
use RonPortal\Controls\FilterFormFilter;
use RonPortal\Controls\RonPortalFilterForm;
use RonPortal\Prava;
use RonPortal\RonPortalSession;
use RonPortal\RonPortalSettings;

class SchvalovaniFilterForm extends RonPortalFilterForm
{
    function __construct($database)
    {
        parent::__construct($database, 'schvalovani', true);

        $this->addFilterObdobi();  // jenotevrene = false

        $this ->addFilterRozdeleni1(null, Prava::vratPrava('prohlizeni'));
        $this ->addFilterRozdeleni2(null, Prava::vratPrava('prohlizeni'));

//        $this->nastavRozdeleni(true);
//
//        $this->f_pravarozdeleni = Prava::vratPrava('prohlizeni');
//
//        if (RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI2_TYP)) {
//            $this->nastavRozdeleni2(true);
//        }

        $this ->addFilter(
            (new FilterFormFilter('urovenschvaleni'))
                ->setSessionName('filtr_urovenschvaleni')
                ->setRowId('filter_urovenschvaleni')
                ->addItem(0, TextManager::_('Vsechno'))
                ->addItem(1, TextManager::_('Neschvaleno1'))
                ->addItem(2, TextManager::_('Neschvaleno2'))
                ->addItem(3, TextManager::_('Neschvaleno3'))
                ->setOtherSettings('dodrzovatHierarchii', RonPortalSettings::getIntValue('dodrzovatcashierarchiischvalovani'))
            );

        $this ->addFilter(
            (new FilterFormFilter('kontroly'))
                ->setSessionName('filtr_kontrolydochazky')
                ->setRowId('filter_kontroly')
                ->addItem(0, TextManager::_('Vsechno'))
                ->addItem(1, TextManager::_('Jen_s_chybami_v_dochazce'))
                ->addItem(2, TextManager::_('Jen_bez_chyb_v_dochazce'))
            );
    }

    /**
     * @param Parameters $params
     * @return string
     */
    function buildSqlFilter($params = null)
    {
        $result = parent::buildSqlFilter($params);

        // kontroly
        if ($this ->getFilterValue('kontroly')) {
            $result .=
                ($result ? ' AND ' : '') .
                ($this ->getFilterValue('kontroly') === '2' ? 'NOT ' : '') . 'EXISTS (' .
                'SELECT * FROM VysledekKontrolyDochazky, TypKontrolyDochazky' .
                ' WHERE VysledekKontrolyDochazky.TypKontroly = TypKontrolyDochazky.Kod' .
                ' AND TypKontrolyDochazky.Kategorie > 10' .  // vynecha typ 10=informace
                ' AND VysledekKontrolyDochazky.Ignorovat = 0' .
                ' AND VysledekKontrolyDochazky.RC = PracovniPomer.RC' .
                ' AND VysledekKontrolyDochazky.OsobniCislo = PracovniPomer.OsobniCislo' .
                ' AND VysledekKontrolyDochazky.DatumNastupu = PracovniPomer.DatumNastupu' .
                ' AND VysledekKontrolyDochazky.Datum >= ' . $params->addDateParameter($this ->getFilterValue('obdobi')) .
                ' AND VysledekKontrolyDochazky.Datum <= ' . $params->addDateParameter($this ->getFilterValue('obdobi', 'datumDo')) .
                ')';
        }

        // uroven schvaleni
        switch ((int)$this ->getFilterValue('urovenschvaleni')) {
            case 1 :
                $filtrUroven =
                    '(SchvaleniDochazky.ZpracovanoDo IS NULL  OR  SchvaleniDochazky.ZpracovanoDo < ' .
                    $params ->addDateParameter($this ->getFilterValue('obdobi', 'datumDo')) . ')';
                break;

            case 2 :
                $filtrUroven =
                    '(SchvaleniDochazky.ZkontrolovanoDo IS NULL  OR  SchvaleniDochazky.ZkontrolovanoDo < ' .
                    $params ->addDateParameter($this ->getFilterValue('obdobi', 'datumDo')) . ')';

                if ($this ->getFilter('urovenschvaleni') ->otherSettings['dodrzovatHierarchii']) {
                    // pokud je zapnuto dodrzovani hierarchie, tak zobrazuje jen ty, ktere jsou schvalene predchozi urovni
                    $filtrUroven .= ' AND SchvaleniDochazky.ZpracovanoDo = ' . $params->addDateParameter($this->getFilterValue('obdobi', 'datumDo'));
                }

                break;

            case 3 :
                $filtrUroven =
                    '(SchvaleniDochazky.SchvalenoDo IS NULL  OR  SchvaleniDochazky.SchvalenoDo < ' .
                    $params ->addDateParameter($this ->getFilterValue('obdobi', 'datumDo')) . ')';

                if ($this ->getFilter('urovenschvaleni') ->otherSettings['dodrzovatHierarchii']) {
                    // pokud je zapnuto dodrzovani hierarchie, tak zobrazuje jen ty, ktere jsou schvalene predchozi urovni
                    $filtrUroven .= ' AND SchvaleniDochazky.ZkontrolovanoDo = ' . $params->addDateParameter($this->getFilterValue('obdobi', 'datumDo'));
                }

                break;

            default :
                $filtrUroven = null;
                break;
        }

        if ($filtrUroven) {
            $result .=
                ($result ? ' AND ' : '') .
                '(' . $filtrUroven . ')';
        }

        return $result;
    }
}
