<?php
/**
 * stanislav.elbl, 21.02.2021
 */

namespace RonPortal\Dochazka;

use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\Parameters;
use Ron\GuiUtils;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\RonPortalSettings;
use RonPortal\User;


class Smena
{
    private static $cache = [];

    public $kod;
    public $popis;
    public $zkratka;
    public $barva;

    public $variabilni;
    /** @var int  pocet sekund */
    public $pocatekSmeny;
    /** @var int  pocet sekund */
    public $konecSmeny;
    /** @var int  pocet sekund */
    public $delkaSmeny;

    public $delena;
    public $preruseniOd;
    public $preruseniDo;

    public $denDochPodleKonceSmeny;

    public $aktivni;
    public $vybrana; // jestli ma uzivatel tuto smenu vybranou

    public $typVkladaniPrestavky;
    public $prestavkaPoIntervalu;
    public $prestavkaDoIntervalu;
    public $delkaPrestavky;
    public $maxPocetPrestavek1Typu;
    public $kodOperacePrestavky1Typu;

    public $pokracovat2TypemPrestavky;
    public $prestavkaPoIntervalu2Typu;
    public $prestavkaDoIntervalu2Typu;
    public $delkaPrestavky2Typu;
    public $maxPocetPrestavek2Typu;
    public $kodOperacePrestavky2Typu;

    /** @var bool   Priznak, ze se ma prepocitavat delka a konec smeny */
    public $vzdyDopocitavatDelkuKonec;


    /**
     * @param Database $database
     * @param array $row
     */
    private function loadRow($database, $row)
    {
        $this ->kod = $row[0];
        $this ->popis = TextManager::_('db_smena_popis_' . $row[0], $row[1]);
        $this ->zkratka = $row[2];
        $this ->barva = $row[3];

        $this ->variabilni = (int)$row[5];
        $this ->pocatekSmeny = isset($row[6]) ? $database ->stringToSecondsOfDay($row[6]) : null;
        $this ->konecSmeny = isset($row[7]) ? $database ->stringToSecondsOfDay($row[7]) : null;
        $this ->delkaSmeny = isset($row[4]) ? $database ->stringToSecondsOfDay($row[4]) : null;

        $this ->denDochPodleKonceSmeny = $row[8];

        $this ->typVkladaniPrestavky = $row[9];

        $this ->prestavkaPoIntervalu = $database ->stringToSecondsOfDay($row[10]);
        $this ->prestavkaDoIntervalu = $database ->stringToSecondsOfDay($row[11]);
        $this ->delkaPrestavky = $database ->stringToSecondsOfDay($row[12]);
        $this ->maxPocetPrestavek1Typu = $row[13];
        $this ->kodOperacePrestavky1Typu = $row[23];

        $this ->pokracovat2TypemPrestavky = $row[14];
        $this ->prestavkaPoIntervalu2Typu = $database ->stringToSecondsOfDay($row[15]);
        $this ->prestavkaDoIntervalu2Typu = $database ->stringToSecondsOfDay($row[16]);
        $this ->delkaPrestavky2Typu = $database ->stringToSecondsOfDay($row[17]);
        $this ->maxPocetPrestavek2Typu = $row[18];
        $this ->kodOperacePrestavky2Typu = $row[24];

        $this ->aktivni = $row[19];

        $this ->delena = (int)$row[20];
        $this ->preruseniOd = isset($row[21]) ? $database ->stringToSecondsOfDay($row[21]) : null;
        $this ->preruseniDo = isset($row[22]) ? $database ->stringToSecondsOfDay($row[22]) : null;

        $this ->vzdyDopocitavatDelkuKonec = (int)$row[25] != 0;
    }


    public static function getSelectPart()
    {
        return
            'Smena.Kod, Smena.Popis, Smena.ZkratkaProPlanovani, Smena.BarvaProVykresleni,' . // 0..3
            ' Smena.DelkaSmeny, Smena.Variabilni, Smena.PocatekSmeny, Smena.KonecSmeny, Smena.DenDochPodleKonceSmeny,' .   // 4..8
            ' Smena.TypVkladaniPrestavky, PrestavkaPoIntervalu, PrestavkaDoIntervalu, DelkaPrestavky, MaxPocetPrestavek1Typu,' . // 9..13
            ' Pokracovat2TypemPrestavky, PrestavkaPoIntervalu2Typu, PrestavkaDoIntervalu2Typu, DelkaPrestavky2Typu, MaxPocetPrestavek2Typu, Smena.Aktivni,' . // 14..19
            ' Smena.Delena, Smena.PreruseniOd, Smena.PreruseniDo,' . // 20..22
            ' Smena.KodOperacePrestavky1Typu, Smena.KodOperacePrestavky2Typu,' . // 23,24
            ' Smena.VzdyDopocitavatDelkuKonec'; // 25
    }

    /**
     * Nacte z databaze smenu se zadanym kodem
     * @param Database $database
     * @param string $kod
     * @return Smena
     * @throws DBException
     */
    public static function nactiSmenu($database, $kod)
    {
        // nacte nastaveni smeny
        $params = $database ->createParameters();
        $sql =
            'SELECT ' . static::getSelectPart() .
            ' FROM Smena WHERE Kod = ' . $params ->addParameter($kod);

        $cursor = $database ->getSqlResult($sql, $params, 'Chyba ve funkci Smena::load!');
        $row = $cursor ->fetch();
        $cursor ->close();

        if (!$row) {
            return null;
        }

        $smena = new static();
        $smena ->loadRow($database, $row);
        return $smena;
    }


    /**
     * Vrati smenu se zadanym kodem. Pokud uz byla drive nactena, tak je v cache, jinak ji nacte z databaze a ulozi do cache
     * @param $database
     * @param $kod
     * @return Smena
     */
    public static function vratSmenu($database, $kod)
    {
        if (isset(self::$cache[$kod])) {
            // uz je v cache
            return self::$cache[$kod];
        }

        // neni v cache => nacte z databaze
        $smena = self::nactiSmenu($database, $kod);

        self::$cache[$kod] = $smena;
        return $smena;
    }


    /**
     * @param Database $database
     * @param string $filter
     * @param Parameters $filterParams
     * @param null $orderBy
     * @return Smena[]
     * @throws DBException
     */
    public static function nactiSmeny($database, $filter = null, $filterParams = null, $orderBy = null)
    {
        // nacte nastaveni smeny
        $sql =
            'SELECT ' . static::getSelectPart() .
            ' FROM Smena';

        if ($filter) {
            $sql .= ' WHERE ' . $filter;
        }

        $sql .= ' ORDER BY ' . ($orderBy ?: 'Smena.Popis');


        $cursor = $database ->getSqlResult($sql, $filterParams, 'Chyba pri nacitani seznamu smen!');

        $result = [];

        while ($row = $cursor ->fetch()) {
            $smena = new static();
            $smena ->loadRow($database, $row);
            $result[] = $smena;
        };
        $cursor ->close();

        return $result;
    }


    /**
     * @param Database $database
     * @param string $kalendar
     * @param int $datum
     * @return Smena[]
     */
    public static function nactiSmenyKalendare($database, $kalendar, $datum = null)
    {
        // nacte nastaveni smeny
        $params = $database ->createParameters();
        $sql =
            'SELECT ' . static::getSelectPart() . ' FROM Smena, ';

        if (isset($datum)) {
            $sql .=
                ' SmenaProDenKalendare' .
                ' WHERE SmenaProDenKalendare.Smena = Smena.Kod' .
                ' AND SmenaProDenKalendare.Kalendar = ' . $params ->addParameter($kalendar) .
                ' AND SmenaProDenKalendare.Den = ' . $params ->addParameter($database ->dateToParam($datum));
        } else {
            $sql .=
                ' PouzivaneSmeny' .
                ' WHERE PouzivaneSmeny.KodSmeny = Smena.Kod' .
                ' AND PouzivaneSmeny.KodKalendare = ' . $params ->addParameter($kalendar);
        }

        $sql .= ' ORDER BY Smena.Popis';

        $cursor = $database ->getSqlResult($sql, $params, 'Chyba pri nacitani seznamu smen!');

        $result = [];

        while ($row = $cursor ->fetch()) {
            $smena = new static();
            $smena ->loadRow($database, $row);
            $result[] = $smena;
        };
        $cursor ->close();

        return $result;
    }

    /**
     * @return array
     */
    function getJson()
    {
        $result = [
            'kod' => StringUtils::jsonText($this ->kod),
            'zkratka' => StringUtils::jsonText($this ->zkratka),
            'barva' => GuiUtils::getHtmlColor($this->barva),
            'delkaStr' => StringUtils::format('minuty', $this->delkaSmeny / 60),
            'pocatekStr' => StringUtils::format('minuty', $this->pocatekSmeny / 60),
            'konecStr' => StringUtils::format('minuty', $this->konecSmeny / 60),
            'popis' => StringUtils::jsonText($this->popis),
            'aktivni' => (int)$this ->aktivni
        ];

        if ($this ->variabilni) {
            $result['variabilni'] = 1;
            $result['prepocitatDelku'] = $this ->vzdyDopocitavatDelkuKonec ? 1 : 0;
        }
        if ($this ->vybrana) {
            $result['vybrana'] = 1;
        }

        if ($this ->delena) {
            $result['delena'] = 1;
            $result['preruseniOdStr'] = StringUtils::format('minuty', $this ->preruseniOd / 60);
            $result['preruseniDoStr'] = StringUtils::format('minuty', $this ->preruseniDo / 60);
        }

        return $result;
    }


    /**
     * Nacte seznam smen
     * @param Database $database
     * @param string $filter
     * @param string $orderBy
     * @return Smena[]
     */
    static function nactiSmenyUzivatele($database, $filter = '', $orderBy = 'Smena.Popis')
    {
        $result = [];

        $params = $database ->createParameters();

        // nacte seznam smen
        $sql =
            'SELECT ' . self::getSelectPart() . ', ' . // 0..25
            '   SmenyUzivatele.RC ' . // 26
            '  FROM Osoba INNER JOIN Smena ON 1 = 1' .
            '   LEFT OUTER JOIN SmenyUzivatele' .
            '   ON SmenyUzivatele.Smena = Smena.Kod ' .
//            "   AND SmenyUzivatele.RC = '" . User::getRC() . "'"; - pozor! toto nefunguje na ODBC s parametrem, proto pripojuji i tabulku osoba a podminku jsem presunul do where
            '   AND SmenyUzivatele.RC = Osoba.RC' .
            ' WHERE Osoba.RC = ' . $params ->addParameter(User::getRC());

        if ($filter) {
            $sql .= ' AND (' . $filter . ')'; // Smena.ZkratkaProPlanovani IS NOT NULL' .
        }
        //"   AND Smena.ZkratkaProPlanovani <> '' ".  - nefunguje na Oracle!

        $sql .= ' ORDER BY ' . $orderBy; // ZkratkaProPlanovani';

        $cursor = $database->getSqlResult($sql, $params);

        while ($row = $cursor ->fetch()) {
            if ($row[2] != '') {
                $smena = new Smena();

                $smena ->loadRow($database, $row);
                $smena ->vybrana = !!($row[26]);

                $result[] = $smena;
            }
        }

        $cursor ->close();

        return $result;
    }



    public static function intervalsPenetration($min1, $max1, $min2, $max2)
    {
        $result = 0;

        //test zda nejsou meze prehozene
        if ($min1 > $max1) {

            $tempDateTime = $min1;
            $min1 = $max1;
            $max1 = $tempDateTime;
        }

        if ($min2 > $max2) {
            $tempDateTime = $min2;
            $min2 = $max2;
            $max2 = $tempDateTime;
        }

        if (!(($max2 <= $min1) or ($min2 >= $max1))) {
            $result = ($max1 < $max2 ? $max1 : $max2) //    min(max1,max2)
                - ($min1 > $min2 ? $min1 : $min2); // - max(min1,min2);
        }

        return $result;
    }


    /**
     * Vraci delku smeny po odecteni prestavek (v sekundach)
     * @param Database $database
     * @param int $pocatek
     * @param int $konec
     * @param int $preruseniOd
     * @param int $preruseniDo
     * @return int
     * @throws DBException
     */
    public function vypocitejDelkuSmeny($database, $pocatek, $konec, $preruseniOd = null, $preruseniDo = null)
    {
        $nevkladatPrestavkuNaKonciSmeny = RonPortalSettings::getIntValue('nevkladatPrestavkuNaKonciSmeny', 1); // TODO: prevzit z aplikace?

        // pokud je konec drive nez pocatek, jedna se o smenu pres pulnoc
        if ($konec < $pocatek) {
            $konec = DateUtils::incDay($konec, 1); // timestringtotimestamp($_POST['smena_konec'], incday($datum, 1) );
        }

        $delka = $konec - $pocatek; // delka bez zapocteni prestavek

        if ($this->delena  &&  $preruseniDo > $preruseniOd) {
            // odecte preruseni
            if ($preruseniOd >= $pocatek  &&  $preruseniDo <= $konec) {
                // odecte cele preruseni (je uvnitr intervalu)
                return $delka - ($preruseniDo - $preruseniOd);
            }

            if ($preruseniOd >= $pocatek) {
                // odecte preruseni na zacatku intervalu
                return $delka - ($konec - $preruseniOd);
            }

            if ($preruseniDo <= $konec) {
                return $delka - ($preruseniDo - $pocatek);
            }

            if ($preruseniOd <= $pocatek  &&  $preruseniDo >= $konec) {
                return 0;
            }
        }

        switch ($this->typVkladaniPrestavky) {
            case 1 : // po intervalu
            case 3 : // podle delky odpracovane smeny
                $lastBreakEnd = $pocatek;

                $interval = $this->prestavkaPoIntervalu;
                $delkaPrestavky = $this->delkaPrestavky;
                $prestavkaDoIntervalu = $this->prestavkaDoIntervalu;

                $operace = $this ->kodOperacePrestavky1Typu
                    ? OperaceDochazky::nactiOperaci($database, $this ->kodOperacePrestavky1Typu)
                    : null;

                // prestavka 1. typu
//                if (!$operace ->narokNaPrestavku) {  // operace, ktere maji zaskrtnuto "narok na prestavku" neodecita
                    for ($i = 0; $i < $this->maxPocetPrestavek1Typu; $i++) {
                        // Dalsi prestavka od-do
                        $breakFrom = $lastBreakEnd + $interval;

                        $breakTimeAfterShift = ($lastBreakEnd + $prestavkaDoIntervalu) - $konec;
                        if ($breakTimeAfterShift < 0) {
                            $breakTimeAfterShift = 0;
                        }

                        if ($breakTimeAfterShift >= $delkaPrestavky) {
                            //break;
                            return $delka;
                        }

                        // skutecna delka vkladane prestavky.
                        //$breakLength = $delkaPrestavky - $breakTimeAfterShift;

                        $breakTo = $breakFrom + $delkaPrestavky;

                        // kolik casu z prestavky se vleze do casu smeny
                        $shiftBreakTime = self::intervalsPenetration($pocatek, $konec, $breakFrom, $breakTo);
                        if ($shiftBreakTime > 0) {
                            if ($operace  &&  !$operace ->narokNaPrestavku) {  // operace, ktere maji zaskrtnuto "narok na prestavku" neodecita
                                if ($breakTo < $konec || !$nevkladatPrestavkuNaKonciSmeny) {
                                    // prestavka neni na konci smeny nebo se ma vkladat i na konci
                                    $delka = $delka - $shiftBreakTime; // Snizeni delky smeny o prestavku v case smeny.
                                }
                            }
                        } else {
                            // vice prestavek se do smeny nevejde.
                            //break;
                            return $delka;
                        }

                        //Nastaveni konce posledni prestavky
                        $lastBreakEnd = $breakTo;
                    }
//                }

                $operace2 = $this ->kodOperacePrestavky2Typu
                    ? OperaceDochazky::nactiOperaci($database, $this ->kodOperacePrestavky2Typu)
                    : null;

                // prestavka 2. typu
                if ($operace2  &&  !$operace2 ->narokNaPrestavku) {
                    if ($this->typVkladaniPrestavky == 1 && $this->pokracovat2TypemPrestavky) {
                        $interval = $this->prestavkaPoIntervalu2Typu;
                        $delkaPrestavky = $this->delkaPrestavky2Typu;
                        $prestavkaDoIntervalu = $this->prestavkaDoIntervalu2Typu;

                        for ($i = 0; $i < $this->maxPocetPrestavek2Typu; $i++) {
                            // Dalsi prestavka od-do
                            $breakFrom = $lastBreakEnd + $interval; // + PrestavkaPoIntervalu

                            $breakTimeAfterShift = ($lastBreakEnd + $prestavkaDoIntervalu) - $konec;
                            if ($breakTimeAfterShift < 0) {
                                $breakTimeAfterShift = 0;
                            }

                            if ($breakTimeAfterShift >= $delkaPrestavky) {
                                // break;
                                return $delka;
                            }

                            // skutecna delka vkladane prestavky.
                            //$breakLength = $delkaPrestavky - $breakTimeAfterShift;

                            $breakTo = $breakFrom + $delkaPrestavky;

                            // kolik casu z prestavky se vleze do casu smeny
                            $shiftBreakTime = self::intervalsPenetration($pocatek, $konec, $breakFrom, $breakTo);
                            if ($shiftBreakTime > 0) {
                                if ($breakTo < $konec || !$nevkladatPrestavkuNaKonciSmeny) {
                                    // prestavka neni na konci smeny nebo se ma vkladat i na konci
                                    $delka = $delka - $shiftBreakTime; // Snizeni delky smeny o prestavku v case smeny.
                                }
                            } else {
                                // vice prestavek se do smeny nevejde.
                                //break;
                                return $delka;
                            }

                            //Nastaveni konce posledni prestavky
                            $lastBreakEnd = $breakTo;
                        }
                    }
                }

                break;

            case 2 : // v pevny cas
                $params = $database->createParameters();
                $sql =
                    'SELECT PevnaPrestavka.CasPocatkuPrestavky, PevnaPrestavka.DelkaPrestavky' .
                    ' FROM PevnaPrestavka, OperaceDochazky' .
                    ' WHERE OperaceDochazky.Kod = PevnaPrestavka.KodOperacePrestavky' .
                    '   AND OperaceDochazky.NarokNaPrestavku = 0' .  // neodecita operace, ktere maji zaskrtnuto "narok na prestavku"
                    '   AND Smena = ' . $params->addParameter($this ->kod) .
                    ' ORDER BY CasPocatkuPrestavky';

                $cursor = $database->getSqlResult($sql, $params, 'Chyba ve funkci vratDelkuSmeny!');

                while ($row = $cursor->fetch()) {
                    // Dalsi prestavka od-do
                    $breakFrom = $database->stringToSecondsOfDay($row[0]);

                    if ($konec > 86400  &&  $breakFrom < $pocatek) {
                        // je to nocni smena a prestavka zacina drive nez je jeji zacatek => tj. je az v dalsim dni
                        $breakFrom = $breakFrom + 86400; // TODO: vyresit posun casu? Ale koluji zvesti, ze se bude rusit!
                    }

                    $breakTo = $breakFrom + $database->stringToSecondsOfDay($row[1]);

                    // kolik casu z prestavky se vleze do casu smeny.
                    $shiftBreakTime = self::intervalsPenetration($pocatek, $konec, $breakFrom, $breakTo);
                    if ($shiftBreakTime > 0) {
                        if ($konec > $breakTo) {
                            // snizeni delky o prestavku
                            $delka = $delka - $shiftBreakTime;
                        }
                        else {
                            if ($nevkladatPrestavkuNaKonciSmeny) {
                                break;
                            } else {
                                // prestavka na konci smeny -> konec se prodluzuje od konce prestavky
                                $delka = $delka - $shiftBreakTime;
                            }
                        }
                    } else {
                        // prestavka uz se do smeny nevejde
                        break;
                    }
                }

                $cursor->close();

                break;

        }

        return $delka;
    }

    /**
     * Vraci delku smeny po odecteni prestavek (v sekundach)
     * @param Database $database
     * @param int $pocatek
     * @param int $konec
     * @param string $kod
     * @param int $preruseniOd
     * @param int $preruseniDo
     * @return int
     * @throws DBException
     */
    public static function vratDelkuSmeny($database, $pocatek, $konec, $kod, $preruseniOd = null, $preruseniDo = null)
    {
        $smena = self::nactiSmenu($database, $kod);
        if (!$smena) {
            return 0;
        }

        return $smena ->vypocitejDelkuSmeny($database, $pocatek, $konec, $preruseniOd, $preruseniDo);
    }


    /**
     * Vraci cas konce smeny (v sekundach od pulnoci)
     * @param Database $database
     * @param int    $pocatek
     * @param int    $delka
     * @param string $kod
     * @return int
     * @throws DBException
     */
    public static function vratKonecSmeny($database, $pocatek, $delka, $kod)
    {
        $NevkladatPrestavkuNakonciSmeny = RonPortalSettings::getIntValue('nevkladatPrestavkuNaKonciSmeny', 1);;

        $konec = $pocatek + $delka; // konec bez zapocteni prestavek

        $smena = static::nactiSmenu($database, $kod);
        if ( $smena ) {

            switch ($smena->typVkladaniPrestavky) {
                case 1 : // po intervalu
                case 3 : // podle delky odpracovane smeny
                    $lastBreakEnd = $pocatek;

                    $interval = $smena ->prestavkaPoIntervalu;
                    $delkaprestavky = $smena ->delkaPrestavky;
                    $prestavkadointervalu = $smena ->prestavkaDoIntervalu;

                    $operace = OperaceDochazky::nactiOperaci($database, $smena ->kodOperacePrestavky1Typu);

                    // prestavka 1. typu
                    for ($i = 0; $i < $smena ->maxPocetPrestavek1Typu; $i++) {
                        // Dalsi prestavka od-do
                        $breakFrom = $lastBreakEnd + $interval;
                        $breakTo = $breakFrom + $delkaprestavky;

                        $breakTimeAfterShift = ($lastBreakEnd + $prestavkadointervalu) - $konec;
                        if ($breakTimeAfterShift < 0) {
                            $breakTimeAfterShift = 0;
                        }

                        if ($breakTimeAfterShift >= $delkaprestavky) {
                            return $konec;
                        }

                        // kolik casu z prestavky se vleze do casu smeny
                        $shiftBreakTime = self::intervalsPenetration($pocatek, $konec, $breakFrom, $breakTo);
                        if ($shiftBreakTime > 0) {
                            if (!$operace ->narokNaPrestavku) {  // operace, ktere maji zaskrtnuto "narok na prestavku" neodecita
                                if ($breakTo < $konec || !$NevkladatPrestavkuNakonciSmeny) { // prestavka neni na konci smeny nebo se ma vkladat i na konci
                                    $konec = $konec + $shiftBreakTime; // navyseni o prestavku v case smeny.
                                }
                            }
                        } else {
                            // vice prestavek se do smeny nevejde.
                            return $konec;
                        }

                        //Nastaveni konce posledni prestavky
                        $lastBreakEnd = $breakTo;
                    }

                    $operace2 = OperaceDochazky::nactiOperaci($database, $smena ->kodOperacePrestavky2Typu);

                    // prestavka 2. typu
                    if (!$operace2 ->narokNaPrestavku) {
                        if ($smena->typVkladaniPrestavky == 1 && $smena->pokracovat2TypemPrestavky) {
                            $interval = $smena->prestavkaPoIntervalu2Typu;
                            $delkaprestavky = $smena->delkaPrestavky2Typu;
                            $prestavkadointervalu = $smena->prestavkaDoIntervalu2Typu;

                            for ($i = 0; $i < $smena->maxPocetPrestavek2Typu; $i++) {
                                // Dalsi prestavka od-do
                                $breakFrom = $lastBreakEnd + $interval; // + PrestavkaPoIntervalu
                                $breakTo = $breakFrom + $delkaprestavky;

                                $breakTimeAfterShift = ($lastBreakEnd + $prestavkadointervalu) - $konec;
                                if ($breakTimeAfterShift < 0) {
                                    $breakTimeAfterShift = 0;
                                }

                                if ($breakTimeAfterShift >= $delkaprestavky) {
                                    return $konec;
                                }

                                // kolik casu z prestavky se vleze do casu smeny
                                $shiftBreakTime = self::intervalsPenetration($pocatek, $konec, $breakFrom, $breakTo);
                                if ($shiftBreakTime > 0) {
                                    if ($breakTo < $konec || !$NevkladatPrestavkuNakonciSmeny) { // prestavka neni na konci smeny nebo se ma vkladat i na konci
                                        $konec = $konec + $shiftBreakTime; // navyseni o prestavku v case smeny.
                                    }
                                } else {
                                    // vice prestavek se do smeny nevejde.
                                    return $konec;
                                }

                                //Nastaveni konce posledni prestavky
                                $lastBreakEnd = $breakTo;
                            }
                        }
                    }

                    break;

                case 2 :
                    $params = $database ->createParameters();
                    $sql =
                        'SELECT CasPocatkuPrestavky, DelkaPrestavky' .
                        ' FROM PevnaPrestavka, OperaceDochazky' .
                        ' WHERE OperaceDochazky.Kod = PevnaPrestavka.KodOperacePrestavky' .
                        '   AND Smena = ' . $params ->addParameter($kod) .
                        '   AND OperaceDochazky.NarokNaPrestavku = 0' .
                        ' ORDER BY CasPocatkuPrestavky';

                    $cursor = $database ->getSqlResult($sql, $params, 'Chyba ve funkci vratDelkuSmeny!');

                    while ($row = $cursor ->fetch()) {
                        // Dalsi prestavka od-do
                        $breakFrom = $database ->stringToSecondsOfDay($row[0]);
                        $breakTo = $breakFrom + $database ->stringToSecondsOfDay($row[1]);

                        // kolik casu z prestavky se vleze do casu smeny.
                        $shiftBreakTime = self::intervalsPenetration($pocatek, $konec, $breakFrom, $breakTo);
                        if ($shiftBreakTime > 0) {
                            if ($konec > $breakTo) $konec = $konec + $shiftBreakTime; // prodlouzeni o prestavku
                            else {
                                if ($NevkladatPrestavkuNakonciSmeny) {
                                    break;
                                } else {
                                    // prestavka na konci smeny -> konec se prodluzuje od konce prestavky
                                    $konec = $breakTo + $shiftBreakTime;
                                }
                            }
                        } else {
                            // prestavka uz se do smeny nevejde
                            break;
                        }
                    }

                    $cursor ->close();

                    break;
            }
        }

        $konec = $konec % 86400; // pokud je pres pulnoc

        return $konec;
        //return DateUtils::minutesToString((int)($konec / 60));
    }

}
