<?php
/**
 * stanislav.elbl, 23.02.2021
 */

namespace RonPortal;

use Ron\DB\Database;
use Ron\DB\SqlStatement;
use Ron\FileUtils;
use Ron\StringUtils;


class Dokument
{
    /** @var int */
    public $id;
    /** @var string */
    public $popis;
    /** @var int */
    public $typ;
    /** @var int    Datum vlozeni dokumentu */
    public $datum;
    /** @var string */
    public $jmenoSouboru;
    /** @var string */
    public $poznamka;


    /**
     * @param Database $database
     * @return string
     */
    public static function getSelectPart($database)
    {
        return
            'Dokument.Id, Dokument.Popis, Dokument.Typ, Dokument.Datum, Dokument.JmenoSouboru, ' . // 0..4
            $database->castWideMemo('Dokument.Poznamka');   // 5
    }


    /**
     * Nacte seznam dokumentu ze zadaneho sql dotazu
     * @param Database $database
     * @param string|SqlStatement $sql
     * @param Parameters [$params]
     * @return Dokument[]
     */
    public static function nactiDokumenty($database, $sql, $params = null)
    {
        if ($sql instanceof SqlStatement) {
            $params = $sql ->parameters;
            $sql = $sql ->sqlText;
        }

        $result = [];

        $cursor = $database ->getSqlResult($sql, $params);
        while ($row = $cursor ->fetch()) {
            $dokument = new static();
            $dokument ->loadRow($database, $row);

            $result[] = $dokument;
        }

        return $result;
    }


    /**
     * @param Database $database
     * @param mixed[] $row
     */
    public function loadRow($database, $row)
    {
        $this ->id = $row[0];
        $this ->popis = $row[1];
        $this ->typ = $row[2];
        $this ->datum = $database ->stringToTimestamp($row[3]);
        $this ->jmenoSouboru = $row[4];
        $this ->poznamka = $row[5];
    }


    /**
     * @param Dokument[] $dokumenty     Seznam dokumentu
     * @param string $downloadUrl       Url pro stahnuti dokumentu - pripoji se za ni id dokumentu
     */
    public static function renderTableRows($dokumenty, $downloadUrl)
    {
        if (!$dokumenty) {
            return;
        }

        foreach ($dokumenty as $dokument) {
            $img = FileUtils::getIcon($dokument ->jmenoSouboru);
            echo
                '<tr>' .
                '<td>' . ($img ? "<i class='fal inline-glyph $img'></i>" : '') . '</td>' .
                '<td>' . StringUtils::format('date', $dokument->datum) . '</td>' .
                '<td>' .
                    ($downloadUrl ? "<a href='$downloadUrl{$dokument->id}' target='_blank'>" : '') .
                    StringUtils::htmlTextConvert($dokument->jmenoSouboru) .
                    ($downloadUrl ? '</a>' : '') .
                '</td>' .
                '<td>' . StringUtils::htmlTextConvert($dokument->popis) . '</td>' .
                '</tr>';
        }
    }

    public static function renderTable($dokumenty, $downloadUrl)
    {
        if (!$dokumenty) {
            return;
        }

        echo '<table>';
        self::renderTableRows($dokumenty, $downloadUrl);
        echo '</table>';
    }
}
