<?php
/**
 * stanislav.elbl, 22.02.2021
 */

namespace RonPortal\Dokumenty;

use Ron\Controls\Dialog;
use Ron\Controls\DialogButton;
use Ron\DB\Database;
use Ron\DB\SqlStatement;
use Ron\TextManager;


class DokumentUploadForm extends Dialog
{
    /** @var Database */
    protected $database;

    /** @var string */
    public $uploadUrl;

    /** @var string */
    public $redirectUrl;

    /** @var string|SqlStatement */
    public $sqlPripojeniStarsiho = null;

    public $hiddenFields = null;

    /** @var int pokud je nastaven v potomku, ve formulari se nezobrazi vyber typu dokumentu */
    public $typDokumentu = null;

    /** @var int pokud je nastavena v potomku, ve formulari se zobrazi jen vyber typu dokumentu s touto kategorii */
    public $kategorieDokumentu = null;

    public $target = null;

    //public $htmlAttrs = [];

    public $csrfRequestId = '';  // TODO: nastavit nejakou default hodnotu? (ale nejdrive je potreba zkontrolovat kod, kde se to pouziva)

    /**
     * DokumentUploadForm constructor.
     * @param Database $database
     * @param string $uploadUrl
     * @param string[] $htmlAttrs   HTML atributy pro dialog - urcuji dalsi chovani
     */
    public function __construct(Database $database, $uploadUrl, $htmlAttrs = null)
    {
        parent::__construct();

        $this->database = $database;
        $this->uploadUrl = $uploadUrl;

        $this ->addClass('form ronform middle-form');
        $this ->id = 'novydokument';

        $this ->title = TextManager::_('Vlozit_novy_dokument');

        if (isset($htmlAttrs)) {
            $this ->htmlParams = $htmlAttrs;
        } else {
            $this ->target = 'upload_target';
            $this ->htmlParams = ['data-action' => 'reload'];
        }

        $this ->setView('view/dokument_novy.php');

        $this ->formTag = [
            'action' => $this ->uploadUrl,
            'method' => 'post',
            'enctype' => 'multipart/form-data'
        ];

        $this ->addButton(
            (new DialogButton())
                ->setValue('form.Insert')
                ->setClass('submitbutton')
                ->setId('vlozitNovyDokumentBtn')
        );

        $this ->addButton(DialogButton::cancelButton('stornoNovyDokumentBtn'));
    }


    protected function getBasicRenderParams()
    {
        $result = parent::getBasicRenderParams();

        if ($this ->redirectUrl)  {
            $result['data-redirect-url'] = $this ->redirectUrl;
        }

        return $result;
    }


    /**
     * Potomek muze doplnit dalsi pole
     */
    public function dalsiRadkyKDokumentu()
    {

    }


    protected function getFormTagHtml()
    {
        if ($this ->target  &&  is_array($this ->formTag)) {
            $this ->formTag['target'] = $this ->target;
        }
        return
            ($this ->target
                ? '<iframe id="upload_target" name="upload_target" src=""></iframe>' //  App::getDebugMode() ? 'class="debug"' : ''
                : '') .
            parent::getFormTagHtml();
    }


    public function renderButtons()
    {
        echo '<div>';
        parent::renderButtons();
        echo "</div>\n";

        echo
            // progress
            '<div id="novydokument_progressrow" class="d-none">' .
            TextManager::_('Probiha_odesilani_souboru') .
            '</div>' .
            // error
            '<div id="novydokument_error"></div>';
    }



//    public function render()
//    {
//        require('view/dokument_novy.php');
//    }
}
