<?php
/**
 * pavel.rudol, 01.11.2022 18:12
 */

namespace RonPortal\Dovednosti;

use Exception;
use Ron\Controls\BrowserRowArray;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\SqlStatement;
use Ron\DB\SQLUtils;
use Ron\StringUtils;


class SeznamDovednostiOsobVyhledani extends SeznamDovednostiOsobBrowser
{
    /**
     * SeznamDovednostiOsobVyhledani constructor.
     * @param Database $database
     * @param $typZobrazeni
     * @throws Exception
     */
    public function __construct(Database $database, $typZobrazeni)
    {
        parent::__construct($database, $typZobrazeni);

        $this->zobrazitKalendar = false;
        $this->zobrazitPocetDosazenychDovednosti = true;

        //strankovani se nepouzije, nactena data jsou razena dle poctu dosazenych dovednosti
        $this->maxPageRows = null;

        $this->filterForm = new VyhledaniDovednostiFilterForm($this->database, $this->typRozdeleniFiltr->kod, $this->typRozdeleniPracMisto->kod);
    }

    /**
     * Doplni do SQL dotazu filtr na rozdeleni, PM podle zvoleneho typu zobrazeni a filtr na dovednosti zvolene ve filtru.
     */
    function getSQLFilter()
    {
        if (!$this->pravaZobrazeni->jsouNejakePrava()) {
            return '1 = 0';
        }

        $params = $this->database->createParameters();

        $result = $this->filterForm->buildSqlFilter($params);

        $datumOd = time();
        $datumDo = time();

        if (isset($this->sqlFilterDovednosti)) {
            $params->appendParameters($this->sqlFilterDovednosti->parameters);

            $filterDovednosti = '';

            if (!empty($this->filterForm->seznamTypuDovednosti)) {
                $filterDovednosti = SQLUtils::getINCondition($this->database, "Dovednost.TypDovednosti", $this->filterForm->seznamTypuDovednosti, $params);
            }

            if (!empty($this->filterForm->seznamDovednosti)) {
                if ($filterDovednosti !== '') {
                    $filterDovednosti .= " AND ";
                }
                $filterDovednosti .= SQLUtils::getINCondition($this->database,"Dovednost.Kod", $this->filterForm->seznamDovednosti, $params);

                if (!empty($this->filterForm->seznamUrovniDovednosti)) {
                    $dovednostiFiltrUrovne = '';

                    foreach ($this->filterForm->seznamUrovniDovednosti as $index => $urovenDovednosti) {
                        if ($urovenDovednosti) {
                            if ($dovednostiFiltrUrovne) {
                                $dovednostiFiltrUrovne .= ' AND ';
                            }
                            $dovednostiFiltrUrovne .=
                                " (EXISTS(SELECT DovednostOsoby.RC FROM DovednostOsoby
                                      WHERE DovednostOsoby.RC = Osoba.RC
                                        AND DovednostOsoby.DatumZiskani <= " . $params->addDateParameter(time()) .
                                "       AND (DovednostOsoby.PlatnostDo >= " . $params->addDateParameter(time()) . " OR DovednostOsoby.PlatnostDo IS NULL) " .
                                "       AND DovednostOsoby.KodDovednosti = " . $params->addParameter($this->filterForm->seznamDovednosti[$index]) .
                                "       AND DovednostOsoby.KodUrovne >= " .  $params->addParameter($urovenDovednosti) .
                                ' ))';
                        }
                    }

                    $filterDovednosti .= " AND " . $dovednostiFiltrUrovne;
                }
            }

            if (empty($filterDovednosti)) {
                $filterDovednosti = '1 = 1';
            }

            $result .= str_replace('%FILTER%', $filterDovednosti, $this->sqlFilterDovednosti->sqlText);
        }

        $result .=
            ($result ? ' AND ' : '') .
            'EXISTS(SELECT P.RC FROM PracovniPomer P' .
            ' WHERE (P.RC = Osoba.RC)' .
            '   AND (' . $this->pravaZobrazeni->sqlFiltr('P', time(), time(), $params) . ')' .
            "   AND (P.DatumNastupu <= " . $params->addDateParameter($datumDo) . ")" .
            "   AND ((P.DatumUkonceni >= " . $params->addDateParameter($datumOd) . ") OR (P.DatumUkonceni IS NULL))" .
            "\n";

        foreach ($this ->filterForm ->getRozdeleniFilters() as $rozdeleniFilter) {
            $filtrSql = $rozdeleniFilter ->sestavSQLFiltr($this ->database, 'P', $params);
            if ($filtrSql) {
                $result .=
                    ($result ? '   AND ' : '') .
                    $filtrSql .
                    "\n";
            }
        }

        $result .= ")\n";

//            (($this->filterForm->kodTypuRozdeleni && isset($this->filterForm->kodRozdeleni))
//                ? ' AND ' .
//                RozdeleniUtils::sqlFilterRozdeleniPP($this->database,
//                    'P', $this->filterForm->kodTypuRozdeleni, $this->filterForm->kodRozdeleni,
//                    time(), time(),
//                    $this->filterForm->vcetnePodrizenychRozdeleni,
//                    $params)
//                : '') .
//            (($this->filterForm->kodTypuRozdeleni2 && isset($this->filterForm->kodRozdeleni2))
//                ? ' AND ' .
//                RozdeleniUtils::sqlFilterRozdeleniPP($this->database,
//                    'P', $this->filterForm->kodTypuRozdeleni2, $this->filterForm->kodRozdeleni2,
//                    time(), time(),
//                    $this->filterForm->vcetnePodrizenychRozdeleni2,
//                    $params)
//                : '') .
//            ")\n";

        return $result
            ? new SqlStatement($result, $params)
            : '';
    }

    /** Nacte ziskane dovednosti osob dle zvolenych dovednosti ve filtru.
     * @return void
     * @throws DBException
     */
    public function loadZiskaneDovednosti()
    {
        $params = $this->database->createParameters();

        $filter = new SqlStatement(
            " EXISTS(SELECT DovednostOsoby.RC FROM DovednostOsoby JOIN Dovednost ON Dovednost.Kod = DovednostOsoby.KodDovednosti WHERE DovednostOsoby.RC = Osoba.RC " .
            "  AND DovednostOsoby.DatumZiskani <= " . $params->addDateParameter(time()) .
            "  AND (DovednostOsoby.PlatnostDo >= " . $params->addDateParameter(time()) . " OR DovednostOsoby.PlatnostDo IS NULL) " .
            "  AND %FILTER% " .
            //17.3.2017 RON-PR doplnena podminka jen na aktivni dovednosti
            "  AND Dovednost.Aktivni = 1)", $params);

        $this->sqlFilterDovednosti = $filter;

        $this->load();

        $this->loadPP();

        $dovednostiFilter = null;
        if (isset($this->filterForm->seznamDovednosti)) {
            $paramsFilter = $this->database->createParameters();
            $sql = SQLUtils::getINCondition($this->database, 'Dovednost.Kod', $this->filterForm->seznamDovednosti, $paramsFilter);
            $dovednostiFilter = new SqlStatement($sql, $paramsFilter);
        }

        if (!empty($this->rcPracPomeru)) {
            $this->loadZiskaneDovednostiOsoby($dovednostiFilter);
        }
    }

    /** Porovna nactene osoby dle dosazenych dovednosti. Pokud maji stejny pocet dosazenych dovednosti, radi se dle vyssich dosazenych urovni. V pripade shody se pak seradi dle prijmeni abecedne.
     * @param OsobaSDovednosti $osoba1
     * @param OsobaSDovednosti $osoba2
     * @return int
     */
    public function porovnejDovednost($osoba1, $osoba2)
    {
        if (count($osoba1->ziskaneDovednosti) == count($osoba2->ziskaneDovednosti)) {
            if ($osoba1->pocetZiskanychDovednosti == $osoba2->pocetZiskanychDovednosti) {
                return StringUtils::compareStrings($osoba1->prijmeni, $osoba2->prijmeni);
            }
            return ($osoba1->pocetZiskanychDovednosti > $osoba2->pocetZiskanychDovednosti) ? -1 : 1;
        }

        return (count($osoba1->ziskaneDovednosti) > count($osoba2->ziskaneDovednosti)) ? -1 : 1;
    }

    /** Seradi osoby dle poctu ziskanych dovednosti. Pokud maji stejny pocet dosazenych dovednosti, radi se dle vyssich dosazenych urovni. V pripade shody se pak seradi dle prijmeni abecedne.
     * @return void
     */
    public function seradOsoby()
    {
        uasort($this->osoby, array($this, "porovnejDovednost"));
    }

    /** Radky se zobrazuji dle serazeneho seznamu v $this->osoby.
     * @param $cursor
     * @return void
     * @throws Exception
     */
    public function renderCursor($cursor)
    {
        if (!$cursor) {
            throw new Exception('Chyba v metode browser::renderCursor - nejsou nactena data!');
        }

        $i = 1;

        $browserRowArray = new BrowserRowArray($this ->fields);

        //prochazim pole osob, ktere je serazeno dle poctu dosazenych dovednosti
        foreach ($this->osoby as $osoba) {
            //nacitam osoby z hlavniho SQL dotazu
            while ($row = $cursor->fetch()) {
                $browserRowArray->setData($row);

                //kdyz osoba z hlavniho SQL odpovida te ze serazenho pole, vypise se standardne radek
                if ($browserRowArray['col-rc'] == $osoba->getRC()) {
                    if ($this->enableLine($browserRowArray)) {
                        // Pozor! Pokud enableLine vypne nektere radky a bude zapnuto strankovani, tak se mohou nektere radky opakovat!
                        $this->renderLine($browserRowArray, ($i % 2) ? 'browsercolor1' : 'browsercolor2', $i);

                        $i++;
                    }
                }
            }
            $cursor->seek(0);
        }
        $cursor ->close();
    }

    /** Seznam osob se zobrazi az po stistku tlacitka Vyhledat. Pote jiz zustava zobrazen i pri pridani/odebrani filtru na dovednost.
     * @return void
     * @throws Exception
     */
    public function renderContent()
    {
        if (isset($_GET['find']) || isset($_GET['showdata'])) {
            parent::renderContent();
        }
    }

}