<?php
/**
 * stanislav.elbl, 27.02.2021
 */

namespace RonPortal\Editace;

use Exception;
use Ron\Controls\LinksList;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\MessageManager;
use Ron\ProxyArray;
use Ron\StringUtils;
use Ron\TextManager;
use Ron\XmlLoader;
use RonPortal\Dochazka\DenDochazky;
use RonPortal\Dochazka\DenniPoznamkaDochazky;
use RonPortal\Dochazka\DenniVysledek;
use RonPortal\Dochazka\DochazkaUtils;
use RonPortal\Dochazka\DokumentDochazky;
use RonPortal\Dochazka\MesicniVysledek;
use RonPortal\Dochazka\SchvaleniDochazky;
use RonPortal\Dochazka\SchvaleniDochazkyRecord;
use RonPortal\Dochazka\Smena;
use RonPortal\Dochazka\SmenaProDen;
use RonPortal\Dochazka\VysledekKontroly;
use RonPortal\Dochazka\ZaznamAbsence;
use RonPortal\Kalendar;
use RonPortal\Kapacity\KapacityUtils;
use RonPortal\Kapacity\PlanovaniKapacitRecord;
use RonPortal\Log;
use RonPortal\NastaveniUzivatele;
use RonPortal\NeniPravoException;
use RonPortal\Obdobi;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\PravaNaOsoby;
use RonPortal\RonPortalSettings;
use RonPortal\Rozdeleni;
use RonPortal\System\AgentManager;
use RonPortal\User;
use RonPortal\XML\RonPortalXMLForm;
use RonPortal\XML\XmlFormsList;
use Tracy\Debugger;
use Tracy\Dumper;

/**
 * Class DenZpracovaneDochazky
 * Data zpracovane dochazky pro jeden den
 */
class DenZpracovaneDochazky extends DenDochazky
{
    public $typDne;

    /** @var bool */
    public $kalendarPodleKapacit;
    /** @var SmenaProDen        informace o smene podle kalendare */
    public $smenaPodleKalendare;
    /** @var SmenaProDen        informace o smene podle kapacit */
    public $smenaPodleKapacit;

    /** @var array      Ciselne hodnoty */
    public $hodnoty;
    /** @var ProxyArray     Textove hodnoty */
    public $texty;
    /** @var ProxyArray     Puvodni textove hodnoty (pri editaci) */
    public $textyPuvodni;
    /** @var bool[]     Priznak, jestli je v policku chyba (spatne zadana hodnota) */
    public $chyby;
    /** @var string[]   Texty pro zobrazeni hintu u sloupcu (title) */
    public $hinty;

    /** @var string   class radku - nastavuje se napr. podle absenci */
    public $cssClass;
    /** @var int   barva radku - nastavuje se podle kontrol */
    public $barvaRadku;


    function __construct($datum, $typDne)
    {
        parent::__construct($datum);
        $this ->typDne = $typDne;

        $this ->hodnoty = [];
        $this ->texty = new ProxyArray();
        $this ->textyPuvodni = new ProxyArray();
        $this ->chyby = [];
        $this ->hinty = [];

        $this ->cssClass = '';
    }
}




class TlacitkoZpracovaneDochazky
{
    /**
     * @var string  tlacitka: ulozit, prepocitat, schvalit, zrusitschvaleni, vlozitprestavky, podlepuvodni, skript, xml, interval
     *              linky: rozdelenidv
     *              pk_zrusitsmenu, pk_zmenitsmenu
     *              TODO: smazat, storno, link, tisk
     */
    public $typ;

    /** @var string     zobrazeny text */
    public $text;
    /** @var string     nepovinny titulek (hint) */
    public $title;
    /** @var string     nepovinny styl pro zobrazeni */
    public $styl;
    /** @var string     nepovinny dotaz, ktery se ma zobrazit pri kliknuti */
    public $dotaz;

    /** @var int        1=vlevo nahore, 2=vpravo nahore, 3=vlevo dole, 4=vpravo dole, 5=ve skryvacim panelu (po oznaceni dne) */
    public $umisteni;

    /** @var string     Omezi zobrazeni tlacitka jen pro zamestnance se zandanymi kalendari */
    public $kalendar;
    /** @var string     Omezeni zobrazeni tlacitka jen pro uzivatele se zadanymi profily */
    public $profil;

    /** @var bool       Priznak urcuje, jestli se ma zpracovat presah do dalsiho mesice (tj. prislusna uloha se spusti az do prvniho dne nasledujiciho mesice) - pouze pro nektere ulohy! */
    public $zpracovatPresah = false;

    /** @var string     pro skript a podlekalendare */
    public $id;

    // pro typ script
    public $scriptCategory;
    public $scriptName;

    /** @var int        ID xml formulare - pro typ xml */
    public $xmlId;

    // pro schvaleni
    public $urovenSchvaleni;

    /** @var string     Kod operace - pro typ podlekalendare, interval */
    public $operace;
    /** @var string     Kod smeny - pro typ interval */
    public $smena;

    // pro typ podlekalendare
    public $agentParameters;

    // pro typ interval
    public $casOd;
    public $casDo;
    /** @var int    priznak, jestli se ma smazat vsechno ostatni v danem dni */
    public $smazatVse;

    // pro pk_zmenitsmenu, pk_zrusitsmenu
    public $prepocitat;



    /**
     * doplni defaultni hodnoty
     */
    function doplnHodnoty()
    {
        switch ($this ->typ) {
            case 'ulozit' :
                $text = TextManager::_('form.Store');
                break;

            case 'storno' :
                $text = TextManager::_('Cancel');
                break;

            case 'prepocitat' :
            case 'prepocet' :
                $text = TextManager::_('Prepocitat');
                break;

            case 'kontrola' :
                $text = TextManager::_('Kontrola_dochazky');
                break;

            case 'schvaleni' :
            case 'schvalit' :
                $text = TextManager::_('Schvalit');
                break;

            case 'dialogschvaleni' :
                $text = TextManager::_('Schvalit_dochazku');
                break;

            case 'zrusitschvaleni' :
                $text = TextManager::_('Zrusit_schvaleni');
                break;

            case 'podlepuvodni' :
                $text = TextManager::_('Podle_puvodni_dochazky');
                break;

            case 'doplnitpuvodni' :
                $text = TextManager::_('Doplnit_puvodni');
                break;

            case 'vlozitprestavky' :
                $text = TextManager::_('Vlozit_prestavky');
                break;

            case 'rozdelenidv' :
                $text = TextManager::_('Rozdeleni_dv_link');
                break;

            case 'rozdelenimv' :
                $text = TextManager::_('Rozdeleni_mv_link');
                break;

            case 'rozdelenivyroby' :
                $text = TextManager::_('Rozdeleni_vyroby_link');
                break;

            case 'vlozitdokument' :
                $text = TextManager::_('Vlozit_dokument_k_dochazce');
                break;

            case 'podlekalendare' :
                $text = TextManager::_('Podle_kalendare');
                $this ->umisteni = $this ->umisteni ?: 5;
                break;

            case 'pk_zrusitsmenu' :
                $text = TextManager::_('Zrusit_naplanovanou_smenu');
                break;

            case 'pk_zmenitsmenu' :
                $text = TextManager::_('Zmenit_naplanovanou_smenu');
                break;
//
            case 'kalendarosoby' :
                $text = TextManager::_('Prirazeni_kalendare');
                break;

            case 'vymazatvse' :
                $text = TextManager::_('Vymazat_data_dochazky');
                if (!isset($this ->dotaz)) {
                    $this->dotaz = TextManager::_('Opravdu_vymazat_data_dochazky_za_cele_obdobi');
                }
                break;

            case 'dialogpodlekalendare' :
                $text = TextManager::_('Podle_kalendare');
                break;

            default :
                $text = null;

//                case 'smazat' :
//                    $this ->text = TextManager::_('Smazat_oznacene_dny');
//                    break;
        }

        if (!$this ->text) {
            $this ->text = $text;
        }

        switch ($this ->umisteni) {
            case 1 :
            case 2 :
            case 4 :
            case 5 :  // v panelu po vyberu
            case -1 : // skryt
                break;

            default :
                $this ->umisteni = 3; // defaultne vlevo dole
        }

    }


    /**
     * Nacte nastaveni z asociativniho pole (nactene z XML)
     * @param array $settings
     */
    function loadSettings($settings)
    {
        $this ->typ = $settings['typ'];
        $this ->text = TextManager::_($settings['text']);
        $this ->title = TextManager::_($settings['title']);
        $this ->styl = $settings['style'];
        $this ->umisteni = (int)$settings['umisteni'];
        $this ->scriptCategory = $settings['kategorie'];
        $this ->scriptName = $settings['skript'];
        $this ->xmlId = $settings['xmlid'];
        $this ->id = $settings['id'];
        $this ->dotaz = TextManager::_($settings['dotaz']);
        // parametry? idpp, obdobi
        $this ->urovenSchvaleni = (int)$settings['uroven']; // pro tlacitka schvalit a zrusitschvaleni

        $this ->operace = $settings['operace']; // pro podlekalendare
        $this ->smena = $settings['smena'] ?: null;
        $this ->agentParameters = $settings['parametry'];

        $this ->casOd = $settings['casod'];
        $this ->casDo = $settings['casdo'];
        $this ->smazatVse = (int)$settings['smazatvse'];

        $this ->prepocitat = (int)$settings['prepocitat'];
        $this ->zpracovatPresah = $settings['zpracovatpresah'] == '1'; // pro podlepuvodni a doplnitpuvodni

        $this ->kalendar = $settings['kalendar'];
        $this ->profil = $settings['profil'];

        if ($this ->kalendar  &&  false !== strpos($this ->kalendar, ';')) {
            // je zadano vice kodu kalendaru oddelenych strednikem
            $this ->kalendar = explode(';', $this ->kalendar);
        }

        $this ->doplnHodnoty();
    }
} // TlacitkoZpracovaneDochazky



//class XMLZpracovaneDochazky
//{
//    public $idXmlFormulare;
//    /** @var string     Kde se ma formular zobrazit (top/bottom) */
//    public $umisteni = 'top'; // "top"
//
//    /** @var string     Obsah, ktery se ma zobrazit */
//    public $obsah = '';
//}


class DataZpracovaneDochazky
{
    /** @var Database */
    public $database;

    /** @var PracovniPomer */
    public $pp;

    /* @var string      Kalendar osoby */
    public $kalendar;

    public $datumOd;
    public $datumDo;

    /** @var SloupecZpracovaneDochazky[] */
    public $sloupce;
    /** @var LinksList */
    public $filterLinks;

    /** @var XmlFormsList  Seznam xml formularu */
    public $xmlFormulare = null;

    /** @var DenZpracovaneDochazky[] f_data */
    public $f_data;

    /** @var Kalendar[]     Kalendare prirazene zamestnanci v zadanem obdobi */
    public $prirazeneKalendare = null;

    /* @var TlacitkoZpracovaneDochazky[] tlacitka */
    public $tlacitka;

    public $grafickaMinWidth = 200;
    public $grafickaMaxWidth = 600;

    /* @var SchvaleniDochazky schvaleni */
    public $schvaleni;
    /** @var bool Jestli uzivatel zadal nejakou chybnou hodnotu */
    public $jeChyba;

    /** @var PravaNaOsoby */
    public $pravaEditace;
    /** @var bool   Priznak, jestli muze uzivatel editovat osobu. POZOR! Kazdy sloupec muze mit sve vlastni pravo! */
    public $jePravoEditovat;
    /** @var bool   Priznak, jestli muze uzivatel editovat poznamku. */
    public $jePravoEditovatPoznamku;

    public $obdobiUzavreno;
    public $dochazkaZpracovanaDo;

    public $datumPrvniZmeny;  // zapamatuje si, kdy doslo k prvni zmene, aby mohl provest prepocet
    public $jeZmenaVMesicnich; // priznak, jestli doslo ke zmene mesicnich vysledku

    public $editovatVzdyCelyDen; // jestli se ma i pri kliknuti na interval editovat cely den
    public $vkladatPrestavky; // jestli ma pri prepoctu vkladat prestavky
    public $absenceClass;     // priznak, jestli pro radky (tr) nastavit class podle naplanovanych absenci
    public $nacitatKontroly;  // jestli se maji nacitat vysledky kontrol (tabulka VysledekKontrolyDochazky)
    public $obarvitKontroly;     // jestli se ma podbarvit cely radek s kontrolami
    /** @var int    Priznak, jestli se maji zobrazit mesicni kontroly (0=ne, 1 (default) = dole, 2 = nahore, 3 = dole i nahore */
    public $zobrazitMesicniKontroly;

    /** @var string[] Seznam casovych slozek, pro ktere se ma na radku generovat class (dv_<kodCasoveSlozky), pokud je v dany den nenulova */
    public $classDV;

    /** @var  VysledekKontroly[] */
    public $kontroly;

    /** @var string Styl nastaveny pomoci tagu <style> */
    public $style;


    function __construct($database) // , $errorsManager)
    {
        $this ->database = $database;

        $this ->sloupce = [];
        $this ->filterLinks = new LinksList();
        $this ->filterLinks ->noGroupInParam = false;
        $this ->filterLinks ->wrapElement = 'span';
        $this ->filterLinks ->itemsIdPrefix = 'zpracovana_filtrsloupce_';
        $this ->tlacitka = [];
        $this ->f_data = [];
        $this ->f_poznamka = false;

        $this ->classDV = [];

        $this ->editovatVzdyCelyDen = false;
        $this ->zobrazitMesicniKontroly = 1; // zobrazit dole

        $this ->kontroly = [];

        $this ->xmlFormulare = new XmlFormsList();
    }


    /**
     * @param PracovniPomer $pp
     */
    function nastavOsobu($pp)
    {
        $this ->pp = $pp;
    }


    /**
     * @param string $typ
     * @return bool
     */
    function existujeTlacitko($typ)
    {
        foreach ($this ->tlacitka as $tlacitko) {
            if ($tlacitko ->typ == $typ) {
                return true;
            }
        }

        return false;
    }


    /**
     * @param string $typ
     */
    function doplnZakladniTlacitko($typ)
    {
        if (!$this ->existujeTlacitko($typ)) {
            $tlacitko = new TlacitkoZpracovaneDochazky();
            $tlacitko ->typ = $typ;
            $tlacitko ->doplnHodnoty();
            $this ->tlacitka[] = $tlacitko;
        }
    }


    /**
     * @param string $xmlText
     */
    function nactiNastaveniXML($xmlText)
    {
        MessageManager::addDebugLog("Zpracovana dochazka - nacitam nastaveni:\n" . StringUtils::xmlTextConvert($xmlText));

        $this ->tlacitka = [];
        $this ->sloupce = [];

        //$settings = XmlLoader::loadXmlToArray($xmlText);
        $settings = XmlLoader::loadXmlToArray($xmlText, [
                XmlLoader::PARAM_CONVERT_FROM_UTF8 => true,
                XmlLoader::PARAM_INCLUDE_ATTRIBUTES => true,
                XmlLoader::PARAM_RAW_DATA_ELEMENT => 'html'
            ]
        );

        if (isset($settings['translation'])) {
            TextManager::getInstance() ->loadArray($settings['translation']);
        }

        $this ->editovatVzdyCelyDen = $settings['#attr']['editovatvzdycelyden'] == '1';
        $this ->vkladatPrestavky = $settings['#attr']['vkladatprestavky'] == '1';
        $this ->absenceClass = $settings['#attr']['absenceclass'] == '1';
        $this ->nacitatKontroly = $settings['#attr']['kontroly'] != '0'; // 21.10.2016 - defaultni hodnota zmenena na true, aby fungovalo i bez licence EDIT

        $obarvitKontroly = $settings['#attr']['obarvitkontroly'];
        $this ->obarvitKontroly = ($obarvitKontroly == '')  ||  $obarvitKontroly == '1';

        $mesicniKontroly = $settings['#attr']['mesicnikontroly'];
        $this ->zobrazitMesicniKontroly = ($mesicniKontroly != '') ? (int)$mesicniKontroly : 1; // defaultne zobrazit dole

        $this ->classDV = explode(';', strtolower($settings['#attr']['classDV'] ?? '') );

        $pom = $settings['#attr']['grafickaminwidth'];
        if ((int)$pom) {
            $this ->grafickaMinWidth = (int)$pom;
        }

        $pom = $settings['#attr']['grafickamaxwidth'];
        if ((int)$pom) {
            $this ->grafickaMaxWidth = (int)$pom;
        }

        if (isset($settings['sloupeczpracovanedochazky'])) {
            foreach ($settings['sloupeczpracovanedochazky'] as $settingsSloupec) {
                $sloupec = new SloupecZpracovaneDochazky();
                $sloupec ->loadSettings($settingsSloupec['#attr']);
                $this ->sloupce[] = $sloupec;
            }
        }

        if (isset($settings['tlacitko'])) {
            foreach ($settings['tlacitko'] as $settingsTlacitko) {
                $tlacitko = new TlacitkoZpracovaneDochazky();
                $tlacitko ->loadSettings($settingsTlacitko['#attr']);
                $this ->tlacitka[] = $tlacitko;
            }
        }

        if (isset($settings['style'][0])) {
            $this ->style = $settings['style'][0]['#text'];
        }

        if (isset($settings['xml'])) {
            $this ->xmlFormulare ->loadSettings($settings['xml']);
        }

        $this ->doplnZakladniTlacitko('ulozit');
        $this ->doplnZakladniTlacitko('prepocitat');
        $this ->doplnZakladniTlacitko('vlozitprestavky');
        $this ->doplnZakladniTlacitko('dialogschvaleni');
    }


    /**
     * prida sloupce podle retezce s nastavenim ve formatu sloupec1;sloupec2;...
     * @param $sloupcestr
     * @return int
     * @deprecated
     */
    function pridejSloupce($sloupcestr)
    {
        $result = 0;

        $poma = explode(';', $sloupcestr);

        foreach ($poma as $nastavenisloupce) {
            if ($nastavenisloupce) {
                $sloupec = new SloupecZpracovaneDochazky();
                $sloupec ->nastavPodleRetezce($nastavenisloupce);
                $this ->sloupce[] = $sloupec;

                $result++;
            }
        }

        return $result;
    }


    function vychoziNastaveni()
    {
        $this ->pridejSloupce('SCHVAL1;SCHVAL2;SCHVAL3;DATUM');

        $sloupec = new SloupecZpracovaneDochazky();
        $sloupec ->typ = 'DOCH_G';
        $sloupec ->zobrazitSmenu = true;
        $this ->sloupce[] = $sloupec;

        $this ->pridejSloupce('DOCH_T;EDIT');

        $this ->tlacitka = [];
        $this ->doplnZakladniTlacitko('ulozit');
        $this ->doplnZakladniTlacitko('prepocitat');
        $this ->doplnZakladniTlacitko('vlozitprestavky');
        $this ->doplnZakladniTlacitko('dialogschvaleni');
    }


    function inicializujSloupce()
    {
        // skryje sloupce, ktere uzivatel nema videt (omezeni na profil, prava nebo rozdeleni zobrazeneho zamestnance)
        foreach ($this ->sloupce as $sloupec) {
            // ma zadany profil ...
            if ($sloupec ->lzeZobrazit  &&  $sloupec ->profil) {
                $sloupec ->lzeZobrazit = User::jeVProfilu($sloupec ->profil);
            }

            // ... nebo pravo ...
            if ($sloupec ->lzeZobrazit  &&  $sloupec ->pravo) {
                $sloupec ->lzeZobrazit = Prava::lzeZobrazitPP($sloupec ->pravo, $this ->pp ->idPP, $this ->datumOd, $this ->datumDo);
            }

            // ... nebo rozdeleni
            if ($sloupec ->lzeZobrazit  &&  $sloupec ->filtrRozdeleni) {
                if (!Rozdeleni::jeVRozdeleniStr($this ->database, null, $this ->pp ->idPP, $sloupec ->filtrRozdeleni, 3, $this ->datumOd, $this ->datumDo)) {
                    $sloupec ->lzeZobrazit = false;
                }
            }

            if ($sloupec ->lzeZobrazit && $sloupec ->filtrKey) {
                $this ->filterLinks ->addOrUpdateItem($sloupec ->filtrKey, $sloupec ->filtrCaption, $sloupec ->filtrGroup);
            }
        }

        // nastavi viditelnost sloupcu podle nastaveni uzivatele
        if (!$this ->filterLinks ->isEmpty()) {
            $oldSettingsStr = NastaveniUzivatele::getUserSettingsItem($this ->database, 'zpracovana_sloupce');
            $this ->filterLinks ->loadSettingString($oldSettingsStr);
            $this ->filterLinks ->loadGet();
            $newStr = $this ->filterLinks ->getSettingsString();
            if ($newStr != $oldSettingsStr) {
                NastaveniUzivatele::setUserSettingsItem($this ->database, 'zpracovana_sloupce', $newStr, '');
            }

            foreach ($this->sloupce as $sloupec) {
                if ($sloupec->filtrKey && $sloupec->lzeZobrazit) {
                    if (!$this->filterLinks->itemIsActive($sloupec->filtrKey)) {
                        $sloupec->lzeZobrazit = false;
                    }
                }
            }
        }
    }


    /**
     * vraci true, pokud je mozne editovat sloupec
     * kontroluje nastaveni sloupce, schvaleni dochazky a prava
     * @param DenZpracovaneDochazky $radek
     * @param SloupecZpracovaneDochazky $sloupec
     * @return bool
     */
    function lzeEditovat($radek, $sloupec)
    {
        // kontrola nastaveni sloupce
        if (!$sloupec ->lzeEditovat) {
            return false;
        }

        // kontrola obdobi
        if ($this ->obdobiUzavreno) {
            return false;
        }

        if (!RonPortalSettings::getIntValue(RonPortalSettings::EDITACE_POVOLENA)) {
            // v modulu PLUS lze konfigurovat sloupce, ale nejde editovat
            return false;
        }

        if ($sloupec ->pravoEditace) {
            // sloupec ma vlastni pravo na editaci
            if (!Prava::lzeZobrazitPP($sloupec ->pravoEditace, $this ->pp ->idPP, $radek ->datum)) {
                return false;
            }
        } else {
            if (!$this->jePravoEditovat  ||  !$this ->pravaEditace ->lzeZobrazitPP($this ->pp ->idPP, $radek ->datum)) {
                // poznamka muze mit vlastni pravo
                if ($sloupec->typ == 'P') {
                    if (!$this->jePravoEditovatPoznamku  ||  !Prava::lzeZobrazitPP('editacepozn', $this ->pp ->idPP, $radek ->datum)) {
                        return false;
                    }
                } else {
                    return false;
                }
            }
        }

        // kontrola schvaleni
        if (!$this ->schvaleni ->lzeEditovatPP($this ->pp ->idPP, $radek ->datum)) {
            return false;
        }

        // kontrola zpracovani dochazky
        if (!$sloupec ->editovatNezpracovanou  &&  $radek ->datum > $this ->dochazkaZpracovanaDo) {
            return false;
        }

        if ($radek ->datum < $this ->pp ->datumNastupu) {
            return false;
        }
        if ($this ->pp ->datumUkonceni  &&  $radek ->datum > $this ->pp ->datumUkonceni) {
            return false;
        }

        if ($sloupec ->typ == 'DOCH1' || $sloupec ->typ == 'DOCH2') {
            // kontrola, jestli nema v pravech omezen pocet dnu
            if ($this ->pravaEditace ->priznak2 && DateUtils::incDayStart($radek ->datum, $this ->pravaEditace ->priznak2) < DateUtils::startOfTheDay(time())) {
                return false;
            }
        }

        return true;
    }


    /**
     * Vraci true, pokud existuje sloupec se zadanym typem (nebo s jednim ze zadanych typu v pripade ze $typ je pole)
     * @param string|string[] $typ
     * @return bool
     */
    function existujeSloupec($typ): bool
    {
        foreach ($this ->sloupce as $sloupec) {
            if ($sloupec->lzeZobrazit) {
                if (is_array($typ)) {
                    if (in_array($sloupec ->typ, $typ)) {
                        return true;
                    }
                } else if ($sloupec->typ === $typ) {
                    return true;
                }
            }
        }

        return false;
    }



    /**
     * @return bool   Vraci true, pokud je pro nektery sloupec potreba nacist puvodni dochazku
     */
    function jeSloupecSPuvodni()
    {
        foreach ($this ->sloupce as $sloupec) {
            if ($sloupec ->lzeZobrazit) {
                switch ($sloupec->typ) {
                    case 'PUV' :
                    case 'PUV1' :
                    case 'PUV2' :
                        return true;

                    case 'DOCH1' :
                    case 'DOCH2' :
                        if ($sloupec->zobrazitPuvodni) {
                            return true;
                        }
                        break;
                }
            }
        }

        return false;
    }


    function jeSloupecSPruchody()
    {
        foreach ($this ->sloupce as $sloupec) {
            if ($sloupec ->lzeZobrazit) {
                switch ($sloupec->typ) {
                    case 'PRUCH' :
                    case 'PRUCH1' :
                    case 'PRUCH2' :
                        return true;

                    case 'DOCH1' :
                    case 'DOCH2' :
                        if ($sloupec->zobrazitPruchody) {
                            return true;
                        }
                        break;
                }
            }
        }

        return false;
    }


    /**
     * @return bool  Vraci true, pokud je potreba nacist absence
     */
    function nacitatAbsence()
    {
        if ($this ->absenceClass) {
            // radky maji mit priznak podle absenci
            return true;
        }

        foreach ($this ->sloupce as $sloupec) {
            if ($sloupec ->lzeZobrazit  &&  $sloupec->typ == 'ABSENCE') {
                return true;
            }
        }

        return false;
    }


    /**
     * Vytvori zaznamy pro jednotlive dny obdobi
     * @param int  $datumOd
     * @param int  $datumDo
     * @return bool
     */
    function inicializujData($datumOd, $datumDo)
    {
        $this ->datumOd = $datumOd;
        $this ->datumDo = $datumDo;

        $datum = DateUtils::startOfTheDay($datumOd);
        $poslednidatum = $datumDo;

        $this ->f_data = [];

        // Pro kazdy den vytvori radek
        while ($datum <= $poslednidatum) {
            $typdne = Kalendar::vratTypDne($this ->database, $datum, true, $this ->kalendar); // nacist svatky = true

            $radek = new DenZpracovaneDochazky($datum, $typdne);
            $this ->f_data[] = $radek;

            $datum = DateUtils::startOfTheDay(DateUtils::incDay($datum, 1));
        }

        return true;
    }


    /**
     * @param int $datum
     * @return int
     */
    function indexRadku($datum)
    {
        $datum = DateUtils::startOfTheDay($datum);

        foreach ($this ->f_data as $index => $radek) {
            if ($datum == $radek ->datum) {
                return $index;
            }

            if ($datum < $radek ->datum) {
                return -1;
            }
        }

        return -1;
    }


    /**
     * Vraci RadekDochazky z pole $f_data pro zadane datum
     * @param int $datum
     * @return DenZpracovaneDochazky|null
     */
    function najdiRadekZpracovaneDochazky($datum)
    {
        $index = $this ->indexRadku($datum);
        if ($index < 0) {
            return null;
        }

        return $this ->f_data[$index];
    }


    /**
     * Nastavi text zadaneho sloupce pro zadany den
     * @param int    $datum
     * @param int    $indexSloupce
     * @param string $text
     * @return bool
     */
    function nastavText($datum, $indexSloupce, $text)
    {
        $radek = $this ->najdiRadekZpracovaneDochazky($datum); // objekt radku
        if (!$radek) {
            return false;
        }

        $radek ->texty[$indexSloupce] = $text;
        $radek ->textyPuvodni[$indexSloupce] = $text;
        $radek ->chyby[$indexSloupce] = false;
        return true;
    }


    /**
     * Nastavi hodnotu (text) sloupce zadaneho typu pro zadany datum
     * @param int  $datum
     * @param string $typSloupce
     * @param string $text
     * @param string $hint
     * @param bool   $pouzeNeprazdny    Pokud je true, tak neprepisuje drive nastavenou hodnotu
     */
    function nastavTextSloupce($datum, $typSloupce, $text, $hint = null, $pouzeNeprazdny = false): void
    {
        $radek = $this ->najdiRadekZpracovaneDochazky($datum); // objekt radku
        if (!$radek) {
            return;
        }

        foreach ($this ->sloupce as $indexSloupce => $sloupec) {
            if ($sloupec ->lzeZobrazit  &&  $sloupec ->typ == $typSloupce) {
                if (!$pouzeNeprazdny || !$radek ->texty[$indexSloupce]) {
                    $radek->texty[$indexSloupce] = $text;
                    $radek->textyPuvodni[$indexSloupce] = $text;
                    $radek->chyby[$indexSloupce] = false;
                    $radek->hinty[$indexSloupce] = $hint;
                }
            }
        }
    }


    private function formatujText($format, $data)
    {
        return str_replace(array_keys($data), array_values($data), $format);
    }

    /**
     * @param int $datum
     * @param string $typSloupce
     * @param array $data
     * @param bool $hromadne
     * @param string $defaultFormat
     * @return void
     */
    private function nastavFormatovanyText($datum, $typSloupce, $data, $hromadne, $defaultFormat = '%CAS% - %POPIS%')
    {
        $radek = $this ->najdiRadekZpracovaneDochazky($datum); // objekt radku
        if (!$radek) {
            return;
        }

        foreach ($this ->sloupce as $indexSloupce => $sloupec) {
            if ($sloupec ->lzeZobrazit  &&  $sloupec ->typ == $typSloupce) {
                $format = $sloupec ->format ?: $defaultFormat;
                if ($hromadne) {
                    $text = '';
                    foreach ($data as $item) {
                        $text .=
                            ($text ? ', ' : '') .
                            $this ->formatujText($format, $item);
                    }
                } else {
                    $text = $this ->formatujText($format, $data);

                    if ($sloupec ->titleFormat) {
                        $radek->hinty[$indexSloupce] = $this ->formatujText($sloupec ->titleFormat, $data);
                    }
                }

                $radek->textyPuvodni[$indexSloupce] = $radek->texty[$indexSloupce] = $text;
                $radek->chyby[$indexSloupce] = false;
            }
        }
    }


    /**
     * Nastavi hodnotu ve sloupcich DOCH1 a DOCH2 podle nactene dochazky
     */
    function nastavPrichodOdchod()
    {
        foreach ($this ->f_data as $radek_original) {
            foreach ($radek_original ->zaznamy as $zaznam) {
                /** @public $radek DenZpracovaneDochazky|null  -  radek, do ktereho se zaznam zapocitava (muze byt jiny nez $radek_original) */
                $radek = $this ->najdiRadekZpracovaneDochazky($zaznam ->den);

                if ($radek) {

                    foreach ($this ->sloupce as $indexsloupce => $sloupec) {
                        if ( $sloupec ->lzeZobrazit  &&  ($sloupec ->typ == 'DOCH1' || $sloupec ->typ == 'DOCH2') ) {
                            if ($this->operaceJeVSeznamu($zaznam->kodOperace, $sloupec->zapocitatOperace)) {
                                if ($sloupec->typ == 'DOCH1') {
                                    if (!$radek->texty[$indexsloupce]) { // pokud se ma zobrazit
                                        if ($sloupec->zobrazenyCas || $zaznam ->upravaOd != $zaznam ->upravaDo) { // pokud se ma zobrazit jen zapocteny cas, tak ignoruje intervaly nulove delky
                                            // pouze pokud jeste neni nastaven (pocita se prvni interval)
                                            if ($sloupec->zobrazenyCas) {
                                                $radek->texty[$indexsloupce] = date('G:i', $zaznam->datumCasPocatku);
                                            } else {
                                                $radek->texty[$indexsloupce] = $zaznam->textCasOd;
                                            }
                                            $radek->textyPuvodni[$indexsloupce] = $radek->texty[$indexsloupce];
                                            $radek->chyby[$indexsloupce] = false;
                                        }
                                    }
                                } else if ($sloupec->typ == 'DOCH2') {
                                    if ($sloupec->zobrazenyCas || $zaznam ->upravaOd != $zaznam ->upravaDo) { // pokud se ma zobrazit jen zapocteny cas, tak ignoruje intervaly nulove delky
                                        if ($sloupec->zobrazenyCas) {
                                            $radek->texty[$indexsloupce] = date('G:i', $zaznam->datumCasUkonceni);
                                        } else {
                                            $radek->texty[$indexsloupce] = $zaznam->textCasDo;
                                        }
                                        $radek->textyPuvodni[$indexsloupce] = $radek->texty[$indexsloupce];
                                        $radek->chyby[$indexsloupce] = false;
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
    }


    /**
     * Nastavi text sloupcu s typem DOCH_Z podle nactenych zaznamu zpracovane dochazky
     * (tj. textovy popis intervalu v jednom sloupci, oddeleno carkami
     */
    function nastavDochZ()
    {
        foreach($this ->sloupce as $index => $sloupec) {
            if ($sloupec ->lzeZobrazit  &&  $sloupec ->typ == 'DOCH_Z') {
                foreach ($this ->f_data as $radek) {
                    $text = '';

                    foreach ($radek ->zaznamy as $zaznam) {
                        $pomstr = $zaznam ->format($sloupec ->format);
                        if ($pomstr != '') {
                            $text .= ($text ? ', ' : '') . $pomstr;
                        }
                    }

                    $radek ->texty[$index] = $text;
                    $radek ->textyPuvodni[$index] = $text;
                    $radek ->chyby[$index] = false;
                }
            }
        }
    }


    /**
     * Nacte dochazku
     * @return void
     */
    function nactiOperaceDochazky()
    {
        if (!$this ->pp  ||  $this ->pp ->rc === null) {
            // kvuli firebird
            return;
        }

        MessageManager::addDebugMessage('Nacitam operace dochazky z databaze ...');

        DenDochazky::nactiZpracovanouDochazku($this ->database,
            $this ->f_data, $this ->pp,
            NastaveniUzivatele::getUserSettingsItem($this ->database, 'zobrazitpoznamkyzpracovane'));

        $this ->nastavPrichodOdchod(); // TODO: tohle asi bude lepsi volat pro kazdy zaznam samostatne uz pri nacitani, aby se spravne nastavila data i na prelomu mesice
        $this ->nastavDochZ(); // sloupec DOCH_Z (tj. zaznamy zpracovane dochazky)
    }


    /**
     * @param DenniVysledek $denniVysledek
     * @param int $indexSloupce
     * @internal param array $dbRow
     */
    function nastavSloupecDV($denniVysledek, $indexSloupce)
    {
        $radek = $this->najdiRadekZpracovaneDochazky($denniVysledek ->den); // objekt radku

        $sloupec = $this->sloupce[$indexSloupce];

        if ($sloupec->lzeZobrazit && $radek) {
            $radek ->hodnoty[$indexSloupce] = $denniVysledek ->vratHodnotu($sloupec ->jednotka);
            $radek ->texty[$indexSloupce] = $denniVysledek ->vratText($sloupec ->jednotka, null, $sloupec ->zobrazitNuloveHodnoty);

            $radek->textyPuvodni[$indexSloupce] = $radek->texty[$indexSloupce];
            $radek->chyby[$indexSloupce] = false;
        }
    }


    function nactiDenniVysledky()
    {
        $vysledky = DenniVysledek::nactiDenniVysledky($this ->database, $this ->pp, $this ->datumOd, $this ->datumDo);

        foreach ($vysledky as $denniVysledek) {
            // nastavi hodnoty ve sloupcich
            foreach ($this ->sloupce as $i => $sloupec)
                if ($sloupec ->lzeZobrazit  &&  ($sloupec ->typ == 'DV' || $sloupec ->typ == 'DV_CHECK') && $sloupec ->kodCasoveSlozky === $denniVysledek ->typCasoveSlozkyMzdy) {
                    $this ->nastavSloupecDV($denniVysledek, $i);
                }

            // nastavi class na radku, pokud je nektera hodnota nenulova a casova slozka je v nastaveni v classDV
            if ($denniVysledek ->hodiny  ||  $denniVysledek ->dny  ||  $denniVysledek ->kalendarniDny) {
                if ( in_array(strtolower($denniVysledek ->typCasoveSlozkyMzdy), $this ->classDV) ) {
                    $radek = $this ->najdiRadekZpracovaneDochazky($denniVysledek ->den);
                    $radek ->cssClass .= ($radek ->cssClass ? ' ' : '') . 'DV_' . $denniVysledek ->typCasoveSlozkyMzdy;
                }
            }
        }
    }


    function nactiSchvaleni()
    {
        $this ->schvaleni = new SchvaleniDochazky();
        $this ->schvaleni ->load($this ->database,
            $this ->pp ->rc, $this ->pp ->osobniCislo, $this ->pp ->datumNastupu,
            $this ->datumOd, $this ->datumOd); // datumOd i pro konec, aby se zbytecne na ODBC nezobrazovaly ansi warning "null value eliminated"
    }


    function nactiPoznamky()
    {
        $poznamky = DenniPoznamkaDochazky::nactiPoznamkyDochazky($this ->database, $this ->datumOd, $this ->datumDo, $this ->pp ->idPP);
        foreach ($poznamky as $poznamka) {
            $this ->nastavTextSloupce($poznamka ->den, 'P', $poznamka ->poznamka);
        }
    }


    /**
     * @param int $datum
     * @param string $textprvni
     * @param string $textposledni
     * @param string $textvsechny
     * @return bool
     */
    function nastavTextyPuvodni($datum, $textprvni, $textposledni, $textvsechny)
    {
        $this -> nastavTextSloupce($datum, 'PUV1', $textprvni);
        $this -> nastavTextSloupce($datum, 'PUV2', $textposledni);
        $this -> nastavTextSloupce($datum, 'PUV', $textvsechny);

        $radek = $this ->najdiRadekZpracovaneDochazky($datum); // objekt radku
        if (!$radek) {
            return false;
        }

        foreach ($this ->sloupce as $indexsloupce => $sloupec) {
            if ($sloupec ->typ == 'DOCH1'  ||  $sloupec ->typ == 'DOCH2') {
                if ($sloupec ->lzeZobrazit  &&  $sloupec ->zobrazitPuvodni) {
                    $radek ->hinty[$indexsloupce] = $textvsechny;
                }
            }
        }

        return true;
    }


    /**
     * @param int $datum
     * @param string $textPrvni
     * @param string $textPosledni
     * @param string $textVsechny
     * @return bool
     */
    function nastavTextyPruchody($datum, $textPrvni, $textPosledni, $textVsechny)
    {
        $this -> nastavTextSloupce($datum, 'PRUCH1', $textPrvni);
        $this -> nastavTextSloupce($datum, 'PRUCH2', $textPosledni);
        $this -> nastavTextSloupce($datum, 'PRUCH', $textVsechny);

        $radek = $this ->najdiRadekZpracovaneDochazky($datum); // objekt radku
        if (!$radek) {
            return false;
        }

        foreach ($this ->sloupce as $indexsloupce => $sloupec) {
            if ($sloupec ->typ == 'DOCH1'  ||  $sloupec ->typ == 'DOCH2') {
                if ($sloupec ->lzeZobrazit  &&  $sloupec ->zobrazitPruchody) {
                    $radek ->hinty[$indexsloupce] = $textVsechny;
                }
            }
        }

        return true;
    }


    /**
     * Nacte puvodni dochazku/historii pruchodu a nastavi prislusne datove bunky
     * @param bool $pruchody
     */
    function nactiPuvodni($pruchody = false)
    {
        $params = $this ->database ->createParameters();

        if ($pruchody) {
            $sql =
                'SELECT HistoriePruchodu.DatumCasPruchodu, OperaceDochazky.PopisProDochazku '.
                '  FROM HistoriePruchodu, OperaceDochazky '.
                ' WHERE HistoriePruchodu.KodOperace = OperaceDochazky.Kod '.
                '   AND RC = ' . $params ->addParameter($this ->pp ->rc) .
                '   AND OsobniCislo = ' . $params ->addParameter($this ->pp ->osobniCislo) .
                '   AND DatumNastupu = ' . $params ->addParameter($this ->database ->dateToParam($this ->pp ->datumNastupu)) .
                '   AND DatumCasPruchodu >= ' . $params ->addParameter($this ->database ->dateToParam($this ->datumOd)) .
                '   AND DatumCasPruchodu < ' . $params ->addParameter($this ->database ->dateToParam(DateUtils::incDay($this ->datumDo, 1))) .
                ' ORDER BY DatumCasPruchodu';

            MessageManager::addDebugLog('Nacitam historii pruchodu: ' . $sql);
        } else {
            $sql =
                'SELECT PuvodniDochazka.DatumCasOperace, OperaceDochazky.PopisProDochazku '.
                '  FROM PuvodniDochazka, OperaceDochazky '.
                ' WHERE PuvodniDochazka.KodOperace = OperaceDochazky.Kod '.
                '   AND RC = ' . $params ->addParameter($this ->pp ->rc) .
                '   AND OsobniCislo = ' . $params ->addParameter($this ->pp ->osobniCislo) .
                '   AND DatumNastupu = ' . $params ->addParameter($this ->database ->dateToParam($this ->pp ->datumNastupu)) .
                '   AND DatumCasOperace >= ' . $params ->addParameter($this ->database ->dateToParam($this ->datumOd)) .
                '   AND DatumCasOperace < ' . $params ->addParameter($this ->database ->dateToParam(DateUtils::incDay($this ->datumDo, 1))) .
                ' ORDER BY DatumCasOperace';

            MessageManager::addDebugLog('Nacitam puvodni dochazku: ' . $sql);
        }

        $cursor = $this ->database ->getSqlResult($sql, $params, 'Chyba pri nacitani puvodni dochazky/historie pruchodu!');

        $datum = 0;
        $textPrvni = '';
        $textPosledni = '';
        $textVsechny = '';

        while ($row = $cursor ->fetch()) {
            $cas = $cursor ->stringToTimestamp($row[0]);
            $den = DateUtils::startOfTheDay($cas);

            if ($den != $datum) {
                if ($datum) {
                    if ($pruchody) {
                        $this ->nastavTextyPruchody($datum, $textPrvni, $textPosledni, $textVsechny);
                    } else {
                        $this -> nastavTextyPuvodni($datum, $textPrvni, $textPosledni, $textVsechny);
                    }
                }
                $textPrvni = '';
                $textVsechny = '';
            }

            $datum = $den;

            $textPosledni = date('G:i', $cas) . ' - ' . $row[1];
            if (!$textPrvni) {
                $textPrvni = $textPosledni;
            }
            $textVsechny .= ($textVsechny ? ', ' : '') . $textPosledni;
        }

        $cursor ->close();

        if ($datum) {
            if ($pruchody) {
                $this ->nastavTextyPruchody($datum, $textPrvni, $textPosledni, $textVsechny);
            } else {
                $this -> nastavTextyPuvodni($datum, $textPrvni, $textPosledni, $textVsechny);
            }
        }
    }


    /**
     * Nacte puvodni vyrobu a nastavi prislusne datove bunky
     */
    function nactiPuvodniVyrobu()
    {
        $params = $this ->database ->createParameters();

        $sql =
            'SELECT PuvodniVyroba.DatumCas, VyrobniCinnost.Popis, PuvodniVyroba.KodVyrobniCinnosti, PuvodniVyroba.Terminal, Terminal.Popis, '. // 0..4
            ' PuvodniVyroba.Vyrobek, PuvodniVyroba.VyrobniZakazka, PuvodniVyroba.VyrobniCislo, PuvodniVyroba.MnozstviDobre, PuvodniVyroba.MnozstviSpatne, ' . // 5..9
            ' PuvodniVyroba.Hodiny, PuvodniVyroba.Rozdeleni, PuvodniVyroba.Poznamka' . // 10..12
            '  FROM PuvodniVyroba INNER JOIN VyrobniCinnost'.
            '    ON PuvodniVyroba.KodVyrobniCinnosti = VyrobniCinnost.Kod' .
            '  LEFT OUTER JOIN Terminal' .
            '    ON Terminal.Kod = PuvodniVyroba.Terminal' .
            ' WHERE RC = ' . $params ->addParameter($this ->pp ->rc) .
            '   AND OsobniCislo = ' . $params ->addParameter($this ->pp ->osobniCislo) .
            '   AND DatumNastupu = ' . $params ->addParameter($this ->database ->dateToParam($this ->pp ->datumNastupu)) .
            '   AND DatumCas >= ' . $params ->addDateParameter($this ->datumOd) .
            '   AND DatumCas < ' . $params ->addDateParameter(DateUtils::incDay($this ->datumDo, 1)) .
            ' ORDER BY DatumCas';

        $cursor = $this ->database ->getSqlResult($sql, $params, 'Chyba pri nacitani puvodni vyroby!');

        $datum = 0;
        $dataPrvni = null;
        $dataPosledni = null;
        $dataVsechny = [];

        while ($row = $cursor ->fetch()) {
            $cas = $cursor ->stringToTimestamp($row[0]);
            $den = DateUtils::startOfTheDay($cas);

            if ($den != $datum) {
                if ($datum) {
                    $this -> nastavFormatovanyText($datum, 'VYR1', $dataPrvni, false);
                    $this -> nastavFormatovanyText($datum, 'VYR2', $dataPosledni, false);
                    $this -> nastavFormatovanyText($datum, 'VYR', $dataVsechny, true);
                }
                $dataPrvni = null;
                $dataVsechny = [];
            }

            $datum = $den;

            $dataPosledni = [
                '%CAS%' => date('G:i', $cas),
                '%POPIS%' => StringUtils::htmlTextConvert($row[1]),
                '%KOD%' => StringUtils::htmlTextConvert($row[2]),
                '%TERMINAL%' => StringUtils::htmlTextConvert($row[3]),
                '%TERMINAL_POPIS%' => StringUtils::htmlTextConvert($row[4]),
                '%VYROBEK%' => StringUtils::htmlTextConvert($row[5]),
                '%ZAKAZKA%' => StringUtils::htmlTextConvert($row[6]),
                '%VYROBNICISLO%' => StringUtils::htmlTextConvert($row[7]),
                '%MNOZSTVIDOBRE%' => StringUtils::format('number_1', $row[8]),
                '%MNOZSTVISPATNE%' => StringUtils::format('number_1', $row[9]),
                '%HODINY%' => StringUtils::format('hodiny', $row[10]),
                '%ROZDELENI%' => StringUtils::htmlTextConvert($row[11]),
                '%POZNAMKA%' => StringUtils::htmlTextConvert($row[12])
            ];

            if (!$dataPrvni) {
                $dataPrvni = $dataPosledni;
            }
            $dataVsechny[] = $dataPosledni;
        }

        $cursor ->close();

        if ($datum) {
            $this -> nastavFormatovanyText($datum, 'VYR1', $dataPrvni, false);
            $this -> nastavFormatovanyText($datum, 'VYR2', $dataPosledni, false);
            $this -> nastavFormatovanyText($datum, 'VYR', $dataVsechny, true);
        }
    }


    function nactiSmenyPodleKalendare($datumOd, $datumDo, $automaticke)
    {
        $smeny = SmenaProDen::nactiSmenyPodleKalendare($this ->database, $this ->pp, $datumOd, $datumDo);
        foreach ($smeny as $smena) {
            $radek = $this ->najdiRadekZpracovaneDochazky($smena ->datum);
            $radek ->smenaPodleKalendare = $smena;

            $zacatek = $smena ->pocatekSmeny ? DateUtils::minutesToString($smena ->pocatekSmeny / 60) : null;
            $this ->nastavTextSloupce($smena ->datum, 'KAL_Z', $zacatek);
            if ($automaticke) {
                $this ->nastavTextSloupce($smena ->datum, 'SMENA_Z', $zacatek);
            }

            $konec = $smena ->konecSmeny ? DateUtils::minutesToString($smena ->konecSmeny / 60) : null;
            $this ->nastavTextSloupce($smena ->datum, 'KAL_K', $konec);
            if ($automaticke) {
                $this ->nastavTextSloupce($smena ->datum, 'SMENA_K', $konec);
            }

            if ($zacatek || $konec) {
                $this ->nastavTextSloupce($smena ->datum, 'KAL_ZK', $zacatek . ' - ' . $konec);
                if ($automaticke) {
                    $this ->nastavTextSloupce($smena ->datum, 'SMENA_ZK', $zacatek . ' - ' . $konec);
                }
            }

            $delka = $smena ->delkaSmeny ? DateUtils::minutesToString($smena ->delkaSmeny / 60) : null;
            $this ->nastavTextSloupce($smena ->datum, 'KAL_DELKA', $delka);
            if ($automaticke) {
                $this ->nastavTextSloupce($smena ->datum, 'SMENA_DELKA', $delka);
            }

            // jadro
            if ($smena ->pocatekPevneCasti  &&  $smena ->konecPevneCasti) {
                $text = DateUtils::minutesToString($smena ->pocatekPevneCasti / 60) . ' - ' . DateUtils::minutesToString($smena ->konecPevneCasti / 60);
                $this->nastavTextSloupce($smena ->datum, 'KAL_J', $text);
                if ($automaticke) {
                    $this->nastavTextSloupce($smena ->datum, 'SMENA_J', $text);
                }
            }

            // kod, popis
            $this ->nastavTextSloupce($smena ->datum, 'KAL', $smena ->kod, $smena ->popis);
            if ($automaticke) {
                $this ->nastavTextSloupce($smena ->datum, 'SMENA', $smena ->kod, $smena ->popis);
            }
            $this ->nastavTextSloupce($smena ->datum, 'KAL_POPIS', $smena ->popis);
            if ($automaticke) {
                $this ->nastavTextSloupce($smena ->datum, 'SMENA_POPIS', $smena ->popis);
            }
        }

        return true;
    }



    /**
     * @param int  $datumOd
     * @param int  $datumDo
     * @param bool $automaticke
     */
    function nactiSmenyPodleKapacit($datumOd, $datumDo, $automaticke)
    {
        $smeny = SmenaProDen::nactiSmenyPodleKapacit($this ->database, $this ->pp, $datumOd, $datumDo);

        foreach ($smeny as $smena) {
            $radek = $this ->najdiRadekZpracovaneDochazky($smena ->datum);
            $radek ->smenaPodleKapacit = $smena;

            $zacatek = isset($smena ->pocatekSmeny) ? DateUtils::minutesToString($smena ->pocatekSmeny / 60) : null;
            $this ->nastavTextSloupce($smena ->datum, 'KAP_Z', $zacatek);
            if ($automaticke) {
                $this ->nastavTextSloupce($smena ->datum, 'SMENA_Z', $zacatek);
            }

            $konec = isset($smena ->konecSmeny) ? DateUtils::minutesToString($smena ->konecSmeny / 60) : null;
            $this ->nastavTextSloupce($smena ->datum, 'KAP_K', $konec);
            if ($automaticke) {
                $this ->nastavTextSloupce($smena ->datum, 'SMENA_K', $konec);
            }

            if ($zacatek || $konec) {
                $this ->nastavTextSloupce($smena ->datum, 'KAP_ZK', $zacatek . ' - ' . $konec);
                if ($automaticke) {
                    $this ->nastavTextSloupce($smena ->datum, 'SMENA_ZK', $zacatek . ' - ' . $konec);
                }
            }

            $delka = $smena ->delkaSmeny ? DateUtils::minutesToString($smena ->delkaSmeny / 60) : null;
            $this ->nastavTextSloupce($smena ->datum, 'KAP_DELKA', $delka);
            if ($automaticke) {
                $this ->nastavTextSloupce($smena ->datum, 'SMENA_DELKA', $delka);
            }

            // jadro
            if ($smena ->pocatekPevneCasti  &&  $smena ->konecPevneCasti) {
                $text = DateUtils::minutesToString($smena ->pocatekPevneCasti / 60) . ' - ' . DateUtils::minutesToString($smena ->konecPevneCasti / 60);
                $this->nastavTextSloupce($smena ->datum, 'KAP_J', $text);
                if ($automaticke) {
                    $this->nastavTextSloupce($smena ->datum, 'SMENA_J', $text);
                }
            }

            // kod, popis
            $this ->nastavTextSloupce($smena ->datum, 'KAP', $smena ->kod, $smena ->popis);
            $this ->nastavTextSloupce($smena ->datum, 'KAP_PUV', $smena ->kodPuvodniSmeny);
            if ($automaticke) {
                $this ->nastavTextSloupce($smena ->datum, 'SMENA', $smena ->kod, $smena ->popis);
            }
            $this ->nastavTextSloupce($smena ->datum, 'KAP_POPIS', $smena ->popis);
            if ($automaticke) {
                $this ->nastavTextSloupce($smena ->datum, 'SMENA_POPIS', $smena ->popis);
            }
        }
    }


    function nactiPrirazeneKalendare()
    {
        if (!isset($this ->prirazeneKalendare)) {
            $this ->prirazeneKalendare = Kalendar::nactiKalendareOsoby($this ->database, $this ->pp, $this ->datumOd, $this ->datumDo);
        }

        return $this ->prirazeneKalendare;
    }


    /**
     * Zjisti, jestli je zamestnanec v zadanem kalendari. Pokud neni zadan datum, tak staci kdykoliv behem nacteneho obdobi.
     * @param string|string[] $kod      Kod kalendare nebo pole kodu
     * @param int $datum
     * @return bool
     */
    function jeVKalendari($kod, $datum = null)
    {
        $kalendare = $this ->nactiPrirazeneKalendare();

        foreach ($kalendare as $kalendar) {
            if (is_array($kod)) {
                $nasel = in_array($kalendar ->kod, $kod, true);
            } else {
                $nasel = $kalendar ->kod === $kod;
            }

            if ($nasel) {
                if (!$datum) {
                    return true;
                }

                if ($datum >= $kalendar ->platnostOd  &&  (!$kalendar ->platnostDo  ||  $datum <= $kalendar ->platnostDo)) {
                    return true;
                }
            }
        }

        return false;
    }

    /**
     * @param bool $nacistSmeny
     */
    function nactiKalendare($nacistSmeny)
    {
        $kalendare = $this ->nactiPrirazeneKalendare();

        foreach ($kalendare as $kalendar) {
            $datumod = $kalendar ->platnostOd;
            if ($this ->datumOd > $datumod) {
                $datumod = $this ->datumOd;
            }

            $datumdo = $this ->datumDo;
            if ($kalendar ->platnostDo  &&  $kalendar ->platnostDo < $datumdo) {
                $datumdo = $kalendar ->platnostDo;
            }

            $datum = $datumod;
            while ($datum <= $datumdo) {
                $radek = $this ->najdiRadekZpracovaneDochazky($datum);
                $radek ->kalendarPodleKapacit = $kalendar ->dochazkaPodlePlanKapacit;

                $this ->nastavTextSloupce($datum, 'K_KOD', $kalendar ->kod, $kalendar ->popis);
                $this ->nastavTextSloupce($datum, 'K_POPIS', $kalendar ->popis);

                $datum = DateUtils::incDay($datum, 1);
            }

            if ($nacistSmeny) {
                if ($kalendar ->dochazkaPodlePlanKapacit) {
                    $this ->nactiSmenyPodleKapacit($datumod, $datumdo, true);
                } else {
                    $this ->nactiSmenyPodleKalendare($datumod, $datumdo, true);
                }
            }
        }
    }


    /**
     */
    function nactiSmeny()
    {
        $jeKalendar = false;
        $jeSmenaPodleKalendare = false;
        $jeSmenaPodleKapacit = false;
        $jeSmenaAutomaticky = false;

        foreach ($this ->sloupce as $sloupec) {
            if ($sloupec ->lzeZobrazit) {
                switch ($sloupec->typ) {
                    case 'K_KOD' :
                    case 'K_POPIS' :
                        $jeKalendar = true;
                        break;

                    case 'KAL' : // kod smeny
                    case 'KAL_POPIS' :
                    case 'KAL_Z' :
                    case 'KAL_K' :
                    case 'KAL_ZK' :
                    case 'KAL_J' :
                    case 'KAL_DELKA' :
                        $jeSmenaPodleKalendare = true;
                        break;

                    case 'KAP' : // kod smeny
                    case 'KAP_PUV' : // kod smeny v puvodnim planu
                    case 'KAP_POPIS' :
                    case 'KAP_Z' :
                    case 'KAP_K' :
                    case 'KAP_ZK' :
                    case 'KAP_J' :
                    case 'KAP_DELKA' :
                        $jeSmenaPodleKapacit = true;
                        break;

                    case 'SMENA' :
                    case 'SMENA_POPIS' :
                    case 'SMENA_Z' :
                    case 'SMENA_K' :
                    case 'SMENA_ZK' :
                    case 'SMENA_J' :
                    case 'SMENA_DELKA' :
                        $jeSmenaAutomaticky = true; // smena z planovani nebo kalendare podle nastaveni v kalendari
                        break;

                    case 'DOCH_G' :
                        if ($sloupec->zobrazitSmenu == 'KAL') {
                            $jeSmenaPodleKalendare = true;
                        } else if ($sloupec->zobrazitSmenu == 'KAP') {
                            $jeSmenaPodleKapacit = true;
                        } else if ($sloupec->zobrazitSmenu == '1') {
                            $jeSmenaAutomaticky = true;
                        }
                        break;
                }
            }
        }

        if ($jeKalendar || $jeSmenaAutomaticky) {
            $this ->nactiKalendare($jeSmenaAutomaticky);
        }

        if ($jeSmenaPodleKalendare) {
            $this ->nactiSmenyPodleKalendare($this ->datumOd, $this ->datumDo, false);
        }

        if ($jeSmenaPodleKapacit) {
            $this ->nactiSmenyPodleKapacit($this ->datumOd, $this ->datumDo, false);
        }
    }


    /**
     * @return bool
     */
    function nactiMesicniVysledky()
    {
        $vysledky = MesicniVysledek::nactiMesicniVysledky($this ->database, $this ->pp ->idPP, $this ->datumOd);

        foreach ($this ->sloupce as $sloupec)
            if ($sloupec ->lzeZobrazit  &&  $sloupec ->typ == 'DV' && $sloupec ->jednotka != 'P') {
                $vysledek = isset($vysledky[$sloupec ->kodCasoveSlozky]) ? $vysledky[$sloupec ->kodCasoveSlozky] : null;
                if ($vysledek) {
                    $sloupec->soucet = $vysledek->vratText($sloupec->jednotka);
                }
            }
    }


    /**
     * @param int $indexSloupce
     * @param int $typRozdeleni
     * @return bool
     */
    function nactiSloupecRozdeleniDV($indexSloupce, $typRozdeleni)
    {
        $params = $this ->database ->createParameters();

        $tableName = $this ->sloupce[$indexSloupce] ->typ == 'DENNIVYSLEDEK_R'
            ? 'DenniVysledek_R'
            : 'DenniVysledek_R_E';

        $sql =
            "SELECT $tableName.Den, $tableName.TypCasoveSlozkyMzdy, $tableName.Rozdeleni, $tableName.Hodiny, " .
            '   Rozdeleni.Popis, TypCasoveSlozkyMzdy.Popis, ' . // 4,5
            '   Rozdeleni.Uzivatelska1, Rozdeleni.Uzivatelska2, Rozdeleni.Uzivatelska3, Rozdeleni.Uzivatelska4, Rozdeleni.Uzivatelska5, ' . // 6..10
            '   Rozdeleni.Uzivatelska6, Rozdeleni.Uzivatelska7, Rozdeleni.Uzivatelska8, Rozdeleni.Uzivatelska9, Rozdeleni.Uzivatelska10 ' . // 11..15
            " FROM $tableName, Rozdeleni, TypCasoveSlozkyMzdy " .
            " WHERE $tableName.KodTypuRozdeleni = " . $params ->addParameter((int)$typRozdeleni) .
            " AND $tableName.RC = " . $params ->addParameter($this ->pp ->rc) .
            " AND $tableName.OsobniCislo = " . $params->addParameter($this ->pp ->osobniCislo) .
            " AND $tableName.DatumNastupu = " . $params ->addParameter($this ->database ->dateToParam($this ->pp ->datumNastupu)) .
            " AND $tableName.Den >= " . $params ->addParameter($this ->database ->dateToParam($this ->datumOd)) .
            " AND $tableName.Den <= " . $params ->addParameter($this ->database ->dateToParam($this ->datumDo)) .
            " AND Rozdeleni.KodTypuRozdeleni = $tableName.KodTypuRozdeleni " .
            " AND Rozdeleni.Kod = $tableName.Rozdeleni " .
            " AND TypCasoveSlozkyMzdy.Kod = $tableName.TypCasoveSlozkyMzdy " .
            ($this ->sloupce[$indexSloupce] ->kodCasoveSlozky
                ? ' AND TypCasoveSlozkyMzdy.Kod = ' . $params ->addParameter($this ->sloupce[$indexSloupce] ->kodCasoveSlozky)
                : ''
            ) .
            " ORDER BY Den, $tableName.Rozdeleni";

        $cursor = $this ->database ->getSqlResult($sql, $params, 'Chyba pri nacitani rozdeleni dennich vysledku!');
        MessageManager::addDebugLog($sql);

        $datum_last = null;
        $text = '';
        while ($row = $cursor ->fetch()) {
            $datum = $cursor ->stringToTimestamp($row[0]);

            if ($datum > $datum_last) {
                if ($datum_last && $text) {
                    $this ->nastavText($datum_last, $indexSloupce, "<DIV class='seznamrozdelenidv'>$text</DIV>");
                }
                $datum_last = $datum;
                $text = '';
            }

            //if ($text) $text .= ', '; - presunuto do css
            if ((float)$row[3] || $this ->sloupce[$indexSloupce] ->zobrazitNuloveHodnoty) {
                if ($this->sloupce[$indexSloupce]->format) {
                    $text .= str_replace(
                        [   '%HODINY%', // hodiny
                            '%KOD%', '%POPIS%', // rozdeleni
                            '%KOD_CS%', '%POPIS_CS%', // casova slozka
                            '%UZIVATELSKA1%', '%UZIVATELSKA2%',
                            '%UZIVATELSKA3%', '%UZIVATELSKA4%',
                            '%UZIVATELSKA5%', '%UZIVATELSKA6%',
                            '%UZIVATELSKA7%', '%UZIVATELSKA8%',
                            '%UZIVATELSKA9%', '%UZIVATELSKA10%'
                        ],
                        [
                            StringUtils::format('minuty', round($row[3] * 60)),
                            StringUtils::htmlTextConvert($row[2]), StringUtils::htmlTextConvert($row[4]),
                            StringUtils::htmlTextConvert($row[1]), StringUtils::htmlTextConvert($row[5]),
                            StringUtils::htmlTextConvert($row[6]), StringUtils::htmlTextConvert($row[7]),
                            StringUtils::htmlTextConvert($row[8]), StringUtils::htmlTextConvert($row[9]),
                            StringUtils::htmlTextConvert($row[10]), StringUtils::htmlTextConvert($row[11]),
                            StringUtils::htmlTextConvert($row[12]), StringUtils::htmlTextConvert($row[13]),
                            StringUtils::htmlTextConvert($row[14]), StringUtils::htmlTextConvert($row[15])
                        ],
                        $this->sloupce[$indexSloupce]->format
                    );
                } else {
                    $hint = $row[4] . ' - ' . $row[5];
                    $text .=
                        "<SPAN class='zaznamrozdelenidv' title='" . StringUtils::htmlTextConvert($hint) . "'>" . StringUtils::htmlTextConvert($row[2]) .
                        ' (' . StringUtils::format('minuty', round($row[3] * 60)) . ') ' . StringUtils::htmlTextConvert($row[1]) . '</SPAN>';
                }
            }
        }

        $cursor ->close();

        if ($text && $datum_last) {
            $this ->nastavText($datum_last, $indexSloupce, "<DIV class='seznamrozdelenidv'>$text</DIV>");
        }

        return true;
    }


    /**
     */
    function nactiRozdeleniDV()
    {
        foreach ($this ->sloupce as $i => $sloupec) {
            $jeSloupecRozdeleni = $sloupec ->typ == 'DV_ROZDELENI' || $sloupec ->typ == 'DENNIVYSLEDEK_R'  || $sloupec ->typ == 'DENNIVYSLEDEK_R_E';
            if ($sloupec ->lzeZobrazit  &&  $jeSloupecRozdeleni) {
                $this ->nactiSloupecRozdeleniDV($i, $sloupec ->typRozdeleni ?: RonPortalSettings::getIntValue(RonPortalSettings::DV_TYP_ROZDELENI));
            }
        }
    }


    /**
     * @return bool
     */
    function nactiAbsence()
    {
        $seznamAbsenci = ZaznamAbsence::nactiAbsenceOsoby($this ->database, $this ->pp, $this ->datumOd, $this ->datumDo);

        foreach ($seznamAbsenci as $absence) {
            $datumOd = $absence ->datumOd;
            if ($datumOd < $this ->datumOd) {
                $datumOd = $this ->datumOd;
            }

            $index = $this -> indexRadku($datumOd);
            while ($index >= 0  &&  $index < count($this ->f_data)  &&  $this ->f_data[$index] ->datum <= $absence ->datumDo) {
                // prida absenci do vsech radku (podle rozsahu datumu)
                if ($this ->absenceClass) {
                    $this ->f_data[$index] ->cssClass .= ( (RonPortalSettings::getIntValue(RonPortalSettings::ABSENCE_SCHVALOVANI) && !$absence ->schvaleni) ? ' neschvalenaabsence_' : ' absence_' ) . $absence ->operaceDochazky;
                }

                foreach ($this ->sloupce as $indexsloupce => $sloupec) {
                    if ($sloupec ->lzeZobrazit  && $sloupec ->typ == 'ABSENCE') {
                        if (!isset($this ->f_data[$index] ->texty[$indexsloupce])) {
                            $this ->f_data[$index] ->texty[$indexsloupce] = '';
                        }
                        if ($this ->f_data[$index] ->texty[$indexsloupce]) {
                            $this ->f_data[$index] ->texty[$indexsloupce] .= ', ';
                            //$this ->f_data[$index] ->f_hinty[$indexsloupce] .= ', ';
                        }
                        $this ->f_data[$index] ->texty[$indexsloupce] .=
                            ($this ->f_data[$index] ->texty[$indexsloupce] ? ', ' : '') .
                            "<a href='?menuid=absence&action=zobrazabsenci&idpp={$absence ->idPP}&datum=" . DateUtils::dateToUrlString($absence ->datumOd) . "&poradi={$absence ->poradi}&backaction=zpracovanadochazka'>" .
                            $absence ->format($sloupec ->format) .
                            "</a>";

                        //$this ->f_data[$index] ->f_hinty[$indexsloupce] .= htmltext_convert($row[1]);
                    }
                }

                $index ++;
            }
        }
    }


    /**
     */
    function nactiDokumenty()
    {
        $dokumenty = DokumentDochazky::nactiDokumentyOsoby($this ->database, $this ->datumOd, $this ->datumDo, $this ->pp ->idPP);

        foreach ($dokumenty as $dokument) {
            $index = $this -> indexRadku($dokument ->datum);
            if ($index >= 0  &&  $index < count($this ->f_data)) {
                foreach ($this ->sloupce as $indexSloupce => $sloupec) {
                    if ($sloupec ->lzeZobrazit  &&  $sloupec ->typ == 'DOC') {
                        $this ->f_data[$index] ->texty[$indexSloupce] .= $dokument ->getDownloadHtml($this ->datumOd); // $text;
                    }
                }
            }
        }
    }


    /**
     */
    function nactiKontroly()
    {
        $kontroly = VysledekKontroly::nactiVysledkyKontrol($this ->database, $this ->pp, $this ->datumOd, $this ->datumDo);

        foreach ($kontroly as $kontrola) {
            if (!$kontrola ->mesicni) {
                $class = $kontrola ->cssClass
                    ? $kontrola ->cssClass
                    : ('kontrola_' . $kontrola ->kategorie);

                if ($kontrola ->ignorovat) {
                    $class = 'ignorovana_' . $class;
                }

                $radek = $this->najdiRadekZpracovaneDochazky($kontrola->datum);
                $radek ->cssClass .=
                    ($radek->cssClass ? ' ' : '') . $class;

                if (!$kontrola ->ignorovat  &&  isset($kontrola ->barva)) {
                    $radek ->barvaRadku = $kontrola ->barva;
                }
            }
        }

        $this ->kontroly = $kontroly;
    }


    /**
     * Nacte data aktualni osoby
     */
    function nactiDataZDatabaze()
    {
        $this ->nactiOperaceDochazky();

        if (!count($this ->sloupce)) {
            // neni co nacitat
            return;
        }

        $this ->nactiDenniVysledky();

        if ($this ->existujeSloupec('P')) {
            $this ->nactiPoznamky();
        }
        if ($this ->jeSloupecSPuvodni()) {
            $this ->nactiPuvodni();
        }
        if ($this ->jeSloupecSPruchody()) {
            $this ->nactiPuvodni(true); // pruchody = true
        }
        if ($this ->existujeSloupec(['VYR', 'VYR1', 'VYR2'])) {
            $this ->nactiPuvodniVyrobu();
        }
        if ($this ->existujeSloupec('DOC')) {
            $this ->nactiDokumenty();
        }
        if ($this ->nacitatKontroly  ||  $this ->existujeSloupec('KONTROLY')) {
            $this ->nactiKontroly();
        }

        $this ->nactiSmeny();

        $this ->nactiMesicniVysledky();
        $this ->nactiRozdeleniDV();

        if ($this ->nacitatAbsence()) {
            $this ->nactiAbsence();
        }
    }


    /**
     * @param DenZpracovaneDochazky $radek
     * @param int $indexSloupce
     * @return bool
     */
    function nactiHodnotuPodlePost($radek, $indexSloupce)
    {
        $sloupec = $this ->sloupce[$indexSloupce];

        if (!$sloupec ->lzeZobrazit) {
            return true;
        }

        if (!$this ->lzeEditovat($radek, $sloupec)) {
            return true; // nedela nic, neni chyba
        }

        $editname = $sloupec ->getEditName($radek ->datum);
        if (!$editname) {
            return true;
        }
        if ($sloupec ->typ == 'DV_CHECK') {
            // u checkboxu musi byt nastaven "priznak" odeslani dat daneho sloupce
            if (!isset($_POST['DV_CHECK_' . trim($sloupec ->kodCasoveSlozky) . '_' . trim($sloupec ->jednotka)])) {
                return true;
            }
            $text = (isset($_POST[$editname])  &&  $_POST[$editname]) ? '1' : '0';
        } else {
            if (!isset($_POST[$editname])) {
                // hodnota nebyla odeslana
                return true;
            }
            $text = $_POST[$editname];
        }

        try {
            switch ($sloupec->typ) {
                /** @noinspection PhpMissingBreakStatementInspection */
                case 'DV' :
                    if ($sloupec->jednotka == 'P') {
                        $radek->texty[$indexSloupce] = $text;
                    } else {
                        $radek->texty[$indexSloupce] = $text;
                        $radek->hodnoty[$indexSloupce] = $sloupec->vratHodnotuZTextu($text);
                    }
                    return true;

                case 'DV_CHECK' :
                    $radek->texty[$indexSloupce] = $text;
                    $radek->hodnoty[$indexSloupce] = (int)$text;
                    return true;

                case 'DOCH1' :
                case 'DOCH2' :
                    $radek->texty[$indexSloupce] = $text;
                    $radek->hodnoty[$indexSloupce] = $sloupec->vratHodnotuZTextu($text);
                    return true;

                case 'P' :
                    $radek->texty[$indexSloupce] = $text;
                    return true;
            }
        } catch (Exception $e) {
            $this->jeChyba = true;
            $radek->chyby[$indexSloupce] = true;
            MessageManager::addErrorMessage( TextManager::_('Spatna_hodnota_pro') . $sloupec ->nadpis . ' (' . StringUtils::format('date', $radek ->datum) . ')' );
        }

        return true;
    }


    /**
     * nacte postnute data a overi jejich spravnost
     * @return bool
     */
    function nactiPost()
    {
        if (!isset($_POST['zpracovana_dochazka']) || !$_POST['zpracovana_dochazka']) {
            return true; // neni co nacitat
        }
        if ($this ->obdobiUzavreno) {
            return true;
        }
//        if (!$this ->jePravoEditovat  &&  !$this ->jePravoEditovatPoznamku) { - kazdy sloupec muze mit jine pravo na editaci
//            return true;
//        }

        $this ->jeChyba = false;

        // kontrola zajisti, ze se uzivatel v jinem okne neprepnul na jinou osobu a rozdeleni
        if ($this ->pp ->idPP != $_POST['osoba']) {
            return MessageManager::addErrorMessage('Chyba pri zpracovani odeslanych dat! Byla odeslana data pro jinou osobu!');
        }
        if ($this ->datumOd != DateUtils::urlStringToDate($_POST['obdobi'])) {
            return MessageManager::addErrorMessage('Chyba pri zpracovani odeslanych dat! Byla odeslana data pro jine obdobi!');
        }

        foreach ($this ->f_data as $radek) {
            foreach ($this ->sloupce as $indexsloupce => $sloupec) {
                if ($this ->lzeEditovat($radek, $sloupec)) {
                    $this->nactiHodnotuPodlePost($radek, $indexsloupce);
                }
            }
        }

        return !$this ->jeChyba;
    }


    /**
     * @param DenZpracovaneDochazky $radek
     * @param int $indexSloupce
     * @param string $text
     * @return bool|false
     */
    function ulozDenniVysledek($radek, $indexSloupce, $text)
    {
        $sloupec = $this ->sloupce[$indexSloupce];
        if (!$sloupec ->lzeZobrazit) {
            // nezobrazovany sloupec -> nedela nic
            return true;
        }

        $hodnota = $sloupec ->vratHodnotuZTextu($text);
        if ($hodnota === false) {
            // nemelo by nastat - uz bylo zkontrolovano predem
            return MessageManager::addErrorMessage('Chyba pri ukladani hodnot! Spatne zadana hodnota: ' . $text);
        }

        $rec = new DenniVysledekVypoctuRecord($this ->database, $this ->pp, $radek ->datum, $sloupec ->kodCasoveSlozky, $this);
        $rec ->zapisHodnotu($sloupec ->jednotka, $hodnota, $sloupec ->zobrazitNuloveHodnoty);

        if (!$this ->datumPrvniZmeny  ||  $this ->datumPrvniZmeny > $radek ->datum) {
            $this ->datumPrvniZmeny = $radek ->datum;
        }

        $radek ->texty[$indexSloupce] = $sloupec ->vratTextProHodnotu($hodnota);
        $radek ->textyPuvodni[$indexSloupce] = $radek ->texty[$indexSloupce];

        return true;
    }


    // -----     DenniVysledky     -------------------------------------------
    function ulozPoznamku($datum, $text)
    {
        $rec = new DenniPoznamkaDochazkyRecord($this ->database, $this ->pp, $datum, $this);
        $rec ->zapisPoznamku($text);
    }


    // -----     DenniVysledky     -------------------------------------------
    function ulozHodnotu($radek, $indexSloupce, $text)
    {
        $sloupec = $this ->sloupce[$indexSloupce];

        if ($sloupec ->lzeZobrazit) {
            switch ($sloupec->typ) {
                case 'DV' :
                case 'DV_CHECK' :
                    return $this->ulozDenniVysledek($radek, $indexSloupce, $text);

                case 'P' :
                    $this->ulozPoznamku($radek->datum, $text);
                    return true;

                case 'DOCH1' :
                case 'DOCH2' :
                    return MessageManager::addErrorMessage('Chyba ve funkci UlozHodnotu!'); // tady se nesmi vubec dostat
            }
        }

        return false;
    }


    // -----     DenniVysledky - editace prichodu a odchodu    ----------------
    /**
     * Vraci true, pokud seznam strednikem oddelenych operaci ($operacestr) obsahuje zadanou operaci.
     * Pokud je seznam prazdny, vracit $defaultValue
     * @param string $kodOperace
     * @param string $operaceStr
     * @param bool $defaultValue
     * @return bool
     */
    function operaceJeVSeznamu($kodOperace, $operaceStr, $defaultValue = true)
    {
        if (trim($operaceStr) == '') {
            return $defaultValue;
        }

        $seznamOperaci = explode(';', $operaceStr);
        return in_array( strtolower($kodOperace), $seznamOperaci) || in_array( '*', $seznamOperaci);
    }


    /**
     * SQL filtr pro vyber zaznamu s pozadovanou operaci
     * @param $operaceStr string   seznam zapocitavanych operaci (kody oddelene strednikem)
     * @return string
     */
    function filtrPovoleneOperace($operaceStr) // , $not_eq
    {
        if (!$operaceStr) {
            return '';
        }

        $operace = explode(';', $operaceStr);

        $result = '';
        foreach($operace as $kodoperace) {
            if (trim($kodoperace)) {
                $result .=
                    ($result ? ' OR ' : '') .
                    "Operace = '" . $this ->database ->addSlashes($kodoperace) . "'";
            }
        }

        if (!$result) {
            return '';
        }

        return '('.$result.')';
    }


    /**
     * @param $radek DenZpracovaneDochazky
     * @param int $indexPrichod index sloupce s editovanym prichodem
     * @param int $indexOdchod index sloupce s editovanym odchodem
     * @return bool
     * @throws DBException|NeniPravoException
     */
    function ulozPrichodOdchod($radek, $indexPrichod, $indexOdchod)
    {
        // TODO: umoznit vymazat dochazku vymazanim prichodu i odchodu

        $sloupecPrichod = isset($indexPrichod) ? $this ->sloupce[$indexPrichod] : null;
        $sloupecOdchod = isset($indexOdchod) ? $this ->sloupce[$indexOdchod] : null;

        $zmenaPrichod =
            $sloupecPrichod
            &&  $sloupecPrichod ->lzeEditovat
            &&  ($radek ->texty[$indexPrichod] != $radek ->textyPuvodni[$indexPrichod]);

        $zmenaOdchod  = $sloupecOdchod   &&  $sloupecOdchod ->lzeEditovat   &&  $radek ->texty[$indexOdchod] != $radek ->textyPuvodni[$indexOdchod];

        if (!$zmenaPrichod && !$zmenaOdchod) {
            // neni co menit
            return true;
        }

        if (!$this ->lzeEditovatDen($radek ->datum)) {
            return false;
        }

        try {
            $casPrichod = DateUtils::timeStringToTimestamp($radek->texty[$indexPrichod]);   // jen cas, bez datumu
            $hodnotaPrichod = DateUtils::timeStringToTimestamp($radek->texty[$indexPrichod], $radek->datum);
        } catch (Exception $e) {
            return MessageManager::addErrorMessage('Chyba pri ukladani hodnot! Spatne zadana hodnota: ' . $radek->texty[$indexPrichod]);
        }

        try {
            $casOdchod = DateUtils::timeStringToTimestamp($radek ->texty[$indexOdchod]);
            $hodnotaOdchod = DateUtils::timeStringToTimestamp($radek ->texty[$indexOdchod], $radek ->datum);
        } catch (Exception $e) {
            return MessageManager::addErrorMessage('Chyba pri ukladani hodnot! Spatne zadana hodnota: ' . $radek->texty[$indexOdchod]);
        }

        if ($casOdchod < $casPrichod) {
            // pres pulnoc - posunu odchod do dalsiho dne nebo prichod do predchoziho

            // TODO: pokud je u smeny "zapocitavat do dne podle konce", tak posunout prichod do predchoziho dne!
            // jak ale urcit smenu? co kdyz uz jsou tam zaznamy se zmenenou smenou?
            $smena = EditaceUtils::vychoziSmenaProOsobu($this ->database, $this ->pp, $radek ->datum, $casPrichod, $casOdchod);

            if ($smena &&  $smena ->denDochPodleKonceSmeny) {
                // pocita se podle konce smeny - posunout prichod o den dozadu
                $hodnotaPrichod = DateUtils::timeStringToTimestamp($radek->texty[$indexPrichod], DateUtils::incDay($radek->datum, -1));
            } else {
                $hodnotaOdchod = DateUtils::timeStringToTimestamp($radek->texty[$indexOdchod], DateUtils::incDay($radek->datum, 1));
            }
        }

        EditaceUtils::editaceDochazkyEvent($this ->database, $this ->pp, $radek ->datum, 'before');

        $filtr = $this ->filtrPovoleneOperace($sloupecPrichod ->zapocitatOperace);

        // Zrusi zaznamy, ktere jsou kompletne pred novym casem prichodu nebo za novym casem odchodu
        // - zaznamy, ktere maji jinou operaci nez seznam zapocitavanych operaci ponecha
        // - zaznamy, ktere patri do jineho dne ponecha
        $rec = new DochazkaRecord($this ->database, $this ->pp, $this);
        $rec ->logInfoMessage =
            sprintf('Editace prichodu/odchodu pro den %s (%s - %s) - smazani zaznamu mimo interval',
                date('j.n.Y', $radek ->datum), $radek ->texty[$indexPrichod], $radek ->texty[$indexOdchod]);

        $params = $this ->database ->createParameters();
        $pocetSmazanych = $rec ->smazZaznamyDochazky(
            ' Den = ' . $params ->addParameter($this ->database ->dateToParam($radek ->datum)) . // zaznamy, ktere patri do jineho dne ponecha
            ' AND ( ' .
            '(DatumCasUkonceni <= ' . $params ->addTimestampParameter($hodnotaPrichod).
            '  AND DatumCasPocatku < ' . $params ->addTimestampParameter($hodnotaPrichod) . ')' .  // kompletne pred
            ' OR DatumCasPocatku >= ' . $params ->addTimestampParameter($hodnotaOdchod).   // nebo za
            ' )'.
            ($filtr ? ' AND ('.$filtr .') ' : ''),
            $params
        );

        $zacatekUpraven = false;
        $konecUpraven = false;
        $zacatekOK = !$zmenaPrichod; // Priznak, ze uz byl zmenen cas pocatku
        $konecOK = false;

        if ($zmenaPrichod) {
            // Projede vsechny zaznamy zasahujici do intervalu.
            // Nezapocitavane operace smaze. Zapocitavane upravi nebo pred ne/za ne doplni novou.

            $rec = new DochazkaRecord($this ->database, $this->pp, $this);
            $rec->logInfoMessage =
                sprintf(
                    'Editace prichodu/odchodu pro den %s (%s - %s)',
                    date('j.n.Y', $radek->datum), $radek ->texty[$indexPrichod], $radek ->texty[$indexOdchod]);

            $params = $this ->database ->createParameters();
            $rec->loadRecords(
                //' Den = ' . $params ->addDateParameter($radek->datum) . ' AND ' . - odstraneno 20.12.2022 - jinak se nenacetly zaznamy patrici do jineho dne a mohly se pak prekryvat
                'DatumCasUkonceni >= ' . $params ->addTimestampParameter($hodnotaPrichod) .
                ' AND DatumCasPocatku <= ' . $params ->addTimestampParameter($hodnotaOdchod),
                $params);

            // TODO: zkontrolovat chovani pro nocni smeny
            while (!$zacatekOK && $rec->next()) {
                if ($rec ->fDen ->value !== $radek ->datum  ||  !$this->operaceJeVSeznamu($rec->fOperace->value, $sloupecPrichod->zapocitatOperace)) {
                    // tato operace se nezapocitava (bud je zapocitavana do jineho dne nebo neni v seznamu zapocitavanych operaci)
                    // - zkrati ji, prerusi nebo uplne smaze
                    if (!$rec->odstranZIntervalu($hodnotaPrichod, $hodnotaOdchod)) {
                        return false;
                    }
                } else {
                    // nasel prvni zapocitavanou operaci - opravi zacatek
                    $datumCasPocatkuOriginal = $sloupecPrichod ->zobrazenyCas
                        ? $rec->fDatumCasPocatku ->originalValue
                        : $rec->getZapoctenyCasOd(true);

                    if ($datumCasPocatkuOriginal < $hodnotaPrichod) {
                        // useknuti prvni operace (posunuti prichodu)
                        $rec->nastavCasPrichodu($hodnotaPrichod, !$sloupecPrichod ->zobrazenyCas);
                        $zacatekUpraven = true;
                    } else if ($datumCasPocatkuOriginal > $hodnotaPrichod) {
                        // prvni operace zacina pozdeji => je potreba prodlouzit, nebo doplnit zaznam na zacatek
                        if ($rec->fDatumCasPocatku ->originalValue <= $hodnotaPrichod) {
                            // zapocitany cas zacina pozdeji, ale zobrazeny zacina drive => prodlouzit zapocitany cas (zkratit zobacek)
                            $rec->nastavCasPrichodu($hodnotaPrichod, true);
                        } else if ($sloupecPrichod->operace) {
                            // je nastaveno, ze se ma vzdy vkladat novy zaznam se zadanou operaci
                            $newRec = new DochazkaRecord($this ->database, $this->pp, $this);

                            $casDo = $rec->fDatumCasPocatku->originalValue;
                            if ($casDo > $hodnotaOdchod) {
                                // nemelo by byt potreba, ale pro jistotu
                                $casDo = $hodnotaOdchod;
                            }

                            if (!$newRec->vlozZaznamDochazky($radek->datum, $hodnotaPrichod, $casDo, $sloupecPrichod->operace)) {
                                return MessageManager::addErrorMessage('Chyba ve funkci ulozPrichodOdchod - nelze vlozit novy zaznam podle zadaneho prichodu!');
                            }

                            if ($rec ->fDatumCasPocatku ->originalValue < $datumCasPocatkuOriginal) {
                                // jeste odstranit zobacek v existujici operaci
                                $rec ->nastavCasPrichodu($rec ->fDatumCasPocatku ->originalValue, true);
                            }
                        } else {
                            $rec->nastavCasPrichodu($hodnotaPrichod, !$sloupecPrichod ->zobrazenyCas);
                        }

                        $zacatekUpraven = true;
                    }

                    $zacatekOK = true;

                    $datumCasUkonceniOriginal = $sloupecOdchod ->zobrazenyCas
                        ? $rec->fDatumCasUkonceni ->originalValue
                        : $rec->getZapoctenyCasDo(true);

                    if ($datumCasUkonceniOriginal > $hodnotaOdchod) {
                        // zaznam presahuje az za novy odchod - rovnou usekne i konec zaznamu
                        // automaticke prodluzovani nedela, protoze tam muze byt jeste dalsi zaznam
                        if (!$rec->nastavCasOdchodu($hodnotaOdchod, !$sloupecOdchod ->zobrazenyCas)) {
                            return MessageManager::addErrorMessage('Chyba ve funkci ulozPrichodOdchod - nelze zmenit cas ukonceni podle zadaneho odchodu!');
                        }

                        $konecUpraven = true;
                        $konecOK = true;
                    } else if ($datumCasUkonceniOriginal == $hodnotaOdchod) {
                        // spravna hodnota odchodu - konec neni potreba resit
                        $konecOK = true;
                    } else if ($zacatekUpraven && $sloupecPrichod->prepsatOperace) {
                        // zaznam konci drive - promaze vsechny bezprostredne navazujici zaznamy, ktere maji stejnou operaci,
                        // nebo operaci ze seznamu "prepsatoperace"
                        // a nasledne prodlouzi tento zaznam tak, aby promazane operace nahradil

                        // zapamatuje si puvodni casOd a casDo
                        $zaznam_casod = $rec->fDatumCasPocatku->value;
                        $zaznam_casdo = $rec->fDatumCasUkonceni->value;
                        $zaznam_operace = $rec->fOperace->value;

                        $casDo = $zaznam_casdo;

                        $prepsaniOK = false;
                        while (!$prepsaniOK && $rec->next()) {
                            if ($rec->fDatumCasPocatku->value == $casDo &&
                                ($rec->fOperace->value == $zaznam_operace || $this->operaceJeVSeznamu($rec->fOperace->value, $sloupecPrichod->prepsatOperace))
                            ) {
                                // interval bezprostredne navazuje a operace je v seznamu operaci, ktere se maji prepsat
                                $casDo = $rec->fDatumCasUkonceni->value;
                                $deleteParams = $this ->database ->createParameters();
                                $rec->deleteRecord(
                                    'DatumCasPocatku = ' . $deleteParams ->addParameter($this ->database ->timestampToParam($rec->fDatumCasPocatku->value)),
                                    $deleteParams);
                            } else {
                                // nasel operaci, ktera se nema prepisovat, nebo prazdny interval
                                $prepsaniOK = true;
                            }
                        }

                        if ($casDo != $zaznam_casdo) {
                            // doslo k promazani navazujicich operaci -> prodlouzi prvni zaznam tak, aby pokryl smazane zaznamy
                            $newRec = new DochazkaRecord($this ->database, $this->pp, $this);
                            $updateParams = $this ->database ->createParameters();
                            $filtr =
                                'DatumCasPocatku = ' . $updateParams ->addParameter($this ->database ->timestampToParam($zaznam_casod));
                            if (!$newRec->loadRecord($filtr, $updateParams)) {
                                return MessageManager::addErrorMessage('Chyba ve funkci ulozPrichodOdchod - nelze nacist zaznam pro upravu koncoveho casu!');
                            }

                            if ($casDo >= $hodnotaOdchod) {
                                $casDo = $hodnotaOdchod;
                                $konecOK = true;
                                $konecUpraven = true;
                            }

                            $newRec->checkMultiple = false; // jinak by rval, ze neni zadan stejny filtr jako pri load
                            $newRec->nastavCasOdchodu($casDo, !$sloupecOdchod ->zobrazenyCas);
                        }
                    }
                }
            }
        }

        if ($zacatekOK  &&  !$konecOK) {
            // jeste musi upravit konec - stejny algoritmus jako pro zacatek, ale odzadu. Pripadne promazani (napr. prestavek) neprovadi

            $rec = new DochazkaRecord($this ->database, $this ->pp, $this);
            $rec ->logInfoMessage = sprintf(
                'Editace prichodu/odchodu pro den %s (%s - %s)',
                date('j.n.Y', $radek ->datum), $radek ->texty[$indexPrichod], $radek ->texty[$indexOdchod]);
            $rec ->orderBy = 'DatumCasPocatku DESC';

            $params = $this ->database ->createParameters();
            $rec ->loadRecords(
                //' Den = ' . $params ->addParameter($this ->database ->dateToParam($radek ->datum)) . ' AND ' . // - odstraneno 20.12.2022 - pokud existovaly zaznamy patrici do jineho (nasledujiciho) dne, tak mohlo dojit k prekryvani
                'DatumCasUkonceni >= ' . $params ->addParameter($this ->database ->timestampToParam($hodnotaPrichod)) .
                ' AND DatumCasPocatku <= ' . $params ->addParameter($this ->database ->timestampToParam($hodnotaOdchod)),
                $params);

            while ( !$konecOK  &&  $rec ->next() ) {
                if ( $this ->operaceJeVSeznamu($rec ->fOperace ->value, $sloupecOdchod ->zapocitatOperace) ) {
                    // nasel posledni zapocitavanou operaci - opravi konec
                    $datumCasUkonceniOriginal = $sloupecOdchod ->zobrazenyCas
                        ? $rec->fDatumCasUkonceni ->originalValue
                        : $rec->getZapoctenyCasDo(true);

                    if ($datumCasUkonceniOriginal > $hodnotaOdchod) {
                        // useknuti posledni operace (posunuti odchodu)
                        if ( !$rec ->nastavCasOdchodu($hodnotaOdchod, !$sloupecOdchod ->zobrazenyCas) ) {
                            return MessageManager::addErrorMessage('Chyba ve funkci ulozPrichodOdchod - chyba pri zmene casu odchodu!');
                        }
                        $konecUpraven = true;
                    } else if ($datumCasUkonceniOriginal < $hodnotaOdchod) {
                        // posledni operace konci drive => je potreba prodlouzit, nebo doplnit zaznam na konec
                        if ($rec->fDatumCasUkonceni ->originalValue >= $hodnotaOdchod) {
                            // zapocitany cas konci drive, ale zobrazeny konci pozdeji => prodlouzit zapocitany cas (zkratit zobacek)
                            $rec->nastavCasOdchodu($hodnotaOdchod, true);
                        } else if ($sloupecOdchod ->operace) {
                            if ($rec ->fDatumCasPocatku ->value == $rec ->fDatumCasUkonceni ->value) {
                                // specialni pripad - zaznam zacina i konci ve stejny cas - je potreba ho zrusit, aby se mohl vlozit novy zaznam
                                $deleteParams = $this ->database ->createParameters();
                                $rec->deleteRecord(
                                    'DatumCasPocatku = ' . $deleteParams->addParameter($this->database->timestampToParam($rec->fDatumCasPocatku->value)),
                                    $deleteParams);
                            }

                            // je nastaveno, ze se ma vzdy vkladat novy zaznam se zadanou operaci
                            $newRec = new DochazkaRecord($this ->database, $this ->pp, $this);

                            $casOd = $rec ->fDatumCasUkonceni ->value;
                            if ($casOd < $hodnotaPrichod) $casOd = $hodnotaPrichod; // nemelo by byt potreba, ale pro jistotu

                            if ( !$newRec ->vlozZaznamDochazky($radek ->datum, $casOd, $hodnotaOdchod, $sloupecOdchod ->operace) ) {
                                return MessageManager::addErrorMessage('Chyba ve funkci ulozPrichodOdchod - nelze vlozit novy zaznam podle zadaneho odchodu!');
                            }

                            if ($rec ->fDatumCasUkonceni ->originalValue > $datumCasUkonceniOriginal) {
                                // jeste odstranit zobacek v existujici operaci
                                $rec ->nastavCasOdchodu($rec ->fDatumCasUkonceni ->originalValue, true);
                            }
                        } else {
                            if ( !$rec ->nastavCasOdchodu($hodnotaOdchod, !$sloupecOdchod ->zobrazenyCas) ) {
                                return MessageManager::addErrorMessage('Chyba ve funkci ulozPrichodOdchod - nelze zmenit cas ukonceni podle zadaneho odchodu!');
                            }
                        }

                        $konecUpraven = true;
                    }

                    $konecOK = true;
                }
                else {
                    // tato operace se nezapocitava - zkrati ji, prerusi nebo uplne smaze
                    if ( !$rec ->odstranZIntervalu($hodnotaPrichod, $hodnotaOdchod) ) {
                        return false;
                    }
                }
            }

            if ( !$konecOK) {
                // TODO: pokud je zapocitany cas zkraceny a interval se upravuje v ramci toho zkraceni, tak dojde sem!
                MessageManager::addErrorMessage('Je potreba doplnit operaci!');
            }

        } else if (!$zacatekOK) {
            // nenasel zaznam pro editaci, je potreba vlozit novy zaznam
            $newRec = new DochazkaRecord($this ->database, $this ->pp, $this);

            $operace = $sloupecPrichod ->operace;
            if ( trim($operace ?? '') == '' ) {
                $operace = $sloupecOdchod ->operace;
            }

            if ( trim($operace ?? '') == '' ) {
                // u sloupce neni zadana operace - pouzije prvni operaci ze zapocitavanych
                $poma = explode(';', $sloupecPrichod ->zapocitatOperace ?? '');
                if ( isset($poma[0]) ) {
                    $operace = trim($poma[0]);
                }

                if (trim($operace) == '') {
                    return MessageManager::addErrorMessage('Nelze vlozit novy zaznam v nastaveni sloupce neni vybrana operace!');
                }
            }

            if ( !$newRec ->vlozZaznamDochazky($radek ->datum, $hodnotaPrichod, $hodnotaOdchod, $operace) ) {
                return MessageManager::addErrorMessage('Chyba ve funkci ulozPrichodOdchod - nelze vlozit novy zaznam pro zadany interval!');
            }
            $zacatekUpraven = true;
            $konecUpraven = true;
        }


        // byla provedena nejaka zmena
        if ($pocetSmazanych  ||  $zacatekUpraven  ||  $konecUpraven) {
            if (!$this ->datumPrvniZmeny  ||  $this ->datumPrvniZmeny > $radek ->datum) {
                $this ->datumPrvniZmeny = $radek ->datum;
            }

            // zapise aktualizovane hodnoty i do dat, aby se uz pri dalsim POST zbytecne nezpracovavaly
            $radek ->texty[$indexPrichod] = $sloupecPrichod ->vratTextProHodnotu($hodnotaPrichod);
            $radek ->textyPuvodni[$indexPrichod] = $radek ->texty[$indexPrichod];

            $radek ->texty[$indexOdchod] = $sloupecOdchod ->vratTextProHodnotu($hodnotaOdchod);
            $radek ->textyPuvodni[$indexOdchod] = $radek ->texty[$indexOdchod];
        }

        // POZOR! Jeste nebyl volan prepocet dochazky! Ve skriptu tedy nelze porovnavat napr. odpracovanou dobu pred a po!
        EditaceUtils::editaceDochazkyEvent($this ->database, $this ->pp, $radek ->datum, 'before');

        return true;
    }


    /**
     * Pokud doslo ke zmene nejakeho denniho vysledku, provede prepocet a vraci true. Jinak vraci false.
     * @return bool
     */
    function provedPrepocet()
    {
        if (!$this ->datumPrvniZmeny  &&  !$this ->jeZmenaVMesicnich) {
            // neni nic potreba delat
            return false;
        }

        $datumOd = ($this ->jeZmenaVMesicnich || !$this ->datumPrvniZmeny)
            ? $this ->datumOd
            : $this ->datumPrvniZmeny;

        // pokud nedoslo k editaci dennich zaznamu, neni potreba vkladat prestavky
        $vkladatprestavky = $this ->datumPrvniZmeny ? $this ->vkladatPrestavky : false;

        AgentManager::prepocetDochazky(
            $this ->pp ->idPP,
            $datumOd, $this ->datumDo, // DateUtils::lastDayOfMonth($datumOd),
            $vkladatprestavky);

        return true;
    }


    /**
     * @return bool
     */
    function zapisZmeneneHodnoty()
    {
        if ($this ->obdobiUzavreno) {
            return true;
        }

        $result = true;

        $this->datumPrvniZmeny = 0;

        foreach ($this->f_data as $radek) {
            $indexyPrichodOdchod = [];

            $indexPrichod = null; // index sloupce (prichod)
            $indexOdchod = null;
            // zapamatuje si prichod a odchod a potom jej zmeni hromadne
            // (musi to tak byt, aby se dalo osetrit zadani smeny pres pulnoc)

            foreach ($this->sloupce as $indexSloupce => $sloupec) {
                if (!$sloupec->lzeZobrazit) {
                    // nezobrazovany sloupec - ignoruje ho
                } else if ($sloupec->typ == 'DOCH1') {
                    if (!isset($indexPrichod) || $sloupec->lzeEditovat) {
                        if (!isset($indexyPrichodOdchod[$sloupec->name])) {
                            $indexyPrichodOdchod[$sloupec->name] = ['doch1' => $indexSloupce];
                        } else {
                            $indexyPrichodOdchod[$sloupec->name]['doch1'] = $indexSloupce;
                        }
                        //$indexPrichod = $indexSloupce;
                    }
                } else if ($sloupec->typ == 'DOCH2') {
                    if (!isset($indexOdchod) || $sloupec->lzeEditovat) {
                        if (!isset($indexyPrichodOdchod[$sloupec->name])) {
                            $indexyPrichodOdchod[$sloupec->name] = ['doch2' => $indexSloupce];
                        } else {
                            $indexyPrichodOdchod[$sloupec->name]['doch2'] = $indexSloupce;
                        }
                        //$indexOdchod = $indexSloupce;
                    }
                } else if ($sloupec->lzeEditovat && ($radek->texty[$indexSloupce] != $radek->textyPuvodni[$indexSloupce])) {
                    // hodnota v bunce je zmenena - provede update
                    if ($this->lzeEditovatDen($radek->datum)) {
                        if ($this->ulozHodnotu($radek, $indexSloupce, $radek->texty[$indexSloupce])) {
                            $radek->textyPuvodni[$indexSloupce] = $radek->texty[$indexSloupce];
                        } else {
                            $result = false;
                        }
                    }
                }
            }

            // nakonec jeste prichod a odchod
            if ($this ->pravaEditace ->lzeZobrazitPP($this ->pp ->idPP, $radek ->datum)) {
                foreach ($indexyPrichodOdchod as $indexy) {
                    $this->ulozPrichodOdchod($radek, $indexy['doch1'], $indexy['doch2']);
                }
            }
        }

        //$this ->ulozPrichodOdchod($radek, $indexPrichod, $indexOdchod);

        return $result;
    }


//    function nastavViditelnostSloupcu()
//    {
//        if ($this ->filterLinks ->isEmpty()) return;
//
//        $this ->filterLinks ->loadGet();
//
//        foreach ($this ->sloupce as $sloupec) {
//            if ($sloupec ->filtrKey  &&  $sloupec ->lzeZobrazit) {
//                if (!$this ->filterLinks ->itemIsActive($sloupec ->filtrKey)) {
//                    $sloupec ->lzeZobrazit = false;
//                }
//            }
//        }
//    }

    function nactiXmlFormulare()
    {
        $variables = [
            'idPP' => $this ->pp ->idPP,
            'RC' => $this ->pp ->rc,
            'osobniCislo' => $this ->pp ->osobniCislo,
            'datumNastupu' => $this ->pp ->datumNastupu,
            'jmeno' => $this ->pp ->jmeno,
            'prijmeni' => $this ->pp ->prijmeni,
            'jmenoPrijmeni' => $this ->pp ->jmenoPrijmeni,
            'obdobi' => $this ->datumOd
        ];

        $this ->xmlFormulare ->processXMLForms($this ->database, $variables);
//        if ($this ->xmlFormulare) {
//            foreach ($this->xmlFormulare as $xml) {
//                if ($xml->idXmlFormulare) {
//                    $xml->obsah = RonPortalXMLForm::xmlEvent($this->database, $xml->idXmlFormulare, $variables, null, true, false);
//                }
//            }
//        }
    }


    function nactiData($datumOd, $datumDo)
    {
        $this ->kalendar = Kalendar::vratKalendarOsoby($this ->database,
            $this ->pp ->rc, $this ->pp ->osobniCislo, $this ->pp ->datumNastupu,
            $datumOd, $datumDo);

        $this ->dochazkaZpracovanaDo = DochazkaUtils::datumCasZpracovaniDochazky($this ->database);
        $this ->obdobiUzavreno = Obdobi::jeObdobiUzavreno($this ->database, 'ADS', $datumOd, $datumDo);

        $this ->inicializujData($datumOd, $datumDo);

        $this ->inicializujSloupce();
        $this ->nactiSchvaleni(); // az za inicializujData (kvuli datumum)!

        if (!$this ->obdobiUzavreno) {
            $this ->pravaEditace = Prava::vratPrava('editace');
            $this ->jePravoEditovat = $this ->pravaEditace ->lzeZobrazitPP($this ->pp ->idPP, $datumOd, $datumDo);

            $prava = Prava::vratPrava('editacepozn');
            $this ->jePravoEditovatPoznamku = $prava ->lzeZobrazitPP($this ->pp ->idPP, $datumOd, $datumDo);
        }

        $this ->nactiXmlFormulare();

        $this ->nactiDataZDatabaze();
    }


    /**
     * @param int $datum
     * @return bool
     */
    function lzeEditovatDen($datum) {
        if (!$this ->schvaleni ->lzeEditovatPP($this ->pp ->idPP, $datum)) {
            return MessageManager::addErrorMessage('Nelze editovat schvalenou dochazku!'); // nemelo by nastat
        }

        return true;
    }


    /**
     * @param int[] $datumy
     * @throws NeniPravoException
     * @throws Exception
     */
    function smazVybraneDny($datumy)
    {
        MessageManager::addDebugMessage('Hromadne smazani pro dny: ' . $_POST['vybranedny']);

        foreach ($datumy as $datum) {
            if (!$this->lzeEditovatDen($datum) || !$this->pravaEditace->lzeZobrazitPP($this->pp->idPP, $datum, $datum)) {
                throw new NeniPravoException();
            }

            $rec = new DochazkaRecord($this->database, $this->pp, $this);
            $rec->logInfoMessage = 'Smazani dochazky pro den ' . date('j.n.Y', $datum);

            $params = $this->database->createParameters();
            $rec->smazZaznamyDochazky(
                ' DatumCasPocatku >= ' . $params->addParameter($this->database->dateToParam($datum)) .
                ' AND DatumCasPocatku < ' . $params->addParameter($this->database->dateToParam(DateUtils::incDay($datum, 1))),
                $params
            );

            if (!$this->datumPrvniZmeny || $this->datumPrvniZmeny > $datum) {
                $this->datumPrvniZmeny = $datum;
            }
        }

        MessageManager::addDebugMessage('Nastavil jsem datumPrvniZmeny = ' . date('j.n.Y', $this->datumPrvniZmeny));
    }


    function smazatVse()
    {
        MessageManager::addDebugMessage('Hromadne smazani pro obdobi: ' . date('j.n.Y', $this ->datumOd) . ' - ' . date('j.n.Y', $this ->datumDo));

        // najde prvni neschvaleny den, ktery uzivatel muze editovat
        $datumOd = $this ->schvaleni ->prvniEditovatelnyDen($this ->pp ->idPP, $this ->datumOd, $this ->datumDo);
        if (!$datumOd) {
            // jiz je schvaleno, nelze editovat zadny den
            return;
        }

        // najde prvni den, pro ktery muze uzivatel editovat zadany PP
        while ($datumOd <= $this ->datumDo  &&  !$this->pravaEditace->lzeZobrazitPP($this->pp->idPP, $datumOd, $datumOd)) {
            $datumOd = DateUtils::incDayStart($datumOd);
        }

        // datumDo nastavi ZA(!) posledni editovatelny den
        $datumDo = DateUtils::incDayStart($datumOd);
        while ($datumDo <= $this ->datumDo  &&  $this->pravaEditace->lzeZobrazitPP($this->pp->idPP, $datumDo, $datumDo)) {
            $datumDo = DateUtils::incDayStart($datumDo);
        }

        if ($datumDo > $datumOd) {
            $rec = new DochazkaRecord($this->database, $this->pp, $this);
            $rec->logInfoMessage = 'Smazani dochazky pro obdobi ' . date('j.n.Y', $datumOd) . ' - ' . date('j.n.Y', $datumDo);

            $params = $this->database->createParameters();
            $rec->smazZaznamyDochazky(
                ' Den >= ' . $params->addDateParameter($datumOd) .
                ' AND Den < ' . $params->addDateParameter($datumDo),
                $params
            );

            if (!$this->datumPrvniZmeny || $this->datumPrvniZmeny > $datumOd) {
                $this->datumPrvniZmeny = $datumOd;
            }
        }
    }


    /**
     * @throws NeniPravoException
     * @throws DBException
     * @throws Exception
     */
    protected function smazPlanovaneSmeny($datumy)
    {
        $prava = Prava::vratPrava('kapacity_edit');
        foreach ($datumy as $datum) {
            if (!$this->lzeEditovatDen($datum) || !$prava->lzeZobrazitPP($this->pp->idPP, $datum, $datum)) {
                throw new NeniPravoException();
            }

            KapacityUtils::zkontrolujSchvaleni($this ->database, $this ->pp, $datum, false);

            $rec = new PlanovaniKapacitRecord($this->database, $this ->pp, $this);
            $rec->logInfoMessage = 'Smazani naplanovane smeny pro den ' . date('j.n.Y', $datum);
            $rec->smazNaplanovaneSmeny($datum, $datum, false); // rezim uprav prozatim false. TODO: nejak vyresit rezim uprav?

            if (RonPortalSettings::getSpousteniADS()  &&  (int)$_POST['pk_zrusitsmenu_prepocitat']) {
                $params = 8 .  // uloha - podle puvodni
                    ' -interval:' . date('Y-n-j', $datum) . ';' . date('Y-n-j', $datum) .
                    ' -IDPP:' . $this ->pp ->idPP;

                AgentManager::executeADS('ADS_Agent_Processing.exe', $params );
            }
        }
    }


    function vratVybraneDatumy()
    {
        $datumy = [];
        $datumyStr = explode(';', $_POST['vybranedny']);
        foreach ($datumyStr as $datumStr) {
            if (trim($datumStr) != '') {
                $datum = DateUtils::dateStringToTimestamp($datumStr);

                if (!$datum) {
                    Log::addRecordToLog($this ->database,
                        'Chyba pri hromadnem operaci', 'DenniVysledky',
                        sprintf("Spatne zadane datum: %s \nvybranedny: %s", $datumStr, $_POST['vybranedny']));
                    MessageManager::addErrorMessage('Chyba pri hromadnem mazani - spatne zadane datum!');
                    return null;
                }

                $datumy[] = $datum;
            }
        }

        return $datumy;
    }

    /**
     * Provede operaci s vybranymi radky
     */
    function zpracujVybraneRadky()
    {
        if (!RonPortalSettings::getIntValue(RonPortalSettings::EDITACE_POVOLENA)) {
            return;
        }

        if (!isset($_POST['vybranedny']) || $_POST['vybranedny'] == '') {
            return;
        }

        $datumy = $this->vratVybraneDatumy();
        if (!$datumy) {
            return;
        }

        if (isset($_POST['smazatvybrane']) && $_POST['smazatvybrane']) {
            // smazani vybranych radku
            MessageManager::addDebugMessage('Hromadne smazani pro dny: ' . $_POST['vybranedny']);
            $this->smazVybraneDny($datumy);
        }

        if (isset($_POST['pk_zrusitsmenu']) && $_POST['pk_zrusitsmenu']) {
            $this->smazPlanovaneSmeny($datumy);
        }
    }




//    function nactiAZapisZmeneneHonoty()
//    {
//        if ($this ->obdobiUzavreno) {
//            return;
//        }
//        if (!$this ->jePravoEditovat  &&  !$this ->jePravoEditovatPoznamku) {
//            return;
//        }
//
//        if (isset($_POST['zpracovana_dochazka'])) if ($_POST['zpracovana_dochazka']) {
//            if ($this ->nactiPost()) {
//                if ($this ->jeChyba) {
//                    MessageManager::addErrorMessage( TextManager::_('Spatna_hodnota') );
//                    return;
//                } else {
//
//                    // v teto promenne bude prvni datum, ve kterem doslo ke zmene vysledku
//                    // nasledne se provede prepocet dochazky od tohoto data
//                    $this ->zapisZmeneneHodnoty();
//                }
//            }
//        }
//    }


    /**
     * Vraci true, pokud je mozne pouzit/zobrazit tlacitko v zadany den (pripadne v aktualnim obdobi)
     * @param TlacitkoZpracovaneDochazky $tlacitko
     * @param int $datum    Pokud je zadano, kontroluje se pro zadany datum, jinak v celem obdobi
     * @return bool
     */
    function lzePouzitTlacitko($tlacitko, $datum = null)
    {
        if ($tlacitko ->kalendar) {
            if (!$this ->jeVKalendari($tlacitko ->kalendar, $datum)) {
                return false;
            }
        }

        if ($tlacitko ->profil) {
            if (!User::jeVProfilu($tlacitko ->profil)) {
                return false;
            }
        }

        return true;
    }

    /**
     * vraci true, pokud uspesne probehne prepocet
     * @return bool
     */
    function zpracujTlacitkoPrepocet()
    {
        $provestPrepocet = false;
        $kontrola = false;

        if (isset($_POST['prepocet'])  &&  $_POST['prepocet']) {
            $provestPrepocet = true;
        } else if (isset($_POST['kontrola'])  &&  $_POST['kontrola']) {
            $provestPrepocet = true;
            $kontrola = true;
        }

        // kontrola jestli byl stisknut prepocet
        if ($provestPrepocet) {
            // spusteni prepoctu na povel uzivatele
            AgentManager::prepocetDochazky(
                $this ->pp ->idPP,
                $this ->datumOd, $this ->datumDo, // DateUtils::lastDayOfMonth($this ->datumOd),
                $this ->vkladatPrestavky,
                $kontrola);

            return true;
        }

        return false; // nic neprobehlo
    }


    /**
     * Vraci true, pokud byl spusten nejaky skript
     * @return bool
     */
    function zpracujTlacitkoSkript()
    {
        foreach ($this ->tlacitka as $tlacitko) {
//            var_dump($tlacitko);
            //if ($tlacitko ->typ == 'skript'  )

            if ($tlacitko ->typ == 'skript'  &&  $tlacitko ->id  &&  isset($_POST['skript_' . $tlacitko ->id])  &&  $_POST['skript_' . $tlacitko ->id]) {
                if ($tlacitko ->scriptName) {
                    AgentManager::executeScript(
                        $tlacitko ->scriptCategory, $tlacitko ->scriptName,
                        'idpp=' . $this ->pp ->idPP . ';obdobi=' . date('j.n.Y', $this ->datumOd) );

                    return true;
                }

                return false;
            }
        }

        return false;
    }


    /**
     * Vraci true, pokud byla spustena nejaka XML udalost
     * @return bool
     */
    function zpracujTlacitkoXML()
    {
        foreach ($this ->tlacitka as $tlacitko) {
            if ($tlacitko ->typ == 'xml'  &&  $tlacitko ->id  &&  isset($_POST['xml_' . $tlacitko ->id])  &&  $_POST['xml_' . $tlacitko ->id]) {
                if ($tlacitko ->xmlId) {
                    $variables = [
                        'idpp' => $this ->pp ->idPP,
                        'RC' => $this ->pp ->rc,
                        'osobniCislo' => $this ->pp ->osobniCislo,
                        'datumNastupu' => $this ->pp ->datumNastupu,
                        'jmeno' => $this ->pp ->jmeno,
                        'prijmeni' => $this ->pp ->prijmeni,
                        'jmenoPrijmeni' => $this ->pp ->jmenoPrijmeni,
                        'obdobi' => $this->datumOd
                    ];
                    RonPortalXMLForm::xmlEvent($this ->database, $tlacitko ->xmlId, $variables, $tlacitko ->id, false);

                    return true;
                }

                return false;
            }
        }

        return false;
    }


    /**
     * @param int $uloha   8 (prepsat vse) nebo 10 (neprepisovat editovane dny)
     * @param int $datumOd
     * @param int $datumDo
     * @return bool
     */
    function prepocetPodlePuvodni($uloha, $datumOd, $datumDo)
    {
        MessageManager::addDebugMessage("Provedu <strong>zpracovani podle puvodni (uloha $uloha)</strong> pro interval " . date('j.n.Y', $datumOd) . ' - ' . date('j.n.Y', $datumDo));

        $params = $uloha .
            ' -interval:' . date('Y-n-j', $datumOd) . ';' . date('Y-n-j', $datumDo) .
            ' -IdPP:' . $this ->pp ->idPP;

        AgentManager::executeADS('ADS_Agent_Processing.exe', $params);
    }


    /**
     * @param int $uloha
     * @param int $datumOd
     * @param int $datumDo
     * @param string $params
     */
    function spustAgentaProInterval($uloha, $datumOd, $datumDo, $params)
    {
        $agentParams = $uloha .  // uloha
            ' ' . $params .
            ' -interval:' . date('Y-n-j', $datumOd) . ';' . date('Y-n-j', $datumDo) .
            ' -IDPP:' . $this->pp->idPP;

        AgentManager::executeADS('ADS_Agent_Processing.exe', $agentParams);
    }


    function spustAgentaProVybraneDny($uloha, $params)
    {
        $datumyStr = explode(';', $_POST['vybranedny']);

        $datumOd = null;
        $datumDo = null;

        foreach ($datumyStr as $datumStr) {
            if ( trim($datumStr) != '' ) {
                try {
                    $datum = DateUtils::dateStringToTimestamp($datumStr);
                } catch (Exception $e) {
                    Log::addRecordToLog($this ->database,
                        'Chyba pri spusteni agenta pro vybrane dny', 'DenniVysledky',
                        sprintf("Spatne zadane datum: %s \nvybranedny: %s", $datumStr, $_POST['vybranedny']));
                    return MessageManager::addErrorMessage('Chyba pri spusteni agenta pro vybrane dny - spatne zadane datum!');
                }

                if (!$this ->lzeEditovatDen($datum)  ||  !$this ->pravaEditace ->lzeZobrazitPP($this ->pp ->idPP, $datum, $datum) ) {
                    throw new NeniPravoException();
                }

                if (!EditaceDneUtils::jePravoEditovatDen($this ->database, $this ->pp, $datum)) {
                    // den obsahuje zaznamy, ktere nelze editovat
                    throw new NeniPravoException();
                }

                // pokud skoncil souvisly interval, vola agenta
                if ($datumOd) {
                    if ($datum == DateUtils::incDayStart($datumDo)) {
                        // pokracovani souvisleho intervalu - jen nastavi $datumDo
                    } else {
                        // preruseni intervalu - vola agenta pro predchozi interval
                        $this ->spustAgentaProInterval($uloha, $datumOd, $datumDo, $params);
                        $datumOd = $datum;
                    }
                } else {
                    // prvni interval
                    $datumOd = $datum;
                }

                $datumDo = $datum;
            }
        }

        if ($datumOd  &&  $datumDo) {
            $this ->spustAgentaProInterval($uloha, $datumOd, $datumDo, $params);
        }

        return true;
    }


    /**
     * @return bool
     */
    function zpracujTlacitkoPodleKalendare()
    {
        foreach ($this ->tlacitka as $tlacitko) {
            if ($tlacitko ->typ == 'podlekalendare'  &&  isset($_POST['podlekalendare_' . $tlacitko ->id])  &&  $_POST['podlekalendare_' . $tlacitko ->id]) {
                if ($this ->lzePouzitTlacitko($tlacitko)) {
                    $params = $tlacitko->agentParameters;
                    if (!$params) {
                        // defaultne cela smena
                        $params = '-delka:0';
                    }
                    $this->spustAgentaProVybraneDny('9', ' -operace:"' . $tlacitko->operace . '" ' . $params);
                }
            }
        }
    }


    /**
     * @param int $casOd Cas pocatku intervalu (pocet sekund od zacatku dne)
     * @param int $casDo Cas konce intervalu (pocet sekund od zacatku dne)
     * @param string $operace
     * @param bool $smazatVse   Pokud je true, smaze se vse v danem dni
     * @param string $kodSmeny     Nepovinne zadana smena
     * @throws Exception
     */
    function vlozIntervalProVybraneDny($casOd, $casDo, $operace, $smazatVse, $kodSmeny = null)
    {
        if (!RonPortalSettings::getIntValue(RonPortalSettings::EDITACE_POVOLENA)) {
            return;
        }

        if (!isset($_POST['vybranedny']) || $_POST['vybranedny'] == '') {
            return;
        }

        $datumy = $this ->vratVybraneDatumy();
        if (!$datumy) {
            return;
        }

        foreach ($datumy as $datum) {
            if ($datum > $this ->dochazkaZpracovanaDo) {
                throw new Exception(TextManager::_('Dochazka_jeste_neni_zpracovana'));
            }
            $rec = new DochazkaRecord($this ->database, $this ->pp, $this);

            // podle smeny zjisti, jestli se ma zapocitavat do dne podle konce
            $datumPodleKonce = false;
            if ($casDo < $casOd  &&  $kodSmeny) {
                // zaznam pres pulnoc - podle smeny urci, jestli se ma zapocitat podle konce
                $smena = Smena::nactiSmenu($this ->database, $kodSmeny);
                if ($smena && $smena ->denDochPodleKonceSmeny) {
                    $datumPodleKonce = true;
                }
            }

            $datumCasOd = DateUtils::dateAndTimeToTimestamp(($casDo < $casOd  &&  $datumPodleKonce) ? DateUtils::incDayStart($datum, -1) : $datum, $casOd);
            $datumCasDo = DateUtils::dateAndTimeToTimestamp(($casDo < $casOd  &&  !$datumPodleKonce) ? DateUtils::incDayStart($datum) : $datum, $casDo);

            $rec ->fDatumCasPocatku ->setNewValue($datumCasOd);
            $rec ->fDatumCasUkonceni ->setNewValue($datumCasDo);

            if ($rec ->kontrolaPrekrytiZaznamuDochazky(true)) {
                if ($smazatVse && $casDo >= $casOd) { // priznak smazat vse (prozatim) podporovan jen pro zaznamy, ktere nejsou pres pulnoc
                    $rec->zkratPrekryvaneZaznamy(true, $datum, DateUtils::incDayStart($datum));
                } else {
                    $rec->zkratPrekryvaneZaznamy(true, $datumCasOd, $datumCasDo);
                }

                $rec->vlozZaznamDochazky($datum, $datumCasOd, $datumCasDo, $operace, $kodSmeny);

                if (!$this->datumPrvniZmeny || $this->datumPrvniZmeny > $datum) {
                    $this->datumPrvniZmeny = $datum;
                }
            }
        }

    }


    function zpracujTlacitkoInterval()
    {
        foreach ($this ->tlacitka as $tlacitko) {
            if ($tlacitko ->typ == 'interval'  &&  isset($_POST['interval_' . $tlacitko ->id])  &&  $_POST['interval_' . $tlacitko ->id]) {
                if ($this ->lzePouzitTlacitko($tlacitko)) {
                    $casOd = DateUtils::timeStringToTimestamp($tlacitko->casOd, 0, true);
                    $casDo = DateUtils::timeStringToTimestamp($tlacitko->casDo, 0, true);

                    $this->vlozIntervalProVybraneDny($casOd, $casDo, $tlacitko->operace, $tlacitko->smazatVse, $tlacitko->smena);
                }
            }
        }
    }


    /**
     * @param int $uroven
     * @param int $hodnota
     * @return bool
     */
    function nastavSchvaleni($uroven, $hodnota)
    {
        $obdobi = new Obdobi();
        if ( !$obdobi ->nactiObdobi($this ->database, 'ADS', $this ->datumOd) ) {
            return MessageManager::addErrorMessage('Chyba - nelze nacist obdobi!');
        }

        try {
            $rec = new SchvaleniDochazkyRecord($this ->database, $this);

            $result = $rec->nastavSchvaleni($this->pp, $obdobi, $uroven, $hodnota);
        } catch (Exception $e) {
            MessageManager::addErrorMessage($e ->getMessage());
            $result = false;
        }

        $this ->nactiSchvaleni();
        // obcerstvi i pri chybe - chyba mohla byt az ve skriptu "after"

        return $result;
    }


    /**
     * Vraci true, pokud uspesne probehne prepocet
     * @return bool
     */
    function zpracujTlacitkoSchvalit()
    {
        if (isset($_POST['schvalit'])  &&  $_POST['schvalit']) {
            return $this ->nastavSchvaleni(null, $this ->datumDo);
        }
        else if (isset($_POST['schvalit1'])  &&  $_POST['schvalit1']) {
            return $this ->nastavSchvaleni(1, $this ->datumDo);
        }
        else if (isset($_POST['schvalit2'])  &&  $_POST['schvalit2']) {
            return $this ->nastavSchvaleni(2, $this ->datumDo);
        }
        else if (isset($_POST['schvalit3'])  &&  $_POST['schvalit3']) {
            return $this ->nastavSchvaleni(3, $this ->datumDo);
        }

        return true;
    }


    /**
     * @return bool
     */
    function zpracujTlacitkoZrusitSchvaleni()
    {
        if (isset($_POST['zrusitschvaleni'])  &&  $_POST['zrusitschvaleni']) {
            $this ->nastavSchvaleni(null, null);
        }
        else if (isset($_POST['zrusitschvaleni1'])  &&  $_POST['zrusitschvaleni1']) {
            return $this ->nastavSchvaleni(1, null);
        }
        else if (isset($_POST['zrusitschvaleni2'])  &&  $_POST['zrusitschvaleni2']) {
            return $this ->nastavSchvaleni(2, null);
        }
        else if (isset($_POST['zrusitschvaleni3'])  &&  $_POST['zrusitschvaleni3']) {
            return $this ->nastavSchvaleni(3, null);
        }

        // bylo stisknuto "zrusit schvaleni"


        return true;
    }

}
