<?php
/**
 * pavel.rudol, 20.10.2019
 * Formular pro zapis rozdeleni dennich vysledku pro jednu Osobu
 * Jeden radek pro kazde rozdeleni + casovou slozku, ve sloupcich jsou editovatelne hodiny pro jednotlive dny
 */

namespace RonPortal\Editace;

use Ron\Controls\ComboBox;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DebugException;
use Ron\MessageManager;
use Ron\MVC\ViewObject;
use Ron\ProxyArray;
use Ron\StringUtils;
use RonPortal\Dochazka\DenniVysledek;
use RonPortal\Dochazka\SchvaleniDochazky;
use RonPortal\NastaveniUzivatele;
use RonPortal\NeniPravoException;
use RonPortal\Obdobi;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\RonPortalSettings;
use RonPortal\System\AgentManager;
use RonPortal\TypRozdeleni;

class ZaznamRozdeleniDennichVysledku extends ZaznamRozdeleniVysledku
{
    /** @var string */
    public $hodinyStr;

    /** @var bool   Priznak, ze byl zaznam zmenen v POST - bude se provadet update */
    public $jeZmena = false;

    /** @var bool   Priznak, ze byl zaznam nacten pred POST - radek jiz v db existuje */
    public $zapsano = false;

    public function nastavTexty()
    {
        $this ->hodinyStr = $this->hodiny
            ? StringUtils::format('minuty', round($this->hodiny * 60))
            : '';
    }
}

class RadekRozdeleniDV
{
    public $rozdeleni;
    public $casovaSlozka;

    public $popisRozdeleni;
    public $popisCasoveSlozky;

    /** @var bool   Priznak, ze se jedna o novy radek */
    public $novyRadek;
    /** @var bool Priznak, ze se ma radek smazat */
    public $smazatRadek;
    /** @var bool Priznak, ze se maji k hodinam pricist zbyvajici odpracovane hodiny */
    public $doplnOdpracovanouDobou;

    /** @var  ZaznamRozdeleniDennichVysledku[]   Zaznam pro kazdy den editovaneho obdobi */
    public $zaznamy;
}


class RozdeleniDVForm implements  ViewObject
{
    /** @var Database */
    public $database;

    /** @var PracovniPomer */
    public $pp;
    /** @var Obdobi */
    public $obdobi;

    /* @var TypRozdeleni */
    public $typRozdeleni;

    /** @var SchvaleniDochazky */
    public $schvaleni;

    /** @var int */
    public $datumOd;
    /** @var int */
    public $datumDo;

    /** @var bool       Nastavi se na true, pokud je napriklad uzavrene obdobi */
    public $readOnly;


    // data
    /** @var RadekRozdeleniDV[] */
    public $radky;
    /** @var float[]    Vypoctene soucty rozdelenych hodin */
    public $soucty;
    /** @var float[]    Nactene odpracovane hodiny za jednotlive dny */
    public $odpracovano;
    /** @var bool[]     Priznak, ze je ve sloupci chyba (prekroceny limit) */
    public $chybySloupcu;
    /** @var bool[]     Priznak, ze je mozne sloupec editovat (tj. neni schvalena dochazka) */
    public $lzeEditovatDen;

    public $renderHtmlForm = true;

    /** @var string */
    private $casovaSlozkaSoucet = null;


    /**
     * RozdeleniDVForm constructor.
     * @param Database $database
     * @param PracovniPomer $pp
     * @param int $datumOd
     * @param int $datumDo
     * @throws \Exception
     */
    public function __construct(Database $database, PracovniPomer $pp, $datumOd, $datumDo)
    {
        $this->database = $database;
        $this->pp = $pp;
        $this->datumOd = $datumOd;
        $this->datumDo = $datumDo;

        $this->typRozdeleni = TypRozdeleni::getTypRozdeleni($database, RonPortalSettings::getIntValue(RonPortalSettings::DV_TYP_ROZDELENI));

        $this ->chybySloupcu = new ProxyArray();
    }


    /**
     * @param RadekRozdeleniDV $radek
     * @param int $indexDne
     * @return bool
     */
    function lzeEditovatZaznam($radek, $indexDne)
    {
        if (!$this ->lzeEditovatDen[$indexDne]) {
            return false;
        }

        if ($radek->zaznamy[$indexDne]  &&  $radek->zaznamy[$indexDne]->rozdeleniSchvaleno) {
            return false;
        }

        return true;
    }

    /**
     * @param ZaznamRozdeleniVysledku $zaznam
     * @return RadekRozdeleniDV
     */
    private function pridejRadek($zaznam)
    {
        $radek = new RadekRozdeleniDV();
        $radek ->rozdeleni = $zaznam ->rozdeleni;
        $radek ->popisRozdeleni = $zaznam ->popisRozdeleni;
        $radek ->casovaSlozka = $zaznam ->casovaSlozka;
        $radek ->popisCasoveSlozky = $zaznam ->popisCasoveSlozky;

        $radek ->zaznamy = new ProxyArray();

        $this ->radky[] = $radek;

        return $radek;
    }


    /**
     * Najde existujici radek k zadanemu zaznamu, nebo prida novy (pokud je $pridatNovy=true)
     * @param ZaznamRozdeleniVysledku $zaznam
     * @param $pridatNovy
     * @return RadekRozdeleniDV
     */
    private function najdiRadek($zaznam, $pridatNovy)
    {
        foreach ($this->radky as $radek) {
            // pokud se shoduji nakonfigurovane sloupce, tak nasel radek
            if (($radek->rozdeleni == $zaznam->rozdeleni) && ($radek->casovaSlozka == $zaznam->casovaSlozka)) {
                return $radek;
            }
        }

        if ($pridatNovy) {
            return $this ->pridejRadek($zaznam);
        }

        return null;
    }


    /**
     * Prida novy radek, ale jen kdyz jeste neexistuje zadny radek s priznakem "novy"
     */
    public function doplnNovyRadek()
    {
        if ($this ->readOnly) return;

        $existuje = false;
        foreach ($this ->radky as $radek) {
            if ($radek ->novyRadek) {
                $existuje = true; // novy radek uz existuje
            }
        }

        if (!$existuje) {
            $radek = new RadekRozdeleniDV();
            $radek->zaznamy = new ProxyArray();
            $radek->novyRadek = true;
            $this->radky[] = $radek;
        }
    }



    /**
     * @param int $indexRadku
     * @return RadekRozdeleniDV
     */
    private function najdiRadekProPost($indexRadku)
    {

        // vytvori a naplni zaznam udaji z POST - slouzi jen pro vyhledani radku v metode najdiRadek()
        $zaznam = new ZaznamRozdeleniDennichVysledku();
        $zaznam->rozdeleni = $_POST['rozdeleni_' . $indexRadku];
        $zaznam->casovaSlozka = $_POST['casova_slozka_' . $indexRadku];

        $result = $this ->najdiRadek($zaznam, false);

        // nacte priznak, ze se jedna o novy radek
        $jeNovyRadek = isset($_POST['novyRadek_' . $indexRadku])  &&  (int)$_POST['novyRadek_' . $indexRadku];
        $smazatRadek = isset($_POST['deleteRowIndex']) && $_POST['deleteRowIndex'] != '' && (int)$_POST['deleteRowIndex'] == $indexRadku;
        $doplnRadek = isset($_POST['remainRowIndex']) && $_POST['remainRowIndex'] != '' && (int)$_POST['remainRowIndex'] == $indexRadku;

        if ($jeNovyRadek) {
            if ($smazatRadek) {
                // novy radek, ktery se nema ukladat
                return null;
            }

            if ($result) {
                // jedna se o nove pridany radek, ale stejny radek uz existuje => chyba
                MessageManager::addErrorMessage(\Ron\ronText('Radek_rozdeleni_vysledku_jiz_existuje'));

                // radek prida, aby uzivatel mohl vybrat jinou slozku
                $result = $this->pridejRadek($zaznam);
                $result->novyRadek = true;
            }
        } else if ($result) {
            // jiz existujici radek - nastavi priznak smazani
            $result ->smazatRadek = $smazatRadek;
        }

        if (!$result  &&  !$smazatRadek) {
            // doplni novy radek
            $result = $this ->pridejRadek($zaznam);
            $result ->novyRadek = true;
        }

        $result->doplnOdpracovanouDobou = $doplnRadek;

        return $result;
    }


    /**
     * Vytvori novy zaznam na zadanem radku v zadanem poli (index dne)
     * @param RadekRozdeleniDV $radek
     * @param int $index
     * @return ZaznamRozdeleniDennichVysledku
     * @throws \Exception
     */
    private function vytvorNovyZaznam($radek, $index)
    {
        if ($radek->zaznamy[$index]) {
            throw new \Exception('Chyba v metode RozdeleniDVForm::vytvorNovyZaznam - zaznam na zadanem indexu uz existuje!');
        }

        $zaznam = new ZaznamRozdeleniDennichVysledku();
        $zaznam->den = DateUtils::incDay($this->datumOd, $index);
        $zaznam->casovaSlozka = $radek->casovaSlozka;
        $zaznam->rozdeleni = $radek->rozdeleni;

        $radek->zaznamy[$index] = $zaznam;
        return $zaznam;
    }


    /**
     * Zpracuje seznam zaznamu vyroby - doplni je na spravna mista
     * @param ZaznamRozdeleniDennichVysledku[] $zaznamy
     */
    private function doplnZaznamy($zaznamy)
    {
        foreach ($zaznamy as $zaznam) {
            $zaznam->nastavTexty();

            $radek = $this ->najdiRadek($zaznam, true); // pridat=true - automaticky prida novy radek, pokud jeste neexistuje
            $index = DateUtils::daysBetween($this ->datumOd, $zaznam ->den);

            $radek ->zaznamy[$index] = $zaznam;
        }
    }

    private function spocitejSoucty()
    {
        $this ->soucty = [];

        $daysCount = DateUtils::daysBetween($this ->datumOd, $this ->datumDo) + 1;
        for ($i = 0;  $i < $daysCount;  $i ++) {
            $this ->soucty[$i] = 0;

            foreach ($this ->radky as $radek) {
                $this ->soucty[$i] += isset($radek ->zaznamy[$i]) ? $radek ->zaznamy[$i] ->hodiny : 0;
            }
        }
    }


    private function nactiSchvaleni()
    {
        $this ->schvaleni = SchvaleniDochazky::nactiSchvaleni($this ->database, $this ->pp, $this ->datumOd);
        $this ->lzeEditovatDen = [];

        $daysCount = DateUtils::daysBetween($this ->datumOd, $this ->datumDo) + 1;
        for ($i = 0;  $i < $daysCount;  $i ++) {
            $this->lzeEditovatDen[$i] = $this ->schvaleni ->lzeEditovatPP($this ->pp ->idPP, DateUtils::incDay($this ->datumOd, $i));
        }
    }

    /**
     * @throws \Exception
     */
    private function nactiDenniVysledky()
    {
        $this ->odpracovano = [];

        $daysCount = DateUtils::daysBetween($this ->datumOd, $this ->datumDo) + 1;
        for ($i = 0; $i < $daysCount; $i++) {
            $this ->odpracovano[$i] = 0;
        }

        $this ->casovaSlozkaSoucet = RonPortalSettings::getValue('dv_casovaslozkasoucet');

        if ($this ->casovaSlozkaSoucet) {
            $denniVysledky = DenniVysledek::nactiDenniVysledky($this->database,
                $this->pp, $this->datumOd, $this ->datumDo, $this ->casovaSlozkaSoucet);

            foreach ($denniVysledky as $denniVysledek) {
                $index = DateUtils::daysBetween($this ->datumOd, $denniVysledek ->den);
                $this ->odpracovano[$index] = $denniVysledek ->hodiny;
            }
        }
    }


    /**
     * Nacte data z databaze
     * @throws \Exception
     */
    public function load()
    {
        $this ->obdobi = Obdobi::vratObdobi($this ->database, 'ADS', $this ->datumOd);

        $this ->nactiSchvaleni();

        $this ->readOnly = $this ->obdobi ->uzavreno;

        Prava::nastavObdobiProPrava($this->datumOd, $this->datumDo);

        $prava = Prava::vratPrava('prohlizeni');
        $lzezobrazit = $prava ->lzeZobrazitPP($this ->pp ->idPP, $this ->obdobi ->zacatek, $this ->obdobi ->konec);

        if (!$this ->readOnly || !$lzezobrazit) {
            // jeste zkontroluje pravo na editaci
            $prava = Prava::vratPrava('editace');

            $jePravoEditovat = $prava ->lzeZobrazitPP($this ->pp ->idPP, $this ->obdobi ->zacatek, $this ->obdobi ->konec);
            if (!$jePravoEditovat) {
                // nemuze editovat, jeste zkontroluje, jestli ma pravo editovat rozdeleni vysledku
                $prava = Prava::vratPrava('editace_r');
                $jePravoEditovat = $prava ->lzeZobrazitPP($this ->pp ->idPP, $this ->obdobi ->zacatek, $this ->obdobi ->konec);
            }

            if ($jePravoEditovat) {
                // ma pravo na editaci => muze zobrazit
                $lzezobrazit = true;
            } else {
                // nema pravo na editaci => readonly
                $this ->readOnly = true;
            }
        }

        if (!$lzezobrazit) { // nema pravo zobrazit ani editovat
            throw new NeniPravoException();
        }

        $this ->radky = [];

        // nacte zaznamy
        /** @var ZaznamRozdeleniDennichVysledku[] $zaznamy */
        $zaznamy = ZaznamRozdeleniDennichVysledku::nactiZaznamyOsoby($this->database,
            RonPortalSettings::getIntValue(RonPortalSettings::DV_TYP_ROZDELENI),
            $this->pp, null,
            $this->datumOd, $this->datumDo);

        // podle nactenych dat vytvori seznam radku a doplni je
        $this ->doplnZaznamy($zaznamy);

        $this ->nactiDenniVysledky();
        $this ->spocitejSoucty();
    }


    /**
     * Doplni zaznamy z predchoziho obdobi
     * @throws \Exception
     */
    public function doplnZaznamyZPredchozihoObdobi()
    {
        $params = $this ->database ->createParameters();
        $sql =
            'SELECT DISTINCT DV.Rozdeleni, DV.TypCasoveSlozkyMzdy' .
            ' FROM DenniVysledek_R_E DV' .
            ' WHERE 1 = 1 ' .
            // zaznamy z predchoziho mesice
            ' AND DV.Den < ' . $params ->addParameter($this ->database ->dateToParam($this ->datumOd)) .
            ' AND DV.Den >= ' . $params ->addParameter($this ->database ->dateToParam(DateUtils::incMonth($this ->datumOd, -1))) .
            // aktualniho zamestnance
            ' AND DV.RC = ' . $params ->addParameter($this ->pp ->rc) .
            ' AND DV.OsobniCislo = ' . $params ->addParameter($this ->pp ->osobniCislo) .
            ' AND DV.DatumNastupu = ' . $params ->addParameter($this ->database ->dateToParam($this ->pp ->datumNastupu)) .
            // ktere jeste neexistuji v aktualnim obdobi
            ' AND NOT EXISTS (' .  //
            '   SELECT * FROM DenniVysledek_R_E DV2' .
            '   WHERE DV2.RC = DV.RC  AND  DV2.OsobniCislo = DV.OsobniCislo  AND  DV2.DatumNastupu = DV.DatumNastupu' .
            '   AND DV2.Den >= ' . $params ->addParameter($this ->database ->dateToParam($this ->datumOd)) .
            '   AND DV2.Den <= ' . $params ->addParameter($this ->database ->dateToParam($this ->datumDo)) .
            ')';

        $datum = DateUtils::today();
        if ($datum < $this ->datumOd) $datum = $this ->datumOd;
        else if ($datum > $this ->datumDo) $datum = $this ->datumDo;

        $platnaRozdeleniSQL = RozdeleniVysledkuUtils::getSQLVyberRozdeleniDV($this->database, $this->pp, $this->typRozdeleni->kod, null, $this->datumOd, $this->datumDo);
        $cursor = $this->database->getSqlResult($platnaRozdeleniSQL);
        $platnaRozdeleni = [];
        while ($row = $cursor->fetch()) {
            $platnaRozdeleni[$row[0]] = $row;
        }
        $cursor ->close();

        $cursor = $this ->database ->getSqlResult($sql, $params);
        while ($row = $cursor ->fetch()) {
            //vlozi se jen rozdeleni, ktera jsou platna v aktualnim mesici (tzn. je mozne je vybrat v selectu)
            if (array_key_exists($row[0], $platnaRozdeleni)) {
                $rec = new RozdeleniDennichVysledkuRecord($this->database, $this->typRozdeleni->kod, $this->pp->rc, $this->pp->osobniCislo, $this->pp->datumNastupu, $datum, $this);

                $rec->fRozdeleni->setNewValue($row[0]);
                $rec->fSlozka->setNewValue($row[1]);
                $rec->fHodnota->setNewValue(0);
                $rec->addField('', '', true, 0, '   ... automaticke doplneni podle predchoziho obdobi');

                $rec->insertRecord();
            }
        }
        $cursor ->close();
    }


    /**
     * Smaze radky, urcene ke smazani a odstrani je ze seznamu (Pokud je v radku schvaleny zaznam, radek v seznamu ponecha.)
     * @throws \Exception
     */
    private function smazRadky()
    {
        for ($i = count($this ->radky) - 1;  $i >= 0;  $i--) {
            if ($this ->radky[$i]->smazatRadek) {
                $smazanyRadek = true;

                foreach ($this ->radky[$i] ->zaznamy as $zaznam) {
                    //zaznamy pro schvalene dny nemaze, pokud zustane v radku zaznam, radek ponecha v seznamu
                    if ($zaznam->rozdeleniSchvaleno || !$this->schvaleni->lzeEditovatPP($this ->pp ->idPP, $zaznam->den)) {
                        $this ->radky[$i]->smazatRadek = false;
                        $smazanyRadek = false;
                    } else {
                        $rozdeleniVysledkuRecord = new RozdeleniDennichVysledkuRecord($this->database,
                            $this->typRozdeleni->kod, $this->pp->rc, $this->pp->osobniCislo, $this->pp->datumNastupu, $zaznam->den, $this);

                        $params = $this->database->createParameters();
                        $rozdeleniVysledkuRecord->deleteRecord('Rozdeleni = ' . $params->addParameter($zaznam->rozdeleni) . ' AND TypCasoveSlozkyMzdy = ' . $params->addParameter($zaznam->casovaSlozka), $params);
                    }
                }

                if ($smazanyRadek)
                    array_splice($this ->radky, $i, 1);
            }
        }
    }

    /**
     * @throws \Exception
     */
    private function saveData()
    {
        $prvniZmena = 0;

        foreach ($this ->radky as $radek) {
            $ulozeno = false;
            foreach ($radek ->zaznamy as $index => $zaznam) {
                if ($this ->chybySloupcu[$index]) {
                    // ve sloupci je chyba (prekroceny limit) => neuklada v nem data
                } else {
                    if ($radek->smazatRadek) {
                        throw new DebugException('Chyba v metode RozdeleniDVForm::saveData - nalezen radek, ktery ma byt smazany!'); // nemelo by nastat
                    }

                    if ($zaznam->jeZmena) {
                        RozdeleniVysledkuUtils::overSchvaleni($this->database, $zaznam->rozdeleni, $zaznam->den);

                        $rozdeleniVysledkuRecord = new RozdeleniDennichVysledkuRecord($this->database,
                            $this->typRozdeleni->kod, $this->pp->rc, $this->pp->osobniCislo, $this->pp->datumNastupu, $zaznam->den, $this);

                        $params = $this->database->createParameters();
                        if ($rozdeleniVysledkuRecord->loadRecord('Rozdeleni = ' . $params->addParameter($zaznam->rozdeleni) . ' AND TypCasoveSlozkyMzdy = ' . $params->addParameter($zaznam->casovaSlozka), $params)) {
                            $rozdeleniVysledkuRecord->fHodnota->setNewValue($zaznam->hodiny);

                            $params = $this->database->createParameters();
                            $rozdeleniVysledkuRecord->updateRecord('Rozdeleni = ' . $params->addParameter($zaznam->rozdeleni) . ' AND TypCasoveSlozkyMzdy = ' . $params->addParameter($zaznam->casovaSlozka), $params);
                            $ulozeno = true;
                        } else {
                            //var_dump($zaznam);
                            $rozdeleniVysledkuRecord->fHodnota->setNewValue($zaznam->hodiny);
                            $rozdeleniVysledkuRecord->fRozdeleni->setNewValue($zaznam->rozdeleni);
                            $rozdeleniVysledkuRecord->fSlozka->setNewValue($zaznam->casovaSlozka);

                            $rozdeleniVysledkuRecord->insertRecord();
                            $ulozeno = true;
                        }

                        if (!$prvniZmena) $prvniZmena = $zaznam->den;
                    }
                }

                if ($zaznam->zapsano) {
                    $ulozeno = true;
                }
            }

            if (!$ulozeno && !empty($radek->rozdeleni)) {
                // novy radek, u ktereho nebyl vyplnen zadny edit => radek se neulozil
                // je potreba doplnit zaznam s nulovou hodnotou - do aktualniho dne nebo nejblize k nemu

                // urci datum, do ktereho se zaznam vlozi
                $datum = DateUtils::today();
                if ($datum < $this ->datumOd) {
                    // edituji budouci obdobi
                    $datum = $this ->datumOd;
                } else if ($datum > $this ->datumDo) {
                    $datum = $this ->datumDo;
                }

                $index = DateUtils::daysBetween($this ->datumOd, $datum);
                $zaznam = $this ->vytvorNovyZaznam($radek, $index);

                $rozdeleniVysledkuRecord = new RozdeleniDennichVysledkuRecord($this->database,
                    $this->typRozdeleni->kod, $this->pp->rc, $this->pp->osobniCislo, $this->pp->datumNastupu,
                    $zaznam->den, $this);
                $rozdeleniVysledkuRecord->fHodnota->setNewValue(is_null($zaznam->hodiny) ? 0 : $zaznam->hodiny);
                $rozdeleniVysledkuRecord->fRozdeleni->setNewValue($zaznam->rozdeleni);
                $rozdeleniVysledkuRecord->fSlozka->setNewValue($zaznam->casovaSlozka);

                $rozdeleniVysledkuRecord->insertRecord();

                if (!$prvniZmena) {
                    $prvniZmena = $zaznam->den;
                }
            }
        }

        if ($prvniZmena) {
            AgentManager::prepocetDochazky($this ->pp ->idPP, $prvniZmena, $this ->obdobi ->konec, false, true);
        }
    }


    /**
     * Zpracuje POST
     * @throws \Exception
     */
    public function process()
    {
        if ($this ->obdobi ->uzavreno) {
            MessageManager::addErrorMessage( \Ron\ronText('Obdobi_uzavreno') );
            return;
        }

        // originalni data uz jsou nactena
        $count = (int)$_POST['rozdelenidv_rows_count'];
        $pocetDnu = DateUtils::daysBetween($this ->datumOd, $this ->datumDo) + 1;
        $this ->chybySloupcu = new ProxyArray();

        $radekProDoplneni = null;
        $indexRadkuDoplneni = null;

        for ($indexRadku = 0;  $indexRadku < $count;  $indexRadku++) {
            $chybaRozdeleni = false;

            $radek = $this ->najdiRadekProPost($indexRadku);

            if ($radek  &&  $radek->doplnOdpracovanouDobou) {
                $radekProDoplneni = $radek;
                $indexRadkuDoplneni = $indexRadku;

                if ('' === trim($radek ->rozdeleni ?? '')) {
                    MessageManager::addErrorMessage(\Ron\ronText('Nelze_zapsat_radek_bez_rozdeleni'), "rozdeleni_{$indexRadku}");
                    $chybaRozdeleni = true;
                }
            }

            if ($radek && !$radek ->smazatRadek) {
                for ($i = 0; $i < $pocetDnu; $i++) {
                    $zaznam = $radek ->zaznamy[$i];
                    if ($zaznam) {
                        $zaznam->zapsano = true;
                    }

                    $originalStr = $zaznam ? $zaznam->hodinyStr : '';
                    $newStr = trim($_POST["hodnota_{$indexRadku}_{$i}"]);

                    $jeZmena = $originalStr != $newStr;

                    if ($jeZmena) {
                        $datum = $zaznam ? $zaznam ->den : DateUtils::incDay($this->datumOd, $i);

                        if ('' === trim($radek ->rozdeleni ?? '')) {
                            if (!$chybaRozdeleni) {
                                MessageManager::addErrorMessage(\Ron\ronText('Nelze_zapsat_radek_bez_rozdeleni'), "rozdeleni_{$indexRadku}");
                                $chybaRozdeleni = true;
                            }
                        } else if (!$this ->schvaleni ->lzeEditovatPP($this ->pp ->idPP, $datum) || ($zaznam && $zaznam->rozdeleniSchvaleno)) {
                            // dochazka uz je schvalena => chyba - nelze editovat
                            MessageManager::addErrorMessage( sprintf(\Ron\ronText('Nelze_editovat_den_dochazka_schvalena'), StringUtils::format('date', $datum)) );
                        } else {
                            if (!$zaznam) {
                                $zaznam = $this->vytvorNovyZaznam($radek, $i);
                            }

                            $zaznam->jeZmena = true;

                            if ($originalStr != $newStr) {
                                $zaznam->hodinyStr = $newStr;

                                try {
                                    $zaznam->hodiny = ($newStr === '')
                                        ? 0
                                        : DateUtils::timeStringToFloat($newStr, true);
                                } catch (\Exception $e) {
                                    MessageManager::addErrorMessage(StringUtils::format('date', $datum) . ': ' . $e->getMessage(), "hodnota_{$indexRadku}_{$i}");
                                }

                                if ($zaznam->hodiny  &&  RonPortalSettings::getIntValue(RonPortalSettings::DV_MINIMALNI_CAS) > 0  &&  60 * $zaznam->hodiny < RonPortalSettings::getIntValue(RonPortalSettings::DV_MINIMALNI_CAS)) {
                                    $zaznam->error_hodiny = sprintf(\Ron\ronText('Nelze_vykazat_mene'), RonPortalSettings::getIntValue(RonPortalSettings::DV_MINIMALNI_CAS));
                                    MessageManager::addErrorMessage( sprintf(\Ron\ronText('Nelze_vykazat_mene'), RonPortalSettings::getIntValue(RonPortalSettings::DV_MINIMALNI_CAS)), "hodnota_{$indexRadku}_{$i}");
                                }
                            }

                        }
                    }
                }
            }
        }

        $this ->spocitejSoucty();

        if (!MessageManager::getInstance() ->isError()) {
            // odstrani radky, ktere se maji smazat
            $this ->smazRadky();

            if (isset($_POST['doplnitRadky'])) {
                $this ->doplnZaznamyZPredchozihoObdobi();
            }

            // kontrola limitu
            if ($this ->casovaSlozkaSoucet) {
                for ($i = 0; $i < $pocetDnu; $i++) {
                    if (round(($this->odpracovano[$i] - $this->soucty[$i]) * 60) < 0) {
                        // chyba - zadano vice hodin, nez je odpracovano
                        $this->chybySloupcu[$i] = true;
                        MessageManager::addErrorMessage(date('j.n.Y', DateUtils::incDay($this->datumOd, $i)) . ': ' . \Ron\ronText('Nelze_rozdelit_vice_dv'));
                    }
                }
            }

            if (!is_null($radekProDoplneni) && !MessageManager::getInstance() ->isError()) {
                for ($i = 0; $i < $pocetDnu; $i++) {
                    if (($this->odpracovano[$i] - $this->soucty[$i]) > 0) {
                        $zaznam = $radekProDoplneni->zaznamy[$i];
                        if ($zaznam) {
                            $zaznam->zapsano = true;
                        }

                        $datum = $zaznam ? $zaznam->den : DateUtils::incDay($this->datumOd, $i);

                        if (!$this->schvaleni->lzeEditovatPP($this->pp->idPP, $datum) || $zaznam->rozdeleniSchvaleno) {
                            // dochazka uz je schvalena => chyba - nelze editovat
                            MessageManager::addErrorMessage(sprintf(\Ron\ronText('Nelze_editovat_den_dochazka_schvalena'), StringUtils::format('date', $datum)));
                        } else {
                            if (!$zaznam) {
                                $zaznam = $this->vytvorNovyZaznam($radekProDoplneni, $i);
                            }

                            $zaznam->jeZmena = true;

                            $zaznam->hodiny += $this->odpracovano[$i] - $this->soucty[$i];

                            if (RonPortalSettings::getIntValue(RonPortalSettings::DV_MINIMALNI_CAS) > 0 && 60 * $zaznam->hodiny < RonPortalSettings::getIntValue(RonPortalSettings::DV_MINIMALNI_CAS)) {
                                $zaznam->error_hodiny = sprintf(\Ron\ronText('Nelze_vykazat_mene'), RonPortalSettings::getIntValue(RonPortalSettings::DV_MINIMALNI_CAS));
                                MessageManager::addErrorMessage(sprintf(\Ron\ronText('Nelze_vykazat_mene'), RonPortalSettings::getIntValue(RonPortalSettings::DV_MINIMALNI_CAS)), "hodnota_{$indexRadkuDoplneni}_{$i}");
                            }
                        }
                    }
                }
            }

            $this ->saveData();
        }
    }

    /**
     * @param RadekRozdeleniDV $radek
     * @param int $cisloRadku
     * @throws \Exception
     */
    public function renderInputRozdeleni($radek, $cisloRadku)
    {
        $inputName = "rozdeleni_$cisloRadku";

        if ($radek ->novyRadek) {
            // priznak "novy radek"
            echo "<input type='hidden' name='novyRadek_$cisloRadku' value='1'/>";

            $sql = RozdeleniVysledkuUtils::getSQLVyberRozdeleniDV($this ->database, $this ->pp, $this ->typRozdeleni->kod, $radek->rozdeleni, $this->obdobi->zacatek, $this->obdobi->konec, false);

            $comboPopis = new ComboBox($inputName);
            $comboPopis ->alwaysShowSelected = true;
            $comboPopis ->emptyRow = (isset($radek->rozdeleni) && $radek->rozdeleni != '') ? null : '&nbsp;';
            $comboPopis ->cssClassIndex = 2;
            $comboPopis ->className = 'form-control ronselect';

            $comboPopis ->htmlParams['id'] = 'vyberRozdeleni';
            $comboPopis ->htmlParams['data-ron-select2'] = $inputName . '_kod';
            $comboPopis ->htmlParams['data-max-visible-rows'] = 200;
            if ($this->readOnly) {
                $comboPopis->htmlParams['disabled'] = 'disabled';
            }

            $comboPopis ->loadData($this ->database, $sql, $radek->rozdeleni);
            $comboPopis->render();

            $sql = RozdeleniVysledkuUtils::getSQLVyberRozdeleniDV($this ->database, $this ->pp, $this ->typRozdeleni->kod, $radek->rozdeleni, $this->obdobi->zacatek, $this->obdobi->konec, true); // podle kodu

            $comboKod = new ComboBox($inputName . '_kod');
            $comboKod ->alwaysShowSelected = true;
            $comboKod ->emptyRow = (isset($radek->rozdeleni) && $radek->rozdeleni != '') ? null : ' ';
            $comboKod ->cssClassIndex = 2;
            $comboKod ->captionIndex = 0;
            $comboKod ->className = 'form-control ronselect';
            $comboKod ->htmlParams['data-columns'] = 'value;popis';
            $comboKod->attrIndexes = ['data-popis' => 1];
            $comboKod->htmlParams['data-columns'] = 'value;popis';
            $comboKod ->htmlParams['data-ron-select2'] = $inputName;

            $comboKod ->htmlParams['data-max-visible-rows'] = 200;
            $comboKod ->htmlParams['id'] = 'vyberRozdeleniKod';
            if ($this->readOnly) {
                $comboKod->htmlParams['disabled'] = 'disabled';
            }

            $comboKod ->loadData($this ->database, $sql, $radek->rozdeleni);

            $comboKod ->render();
        } else {
            // jiz existujici radek - jen text
            echo
                "<input type='hidden' name='$inputName' value='" . StringUtils::htmlTextConvert($radek->rozdeleni) . "'/>" .
                StringUtils::htmlTextConvert($radek->popisRozdeleni) . " (" . StringUtils::htmlTextConvert($radek->rozdeleni) . ")";
        }
    }

    /**
     * @param RadekRozdeleniDV $radek
     * @param int $cisloRadku
     * @throws \Exception
     */
    public function renderInputCasoveSlozky($radek, $cisloRadku)
    {
        $inputName = "casova_slozka_$cisloRadku";

        if ($radek ->novyRadek) {
            $sql = RozdeleniVysledkuUtils::getSQLVyberCasSlozkyDV($this ->database);

            $value = '';

            if (isset($_POST['casova_slozka'])) {
                $value = $_POST['casova_slozka'];
            } else {
                $slozky = explode(';', RonPortalSettings::getValue(RonPortalSettings::DV_CASOVE_SLOZKY));
                if (trim($slozky[0]) != '') $value = trim($slozky[0]); // natvrdo zadano v nastaveni intrawebu
                else {
                    // nacte z nastaveni uzivatele
                    NastaveniUzivatele::getUserSettingsItem($this ->database, NastaveniUzivatele::CASOVA_SLOZKA_ROZDELENI_DV);
                }
            }

            $combo = new ComboBox($inputName);
            $combo ->emptyRow = isset($value) ? false : ' ';

            $combo ->className .= ' ronselect';

            $combo ->loadData($this ->database, $sql, $value);
            $combo ->render();
        } else {
            // jiz existujici radek - jen text
            echo
                "<input type='hidden' name='$inputName' value='" . StringUtils::htmlTextConvert($radek->casovaSlozka) . "'/>" .
                StringUtils::htmlTextConvert($radek->popisCasoveSlozky);
        }
    }

    public function render()
    {
        if (!$this ->readOnly) {
            if (!$this ->radky  ||  (isset($_GET['novyRadek']) && (int)$_GET['novyRadek']) || isset($_POST['novyRadek'])) {
                $this->doplnNovyRadek();
            }
        }

        require 'ronportal/editace/view_rozdeleni_dv_form.php';
    }

}
