<?php

/**
 * Seznam hodnoceni zamestnancu
 */

namespace RonPortal\Hodnoceni;

use Exception;
use Ron\Controls\BrowserField;
use Ron\Controls\LinksList;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\SqlStatement;
use Ron\DB\SQLUtils;
use Ron\Http\App;
use RonPortal\Controls\RonPortalBrowser;
use RonPortal\PredavaciProcesyUtils;
use RonPortal\RonPortalSettings;
use RonPortal\RozdeleniUtils;
use RonPortal\User;
use function Ron\ronText;


/**
 * Odkaz na zobrazeni hodnoceni - podle prav
 * Class OdkazHodnoceniField
 * @package Ron\Personalistika
 */
class OdkazHodnoceniField extends BrowserField
{
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if ($rowData['col-hodnotilrc']  &&  (int)$rowData['col-stavhodnoceni']  &&  (int)$rowData['col-stavhodnoceni'] !== 4) {
            // je vyplneno RC hodnotitele a u hodnoceni jiz byl zmenen stav z 0 (nezpracovano) nebo 4 (vraceno - nezpracovano)
            // hodnoceni uz nekdo vyplnil - bude zde odkaz na zobrazeni
            $link = "?menuid=hodnoceni&record={$rowData[0]}";
            $text = ronText('zobrazit_hodnoceni');
        } else {
            $link = "?menuid=hodnoceni&hodnoceniaction=editujhodnoceni&record={$rowData[0]}";
            $text = ronText('vyplnit_hodnoceni');
        }

        return '<A href="' . $link . '">' . $text . "</A>";
    }
}


/**
 * Odkaz na zobrazeni hodnoceni - podle prav
 * Class VyberCheckboxField
 * @package Ron\Personalistika
 */
class VyberCheckboxField extends BrowserField
{
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        return "<input type='checkbox'  name='hodnocenicheckbox_{$rowData[0]}' />";
    }
}


/**
 * Datum pro zobrazeni hodnoceni - od kdy
 * Class DatumZobrazeniField
 * @package Ron\Personalistika
 */
class DatumZobrazeniField extends BrowserField
{
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if  ($rowData['col-poradi-limitniuroven'] != 1) {
            // je za limitni urovni
            $date = DateUtils::incDay($this ->database ->stringToTimestamp($rowData['col-konecobdobi']), $rowData['col-ucel-presahprovlastnihodnoceni']);

            return date('j.n.Y', $date);
        }

        return '';
    }
}



class HodnoceniBrowser extends RonPortalBrowser
{
    //var $f_pravaeditace;
    //var $f_pravaprohlizeni;

    var $hromadneSchvaleni; // priznak, jestli se ma zobrazit hromadne schvalovani

    /** @var bool   Urcuje, podle ktereho datumu se filtruji osoby (platnost rozdeleni) - jestli podle obdobi zaznamu hodnoceni nebo podle aktualniho dne */
    private $obdobiPravPodleHodnoceni = true;

    var $fieldRozdeleni;

//    /** @var  HodnoceniFilterForm */
//    var $filterForm;

    /**
     * @param Database $database
     */
    function __construct($database)
    {
        parent::__construct($database, 'hodnoceni');

        $this->title = ronText('Prehled_hodnoceni');

        $this->code = 'browserseznamhodnoceni';
        $this->tableId = 'browserseznamhodnoceni';
        $this->browserUrl = '?menuid=hodnoceni&';



        $this->sqlSelectPart =
            'PripraveneTemaHodnoceni.ID'; // 0
            // 'PripraveneTemaHodnoceni.HodnotilRC, PripraveneTemaHodnoceni.HodnocenyRC'; // 1 - 2
            //'TemaHodnoceni.PopisTematu, Osoba.Jmeno, Osoba.Prijmeni'; // 3 - 5
            //'VysledekHodnoceni.PocatekObdobi, VysledekHodnoceni.KonecObdobi, PripraveneTemaHodnoceni.Stav'; // 6 - 8
            //'HodnoceniOsoby.Znamka, PripraveneTemaHodnoceni.Hodnotici, PripraveneTemaHodnoceni.DatumHodnoceni'; // 9 - 11
            //'HodnoceniOsoby.KodProcesu, HodnoceniOsoby.PocatecniUroven, HodnoceniOsoby.PoradiPredavani, HodnoceniOsoby.UrovenPredavani, PoradiPredavani.KodUrovne'; // 12-16
            //'HodnoceniOsoby.IdVysledku, TemaHodnoceni.XMLProZobrazeni, PoradiPredavani.LimitniUroven'; // 17,18,19
            //'(SELECT PresahProVlastniHodnoceni FROM UcelHodnoceni WHERE Kod = VysledekHodnoceni.KodUcelu)'; //20

        $this->sqlFromPart =
            ' ( ( ( ( PripraveneTemaHodnoceni LEFT OUTER JOIN HodnoceniOsoby ' .
            '         ON HodnoceniOsoby.ID = PripraveneTemaHodnoceni.ID ' .
            '       ) ' .
            '       LEFT OUTER JOIN PoradiPredavani ' .
            '       ON PoradiPredavani.KodProcesu = HodnoceniOsoby.KodProcesu ' .
            '          AND PoradiPredavani.PocatecniUroven = HodnoceniOsoby.PocatecniUroven ' .
            '          AND PoradiPredavani.Poradi = HodnoceniOsoby.PoradiPredavani ' .
            '     ) ' .
            '     LEFT OUTER JOIN Osoba ' .
            '     ON Osoba.RC = PripraveneTemaHodnoceni.HodnocenyRC ' .
            '   )' .
            '   LEFT OUTER JOIN VysledekHodnoceni ' .
            '   ON VysledekHodnoceni.Id = HodnoceniOsoby.IdVysledku ' .
            ' )' .
            ' INNER JOIN TemaHodnoceni ' .
            ' ON TemaHodnoceni.DruhHodnoceni = PripraveneTemaHodnoceni.DruhHodnoceni ' .
            '    AND TemaHodnoceni.TemaCislo = PripraveneTemaHodnoceni.TemaCislo ';


        if (RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_ZOBRAZENI_TYP)) {
            // vybere rozdeleni pouze pro HPP platny v posledni den hodnoceneho obdobi
            $this->sqlFromPart =
                '( ( (' . $this->sqlFromPart .
                '    ) ' .
                '    LEFT OUTER JOIN PracovniPomer ' .
                '    ON PracovniPomer.RC = Osoba.RC ' .
                "    AND PracovniPomer.TypPracovnihoPomeru = 'HPP' " .
                '    AND PracovniPomer.DatumNastupu <= VysledekHodnoceni.KonecObdobi ' . // platny na konci hodnoceneho obdobi
                '    AND (PracovniPomer.DatumUkonceni >= VysledekHodnoceni.KonecObdobi  OR  PracovniPomer.DatumUkonceni IS NULL)' .
                '  ) ' .
                '  LEFT OUTER JOIN RozdeleniOsob ' .
                '  ON RozdeleniOsob.RC = PracovniPomer.RC ' .
                '  AND RozdeleniOsob.OsobniCislo = PracovniPomer.OsobniCislo ' .
                '  AND RozdeleniOsob.DatumNastupu = PracovniPomer.DatumNastupu ' .
                '  AND RozdeleniOsob.KodTypuRozdeleni = ' . RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_ZOBRAZENI_TYP) .
                '  AND RozdeleniOsob.PlatnostOd <= VysledekHodnoceni.KonecObdobi ' .
                '  AND (RozdeleniOsob.PlatnostDo >= VysledekHodnoceni.KonecObdobi  OR  RozdeleniOsob.PlatnostDo IS NULL)' .
                ') ' .
                ' LEFT OUTER JOIN Rozdeleni ' .
                ' ON Rozdeleni.KodTypuRozdeleni = RozdeleniOsob.KodTypuRozdeleni ' .
                ' AND Rozdeleni.Kod = RozdeleniOsob.Rozdeleni ';
        }

        // ---------------------------------------------------------------------
        // zjisti do kterych urovni hodnoceni jsem zarazeny (podle mych pracovnich mist)
        $seznamUrovni = PredavaciProcesyUtils::seznamUrovniPredavaniUzivatele($database);
        $this->hromadneSchvaleni = PredavaciProcesyUtils::muzeHromadneSchvalovatHodnoceni($seznamUrovni);

        $params = $this ->database ->createParameters();

        //jen hodnoceni
        $sqlWherePart =
            " PripraveneTemaHodnoceni.DruhHodnoceni = " . $params->addParameter(8);

        // vynechani hodnoceni skoleni
        $sqlWherePart .=
            ' AND NOT EXISTS(SELECT HodnoceniSkoleni.DruhHodnoceni FROM HodnoceniSkoleni ' .
            '   WHERE PripraveneTemaHodnoceni.DruhHodnoceni = HodnoceniSkoleni.DruhHodnoceni ' .
            '     AND PripraveneTemaHodnoceni.TemaCislo = HodnoceniSkoleni.TemaCislo)';

        $this ->sqlWherePart = new SqlStatement($sqlWherePart, $params);

        $this->defaultSort = 'col-pocatekobdobi';
        $this->primaryField = 0;

        // pole pouze pro debug
        $this->addField()
            ->setSqlFieldName('PripraveneTemaHodnoceni.ID') ->setHeader('DEBUG> Id')
            ->setAlign('center') ->setOrdering(true)->setCode('col-idhodnoceni')
            ->setVisible(App::getDebugMode());

        $this->addField()
            ->setSqlFieldName('HodnoceniOsoby.IDVysledku') ->setHeader('DEBUG> IdVysledku')
            ->setAlign('center') ->setOrdering(true)
            ->setCode('col-idvysledku')
            ->setVisible(App::getDebugMode());

        if ($this->hromadneSchvaleni) {
            $this->addField( new VyberCheckboxField() );
        }

        $this->addField()
            ->setSqlFieldName('TemaHodnoceni.PopisTematu') ->setHeaderText('Popis')
            ->setLink("?menuid=hodnoceni")->setOrdering(true)
            ->setCode('col-popistematu');

        // rozdeleni
        if (RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_ZOBRAZENI_TYP)) {
            $coZobrazit = RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_ZOBRAZENI_CO_ZOBRAZIT);
            if ($coZobrazit == 1 || $coZobrazit == 2) {
                // jen kod | kod+popis
                $this->addField()
                    ->setSqlFieldName('Rozdeleni.Kod') ->setHeader(RonPortalSettings::getValue(RonPortalSettings::ROZDELENI_ZOBRAZENI_NADPIS))
                    ->setOrdering(true)
                    ->setSortSQL('Rozdeleni.Kod, Osoba.Prijmeni')
                    ->setCode('col-rozdeleni');
            }

            if ($coZobrazit == 0 || $coZobrazit == 2) {
                // jen popis | kod+popis
                $this->addField()
                    ->setSqlFieldName('Rozdeleni.Popis') ->setHeader(RonPortalSettings::getValue(RonPortalSettings::ROZDELENI_ZOBRAZENI_NADPIS))
                    ->setOrdering(true)
                    ->setSortSQL('Rozdeleni.Popis, Osoba.Prijmeni')
                    ->setCode('col-rozdeleni');
            }
        }

        // jmeno a prijmeni
        $this->addField()
            ->setSqlFieldName('Osoba.Jmeno') ->setHeaderText('Jmeno') ->setOrdering(true)
            ->setCode('col-jmeno')
            ->setSortSQL('Osoba.Jmeno, Osoba.Prijmeni')
            ->setSortSQLDesc('Osoba.Jmeno DESC, Osoba.Prijmeni DESC');

        $this->addField()
            ->setSqlFieldName('Osoba.Prijmeni') ->setHeaderText('Prijmeni') ->setOrdering(true)
            ->setCode('col-prijmeni')
            ->setSortSQL('Osoba.Prijmeni, Osoba.Jmeno')
            ->setSortSQLDesc('Osoba.Prijmeni DESC, Osoba.Jmeno DESC');

        // Obdobi hodnoceni (od-do)
        $this->addField()
            ->setSqlFieldName('VysledekHodnoceni.PocatekObdobi') ->setHeaderText('Datum_od') ->setFormat('date')
            ->setCode('col-pocatekobdobi')
            ->setOrdering(true)
            ->setSortSQL('VysledekHodnoceni.PocatekObdobi, Osoba.Prijmeni, Osoba.Jmeno')
            ->setSortSQLDesc('VysledekHodnoceni.PocatekObdobi DESC, Osoba.Prijmeni, Osoba.Jmeno');

        $this->addField()
            ->setSqlFieldName('VysledekHodnoceni.KonecObdobi') ->setHeaderText('Datum_do') ->setFormat('date')
            ->setCode('col-konecobdobi')
            ->setOrdering(true)
            ->setSortSQL('VysledekHodnoceni.KonecObdobi, Osoba.Prijmeni, Osoba.Jmeno')
            ->setSortSQLDesc('VysledekHodnoceni.KonecObdobi DESC, Osoba.Prijmeni, Osoba.Jmeno');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('(SELECT PresahProVlastniHodnoceni FROM UcelHodnoceni WHERE Kod = VysledekHodnoceni.KodUcelu)')
            ->setCode('col-ucel-presahprovlastnihodnoceni');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('PoradiPredavani.LimitniUroven')
            ->setCode('col-poradi-limitniuroven');

        $this->addField( new DatumZobrazeniField($database) )
            ->setHeaderText('Zobrazit_hodnoceni_od') ->setAlign('right')
            ->setCode('col-zobrazitod');

        // stav
        $this->addField(BrowserField::FIELD_TYPE_SELECT)
            ->setSqlFieldName('PripraveneTemaHodnoceni.Stav') ->setHeaderText('Stav_hodnoceni')
            ->setCode('col-stavhodnoceni')
            ->setOptions([
                0 => ronText('Stav_hodnoceni_0'),
                1 => ronText('Stav_hodnoceni_1'),
                2 => ronText('Stav_hodnoceni_2'),
                3 => ronText('Stav_hodnoceni_3'),
                4 => ronText('Stav_hodnoceni_4'),
                5 => ronText('Stav_hodnoceni_5')
            ])
            ->setOrdering(true);

        $this->addField()
            ->setSqlFieldName('TemaHodnoceni.XMLProZobrazeni') ->setHeader('DEBUG> Id XML')
            ->setAlign('right') ->setOrdering(true)
            ->setCode('col-xmlid')
            ->setVisible(App::getDebugMode());

        // debug - cislo stavu
        $this->addField()
            ->setSqlFieldName('PripraveneTemaHodnoceni.Stav') ->setHeader('DEBUG> Stav')
            ->setCode('col-stav')
            ->setAlign('center') ->setOrdering(true)
            ->setVisible(App::getDebugMode());

        $this->addField()
            ->setSqlFieldName('HodnoceniOsoby.Znamka') ->setHeaderText('Znamka')
            ->setFormat('number_2') ->setOrdering(true)
            ->setCode('col-znamka')
            ->setVisible($this->hromadneSchvaleni); // navrzene znamky vidi jen ti, kteri mohou hromadne schvalovat

        $this->addField( new OdkazHodnoceniField() )->setCode('col-zobrazenihodnoceni'); // odkaz zobrazit/vyplnit

        $this->addField()
            ->setSqlFieldName('PripraveneTemaHodnoceni.Hodnotici') ->setHeaderText('Hodnotil')
            ->setOrdering(true)
            ->setCode('col-hodnotici');

        $this->addField()
            ->setSqlFieldName('PripraveneTemaHodnoceni.DatumHodnoceni') ->setHeaderText('Datum_hodnoceni') ->setFormat('datetime')
            ->setOrdering(true)
            ->setCode('col-datumhodnoceni');

        $this->addField()
            ->setSqlFieldName('HodnoceniOsoby.PoradiPredavani')->setHeaderText('Poradi')
            ->setOrdering(true)
            ->setCode('col-poradi');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('PripraveneTemaHodnoceni.HodnotilRC')
            ->setCode('col-hodnotilrc');

        // Pouze pro DEBUG
        $this->addField()
            ->setSqlFieldName('HodnoceniOsoby.KodProcesu') ->setHeader('Proces') ->setOrdering(true)
            ->setCode('col-proces')
            ->setVisible(App::getDebugMode());
        $this->addField()
            ->setSqlFieldName('HodnoceniOsoby.PocatecniUroven') ->setHeader('Poc.uroven') ->setOrdering(true)
            ->setCode('col-pocatecniuroven')
            ->setVisible(App::getDebugMode());
        $this->addField()
            ->setSqlFieldName('HodnoceniOsoby.UrovenPredavani') ->setHeader('Kat.PM hodnoceni')
            ->setCode('col-urovenpredavani')
            ->setOrdering(true)
            ->setVisible(App::getDebugMode());
        $this->addField()
            ->setSqlFieldName('PoradiPredavani.KodUrovne')->setHeader('Kat.PM dle procesu')->setOrdering(true)
            ->setCode('col-kodurovne')
            ->setVisible(App::getDebugMode());

        $this->maxPageRows = 100;
    }


    public function loadConfigurationItem($key, $value, $valueText)
    {
        if ($key === 'obdobiPravPodleHodnoceni') {
            $this ->obdobiPravPodleHodnoceni = (int)$value;
        } else {
            parent::loadConfigurationItem($key, $value, $valueText);
        }
    }


    function setFilterForm()
    {
        $this ->filterForm = new HodnoceniFilterForm($this ->database, 'hodnoceni', false);
        //$this ->filterForm->zobrazitZnamky = $this->hromadneSchvaleni;

        $this ->filterForm ->temaHodnoceniFilter
            ->setSqlFilter('PripraveneTemaHodnoceni.TemaCislo = ');

        if ($this ->hromadneSchvaleni) {
            $this ->filterForm ->pridejFiltrZnamky();
        }
    }


    /**
     * @param string $sqlFilter
     * @throws Exception
     */
    function load($sqlFilter = '')
    {
        if (isset($_GET['stavhodnoceni'])) {
            $_SESSION['filtrstavhodnoceni'] = (int)$_GET['stavhodnoceni'];
        }

        $stav = HodnoceniFilterForm::getStavHodnoceni();
        if ( !isset($stav) ) {
            $_SESSION['filtrstavhodnoceni'] = -1;  // vychozi hodnota - neukoncene
        }

        parent::load($sqlFilter);
    }


    function renderFilterStav()
    {
        $stav = (int)HodnoceniFilterForm::getStavHodnoceni();

        $data = array(
            -1 => ronText('Neukonceno'),
            0 => ronText('Stav_hodnoceni_0'),
            1 => ronText('Stav_hodnoceni_1'),
            2 => ronText('Stav_hodnoceni_2'),
            3 => ronText('Stav_hodnoceni_3'),
            4 => ronText('Stav_hodnoceni_4'),
            100 => ronText('Stav_hodnoceni_vse')
        );

        LinksList::renderLinks($data, $stav, '?menuid=hodnoceni&stavhodnoceni=');
    }


    function renderFilter()
    {
        $this->renderFilterStav();

        $this ->filterForm ->render();

        if ($this->hromadneSchvaleni) {
            echo
                "<div class='margin-top'>" .
                '<input type="button" class="button" name="vybratvse" id="vyberVsechnaHodnoceni" value="' . ronText('Vybrat_vse') . '" /> ' .
                '<input type="button" class="button" name="vybratvse" id="zrusVyberHodnoceni" value="' . ronText('Zrusit_vyber') . '" /> ' .
                '<span class="margin-left-4"><input type="button" class="button" id="hromadne_odeslat_hodnoceni" name="odeslathodnoceni" value="' . ronText('Odeslat_vybrane') . '"/></span>' .
                "</div>\n";
        }
    }


    function getSQLFilter()
    {
        $params = $this ->database ->createParameters();

        $result =
            $this ->filterForm ->buildSqlFilter($params);

        $stav = (int)HodnoceniFilterForm::getStavHodnoceni();

        // ---------------------------------------------------------------------
        // filtr pro stav
        switch ($stav) {
            case 1 : // rozpracovano
            case 2 : // ukonceno
            case 3 : // ukonceno - vraceno
            case 4 : // nezpracovano - vraceno
            case 5 : // vyrazene
                $result .=
                    ($result ? ' AND ' : '') .
                    'PripraveneTemaHodnoceni.Stav = ' . $stav;
                break;

            case 0 :
                $result .=
                    ($result ? ' AND ' : '') .
                    '(PripraveneTemaHodnoceni.Stav = 0  OR PripraveneTemaHodnoceni.Stav = 4) ';
                break;

            case -1 :
                // neukoncene = pripravene + rozpracovane + vracene k prepracovani
                $result .=
                    ($result ? ' AND ' : '') .
                    '(PripraveneTemaHodnoceni.Stav = 0  OR PripraveneTemaHodnoceni.Stav = 1  OR PripraveneTemaHodnoceni.Stav = 4) ';
                break;

            // 100 = vse
        }

        foreach ($this ->filterForm ->getRozdeleniFilters() as $rozdeleniFilter) {
            if ($rozdeleniFilter ->getValue() !== null) {
                $result .=
                    ($result ? ' AND ' : '') .
                    RozdeleniUtils::sqlFilterRozdeleniPP($this ->database,
                        'Osoba',
                        $rozdeleniFilter ->getTypRozdeleni(), $rozdeleniFilter ->getValue(),
                        'VysledekHodnoceni.PocatekObdobi', 'VysledekHodnoceni.KonecObdobi',
                        $rozdeleniFilter ->getValue('vcetnePodrizenych'),
                        $params,
                        true);
            }
        }
//        if (isset($this ->filterForm ->kodRozdeleni)) {
//            $result .=
//                ($result ? ' AND ' : '') .
//                RozdeleniUtils::sqlFilterRozdeleniPP($this ->database,
//                    'Osoba',
//                    $this ->filterForm ->kodTypuRozdeleni, $this ->filterForm ->kodRozdeleni,
//                    'VysledekHodnoceni.PocatekObdobi', 'VysledekHodnoceni.KonecObdobi',
//                    $this ->filterForm ->vcetnePodrizenychRozdeleni, $params, true);
//        }

        // pro nektere stavy je filtr na datum povinny, pro jine jde vypnout checkboxem
        // Obdobi
        if ($this ->filterForm ->getFilterValue('datumod')) {
            $result .=
                ($result ? ' AND ' : '') .
                'VysledekHodnoceni.KonecObdobi >= ' . $params ->addDateParameter($this ->filterForm ->getFilterValue('datumod'));
        }

        if ($this ->filterForm ->getFilterValue('datumod', 'datumDo')) {
            $result .=
                ($result ? ' AND ' : '') .
                'VysledekHodnoceni.PocatekObdobi <= ' . $params ->addDateParameter($this ->filterForm ->getFilterValue('datumod', 'datumDo'));
        }

        // "Jen ucel hodnoceni"
//        if (HodnoceniFilterForm::getJenUcelHodnoceni() && HodnoceniFilterForm::getUcelHodnoceni()) {
//            $result .=
//                ($result ? ' AND ' : '') .
//                '(VysledekHodnoceni.KodUcelu = ' . $params ->addParameter((int)HodnoceniFilterForm::getUcelHodnoceni()). ')';
//        }

//        // "Jen tema hodnoceni"
//        if (HodnoceniFilterForm::getJenTemaHodnoceni() && HodnoceniFilterForm::getTemaHodnoceni()) {
//            $result .=
//                ($result ? ' AND ' : '') .
//                'PripraveneTemaHodnoceni.TemaCislo = ' . $params ->addParameter((int)HodnoceniFilterForm::getTemaHodnoceni());
//        }

        if ($this->hromadneSchvaleni) { //  &&  $this ->filterForm ->getFilterValue('jenznamkaoddo')) {
            $znamkaOd = $this ->filterForm ->getFilterValue('jenznamkaoddo', 'znamkaOd');
            if ($znamkaOd !== null) {
                $result .=
                    ($result ? ' AND ' : '') .
                    'HodnoceniOsoby.Znamka >= ' . $params ->addParameter((float)$znamkaOd);
            }

            $znamkaDo = $this ->filterForm ->getFilterValue('jenznamkaoddo', 'znamkaDo');
            if ($znamkaDo !== null) {
                $result .=
                    ($result ? ' AND ' : '') .
                    'HodnoceniOsoby.Znamka <= ' . $params ->addParameter((float)$znamkaDo);
            }
        }

        //jen osoba
        if ($this->filterForm->getFilterValue('osoba')) {
            $result .=
                ($result ? ' AND ' : '') .
                'PripraveneTemaHodnoceni.HodnocenyRC = ' . $params ->addParameter($this->filterForm->getFilterValue('osoba', 'rc'));
        }

        return new SqlStatement($result, $params);
    }


    private function getSQLFIlterVlastniHodnoceni()
    {
        $params = $this->database->createParameters();

        if (User::idZastupu()) {
            // Zastup - vlastni hodnoceni nezobrazuje (... nebo profiltrovat hodnoceni podle povolenych predavacich procesu?)
            return '0 = 1';
        }

        // vybere zaznamy, na ktere ma uzivatel pravo
        $result =
            ' ((PripraveneTemaHodnoceni.IndividualniPoradi = 0 OR' .    // POZOR! tato podminka to muze zpomalit (testovano SE)!
            ' PripraveneTemaHodnoceni.HodnotilRC = ' . $params ->addParameter(User::getRC()) . ')' . // je nastaveny "individualni" hodnotitel
            'AND (' .
            ' PripraveneTemaHodnoceni.HodnotilRC = ' . $params ->addParameter(User::getRC());       // vyplnil jsem "ja"

        $result .=
            ' OR ( PoradiPredavani.Vlastni = 1   AND  PripraveneTemaHodnoceni.HodnocenyRC = ' . $params ->addParameter(User::getRC()) . ')';

        $result .=
            "\n))";

        return new SqlStatement($result, $params);
    }


    private function getSQLFIlterHodnoceniDleProcesu()
    {
//        $kodTypuRozdeleni = RonPortalSettings::getIntValue(RonPortalSettings::PREDAVANI_TYP_ROZDELENI);

        $params = $this->database->createParameters();

        // vybere hodnoceni zamestnancu, na ktereho mam pravo (podle pracovniho mista a hierarchie)
        $result =
            ' (PoradiPredavani.Vlastni = 0  AND ' .
            '  PoradiPredavani.NekontrolovatHierarchii = 0 AND ' .
            '( EXISTS ' .
            '  ( SELECT PracovniPomer.Id ' .
            '  FROM PracovniPomer, RozdeleniOsob R_PracovniMisto, PMProUrovne, ' .
            '       RozdeleniOsob R_JA, HierarchieRozdeleni H, RozdeleniOsob R_Hodnoceny, PredavaciProces ' .
            ( User::idZastupu()
                ? ', ZastupProPredavaciProces, VybraneRoleProZastup'   // pri zastupovani jeste omezi jen predavaci procesy vybrane v roli pro zastup
                : ''
            ) .
            ' WHERE PracovniPomer.RC = ' . $params ->addParameter(User::getRC()) .
            // jen "moje" urovne predavani podle pracovniho mista
            '   AND R_PracovniMisto.RC = PracovniPomer.RC ' .
            '   AND R_PracovniMisto.OsobniCislo = PracovniPomer.OsobniCislo ' .
            '   AND R_PracovniMisto.DatumNastupu = PracovniPomer.DatumNastupu ' .
            '   AND R_PracovniMisto.PlatnostOd <= ' . ($this ->obdobiPravPodleHodnoceni ? 'VysledekHodnoceni.KonecObdobi' : $params ->addDateParameter(time())) .
            '   AND (R_PracovniMisto.PlatnostDo >= ' . ($this ->obdobiPravPodleHodnoceni ? 'VysledekHodnoceni.PocatekObdobi' : $params ->addDateParameter(time())) . '  OR  R_PracovniMisto.PlatnostDo IS NULL) ' .
            '   AND R_PracovniMisto.KodTypuRozdeleni = PMProUrovne.KodTypuRozdeleni ' .
            '   AND R_PracovniMisto.Rozdeleni = PMProUrovne.KodRozdeleni ' .
            '   AND HodnoceniOsoby.UrovenPredavani = PMProUrovne.KodUrovne ' .
            '   AND PredavaciProces.Kod = PoradiPredavani.KodProcesu' .
            // jen pordrizena rozdeleni (vzhledem k PP!)
            '   AND R_Ja.RC = PracovniPomer.RC ' .
            '   AND R_Ja.OsobniCislo = PracovniPomer.OsobniCislo ' .
            '   AND R_Ja.DatumNastupu = PracovniPomer.DatumNastupu ' .
            '   AND H.TypNadrazene = R_Ja.KodTypuRozdeleni ' .
            '   AND H.KodNadrazene = R_Ja.Rozdeleni ' .
            '   AND H.TypPodrizene = R_Hodnoceny.KodTypuRozdeleni ' .
            '   AND H.KodPodrizene = R_Hodnoceny.Rozdeleni ' .
            //26.1.2024 PaRu odstranena OR podminka - zpusobovala problem s rychlosti nacteni dotazu, v predavacim procesu musi byt nastaven typ rozdeleni pro prava, obdobny dotaz v seznamu vyplnenych XML formularu ponechan
            "   AND (R_JA.KodTypuRozdeleni = PredavaciProces.KodTypuRozdeleni)" . //OR (PredavaciProces.KodTypuRozdeleni IS NULL AND R_JA.KodTypuRozdeleni = $kodTypuRozdeleni)
            "   AND R_Hodnoceny.KodTypuRozdeleni = R_JA.KodTypuRozdeleni" .
            '   AND R_Hodnoceny.RC = VysledekHodnoceni.HodnocenyRC' .
            '   AND R_Hodnoceny.OsobniCislo = VysledekHodnoceni.HodnocenyOsobniCislo' .
            '   AND R_Hodnoceny.DatumNastupu = VysledekHodnoceni.HodnocenyDatumNastupu' .
            '   AND R_Ja.PlatnostOd <= ' . ($this ->obdobiPravPodleHodnoceni ? 'VysledekHodnoceni.KonecObdobi' : $params ->addDateParameter(time())) .
            '   AND (R_Ja.PlatnostDo >= ' . ($this ->obdobiPravPodleHodnoceni ? 'VysledekHodnoceni.PocatekObdobi' : $params ->addDateParameter(time())) . '  OR  R_Ja.PlatnostDo IS NULL)' .
            '   AND R_Hodnoceny.PlatnostOd <= ' . ($this ->obdobiPravPodleHodnoceni ? 'VysledekHodnoceni.KonecObdobi' : $params ->addDateParameter(time())) .
            '   AND (R_Hodnoceny.PlatnostDo >= ' . ($this ->obdobiPravPodleHodnoceni ? 'VysledekHodnoceni.PocatekObdobi' : $params ->addDateParameter(time())) . '  OR  R_Hodnoceny.PlatnostDo IS NULL)' .
            ( User::idZastupu()
                ? ' AND ZastupProPredavaciProces.KodProcesu = PoradiPredavani.KodProcesu' .
                  ' AND VybraneRoleProZastup.KodRole = ZastupProPredavaciProces.KodRole' .
                  ' AND VybraneRoleProZastup.IdZastupu = ' . $params ->addParameter((int)User::idZastupu())
                : ''
            ) .
            ')' .
            '))';

        return new SqlStatement($result, $params);
    }

    private function getSQLFIlterHodnoceniDleProcesuBezHiearchie()
    {
        $params = $this->database->createParameters();

        // vybere hodnoceni zamestnancu, na ktereho mam pravo (podle pracovniho mista bez hierarchie)
        $result =
            ' (PoradiPredavani.Vlastni = 0  AND ' .
            '  PoradiPredavani.NekontrolovatHierarchii = 1 AND ' .
            '( EXISTS ' .
            '  ( SELECT PracovniPomer.Id ' .
            '  FROM PracovniPomer, RozdeleniOsob R_PracovniMisto, PMProUrovne ' .
            ( User::idZastupu()
                ? ', ZastupProPredavaciProces, VybraneRoleProZastup'   // pri zastupovani jeste omezi jen predavaci procesy vybrane v roli pro zastup
                : ''
            ) .

            ' WHERE PracovniPomer.RC = ' . $params ->addParameter(User::getRC()) .
            // jen "moje" urovne predavani podle pracovniho mista
            '   AND R_PracovniMisto.RC = PracovniPomer.RC ' .
            '   AND R_PracovniMisto.OsobniCislo = PracovniPomer.OsobniCislo ' .
            '   AND R_PracovniMisto.DatumNastupu = PracovniPomer.DatumNastupu ' .
            '   AND R_PracovniMisto.PlatnostOd <= ' . ($this ->obdobiPravPodleHodnoceni ? 'VysledekHodnoceni.KonecObdobi' : $params ->addDateParameter(time())) .
            '   AND (R_PracovniMisto.PlatnostDo >= ' . ($this ->obdobiPravPodleHodnoceni ? 'VysledekHodnoceni.PocatekObdobi' : $params ->addDateParameter(time())) . '  OR  R_PracovniMisto.PlatnostDo IS NULL) ' .
            '   AND R_PracovniMisto.KodTypuRozdeleni = PMProUrovne.KodTypuRozdeleni ' .
            '   AND R_PracovniMisto.Rozdeleni = PMProUrovne.KodRozdeleni ' .
            '   AND HodnoceniOsoby.UrovenPredavani = PMProUrovne.KodUrovne '.
            ( User::idZastupu()
                ? ' AND ZastupProPredavaciProces.KodProcesu = PoradiPredavani.KodProcesu' .
                ' AND VybraneRoleProZastup.KodRole = ZastupProPredavaciProces.KodRole' .
                ' AND VybraneRoleProZastup.IdZastupu = ' . $params ->addParameter((int)User::idZastupu())
                : ''
            ) .
            ')' .
            '))';

        return new SqlStatement($result, $params);
    }

    public function getSQL($sqlFilter = '')
    {
        //kvuli nastaveni poli pridanych v konfiguraci se select nacte jen jednou
        $selectPart = $this ->getSelectSQLPart();

        $wherePart1 = SQLUtils::concatenateWherePartsOfSql(
            [$this ->sqlWherePart, $sqlFilter, $this ->getSQLFilter(), $this->getSQLFIlterVlastniHodnoceni()]);

        $wherePart2 = SQLUtils::concatenateWherePartsOfSql(
            [$this ->sqlWherePart, $sqlFilter, $this ->getSQLFilter(), $this->getSQLFIlterHodnoceniDleProcesu()]);

        $wherePart3 = SQLUtils::concatenateWherePartsOfSql(
            [$this ->sqlWherePart, $sqlFilter, $this ->getSQLFilter(), $this->getSQLFIlterHodnoceniDleProcesuBezHiearchie()]);

        if (!$wherePart1 instanceof SqlStatement) {
            $wherePart1 = new SqlStatement($wherePart1, $this ->database ->createParameters());
        }
        if (!$wherePart2 instanceof SqlStatement) {
            $wherePart2 = new SqlStatement($wherePart2, $this ->database ->createParameters());
        }
        if (!$wherePart3 instanceof SqlStatement) {
            $wherePart3 = new SqlStatement($wherePart3, $this ->database ->createParameters());
        }

        $params = $this ->database ->createParameters();
        $params ->appendParameters($wherePart1 ->parameters);
        $params ->appendParameters($wherePart2 ->parameters);
        $params ->appendParameters($wherePart3 ->parameters);

        $orderBy = $this ->getOrderSQLPart();

        // kvuli rychlosti je dotaz rozdelen na 2 poddotazy
        return new SqlStatement(
            'SELECT DISTINCT ' . $selectPart . "\n" .
            ' FROM ' . $this ->sqlFromPart . "\n" .
            ($wherePart1->sqlText ? ' WHERE ' . $wherePart1 ->sqlText: '') .
            "\n\n" .
            "UNION\n\n" .
            'SELECT DISTINCT ' . $selectPart . "\n" .
            ' FROM ' . $this ->sqlFromPart . "\n" .
            ($wherePart2->sqlText ? ' WHERE ' . $wherePart2 ->sqlText : '') .
            "\n\n" .
            "UNION\n\n" .
            'SELECT DISTINCT ' . $selectPart . "\n" .
            ' FROM ' . $this ->sqlFromPart . "\n" .
            ($wherePart3->sqlText ? ' WHERE ' . $wherePart3 ->sqlText : '') .
            "\n" .
            ($orderBy ? 'ORDER BY ' . $orderBy : ''),
            $params
        );
    }

    function enableLine($values)
    {
        //pokud je hodnoceni na urovni predavani, ktera ma limitni uroven ruznou od 1 (je za limitni urovni, kdy se nadefinovane otazky stanou otazkami na ktere odpovida)
        if ($values['col-poradi-limitniuroven'] != 1) {
            //a aktualni cas je vetsi nez konec obdobi hodnoceni + PresahProVlastniHodnoceni (pocet dnu), zobrazi se hodnoceni
            return time() > DateUtils::incDay($this ->database ->stringToTimestamp($values['col-konecobdobi']), $values['col-ucel-presahprovlastnihodnoceni']);
        }

        return parent::enableLine($values);
    }


    public function getTableAttributes()
    {
        $result = parent::getTableAttributes();
        $result['data-obdobiPravPodleHodnoceni'] = $this ->obdobiPravPodleHodnoceni ? '1' : '0';

        return $result;
    }


    function render()
    {
        echo "\n<div id='prehled_hodnoceni'>\n"; // id pro javascript v hodnoceni.js
        parent::render();
        echo "\n</div>\n";
    }


}
