<?php
/**
 * stanislav.elbl, 13.11.2015
 */

namespace RonPortal\Hodnoceni;


use Exception;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\SqlStatement;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Controls\FilterFormFilter;
use RonPortal\Controls\FilterFormFilterDatumOdDo;
use RonPortal\Controls\RonPortalFilterForm;
use RonPortal\Prava;
use RonPortal\RonPortalSession;
use RonPortal\RonPortalSettings;


/**
 * Filtr na vyber ucelu hodnoceni - polozky se filtruji podle zvoleneho obdobi
 */
class UcelHodnoceniFilter extends FilterFormFilter
{
    private $parentFilterForm;

    public function __construct(string $name, RonPortalFilterForm $parentFilterForm)
    {
        parent::__construct($name, self::TYPE_OPTIONS);
        $this ->parentFilterForm = $parentFilterForm;
        $this
            ->setOptionsSqlFilter('1 = 1')
            ->setOptionsSqlOrder('UcelHodnoceni.Popis')
            ->setOptionsSqlOrder2('UcelHodnoceni.Kod');
    }

    public function loadOptions(Database $database): FilterFormFilter
    {
        $params = $database ->createParameters();

        $sql =
            'SELECT DISTINCT UcelHodnoceni.Kod, UcelHodnoceni.Popis ' .
            ' FROM UcelHodnoceni, VysledekHodnoceni ' .
            ' WHERE VysledekHodnoceni.KodUcelu = UcelHodnoceni.Kod ' .
            // jen ucely hodnoceni pouzite v zobrazenem obdobi
            '   AND VysledekHodnoceni.KonecObdobi >= ' . $params->addDateParameter($this ->parentFilterForm ->getOmezitObdobiOd()) .
            '   AND VysledekHodnoceni.PocatekObdobi <= ' . $params->addDateParameter($this ->parentFilterForm ->getOmezitObdobiDo()) .
            '   AND %FILTER%' .
            ' ORDER BY %ORDER%';

        $this ->optionsSql = new SqlStatement($sql, $params);

        return parent::loadOptions($database);
    }
}



/**
 * Filtr na vyber tema hodnoceni - polozky se filtruji podle zvoleneho obdobi
 */
class TemaHodnoceniFilter extends FilterFormFilter
{
    private $parentFilterForm;

    public function __construct(string $name, RonPortalFilterForm $parentFilterForm)
    {
        parent::__construct($name, self::TYPE_OPTIONS);
        $this ->parentFilterForm = $parentFilterForm;
        $this
            ->setOptionsSqlFilter('1 = 1')
            ->setOptionsSqlOrder('PopisTematu')
            ->setOptionsSqlOrder2('TemaCislo');
    }

    public function loadOptions(Database $database): FilterFormFilter
    {
        $filtrovatDatum = $this ->parentFilterForm ->getFilterValue('datumod');

        $params = $database->createParameters();

        $sql =
            'SELECT DISTINCT TemaHodnoceni.TemaCislo, TemaHodnoceni.PopisTematu ' .
            ' FROM TemaHodnoceni, PripraveneTemaHodnoceni ';
        if ($filtrovatDatum) {
            $sql .= ', HodnoceniOsoby, VysledekHodnoceni ';
        }

        $sql .=
            ' WHERE TemaHodnoceni.DruhHodnoceni = 8 ' .
            '   AND TemaHodnoceni.DruhHodnoceni = PripraveneTemaHodnoceni.DruhHodnoceni ' .
            '   AND TemaHodnoceni.TemaCislo = PripraveneTemaHodnoceni.TemaCislo ';

        if ($filtrovatDatum) {
            $filtrovatDatumDo = $this ->parentFilterForm ->getFilterValue('datumod', 'datumDo');

            $sql .=
                ' AND HodnoceniOsoby.ID = PripraveneTemaHodnoceni.ID ' .
                ' AND VysledekHodnoceni.Id = HodnoceniOsoby.IdVysledku ' .
                ' AND VysledekHodnoceni.KonecObdobi >= ' . $params->addDateParameter($filtrovatDatum) .
                ($filtrovatDatumDo
                    ? ' AND VysledekHodnoceni.PocatekObdobi <= ' . $params->addDateParameter($filtrovatDatumDo)
                    : ''
                );

        } else {
            // odfiltruje neukoncene
            $sql .=
                ' AND (PripraveneTemaHodnoceni.Stav = 0  OR PripraveneTemaHodnoceni.Stav = 1  OR PripraveneTemaHodnoceni.Stav = 4) ';
        }

        $sql .=
            ' AND %FILTER%' .
            ' ORDER BY %ORDER%';

        $this ->optionsSql = new SqlStatement($sql, $params);

        return parent::loadOptions($database);
    }
}


class ZnamkaOdDoFilter extends FilterFormFilter
{
    /** @var float|null */
    public $znamkaOd = null;

    /** @var float|null */
    public $znamkaDo = null;

    public function __construct(string $name)
    {
        parent::__construct($name, self::TYPE_CHECKBOX);
    }


    public function load(Database $database)
    {
        parent::load($database);

        if (isset($_GET['znamkaod'])) {
            $znamkaOdStr = trim($_GET['znamkaod'] ?? '');
            if ($znamkaOdStr != '') {
                try {
                    $this->znamkaOd = StringUtils::stringToFloat($znamkaOdStr);
                    $_SESSION['filterform_znamkahodnoceni_od'] = $this ->znamkaOd;
                } catch (Exception $e) {
                    $this->znamkaOd = null;
                }
            } else {
                $this->znamkaOd = null;
            }
        } else {
            $this ->znamkaOd = $_SESSION['filterform_znamkahodnoceni_od'] ?? null;
        }

        if (isset($_GET['znamkado'])) {
            $znamkaDoStr = trim($_GET['znamkado'] ?? '');
            if ($znamkaDoStr != '') {
                try {
                    $this->znamkaDo = StringUtils::stringToFloat($znamkaDoStr);
                    $_SESSION['filterform_znamkahodnoceni_do'] = $this->znamkaDo;
                } catch (Exception $e) {
                    $this->znamkaDo = null;
                }
            } else {
                $this->znamkaDo = null;
            }
        } else {
            $this ->znamkaDo = $_SESSION['filterform_znamkahodnoceni_do'] ?? null;
        }
    }


    public function getValue(?string $specification = null)
    {
        switch ($specification) {
            case 'znamkaOd' :
                return $this ->value ? $this ->znamkaOd : null;

            case 'znamkaDo' :
                return $this ->value ? $this ->znamkaDo : null;
        }

        return parent::getValue($specification);
    }


    public function render()
    {
        parent::render();
        echo
            //"<input type='checkbox' class='clickAutoSubmit' name='{$this->name}' value='1'" . ($this ->value ? " checked='checked'" : '') . '/>' .
            "<span class='margin-left-2 margin-right'>" . TextManager::_('od') . "</span>" .
            "<input name='znamkaod' size='5' value='" . (isset($this ->znamkaOd) ? StringUtils::format("number_2", $this ->znamkaOd) : '') . "'/>" .
            "<span class='margin-left-2 margin-right'>" . TextManager::_('do') . "</span>" .
            "<input name='znamkado' size='5' value='" . (isset($this ->znamkaDo) ? StringUtils::format("number_2", $this ->znamkaDo) : '') . "'/>"
            ;
    }
}


class HodnoceniFilterForm extends RonPortalFilterForm
{
    protected $vysledky;
//    public $zobrazitZnamky;

    /** @var UcelHodnoceniFilter   filtr na ucel hodnoceni */
    public $ucelHodnoceniFilter;

    /** @var TemaHodnoceniFilter   filtr na tema hodnoceni */
    public $temaHodnoceniFilter;

    /**
     * HodnoceniFilterForm constructor.
     * @param Database $database
     * @param string $action
     * @param bool $vysledky
     * @throws Exception
     */
    function __construct($database, $action, $vysledky)
    {
        parent::__construct($database, $action);

        $this->setUseGet(true, $action);

        $this->f_classname = 'filtercontainer';
        $this->vysledky = $vysledky; // jestli se jedna o zobrazeni vysledku

        if ($vysledky) {
            $datumPovinny = true;
        } else {
            $stav = self::getStavHodnoceni();
            $datumPovinny = ($stav == 2 || $stav == 3 || $stav == 100);
        }

        $this->addFilter(
            (new FilterFormFilterDatumOdDo($this))
                ->setDatumOdDoLinks([
                        'currentMonth',
                        'currentYear',
                        [
                            'datumOd' => DateUtils::incMonth(DateUtils::firstDayOfMonth(time()), -2),
                            'datumDo' => DateUtils::lastDayOfMonth(time()),
                            'text' => TextManager::_('Posledni_3_mesice')
                        ],
                    ]
                )
                ->setCaption(TextManager::_('Hodnocene_obdobi') . ' ' . TextManager::_('Od'))
                ->setRequired($datumPovinny),
            true
        );

        if (!$this->vysledky || (int)($_GET['filtrhodnoceni'] ?? 0)) {
            //$this->nastavRozdeleni(true, RonPortalSettings::getIntValue(RonPortalSettings::PREDAVANI_TYP_ROZDELENI));
            ($this ->addFilterRozdeleni('rozdeleni', RonPortalSettings::getIntValue(RonPortalSettings::PREDAVANI_TYP_ROZDELENI)))
                ->setSessionName('ads_rozdeleni_' . RonPortalSettings::getIntValue(RonPortalSettings::PREDAVANI_TYP_ROZDELENI))
                ->setPouzitPopisTypuRozdeleni(true);
        }

        $prava = Prava::vratPrava('hodnoceni');
        if ($prava ->jsouNejakePrava()) {
            $this->addFilterPP(true, $prava);
        }
//        if ($prava->jsouNejakePrava()) {
//            $this->nastavOsoby(true, $prava);
//        }
//
//        $this->povolitVsechyOsoby = true;

        // ucel hodnoceni
        $this ->ucelHodnoceniFilter = new UcelHodnoceniFilter('ucelhodnoceni', $this);
        $this ->ucelHodnoceniFilter
            ->setCaption(TextManager::_('Ucel_hodnoceni'))
            ->addItem('', TextManager::_('_vse_'))
            ->setSqlFilter('VysledekHodnoceni.KodUcelu = ')
            ->setShowCode(true);
        $this ->addFilter($this ->ucelHodnoceniFilter);

        // tema hodnoceni
        if (!$this ->vysledky) {
            $this ->temaHodnoceniFilter = new TemaHodnoceniFilter('temahodnoceni', $this);
            $this ->temaHodnoceniFilter
                ->setCaption(TextManager::_('Jen_tema_hodnoceni'))
                ->addItem('', TextManager::_('_vse_'))
                ->setShowCode(true);
            $this ->addFilter($this ->temaHodnoceniFilter);
        }
    }


    public function pridejFiltrZnamky()
    {
        $this ->addFilter(
            (new ZnamkaOdDoFilter('jenznamkaoddo'))
                ->setCaption(TextManager::_('Znamka'))
        );
    }

    public static function getStavHodnoceni()
    {
        return RonPortalSession::getValue('filtrstavhodnoceni');
    }


    protected function afterLoadFilter($filter)
    {
        if ($filter ->name === 'filtrhodnoceni'  &&  !$filter ->getValue()) {
            // seznam vyslekdu hodnoceni - zapnuta volba "mojehodnoceni" => nezobrazuje filtr osoby
            $filterOsoba = $this ->getFilter('osoba');
            if ($filterOsoba) {
                $filterOsoba ->enabled = false;
            }
        }

        parent::afterLoadFilter($filter);
    }




//    public static function getJenTemaHodnoceni()
//    {
//        return RonPortalSession::getValue('filtrjentemahodnoceni');
//    }
//
//    public static function getTemaHodnoceni()
//    {
//        return RonPortalSession::getValue('filtrtemahodnoceni');
//    }

//    public static function getJenUcelHodnoceni()
//    {
//        return RonPortalSession::getValue('filtrjenucelhodnoceni');
//    }
//
//    public static function getUcelHodnoceni()
//    {
//        return RonPortalSession::getValue('filtrucelhodnoceni');
//    }

//    public static function getFiltrVysledku()
//    {
//        return RonPortalSession::getValue('filtrvysledekhodnoceni');
//    }

//    public static function getFiltrovatZnamky()
//    {
//        return RonPortalSession::getValue('filtrjenznamkyoddo');
//    }
//
//    public static function getZnamkaOd()
//    {
//        return RonPortalSession::getValue('filtrznamkaod');
//    }
//
//    public static function getZnamkaDo()
//    {
//        return RonPortalSession::getValue('filtrznamkado');
//    }

//    public function getJenRozdeleniHodnoceni()
//    {
//        return $this ->kodRozdeleni;
//        //return RonPortalSession::getValue('filtrjenrozdelenihodnoceni');
//    }

//    public static function getRozdeleniHodnoceni()
//    {
//        return RonPortalSession::getValue('filtrrozdelenihodnoceni');
//    }

//    public static function getFiltrObdobi()
//    {
//        return RonPortalSession::getValue('filtrobdobihodnoceni');
//    }


//    function loadData()
//    {
//        parent::loadData();

//        $_SESSION['filtrjenrozdelenihodnoceni'] = isset($_GET['jenrozdeleni']) &&  $_GET['jenrozdeleni'];
//        if (isset($_GET['rozdeleni'])) {
//            $_SESSION['filtrrozdelenihodnoceni'] = $_GET['rozdeleni'];
//        }

//        $_SESSION['filtrjentemahodnoceni'] = isset($_GET['jentemahodnoceni']) && $_GET['jentemahodnoceni'];
//        if (isset($_GET['temahodnoceni'])) {
//            $_SESSION['filtrtemahodnoceni'] = $_GET['temahodnoceni'];
//        }

//        $_SESSION['filtrjenucelhodnoceni'] = isset($_GET['jenucelhodnoceni'])  &&  $_GET['jenucelhodnoceni'];
//        if (isset($_GET['ucelhodnoceni'])) {
//            $_SESSION['filtrucelhodnoceni'] = (int)$_GET['ucelhodnoceni'];
//        }

//        $_SESSION['filtrjenznamkyoddo'] = isset($_GET['jenznamkaoddo'])  &&  $_GET['jenznamkaoddo'];
//
//        if (isset($_GET['znamkaod'])) {
//            $pom = StringUtils::stringToFloat($_GET['znamkaod']);
//            $_SESSION['filtrznamkaod'] = $pom;
//        }
//
//        if (isset($_GET['znamkado'])) {
//            $pom = StringUtils::stringToFloat($_GET['znamkado']);
//            $_SESSION['filtrznamkado'] = $pom;
//        }

//        $nastavDatum = false;
//
//        if (isset($_GET['obdobi'])) {
//            $_SESSION['filtrobdobihodnoceni'] = $_GET['obdobi'];
//            NastaveniUzivatele::setUserSettingsItem($this->database, 'hodnocenifiltrobdobi', $_GET['obdobi'], 0);
//
//            $nastavDatum = true;
//        } else {
//            if (!isset($_SESSION['filtrobdobihodnoceni'])) {
//                $nastavDatum = true;
//            }
//
//            $_SESSION['filtrobdobihodnoceni'] = NastaveniUzivatele::getUserSettingsItem($this->database, 'hodnocenifiltrobdobi', 0);
//        }
//
//        if ($nastavDatum) {
//            switch ((int)$_SESSION['filtrobdobihodnoceni']) {
//                case 2:
//                    $this ->datumOd = DateUtils::incMonth(DateUtils::firstDayOfMonth(time()), -2);
//                    $this ->datumDo = DateUtils::lastDayOfMonth(time());
//                    break;
//                case 1:
//                    $this ->datumOd = DateUtils::firstDayOfYear(time());
//                    $this ->datumDo = DateUtils::lastDayOfMonth(DateUtils::incMonth($this ->datumOd, 11));
//                    break;
//                case 0:
//                default:
//                    $this ->datumOd = DateUtils::firstDayOfMonth(time());
//                    $this ->datumDo = DateUtils::lastDayOfMonth($this ->datumOd);
//            }
//            RonPortalSession::setFiltrovatDatum(true, $this ->datumOd, $this ->datumDo);
//        }
//    }


//    /**
//     * @throws Exception
//     */
//    function renderRozdeleniHodnoceni()
//    {
//        echo '<DIV class="filterrow">';
//
//        // checkbox "Jen rozdeleni"
//        echo '<SPAN class="filtercaption">';
//        GuiUtils::renderCheckbox('jenrozdeleni',
//            TextManager::_('Jen_rozdeleni'), isset($this ->kodRozdeleni),
//            ['id' => 'filterform_jenrozdeleni'], 'clickAutoSubmit', 1);
//
//        echo '</SPAN>';
//
//        $params = $this->database->createParameters();
//
//        // combobox pro vyber
//        $sql =
//            'SELECT Kod, Popis FROM Rozdeleni' .
//            ' WHERE KodTypuRozdeleni = ' . $params->addParameter(RonPortalSettings::getIntValue(RonPortalSettings::PREDAVANI_TYP_ROZDELENI));
//
//        if ($this ->filtrovatDatum) {
//            $sql .=
//                ' AND (DatumZruseni >= ' . $params->addDateParameter($this ->datumOd) . ' OR DatumZruseni IS NULL) ' .
//                ' AND DatumVytvoreni <= ' . $params->addDateParameter($this ->datumDo);
//        } else {
//            $sql .=
//                ' AND (DatumZruseni > ' . $params->addDateParameter(time()) . ' OR DatumZruseni IS NULL) ';
//        }
//
//        $sql .=
//            ' ORDER BY Popis';
//
//        // vyber rozdeleni podle popisu
//        $combo = new ComboBox('rozdeleni');
//        $combo->className = 'app_combo adsfilterforminput_full';
//        $combo ->emptyRow = ' ';
//        $combo->showCode = true;
//        $combo ->loadData($this ->database, $sql, self::getRozdeleniHodnoceni(), $params);
//        $combo->htmlParams = [
//            'data-ron-autosubmit' => '1',
//            'data-ron-autosubmit-check' => '#filterform_jenrozdeleni'
//        ];
//        $combo ->render();
//
//        echo "</DIV>\n";
//    }


//    /**
//     * @throws Exception
//     */
//    function renderFilterTemaHodnoceni()
//    {
//        echo '<DIV class="filterrow">';
//
//        echo '<SPAN class="filtercaption">';
//        // checkbox "Jen tema hodnoceni"
//        GuiUtils::renderCheckbox('jentemahodnoceni',
//            (TextManager::_('Jen_tema_hodnoceni')), self::getJenTemaHodnoceni(),
//            ['id' => 'filterform_jentemahodnoceni'], 'clickAutoSubmit', 1);
//
//        echo '</SPAN>';
//
//        $filtrovatdatum = $this->datumOdDoPovinne || $this ->filtrovatDatum;
//
//        $params = $this->database->createParameters();
//
//        $sql =
//            'SELECT DISTINCT TemaHodnoceni.TemaCislo, TemaHodnoceni.PopisTematu ' .
//            ' FROM TemaHodnoceni, PripraveneTemaHodnoceni ';
//        if ($filtrovatdatum) {
//            $sql .= ', HodnoceniOsoby, VysledekHodnoceni ';
//        }
//
//        $sql .=
//            ' WHERE TemaHodnoceni.DruhHodnoceni = 8 ' .
//            '   AND TemaHodnoceni.DruhHodnoceni = PripraveneTemaHodnoceni.DruhHodnoceni ' .
//            '   AND TemaHodnoceni.TemaCislo = PripraveneTemaHodnoceni.TemaCislo ';
//
//        if ($filtrovatdatum) {
//            $sql .=
//                ' AND HodnoceniOsoby.ID = PripraveneTemaHodnoceni.ID ' .
//                ' AND VysledekHodnoceni.Id = HodnoceniOsoby.IdVysledku ' .
//                ' AND VysledekHodnoceni.KonecObdobi >= ' . $params->addDateParameter($this ->datumOd) .
//                ' AND VysledekHodnoceni.PocatekObdobi <= ' . $params->addDateParameter($this ->datumDo);
//        } else {
//            // odfiltruje neukoncene
//            $sql .=
//                ' AND (PripraveneTemaHodnoceni.Stav = 0  OR PripraveneTemaHodnoceni.Stav = 1  OR PripraveneTemaHodnoceni.Stav = 4) ';
//        }
//
//        $sql .=
//            ' ORDER BY PopisTematu';
//
//        // puvodni sql nefiltrovalo nepouzivane typy
//        //    'SELECT DISTINCT TemaCislo, PopisTematu FROM TemaHodnoceni '.
//        //    ' WHERE DruhHodnoceni = 8 '.
//        //    ' ORDER BY PopisTematu',
//
//
//        // combobox pro vyber (tema hodnoceni)
//        $combo = new ComboBox('temahodnoceni');
//        $combo ->className = 'app_combo adsfilterforminput_full';
//        $combo ->emptyRow = ' ';
//        $combo->showCode = true;
//        //porovnani podle typu je nefunkcni, protoze hodnoty z databaze jsou nacteny jako string
//        $combo ->dataTypeComparison = false;
//        $combo ->loadData($this ->database, $sql, (int)self::getTemaHodnoceni(), $params); // $keynum, $captionnum, $emptyrow, $selectedvalue);
//        $combo->htmlParams = [
//            'data-ron-autosubmit' => '1',
//            'data-ron-autosubmit-check' => '#filterform_jentemahodnoceni'
//        ];
//        $combo ->render();
//
//        echo "</DIV>\n";
//    }


//    /**
//     * @throws Exception
//     */
//    function renderFilterUcelHodnoceni()
//    {
//        echo '<DIV class="filterrow">';
//
//        echo '<SPAN class="filtercaption">';
//
//        // checkbox "Jen rozdeleni"
//        GuiUtils::renderCheckbox('jenucelhodnoceni',
//            TextManager::_('Ucel_hodnoceni'), HodnoceniFilterForm::getJenUcelHodnoceni(),
//            ['id' => 'filterform_jenucelhodnoceni'], 'clickAutoSubmit', 1);
//
//        echo '</SPAN>';
//
//        $params = $this->database->createParameters();
//
//        $sql =
//            'SELECT DISTINCT UcelHodnoceni.Kod, UcelHodnoceni.Popis ' .
//            ' FROM UcelHodnoceni, VysledekHodnoceni ' .
//            ' WHERE VysledekHodnoceni.KodUcelu = UcelHodnoceni.Kod ' .
//            // jen ucely hodnoceni pouzite v zobrazenem obdobi
//            '   AND VysledekHodnoceni.KonecObdobi >= ' . $params->addParameter($this ->database ->timestampToParam(RonPortalSession::getDatumOd())) .
//            '   AND VysledekHodnoceni.PocatekObdobi <= ' . $params->addParameter($this ->database ->timestampToParam(RonPortalSession::getDatumDo())) .
//            ' ORDER BY UcelHodnoceni.Popis';
//
//        $combo = new ComboBox('ucelhodnoceni');
//        $combo ->className = 'app_combo adsfilterforminput_full';
//        $combo ->emptyRow = ' ';
//        $combo->showCode = true;
//        //porovnani podle typu je nefunkcni, protoze hodnoty z databaze jsou nacteny jako string
//        $combo ->dataTypeComparison = false;
//        $combo ->loadData($this ->database, $sql, (int)HodnoceniFilterForm::getUcelHodnoceni(), $params);
//        $combo->htmlParams = [
//            'data-ron-autosubmit' => '1',
//            'data-ron-autosubmit-check' => '#filterform_jenucelhodnoceni'
//        ];
//        $combo ->render();
//
//        echo "</DIV>\n";
//    }


//    function renderFilterZnamka()
//    {
//        global $STR_Znamka; // kompat.
//
//        echo '<DIV class="filterrow">';
//
//        echo '<SPAN class="filtercaption">';
//        //echo '<TR><TD>';
//
//        GuiUtils::renderCheckbox('jenznamkaoddo',
//            ($STR_Znamka ?: TextManager::_('Znamka')) . ' ' . TextManager::_('Od'),
//            HodnoceniFilterForm::getFiltrovatZnamky(),
//            [], 'clickAutoSubmit', 1);
//
//        echo '</SPAN>';
//        //echo '</TD><TD>';
//
//        echo '<SPAN class="no-wrap">';
//        echo '<INPUT type="text" name="znamkaod" size="5" value="' . self::getZnamkaOd() . '">';
//
//        echo '&nbsp;&nbsp;&nbsp; ' . TextManager::_('Do') . ' &nbsp;&nbsp;';
//        echo '<INPUT type="text" name="znamkado" size="5" value="' . self::getZnamkaDo() . '">';
//        echo '</SPAN>';
//
//        echo "</DIV>\n";
//        //echo "</TD></TR>\n";
//    }

//    function renderDatumOdDo()
//    {
//        if ($this->nadpisDatumOdDo) $nadpis = $this->nadpisDatumOdDo;
//        else $nadpis = TextManager::_('Datum_od');
//
//        echo '<DIV class="filterrow">';
//
//        echo '<SPAN class="filtercaption">';
//        if ($this->datumOdDoPovinne) {
//            echo $nadpis; // .' &nbsp;&nbsp;';
//        } else {
//            GuiUtils::renderCheckbox('jendatumoddo', $nadpis, RonPortalSession::getFiltrovatDatum(), '', '', null, true);
//        }
//        echo '</SPAN>';
//
//        echo '<SPAN class="no-wrap">';
//        DateInput::renderDateSelect('datumod', date('j.n.Y', RonPortalSession::getDatumOd()), $this ->f_autosubmit); // submitonenter
//
//        echo '&nbsp;&nbsp;&nbsp; ' . TextManager::_('Do') . ' &nbsp;&nbsp;';
//        DateInput::renderDateSelect('datumdo', date('j.n.Y', RonPortalSession::getDatumDo()), $this ->f_autosubmit); // submitonenter
//
//        echo '<div class="hodnoceni_nastav_datum_filtru">';
//        LinksList::renderLinks(
//            [0 => TextManager::_('hodnoceni_aktualni_mesic'), 1 => TextManager::_('hodnoceni_aktualni_rok'), 2 => TextManager::_('hodnoceni_posledni_3_mesice')],
//            (int)HodnoceniFilterForm::getFiltrObdobi(), $this->f_formaction . '&obdobi=', '', false);
//        echo '</div>';
//
//        echo '</SPAN>';
//
//        echo "</DIV>\n";
//    }

//    /**
//     * @throws Exception
//     */
//    function renderFilters()
//    {
//        if ($this->vysledky &&  !$this ->getFilterValue('filtrhodnoceni')) {
//            $this->nastavOsoby(false);
//        }
//
//        parent::renderFilters(); // datum
//
//        if (!$this->vysledky || (int)self::getFiltrVysledku() != 0) {
//            $this->renderRozdeleniHodnoceni();
//        }
//        // ucel
//        $this->renderFilterUcelHodnoceni();
//        // tema (jen u hodnoceni)
//        if (!$this->vysledky) $this->renderFilterTemaHodnoceni();
//        if ($this->zobrazitZnamky) $this->renderFilterZnamka();
//    }

}
