<?php

/**
 * Prehled pohybu na uctu stravnika po dnech
 */

namespace RonPortal\Jidelna;

use Ron\DateUtils;
use Ron\DB\Cursor;
use Ron\DB\DBException;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Controls\RonPortalBrowser;
use RonPortal\Kalendar;


class DenniPohybyBrowser extends RonPortalBrowser
{

    var $soucet;      // pole souctu jednotlivych polozek

    /** @var  array */
    var $data;

    function __construct($database)
    {
        parent::__construct($database, 'dennipohyby');

        $this->title = TextManager::_('Denni_pohyby');
        $this->cssClass = 'browser grid';
        $this->code = 'browserdennipohyby';

        $this->addField()
            ->setHeaderText('Date') ->setAlign('center' )
            ->setCode('col-datum');

        $this->addField()
            ->setHeaderText('Pohyby') ->setAlign('right' )
            ->setCode('col-pohyby');

        $this->addField()
            ->setHeaderText('Dotace') ->setAlign('right' )
            ->setCode('col-dotace');

        $this->addField()
            ->setHeaderText('Hotovost') ->setAlign('right' )
            ->setCode('col-hotovost');

        $this->addField()
            ->setAlign('right')
            ->setCode('col-upraveno'); // priznak "upraveno"

//        $this ->filterFormAppCode = 'JIDELNA';
//        $this ->filterFormRights = 'prohl_jidelna';
//        $this ->filterFormVcetneExternich = true;
    }


    function setFilterForm()
    {
        $this ->filterForm = $this->createFilterForm('JIDELNA', 'prohl_jidelna', false, true);
        $this ->filterForm ->addFilter(new StavUctuFilter($this ->filterForm));
    }


    /**
     * @return \Ron\DB\Cursor
     * @throws \Ron\DB\DBException
     */
    function nactiPohyby()
    {
        $params = $this ->database ->createParameters();
        $sql =
            'SELECT DatumCasPohybu, Castka, TypPohybu.Kod, PopisPohybu, TypPohybu.Popis FROM PohybNaUctu JOIN TypPohybu ON PohybNaUctu.TypPohybu = TypPohybu.Kod' .
            ' WHERE RC = ' . $params->addParameter($this ->filterForm ->getFilterValue('osoba', 'rc')) .
            ' AND OsobniCislo = ' . $params->addParameter($this ->filterForm ->getFilterValue('osoba', 'osobniCislo')) .
            ' AND DatumNastupu = ' . $params ->addDateParameter($this ->filterForm ->getFilterValue('osoba', 'datumNastupu')) .
            ' AND DatumCasPohybu >= ' . $params ->addDateParameter($this ->filterForm ->getFilterValue('obdobi')) .
            ' AND DatumCasPohybu <= ' . $params ->addTimestampParameter(DateUtils::endOfTheDay($this ->filterForm ->getFilterValue('obdobi', 'datumDo'))) .
            ' ORDER BY DatumCasPohybu';

        return $this ->database ->getSqlResult($sql, $params);
    }


//    function nactiDotace()
//    {
//        $sql =
//            'SELECT DatumCasPohybu, Castka FROM PohybNaUctu' .
//            " WHERE RC = '" . $this ->database ->addSlashes(RonPortalSession::osobaRC()) . "'" .
//            " AND OsobniCislo = '" . $this ->database ->addSlashes(RonPortalSession::osobaOsobniCislo()) . "'" .
//            ' AND DatumNastupu = ' . $this ->database ->dateToString(RonPortalSession::osobaNastup()) .
//            ' AND DatumCasPohybu >= ' . $this ->database ->timestampToString(RonPortalSession::obdobi()) .
//            ' AND DatumCasPohybu <= ' . $this ->database ->timestampToString(DateUtils::endOfTheDay(RonPortalSession::konecObdobi())) .
//            ' AND (PohybNaUctu.TypPohybu = 11)' .
//            ' ORDER BY DatumCasPohybu';
//
//        return $this ->database ->getSqlResult($sql);
//    }


    /**
     * @return Cursor
     * @throws DBException
     */
    function nactiNaroky():Cursor
    {
        $params = $this ->database ->createParameters();
        $sql =
            'SELECT Den, narok_PocetDotaci, narok_PocetDotaciUpraveny FROM DenniDotace' .
            ' WHERE RC = ' . $params->addParameter($this ->filterForm ->getFilterValue('osoba', 'rc')) .
            ' AND OsobniCislo = ' . $params->addParameter($this ->filterForm ->getFilterValue('osoba', 'osobniCislo')) .
            ' AND DatumNastupu = ' . $params ->addDateParameter($this ->filterForm ->getFilterValue('osoba', 'datumNastupu')) .
            ' AND Den >= ' . $params ->addDateParameter($this ->filterForm ->getFilterValue('obdobi')) .
            ' AND Den <= ' . $params ->addDateParameter($this ->filterForm ->getFilterValue('obdobi', 'datumDo')) .
            ' ORDER BY Den';

        return $this ->database ->getSqlResult($sql, $params);
    }


    /**
     * @param string $sqlFilter
     * @throws \Exception
     */
    function load($sqlFilter = '')
    {
        $this ->setFilterForm();
        $this ->filterForm ->loadData();

        $pohyby = $this->nactiPohyby();
        $naroky = $this->nactiNaroky();

        $this->soucet = [0, 0, 0];
        $datum = DateUtils::startOfTheDay($this ->filterForm ->getFilterValue('obdobi'));
        $posledniDatum = DateUtils::endOfTheDay($this ->filterForm ->getFilterValue('obdobi', 'datumDo'));

        $this ->data = [];

        $pohyb = $pohyby ->fetch();
        $narok = $naroky ->fetch();

        // Pro kazdy den vytvori radek
        while ($datum <= $posledniDatum) {
            $dalsiDatum = DateUtils::startOfTheDay($datum + 86400 + 3600); // takhle je to kvuli prechodu letni/zimni cas

            // Secte vsechny pohyby k aktualnimu dni
            $soucetPohybu = 0;
            $popisPohybu = '';
            $soucetDotaci = 0;
            $soucetHotovosti = 0;
            while ($pohyb  &&  $this ->database ->stringToTimestamp($pohyb[0]) < $dalsiDatum) {
                if ($pohyb[2] == 1  ||  $pohyb[2] == 2  ||  $pohyb[2] == 3) {
                    // odber zbozi, objednavka, odber bez objednavky
                    $soucetPohybu += $pohyb[1];
                    $popisPohybu .= ($popisPohybu ? "\n" : '') . $pohyb[3];
                } else if ($pohyb[2] == 11) {
                    // dotace
                    $soucetDotaci += $pohyb[1];
                } else if ($pohyb[2] == 21 || $pohyb[2] == 22) {
                    $soucetHotovosti += $pohyb[1];
                    $popisPohybu .= ($popisPohybu ? "\n" : '') . $pohyb[4] . ' ' . $pohyb[1];
                }

                $pohyb = $pohyby ->fetch();
            }

            $upravenyNarok = false;
            while ($narok  &&  $this ->database ->stringToTimestamp($narok[0]) < $dalsiDatum) {
                if ((int)$narok[1] != (int)$narok[2]) $upravenyNarok = true;
                $narok = $naroky ->fetch();
            }

            $this ->data[] = $this->vytvorRadek($datum, $soucetPohybu, $popisPohybu, $soucetDotaci, $upravenyNarok, false, $soucetHotovosti);

            $datum = DateUtils::startOfTheDay($datum + 86400 + 3600); // takhle je to kvuli prechodu letni/zimni cas
        }
    }


    function vytvorRadek($datum, $soucetPohybu, $popisPohybu, $soucetDotaci, $upravenyNarok, $jeToSoucet, $soucetHotovosti)
    {
        $browserRow = [];
        if (!$jeToSoucet) {
            // V nulte polozce radku ulozi cislo dne - 0 = nedele, 1 = pondeli, ...
            $browserRow[] = date('w', $datum);
            // Vytvori hodnotu v prvnim sloupci - datum
            $browserRow[] =
                '<span class="zkratkadne">' . TextManager::_('Day_short_' . date('w', $datum)) . '</span>' .
                date('j.n.Y', $datum);
        }

        $browserRow[] = ($jeToSoucet || $soucetPohybu)
            ? '<span title="' . StringUtils::htmlTextConvert($popisPohybu) . '">' . StringUtils::format('curr', $soucetPohybu) . '</span>'
            : '';

        $browserRow[] = ($jeToSoucet || $soucetDotaci)
            ? StringUtils::format('curr', $soucetDotaci)
            : '';

        $browserRow[] = ($jeToSoucet || $soucetHotovosti)
            ? '<span title="' . StringUtils::htmlTextConvert($popisPohybu) . '">' . StringUtils::format('curr', $soucetHotovosti) . '</span>'
            : '';

        if (!$jeToSoucet) {
            $this->soucet[0] += $soucetPohybu;
            $this->soucet[1] += $soucetDotaci;
            $this->soucet[2] += $soucetHotovosti;
        }

        $browserRow[] = $upravenyNarok ? TextManager::_('Upraveny_narok') : null;

        return $browserRow;
    }



    function renderDenniPohyby()
    {
        foreach ($this ->data as $index => $row) {
            // Podle toho jestli je to vikend nebo ne zvoli barvu
            $cssClass = Kalendar::getStyleForDay($row[0]);
            if ($index == count($this ->data) - 1) $cssClass .= ' border-bottom';
            if ($cssClass) echo "<tr class='$cssClass'>";
            else echo '<tr>';

            // Vypise obsah radku
            // datum
            echo "<td class='browser_rowheader'>{$row[1]}</td>";

            // hodnoty
            echo "<td class='align-right'>{$row[2]}</td>";
            echo "<td class='align-right'>{$row[3]}</td>";
            echo "<td class='align-right'>{$row[4]}</td>";

            // oznaceni upravenych naroku
            echo "<td>{$row[5]}</td>";

            echo "</tr>";
        }
    }


    function renderFooter()
    {
        echo "<tr class='browserfooter'>";
        echo '<td>' . TextManager::_('Celkem') . '</td>';

        foreach ($this->soucet as $index => $value) {
            echo "<td class='align-right'>" . StringUtils::format('curr', $value) . '</td>';
        }

        echo '<td></td>'; // sloupec pro oznaceni upravenych naroku
        echo '</tr>';

        echo "<tr class='browserfooter'>";
        echo '<td>' . TextManager::_('Rozdil') . '</td>';
        echo "<td colspan='2' class='align-right'>" . StringUtils::format('curr', $this->soucet[0] + $this->soucet[1]) . '</td>';
        echo '<td></td>'; // hotovost
        echo '<td></td>'; // sloupec pro oznaceni upravenych naroku
        echo '</tr>';
    }


    function render()
    {
        $this->renderHeader();

        $this->renderTableStart();

        echo "<tbody>\n";
        $this ->renderDenniPohyby();
        $this ->renderFooter();
        echo "</tbody>\n";

        $this ->renderTableEnd();
    }

}
