<?php
/**
 * stanislav.elbl, 24.12.2015
 */

namespace RonPortal\Jidelna;

use Exception;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\SqlStatement;
use Ron\DB\SQLUtils;
use Ron\StringUtils;
use RonPortal\PracovniPomer;
use RonPortal\RonPortalDBRecord;
use RonPortal\RonPortalSettings;
use RonPortal\User;


class Alergen
{
    var $kod;
    var $popis;
    var $poznamka;

    function __construct($kod, $popis = '')
    {
        $this -> kod = $kod;
        $this ->popis = $popis;
    }


    /**
     * @param Database $database
     * @return bool
     * @throws Exception
     */
    function nactiPolozku($database)
    {
        $sql =
            "SELECT Popis, ". $database ->castWideMemo('Poznamka') .
            " FROM Alergen WHERE Kod = " . (int)$this -> kod;

        $cursor = $database ->select($sql);

        if (!$row = $cursor ->fetch()) {
            throw new Exception('Chyba - zadany alergen neexistuje!');
        }

        $this->popis = $row[0];
        $this->poznamka = $row[1];

        return true;
    }
}


class HodnoceniJidla
{
    public $hodnoceni;
    public $poznamka;

    /**
     * HodnoceniJidla constructor.
     * @param int $hodnoceni
     * @param string $poznamka
     */
    public function __construct($hodnoceni = null, $poznamka = null)
    {
        $this->hodnoceni = $hodnoceni;
        $this->poznamka = $poznamka;
    }
}


class PolozkaJidelnicku
{
    var $f_id;

    var $f_den;
    var $f_typdennihojidla;
    var $f_popistypudennihojidla;
    var $f_jidelna;

    var $f_popisnabidky;
    var $f_cislonabidky;
    var $f_cenajidlaA;
    var $f_popisNabidky_Jazyk2;

    /** @var string  Pro zobrazeni jidelnicku - podle nastaveni cenove kategorie se zobrazuje zadna/plna/dotovana/obe */
    var $zobrazenaCena = '';

    var $objednavkyNepovoleny;
    /** @var int    cas, od kdy je mozne objednavat */
    var $prijemObjednavekOd;
    /** @var int    cas, do kdy je mozne objednavat */
    var $prijemObjednavekDo;
    /** @var int    cas, do kdy je mozne rusit objednavky */
    var $zruseniObjednavekDo;

    /** @var int        Pocet minut pred zacatkem vydeje */
    var $konecPrimeBurzy;
    /** @var int        Pocet minut pred koncem vydeje (nebo pred zacatkem, kdyz je zaporne) */
    var $konecBurzy;

    var $f_maxpocetporciA;

    var $f_pocetobjednavekosoby;
    var $f_pocetobjednavekcelkem; // nacita se jen v pripade, ze je zadan f_maxpocetporciA

    var $f_omezitPocetNaOsobu;
    var $f_maxPocetNaOsobu;

    var $f_pocetjidelvburze;
    var $pocetVBurzeMych;

    var $kodFotografie;

    var $hodnoceni;
    var $hodnoceniPoznamka;
    var $hodnoceniExistuje;

    /** @var Alergen[] */
    var $alergeny;

    var $lzeObjednat;
    var $lzeZrusitObjednavku;
    var $lzePredatObjednavkuDoBurzy;
    var $lzeVyzvednoutZBurzy;


    function __construct()
    {
        $this -> f_id = 0;
        $this -> f_pocetobjednavekosoby = -1;
        $this -> hodnoceni = -1;
        $this -> alergeny = [];
    }


    /**
     * @param Database $database
     * @param int $id
     * @param int $den
     * @param string $jidelna
     * @param string $vybraneTypy
     * @return string
     */
    static function sqlProPolozkyJidelnicku($database, $id, $den, $jidelna, $kodCenoveKategorie = null, $vybraneTypy = '')
    {
        $params = $database ->createParameters();
        $sql =
            'SELECT PolozkaJidelnicku.TypDennihoJidla, TypDennihoJidla.Popis, ' . // 0, 1
            '   PolozkaJidelnicku.Id, PolozkaJidelnicku.Den, ' . // 2, 3
            '   PolozkaJidelnicku.popisNabidky, PolozkaJidelnicku.cisloNabidky, PolozkaJidelnicku.cenaJidla_A, ' . // 4-6
            '   PolozkaJidelnicku.prijemObjednavek_Od, PolozkaJidelnicku.prijemObjednavek_Do, ' . // 7,8
            '   PolozkaJidelnicku.objednavkyNepovoleny, PolozkaJidelnicku.maxPocetPorci_A, ' . // 9,10
            '   PolozkaJidelnicku.Jidelna, PolozkaJidelnicku.popisNabidky_Jazyk2, ' . // 11,12
            '   TypDennihoJidla.KonecPrimeBurzy, TypDennihoJidla.KonecBurzy, ' . // 13,14
            '   PolozkaJidelnicku.omezit_MaxPocetObjOsoby, PolozkaJidelnicku.maxPocetObjOsoby, ' . // 15, 16 12.8.2015 RON-PR doplneno pro kontrolu max. poctu obj. osoby
            '   PolozkaJidelnicku.kodSerialFoto, PolozkaJidelnicku.zruseniObjednavek_Do ' . //17,18

            ' FROM PolozkaJidelnicku, TypDennihoJidla ' .
            ' WHERE PolozkaJidelnicku.typDennihoJidla = TypDennihoJidla.kod ';

        if (isset($kodCenoveKategorie)) {
            // jen typy jidel, ktere maji zaskrtnuto "povolit objednavky"
            $sql .=
                ' AND EXISTS (SELECT * FROM PravidlaCenoveKategorie, VlastnostCenoveKategorie' .
                '   WHERE PravidlaCenoveKategorie.KodTypuRozdeleni = VlastnostCenoveKategorie.KodTypuRozdeleni' .
                '   AND PravidlaCenoveKategorie.CenovaKategorie = VlastnostCenoveKategorie.CenovaKategorie' .
                '   AND PravidlaCenoveKategorie.PlatnostOd = VlastnostCenoveKategorie.PlatnostOd' .
                '   AND PravidlaCenoveKategorie.CenovaKategorie = ' . $params ->addParameter($kodCenoveKategorie) . // KodTypuRozdeleni neni potreba - vzdy je stejne
                // platne k zadanemu dni
                '   AND PravidlaCenoveKategorie.TypDennihoJidla = PolozkaJidelnicku.TypDennihoJidla' .
                '   AND PravidlaCenoveKategorie.TypDennihoJidla = PolozkaJidelnicku.TypDennihoJidla' .
                '   AND (VlastnostCenoveKategorie.PlatnostDo >= PolozkaJidelnicku.Den OR VlastnostCenoveKategorie.PlatnostDo IS NULL)' .
                // zaskrtnuto "povolit objednavky"
                '   AND PovolitObjednavky = 1' .
                ' )';
        }

        //7.10.2016 RON-PR doplnen filtr na typy jidel
        if ($vybraneTypy != '') {
            $sql .= ' AND ' . SQLUtils::getINCondition($database, 'PolozkaJidelnicku.typDennihoJidla', explode(';',$vybraneTypy), $params);
        }

        if ($den) {
            $sql .= ' AND PolozkaJidelnicku.den = ' . $params->addDateParameter($den);
        }

        if ($jidelna) {
            $sql .= ' AND PolozkaJidelnicku.KodTypuRozdeleni = 8 AND PolozkaJidelnicku.jidelna =' . $params->addParameter($jidelna);
        }

        if ($id) {
            $sql .= ' AND PolozkaJidelnicku.Id = ' . $params->addParameter((int)$id);
        }

        $sql .=
            ' ORDER BY TypDennihoJidla.Poradi, PolozkaJidelnicku.cisloNabidky';


        return new SqlStatement($sql, $params);
    }


    /**
     * @param Database $database
     * @param array $row
     */
    function nactiPolozkuZRadku($database, $row)
    {
        $this->f_typdennihojidla = $row[0];
        $this->f_popistypudennihojidla = $row[1];

        $this->f_id = $row[2];
        $this->f_den = $database ->stringToTimestamp($row[3]); // datum

        $this->f_popisnabidky = $row[4];
        $this->f_cislonabidky = $row[5];
        $this->f_cenajidlaA = StringUtils::stringToFloat($row[6]); // nutno prevest - oracle databaze vraci cislo s desetinnou carkou!
        $this->f_popisNabidky_Jazyk2 = $row[12];

        $this->objednavkyNepovoleny = $row[9];
        $this->prijemObjednavekOd = $row[7] ? $database ->stringToTimestamp($row[7]) : 0;
        $this->prijemObjednavekDo = $row[8] ? $database ->stringToTimestamp($row[8]) : 0;
        $this->zruseniObjednavekDo = $row[18] ? $database ->stringToTimestamp($row[18]) : 0;

        $this->f_maxpocetporciA = ($row[10]);

        $this->f_jidelna = $row[11];

        $this->f_pocetobjednavekosoby = 0;

        $this->f_pocetobjednavekcelkem = 0;

        //12.8.2015 RON-PR doplneno pro kontrolu max. poctu obj. osoby
        $this->f_omezitPocetNaOsobu = $row[15];
        $this->f_maxPocetNaOsobu = (int)$row[16];

        $this->f_pocetjidelvburze = 0;
        $this->pocetVBurzeMych = 0;

        $this->konecPrimeBurzy = (int)$row[13];  // pocet minut pred zacatkem vydeje
        $this->konecBurzy = (int)$row[14];       // pocet minut pred koncem vydeje

        $this->kodFotografie = (int)$row[17];
    }


    /**
     * @param Database $database
     * @param int $id
     * @throws Exception
     */
    function nactiPolozku($database, $id)
    {
        $sql = self::sqlProPolozkyJidelnicku($database, $id, 0, '');

        $cursor = $database ->getSqlResult($sql);

        if (!$row = $cursor ->fetch()) {
            throw new Exception('Chyba - zadana polozka jidelnicku neexistuje!');
        }

        $this->nactiPolozkuZRadku($database, $row);
    }


    /**
     * Nacte zadanou polozku jidelnicku
     * @param Database $database
     * @param int $id
     * @return PolozkaJidelnicku
     * @throws Exception
     */
    public static function nactiPolozuJidelnicku($database, $id)
    {
        $polozka = new PolozkaJidelnicku();
        $polozka ->nactiPolozku($database, $id);

        return $polozka;
    }


    /**
     * @param Database $database
     * @param int $datumOd
     * @param int $datumDo
     * @return HodnoceniJidla[]
     * @throws Exception
     */
    public static function nactiHodnoceniPolozek($database, $datumOd, $datumDo)
    {
        $result = [];

        $params = $database ->createParameters();

        $sql =
            'SELECT PolozkaJidelnicku.Id, HodnoceniJidla.Hodnoceni, {fn_CASTWIDEMEMO(HodnoceniJidla.Poznamka)} ' .
            ' FROM PolozkaJidelnicku, HodnoceniJidla ' .
            ' WHERE HodnoceniJidla.IDPolozkyJidelnicku = PolozkaJidelnicku.Id ' .
            '   AND PolozkaJidelnicku.Den >= ' . $params ->addDateParameter($datumOd) .
            '   AND PolozkaJidelnicku.Den <= ' . $params ->addDateParameter($datumDo) .
            '   AND HodnoceniJidla.RC = ' . $params ->addParameter(User::getRC());

        $cursor = $database ->getSqlResult($sql, $params);
        while ($row = $cursor ->fetch()) {
            $result[$row[0]] = new HodnoceniJidla($row[1], $row[2]);
        }
        $cursor ->close();

        return $result;
    }


    /**
     * @param Database $database
     * @return bool
     * @throws Exception
     */
    function nactiHodnoceni($database)
    {
        $params = $database ->createParameters();
        $sql =
            'SELECT Hodnoceni, {fn_CASTWIDEMEMO(Poznamka)} ' .
            '  FROM HodnoceniJidla ' .
            ' WHERE IDPolozkyJidelnicku = ' . $params ->addParameter((int)$this->f_id) .
            '   AND RC = ' . $params ->addParameter(User::getRC());

        $cursor = $database ->getSqlResult($sql, $params);
        $row = $cursor ->fetch();
        $cursor ->close();

        if ($row) {
            $this -> hodnoceni = $row[0];
            $this -> hodnoceniPoznamka = $row[1];
            $this -> hodnoceniExistuje = true;

            return true;
        } else {
            $this -> hodnoceni = 0;
            return false;
        }
    }


    function nactiHodnoceniPost()
    {
        $this -> hodnoceni =  $_POST['hodnoceni'];
        if (isset($_POST['hodnocenipoznamka'])) {
            $this->hodnoceniPoznamka = StringUtils::fromUTF8($_POST['hodnocenipoznamka']);
        }
    }


    /**
     * Nacte seznam alergenu pro vsechny polozky jidelnicku v zadanem obdobi.
     * @param Database $database
     * @param int $datumOd
     * @param int $datumDo
     * @return array    Vraci pole ve formatu IdPolozkyJidelnicku => Alergen[]
     * @throws Exception
     */
    public static function nactiAlergenyPolozek($database, $datumOd, $datumDo)
    {
        $result = [];

        $params = $database ->createParameters();
        $sql =
            'SELECT PolozkaJidelnicku.Id, AlergenPolozkyJidelnicku.KodAlergenu, Alergen.Popis ' .
            '  FROM AlergenPolozkyJidelnicku, Alergen, PolozkaJidelnicku '  .
            ' WHERE Alergen.Kod = AlergenPolozkyJidelnicku.KodAlergenu ' .
            '   AND PolozkaJidelnicku.Id = AlergenPolozkyJidelnicku.IDPolozkyJidelnicku ' .
            '   AND PolozkaJidelnicku.Den >= ' . $params ->addDateParameter($datumOd) .
            '   AND PolozkaJidelnicku.Den <= ' . $params ->addDateParameter($datumDo) .
            ' ORDER BY PolozkaJidelnicku.Id, Alergen.Poradi, Alergen.Kod';

        $cursor = $database ->getSqlResult($sql, $params);

        $alergenyPolozky = [];
        $idPolozky = null;

        while ($row = $cursor ->fetch()) {
            if ($idPolozky  &&  $idPolozky != $row[0]) {
                // radek uz patri nove polozce jidelnicku
                $result[$idPolozky] = $alergenyPolozky;
                $alergenyPolozky = [];
            }

            $idPolozky = $row[0];

            $alergenyPolozky[] = new Alergen($row[1], $row[2]);
        }
        $cursor ->close();

        if ($idPolozky  &&  count($alergenyPolozky)) {
            $result[$idPolozky] = $alergenyPolozky;
        }

        return $result;
    }


    /**
     * @param Database $database
     * @return bool
     * @throws Exception
     */
    function nactiAlergeny($database)
    {
        $params = $database ->createParameters();

        $sql =
            'SELECT AlergenPolozkyJidelnicku.KodAlergenu, Alergen.Popis ' .
            '  FROM AlergenPolozkyJidelnicku, Alergen '  .
            ' WHERE Alergen.Kod = AlergenPolozkyJidelnicku.KodAlergenu ' .
            '   AND IDPolozkyJidelnicku = ' . $params ->addParameter((int)$this ->f_id) .
            ' ORDER BY  Alergen.Poradi, Alergen.Kod';

        $cursor = $database ->getSqlResult($sql, $params);

        $this -> alergeny = [];

        while ($row = $cursor ->fetch()) {
            $this -> alergeny[] = new Alergen($row[0], $row[1]);
        }

        $cursor ->close();

        return true;
    }

    /**
     * @param $database
     * @throws Exception
     */
    function nactiPocetObjednavekCelkem($database)
    {
        if ($this ->f_maxpocetporciA) {
            $this->f_pocetobjednavekcelkem = JidelnaUtils::pocetObjednavekPolozkyCelkovy($database, $this->f_id);
        }
    }

    /**
     * @param Database $database
     * @param PracovniPomer $pp
     */
    function nactiInfoZBurzy($database, $pp)
    {
        if ($pp) {
            $this->f_pocetjidelvburze = JidelnaUtils::pocetVBurzeCelkem($database, $this->f_id);
            $this->pocetVBurzeMych = JidelnaUtils::pocetVBurze_Osoba($database, $this->f_id, $pp);
        }
    }


    /**
     * @param Database $database
     * @param int $id
     * @param PracovniPomer $pp
     * @throws Exception
     */
    function nactiPolozkuACelkovyPocetObjednavek($database, $id, $pp)
    {
        $this->nactiPolozku($database, $id);
        if ($pp) {
            $this->nactiPocetObjednavekCelkem($database);
            $this->nactiInfoZBurzy($database, $pp);
        }
    }

    /**
     * Vraci true, pokud lze jidlo objednavat
     * @return bool
     */
    function lzeObjednavat_PodleCasu()
    {
        return
            (time() >= $this->prijemObjednavekOd) &&
            (time() <= $this->prijemObjednavekDo || $this->prijemObjednavekDo == 0);
    }


    /**
     * vraci true, pokud je burza pro tuto polozku stale aktivni
     * (kontroluje nastaveni konce burzy u typu denniho jidla)
     * @param Database $database
     * @param ?array $casyVydeje    Casy vydeje nactene metodou JidelnaUtils::casVydejePolozkyJidelnicku. Pokud neni zadano, tak se nacte
     * @return bool
     * @throws DBException
     */
    function burzaAktivni($database, $casyVydeje = null)
    {
        if (!$casyVydeje) {
            $casyVydeje = JidelnaUtils::casVydejePolozkyJidelnicku($database, $this->f_id);
        }

        if (!$casyVydeje) {
            // neni zaznam s casem vydeje => neomezuje burzu
            return true;
        }

        if ($casyVydeje['vydejDo'] < time()) {
            // vydej je uz ukoncen
            return false;
        }

        if ($this ->konecBurzy < 0) {
            // je zadano zaporne cislo - pocita se od zacatku vydeje
            if ($casyVydeje['vydejOd'] + 60 * $this->konecBurzy < time()) {
                return false;
            }
        } else {
            // nastavene ukonceni burzy (minuty pred ukoncenim vydeje)
            if ($casyVydeje['vydejDo'] - 60 * $this->konecBurzy < time()) {
                return false;
            }
        }

        return true;
    }


    /**
     * Vraci 1, pokud lze polozku jidelnicku objednavat
     * Vraci 2, pokud lze polozku vykseftovat v burze
     * Vraci 0, pokud nelze polozku objednavat
     * @param Database $database
     * @param bool $zruseniObjednavky Priznak, jestli se jedna o zruseni objednavky
     * @return int
     * @throws DBException
     */
    function stavObjednavani($database, $zruseniObjednavky = false)
    {
        if ($this->objednavkyNepovoleny) {
            return 0;
        }

        $casyVydeje = JidelnaUtils::casVydejePolozkyJidelnicku($database, $this->f_id);

        if (time() < $this->prijemObjednavekOd) {
            // jeste se neobjednava
            return 0;
        }

        $den = $this->f_den;
        if ($casyVydeje['jidelnicekPredchozihoDne'] ?? false) {
            // jidlo se vydava az nasledujici den (po pulnoci)
            $den = DateUtils::incDayStart($den);
        }

        if ($den < DateUtils::startOfTheDay(time())) {
            // uz to bylo
            return 0;
        }


        $casDo = ($zruseniObjednavky && $this ->zruseniObjednavekDo)
                ? $this ->zruseniObjednavekDo
                : $this ->prijemObjednavekDo;

        if (time() <= $casDo || $casDo == 0) {
            // muze se objednavat/rusit
            return 1;
        }

        // Nemuze se objednavat - jeste zkontroluje burzu
        if (!RonPortalSettings::getIntValue(RonPortalSettings::BURZA_POVOLENA)) {
            return 0;
        }

        // Kontrola casoveho omezeni burzy (ukonceni)
        if (!$this->burzaAktivni($database, $casyVydeje)) {
            return 0;
        }

        return 2; // burza povolena
    }


    /**
     * @param Database $database
     * @param string $rc
     * @param string $osobniCislo
     * @param int $datumNastupu
     * @return int
     * @throws DBException
     */
    function novePoradiVydani($database, $rc, $osobniCislo, $datumNastupu)
    {
        $params = $database ->createParameters();
        $sql =
            'SELECT MAX(OJ.poradiVydani) AS MaxporadiVydani ' .
            ' FROM ObjednaneJidlo OJ, PolozkaJidelnicku PJ ' .
            ' WHERE OJ.IDPolozkyJidelnicku = PJ.ID ' .
            '   AND PJ.kodTypuRozdeleni = 8 ' .
            '   AND PJ.jidelna = ' . $params->addParameter($this->f_jidelna) .
            '   AND PJ.den = ' . $params ->addDateParameter($this->f_den) .
            '   AND OJ.DatumCasZruseni IS NULL' .
            '   AND PJ.typDennihoJidla = ' . $params->addParameter($this->f_typdennihojidla) .
            '   AND OJ.RC = ' . $params->addParameter($rc) .
            '   AND OJ.osobniCislo = ' . $params->addParameter($osobniCislo) .
            '   AND OJ.datumNastupu = ' . $params ->addDateParameter($datumNastupu);

        $cursor = $database ->getSqlResult($sql, $params);
        $row = $cursor ->fetch();
        $cursor ->close();

        return $row ? $row[0] + 1 : 1;
    }

    /**
     * @param Database $database
     * @param object|string $logDescription
     * @throws DBException
     * @throws Exception
     */
    function zapisHodnoceni($database, $logDescription)
    {
        $rec = new RonPortalDBRecord($database, 'HodnoceniJidla', $logDescription);
        $rec ->addField('IdPolozkyJidelnicku', 'int', true, 0, (int)$this->f_id);
        $rec ->addField('RC', '', true, 15, User::getRC());
        $rec ->addField('Hodnoceni', 'int', false, 0, (int)$this->hodnoceni);
        $rec ->addField('Poznamka', 'memo', false, 0, $this->hodnoceniPoznamka ?: null);

        $params = $database ->createParameters();
        $filter =
            'IDPolozkyJidelnicku = ' . $params ->addParameter((int)$this->f_id) .
            ' AND RC = ' . $params ->addParameter(User::getRC());
        if ($rec ->loadRecord($filter, $params)) {
            $rec ->updateRecord($filter, $params);
        } else {
            $rec ->insertRecord();
        }
    }
}
