<?php
/**
 * Seznam odberu jidel stravnika
 */

namespace RonPortal\Jidelna;

use Ron\Controls\BrowserField;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\SqlStatement;
use Ron\TextManager;
use RonPortal\Controls\RonPortalBrowser;


class FieldPopisNabidkyOdberu extends BrowserField
{
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        // pokud neni zadan text primo u polozky, tak zobrazuje popis z jidelnicku
        return $rowData[0] ?: $rowData[1];
    }
}



class PuvodniOdberBrowser extends RonPortalBrowser
{

    /**
     * @param Database $database
     */
    function __construct($database)
    {
        parent::__construct($database, 'puvodniodber');

        $this->code = 'browserpuvodniodber';
        $this->title = TextManager::_('Puvodni_odbery');

        $this ->sqlSelectPart =
            'PuvodniOdber.Popis, PolozkaJidelnicku.PopisNabidky'; // 0,1

        // definice dotazu a hlavicky
        $this->sqlFromPart =
            'PuvodniOdber LEFT OUTER JOIN Terminal ON (Terminal.kod = PuvodniOdber.Terminal)' .
            ' LEFT OUTER JOIN PolozkaJidelnicku ON (PolozkaJidelnicku.Id = PuvodniOdber.IdPolozkyJidelnicku)';

        $this->addField()
            ->setSqlFieldName('PuvodniOdber.DatumCasOperace') ->setHeaderText('Datum_a_cas')
            ->setCode('col-datumcas') ->setFormat('datetime') ->setAlign('center') ->setOrdering(1);
        $this->defaultSort = 'col-datumcas';
        $this->primaryField = 0;

        $this->addField()
            ->setSqlFieldName('Terminal.Popis')
            ->setCode('col-terminal') ->setHeaderText('Misto_odberu') ->setOrdering(1);

        $this->addField()
            ->setSqlFieldName('PuvodniOdber.Cena') ->setHeaderText('Cena')
            ->setCode('col-cena') ->setFormat('curr') ->setAlign('right') ->setOrdering(1);

        $this->addField()
            ->setSqlFieldName('PuvodniOdber.Mnozstvi') ->setHeaderText('Mnozstvi')
            ->setCode('col-mnozstvi') ->setFormat('number_2') ->setAlign('right') ->setOrdering(1)
            ->setFilterValue( // nezobrazuje nuly
                function($value, $rowData) { return $value != 0; }
            );

        $this ->addField(new FieldPopisNabidkyOdberu())
            ->setSqlFieldName('PuvodniOdber.Popis', 0)
            ->setCode('col-popis') ->setHeaderText('Popis_odberu') ->setOrdering(0);

        $this->addField()
            ->setSqlFieldName('PuvodniOdber.Karta')
            ->setCode('col-karta') ->setVisible(0) ->setHeaderText('Cislo_karty') ->setOrdering(1);

        $this->addField()
            ->setSqlFieldName('PuvodniOdber.Uzivatel')
            ->setCode('col-uzivatel') ->setVisible(0) ->setHeaderText('Uzivatel') ->setOrdering(1);

        $this->addField()
            ->setSqlFieldName('PuvodniOdber.KodZbozi')
            ->setCode('col-kodzbozi') ->setVisible(0)
            ->setHeaderText('Kod_zbozi') ->setOrdering(1);

        $this->addField()
            ->setSqlFieldName('PuvodniOdber.OdberBezObjednavky')
            ->setCode('col-bezobjednavky') ->setVisible(0) ->setFormat('anone') ->setHeaderText('Kod_zbozi') ->setOrdering(1);

        $this->addField()
            ->setSqlFieldName('PuvodniOdber.DatumCasVlozeniZaznamu')
            ->setCode('col-casvlozeni') ->setVisible(0) ->setHeaderText('Cas_vlozeni')
            ->setFormat('datetime') ->setAlign('center') ->setOrdering(1);

        $this->addField()
            ->setSqlFieldName('PuvodniOdber.Poznamka')
            ->setCode('col-poznamka') ->setVisible(0) ->setHeaderText('Poznamka') ->setOrdering(1);
    }


    function setFilterForm()
    {
        $this ->filterForm = $this->createFilterForm('JIDELNA', 'prohl_jidelna', false, true);
        $this ->filterForm ->addFilter(new StavUctuFilter($this ->filterForm));
//        parent::setFilterForm();
//        $this ->filterForm ->nastavAkciGet('puvodniodber');
//        $this ->filterForm ->addFilter(new StavUctuFilter($this ->filterForm));
    }


    function getSQLFilter()
    {
        $params = $this ->database ->createParameters();
        $result = $this ->filterForm ->buildSqlFilter($params);

        return new SqlStatement(
            $result . ($result ? ' AND ' : '') .
            'PuvodniOdber.DatumCasOperace >= ' . $params ->addDateParameter($this ->filterForm ->getFilterValue('obdobi')) .
            ' AND PuvodniOdber.DatumCasOperace <= ' . $params ->addTimestampParameter(DateUtils::endOfTheDay($this ->filterForm ->getFilterValue('obdobi', 'datumDo'))) .
            ' AND PuvodniOdber.RC = ' . $params->addParameter($this ->filterForm ->getFilterValue('osoba', 'rc')) .
            ' AND PuvodniOdber.OsobniCislo = ' . $params->addParameter($this ->filterForm ->getFilterValue('osoba', 'osobniCislo')) .
            ' AND PuvodniOdber.DatumNastupu = ' . $params ->addDateParameter($this ->filterForm ->getFilterValue('osoba', 'datumNastupu')),
            $params);
    }
}
