<?php
/**
 * Roman.Szablatura RON Software 13.09.2019
 */

namespace RonPortal\Jidelna;


use Exception;
use Ron\Controls\BrowserField;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\SqlStatement;
use Ron\Http\Csrf;
use Ron\ProgressDialog;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Controls\FilterFormFilter;
use RonPortal\Controls\FilterFormFilterDatumOdDo;
use RonPortal\Controls\FilterFormFilterItem;
use RonPortal\Controls\JmenoPrijmeniBrowserField;
use RonPortal\Controls\RonPortalBrowser;
use RonPortal\Controls\RonPortalFilterForm;
use RonPortal\Prava;
use RonPortal\PravaNaOsoby;
use RonPortal\RonPortalSettings;
use RonPortal\RozdeleniUtils;
use Tracy\Dumper;


class JmenoPrijmeniObjednavkyField extends JmenoPrijmeniBrowserField
{
    function getLinkUrl($recordId, $rowData)
    {
        if (Prava::lzeZobrazitPP('objednani', $rowData['col-idpp'])) {
            return
                '?menuid=jidelnicek&datum=' . DateUtils::dateToUrlString($this ->database ->stringToTimestamp($rowData['col-datum'])) .
                '&osoba=' . $rowData['col-idpp'];
        }

        return parent::getLinkUrl($recordId, $rowData);
    }
}

class VybratPolozkuJidelnickuField extends BrowserField
{
    /** @var PravaNaOsoby */
    private $pravo;

    /** @var callable   Funkce, ktera urcuje, jestli se muze hromadna operace pro dany radek pouzit */
    private $filtrFunc;

    public function __construct($pravo, $csrfPrefix = '', $filtrFunc = null)
    {
        parent::__construct();

        $this ->pravo = $pravo;
        $this ->csrfPrefix = $csrfPrefix;
        $this ->filtrFunc = $filtrFunc;

        $this ->setHeader(
            '<input type="checkbox" id="vybrat_vsechny_radky" class="setValue" data-selector=".vybrat-radek" data-value-checked="1" data-value-unchecked="0"/>'
        );
    }

    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if (!$rowData['col-datumcasvydani']  &&  !$rowData['col-datumcaszruseni']  &&  $this ->pravo ->lzeZobrazitPP($rowData['col-idpp'])) {
            if (!$this ->filtrFunc  ||  ($this ->filtrFunc)($rowData))
            return
                "<input type='checkbox' class='vybrat-radek' data-ron-cislo-objednavky='{$rowData[0]}'" .
                ($this ->csrfPrefix ? " data-csrfkey='" . Csrf::getCsrfKey($this ->csrfPrefix . $rowData[0]) . "'" : '') .
                "/>";
        }

        return '';
    }
}


/**
 * Class OdstranitPolozkuJidelnicku
 */
class OdstranitPolozkuJidelnicku extends BrowserField
{
    /** @var PravaNaOsoby $pravoZruseni */
    public $pravoZruseni;


    /**
     * DetailPolozkyJidelnicku constructor.
     * @param Database $database
     * @param PravaNaOsoby $pravoZruseni
     */
    public function __construct($database, $pravoZruseni)
    {
        parent::__construct($database);
        $this->pravoZruseni = $pravoZruseni;
        $this->code = 'col-zrusit';
    }


    function getHtmlTitle($rowNumber, $recordId, $rowData)
    {
        if ($rowData['col-datumcasvydani']) {
            return TextManager::_('Jidelna_objednavka_vydana');
        }

        return parent::getHtmlTitle($rowNumber, $recordId, $rowData);
    }


    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     * @throws Exception
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        //Pokud ma nastaveno datum vydani pak nezobrazit odkaz
        if ($rowData['col-datumcasvydani']  ||  $rowData['col-datumcaszruseni']) {
            return '';
        }

        if ($this->pravoZruseni->lzeZobrazitPP($rowData['col-idpp'])) {
            return
                "<i class='fa inline-glyph glyph-button glyph-delete cursor-pointer zrusit-objednavku'" .
                " title='" . TextManager::_('jidelna_objednavku_odstranit') . "'" .
                " data-csrfkey='" . Csrf::getCsrfKey('zruseni_objednavky_' . $rowData[0]) . "'" .
                '>' .
                "</i>";
        }

        return '';
    }
}


/**
 * Class PrevestPolozkuJidelnicku
 */
class PrevestPolozkuJidelnicku extends BrowserField
{
    /** @var PravaNaOsoby $pravo */
    public $pravoPrevedeni;


    /**
     * PrevestPolozkuJidelnicku constructor.
     * @param Database $database
     * @param PravaNaOsoby $pravoPrevedeni
     */
    public function __construct($database, $pravoPrevedeni)
    {
        parent::__construct($database);
        $this->pravoPrevedeni = $pravoPrevedeni;
    }


    function getHtmlTitle($rowNumber, $recordId, $rowData)
    {
        if ($rowData['col-datumcasvydani']) {
            return TextManager::_('Jidelna_objednavka_vydana');
        }

        return parent::getHtmlTitle($rowNumber, $recordId, $rowData);
    }

    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     * @throws Exception
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        //Pokud ma nastaveno datum vydani pak nezobrazit odkaz
        if ($rowData['col-datumcasvydani']  ||  $rowData['col-datumcaszruseni']) {
            return '';
        }

        if ($this->pravoPrevedeni->lzeZobrazitRC($rowData[2])) {
            //Momentalne nelze pripojit poznamku prevedeni (neni vyreseno datove)
            return
                "<a href='#' class='objednavku-prevest cursor-pointer'" .
                //" title='" . TextManager::_('Jidelna_objednavku_prevest') . "'" .
                " data-ron-akce='prevestobjednavku'" .
                " data-ron-idpp='{$rowData['col-idpp']}'" .
                " data-ron-id-polozky-jidelnicku='$rowData[1]'" .
                " data-ron-jidelna='" . StringUtils::htmlTextConvert($rowData['col-jidelna']) . "'" .
                " data-ron-jidelna-popis='" . StringUtils::htmlTextConvert($rowData['col-jidelna-popis']) . "'" .
                " data-ron-jmeno='" . StringUtils::htmlTextConvert($rowData['@col-osoba']) . "'" .
                " data-ron-popis-objednavky='" . StringUtils::htmlTextConvert($rowData['col-popis']) . "'" .
                " data-ron-datum-objednavky='" . StringUtils::format('date', $this ->database ->stringToTimestamp($rowData['col-datum'])) . "'" .
                " data-csrfkey='" . Csrf::getCsrfKey('prevod_objednavky_' . $rowData[0]) . "'" .
                " data-ron-cislo-objednavky='$rowData[0]'>" .
                //"<i class='inline-glyph fa glyph-exchange'></i>" .
                TextManager::_('Jidelna_objednavku_prevest') .
                "</a>";
        }

        return '';
    }
}



class VlozitObjednavkuDoBurzyField extends BrowserField
{
    public static function lzeVlozitDoBurzy($database, $rowData)
    {
        if ($rowData['col-datumcasvlozeninaburzu']) {
            // uz je v burze
            return false;
        }

        $zruseniDo = $rowData['col-zruseniobjednavek_do']
            ? $database ->stringToTimestamp($rowData['col-zruseniobjednavek_do'])
            : $database ->stringToTimestamp($rowData['col-prijemobjednavek_do']);

        if (!$zruseniDo  ||  $zruseniDo > time()) {
            // jidlo jde jeste standardne rusit => nema smysl davat do burzy
            return false;
        }

        if ($database ->stringToTimestamp($rowData['col-datum']) < DateUtils::today()) {
            // jidlo ze stareho jidelnicku => nema smysl davat do burzy
            return false;
        }

        return true;
    }


    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if (!self::lzeVlozitDoBurzy($this ->database, $rowData)) {
            return '';
        }

        if (!Prava::lzeZobrazitPP('jidelna_do_burzy', $rowData['col-idpp'])) {
            // uzivatel nema pravo
            return '';
        }

        return
            "<a href='#' class='objednavka-do-burzy cursor-pointer'" .
            " data-ron-idpp='{$rowData['col-idpp']}'" .
            " data-ron-id-polozky-jidelnicku='$rowData[1]'" .
            " data-csrfkey='" . Csrf::getCsrfKey('objednavka_do_burzy_' . $rowData[0]) . "'" .
            " data-ron-cislo-objednavky='$rowData[0]'>" .
            TextManager::_('Jidelna_objednavku_do_burzy') .
            "</a>";
    }
}



/**
 * Class SeznamObjednavekJidlaBrowser
 */
class SeznamObjednavekBrowser extends RonPortalBrowser
{

    /** @var PravaNaOsoby $pravoPrevedeni */
    public $pravoPrevedeni;

    /** @var PravaNaOsoby $pravoZruseni */
    public $pravoZruseni;

    private $browserAction = '';
    /** @var PravaNaOsoby */
    private $browserAction_Pravo = null;

    /**
     * SeznamObjednavekJidlaBrowser constructor.
     * @param Database $database
     * @param $pravoPrevedeni
     * @param $pravoZruseni
     * @throws Exception
     */
    public function __construct($database)
    {
        parent::__construct($database, 'jidelnaobjednavky');

        $this
            ->setTitle('Jidelna_seznam_objednavek')
            ->setCode('browserjidelnaobjednavky')
            ->setTableId('browserjidelnaobjednavky')
            ->setBrowserUrl('?menuid=jidelnaobjednavky&');

        $this->pravoPrevedeni = Prava::vratPrava('jidelna_prevedeni_objednavky');
        $this->pravoZruseni = Prava::vratPrava('jidelna_zruseni_objednavky');

        $csrfPrefix = '';
        $vyberFiltrFunc = null;

        switch ($_GET['browseraction'] ?? '') {
            case 'zruseni' :
                $this ->browserAction_Pravo = $this ->pravoZruseni;
                $csrfPrefix = 'zruseni_objednavky_';
                break;

            case 'prevedeni' :
                $this ->browserAction_Pravo = $this ->pravoPrevedeni;
                $csrfPrefix = 'prevod_objednavky_';
                break;

            case 'doburzy' :
                $this ->browserAction_Pravo = Prava::vratPrava('jidelna_do_burzy');
                $csrfPrefix = 'objednavka_do_burzy_';
                $vyberFiltrFunc = function($rowData) {
                    return VlozitObjednavkuDoBurzyField::lzeVlozitDoBurzy($this ->database, $rowData);
                };
                break;
        }

        if ($this ->browserAction_Pravo  &&  $this ->browserAction_Pravo ->jsouNejakePrava()) {
            $this ->browserAction = $_GET['browseraction'];
        }

        $this->sqlSelectPart =
            "OJ.ID, OJ.IDPolozkyJidelnicku, OJ.RC";               //0..2

        $this ->primaryField = 0;

        $this->sqlFromPart =
            'ObjednaneJidlo AS OJ' .
            ' JOIN PracovniPomer ON (PracovniPomer.RC = OJ.RC) AND (PracovniPomer.OsobniCislo = OJ.OsobniCislo) AND (PracovniPomer.DatumNastupu = OJ.DatumNastupu)' .
            ' JOIN Osoba ON (Osoba.RC = PracovniPomer.RC)' .
            ' JOIN PolozkaJidelnicku PJ ON (PJ.ID = OJ.IDPolozkyJidelnicku)' .
            ' JOIN Rozdeleni ON (Rozdeleni.KodTypuRozdeleni = 8 AND Rozdeleni.Kod = PJ.jidelna)' .
            ' JOIN TypDennihoJidla ON (TypDennihoJidla.Kod = PJ.typDennihoJidla)';

        $this ->tableHtmlParams['data-tabulky'] = 'ObjednaneJidlo OJ;PracovniPomer;Osoba;PolozkaJidelnicku PJ;Rozdeleni;TypDennihoJidla';


        if ($this ->browserAction_Pravo  &&  $this ->browserAction_Pravo ->jsouNejakePrava()) {
            $this->addField(new VybratPolozkuJidelnickuField($this->browserAction_Pravo, $csrfPrefix, $vyberFiltrFunc));
        }

        $this->addField()
            ->setSqlFieldName('PJ.Den') ->setHeaderText('Den') ->setOrdering(true)
            ->setFormat('finedate') ->setAlign('center')
            ->setCode('col-datum')
            ->setSortSQL('PJ.Den, Osoba.Prijmeni, Osoba.Jmeno');

        $this->defaultSort = 'col-datum';

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Osoba.Jmeno')
            ->setCode('col-jmeno');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Osoba.Prijmeni')
            ->setCode('col-prijmeni');

        $this ->addField(new JmenoPrijmeniObjednavkyField($this ->database, 'col-jmeno', 'col-prijmeni'))
            ->setHeaderText('Osoba') ->setClassName('jmeno-stravnika')
            ->setCode('col-osoba')
            ->setOrdering(true) ->setSortSQL('Osoba.Prijmeni, Osoba.Jmeno, PJ.Den');

        $this ->addField()
            ->setSqlFieldName('OJ.OsobniCislo') ->setHeaderText('Osobni_cislo') ->setOrdering(true)
            ->setCode('col-osobnicislo');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('OJ.DatumCasObjednani') ->setFormat('finedatetime')
            ->setCode('col-datumcasobjednani');

        $this->addField()
            ->setSqlFieldName('OJ.ID', 0) ->setOrdering(true) ->setHeaderText('Jidelna_ID_polozky')
            ->setCode('col-id')
            ->setTitleFieldIndex('@col-datumcasobjednani') ->setTitlePrefix(TextManager::_('Jidelna_datum_cas_objednani') . ': ')
            ->setVisible(false);

        $this->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('PJ.CisloNabidky')
            ->setCode('col-cislo-nabidky');

        $this->addField()
            ->setSqlFieldName('TypDennihoJidla.Popis') ->setOrdering(true)
            ->setHeaderText('Jidelna_typ_jidla')
            ->setCode('col-typjidla')
            ->setSqlFieldIndex2('col-cislo-nabidky', ' ')
            ->setSortSQL('TypDennihoJidla.Popis, PJ.CisloNabidky', 'TypDennihoJidla.Popis DESC, PJ.CisloNabidky DESC')
            ;

        $this->addField()
            ->setSqlFieldName('Rozdeleni.Popis') ->setOrdering(true)
            ->setHeaderText('Jidelna')
            ->setCode('col-jidelna-popis');

        $this->addField()
            ->setSqlFieldName('PJ.PopisNabidky') ->setHeaderText('Detail')
            ->setCode('col-popis')
            ->setLink('?menuid=jidelnaobjednavky&action=detailobjednavkyjidla&id=')
            ->setLinkAttributes(['class' => 'detail-objednavky']);

        if ($this->pravoZruseni->jsouNejakePrava()) {
            $this->addField(new OdstranitPolozkuJidelnicku($database, $this->pravoZruseni));
        }

        if (!$this ->browserAction  &&  $this->pravoPrevedeni->jsouNejakePrava()) {
            $this->addField(new PrevestPolozkuJidelnicku($database, $this->pravoPrevedeni))
                ->setCode('col-prevest');
        }

        if (RonPortalSettings::getIntValue(RonPortalSettings::BURZA_POVOLENA)  &&  !$this ->browserAction  &&  Prava::jsouNejakePrava('jidelna_do_burzy')) {
            $this ->addField(new VlozitObjednavkuDoBurzyField($database))
                ->setCode('col-do-burzy');
        }

        $this ->addField()
            ->setSqlFieldName('OJ.DatumCasVydani') ->setFormat('finedatetime')
            ->setCode('col-datumcasvydani')
            ->setHeaderText('Jidelna_vydano') ->setAlign('right');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('OJ.Uzivatel')
            ->setCode('col-uzivatel');

        $this ->addField()
            ->setSqlFieldName('OJ.DatumCasObjednani') ->setFormat('finedatetime')
            ->setCode('col-datumcasobjednani')
            ->setHeaderText('Jidelna_cas_objednani') ->setAlign('right')
            ->setTitleFieldIndex('col-uzivatel');

        if ( RonPortalSettings::getIntValue(RonPortalSettings::BURZA_POVOLENA) ) {
            $this ->addField()
                ->setSqlFieldName('OJ.DatumCasVlozeniNaBurzu') ->setFormat('finedatetime')
                ->setCode('col-datumcasvlozeninaburzu')
                ->setHeaderText('Jidelna_cas_vlozeni_na_burzu') ->setAlign('right')
                ->setClassName('cas-vlozeni-na-burzu');
        }

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('OJ.DatumCasZruseni') ->setFormat('finedatetime')
            ->setCode('col-datumcaszruseni')
            ->setHeaderText('Jidelna_zruseno') ->setAlign('right');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('PracovniPomer.Id')
            ->setCode('col-idpp');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('PJ.Jidelna')
            ->setCode('col-jidelna');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('PJ.PrijemObjednavek_Do')
            ->setCode('col-prijemobjednavek_do');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('PJ.ZruseniObjednavek_Do')
            ->setCode('col-zruseniobjednavek_do');
    }


    function setFilterForm()
    {
        //parent::setFilterForm(); - nevolat
        $this ->filterForm = new RonPortalFilterForm($this ->database, $this ->action, true);

        $this ->filterForm ->addFilter(
            (new FilterFormFilterDatumOdDo($this ->filterForm))
                ->setDatumOdDoLinks(['today', 'currentMonth', 'prevMonth'])
                ->setRequired(true),
            true
        );
//        $this ->filterForm ->nastavDatumOdDo(true, true);
//        $this ->filterForm ->datumOdDoLinks = ['today', 'currentMonth', 'prevMonth'];

        $prava = Prava::vratPrava('prohl_jidelna');

        if ($prava ->jsouNejakePrava()  &&  !$prava ->pouzeSebe()) { // napr. pro "pouze vlastni" se vyber nezobrazuje
            $this->filterForm->addFilterRozdeleni1(null, $prava);
            $this->filterForm->addFilterRozdeleni2(null, $prava);
        }

        $this ->filterForm ->addFilterPP(true, $prava)
            ->setVcetneExternich(true);
//        $this ->filterForm ->nastavOsoby(true, $prava, 0, 0, true); // datum od-do podle obdobi
//        $this ->filterForm ->povolitVsechyOsoby = true;

        // typ denniho jidla
        $filterTyp = new FilterFormFilter('typjidla', FilterFormFilter::TYPE_OPTIONS);
        $filterTyp
            ->setCaption(TextManager::_('Jidelna_pouze_typ_jidla'))
            ->addItem('', TextManager::_('_vse_'))
            ->setOptionsSql(
                'SELECT Kod, Popis FROM TypDennihoJidla WHERE %FILTER% ORDER BY %ORDER%',
                'TypDennihoJidla.Aktivni = 1', 'Popis')
            ->setSqlFilter('PJ.typDennihoJidla = ')
            ;

        $this ->filterForm ->addFilter($filterTyp);

        // jidelna
        $filterJidelna = new FilterFormFilter('jidelna');

        $params = $this ->database ->createParameters();
        $filterJidelna
            ->setCaption(TextManager::_('Jidelna'))
            ->addItem('', TextManager::_('Vse'))
            ->setOptionsSql(new SqlStatement(
                'SELECT Kod, Popis FROM Rozdeleni' .
                ' WHERE KodTypuRozdeleni = 8' .
                ' AND %FILTER%' .
                ' AND DatumVytvoreni <= ' . $params ->addDateParameter(time()) . // TODO - podle vybraneho obdobi
                ' AND (DatumZruseni >= ' . $params ->addDateParameter(time()) . ' OR DatumZruseni IS NULL)' .
                ' ORDER BY %ORDER%',
                $params),
                '1 = 1', 'Popis')
            ->setSqlFilter('PJ.Jidelna = ');

        $this ->filterForm ->addFilter($filterJidelna);

        // vydano
        $filtrVydano = new FilterFormFilter('vydano');
        $filtrVydano
            ->setItems([
                (new FilterFormFilterItem('0', TextManager::_('Vse'))) ->setSqlFilter('DatumCasZruseni IS NULL'),
                (new FilterFormFilterItem('1', TextManager::_('Jen_nevydane_objednavky'))) ->setSqlFilter('DatumCasZruseni IS NULL  AND  DatumCasVydani IS NULL'),
                (new FilterFormFilterItem('2', TextManager::_('Jen_vydane_objednavky'))) ->setSqlFilter('DatumCasZruseni IS NULL  AND  DatumCasVydani IS NOT NULL'),
                (new FilterFormFilterItem('3', TextManager::_('Jen_zrusene_objednavky'))) ->setSqlFilter('DatumCasZruseni IS NOT NULL')
            ])
            ->setDefaultValue('0');
        $this ->filterForm ->addFilter($filtrVydano);

        // akce pro hromadne operace
        $this ->filterForm ->addFilter(
            (new FilterFormFilter('browseraction', FilterFormFilter::TYPE_HIDDEN))
            ->setHtmlAttributes(['id' => 'objednavky_browseraction'])
            ->setSessionName('') // neukladat do session
        );
    }


    /**
     * @return SqlStatement|string
     */
    function getSQLFilter()
    {
        $params = $this->database->createParameters();
        $result = $this ->filterForm ->buildSqlFilter($params);

        $result .=
            ($result ? ' AND ' : '') .
            ' PJ.Den >= ' . $params->addDateParameter($this ->filterForm ->getFilterValue('datumod')) .
            ($this ->filterForm ->getFilterValue('datumod', 'datumDo')
                ? ' AND PJ.Den <= ' . $params->addDateParameter($this ->filterForm ->getFilterValue('datumod', 'datumDo'))
                : ''
            ) .
            "\n";

        if ($this ->filterForm ->getFilterValue('osoba')) {
            $result .=
                ' AND PracovniPomer.Id = ' . $params->addParameter($this ->filterForm ->getFilterValue('osoba')) .
                "\n";
        } else {
            foreach ($this ->filterForm ->getRozdeleniFilters() as $rozdeleniFilter) {
                if ($rozdeleniFilter ->getValue() !== null) {
                    $result .=
                        ' AND ' .
                        RozdeleniUtils::sqlFilterRozdeleniPP($this->database, 'PracovniPomer',
                            $rozdeleniFilter->getTypRozdeleni(), $rozdeleniFilter->getValue(),
                            time(), time(),
                            $rozdeleniFilter->getValue('vcetnePodrizenych'),
                            $params) .
                        "\n";
                }
            }
//            if ($this ->filterForm ->kodRozdeleni != '') {
//                $result .=
//                    ' AND ' .
//                    RozdeleniUtils::sqlFilterRozdeleniPP($this ->database, 'PracovniPomer',
//                        $this ->filterForm ->kodTypuRozdeleni, $this ->filterForm ->kodRozdeleni,
//                        time(), time(), $this ->filterForm ->vcetnePodrizenychRozdeleni, $params) .
//                    "\n";
//            }
//
//            if ($this ->filterForm ->kodRozdeleni2 != '') {
//                $result .=
//                    ' AND ' .
//                    RozdeleniUtils::sqlFilterRozdeleniPP($this ->database, 'PracovniPomer',
//                        $this ->filterForm ->kodTypuRozdeleni2, $this ->filterForm ->kodRozdeleni2,
//                        time(), time(), $this ->filterForm ->vcetnePodrizenychRozdeleni2, $params) .
//                    "\n";
//            }
        }

        //$filtrPrava = Prava::sqlFiltrPodlePrav([$this ->pravoPrevedeni, $this ->pravoZruseni, Prava::vratPrava('prohl_jidelna')], 'PracovniPomer');
        //$filtrPrava = Prava::sqlFiltrPodlePrav(Prava::vratPrava('prohl_jidelna'), 'PracovniPomer');
        $filtrPrava = Prava::vratPrava('prohl_jidelna') ->sqlFiltr('PracovniPomer',
            $this ->filterForm ->getOmezitObdobiOd(), $this ->filterForm ->getOmezitObdobiDo(),
            $params
        );

        if ($filtrPrava) {
            $result .= ' AND (' . $filtrPrava . ')' . "\n";
        }

        return new SqlStatement($result, $params);
    }


    /**
     * Za filtry zobrazi tlacitka pro hromadne operace
     * @return void
     */
    function renderFilter()
    {
        parent::renderFilter();

        echo '<div id="browserjidelnaobjednavky_buttons" class="margin-top-2">';
        if ($this ->browserAction) {
            echo
                // tlacitko pro provedeni akce
                "<button class='submitbutton' id='button_provest_{$this ->browserAction}'>" .
                TextManager::_('Jidelna_provest_' . $this ->browserAction) .
                '</button>' .
                '<span class="margin-all-4"> </span>' .
                // tlacitko storno
                '<button class="button setValue" data-value="" data-selector="#objednavky_browseraction" data-submit="1">' .
                TextManager::_('Cancel') .
                '</button>';
        } else {
            // hromadne zrusit
            if ($this->pravoZruseni->jsouNejakePrava()) {
                echo
                    '<button class="button setValue" id="button_hromadnezruseni" data-value="zruseni" data-selector="#objednavky_browseraction" data-submit="1">' .
                    TextManager::_('Jidelna_hromadne_zruseni') .
                    '</button>';
            }

            // hromadne prevest
            if ($this->pravoPrevedeni->jsouNejakePrava()) {
                echo
                    '<button class="button setValue" id="button_hromadneprevedeni" data-value="prevedeni" data-selector="#objednavky_browseraction" data-submit="1">' .
                    TextManager::_('Jidelna_hromadne_prevedeni') .
                    '</button>';
            }

            // hromadne predat do burzy
            if (RonPortalSettings::getIntValue(RonPortalSettings::BURZA_POVOLENA) && !$this->browserAction && Prava::jsouNejakePrava('jidelna_do_burzy')) {
                echo
                    '<button class="button setValue" id="button_hromadnedoburzy" data-value="doburzy" data-selector="#objednavky_browseraction" data-submit="1">' .
                    TextManager::_('Jidelna_hromadne_do_burzy') .
                    '</button>';
            }
        }

        echo '</div>';
    }



    public function getRowAttributes($values, $className, $rowNumber)
    {
        return
            'data-id-objednavky="' . $values['col-id'] . '" ' .
            parent::getRowAttributes($values, ($values['col-datumcaszruseni'] ? 'zrusena-objednavka ' : '') . $className, $rowNumber);
    }

    public function render()
    {
        parent::render();

        if ($this ->browserAction) {
            ProgressDialog::renderProgressDialog(TextManager::_('Jidelna_seznam_objednavek'), [
                'caption' => TextManager::_('Probiha_hromadna_operace_' . $this ->browserAction)
            ]);
        }

    }


}
