<?php

/**
 * Vyuctovani odberu stravenek za stravnika
 */

namespace RonPortal\Jidelna;

use Ron\Controls\Browser;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\TextManager;


class VyuctovaniStravenekBrowser extends Browser
{
    var $data;

    /** @var int */
    var $idPP;

    /**
     * @param Database $database
     * @param int $idPP
     */
    function __construct($database, $idPP)
    {
        parent::__construct($database);

        $this ->idPP = $idPP;

        $this->title = TextManager::_('Vyuctovani_stravenek');
        $this ->cssClass = 'browser oddEvenRows';

        $this->code = 'browservyuctovanistravenek';

        $this->addField()
            ->setSqlFieldName('1', 0) ->setHeaderText('Mesic') ->setAlign('center')
            ->setCode('col-mesic');

        $this->addField()
            ->setSqlFieldName('2', 1) ->setHeaderText('Narok_na_dotace') ->setAlign('right');

        $this->addField()
            ->setSqlFieldName('3', 2) ->setHeaderText('Odebrane_stravenky') ->setAlign('right');

        $this->addField()
            ->setSqlFieldName('4', 3) ->setHeaderText('Zbyva_odebrat') ->setAlign('right');

        $this ->showFooter = true;
    }


    /**
     * @param int $rok
     * @throws DBException
     */
    function loadVyuctovani($rok)
    {
        $this ->data = [];

        for ($i = 1; $i <= 12; $i++) $this ->data[$i] = [$i, 0, 0, 0];

        $datumdo = mktime(0, 0, 0, 12, 31, $rok);

        // Nacte naroky podle mesicniho vyuctovani
        $params = $this ->database ->createParameters();
        $sql =
            'SELECT Obdobi, Pocet_NarokNaDotace' .
            ' FROM UcetStravnikaZaObdobi, PracovniPomer' .
            ' WHERE Obdobi >= ' . $params ->addParameter($this ->database ->dateToParam(mktime(0, 0, 0, 1, 1, $rok))) .
            '   AND Obdobi <= ' . $params ->addParameter($this ->database ->dateToParam($datumdo)) .
            '   AND UcetStravnikaZaObdobi.RC = PracovniPomer.RC' .
            '   AND UcetStravnikaZaObdobi.OsobniCislo = PracovniPomer.OsobniCislo' .
            '   AND UcetStravnikaZaObdobi.DatumNastupu = PracovniPomer.DatumNastupu' .
            '   AND PracovniPomer.Id = ' . $params ->addParameter($this ->idPP) .
            ' ORDER BY Obdobi';

        $cursor = $this ->database ->getSqlResult($sql, $params);
        while ($row = $cursor ->fetch()) {
            $index = (int)date('n', $this ->database ->stringToTimestamp($row[0])); // index mesice
            $this ->data[$index][1] = $row[1]; // pocet naroku na dotace
        }
        $cursor ->close();

        // Nacte Odbery
        $params = $this ->database ->createParameters();
        $sql =
            'SELECT Obdobi, Pocet' .
            ' FROM PohybStravenek, PracovniPomer ' .
            ' WHERE PohybStravenek.Obdobi >= ' . $params ->addParameter($this ->database ->dateToParam(mktime(0, 0, 0, 1, 1, $rok))) .
            '   AND PohybStravenek.Obdobi < ' . $params ->addParameter($this ->database ->dateToParam(mktime(0, 0, 0, 1, 1, $rok + 1))) .
            '   AND PohybStravenek.RC = PracovniPomer.RC' .
            '   AND PohybStravenek.OsobniCislo = PracovniPomer.OsobniCislo' .
            '   AND PohybStravenek.DatumNastupu = PracovniPomer.DatumNastupu' .
            '   AND PracovniPomer.Id = ' . $params ->addParameter($this ->idPP) .
            ' ORDER BY PohybStravenek.Obdobi';

        $cursor = $this ->database ->getSqlResult($sql, $params);
        while ($row = $cursor ->fetch()) {
            $index = (int)date('n', $this ->database ->stringToTimestamp($row[0])); // index mesice
            $this ->data[$index][2] -= $row[1]; // pricte pocet odebranych stravenek (pohyb v DB je minusovy)
        }
        $cursor ->close();

        // spocita rozdily
        for ($i = 1; $i <= 12; $i++)
            $this ->data[$i][3] = $this ->data[$i][1] - $this ->data[$i][2];
    }


    function load($sqlFilter = '')
    {
        if (isset($_GET['rok'])) $_SESSION['rok'] = (int)$_GET['rok'];
        if (!isset($_SESSION['rok'])) $_SESSION['rok'] = date('Y');

        $this ->loadVyuctovani((int)$_SESSION['rok']);
    }


    function renderData()
    {
        foreach ($this ->data as $index => $row) {
            $cssClass = ($index == count($this ->data)) ? 'border-bottom' : '';
            $this->renderLine($row, $cssClass, $index);
        }
    }

    function renderTableFooter()
    {
        $sum = [TextManager::_('Celkem'), 0, 0, 0]; // narok, odbery, rozdil

        foreach ($this ->data as $row) {
            $sum[1] += $row[1];
            $sum[2] += $row[2];
            $sum[3] += $row[3];
        }

        $this->renderLine($sum, "browserfooter", 13);
    }


    function renderFilter()
    {
        $rok = (int)$_SESSION['rok'];

        echo
            '<div>' .
            '<a href="?menuid=stravenky&rok=' . ($rok - 1) . '"> <<  </a>' .
            "<span class='margin-left-2 margin-right-2'>$rok</span>" .
            '<a href="?menuid=stravenky&rok=' . ($rok + 1) . '"> >> </a>' .
            '</div>';
    }

}
