<?php

namespace RonPortal\Kapacity;

use Exception;
use Ron\DB\Database;
use Ron\GuiUtils;
use Ron\StringUtils;
use Ron\TextManager;

class PlanovaniKapacit_Absence
{
    public $kod;
    public $popis;
    public $zkratka;
    public $barva;

    /** @var int[]  Seznam povolenych delek */
    public $povoleneDelky = [];

    private function nactiData($row)
    {
        // SELECT Kod, Popis, ZkratkaProPlanovani, BarvaProVykresleni
        $this ->kod = $row[0];
        $this ->popis = TextManager::_('db_operacedochazky_popis_' . $row[0], $row[1]);
        $this ->zkratka = $row[2];
        $this ->barva = $row[3];

        if (isset($row[4])) {
            $this ->povoleneDelky = [$row[4]];
        }
    }


    /**
     * @param Database $database
     * @param bool $nacistPovoleneDelky     Urcuje, jestli se maji nacist i seznamy povolenych delek absenci
     * @return PlanovaniKapacit_Absence[]
     * @throws Exception
     */
    public static function nactiSeznamOperaci($database, $nacistPovoleneDelky = false)
    {
        $result = [];

        // nacte seznam absenci
        $sql =
            'SELECT OperaceDochazky.Kod, OperaceDochazky.Popis, OperaceDochazky.ZkratkaProPlanovani, OperaceDochazky.BarvaProVykresleni' .
            ($nacistPovoleneDelky ? ',  PovolenaDelkaAbsence.Delka ' : '') .
            ' FROM OperaceDochazky' .
            ($nacistPovoleneDelky ? ' LEFT OUTER JOIN PovolenaDelkaAbsence ON PovolenaDelkaAbsence.OperaceDochazky = OperaceDochazky.Kod' : '') .
            ' WHERE OperaceDochazky.Absence = 1 ' .
            ' ORDER BY OperaceDochazky.Kod';


        /** @var PlanovaniKapacit_Absence $posledniAbsence */
        $posledniAbsence = null;

        if ($cursor = $database ->select($sql)) {
            while ($row = $cursor ->fetch()) {
                if ($nacistPovoleneDelky  &&  $posledniAbsence  &&  $posledniAbsence ->kod == $row[0]) {
                    $posledniAbsence ->povoleneDelky[] = $row[4];
                } else {
                    $posledniAbsence = new self();
                    $posledniAbsence->nactiData($row);
                    $result[] = $posledniAbsence;
                }
            }
        }

        return $result;
    }


    public function getJson()
    {
        $result = [
            'kod' => StringUtils::jsonText($this ->kod),
            'popis' => StringUtils::jsonText($this ->popis),
            'zkratka' => StringUtils::jsonText($this ->zkratka),
            'barva' => GuiUtils::getHtmlColor($this ->barva)
        ];

        if ($this ->povoleneDelky) {
            $result['povoleneDelky'] = $this ->povoleneDelky;
        }

        return $result;
    }
}
