<?php

namespace RonPortal\Kapacity;

use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\SqlStatement;
use Ron\Http\Csrf;
use Ron\StringUtils;
use RonPortal\Dochazka\DokumentDochazky;
use RonPortal\PracovniPomer;
use RonPortal\RonPortalSettings;
use RonPortal\RozdeleniOsob;

class PlanovaniKapacit_Osoba
{
    public $jmenoPrijmeni;
    public $jmeno;
    public $prijmeni;
    public $rc;
    public $osobniCislo;
    public $datumNastupu;
    public $datumUkonceni;
    public $idPP;

    /** @var bool       True, pokud je osoba pouze pujcena z jineho rozdeleni */
    public $cizi;

    public $rozdeleni;
    /** @var string[]   Hodnoty uzivatelskych poli osoby */
    public $uzivatelske;
    /** @var string[] */
    public $uzivatelskePP;

    /** @var bool       Priznak, jestli muze uzivatel planovat smeny */
    public $lzeEditovat;

    /** @var bool       Priznak, jestli muze uzivatel planovat absence */
    public $lzePlanovatAbsence;

    /** @var array      Hodnoty dodatecnych sloupcu (casovych slozek) v hlavicce */
    public $hodnotyHlavicky;
    /** @var string     Atribut class pro dany sloupec hlavicky */
    public $classHlavicky;
    /** @ var int        Barva podle nastaveneho typu rozdeleni (pouzije se pro obarveni radku) */
    //public $barva = null;

    /** @var RozdeleniOsob[]   Seznam ikon rozdeleni, ktere se maji zobrazit */
    public $ikony;

    /** @var RozdeleniOsob[]   Seznam dnu, kdy lze editovat i po schvaleni */
    public $moznostEditaceSchvalenych;

    // prava na schvalovani
    public $lzeSchvalit1;
    public $lzeSchvalit2;
    public $lzeSchvalit3;

    /** @var int  Datum schvaleno do 1 */
    public $schvaleni1;
    /** @var int  Datum schvaleno do 2 */
    public $schvaleni2;
    /** @var int  Datum schvaleno do 3 */
    public $schvaleni3;

    /** @var int  Datum schvaleno do (puvodni plan) 1 */
    public $schvaleniPuvodnihoPlanu1;
    /** @var int  Datum schvaleno do (puvodni plan) 2 */
    public $schvaleniPuvodnihoPlanu2;
    /** @var int  Datum schvaleno do (puvodni plan) 3 */
    public $schvaleniPuvodnihoPlanu3;

    public $schvaleniPuvodnihoPlanu;

    /** @var int|null */
    public $casSchvaleni1;
    /** @var string */
    public $schvalil1;
    /** @var int|null */
    public $casSchvaleni2;
    /** @var string */
    public $schvalil2;
    /** @var int|null */
    public $casSchvaleni3;
    /** @var string */
    public $schvalil3;

    // data pro pouziti v HTML
    /** @var PlanovaniKapacit_PlanovanaSmena[] */
    public $seznamSmen;
    /** @var PlanovaniKapacit_PlanovanaAbsence[] */
    public $seznamAbsenci;
    /** @var PlanovaniKapacit_PrirazeniKalendare[] */
    public $seznamKalendaru;
    /** @var DokumentDochazky[] */
    public $dokumentyDochazky;
    /** @var PlanovaniKapacit_SluzebniCesta[] */
    public $sluzebniCesty;
    /** @var VysledekKontrolyPlanovani[] */
    public $kontroly;

    /** @var RozdeleniOsob[]    Nacita se pro vypocet souctu podle podminek ve slotovem planovani*/
    public $rozdeleniOsoby = [];

    // data pro pouziti v HTML - primo jednotlive bunky
    /** @var PlanovaniKapacit_Bunka[]  */
    //public $data;

    /** @var float Limit - nacte se z (nepovinne) nastavene casove slozky */
    public $maximalniPocetHodin;

    /** @var string     CSS tridy pro oznaceni radku - napr. podle mesicnich vysledku nebo podle barev rozdeleni */
    public $rowClasses = '';

    /**
     * @param Database $database
     * @param int  $datumOd
     * @param int  $datumDo
     * @param bool $rezimUprav
     * @throws DBException
     */
    public function nactiSmeny($database, $datumOd, $datumDo, $rezimUprav)
    {
        $this ->seznamSmen = PlanovaniKapacit_PlanovanaSmena::nactiSmeny($database,
            $this ->idPP, $datumOd, $datumDo, $rezimUprav, true); // nacist rozdeleni = true
    }


    /**
     * @param Database $database
     * @param int $datumOd
     * @param int $datumDo
     * @param bool $rezimUprav
     * @throws DBException
     */
    public function nactiAbsence($database, $datumOd, $datumDo, $rezimUprav)
    {
        // vraci seznam absenci pro zadanou osobu a rozsah dnu
        $this ->seznamAbsenci = PlanovaniKapacit_PlanovanaAbsence::nactiAbsence($database,
            $this ->idPP, $datumOd, $datumDo, false, $rezimUprav);
    }


    /**
     * Nacte datumy, ve kterych lze editovat plan osoby i kdyz je uz schvalena
     * @param Database $database
     * @param int $datumOd
     * @param int $datumDo
     * @return ?RozdeleniOsob[]
     */
    public static function nactiPovoleniEditovatSchvalene($database, $idPP, $datumOd, $datumDo)
    {
        $r_str = trim(RonPortalSettings::getValue('kapacity_editovat_schvalene_r') ?? '');
        if (!$r_str) {
            return null;
        }

        // $r_str obsahuje typ a kod rozdeleni (ve format <typ>/<kod>), ktere se muze editovat i schvalene
        // (tj. kdyz je zamestnanec v tomto rozdeleni, tak lze editovat i ve schvalenych dnech)
        // Lze pouzit napr. pro povoleni editace zamestnancu tesne po nastupu
        $pomA = explode('/', $r_str, 2);
        if ((int)$pomA[0]  &&  isset($pomA[1])) {
            $params = $database ->createParameters();

            return RozdeleniOsob::nactiRozdeleniOsoby($database,
                $idPP, $pomA[0], $datumOd, $datumDo,
                new SqlStatement('Rozdeleni = ' . $params ->addParameter($pomA[1]), $params)
            );
        }

        return null;
    }


    /**
     * Nacte data osoby.
     * @param Database $database
     * @param array $row
     */
    public function nactiData($database, $row)
    {
        $this ->rc = $row[0];
        $this ->osobniCislo = $row[1];
        $this ->datumNastupu = $database ->stringToTimestamp($row[2]);

        $this ->datumUkonceni = $row[7]
            ? $database ->stringToTimestamp($row[7])
            : null;

        $this ->jmeno = $row[3];
        $this ->prijmeni = $row[4];
        $this ->jmenoPrijmeni = $row[4] . " " . $row[3];
        if (trim($row[5] ?? '')) {
            $this ->jmenoPrijmeni .= ', ' . $row[5];
        }

        $this ->idPP = (int)$row[6];

        if (!RonPortalSettings::getIntValue(KapacitySettings::ZPUSOB_SCHVALOVANI)) {
            $this->schvaleni1 = $database->stringToTimestamp($row[8]);
            $this->schvaleni2 = $database->stringToTimestamp($row[9]);
            $this->schvaleni3 = $database->stringToTimestamp($row[10]);

            $this->schvaleniPuvodnihoPlanu1 = $database->stringToTimestamp($row[51]);
            $this->schvaleniPuvodnihoPlanu2 = $database->stringToTimestamp($row[52]);
            $this->schvaleniPuvodnihoPlanu3 = $database->stringToTimestamp($row[53]);


            if (RonPortalSettings::getIntValue(KapacitySettings::POVOLIT_REZIM_UPRAV) && RonPortalSettings::getIntValue(KapacitySettings::MIN_SCHVALENI_PRO_UPRAVY)) {
                $this->schvaleniPuvodnihoPlanu = null;

                if ($this->schvaleniPuvodnihoPlanu1 && RonPortalSettings::getIntValue(KapacitySettings::MIN_SCHVALENI_PRO_UPRAVY) === 1) {
                    $this->schvaleniPuvodnihoPlanu = $this->schvaleniPuvodnihoPlanu1;
                }

                if ($this->schvaleniPuvodnihoPlanu2 && RonPortalSettings::getIntValue(KapacitySettings::MIN_SCHVALENI_PRO_UPRAVY) <= 2) {
                    if (!$this->schvaleniPuvodnihoPlanu || $this->schvaleniPuvodnihoPlanu < $this->schvaleniPuvodnihoPlanu2) {
                        $this->schvaleniPuvodnihoPlanu = $this->schvaleniPuvodnihoPlanu2;
                    }
                }

                if ($this->schvaleniPuvodnihoPlanu3) {
                    if (!$this->schvaleniPuvodnihoPlanu || $this->schvaleniPuvodnihoPlanu < $this->schvaleniPuvodnihoPlanu3) {
                        $this->schvaleniPuvodnihoPlanu = $this->schvaleniPuvodnihoPlanu3;
                    }
                }
            }
        }

        // uzivatelske polozky
        $this ->uzivatelske = [];
        for ($i = 0; $i < 20; $i++) {
            $this ->uzivatelske[] = $row[11 + $i];
        }

        $this ->uzivatelskePP = [];
        for ($i = 0; $i < 20; $i++) {
            $this ->uzivatelskePP[] = $row[31 + $i];
        }

        $this ->casSchvaleni1 = $database ->stringToTimestamp($row[54]);
        $this ->schvalil1 = $row[55];
        $this ->casSchvaleni2 = $database ->stringToTimestamp($row[56]);
        $this ->schvalil2 = $row[57];
        $this ->casSchvaleni3 = $database ->stringToTimestamp($row[58]);
        $this ->schvalil3 = $row[59];

        // kod rozdeleni
        $this ->rozdeleni = $row[60] ?? null;

        $this ->seznamSmen = [];
        $this ->seznamAbsenci = [];
        $this ->seznamKalendaru = [];
    }


    /**
     * @param Database $database
     * @param int $idpp
     * @param int $datumOd
     * @param int $datumDo
     * @param bool $rezimUprav
     * @throws DBException
     */
    public function nactiDataProRefresh($database, $idpp, $datumOd, $datumDo, $rezimUprav)
    {
        $pp = PracovniPomer::nactiPP($database, $idpp);

        $this ->rc = $pp ->rc;
        $this ->osobniCislo = $pp ->osobniCislo;
        $this ->datumNastupu = $pp ->datumNastupu;
        $this ->datumUkonceni = $pp ->datumUkonceni;
        $this ->jmeno = $pp ->jmeno;
        $this ->prijmeni = $pp ->prijmeni;
        $this ->jmenoPrijmeni = $pp ->jmenoPrijmeni;
        $this ->idPP = $idpp;

        // podrobnosti nenacita, ale promenne inicializuje
        $this ->uzivatelske = [];
        $this ->uzivatelskePP = [];
        $this ->hodnotyHlavicky = [];
        $this ->seznamKalendaru = [];

        if (!RonPortalSettings::getIntValue(KapacitySettings::ZPUSOB_SCHVALOVANI)) {
            $schvaleni = SchvaleniKapacitPP::nactiSchvaleni($database, $datumOd, $this->idPP);
            if ($rezimUprav) {
                $this->schvaleni1 = $schvaleni->schvaleniUprav1;
                $this->schvaleni2 = $schvaleni->schvaleniUprav2;
                $this->schvaleni3 = $schvaleni->schvaleniUprav3;
            } else {
                $this->schvaleni1 = $schvaleni->schvaleni1;
                $this->schvaleni2 = $schvaleni->schvaleni2;
                $this->schvaleni3 = $schvaleni->schvaleni3;
            }

            $this->schvaleniPuvodnihoPlanu = $schvaleni->getSchvaleniPuvodnihoPlanu();
        }

        $this ->nactiSmeny($database, $datumOd, $datumDo, $rezimUprav);
        $this ->nactiAbsence($database, $datumOd, $datumDo, $rezimUprav);
    }



    public function getJson($onlyHeader = false)
    {
        $result = [
            'idPP' => $this ->idPP,
            'jmenoPrijmeni' => StringUtils::jsonText($this ->jmenoPrijmeni),
            'jmeno' => StringUtils::jsonText($this ->jmeno),
            'prijmeni' => StringUtils::jsonText($this ->prijmeni),
            'osobniCislo' => StringUtils::jsonText($this ->osobniCislo),
            'datumNastupuStr' => date('j.n.Y', $this ->datumNastupu),
            'csrfkey' => Csrf::getCsrfKey('pk_' . $this ->idPP)
        ];

        if ($this ->datumUkonceni) {
            $result['datumUkonceniStr'] = date('j.n.Y', $this ->datumUkonceni);
        }

        if ($this ->cizi) {
            $result['cizi'] = 1;
        }

        if ($this ->schvaleni1) {
            $result['schvaleni1Str'] = date('j.n.Y', $this ->schvaleni1);
        }
        if ($this ->schvaleni2) {
            $result['schvaleni2Str'] = date('j.n.Y', $this ->schvaleni2);
        }
        if ($this ->schvaleni3) {
            $result['schvaleni3Str'] = date('j.n.Y', $this ->schvaleni3);
        }

        if ($this ->schvaleniPuvodnihoPlanu) {
            $result['schvaleniPuvodnihoPlanuStr'] = date('j.n.Y', $this ->schvaleniPuvodnihoPlanu);
        }

        if ((int)$this ->lzeEditovat) {
            $result['lzeEditovat'] = (int)$this ->lzeEditovat;
        }

        if ($this ->lzePlanovatAbsence) {
            $result['lzePlanovatAbsence'] = $this ->lzePlanovatAbsence;
        }

        if ($this ->lzeSchvalit1) {
            $result['lzeSchvalit1'] = (int)$this ->lzeSchvalit1;
        }
        if ($this ->lzeSchvalit2) {
            $result['lzeSchvalit2'] = (int)$this ->lzeSchvalit2;
        }
        if ($this ->lzeSchvalit3) {
            $result['lzeSchvalit3'] = (int)$this ->lzeSchvalit3;
        }

        $result['maxUrovenSchvaleni'] = $this ->lzeSchvalit3
            ? 3
            : ($this ->lzeSchvalit2 ? 2 : ($this ->lzeSchvalit1 ? 1 : 0));

        //if ($this ->f_error) $result['error'] = StringUtils::jsonText($this ->f_error);

        if ($this ->maximalniPocetHodin) {
            $result['maximalniPocetHodin'] = $this ->maximalniPocetHodin;
        }

        if ($onlyHeader) {
            return $result;
        }

        $smeny = [];
        foreach ($this ->seznamSmen as $smena) {
            $smeny[] = $smena ->getJSON();
        }
        $result['smeny'] = $smeny;

        $seznamAbsenci = [];
        foreach ($this ->seznamAbsenci as $absence) {
            $seznamAbsenci[] = $absence ->getJSON();
        }
        $result['absence'] = $seznamAbsenci;

        $kalendare = [];
        foreach ($this ->seznamKalendaru as $kalendar) {
            $kalendare[] = $kalendar ->getJson();
        }
        $result['kalendare'] = $kalendare;

        // hodnoty v hlavicce (mesicni vysledky atd.)
        $hlavicky = [];
        foreach ($this ->hodnotyHlavicky as $i => $hodnota) {
            $obj = [
                //'sloupec' => $i,
                'text' => StringUtils::jsonText($hodnota)
            ];

            if (isset($this ->classHlavicky[$i])) {
                $obj['className'] = $this ->classHlavicky[$i];
            }

            $hlavicky[$i] = $obj;
        }

        $result['dataHlavicky'] = $hlavicky;

        if ($this ->ikony) {
            $result['ikony'] = [];
            foreach ($this ->ikony as $ikona) {
                $result['ikony'][] = ['css' => $ikona ->cssClassObrazku, 'popis' => StringUtils::jsonText($ikona ->title)];
            }
        }

        if ($this ->rozdeleniOsoby) {
            $result['rozdeleniOsoby'] = [];
            foreach ($this ->rozdeleniOsoby as $rozdeleniOsoby) {
                $result['rozdeleniOsoby'][] = $rozdeleniOsoby ->getJson();
            }
        }

        if ($this ->moznostEditaceSchvalenych) {
            $result['moznostEditaceSchvalenych'] = [];
            foreach ($this ->moznostEditaceSchvalenych as $rozdeleniOsoby) {
                $result['moznostEditaceSchvalenych'][] = $rozdeleniOsoby ->getJson();
            }
        }


        if ($this ->rowClasses) {
            $result['rowClasses'] = StringUtils::jsonText($this ->rowClasses);
        }

        return $result;
    }


    public function jeVRozdeleniOsoby($kodTypuRozdeleni, $kodRozdeleni, $datum)
    {
        if (!$this ->rozdeleniOsoby) {
            return false;
        }

        foreach ($this ->rozdeleniOsoby as $rozdeleniOsoby) {
            if ($rozdeleniOsoby ->kodTypuRozdeleni === $kodTypuRozdeleni  &&  $rozdeleniOsoby ->kod === $kodRozdeleni) {
                if ($datum) {
                    if ($rozdeleniOsoby ->platnostOd <= $datum  &&  (!$rozdeleniOsoby ->platnostDo  ||  $rozdeleniOsoby ->platnostDo >= $datum)) {
                        return true;
                    }
                } else {
                    return true;
                }
            }
        }

        return false;
    }
}
