<?php
/**
 * stanislav.elbl, 13.09.2019
 */

namespace RonPortal\Kapacity;

use Exception;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\Parameters;
use Ron\GuiUtils;
use Ron\StringUtils;
use RonPortal\RonPortalSettings;


/**
 * Pouzije se i v kalendari pro seznam povolenych smen!
 * Class PlanovaniKapacit_PlanovanaSmena
 */
class PlanovaniKapacit_PlanovanaSmena
{
    const SETTINGS_TEXT = 'kapacity_text';  // text v bunce (s parametry)

    public $idPP;

    public $datum;
    public $kod;
    public $potvrzeno;

    public $zkratka;
    public $barva;

    public $variabilni;
    /** @var int    Pocet sekund od pulnoci */
    public $pocatek;
    /** @var int    Pocet sekund od pulnoci */
    public $konec;
    /** @var int    Pocet sekund */
    public $delka;

    public $delena;
    /** @var int */
    public $preruseniOd;
    /** @var int */
    public $preruseniDo;

    /** @var string Smena v puvodnim/opravenem planu - opacny nez kod, zkratka, barva */
    public $kod2;
    public $zkratka2;
    public $barva2;
    /*
    var $f_puvodni_pocatek;
    var $f_puvodni_konec;
    var $f_puvodni_delka; */

    // podrobnosti pro detailni zobrazeni
    public $kodTypuRozdeleni;
    public $rozdeleni;
    public $popisRozdeleni;
    public $barvaRozdeleni;

    public $kodTypuRozdeleni2;
    public $rozdeleni2;
    public $popisRozdeleni2;
    public $barvaRozdeleni2;

    public $harmonogram;
    public $update_UserName;
    public $update_TimeStamp;


    public function __construct()
    {
        $this ->datum = 0;
    }


    /**
     * @param Database $database
     * @param Parameters $params
     * @param int|int[] $idPP
     * @param int $datumOd
     * @param int $datumDo
     * @param bool $rezimUprav
     * @param bool $nacistRozdeleni     Priznak urcuje, jestli se maji nacist popisy a barvy rozdeleni
     * @return string
     */
    public static function getSQL($database, $params, $idPP, $datumOd, $datumDo, $rezimUprav, $nacistRozdeleni = false)
    {
        if ((RonPortalSettings::getIntValue(KapacitySettings::POVOLIT_REZIM_UPRAV) && !$rezimUprav)) {
            // SQL dotaz pro puvodni plan
            $sql =
                'SELECT PlanovaniKapacit.Datum, PlanovaniKapacit.PuvodniPlan_Smena, PlanovaniKapacit.PuvodniPlan_Potvrzeno, ' .
                '       Smena.ZkratkaProPlanovani, Smena.BarvaProVykresleni, ' . // 3,4
                '       Smena.Variabilni, PlanovaniKapacit.PuvodniPlan_Smena_Pocatek, PlanovaniKapacit.PuvodniPlan_Smena_Konec, PlanovaniKapacit.PuvodniPlan_Smena_Delka, ' . // 5..8
                '       PlanovaniKapacit.PuvodniPlan_Rozdeleni, PlanovaniKapacit.Harmonogram, ' . // 9..10
                '       Smena.PocatekSmeny, Smena.KonecSmeny, Smena.DelkaSmeny, ' . // 11..13
                '       PlanovaniKapacit.Smena, PlanovaniKapacit.PuvodniPlan_Rozdeleni2, ' . // 14,15
                '       Smena.Delena, PlanovaniKapacit.PuvodniPlan_PreruseniOd, PlanovaniKapacit.PuvodniPlan_PreruseniDo, ' . // 16..18
                // smena2 (z rezimu uprav)
                '       Smena2.ZkratkaProPlanovani, Smena2.BarvaProVykresleni, ' . // 19..20
                '       PracovniPomer.Id' . // 21
                ($nacistRozdeleni ? ', Rozdeleni1.Popis, Rozdeleni1.BarvaProVykresleni, Rozdeleni2.Popis, Rozdeleni2.BarvaProVykresleni' : '') . // 22..25
                '  FROM PlanovaniKapacit INNER JOIN PracovniPomer' .
                '    ON PlanovaniKapacit.RC = PracovniPomer.RC' .
                '   AND PlanovaniKapacit.OsobniCislo = PracovniPomer.OsobniCislo' .
                '   AND PlanovaniKapacit.DatumNastupu = PracovniPomer.DatumNastupu' .
                '  LEFT OUTER JOIN Smena ' .
                '    ON Smena.Kod = PlanovaniKapacit.PuvodniPlan_Smena ' .
                '  LEFT OUTER JOIN Smena Smena2 ' .
                '    ON Smena2.Kod = PlanovaniKapacit.Smena';
        } else if (RonPortalSettings::getIntValue(KapacitySettings::POVOLIT_REZIM_UPRAV)) {
            // SQL dotaz pro rezim uprav
            $sql =
                'SELECT PlanovaniKapacit.Datum, PlanovaniKapacit.Smena, PlanovaniKapacit.Potvrzeno, ' .
                '       Smena.ZkratkaProPlanovani, Smena.BarvaProVykresleni, ' . // 3,4
                '       Smena.Variabilni, PlanovaniKapacit.Smena_Pocatek, PlanovaniKapacit.Smena_Konec, PlanovaniKapacit.Smena_Delka, ' . // 5..8
                '       PlanovaniKapacit.Rozdeleni, PlanovaniKapacit.Harmonogram, ' . // 9..10
                '       Smena.PocatekSmeny, Smena.KonecSmeny, Smena.DelkaSmeny, ' . // 11..13
                '       PlanovaniKapacit.PuvodniPlan_Smena, PlanovaniKapacit.Rozdeleni2, ' . // 14,15
                '       Smena.Delena, PlanovaniKapacit.Smena_PreruseniOd, PlanovaniKapacit.Smena_PreruseniDo, ' . // 16..18
                // smena 2
                '       Smena2.ZkratkaProPlanovani, Smena2.BarvaProVykresleni, ' . // 19..20
                '       PracovniPomer.Id' . // 21
                ($nacistRozdeleni ? ', Rozdeleni1.Popis, Rozdeleni1.BarvaProVykresleni, Rozdeleni2.Popis, Rozdeleni2.BarvaProVykresleni' : '') . // 22..25
                '  FROM PlanovaniKapacit INNER JOIN PracovniPomer' .
                '    ON PlanovaniKapacit.RC = PracovniPomer.RC' .
                '   AND PlanovaniKapacit.OsobniCislo = PracovniPomer.OsobniCislo' .
                '   AND PlanovaniKapacit.DatumNastupu = PracovniPomer.DatumNastupu' .
                '  LEFT OUTER JOIN Smena ' .
                '    ON Smena.Kod = PlanovaniKapacit.Smena ' .
                '  LEFT OUTER JOIN Smena Smena2 ' .
                '    ON Smena2.Kod = PlanovaniKapacit.PuvodniPlan_Smena';
        } else {
            // rezim uprav se vubec nepouziva
            $sql =
                'SELECT PlanovaniKapacit.Datum, PlanovaniKapacit.Smena, PlanovaniKapacit.Potvrzeno, ' .
                '       Smena.ZkratkaProPlanovani, Smena.BarvaProVykresleni, ' . // 3,4
                '       Smena.Variabilni, PlanovaniKapacit.Smena_Pocatek, PlanovaniKapacit.Smena_Konec, PlanovaniKapacit.Smena_Delka, ' . // 5..8
                '       PlanovaniKapacit.Rozdeleni, PlanovaniKapacit.Harmonogram, ' . // 9..10
                '       Smena.PocatekSmeny, Smena.KonecSmeny, Smena.DelkaSmeny, ' . // 11..13
                '       PlanovaniKapacit.PuvodniPlan_Smena, PlanovaniKapacit.Rozdeleni2, ' . // 14,15
                '       Smena.Delena, PlanovaniKapacit.Smena_PreruseniOd, PlanovaniKapacit.Smena_PreruseniDo, ' . // 16..18
                '       NULL AS Zkratka2, NULL AS Barva2, ' .
                '       PracovniPomer.Id' . // 21
                ($nacistRozdeleni ? ', Rozdeleni1.Popis, Rozdeleni1.BarvaProVykresleni, Rozdeleni2.Popis, Rozdeleni2.BarvaProVykresleni' : '') . // 22..25
                '  FROM PlanovaniKapacit INNER JOIN PracovniPomer' .
                '    ON PlanovaniKapacit.RC = PracovniPomer.RC' .
                '   AND PlanovaniKapacit.OsobniCislo = PracovniPomer.OsobniCislo' .
                '   AND PlanovaniKapacit.DatumNastupu = PracovniPomer.DatumNastupu' .
                '  LEFT OUTER JOIN Smena ' .
                '    ON PlanovaniKapacit.Smena = Smena.Kod ';
        }

        if ($nacistRozdeleni) {
            $sql .=
                ' LEFT OUTER JOIN Rozdeleni Rozdeleni1' .
                '  ON PlanovaniKapacit.KodTypuRozdeleni = Rozdeleni1.KodTypuRozdeleni' .
                '  AND PlanovaniKapacit.Rozdeleni = Rozdeleni1.Kod' .
                ' LEFT OUTER JOIN Rozdeleni Rozdeleni2' .
                '  ON PlanovaniKapacit.KodTypuRozdeleni2 = Rozdeleni2.KodTypuRozdeleni' .
                '  AND PlanovaniKapacit.Rozdeleni2 = Rozdeleni2.Kod';
        }

        $sql .=
            ' WHERE PlanovaniKapacit.Datum >= ' . $params ->addParameter($database ->dateToParam($datumOd)) .
            '   AND PlanovaniKapacit.Datum <= ' . $params ->addParameter($database ->dateToParam($datumDo));

        if (is_array($idPP)) {
            $sql .= ' AND (0 = 1';
            foreach ($idPP as $id) {
                $sql .=
                    ' OR PracovniPomer.Id = ' . $params->addParameter((int)$id);
            }
            $sql .= ')';
        } else {
            $sql .=
                ' AND PracovniPomer.Id = ' . $params->addParameter((int)$idPP);
        }

        return
            $sql .
            ' ORDER BY PracovniPomer.Id, PlanovaniKapacit.Datum';
    }


    /**
     * @param Database $database
     * @param array $row
     */
    public function nactiData($database, $row)
    {
        // nacita jen zakladni data pro hromadne zobrazeni (okno planovani)
        $this ->idPP = (int)$row[21];

        $this ->datum = $database ->stringToTimestamp($row[0]);
        $this ->kod = $row[1];
        $this ->potvrzeno = (int)$row[2];

        $this ->zkratka = $row[3];
        $this ->barva = $row[4];

        $this ->kod2 = $row[14]; // rezim planovani/uprav

        $this ->variabilni = (int)$row[5];
        if ($this ->variabilni) {
            $this ->pocatek = $row[6] ? $database ->stringToSecondsOfDay($row[6]) : null;
            $this ->konec = $row[7] ? $database ->stringToSecondsOfDay($row[7]) : null;
            $this ->delka = $row[8] ? $database ->stringToSecondsOfDay($row[8]) : null;
        } else {
            $this ->pocatek = $row[11] ? $database ->stringToSecondsOfDay($row[11]) : null;
            $this ->konec = $row[12] ? $database ->stringToSecondsOfDay($row[12]) : null;
            $this ->delka = $row[13] ? $database ->stringToSecondsOfDay($row[13]) : null;
        }

        $this ->delena = (int)$row[16];
        if ($this ->delena) {
            $this ->preruseniOd = $row[17] ? $database ->stringToSecondsOfDay($row[17]) : null;
            $this ->preruseniDo = $row[18] ? $database ->stringToSecondsOfDay($row[18]) : null;
        }

        $this ->rozdeleni = $row[9];
        $this ->harmonogram = $row[10];
        $this ->rozdeleni2 = $row[15];

        if (RonPortalSettings::getIntValue(KapacitySettings::POVOLIT_REZIM_UPRAV)) {
            $this ->zkratka2 = $row[19];
            $this ->barva2 = $row[20];
        }

        if (isset($row[22])) {
            $this ->popisRozdeleni = $row[22];
            $this ->barvaRozdeleni = $row[23];
        }

        if (isset($row[24])) {
            $this ->popisRozdeleni2 = $row[24];
            $this ->barvaRozdeleni2 = $row[25];
        }
    }


    /**
     * @param Database $database
     * @param int|int[] $idPP
     * @param int $datumOd
     * @param int $datumDo
     * @param bool $rezimUprav
     * @param bool $nacistRozdeleni     Priznak urcuje, jestli se maji nacist popisy a barvy rozdeleni
     * @return PlanovaniKapacit_PlanovanaSmena[]
     * @throws DBException
     */
    public static function nactiSmeny($database, $idPP, $datumOd, $datumDo, $rezimUprav, $nacistRozdeleni = false)
    {
        $result = [];

        $params = $database ->createParameters();

        $sql = self::getSQL($database, $params, $idPP, $datumOd, $datumDo, $rezimUprav, $nacistRozdeleni);
        $cursor = $database ->getSqlResult($sql, $params);

        while ($row = $cursor ->fetch()) {
            $smena = new self();
            $smena ->nactiData($database, $row);
            $result[] = $smena;
        }

        $cursor ->close();

        return $result;
    }


    /**
     * @param Database $database
     * @param int    $datum
     * @param string $rc
     * @param string $osobniCislo
     * @param int    $datumNastupu
     * @param bool   $rezimUprav
     * @return PlanovaniKapacit_PlanovanaSmena
     * @throws Exception
     */
    public static function nactiDataProDetail($database, $datum, $rc, $osobniCislo, $datumNastupu, $rezimUprav)
    {
        $smena = new PlanovaniKapacit_PlanovanaSmena();

        // nacita vse vcetne podrobnosti - pro zobrazeni detailu
        $params = $database ->createParameters();
        $sql =
            'SELECT PlanovaniKapacit.Datum, PlanovaniKapacit.Smena, PlanovaniKapacit.Potvrzeno, ' .
            '       PlanovaniKapacit.KodTypuRozdeleni, PlanovaniKapacit.Rozdeleni, ' .
            '       PlanovaniKapacit.Smena_Pocatek, PlanovaniKapacit.Smena_Konec, PlanovaniKapacit.Smena_Delka, Smena.Variabilni, ' . // 5..8
            '       PlanovaniKapacit.Update_UserName, PlanovaniKapacit.Update_TimeStamp, PlanovaniKapacit.Harmonogram, ' . // 9..11
            '       Smena.PocatekSmeny, Smena.KonecSmeny, Smena.DelkaSmeny, ' . // 12..14
            '       PlanovaniKapacit.PuvodniPlan_Smena, PlanovaniKapacit.PuvodniPlan_Potvrzeno, ' . // 15,16
            '       PlanovaniKapacit.PuvodniPlan_KodTypu, PlanovaniKapacit.PuvodniPlan_Rozdeleni, ' . // 17,18
            '       PlanovaniKapacit.PuvodniPlan_Smena_Pocatek, PlanovaniKapacit.PuvodniPlan_Smena_Konec, PlanovaniKapacit.PuvodniPlan_Smena_Delka, ' . // 19..21
            '       PlanovaniKapacit.KodTypuRozdeleni2, PlanovaniKapacit.Rozdeleni2, ' . // 22,23
            '       PlanovaniKapacit.PuvodniPlan_KodTypu2, PlanovaniKapacit.PuvodniPlan_Rozdeleni2, ' . // 24,25
            '       Smena.Delena, PlanovaniKapacit.Smena_PreruseniOd, PlanovaniKapacit.Smena_PreruseniDo,' .
            '       PlanovaniKapacit.PuvodniPlan_PreruseniOd, PlanovaniKapacit.PuvodniPlan_PreruseniDo,' . // 26..30
            '       PuvodniSmena.Variabilni' . // 31
            '  FROM PlanovaniKapacit LEFT OUTER JOIN Smena ' . // smena nemusi byt vyplnena!
            '    ON PlanovaniKapacit.Smena = Smena.Kod ' .
            '  LEFT OUTER JOIN Smena PuvodniSmena' . // smena nemusi byt vyplnena!
            '    ON PlanovaniKapacit.PuvodniPlan_Smena = PuvodniSmena.Kod ' .
            ' WHERE  PlanovaniKapacit.RC = ' . $params ->addParameter($rc) .
            '   AND PlanovaniKapacit.OsobniCislo = ' . $params ->addParameter($osobniCislo) .
            '   AND PlanovaniKapacit.DatumNastupu = ' . $params ->addParameter($database ->dateToParam($datumNastupu)) .
            '   AND PlanovaniKapacit.Datum = ' . $params ->addParameter($database ->dateToParam($datum)) .
            ' ORDER BY Datum';

        $cursor = $database ->getSqlResult($sql, $params, 'Chyba pri nacitani planovane smeny!');

        $smena ->zkratka = null; // nenacita se
        $smena ->barva = null;

        $smena ->datum = $datum;

        if ($row = $cursor ->fetch()) {
            if (RonPortalSettings::getIntValue(KapacitySettings::POVOLIT_REZIM_UPRAV)  &&  !$rezimUprav) {
                // puvodni plan
                $smena ->kod = $row[15];
                $smena ->kod2 = $row[1];

                $smena ->potvrzeno = (int)$row[16];
                $smena ->kodTypuRozdeleni = $row[17];
                $smena ->rozdeleni = $row[18];

                $smena ->kodTypuRozdeleni2 = $row[24];
                $smena ->rozdeleni2 = $row[25];
                $smena ->variabilni = $row[31];
            } else {
                // skutecnost (tj. rezim uprav nebo se rezim uprav vubec nepouziva)
                $smena ->kod = $row[1];

                $smena ->kod2 = RonPortalSettings::getIntValue(KapacitySettings::POVOLIT_REZIM_UPRAV)
                    ? $row[15]
                    : $smena ->kod;

                $smena ->potvrzeno = (int)$row[2];
                $smena ->kodTypuRozdeleni = $row[3];
                $smena ->rozdeleni = $row[4];

                $smena ->kodTypuRozdeleni2 = $row[22];
                $smena ->rozdeleni2 = $row[23];
                $smena ->variabilni = $row[8];
            }

            if ($smena ->variabilni) {
                if (RonPortalSettings::getIntValue(KapacitySettings::POVOLIT_REZIM_UPRAV)  &&  !$rezimUprav) {
                    // puvodni plan
                    $smena->pocatek = $row[19] ? $database->stringToSecondsOfDay($row[19]) : null;
                    $smena->konec = $row[20] ? $database->stringToSecondsOfDay($row[20]) : null;
                    $smena->delka = $row[21] ? $database->stringToSecondsOfDay($row[21]) : null;
                } else {
                    // skutecnost
                    $smena->pocatek = $row[5] ? $database->stringToSecondsOfDay($row[5]) : null;
                    $smena->konec = $row[6] ? $database->stringToSecondsOfDay($row[6]) : null;
                    $smena->delka = $row[7] ? $database->stringToSecondsOfDay($row[7]) : null;
                }
            } else {
                // podle nastaveni smeny
                $smena ->pocatek = $row[12] ? $database ->stringToSecondsOfDay($row[12]) : null;
                $smena ->konec = $row[13] ? $database ->stringToSecondsOfDay($row[13]) : null;
                $smena ->delka = $row[14] ? $database ->stringToSecondsOfDay($row[14]) : null;
            }

            $smena ->delena = $row[26];
            if ($smena ->delena) {
                if (RonPortalSettings::getIntValue(KapacitySettings::POVOLIT_REZIM_UPRAV)  &&  !$rezimUprav) {
                    // puvodni plan
                    $smena->preruseniOd = $row[29] ? $database->stringToSecondsOfDay($row[29]) : null;
                    $smena->preruseniDo = $row[30] ? $database->stringToSecondsOfDay($row[30]) : null;
                } else {
                    // skutecnost
                    $smena->preruseniOd = $row[27] ? $database->stringToSecondsOfDay($row[27]) : null;
                    $smena->preruseniDo = $row[28] ? $database->stringToSecondsOfDay($row[28]) : null;
                }
            }

            $smena ->update_UserName = $row[9];
            $smena ->update_TimeStamp = $row[10] ? $database ->stringToTimestamp($row[10]) : '';
            $smena ->harmonogram = $row[11];
        } else {
            // zaznam pro tento den neexistuje
            $smena ->kod = null;
            $smena ->potvrzeno = false;
            $smena ->kodTypuRozdeleni = null;
            $smena ->rozdeleni = null;
            $smena ->kodTypuRozdeleni2 = null;
            $smena ->rozdeleni2 = null;

            $smena ->variabilni = false;
            $smena ->pocatek = null;
            $smena ->konec = null;
            $smena ->delka = null;

            $smena ->delena = false;
            $smena ->preruseniOd = null;
            $smena ->preruseniDo = null;

            $smena ->update_UserName = null;
            $smena ->update_TimeStamp = null;
            $smena ->harmonogram = null;
        }

        return $smena;
    }



    public function vratPodrobnostiText()
    {
        if (!RonPortalSettings::getValue(self::SETTINGS_TEXT)) {
            return '';
        }

        $result = RonPortalSettings::getMultilineValue(self::SETTINGS_TEXT);

        if ($this ->variabilni) {
            // smena je variabilni - odstrani tagy VARIABILNI (obsah ponecha)
            $result = str_replace(['<VARIABILNI>', '</VARIABILNI>'], '', $result);
            // a odstrani cele sekce <PEVNA>...</PEVNA> vcetne obsahu
            $result = preg_replace("/<PEVNA>.*?<\/PEVNA>/i", "", $result);
        } else {
            if ($this ->kod) {
                // smena neni variabilni
                $result = str_replace(['<PEVNA>', '</PEVNA>'], '', $result);
            }
            else {
                // neni zadna smena (muze byt napr. prerazeni na jine rozdeleni)
                $result = preg_replace("/<PEVNA>.*?<\/PEVNA>/i", "", $result);
            }

            // odstrani cele sekce <VARIABILNI>...</VARIABILNI>
            $result = preg_replace("/<VARIABILNI>.*?<\/VARIABILNI>/i", "", $result);
        }

        if ($this ->delena) {
            // smena je delena - odstrani tagy DELENA (obsah ponecha)
            $result = str_replace(['<DELENA>', '</DELENA>'], '', $result);
        } else {
            // smena neni delena - odstrani cele sekce <DELENA>...</DELENA>
            $result = preg_replace("/<DELENA>.*?<\/DELENA>/i", "", $result);
        }

        if ($this ->harmonogram) {
            $result = str_replace(['<HARMONOGRAM>', '</HARMONOGRAM>'], '', $result);
        }
        else {
            $result = preg_replace("/<HARMONOGRAM>.+?<\/HARMONOGRAM>/i", "", $result);
        }

        return str_replace(
            ['%HARMONOGRAM%', '%SMENA_POCATEK%', '%SMENA_KONEC%', '%SMENA_DELKA%', '%SMENA%', '%SMENA_PRERUSENI_OD%', '%SMENA_PRERUSENI_DO%'],
            [
                $this->harmonogram,
                isset($this->pocatek) ? StringUtils::format('minuty', $this->pocatek / 60) : '',
                isset($this ->konec) ? StringUtils::format('minuty', $this ->konec / 60) : '',
                $this ->delka ? StringUtils::format('minuty', $this ->delka / 60) : '',
                $this ->kod,
                $this ->preruseniOd ? StringUtils::format('minuty', $this ->preruseniOd / 60) : '',
                $this ->preruseniDo ? StringUtils::format('minuty', $this ->preruseniDo / 60) : ''
            ],
            $result);
    }


    /**
     * @param bool $detail
     * @return array
     */
    public function getJSON($detail = false)
    {
        $json = [];

        $json['datum'] = date('j.n.Y', $this ->datum);
        if ($this ->potvrzeno) $json['potvrzeno'] = $this ->potvrzeno;
        $json['kod'] = StringUtils::jsonText($this ->kod, false);

        if ($this ->kod !== $this ->kod2  &&  RonPortalSettings::getIntValue(KapacitySettings::POVOLIT_REZIM_UPRAV)) {
            $json['kod2'] = StringUtils::jsonText($this ->kod2, false);
        }

        if ($this ->variabilni) {
            $json['variabilni'] = 1;
            $json['delka'] = StringUtils::format('minuty', $this ->delka / 60);
            $json['pocatek'] = StringUtils::format('minuty', $this ->pocatek / 60);
            $json['konec'] = StringUtils::format('minuty', $this ->konec / 60);
        }

        if ($this ->delena) {
            $json['delena'] = 1;
            $json['preruseniOd'] = StringUtils::format('minuty', $this ->preruseniOd / 60);
            $json['preruseniDo'] = StringUtils::format('minuty', $this ->preruseniDo / 60);
        }

        if ($this ->rozdeleni) {
            $json['rozdeleni'] = StringUtils::jsonText($this ->rozdeleni);
        }

        if ( isset($this ->rozdeleni2) ) {
            $json['rozdeleni2'] = StringUtils::jsonText($this ->rozdeleni2);
        }

        $pomtext = $this ->vratPodrobnostiText();
        if ($pomtext) {
            $json['podrobnosti'] = StringUtils::jsonText($pomtext, false);
        }

        if ($detail) {
            if ($this ->rozdeleni) {
                $json['kodTypuRozdeleni'] = $this ->kodTypuRozdeleni;
            }

            if ($this ->popisRozdeleni) {
                $json['popisRozdeleni'] = StringUtils::jsonText($this ->popisRozdeleni);
            }

            if (isset($this ->barvaRozdeleni)) {
                $json['barvaRozdeleni'] = GuiUtils::getHtmlColor($this ->barvaRozdeleni);
            }

            if ($this ->rozdeleni2) {
                $json['kodTypuRozdeleni2'] = $this ->kodTypuRozdeleni2;
            }

            if ($this ->popisRozdeleni2) {
                $json['popisRozdeleni2'] = StringUtils::jsonText($this ->popisRozdeleni2);
            }

            if (isset($this ->barvaRozdeleni)) {
                $json['barvaRozdeleni2'] = GuiUtils::getHtmlColor($this ->barvaRozdeleni2);
            }

            if ($this ->harmonogram) {
                $json['harmonogram'] = StringUtils::jsonText($this ->harmonogram, false);
            }

            if ($this ->update_UserName) {
                $json['update_username'] = StringUtils::jsonText($this->update_UserName, false);
            }

            if ($this ->update_TimeStamp) {
                $json['update_timestamp'] = date('j.n.Y', $this->update_TimeStamp);
            }
        }

        return $json;
    }
}
