<?php
/**
 * stanislav.elbl, 10.03.2021
 */

namespace RonPortal\Kapacity;

use Ron\DateUtils;
use Ron\StringUtils;

class PodminkaPlanovaniKapacit
{
    const TYP_SMENA = 0;
    const TYP_CAS_OD_DO = 1;
    const TYP_OPERACE = 2;
    const TYP_PRACOVNI_DOBA = 3;
    const TYP_ZACATEK = 4;
    const TYP_KONEC = 5;

    public $id;

    /** @var int    0=smena, 1=cas od do, 2=operacedochazky, 3=pracovni doba, 4=zacatek pracovni doby, 5=konec pracovni doby */
    public $typPodminky;

    /** @var string */
    public $typKontroly;

    /** @var int    Priznak, ze se jedna o slot pro planovani kapacit */
    public $slotProPlanovani;

    /** @var int    Priznak, ze se muze editovat - 0=ne, 1=ano, 2=pouze zobrazit */
    public $lzeEditovat;

    /** @var string Kod smeny */
    public $smena;

    /** @var int    Pocet sekund */
    public $casOd;
    /** @var int    Pocet sekund */
    public $casDo;

    /** @var int */
    public $pocetMin;
    /** @var int */
    public $pocetMax;

    public $kodTypuRozdeleni;
    public $rozdeleni;

    public $kodDovednosti;
    public $kodUrovne;

    /** @var string */
    public $updateUserName;
    /** @var int */
    public $updateTimestamp;

    /**
     * @return array
     */
    public function getJson()
    {
        $result = [
            'typPodminky' => $this ->typPodminky,
            'smena' => StringUtils::jsonText($this ->smena),
            'casOdStr' => $this ->casOd ? DateUtils::minutesToString($this ->casOd / 60) : null,
            'casDoStr' => $this ->casDo ? DateUtils::minutesToString($this ->casDo / 60) : null,
            'pocetMin' => $this ->pocetMin,
            'pocetMax' => $this ->pocetMax
        ];

        if ($this ->kodTypuRozdeleni  &&  $this ->rozdeleni) {
            // pouzije se pro vypocet souctu ve slotech
            $result['kodTypuRozdeleni'] = $this ->kodTypuRozdeleni;
            $result['rozdeleni'] = StringUtils::jsonText($this ->rozdeleni);
            // pouzije se pro zobrazeni
            $result['specifikace'] = StringUtils::jsonText($this ->rozdeleni);
        } else if ($this ->kodDovednosti  &&  $this ->kodUrovne) {
            $result['specifikace'] = StringUtils::jsonText($this ->kodDovednosti) . '/' . $this ->kodUrovne;
        }

        return $result;
    }


    public function getPopis()
    {
        // 0=smena, 1=cas od do, 2=operacedochazky, 3=pracovni doba, 4=zacatek pracovni doby, 5=konec pracovni doby
        switch ($this ->typPodminky) {
            case 0 :
                return $this ->smena;

            case 1 :
                return
                    StringUtils::format("timeofday", $this ->casOd / 3600) .
                    ' - ' .
                    StringUtils::format("timeofday", $this ->casDo / 3600);

            case 2 :
                return '(operace)';

            case 3 :
                return \Ron\ronText('Pracovni_doba');

            case 4 :
                return \Ron\ronText('Zacatek_pracovni_doby');

            case 5 :
                return \Ron\ronText('Konec_pracovni_doby');

            default :
                return 'Neznamy typ podminky!';
        }
    }


    /**
     * @param PodminkaPlanovaniKapacit $podminka
     * @return bool
     */
    public function isSameAs($podminka)
    {
        if ($this == $podminka) {
            // primo stejny objekt podminky ... (nebo porovnat id?)
            return true;
        }

        return
            $this ->casOd == $podminka ->casOd  &&
            $this ->casDo == $podminka ->casDo  &&
            $this ->smena == $podminka ->smena  &&
            $this ->kodTypuRozdeleni == $podminka ->kodTypuRozdeleni &&
            $this ->rozdeleni == $podminka ->rozdeleni &&
            $this ->typPodminky == $podminka ->typPodminky &&
            $this ->kodDovednosti == $podminka ->kodDovednosti &&
            $this ->kodUrovne == $podminka ->kodUrovne;
    }


    public static function compare($podminka1, $podminka2)
    {
        if (!$podminka1) {
            return $podminka2 ? -1 : 0;
        }
        if (!$podminka2) {
            return 1;
        }

        // oba radky maji prirazenou podminku
        if ($podminka2 ->typPodminky < $podminka1 ->typPodminky) {
            return 1; // seradit podle typu podminky (0=smeny, 1=cas od-do, ...)
        }
        if ($podminka2 ->typPodminky > $podminka1 ->typPodminky) {
            return -1;
        }

        // stejny typ podminky
        if (!$podminka1 ->typPodminky) {
            // smena
            if ($podminka1 ->smena > $podminka2 ->smena) {
                return 1;
            }
            if ($podminka1 ->smena < $podminka2 ->smena) {
                return -1;
            }
            return 0;
        }

        if ($podminka1 ->typPodminky == 1) {
            // cas od - do
            if ($podminka1 ->casOd > $podminka2 ->casOd) {
                return 1;
            }
            if ($podminka1 ->casOd < $podminka2 ->casOd) {
                return -1;
            }
            // stejny cas od - jeste zkontroluju cas do
            if ($podminka1 ->casDo > $podminka2 ->casDo) {
                return 1;
            }
            if ($podminka1 ->casDo < $podminka2 ->casDo) {
                return -1;
            }
            return 0;
        }

        return 0;

    }
}
