<?php

namespace RonPortal\Kapacity;

use Exception;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\StringUtils;
use RonPortal\PracovniPomer;
use RonPortal\PravaNaOsoby;

class SchvaleniKapacitDBRecord extends SchvaleniKapacitAbstractRecord
{
    /** @var PracovniPomer */
    private $pp;

    /**
     * @param Database $database
     * @param PracovniPomer $pp
     * @param int $obdobi
     * @param string|object $logDescription
     */
    public function __construct($database, $pp, $obdobi, $logDescription = 'Planovani kapacit')
    {
        parent::__construct($database, 'SchvaleniKapacit', $obdobi, $logDescription);

        $this ->pp = $pp;

        $this ->addField('RC', '', true, 15, $pp ->rc); // readonly = true
        $this ->addField('OsobniCislo', '', true, 15, $pp ->osobniCislo); // readonly = true
        $this ->addField('DatumNastupu', 'date', true, 0, $pp ->datumNastupu);

        $this ->fSchvaleni1 = $this ->addField('ZpracovanoDo', 'date', true);
        $this ->fSchvalil1 = $this ->addField('Zpracoval', '', true, 25);
        $this ->fSchvalil1 ->updateLog = true;
        $this ->fCas_Schvaleni1 = $this ->addField('Cas_Zpracovano', 'timestamp', true);
        $this ->fCas_Schvaleni1 ->updateLog = true;

        $this ->fSchvaleni2 = $this ->addField('ZkontrolovanoDo', 'date', true);
        $this ->fSchvalil2 = $this ->addField('Zkontroloval', '', true, 25);
        $this ->fSchvalil2 ->updateLog = true;
        $this ->fCas_Schvaleni2 = $this ->addField('Cas_Zkontrolovano', 'timestamp', true);
        $this ->fCas_Schvaleni2 ->updateLog = true;

        $this ->fSchvaleni3 = $this ->addField('SchvalenoDo', 'date', true);
        $this ->fSchvalil3 = $this ->addField('Schvalil', '', true, 25);
        $this ->fSchvalil3 ->updateLog = true;
        $this ->fCas_Schvaleni3 = $this ->addField('Cas_Schvaleno', 'timestamp', true);
        $this ->fCas_Schvaleni3 ->updateLog = true;

        $this ->fSchvaleniUprav1 = $this ->addField('UpravyZpracovanyDo', 'date', true);
        $this ->UpravySchvalil1 = $this ->addField('UpravyZpracoval', '', true, 25);
        $this ->UpravySchvalil1 ->updateLog = true;
        $this ->Cas_SchvaleniUprav1 = $this ->addField('Cas_UpravyZpracovany', 'timestamp', true);
        $this ->Cas_SchvaleniUprav1 ->updateLog = true;

        $this ->fSchvaleniUprav2 = $this ->addField('UpravyZkontrolovanyDo', 'date', true);
        $this ->UpravySchvalil2 = $this ->addField('UpravyZkontroloval', '', true, 25);
        $this ->UpravySchvalil2 ->updateLog = true;
        $this ->Cas_SchvaleniUprav2 = $this ->addField('Cas_UpravyZkontrolovany', 'timestamp', true);
        $this ->Cas_SchvaleniUprav2 ->updateLog = true;

        $this ->fSchvaleniUprav3 = $this ->addField('UpravySchvalenyDo', 'date', true);
        $this ->UpravySchvalil3 = $this ->addField('UpravySchvalil', '', true, 25);
        $this ->UpravySchvalil3 ->updateLog = true;
        $this ->Cas_SchvaleniUprav3 = $this ->addField('Cas_UpravySchvaleny', 'timestamp', true);
        $this ->Cas_SchvaleniUprav3 ->updateLog = true;

        $params = $database ->createParameters();
        $this ->setWhere(
            ' RC = ' . $params ->addParameter($pp ->rc) .
            ' AND OsobniCislo = ' . $params->addParameter($pp ->osobniCislo) .
            ' AND DatumNastupu = ' . $params ->addParameter($database ->dateToParam($pp ->datumNastupu)) .
            " AND Aplikace = 'ADS'" .
            ' AND PocatekObdobi = ' . $params ->addParameter($database ->dateToParam($obdobi)),
            $params);
    }


    /**
     * @param PravaNaOsoby $pravo
     * @return bool
     * @throws Exception
     */
    public function zkontrolujPravo($pravo)
    {
        return $pravo ->lzeZobrazitPP($this ->pp ->idPP);
    }


    /**
     * @param int $uroven
     * @param int $datumDo
     * @throws Exception
     * @throws DBException
     */
    public function overKontroly($uroven, $datumDo)
    {
        // overi kontroly osoby
        $params = $this ->database ->createParameters();
        $sql =
            'SELECT VysledekKontrolyPlanovani.Datum, TypKontrolyPlanovani.Popis' .
            ' FROM VysledekKontrolyPlanovani, TypKontrolyPlanovani' .
            ' WHERE VysledekKontrolyPlanovani.TypKontroly = TypKontrolyPlanovani.Kod' .
            ' AND VysledekKontrolyPlanovani.Ignorovat = 0' .
            ' AND TypKontrolyPlanovani.BlokovatSchvaleni' . (int)$uroven . ' = 1' .
            ' AND VysledekKontrolyPlanovani.Datum >= ' . $params ->addParameter($this ->database ->dateToParam($this ->obdobi)) .
            ' AND VysledekKontrolyPlanovani.Datum <= ' . $params ->addParameter($this ->database ->dateToParam($datumDo)) .

            ' AND VysledekKontrolyPlanovani.RC = ' . $params ->addParameter($this ->pp ->rc) .
            ' AND VysledekKontrolyPlanovani.OsobniCislo = ' . $params ->addParameter($this ->pp ->osobniCislo) .
            ' AND VysledekKontrolyPlanovani.DatumNastupu = ' . $params ->addParameter($this ->database ->dateToParam($this ->pp ->datumNastupu)) .

            ' ORDER BY VysledekKontrolyPlanovani.Datum';

        $cursor = $this ->database ->getSqlResult($sql, $params);
        $row = $cursor ->fetch();
        $cursor ->close();

        if ($row) {
            throw new Exception( \Ron\ronText('Nelze_schvalit_plan') . "\n" . StringUtils::format('date', $this ->database ->stringToTimestamp($row[0])) . ': ' . $row[1]);
        }

        // TODO: zkontrolovat i rozdeleni, na ktere je prerazen!

        // overi kontroly rozdeleni
        $params = $this ->database ->createParameters();
        $sql =
            'SELECT VysledekKontrolyPlanovani.Datum, TypKontrolyPlanovani.Popis' .
            ' FROM VysledekKontrolyPlanovani, TypKontrolyPlanovani, RozdeleniOsob' .
            ' WHERE VysledekKontrolyPlanovani.TypKontroly = TypKontrolyPlanovani.Kod' .
            ' AND VysledekKontrolyPlanovani.Ignorovat = 0' .
            ' AND VysledekKontrolyPlanovani.Datum >= ' . $params ->addParameter($this ->database ->dateToParam($this ->obdobi)) .
            ' AND VysledekKontrolyPlanovani.Datum <= ' . $params ->addParameter($this ->database ->dateToParam($datumDo)) .
            ' AND TypKontrolyPlanovani.BlokovatSchvaleni' . (int)$uroven . ' = 1' .

            ' AND VysledekKontrolyPlanovani.KodTypuRozdeleni = RozdeleniOsob.KodTypuRozdeleni' .
            ' AND VysledekKontrolyPlanovani.Rozdeleni = RozdeleniOsob.Rozdeleni' .

            ' AND RozdeleniOsob.RC = ' . $params ->addParameter($this ->pp ->rc) .
            ' AND RozdeleniOsob.OsobniCislo = ' . $params ->addParameter($this ->pp ->osobniCislo) .
            ' AND RozdeleniOsob.DatumNastupu = ' . $params ->addParameter($this ->database ->dateToParam($this ->pp ->datumNastupu)) .

            ' AND RozdeleniOsob.PlatnostOd <= VysledekKontrolyPlanovani.Datum' .
            ' AND (RozdeleniOsob.PlatnostDo >= VysledekKontrolyPlanovani.Datum  OR  RozdeleniOsob.PlatnostDo IS NULL)' .

            ' ORDER BY VysledekKontrolyPlanovani.Datum';

        $cursor = $this ->database ->getSqlResult($sql, $params);
        $row = $cursor ->fetch();
        $cursor ->close();

        if ($row) {
            throw new Exception( \Ron\ronText('Nelze_schvalit_plan') . "\n" . StringUtils::format('date', $this ->database ->stringToTimestamp($row[0])) . ': ' . $row[1]);
        }
    }


    /**
     * @param int $uroven
     * @param bool $rezimUprav
     * @param int $datumDo
     * @return string
     */
    public function getParametrSkriptu($uroven, $rezimUprav, $datumDo)
    {
        return
            "RC={$this ->pp ->rc};OsobniCislo={$this ->pp ->osobniCislo};DatumNastupu=" . date('j.n.Y', $this ->pp ->datumNastupu) . ';' .
            parent::getParametrSkriptu($uroven, $rezimUprav, $datumDo) .
            ";IdPP={$this ->pp ->idPP}";
    }


    public function getAgentParameters($uroven, $rezimUprav, $hodnota)
    {
        return '1 -IDPP:' . $this ->pp ->idPP  . parent::getAgentParameters($uroven, $rezimUprav, $hodnota);
    }


    /**
     * @param Database $database
     * @param PracovniPomer $pp
     * @param int $uroven
     * @param int $obdobi
     * @param bool $rezimUprav
     * @param int $hodnota  Datum, do kdy se ma schvalit
     * @throws Exception
     */
    public static function schvalKapacity($database, $pp, $uroven, $obdobi, $rezimUprav, $hodnota)
    {
        $rec = new static($database, $pp, $obdobi);
        $rec->zapisSchvaleni($uroven, $rezimUprav, $hodnota);
    }
}
