<?php
/**
 * stanislav.elbl, 10.03.2021
 */

namespace RonPortal\Kapacity;

use Ron\DB\Database;
use RonPortal\NeniPravoException;
use RonPortal\Obdobi;
use RonPortal\Prava;
use RonPortal\PravaNaOsoby;
use RonPortal\RonPortalDBRecord;
use RonPortal\RonPortalUtils;
use RonPortal\RonPortalXMLEvents;
use RonPortal\Rozdeleni;
use RonPortal\Schvaleni;
use RonPortal\XML\RonPortalXMLForm;

class SchvaleniPodminekPlanovani extends Schvaleni
{
    var $kodTypuRozdeleni;
    var $rozdeleni;
    var $pocatekObdobi;


    /**
     * @param Database $database
     * @param int    $pocatekObdobi
     * @param int    $typRozdeleni
     * @param string $kodRozdeleni
     * @return SchvaleniPodminekPlanovani
     * @throws \Ron\DB\DBException
     */
    public static function nactiSchvaleni($database, $pocatekObdobi, $typRozdeleni, $kodRozdeleni)
    {
        $params = $database ->createParameters();
        $where =
            'Aplikace = ' . $params ->addParameter('ADS') .
            ' AND PocatekObdobi = ' . $params ->addParameter($database ->dateToParam($pocatekObdobi)) .
            ' AND KodTypuRozdeleni = ' . $params ->addParameter($typRozdeleni) .
            ' AND Rozdeleni = ' . $params ->addParameter($kodRozdeleni);

        /**
         * @var SchvaleniPodminekPlanovani $result
         */
        $result = static::nactiZaznamSchvaleni($database, 'SchvaleniPodminekRozd', $where, $params);
        $result ->kodTypuRozdeleni = $typRozdeleni;
        $result ->rozdeleni = $kodRozdeleni;
        $result ->pocatekObdobi = $pocatekObdobi;
        return $result;
    }


    /**
     * @param int $uroven
     * @return PravaNaOsoby
     */
    public function vratPravoProUroven($uroven)
    {
        return Prava::vratPrava('kapac_schval_poc' . $uroven);
    }


    public function lzeSchvalovatDen($uroven, $datum)
    {
        $pravo = $this ->vratPravoProUroven($uroven);
        return $pravo ->jePravoNaRozdeleni($this ->kodTypuRozdeleni, $this ->rozdeleni, $datum, $datum);
    }


    /**
     * Vraci true, pokud uzivatel muze planovat zadany den - tj. jsou schvalene podminky planovani nastavenou urovni nebo vyssi
     * @param $datum
     * @return bool
     */
    public function lzePlanovatDen($datum)
    {
        $pozadovanaUroven = Prava::vratPrava('kapacity_edit') ->priznak2;

        switch ($pozadovanaUroven) {  // RonPortalSettings::getIntValue('kapacity_schvalenislotu')
            case 0 :
                return true;

            case 1 :
                if ($this->schvaleni1 >= $datum) return true;
            // no break, pokracuje dale!
            case 2 :
                if ($this->schvaleni2 >= $datum) return true;
            // no break, pokracuje dale!
            case 3 :
                if ($this->schvaleni3 >= $datum) return true;
        }

        return false;
    }


    /**
     * @param Database $database
     * @param string $eventName
     * @param string $context
     * @param Rozdeleni $rozdeleni
     * @param Obdobi $obdobi
     * @param RonPortalDBRecord $record
     */
    public static function executeEvents($database, $eventName, $context, $rozdeleni, $obdobi, $record = null)
    {
        RonPortalXMLForm::xmlEvents($database, $eventName,
            [
                'kodTypuRozdeleni' => $rozdeleni ->typ,
                'rozdeleni' => $rozdeleni ->kod,
                'obdobi' => $obdobi ->zacatek,
                'record' => $record
            ],
            $context);
    }


    /**
     * @param Database $database
     * @param Rozdeleni $rozdeleni
     * @param int|Obdobi $obdobi
     * @param int $uroven
     * @param int|null $datumDo
     * @throws \Exception
     */
    public static function nastavSchvaleniPodminek($database, $rozdeleni, $obdobi, $uroven, $datumDo)
    {
        if (!$obdobi instanceof Obdobi) {
            $obdobi = Obdobi::vratObdobi($database, 'ADS', $obdobi);
        }

        $pravo = Prava::vratPrava('kapac_schval_poc' . $uroven);

        if (!$pravo->jePravoNaRozdeleni($rozdeleni->typ, $rozdeleni->kod, $obdobi->konec, $obdobi->konec)) {
            throw new NeniPravoException();
        }

        // provede schvaleni
        $rec = new RonPortalDBRecord($database, 'SchvaleniPodminekRozd');

        $rec->addField('PocatekObdobi', 'date', true, 0, $obdobi->zacatek);
        $rec->addField('KodTypuRozdeleni', 'int', true, 0, $rozdeleni->typ);
        $rec->addField('Rozdeleni', '', true, 30, $rozdeleni->kod);
        $rec->addField('', '', true, 0, '   ' . $rozdeleni->popis);
        // uroven 1
        $rec->addField('Schvaleni1', 'date', $uroven != 1, 0, ($uroven == 1) ? $datumDo : null);
        $rec->addField('Schvalil1', '', $uroven != 1, 25, $uroven == 1 ? RonPortalUtils::getLogUserName() : null);
        $rec->addField('Cas_Schvaleni1', 'timestamp', $uroven != 1, 0, $uroven == 1 ? time() : null);
        // uroven 2
        $rec->addField('Schvaleni2', 'date', $uroven != 2, 0, ($uroven == 2) ? $datumDo : null);
        $rec->addField('Schvalil2', '', $uroven != 2, 25, $uroven == 2 ? RonPortalUtils::getLogUserName() : null);
        $rec->addField('Cas_Schvaleni2', 'timestamp', $uroven != 2, 0, $uroven == 2 ? time() : null);
        // uroven 3
        $rec->addField('Schvaleni3', 'date', $uroven != 3, 0, ($uroven == 3) ? $datumDo : null);
        $rec->addField('Schvalil3', '', $uroven != 3, 25, $uroven == 3 ? RonPortalUtils::getLogUserName() : null);
        $rec->addField('Cas_Schvaleni3', 'timestamp', $uroven != 3, 0, $uroven == 3 ? time() : null);

        $rec->addField('Aplikace', '', true, 15, 'ADS');

        $params = $database->createParameters();
        $rec->setWhere(
            'Aplikace = ' . $params->addParameter('ADS') .
            ' AND PocatekObdobi = ' . $params->addParameter($database->dateToParam($obdobi->zacatek)) .
            ' AND KodTypuRozdeleni = ' . $params->addParameter($rozdeleni->typ) .
            ' AND Rozdeleni = ' . $params->addParameter($rozdeleni->kod),
            $params);

        self::executeEvents($database, RonPortalXMLEvents::KAPACITY_PODMINKY_SCHVALENI_BEFORE, 'schvaleni' . $uroven, $rozdeleni, $obdobi, $rec);

        $rec->updateOrInsert();

        self::executeEvents($database, RonPortalXMLEvents::KAPACITY_PODMINKY_SCHVALENI_AFTER, 'schvaleni' . $uroven, $rozdeleni, $obdobi, $rec);
    }
}
