<?php
/**
 * stanislav.elbl, 24.06.2019
 * Zobrazeni a editace povolenych smen kalendare pro jednotlive dny
 * Podle
 */

namespace RonPortal\Kapacity;

use Exception;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\SQLUtils;
use Ron\Http\HtmlHeaders;
use Ron\MessageManager;
use Ron\MVC\JsonController;
use Ron\MVC\JsonOutputObject;
use Ron\MVC\Model;
use Ron\MVC\TextViewObject;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Controls\RonPortalFilterForm;
use RonPortal\Log;
use RonPortal\NeniPravoException;
use RonPortal\Obdobi;
use RonPortal\Prava;
use RonPortal\RonPortalDBRecord;
use RonPortal\RonPortalSettings;
use RonPortal\Rozdeleni;

class SmenyKalendareController extends JsonController
{
    /** @var Database */
    private $database;

    /**
     * SmenyKalendareController constructor.
     * @param Database $database
     */
    public function __construct(Database $database)
    {
        $this->database = $database;
    }


    /**
     * Zkopiruje zadanou podminku ($idPodminky) do zadane sady ($idSady). Priradi podmince zadanou smenu
     * @param int $idPodminky
     * @param int $idSady
     * @param string $smena Kod smeny, ktera se ma podmince priradit
     * @param int $casOd    Pokud neni zadan, zkopiruje se z puvodni podminky ze sablony
     * @param int $casDo    -//-
     * @return int Vraci id nove vytvorene podminky
     * @throws DBException
     * @throws Exception
     */
    private function zkopirujPodminku($idPodminky, $idSady, $smena, $casOd, $casDo)
    {
        $id = SQLUtils::nextSequenceValue($this ->database, 'SEQ_PodminkaPlanovaniKapacit');

        $params = $this ->database ->createParameters();
        $sql =
            "INSERT INTO PodminkaPlanovaniKapacit (\n" .
            ' Id, IdSady, PocetMin, PocetMax, TypKontroly, KodTypuRozdeleni, Rozdeleni,' .
            ' KodDovednosti, KodUrovne, Smena, CasOd, CasDo, PriraditOsobeSplneniPodm, TypPodminky,' .
            ' TextHlaseniMin, TextHlaseniMax, SlotProPlanovani, LzeEditovat' .
            ")\n" .
            ' SELECT ' . $params ->addParameter($id) . ', ' . $params ->addParameter($idSady) . ', ' .
            ' PocetMin, PocetMax, TypKontroly, KodTypuRozdeleni, Rozdeleni,' .
            ' KodDovednosti, KodUrovne, ' .
            $params ->addParameter($smena) . ', ' .
            (isset($casOd) ? $params ->addParameter($this ->database ->timeToParam($casOd)) : 'CasOd') . ', ' .
            (isset($casDo) ? $params ->addParameter($this ->database ->timeToParam($casDo)) : 'CasDo') . ',' .
            ' PriraditOsobeSplneniPodm, TypPodminky,' .
            ' TextHlaseniMin, TextHlaseniMax, SlotProPlanovani, LzeEditovat' .
            ' FROM PodminkaPlanovaniKapacit ' .
            ' WHERE Id = ' . $params ->addParameter($idPodminky);

        $this ->database ->sql($sql, $params);

        return $id;
    }


    /**
     * @param Rozdeleni $rozdeleni
     * @param int $datum
     * @throws DBException
     */
    private function generujPodminkyPodleSablon($rozdeleni, $datum)
    {
        // nacte sadu podminek rozdeleni platnou pro zadany den
        $idSady = SadaPodminekPlanovani::vratIdSadyPodminekProDen($this ->database, $rozdeleni, $datum, true, $this); // vytvoritNovou = true

        $logText =
            StringUtils::fromUTF8('Kopie podminek ze sablon pro rozdeleni ') . $rozdeleni ->kod . ' pro den ' . date('j.n.Y', $datum) . "\n" .
            StringUtils::fromUTF8('Id sady: ') . $idSady . "\n\n";

        // smaze existujici podminky
        $params = $this ->database ->createParameters();
        $sql = 'DELETE FROM PodminkaPlanovaniKapacit WHERE IdSady = ' . $params ->addParameter($idSady);
        $this ->database ->sql($sql, $params);

        $logText .=
            "Smazani puvodnich podminek:\n" .
            $sql . "\n\n";


        // nacte podminky ze sablon pro zadany den
        $params = $this ->database ->createParameters();
        $sql =
            'SELECT Rozdeleni.Kalendar, SmenaProDenKalendare.Smena,' .
            '  SmenaProDenKalendare.SadaPodminekPlanovani, SadaPodminekPlanovani.Zkratka, SadaPodminekPlanovani.Popis,' . // nepotrebne - jen pro info v logu
            '  PodminkaPlanovaniKapacit.Id, PodminkaPlanovaniKapacit.TypPodminky, PodminkaPlanovaniKapacit.Smena,' . // 5..7
            '  Smena.PocatekSmeny, Smena.KonecSmeny' . //8,9
            //'  PodminkaPlanovaniKapacit.CasOd, PodminkaPlanovaniKapacit.CasDo, ' .
            //'  PodminkaPlanovaniKapacit.TypKontroly, PodminkaPlanovaniKapacit.KodTypuRozdeleni, PodminkaPlanovaniKapacit.Rozdeleni, ' .
            //'  PodminkaPlanovaniKapacit.KodDovednosti, PodminkaPlanovaniKapacit.KodUrovne, '.
            //'  PodminkaPlanovaniKapacit.PocetMin, PodminkaPlanovaniKapacit.PocetMax, PodminkaPlanovaniKapacit.PriraditOsobeSplneniPodm, ' .
            //'  PodminkaPlanovaniKapacit.TextHlaseniMin, PodminkaPlanovaniKapacit.TextHlaseniMax, ' .
            //'  PodminkaPlanovaniKapacit.SlotProPlanovani, PodminkaPlanovaniKapacit.LzeEditovat' .
            "\n" .
            ' FROM Rozdeleni, SmenaProDenKalendare, SadaPodminekPlanovani, PodminkaPlanovaniKapacit, Smena' .
            "\n" .
            ' WHERE Rozdeleni.KodTypuRozdeleni = ' . $params ->addParameter($rozdeleni ->typ) .
            '  AND Rozdeleni.Kod = ' . $params ->addParameter($rozdeleni ->kod) .
            '  AND SmenaProDenKalendare.Kalendar = Rozdeleni.Kalendar' .
            '  AND SmenaProDenKalendare.Den = ' . $params ->addParameter($this ->database ->dateToParam($datum)) .
            '  AND SadaPodminekPlanovani.Id = SmenaProDenKalendare.SadaPodminekPlanovani' .
            '  AND PodminkaPlanovaniKapacit.IdSady = SmenaProDenKalendare.SadaPodminekPlanovani' .
            '  AND SmenaProDenKalendare.Smena = Smena.Kod' .
            "\n" .
            ' ORDER BY SmenaProDenKalendare.Smena, PodminkaPlanovaniKapacit.Id';

        $logText .=
            "Nacteni podminek ze sablon:\n" .
            $sql . "\n\n";

        $cursor = $this ->database ->getSqlResult($sql, $params);
        while ($row = $cursor ->fetch()) {
            // kod smeny se urci podle typu podminky
            $newId = $this ->zkopirujPodminku($row[5], $idSady,
                !$row[6] ? $row[1] : null,  // typ podmiky 0 = smena
                ($row[6] == 1) ? $this ->database ->stringToSecondsOfDay($row[8]) : null,   // typ podminky cas od-do
                ($row[6] == 1) ? $this ->database ->stringToSecondsOfDay($row[9]) : null
            );

            $logText .=
                'Smena ' . $row[1] .
                ', id nove podminky: ' . $newId .
                ', id puvodni podminky: ' . $row[5] .
                ', puvodni sada: ' . "$row[4] ({$row[2]})" .
                "\n";
        }
        $cursor ->close();

        Log::addRecordToDBLog($this ->database, 'PodminkaPlanovaniKapacit', 'INSERT', $this, $logText);
    }


    /**
     * Nacte podrobnosti zadane sablony prirazene rozdeleni. Pokud neni id zadano, nacita vychozi sablonu.
     * @param Rozdeleni $rozdeleni
     * @param int|null $id
     * @return int|null
     * @throws DBException
     */
    private function nactiSablonuProRozdeleni($rozdeleni, $id = null)
    {
        $params = $this ->database ->createParameters();
        $cursor = $this ->database ->getSqlResult(
            'SELECT PovolenaSablonaPodminek.IdSady, SadaPodminekPlanovani.Zkratka, SadaPodminekPlanovani.Popis' .
            ' FROM PovolenaSablonaPodminek, SadaPodminekPlanovani' .
            ' WHERE PovolenaSablonaPodminek.KodTypuRozdeleni = ' . $params ->addParameter($rozdeleni ->typ) .
            ' AND PovolenaSablonaPodminek.Rozdeleni = ' . $params ->addParameter($rozdeleni ->kod) .
            (isset($id)
                ? ' AND PovolenaSablonaPodminek.IdSady = ' . $params ->addParameter($id)
                : ' AND PovolenaSablonaPodminek.Vychozi = 1'
            ) .
            ' AND PovolenaSablonaPodminek.IdSady = SadaPodminekPlanovani.Id',
            $params
        );
        $row = $cursor ->fetch();
        $cursor ->close();

        return $row
            ? [ 'id' => $row[0], 'zkratka' => $row[1], 'popis' => $row[2] ]
            : null;
    }

    /**
     * @param $zapnuta
     * @param JsonOutputObject $output
     * @throws NeniPravoException
     * @throws Exception
     */
    private function nastavSmenu($zapnuta, $output)
    {
        // GET: rozdeleni, smena, kalendar, datumOd, datumDo, sablonaPodminek
        $datum = DateUtils::urlStringToDate($_GET['datumOd']);
        $datumDo = DateUtils::urlStringToDate($_GET['datumDo']);

        if (!$datum) {
            throw new Exception('Neni zadano datum!');
        }

        $kodSmeny = $_GET['smena'];
        if (!isset($kodSmeny)  ||  $kodSmeny === '') {
            // neni zadana smena => nelze zapnout
            return;
        }

        $kodSmeny = StringUtils::fromUTF8($kodSmeny);

        $datumDo = $datumDo ?: $datum;

        if ($datumDo < $datum) {
            return;
        }

        Prava::nastavObdobiProPrava($datum, $datumDo);
        $prava = Prava::vratPrava('smenykalendare');

        $rozdeleni = null;
        $kodRozdeleni = $_GET['rozdeleni'];

        if (isset($kodRozdeleni) && $kodRozdeleni !== '') {
            $kodRozdeleni = StringUtils::fromUTF8($kodRozdeleni);

            if (!$prava ->jePravoNaRozdeleni(KapacitySettings::typRozdeleniProVyber(), $kodRozdeleni, $datum)) {
                throw new NeniPravoException();
            }

            // nacist kalendar rozdeleni
            $rozdeleni = Rozdeleni::loadRozdeleni($this ->database, KapacitySettings::typRozdeleniProVyber(), $kodRozdeleni);
            $kodKalendare = $rozdeleni ->kalendar;
        } else {
            // editace kalendare bez rozdeleni => musi mit pravo na vse
            if ($prava ->prava != Prava::PRAVA_VSE) {
                throw new NeniPravoException();
            }

            $kodKalendare = StringUtils::fromUTF8($_GET['kalendar']);
        }

        if (!isset($kodKalendare) ||  $kodKalendare === '') {
            // Neni zadan ani prirazen kalendar
            throw new Exception('Neni zadan kod kalendare ani rozdeleni s prirazenym kalendarem!');
        }

        // kontrola obdobi - nesmi byt uzavreno a musi byt vytvoreno!
        $obdobi = Obdobi::vratObdobi($this ->database, 'ADS', $datum);
        if ($obdobi ->uzavreno) {
            throw new Exception( \Ron\ronText('Obdobi_uzavreno') );
        }

        if ($datumDo > $obdobi ->konec) {
            // zadany rozsah datumu zasahuje do vice obdobi => zkontrolovat i ostatni
            if (Obdobi::jeObdobiUzavreno($this ->database, 'ADS', $datum, $datumDo)) {
                throw new Exception( \Ron\ronText('Obdobi_uzavreno') );
            }
            // a zaroven musi byt vytvorene
            $obdobi2 = Obdobi::vratObdobi($this ->database, 'ADS', $datumDo);
            if (!$obdobi2) {
                // nemelo by nastat
                throw new Exception('V zadanem rozsahu datumu jeste neni vytvoreno obdobi!');
            }
        }

        // kontrola schvaleni
        $schvaleni = SchvaleniPodminekPlanovani::nactiSchvaleni($this ->database, $obdobi ->zacatek, $rozdeleni ->typ, $rozdeleni ->kod);
        if (!$schvaleni ->lzeEditovatDen($datum)) {
            throw new Exception('Data nelze editovat, protoze plan je jiz schvalen!');
        }

        while ($datum <= $datumDo) {
            $rec = new RonPortalDBRecord($this ->database, 'SmenaProDenKalendare', $this);

            $rec ->addField('Kalendar', '', true, 30, $kodKalendare);
            $rec ->addField('Den', 'timestamp', true, 0, $datum);
            $rec ->addField('Smena', '', true, 10, $kodSmeny);

            $params = $this ->database ->createParameters();
            $rec ->setWhere(
                'Kalendar = ' . $params ->addParameter($kodKalendare) .
                ' AND Smena = ' . $params ->addParameter($kodSmeny) .
                ' AND Den = ' . $params ->addParameter($this ->database ->dateToParam($datum)),
                $params);

            if ($zapnuta && $rozdeleni && RonPortalSettings::getIntValue('sablonyprodenkalendare')) {
                $idSablony = isset($_GET['sablonaPodminek']) ? (int)$_GET['sablonaPodminek'] : null;

                // nacte vychozi sablonu
                $podrobnostiSablony = $this ->nactiSablonuProRozdeleni($rozdeleni, $idSablony);
                if ($idSablony  &&  !$podrobnostiSablony) {
                    throw new Exception('Zadana sablona podminek nebyla nalezena!');
                }
                $idSablony = $podrobnostiSablony ? $podrobnostiSablony['id'] : null;

                if ($podrobnostiSablony) {
                    $output ->addData('sablona', [
                        'id' => $podrobnostiSablony['id'],
                        'zkratka' => StringUtils::jsonText($podrobnostiSablony['zkratka']),
                        'popis' => StringUtils::jsonText($podrobnostiSablony['popis'])
                    ]);
                }

                $rec ->addField('SadaPodminekPlanovani', 'int', !isset($idSablony), 0, $idSablony);
            }

            if ($rec ->loadRecord()) {
                if (!$zapnuta) {
                    $rec ->deleteRecord();
                } else {
                    $rec ->updateRecord();
                }
            } else {
                if ($zapnuta) {
                    $rec ->insertRecord();
                }
            }

            $this ->generujPodminkyPodleSablon($rozdeleni, $datum);

            $datum = DateUtils::incDayStart($datum);
        }
    }



    /**
     * @param JsonOutputObject $output
     */
    private function zapniSmenu($output)
    {
        $this ->nastavSmenu(true, $output);
    }


    /**
     * @param JsonOutputObject $output
     */
    private function vypniSmenu($output)
    {
        $this ->nastavSmenu(false, $output);
    }


    /**
     * @param JsonOutputObject $output
     * @throws DBException
     */
    private function nactiSeznamSablon($output)
    {
        $kodRozdeleni = $_GET['rozdeleni'];
        $typRozdeleni = KapacitySettings::typRozdeleniProVyber();

        $params = $this ->database ->createParameters();
        $cursor = $this ->database ->getSqlResult(
            'SELECT PovolenaSablonaPodminek.IdSady, SadaPodminekPlanovani.Zkratka, SadaPodminekPlanovani.Popis' .
            ' FROM PovolenaSablonaPodminek, SadaPodminekPlanovani' .
            ' WHERE PovolenaSablonaPodminek.KodTypuRozdeleni = ' . $params ->addParameter($typRozdeleni) .
            ' AND PovolenaSablonaPodminek.Rozdeleni = ' . $params ->addParameter($kodRozdeleni) .
            //' AND PovolenaSablonaPodminek.Vychozi = 1' .
            ' AND PovolenaSablonaPodminek.IdSady = SadaPodminekPlanovani.Id' .
            ' ORDER BY Vychozi DESC, Popis',
            $params
        );

        $data = [];
        while ($row = $cursor ->fetch()) {
            $data[] = [ 'id' => $row[0], 'zkratka' => StringUtils::jsonText($row[1]), 'popis' => StringUtils::jsonText($row[2]) ];
        }

        $cursor ->close();

        $output ->addData('sablony', $data);
    }


    private function generujSablonu($rozdeleni, $datum)
    {
        // zjisti, jestli existuje sada podminek a pokud ne, tak ji vytvori
//        $newId = SadaPodminekPlanovani::vytvorSaduPodminekProDen($this ->database, $this ->rozdeleni, $this ->dny[$indexDne] ->datum);
    }


    public function showForm()
    {
        HtmlHeaders::getInstance()
            ->addCss('css/s_kapacity.css')
            ->addScript('script/dochazka.js')
            ->setAppTitle('Povolene_smeny_v_kalendarich');

        $typRozdeleni = KapacitySettings::typRozdeleniProVyber();
        $prava = Prava::vratPrava('smenykalendare');

        $filterForm = new RonPortalFilterForm($this ->database, 'smenykalendare', true);
        $filterForm ->addFilterObdobi(); // jenOtevrene = false
        $filterForm ->header = \Ron\ronText('Povolene_smeny_v_kalendarich');

        ($filterForm ->addFilterRozdeleni('rozdeleni', $typRozdeleni))
            ->setSessionName('ads_rozdeleni_' . $typRozdeleni)
            ->setPouzitPopisTypuRozdeleni(true)
            ->setJenVlastniRozdeleni(0)
            ->setPrava($prava);
//        $filterForm ->f_pravarozdeleni = $prava;
//        $filterForm ->nastavRozdeleni(true, $typRozdeleni, 0, '', true);

        $filterForm ->loadData();

        Model::addViewObject($filterForm);

        $kodRozdeleni = $filterForm->getFilterValue('rozdeleni');

        $rozdeleni = Rozdeleni::loadRozdeleni($this ->database, $typRozdeleni, $kodRozdeleni);
        $obdobi = Obdobi::vratObdobi($this ->database, 'ADS', $filterForm ->getOmezitObdobiOd());

        Prava::nastavObdobiProPrava(DateUtils::today());

        if ($prava ->jePravoNaRozdeleni($rozdeleni ->typ, $rozdeleni ->kod, $obdobi ->zacatek, $obdobi ->konec)) {
            $form = new SmenyKalendareFormular($this ->database);
            $form ->load($rozdeleni, $obdobi, true);

            return $form;
        }

        MessageManager::addErrorMessage(\Ron\ronText('Nemate_pravo'));

        return new TextViewObject("");
    }


    /**
     * @param JsonOutputObject $output
     */
    function nastavSchvaleni($output)
    {
        $kodTypuRozdeleni = KapacitySettings::typRozdeleniProVyber();

        $rozdeleni = Rozdeleni::loadRozdeleni($this->database, $kodTypuRozdeleni, $_POST['rozdeleni']);

        $pocatekObdobi = DateUtils::urlStringToDate($_POST['obdobi']);

        SchvaleniPodminekPlanovani::nastavSchvaleniPodminek($this ->database,
            $rozdeleni, $pocatekObdobi,
            (int)$_POST['uroven'], $_POST['datum'] != '' ? DateUtils::urlStringToDate($_POST['datum']) : null);

        $schvaleni = SchvaleniPodminekPlanovani::nactiSchvaleni($this ->database, DateUtils::urlStringToDate($_POST['obdobi']), $rozdeleni ->typ, $rozdeleni ->kod);
        $output ->addData('schvaleni1', $schvaleni ->schvaleni1 ? DateUtils::dateToUrlString($schvaleni ->schvaleni1) : null);
        $output ->addData('schvaleni2', $schvaleni ->schvaleni2 ? DateUtils::dateToUrlString($schvaleni ->schvaleni2) : null);
        $output ->addData('schvaleni3', $schvaleni ->schvaleni3 ? DateUtils::dateToUrlString($schvaleni ->schvaleni3) : null);
    }


    /**
     * @param string $action
     * @param JsonOutputObject $output
     */
    function handleJsonAction($action, $output)
    {
        switch ($_GET['action']) {
            case "zapnismenu" :
                $this ->zapniSmenu($output);
                break;

            case "vypnismenu" :
                $this ->vypniSmenu($output);
                break;

            case "seznamsablon" :
                $this ->nactiSeznamSablon($output);
                break;

            case "nastavschvaleni" :
                $this ->nastavSchvaleni($output);
                break;
        }
    }


    public function handleAction($action)
    {
        TextManager::getInstance() ->loadStrings('dochazka.ini');
        TextManager::getInstance() ->loadStrings('kapacity.ini');

        if (isset($_GET['action'])) {
            return parent::handleAction($action);
        }

        return $this ->showForm();
    }

}
