<?php
/**
 * stanislav.elbl, 07.03.2017
 */

namespace RonPortal\Kapacity;

use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\GuiUtils;
use Ron\StringUtils;

class VysledekKontrolyPlanovani
{
    var $id;
    var $typKontroly;
    var $datum;
    var $ignorovat;

    var $RC;
    var $osobniCislo;
    var $datumNastupu;
    var $idPP;

    var $kodTypuRozdeleni;
    var $rozdeleni;

    // nastaveni kontroly
    var $popis;
    var $mesicni;

    var $poznamka;

    var $blokovatSchvaleni1;
    var $blokovatSchvaleni2;
    var $blokovatSchvaleni3;

    /** @var {string}   HTML kod barvy */
    var $barva;
    var $zvyraznit;
    var $cssClass;
    var $kategorie;


    function getJson()
    {
        $result = [
            'id' => $this ->id,
            'typKontroly' => StringUtils::jsonText($this ->typKontroly),
            'datumStr' => StringUtils::jsonDate($this ->datum),
            'ignorovat' => $this ->ignorovat ? 1 : 0,
            'popis' => StringUtils::jsonText($this ->popis),
            'mesicni' => $this ->mesicni ? 1 : 0,
            'blokovatSchvaleni1' => $this ->blokovatSchvaleni1,
            'blokovatSchvaleni2' => $this ->blokovatSchvaleni2,
            'blokovatSchvaleni3' => $this ->blokovatSchvaleni3,
            'zvyraznit' => $this ->zvyraznit ? 1 : 0,
            'cssClass' => StringUtils::jsonText($this ->cssClass),
            'kategorie' => $this ->kategorie
        ];

        if ($this ->barva) {
            $result['barva'] = $this ->barva;
        }

        if ($this ->poznamka) {
            $result['poznamka'] = StringUtils::jsonText($this ->poznamka);
        }

        if ($this ->idPP) {
            $result['idPP'] = $this ->idPP;
        }

        if ($this ->rozdeleni) {
            $result['kodTypuRozdeleni'] = $this ->kodTypuRozdeleni;
            $result['rozdeleni'] = StringUtils::jsonText($this ->rozdeleni);
        }

        return $result;
    }


    /**
     * Nacte kontroly pro zadany $idPP a za zadane rozdeleni
     * @param Database $database
     * @param int $datumOd
     * @param int $datumDo
     * @param int|int[] $idPP
     * @param int $typRozdeleni
     * @param string $rozdeleni
     * @return VysledekKontrolyPlanovani[]
     * @throws DBException
     */
    public static function nactiKontroly1($database, $datumOd, $datumDo, $idPP, $typRozdeleni, $rozdeleni)
    {
        $params = $database ->createParameters();
        $sql =
            'SELECT VysledekKontrolyPlanovani.Id, VysledekKontrolyPlanovani.TypKontroly, VysledekKontrolyPlanovani.Datum, ' .
            ' VysledekKontrolyPlanovani.RC, VysledekKontrolyPlanovani.OsobniCislo, VysledekKontrolyPlanovani.DatumNastupu, PracovniPomer.Id, ' .
            ' VysledekKontrolyPlanovani.KodTypuRozdeleni, VysledekKontrolyPlanovani.Rozdeleni, ' .
            ' VysledekKontrolyPlanovani.Ignorovat, ' . $database ->castWideMemo('VysledekKontrolyPlanovani.Poznamka') . ', ' .
            ' TypKontrolyPlanovani.Popis, TypKontrolyPlanovani.Mesicni, ' .
            ' TypKontrolyPlanovani.Barva, TypKontrolyPlanovani.Zvyraznit, TypKontrolyPlanovani.cssClass, TypKontrolyPlanovani.kategorie, ' .
            ' TypKontrolyPlanovani.BlokovatSchvaleni1, TypKontrolyPlanovani.BlokovatSchvaleni2, TypKontrolyPlanovani.BlokovatSchvaleni3' .
            ' FROM VysledekKontrolyPlanovani INNER JOIN TypKontrolyPlanovani '.
            '   ON TypKontrolyPlanovani.Kod = VysledekKontrolyPlanovani.TypKontroly ' .
            ' LEFT OUTER JOIN PracovniPomer ' .
            '   ON PracovniPomer.RC = VysledekKontrolyPlanovani.RC ' .
            '  AND PracovniPomer.OsobniCislo = VysledekKontrolyPlanovani.OsobniCislo ' .
            '  AND PracovniPomer.DatumNastupu = VysledekKontrolyPlanovani.DatumNastupu ' .
            ' WHERE Datum >= ' . $params ->addParameter($database ->dateToParam($datumOd)) .
            '   AND Datum <= ' . $params ->addParameter($database ->dateToParam($datumDo)) .
            '   AND Ignorovat = 0';

        if ($idPP  ||  $rozdeleni) {
            $sql .= ' AND (';

            if ($idPP) {
                if (is_array($idPP)) {
                    $first = true;
                    $sql .= ' PracovniPomer.Id IN (';
                    foreach ($idPP as $id) {
                        if ($id !== '') {
                            $sql .= ($first ? '' : ', ') . $params->addParameter($id);
                            $first = false;
                        }
                    }
                    $sql .= ')';

                    if ($first) {
                        return [];  // pole neobsahuje zadne id PP
                    }
                } else {
                    $sql .= ' PracovniPomer.Id = ' . $params->addParameter($idPP);
                }
            }

            if ($idPP  &&  $rozdeleni) $sql .= ' OR ';

            if ($rozdeleni) $sql .=
                ' (KodTypuRozdeleni = ' . $params ->addParameter($typRozdeleni) .
                ' AND Rozdeleni = ' . $params ->addParameter($rozdeleni) .
                ' )';

            $sql .= ')';
        }

        $sql .= ' ORDER BY PracovniPomer.Id, VysledekKontrolyPlanovani.Datum';

        $cursor = $database ->getSqlResult($sql, $params, 'Pri nacitani vysledku kontrol planovani doslo k chybe!');

        $result = [];
        while ($row = $cursor ->fetch()) {
            $kontrola = new self();
            $kontrola ->id = $row[0];
            $kontrola ->typKontroly = $row[1];
            $kontrola ->datum = $cursor ->stringToTimestamp($row[2]);

            $kontrola ->RC = $row[3];
            $kontrola ->osobniCislo = $row[4];
            $kontrola ->datumNastupu = $cursor ->stringToTimestamp($row[5]);
            $kontrola ->idPP = isset($row[6]) ? (int)$row[6] : null;

            $kontrola ->kodTypuRozdeleni = $row[7];
            $kontrola ->rozdeleni = $row[8];

            $kontrola ->ignorovat = $row[9];

            $kontrola ->poznamka = $row[10];

            $kontrola ->popis = $row[11];
            $kontrola ->mesicni = $row[12];
            $kontrola ->barva = $row[13] ? GuiUtils::getHtmlColor($row[13]) : null;
            $kontrola ->zvyraznit = (int)$row[14];
            $kontrola ->cssClass = $row[15];
            $kontrola ->kategorie = $row[16];

            $result[] = $kontrola;
        }

        return $result;
    }


    /**
     * @param Database  $database
     * @param int $datumOd
     * @param int $datumDo
     * @param int|int[] $idPP
     * @param int $typRozdeleni
     * @param string    $rozdeleni
     * @return VysledekKontrolyPlanovani[]
     */
    public static function nactiKontroly($database, $datumOd, $datumDo, $idPP, $typRozdeleni, $rozdeleni)
    {
        if (!is_array($idPP)  ||  count($idPP) <= 40) {
            return self::nactiKontroly1($database, $datumOd, $datumDo, $idPP, $typRozdeleni, $rozdeleni);
        }

        // vice nez 40 idPP
        $result = [];

        $seznam = [];
        foreach ($idPP as $id) {
            $seznam[] = $id;
            if (count($seznam) >= 40) {
                $kontroly = self::nactiKontroly1($database, $datumOd, $datumDo, $seznam, $typRozdeleni, $rozdeleni);
                $result = array_merge($result, $kontroly);
                // vynuluje pro dalsi
                $seznam = [];
                $typRozdeleni = null;
                $rozdeleni = null;
            }
        }

        if ($seznam) {
            $kontroly = self::nactiKontroly1($database, $datumOd, $datumDo, $seznam, $typRozdeleni, $rozdeleni);
            $result = array_merge($result, $kontroly);
        }

        return $result;
    }


    /**
     * Nacte kontroly pro zadany $idPP a za zadane rozdeleni
     * @param Database $database
     * @param int $datumOd
     * @param int $datumDo
     * @param int|int[] $idPP
     * @param int $typRozdeleni
     * @param string $rozdeleni
     * @return VysledekKontrolyPlanovani[]
     */
    public static function nactiKontrolyJSON($database, $datumOd, $datumDo, $idPP, $typRozdeleni, $rozdeleni)
    {
        $result = [];

        $data = static::nactiKontroly($database, $datumOd, $datumDo, $idPP, $typRozdeleni, $rozdeleni);

        foreach ($data as $kontrola) {
            $result[] = $kontrola->getJson();
        }

        return $result;
    }
}
