<?php
/**
 * User: pavel.rudol
 * 13.6.2016 13:09
 */

namespace RonPortal\Kapacity;

use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\MessageManager;
use RonPortal\Prava;

class Zastup
{
    public $id;
    public $datum;
    public $casOd;
    public $casDo;

    public $jmeno;
    public $prijmeni;
    public $titul;
    public $osobniCislo;
    /** @var  int id PP prirazeneho zamestnance */
    public $idPP;

    public $kodTypuRozdeleni;
    public $rozdeleni;
    public $rozdeleniPopis;
    public $pozice;
    public $pozicePopis;
    public $kategoriePozice;
    public $kategoriePozicePopis;
    public $duvodZadosti;
    public $poznamka;
    public $specifikace;

    /** @var int Typ schvaleni: 0 = nerozhodnuto, 1 = schvaleno, 2 = zamitnuto, 3 = zrusena zadost (jiz je prirazen zamestnanec) */
    public $typSchvaleni;
    public $datumSchvaleni;
    public $schvalil;

    public $lzeEditovat = false;
    public $lzePriraditOsobu = false;
    public $lzeSchvalit = false;
    public $lzeZobrazit = false;
    public $lzeZrusitPlan = false;

    /**
     * @param Database $database
     * @param array $row
     */
    private function nactiRadek($database, $row)
    {
        $this->id = $row[0];
        $this->datum = $database->stringToTimestamp($row[1]);
        $this->casOd = $database->stringToTimestamp($row[2]);
        $this->casDo = $database->stringToTimestamp($row[3]);

        $this->jmeno = $row[11];
        $this->prijmeni = $row[10];
        $this->titul = $row[15];
        $this->osobniCislo = $row[16];
        $this->idPP = $row[12];
        $this->poznamka = $row[13];
        $this->duvodZadosti = $row[18];
        $this->specifikace = $row[19];

        $this->kodTypuRozdeleni = $row[14];
        $this->rozdeleni = $row[6];
        $this->rozdeleniPopis = $row[8];
        $this->pozice = $row[7];
        $this->pozicePopis = $row[9];
        $this->kategoriePozice = $row[20];
        $this->kategoriePozicePopis = $row[21];

        $this->typSchvaleni = $row[17];
        $this->datumSchvaleni = $database->stringToTimestamp($row[4]);
        $this->schvalil = $row[5];

    }

    /**
     * @return bool
     * @throws \Exception
     */
    public function lzeZobrazitZaznam()
    {
        $pravaEditovat = Prava::vratPrava('zastupy_vlozeni');
        $pravaSchvalit = Prava::vratPrava('zastupy_schvaleni');
        $pravaPriradit = Prava::vratPrava('zastupy_prideleni');
        $pravaZobrazit = Prava::vratPrava('zastupy_prohl');
        $pravaZrusitPlan = Prava::vratPrava('zastupy_zruseni');

        $this->lzeZobrazit = $pravaZobrazit->jePravoNaRozdeleni($this->kodTypuRozdeleni, $this->rozdeleni, $this->datum, $this->datum) ||
            $pravaZobrazit->lzeZobrazitPP($this->idPP, $this->datum, $this->datum);

        //editovat lze jen neschvalene zaznamy
        $this->lzeEditovat = ($this->typSchvaleni == 0) && $pravaEditovat->jePravoNaRozdeleni($this->kodTypuRozdeleni, $this->rozdeleni, $this->datum, $this->datum);
        $this->lzeSchvalit = ($this->typSchvaleni != 3) && $pravaSchvalit->jePravoNaRozdeleni($this->kodTypuRozdeleni, $this->rozdeleni, $this->datum, $this->datum);
        $this->lzePriraditOsobu = ($this->typSchvaleni == 1) &&
            $pravaPriradit->jePravoNaRozdeleni($this->kodTypuRozdeleni, $this->rozdeleni, $this->datum, $this->datum);
        $this->lzeZrusitPlan =
            ($this->typSchvaleni == 1) && $this->idPP && //zrusit lze schvalene zadosti s prirazenym zamestnancem
            $pravaZrusitPlan->jePravoNaRozdeleni($this->kodTypuRozdeleni, $this->rozdeleni, $this->datum, $this->datum) && //musi byt pravo na rozdeleni
            $pravaZrusitPlan->lzeZobrazitPP($this->idPP, $this->datum, $this->datum); //i pravo na prirazeneho zamestnance

        $lzeZobrazit = $pravaEditovat->jePravoNaRozdeleni($this->kodTypuRozdeleni, $this->rozdeleni, $this->datum, $this->datum);
        $lzeZobrazit = $lzeZobrazit || $this->lzeSchvalit || $pravaPriradit->jePravoNaRozdeleni($this->kodTypuRozdeleni, $this->rozdeleni, $this->datum, $this->datum);
        $lzeZobrazit = $lzeZobrazit || $this->lzeZobrazit || $this->lzeZrusitPlan;

        return $lzeZobrazit;
    }

    /** Nacte zastup z databaze.
     * @param Database $database
     * @param int $id
     * @return null|Zastup
     * @throws DBException
     * @throws \Exception
     */
    public static function nactiZastup($database, $id)
    {
        $params = $database->createParameters();

        $sql =
            "SELECT ZadostOZastup.ID, ZadostOZastup.Datum, ZadostOZastup.CasOd, ZadostOZastup.CasDo, " . // 0 - 3
            "ZadostOZastup.Schvaleni, ZadostOZastup.Schvalil, " . // 4, 5
            "ZadostOZastup.Rozdeleni, ZadostOZastup.Rozdeleni2, Rozdeleni.Popis, Pozice.Popis, Osoba.Jmeno, Osoba.Prijmeni, PracovniPomer.ID, ZadostOZastup.Poznamka, " . // 6 - 13
            "ZadostOZastup.KodTypuRozdeleni, Osoba.Titul, PracovniPomer.OsobniCislo, ZadostOZastup.TypSchvaleni, DuvodZastupu.Popis, ZadostOZastup.Specifikace, " . //14 - 19
            "ZadostOZastup.KodKategorie, Kategorie.Popis " . // 20 - 21
            " FROM ZadostOZastup LEFT JOIN Osoba ON (ZadostOZastup.RC = Osoba.RC) " .
            " LEFT JOIN PracovniPomer ON (PracovniPomer.RC = ZadostOZastup.RC AND PracovniPomer.OsobniCislo = ZadostOZastup.OsobniCislo AND PracovniPomer.DatumNastupu = ZadostOZastup.DatumNastupu) " .
            " LEFT JOIN Rozdeleni ON (Rozdeleni.KodTypuRozdeleni = ZadostOZastup.KodTypuRozdeleni AND Rozdeleni.Kod = ZadostOZastup.Rozdeleni) " .
            " LEFT JOIN Rozdeleni Pozice ON (Pozice.KodTypuRozdeleni = ZadostOZastup.KodTypuRozdeleni2 AND Pozice.Kod = ZadostOZastup.Rozdeleni2)" .
            " LEFT JOIN DuvodZastupu ON (DuvodZastupu.Kod = ZadostOZastup.DuvodZadosti)" .
            " LEFT JOIN Kategorie ON (Kategorie.TypKategorie = ZadostOZastup.TypKategorie AND Kategorie.Kod = ZadostOZastup.KodKategorie)" .
            "WHERE ZadostOZastup.ID = " . $params->addParameter($id);

        $cursor = $database->getSqlResult($sql,$params);
        MessageManager::addDebugLog($sql);

        $row = $cursor->fetch();
        $cursor ->close();

        if (!$row) {
            throw new \Exception('Zaznam zadosti o zastup nenalezen.');
        }

        $zastup = new Zastup();
        $zastup->nactiRadek($database, $row);

        $zastup->lzeZobrazitZaznam();

        return $zastup;
    }

    /**
     * @return string Prijmeni Jmeno, Titul
     */
    public function vratJmenoPrijmeni()
    {
        return $this->prijmeni . ' ' . $this->jmeno . ($this->titul ? ', ' . $this->titul : '');
    }
}
