<?php
/**
 * pavel.rudol, 19.12.2024 15:06
 */

namespace RonPortal\Mzdy;

use Ron\Controls\BrowserField;
use Ron\DB\SqlStatement;
use Ron\TextManager;
use RonPortal\Controls\RonPortalBrowser;
use RonPortal\Prava;
use RonPortal\PravaNaOsoby;
use RonPortal\RozdeleniUtils;

class StatusRozhodnutiBrowserField extends BrowserField
{
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        return TextManager::_("StatusRozhodnuti_$value");
    }

}

class EditaceOznameniField extends BrowserField
{
    function __construct()
    {
        parent::__construct();
        $this ->code = 'col-edit';
        $this ->align = 'left';
    }

    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if (!(Prava::vratPrava('davkynz_zapis')->lzeZobrazitPP($rowData['col-pp-id']) && $rowData['col-statusrozhodnuti'] == 0)) {
            return '';
        }

        return
            '<i class="inline-glyph glyph-button fa glyph-edit clickHref"' .
            ' data-ron-href="?menuid=davkynz&formaction=edit&record=' . $rowData['col-davky-id'] . '"' . // id radku
            ' title="' . TextManager::_('Editovat_zaznam') . '">';
    }
}

class DavkyNZBrowser extends RonPortalBrowser
{
    private PravaNaOsoby $pravaProhlizeni;

    public function __construct($database)
    {
        parent::__construct($database, 'davkynz');

        $this->pravaProhlizeni = Prava::vratPrava('davkynz_prohl');
        if (!$GLOBALS['development']) //TODO: po dokonceni smazat
        $this->filterFormAppCode = 'MZDY';
        $this->filterFormRights = 'davkynz_prohl';

        $this->title = TextManager::_('DavkyNZ_oznameni');

        $this->code = 'browserdavkynz';

        $this->sqlSelectPart = "OznameniKDavkamNZ.ID";

        $this->sqlFromPart =
            "OznameniKDavkamNZ 
            INNER JOIN PracovniPomer ON (OznameniKDavkamNZ.RC = PracovniPomer.RC AND OznameniKDavkamNZ.OsobniCislo = PracovniPomer.OsobniCislo AND OznameniKDavkamNZ.DatumNastupu = PracovniPomer.DatumNastupu )
            INNER JOIN Osoba ON (Osoba.RC = PracovniPomer.RC)
            INNER JOIN DruhDavkyNZ ON (DruhDavkyNZ.Kod = OznameniKDavkamNZ.DruhDavky)";

        $this->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('OznameniKDavkamNZ.ID')->setCode('col-davky-id');
        $this->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('PracovniPomer.ID')->setCode('col-pp-id');

        $this->primaryField = 0;

        $this->addField()
            ->setSqlFieldName('PracovniPomer.OsobniCislo')->setHeaderText('Osobni_cislo')->setAlign('align-right')->setOrdering(true)
            ->setCode('col-osobnicislo');
        $this->addField()
            ->setSqlFieldName('Osoba.Jmeno')->setHeaderText('Jmeno')->setOrdering(true)
            ->setCode('col-jmeno');
        $this->addField()
            ->setSqlFieldName('Osoba.Prijmeni')->setHeaderText('Prijmeni')->setOrdering(true)
            ->setCode('col-prijmeni')->setLink($GLOBALS['development'] ? '?menuid=davkynz&record=' : null);
        $this->addField()
            ->setSqlFieldName('DruhDavkyNZ.Popis')->setHeaderText('DavkyNZ_DruhDavky')->setOrdering(true)
            ->setCode('col-druhdavky');
        $this->addField()
            ->setSqlFieldName('OznameniKDavkamNZ.CisloDokladu')->setHeaderText('DavkyNZ_CisloDokladu')->setAlign('align-right')->setOrdering(true)
            ->setCode('col-cislodavky')->setLink($GLOBALS['development'] ? '?menuid=davkynz&record=' : null);
        $this->addField(new StatusRozhodnutiBrowserField())
            ->setSqlFieldName('OznameniKDavkamNZ.CisloRozhodnutiStatus')->setHeaderText('DavkyNZ_StatusRozhodnuti')->setAlign('align-center')
            ->setCode('col-statusrozhodnuti');
        $this->addField()
            ->setSqlFieldName('OznameniKDavkamNZ.DatumCasVlozeni')->setHeaderText('DavkyNZ_DatumVlozeni')->setAlign('align-right')->setOrdering(true)->setFormat('datetime')
            ->setCode('col-datumvlozeni');

        $this->addField(new EditaceOznameniField());

    }

    function setFilterForm()
    {
        $this ->filterForm = $this->createFilterForm(
            $this->filterFormAppCode, $this ->filterFormRights,
            true, $this ->filterFormVcetneExternich);
    }

    function renderFilter()
    {
        parent::renderFilter();

        if (Prava::jsouNejakePrava('davkynz_zapis'))
        echo "<a href='?menuid=davkynz&formaction=new'>" . TextManager::_('New_record') . "</a>";
    }


    public function getSQLFilter()
    {
        $result = '';
        
        $filterFormFilter = $this ->filterForm ->buildSqlFilter();

        if ($filterFormFilter) {
            $params = $filterFormFilter ->parameters;
            $result .=
                ($result ? ' AND ' : '') .
                '(' . $filterFormFilter ->sqlText . ')';
        } else {
            $params = $this->database->createParameters();
        }

        $result .= $this->pravaProhlizeni->sqlFiltr('PracovniPomer', 'OznameniKDavkamNZ.DatumCasVlozeni', 'OznameniKDavkamNZ.DatumCasVlozeni', $params);

        $result .=
            ' AND OznameniKDavkamNZ.DatumCasVlozeni >= ' . $params->addDateParameter($this->filterForm->getFilterValue('obdobi')) .
            ' AND OznameniKDavkamNZ.DatumCasVlozeni <= ' . $params->addDateParameter($this->filterForm->getFilterValue('obdobi', 'datumDo'));


        foreach ($this ->filterForm ->getRozdeleniFilters() as $rozdeleniFilter) {
            $value = $rozdeleniFilter ->getValue();
            if (isset($value)) {
                $filtrR = RozdeleniUtils::sqlFilterRozdeleniPP($this->database, 'PracovniPomer',
                    $rozdeleniFilter ->getTypRozdeleni(), $rozdeleniFilter ->getValue(),
                    '{fn_EXTRACTDATE(OznameniKDavkamNZ.DatumCasVlozeni)}','{fn_EXTRACTDATE(OznameniKDavkamNZ.DatumCasVlozeni)}',
                    $rozdeleniFilter ->getValue('vcetnePodrizenych'),
                    $params);

                if ($filtrR) {
                    $result .=
                        ($result ? ' AND ' : '') .
                        '(' . $filtrR . ')';
                }
            }
        }

        // jen vybrana osoba
        if ($this ->filterForm ->getFilterValue('osoba') !== null) {
            $result .=
                ($result ? ' AND ' : '') .
                ' PracovniPomer.RC = ' . $params ->addParameter($this ->filterForm ->getFilterValue('osoba', 'rc')) .
                ' AND PracovniPomer.OsobniCislo = ' . $params ->addParameter($this ->filterForm ->getFilterValue('osoba', 'osobniCislo')) .
                ' AND PracovniPomer.DatumNastupu = ' . $params ->addDateParameter($this ->filterForm ->getFilterValue('osoba', 'datumNastupu'));
        }

        return new SqlStatement($result, $params);
    }


}