<?php
/**
 * pavel.rudol, 19.12.2024 15:01
 */

namespace RonPortal\Mzdy;

use Ron\DB\Database;
use Ron\Http\HtmlHeaders;
use Ron\MVC\BaseViewObject;
use Ron\MVC\JsonController;
use Ron\MVC\Redirect;
use Ron\MVC\TextViewObject;
use Ron\TextManager;
use RonPortal\NeniPravoException;
use RonPortal\Prava;

class DavkyNZController extends JsonController
{
    private Database $database;

    private $pravaProhlizeni;
    private $pravaZapis;

    /**
     * @param Database $database
     */
    public function __construct($database)
    {
        $this->database = $database;

        $this->pravaProhlizeni = Prava::vratPrava('davkynz_prohl');
        $this->pravaZapis = Prava::vratPrava('davkynz_zapis');
    }

    public function browserDavky()
    {
        $b = new DavkyNZBrowser($this->database);
        $b->load();
        return $b;
    }

    public function formDavky()
    {
        if (isset($_GET['record'])) {
            $oznameni = OznameniDavekNZ::load($this->database, $_GET['record']);

            if (!($this->pravaZapis->lzeZobrazitPP($oznameni->idPP) && $oznameni->status == 0)) {   //TODO: aktualne lze editovat jen ve stavu neovereno
                throw new NeniPravoException();
            }
        }

        $f = new DavkyNZForm($this->database);

        if ($f->handleAction()) {
            return new Redirect(null, '?menuid=davkynz');
        }

        return $f;
    }

    public function detail($id)
    {
        $oznameni = OznameniDavekNZ::load($this->database, $id);

        if (!$this->pravaZapis->lzeZobrazitPP($oznameni->idPP)) {
            throw new NeniPravoException();
        }

        return new BaseViewObject('ronportal/mzdy/davkynz_detail.php');
    }

    /**
     * @inheritDoc
     */
    function handleJsonAction($action, $output)
    {
        // TODO: Implement handleJsonAction() method.
    }

    public function handleAction($action)
    {
        TextManager::getInstance()->loadStrings('mzdy.ini');
        HtmlHeaders::getInstance()->addScript('script/mzdy.js');
        HtmlHeaders::getInstance()->addScript('', '?menuid=davkynz&action=translations&ver=' . HtmlHeaders::getInstance() ->versionId);

        if (!($this->pravaProhlizeni->jsouNejakePrava() || $this->pravaZapis->jsouNejakePrava()) ) {
            throw new NeniPravoException();
        }

        if (isset($_GET['formaction']) || isset($_POST['formaction'])) {
            return $this->formDavky();
        }

        if ($GLOBALS['development']) {
            if (isset($_GET['record'])) {
                return $this->detail($_GET['record']);
            }
        }

        return $this->browserDavky();
    }
}