<?php
/**
 * pavel.rudol, 19.12.2024 17:49
 */

namespace RonPortal\Mzdy;

use Ron\Controls\FormField;
use Ron\TextManager;
use RonPortal\Controls\OsobaFormField;
use RonPortal\Controls\RonPortalForm;
use RonPortal\Controls\UserLoginFormField;

class DavkyNZForm extends RonPortalForm
{
    public function __construct($database, $formAction = 'new')
    {
        parent::__construct($database, $formAction);

        $this->code = 'davkynzform';

        $this->defaultRowClass = 'row';
        $this->defaultCaptionColClass = 'col-12 col-sm-3 caption';
        $this->defaultDataColClass = 'col-12 col-sm-9';

        $this ->htmlAction = '?menuid=davkynz';
        $this ->stornoAction = '?menuid=davkynz';

        $this ->title = TextManager::_('DavkyNZ_oznameni');

        $this ->sqlTableName = 'OznameniKDavkamNZ';

        // ID
        $this->addField(FormField::FIELD_TYPE_ID)
            ->setFieldName('Id') ->setPrimary(true) //->setSequence(true)
            ->setCode('field-id');

        $this->addField()
            ->setFieldName('CisloDokladu')
            ->setCode('field-cislodokladu')
            ->setCaption('DavkyNZ_CisloDokladu')
            ->setRowId('davkynz-row-cislodokladu')
            ->setFieldId('davkynz-cislodokladu')
            ->addFieldCssClass('davkynz-vyberdavky')
            ->setDataColumnClass('col-12 col-sm-3')
            ->setMaxLength(18);

        $this->addField(FormField::FIELD_TYPE_CHECK)
            ->setFieldName('ePodani')
            ->setCode('field-epodani')
            ->setCaption2('DavkyNZ_epodani')
            ->setFieldId('davkynz-epodani')
            ->setDataColumnClass('col-12 col-sm-3')
            ->setDefaultValue(true)
            ->setNewRow(false);

        $this->addField(new OsobaFormField('davkynz_zapis'))
            ->setFieldName('osobadavky')->setSqlFieldName('')
            ->setCode('field-osobadavky')
            ->setCaption('Zamestnanec')
            ->setRowId('davkynz-row-osobadavky')
            ->setIncludeToDBRecord(true);

        $this->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('DruhDavky')
            ->setCode('field-druh-davky')
            ->setCaption('DavkyNZ_DruhDavky')
            ->setRowId('davkynz-row-druhdavky') //TODO: pokud neni cislo dokladu lze zde vybirat, kdyz vyberou zrusit editaci cisla?
            ->setFieldId('davkynz-druhdavky')
            ->addFieldCssClass('davkynz-vyberdavky')
            ->setReadOnly(true)
            ->setSelectSql('SELECT Kod, Popis FROM DruhDavkyNZ WHERE (%FILTER%) ORDER BY %ORDER%', $this->isInsert() ? 'Kod IN (5,6)' : 'Kod IN (1,3,5,6)', 'Popis');

        $this->addField(FormField::FIELD_TYPE_DATE)
            ->setFieldName('ZadostOdDne')
            ->setCode('field-zadostoddne')
            ->setCaption('DavkyNZ_PocatekDavky')
            ->setRowId('davkynz-row-zadostoddne');


        if ($GLOBALS['development']) {
            $this->addField(FormField::FIELD_TYPE_DATE)
                ->setFieldName('ZadostDoDne')
                ->setCode('field-zadostdodne')
                ->setCaption('DavkyNZ_KonecDavky')
                ->setRowId('davkynz-row-zadostdodne');

            //TODO editovat ve stavu 0 a 3, po editaci stavu 3 změnit stav na 0
            //if

            //TODO ostatni pole ve stavu 2
        }

        $this->addField(FormField::FIELD_TYPE_ACTUAL_TIME)
            ->setFieldName('DatumCasVlozeni')
            ->setCode('field-datumcasvlozeni');

        $this->addField(FormField::FIELD_TYPE_ACTUAL_TIME)
            ->setFieldName('Insert_Timestamp') ->setReadOnly(true)
            ->setCode('field-insert-timestamp');

        $this ->addField(new UserLoginFormField(true)); // Insert_UserName

        $this->addField(FormField::FIELD_TYPE_ACTUAL_TIME)
            ->setFieldName('Update_Timestamp') ->setUpdateLog(true)
            ->setCode('field-update-timestamp');

        $this ->addField(new UserLoginFormField(false)); // Update_UserName
    }

    function beforeInsertRecord($rec)
    {
        // Kontrola na delku znaku - ne ePodani 8 znaku, ePodani 18 znaku
        if ($rec['ePodani']) {
            if (strlen($rec['cisloDokladu']) != 18) {
                throw new \Exception(sprintf(TextManager::_('DavkyNZ_chyba_delka'), 18));
            }
        } else {
            if (strlen($rec['cisloDokladu']) != 8) {
                throw new \Exception(sprintf(TextManager::_('DavkyNZ_chyba_delka'), 8));
            }
        }

        /*
         * Podle posledního písmne čísla dokladu, se určuje druh dávky:
            N, Z  -> OSE (DruhDavky=1)
            M      -> PPM (DruhDavky=3)
            T       -> OPP (DruhDavky=5)
            L        -> DLO (DruhDavky=6)
         * */
        switch (strtoupper(substr($rec['CisloDokladu'], -1))) {
            case 'N':
            case 'Z':
                $rec->findFieldByName('DruhDavky')->setNewValue(1);
                break;
            case 'M':
                $rec->findFieldByName('DruhDavky')->setNewValue(3);
                break;
            case 'T':
                $rec->findFieldByName('DruhDavky')->setNewValue(5);
                break;
            case 'L':
                $rec->findFieldByName('DruhDavky')->setNewValue(6);
                break;
            default:
                throw new \Exception(TextManager::_('DavkyNZ_chyba_posledniznak'));
        }

        // Kontrola toho, ze krome posledniho znaku musi byt vse cisla
        if (!is_numeric(substr($rec['CisloDokladu'],0,-1))) {
            throw new \Exception(TextManager::_('DavkyNZ_chyba_cisla'));
        }

        return parent::beforeInsertRecord($rec);
    }


}