<?php
/**
 * pavel.rudol, 16.12.2022 16:23
 */

namespace RonPortal\Mzdy;

use Exception;
use Ron\Controls\BrowserField;
use Ron\DateUtils;
use Ron\DB\DBUtils;
use Ron\DB\SqlStatement;
use Ron\Http\Csrf;
use Ron\StringUtils;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\RonPortalSettings;
use RonPortal\User;
use Ron\TextManager;
use RonPortal\Controls\RonPortalBrowser;
use RonPortal\Controls\RonPortalFilterForm;

class DatumProhlaseniBrowserField extends BrowserField
{
    /**
     * @param $value
     * @param $rowNumber
     * @param $recordId
     * @param $rowData
     * @return string
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if ($rowData['col-prohlaseni-elektronicky'] == 1) {
            if ($rowData['col-prohlaseni-datum-portal']) {
                $result = "<span class='font-weight-bold'>" . DBUtils::getFormatedValue($this->database,"date", $rowData['col-prohlaseni-datum-portal'])  . "</span>";
            }
        } else {
            if ($rowData['col-prohlaseni-datum']) {
                $result = "<span class='font-weight-bold'>" . DBUtils::getFormatedValue($this->database,"date", $rowData['col-prohlaseni-datum'])  . "</span>";
            }
        }

        if (!isset($result)) {
            $result = TextManager::_('Mzdy_nepodepsano');
        }

        return "<span class='align-left '>" . $result . "</span>";
    }
}

class ProhlaseniElektronickyBrowserField extends BrowserField
{
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = "";

        if ($rowData['col-prohlaseni-portal'] == 0) {
            $result = TextManager::_("prohlaseni_k_dani_necinim");
        } elseif ($rowData['col-prohlaseni-elektronicky'] == 1) {
            $result = TextManager::_("Prohlaseni_elektronicky");
        } elseif ($rowData['col-prohlaseni-elektronicky'] == 0) {
            $result = TextManager::_("Prohlaseni_pisemne");
        }

        return $result;
    }
}

class PracovniPomerProhlaseniBrowserField extends BrowserField
{
    /** @var RocniVyuctovaniDaneBrowser */
    private $browser;

    public function __construct($database, $browser)
    {
        parent::__construct($database);

        $this->browser = $browser;
    }


    /**
     * @param $value
     * @param $rowNumber
     * @param $recordId
     * @param $rowData
     * @return string
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = "";
        //puvodne byla kontrola na zobrazeni jen v aktualnim roce
        if (/*($rokRVD == $aktualniRok) &&*/ $rowData['col-prohlaseni-elektronicky'] == 1) {
            if (($rowData['col-prohlaseni-elektronicky'] == 1) && ($this->browser->filterForm->getFilterValue('osoba') != $rowData['col-pp-id'])) {
                if (isset($rowData['col-pp-id'])) {
                    $result .= TextManager::_('Prohlaseni_uplatnujete_na_jiny_ppid');
                }
            } else {
                $result .= TextManager::_('Prohlaseni_uplatnujete_na_tento_ppid');
            }
        }

        return $result;
    }

}

class ProhlaseniBrowserField extends BrowserField
{
    /** @var RocniVyuctovaniDaneBrowser */
    protected $browser;

    public function __construct($database, $browser)
    {
        parent::__construct($database);

        $this->browser = $browser;
    }

    function getDataCellClass($value, $rowNumber, $recordId, $rowData)
    {
        $aktualniRok = date('Y', time());
        $rokRVD = date('Y', $this->database->stringToTimestamp($rowData['col-rok']));
        $class = '';
        if ($aktualniRok == $rokRVD) {
            switch ($rowData['col-prohlaseni-portal-stav']) {
                case RocniVyuctovaniDaneRecord::PROHLASENI_SCHVALENO:
                    switch ($rowData['col-prohlaseni-portal-zmena-stav']) {
                        case RocniVyuctovaniDaneRecord::ZMENA_SCHVALENA:
                            $class = 'prohlaseni_zmena_schvalena';
                            break;
                        case RocniVyuctovaniDaneRecord::ZMENA_NAVRH:
                            $class = 'prohlaseni_zmena_navrh';
                            break;
                        case RocniVyuctovaniDaneRecord::ZMENA_NEZADANA:
                        default:
                            $class = 'prohlaseni_schvaleno';
                            break;
                    }
                    break;
                case RocniVyuctovaniDaneRecord::PROHLASENI_PODEPSANO:
                    $class = 'prohlaseni_podepsano_uzivatel';
                    break;
                case RocniVyuctovaniDaneRecord::PROHLASENI_VRACENO:
                    $class = 'prohlaseni_vraceno';
                    break;
                case RocniVyuctovaniDaneRecord::PROHLASENI_NAVRH:
                default:
                    $class = 'prohlaseni_navrh';
                    break;
            }
        }

        return parent::getDataCellClass($value, $rowNumber, $recordId, $rowData) . " " . $class;
    }


    /**
     * @param $value
     * @param $rowNumber
     * @param $recordId
     * @param $rowData
     * @return string
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $podepsat = "";
        $zmenovaCast = "";

        $obdobi = $this->database->stringToTimestamp($rowData['col-rok']);

        $zvolenyPPID = $this->browser->filterForm->getFilterValue('osoba');

        $aktualniRok = date('Y', time());
        $rokRVD = date('Y', $obdobi);

        if ($obdobi > 0) {
            $pracPomeryOsoby = PracovniPomer::nactiPPOsoby($this->database, $rowData['col-rc'], $obdobi, DateUtils::lastDayOfYear($obdobi));

            $platnyPP = false;

            foreach ($pracPomeryOsoby as $pracovniPomer) {
                if ($pracovniPomer->zamestnanec == 1) {
                    $platnyPP = true;
                }
            }

            if (!$platnyPP) {
                return TextManager::_('Neexistuje_platny_pp');
            }

            $podepsat .= "<a href='?menuid=dane&action=formularprohlaseni&osoba=$zvolenyPPID&rok=$rokRVD' >";

            //Pro praci s elektronickym podpisem prohlaseni na portale.
            //Schvalene prohlaseni provede uzamceni zmen na portale a prepsani datumu prohlasení datem z portalu.
            //pokud bude prohlaseni schvalene ucetni (1,2), pak umoznit pouze nahled jiz ne editaci,
            // v pripade chyby (3) se musi provest korekce mzdovym pracovnikem
            //Pokud nebude povoleno vyplneni elektronicky na karte zamestnance, pak umoznit pouze "Nahled"
            if (($rowData['col-prohlaseni-portal-stav'] != 1) && ($rokRVD == $aktualniRok)) {

                if (($rowData['col-prohlaseni-portal-stav'] == 4) && ($rowData['col-prohlaseni-datum-portal'] != "") && ($rowData['col-prohlaseni-elektronicky'] == 1)) {

                    if (RonPortalSettings::getIntValue('dane_editovat_podepsane_prohlaseni')) {
                        if ($zvolenyPPID == $rowData['col-pp-id']) {
                            //Ma podepsano, a ve filtru ma vybrany stejny pp
                            $podepsat .= "<span class='font-weight-bold'>" . TextManager::_("Mzdy_editovat_prohlaseni") . "</span>";
                        } else {
                            //Ma sice podepsano, ale ve filtru ma vybrany jiny pp, proto se musi upravit popis odkazu
                            $podepsat .= "<span class='font-weight-bold'>" . TextManager::_("Prohlaseni_zmenit_pracovni_pomer_na_prohlaseni") . "</span>";
                        }
                    } else {
                        //Ma podepsano, a ve filtru ma vybrany stejny pp
                        $podepsat .= "<span class='font-weight-bold'>" . TextManager::_("Prohlaseni_podepsano") . "</span>";
                    }

                } elseif ($rowData['col-prohlaseni-datum-portal'] == "" && $rowData['col-prohlaseni-elektronicky'] == 1) {
                    $podepsat .= "<span class='font-weight-bold'>" . TextManager::_("Mzdy_podepsat_prohlaseni") . "</span>";
                } elseif ($rowData['col-prohlaseni-portal-stav'] == 2 && $rowData['col-prohlaseni-elektronicky'] == 1) {
                    $podepsat .= "<span class='font-weight-bold'>" . TextManager::_("prohlaseni_vraceno") . "</span>";
                } elseif ($rowData['col-prohlaseni-portal-stav'] == 0  && $rowData['col-prohlaseni-elektronicky'] == 0) {
                    //Je v navrhu (jeste neschvalen, ale neni povoleno vyplnit na portale)
                    $podepsat .= TextManager::_('Mzdy_elektronicky_podepsat_neni_povoleno_mzdovym_pracovnikem');
                } else {
                    //Overeni zda lze vyplnit na portale jinak pouze detail
                    $podepsat .= "<span>" . TextManager::_("Mzdy_nahled") . "</span>";
                }

            } else {

                $podepsat .= "<span>" . TextManager::_("Mzdy_nahled") . "</span>";
            }

            $podepsat .= "</a>";
        }

        //u schvaleneho prohlaseni pro aktualni rok, umozni zobrazeni zmenoveho formulare
        if ($obdobi > 0 && $aktualniRok == $rokRVD && $rowData['col-prohlaseni-elektronicky'] == 1 && $rowData['col-prohlaseni-portal-stav'] == 1) {
            $zmenovaCast .= "<a href='?menuid=dane&action=formularprohlaseni&osoba=$zvolenyPPID&rok=$rokRVD&zmenovacast=1' >";

            $zmenovaCast .= "<span>" . TextManager::_("Prohlaseni_zmenova_cast") . "</span>";

            $zmenovaCast .= "</a>";
        }

        if (($zmenovaCast != "") && ($zvolenyPPID == $rowData['col-pp-id'])) {
            $result = $podepsat . "&nbsp;/&nbsp;" .  $zmenovaCast;
        } else {
            $result = $podepsat;
        }

        return $result;
    }

}

class VyuctovaniBrowserField extends BrowserField
{
    /** @var RocniVyuctovaniDaneBrowser */
    private $browser;

    public function __construct($database, $browser = null)
    {
        parent::__construct($database);
        if (!is_null($browser)) {
            $this->browser = $browser;
        }
    }

    function getDataCellClass($value, $rowNumber, $recordId, $rowData)
    {
        $minulyRok = (int)date('Y', time()) - 1;
        $rokRVD = date('Y', $this->database->stringToTimestamp($rowData['col-rok']));
        $class = '';
        if ($minulyRok == $rokRVD) {
            switch ($rowData['col-zadost-stav-info']) {
                case RocniVyuctovaniDaneRecord::ZADOST_SCHVALENO:
                    $class = 'zadost_podepsano_uzivatel';
                    break;
                case RocniVyuctovaniDaneRecord::ZADOST_SCHVALENO_UCETNI:
                    $class = 'zadost_schvaleno';
                    break;
                case RocniVyuctovaniDaneRecord::ZADOST_VRACENO:
                    $class = 'zadost_vraceno';
                    break;
                case RocniVyuctovaniDaneRecord::ZADOST_NAVRH:
                default:
                    $class = 'zadost_navrh';
                    break;
            }
        }

        return parent::getDataCellClass($value, $rowNumber, $recordId, $rowData) . " " . $class;
    }

    /**
     * @param $value
     * @param $rowNumber
     * @param $recordId
     * @param $rowData
     * @return string
     * @throws Exception
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = "";
        $zadam = "";

        $rokVyuctovani = $this->database->stringToTimestamp($rowData['col-rok']);
        $rokStr = date('Y', $rokVyuctovani);
        $zvolenyPPID = $this->browser->filterForm->getFilterValue('osoba');

        if ($rokVyuctovani >= DateUtils::firstDayOfYear(time()) and !$GLOBALS['debug_dane']) {
            return TextManager::_('zadost_rok_editovatelny_v_budoucnu');
        }

        //pokud neni podepsane prohlaseni na dany rok, neumoznit editaci
        /*if ($rowData['col-prohlaseni-datum'] == '') {
            return TextManager::_('zadost_neni_podepsano_prohlaseni');
        }*/

        if ($rokVyuctovani > 0) {
            $result .= "<a href='?menuid=dane&action=formularzadosti&rok=$rokStr&osoba=$zvolenyPPID' >";

            $lzeEditovat = DaneController::lzeEditovatVyuctovani($rokVyuctovani, $rowData['col-zadost-portal-stav']);

            //pokud bude zadost schvalena ucetni, pak umoznit pouze nahled jiz ne editaci, editace je mozna ve stavu navrh = 0 nebo vraceno = 2
            if ((($rowData['col-zadost-portal-stav'] == 0) || ($rowData['col-zadost-portal-stav'] == 2)) && $rowData['col-zadost-elektronicky'] == 1 /*musi byt povoleno zpracovani na portale*/ && $rowData['col-schvaleno'] == 0 /*jen pokud neni schvaleno*/ && $lzeEditovat) {
                $lzeMenitZadost = true;

                if ($rowData['col-provest-vyuctovani'] == 1) { //je vyzadovano vuctovani
                    $result .= "<span class='bold'>" . TextManager::_("Mzdy_vyplnit") . "</span>";
                } else {
                    $result .= "<span>" . TextManager::_("Mzdy_nahled") . "</span>";
                }
            } else {
                $lzeMenitZadost = false;
                $result .= "<span>" . TextManager::_("Mzdy_nahled") . "</span>";
            }

            $result .= "</a>";

            switch ($rowData['col-provest-vyuctovani']) {
                //Stav zadosti na Portale
                case '0':
                    if ($lzeMenitZadost) {
                        $url = "<a href='?menuid=dane&action=rocnizuctovani&rok=$rokStr&osoba=$zvolenyPPID&zadamzuctovani=1" . Csrf::csrfUrlParameter('provestvyuctovani_' . $zvolenyPPID) . "'>" . TextManager::_('zadost_zadam') . "</a>";
                        $zadam .= "<span class=''>" . $url . "</span>";
                        $zadam .= " / ";

                        //u neschvalene se misto odkazu na zobrazeni formulare zobrazi odkaz na podepsani, pokud nezada o rocni vyuctovani
                        if ($rowData['col-zadost-portal-stav'] == 0) {
                            $result = "<span class='js confirmQuestion' data-ron-confirm-text='" . sprintf(TextManager::_('zadost_potvrzeni_nezadam'), $rokStr) . "' 
                            data-ron-href='?menuid=dane&action=rocnizuctovani&rok=$rokStr&osoba=$zvolenyPPID&zadamzuctovani=0" . Csrf::csrfUrlParameter('provestvyuctovani_' . $zvolenyPPID) . "'>" .
                                TextManager::_('zadost_podepsat_bez_editace') . "</span>";
                        }
                    }


                    if ($rowData['col-vystavit-potvrzeni-prijmu'] == 1) {
                        if ($rowData['col-potvrzeni-prijmu-vystaveno'] == 1) {
                            $params = $this->database->createParameters();
                            $sql =
                                "SELECT Dokument.id FROM DokumentyOsoby JOIN Dokument ON Dokument.id = DokumentyOsoby.id " .
                                "WHERE RC = " . $params->addParameter(User::getRC()) .
                                " AND VyuctovaniDane_RC = " . $params->addParameter(User::getRC()) .
                                " AND VyuctovaniDane_Rok = " . $params->addDateParameter($rokVyuctovani) .
                                " AND Dokument.typ = " . $params->addParameter(DaneController::TYP_DOKUMENTU_POTVRZENI_PRIJMU);
                            $idDokumentu = DBUtils::getSqlValue($this->database, $sql, $params);

                            if ($idDokumentu) {
                                $result = "<a href='?menuid=dane&action=dokumentdownload&rok=$rokStr&osoba=$zvolenyPPID&record=$idDokumentu" . Csrf::csrfUrlParameter('dokumentdane_' . $zvolenyPPID) . "'>" . TextManager::_('Stahnout_prilohu') . "</a>";
                            }
                            //TODO: pokud bude existovat potvrzeni, zobrazit odkaz ke stazeni dokumentu
                        }
                    } elseif ($lzeEditovat) {
                        //TODO: proklik pro nastaveni priznaku k vygenerovani potvrzeni
                        $result = "<a href='?menuid=dane&action=potvrzeniprijmu&rok=$rokStr&osoba=$zvolenyPPID&generovatpotvrzeni=1" . Csrf::csrfUrlParameter('potvrzeni_' . $zvolenyPPID) . "'>" .
                            TextManager::_('zadost_generovat_potvrzeni') . "</a>";
                    }


                    //Nepozaduje rocni vyuctovani dane
                    $zadam .= "<span class=''>" . TextManager::_('zadost_nezadam') . "</span>&nbsp;-&nbsp;" . $result;

                    break;

                case '1':
                    //Pozaduje provest rocni vyuctovani
                    $zadam .= "<span class='font-weight-bold'>" . TextManager::_('zadost_zadam') . "</span>";

                    $zadam .= "&nbsp;-&nbsp;$result";

                    if ($lzeMenitZadost) {
                        $zadam .= " / ";
                        $url = "<span class='js confirmQuestion' data-ron-confirm-text='" . sprintf(TextManager::_('zadost_potvrzeni_nezadam'), $rokStr) . "' 
                            data-ron-href='?menuid=dane&action=rocnizuctovani&rok=$rokStr&osoba=$zvolenyPPID&zadamzuctovani=0" . Csrf::csrfUrlParameter('provestvyuctovani_' . $zvolenyPPID) . "'>" .
                            TextManager::_('zadost_nezadam') . "</span>";
                        $zadam .= "<span class=''>" . $url . "</span>";
                    }
                    break;
            }
        }

        return ($zadam ? "<span class='align-left'>$zadam</span>" : '');
    }

}

class ProhlaseniStavBrowserField extends ProhlaseniBrowserField
{
    public function __construct($database, $browser)
    {
        parent::__construct($database, $browser);
    }

    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $zmenovaCast = '';

        $obdobi = $this->database->stringToTimestamp($rowData['col-rok']);

        $zvolenyPPID = $this->browser->filterForm->getFilterValue('osoba');

        $aktualniRok = date('Y', time());
        $rokRVD = date('Y', $obdobi);

        //u schvaleneho prohlaseni pro aktualni rok, umozni zobrazeni zmenoveho formulare
        if ($obdobi > 0 && $aktualniRok == $rokRVD && $rowData['col-prohlaseni-elektronicky'] == 1 && $rowData['col-prohlaseni-portal-stav'] == 1 && $zvolenyPPID == $rowData['col-pp-id']) {
            $zmenovaCast = "&nbsp;/&nbsp;" .  TextManager::_("prohlaseni_zmena_stav_{$rowData['col-prohlaseni-portal-zmena-stav']}");
        }

        return "<span>" . TextManager::_("prohlaseni_stav_{$value}") . ($zmenovaCast ?: '') . "</span>";
    }
}

class VyuctovaniStavBrowserField extends VyuctovaniBrowserField
{
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        return TextManager::_("zadost_stav_{$value}");
    }
}

class RocniVyuctovaniDaneBrowser extends RonPortalBrowser
{
    private $rok;

    public function __construct($database, $rok)
    {
        parent::__construct($database, 'dane');

        $this->rok = $rok;

        $this->title = TextManager::_('prohlaseni_zadost_poplatnika_dane');

        $this->code = 'browserdane';

        $this->sqlFromPart =
            "RocniVyuctovaniDane 
            LEFT OUTER JOIN PracovniPomer ON (RocniVyuctovaniDane.ProhlaseniPortalRC = PracovniPomer.RC 
                AND RocniVyuctovaniDane.ProhlaseniPortalOsc = PracovniPomer.OsobniCislo 
                AND RocniVyuctovaniDane.ProhlaseniPortalDatNastup = PracovniPomer.DatumNastupu )";

        $this->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('RocniVyuctovaniDane.ProhlaseniElektronicky')->setCode('col-prohlaseni-elektronicky');
        $this->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('RocniVyuctovaniDane.DatumProhlaseniPortal')->setCode('col-prohlaseni-datum-portal');
        $this->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('RocniVyuctovaniDane.DatumProhlaseni')->setCode('col-prohlaseni-datum');
        $this->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('RocniVyuctovaniDane.ProhlaseniPortal')->setCode('col-prohlaseni-portal');
        $this->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('RocniVyuctovaniDane.ProhlaseniPortalStav')->setCode('col-prohlaseni-portal-stav');
        $this->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('RocniVyuctovaniDane.ProhlaseniPortalZmenaStav')->setCode('col-prohlaseni-portal-zmena-stav');
        $this->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('RocniVyuctovaniDane.ZadostElektronicky')->setCode('col-zadost-elektronicky');
        $this->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('RocniVyuctovaniDane.ZadostPortalStav')->setCode('col-zadost-portal-stav');
        $this->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('RocniVyuctovaniDane.Schvaleno')->setCode('col-schvaleno');
        $this->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('RocniVyuctovaniDane.ProvestRocniVyuctovani')->setCode('col-provest-vyuctovani');
        $this->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName("COALESCE({fn_castwidememo(RocniVyuctovaniDane.PoznamkaProZamestnance)}, '" . RonPortalSettings::getValue('') . "')")->setCode('col-prohlaseni-poznamka');
        $this->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName("COALESCE({fn_castwidememo(RocniVyuctovaniDane.PoznamkaProZamZadost)}, '" . RonPortalSettings::getValue('') . "')")->setCode('col-zadost-poznamka');
        $this->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('PracovniPomer.ID')->setCode('col-pp-id');
        $this->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('RocniVyuctovaniDane.RC')->setCode('col-rc');

        $this->addField()
            ->setSqlFieldName('RocniVyuctovaniDane.Rok')->setFormat('year')->setHeaderText('rok')->setAlign('align-center')->setOrdering(true)
            ->setCode('col-rok');

        $this->addField(new DatumProhlaseniBrowserField($this->database))
            ->setClassName('align-center')->setHeaderText('Prohlaseni_podepsano_dne')->setCode('col-datum-prohlaseni');

        $this->addField(new ProhlaseniElektronickyBrowserField())
            ->setHeaderText('prohlaseni_zpusob')
            ->setClassName('align-center')->setCode('col-prohlaseni-portal-text');

        $this->addField(new PracovniPomerProhlaseniBrowserField($this->database, $this))
            ->setClassName('font-weight-bold')->setAlign('center')->setHeaderText('prohlaseni_pracovni_pomer')->setCode('col-pp-info');

        $this->addField(new ProhlaseniBrowserField($this->database, $this))
            ->setClassName('align-center')->setHeaderText('formular_prohlaseni')->setCode('col-prohlaseni');

        $this->addField(new ProhlaseniStavBrowserField($this->database, $this))
            ->setSqlFieldName('RocniVyuctovaniDane.ProhlaseniPortalStav')
            ->setClassName('align-center')->setHeaderText('prohlaseni_stav')->setCode('col-prohlaseni-stav-info');
        
        if (Prava::jsouNejakePrava('dane_vyuctovani')) {
            $this->addField(new VyuctovaniBrowserField($this->database, $this))
                ->setHeaderText('Mzdy_formular_zadosti')->setCode('col-zuctovani')->setHeaderClass('align-center')->setClassName('min-width-200px');

            $this->addField(new VyuctovaniStavBrowserField($this->database))
                ->setSqlFieldName('RocniVyuctovaniDane.ZadostPortalStav')
                ->setClassName('align-center')->setHeaderText('zadost_stav')->setCode('col-zadost-stav-info');

            $this->addField(BrowserField::FIELD_TYPE_HIDDEN)
                ->setSqlFieldName('RocniVyuctovaniDane.PotvrzeniPrijmuVystavit')->setCode('col-vystavit-potvrzeni-prijmu');
            $this->addField(BrowserField::FIELD_TYPE_HIDDEN)
                ->setSqlFieldName('RocniVyuctovaniDane.PotvrzeniPrijmuVystaveno')->setCode('col-potvrzeni-prijmu-vystaveno');
        }

        $this->defaultSort = 'col-rok';
    }

    function setFilterForm()
    {
        $this->filterForm = new RonPortalFilterForm($this ->database, $this ->action, true);

        $filtrPP = $this ->filterForm ->addFilterPP(false, Prava::vratPrava('dane_prohlaseni'));

        $params = $this ->database ->createParameters();
        //$filtrPP ->setOptionsSqlFilter(new SqlStatement('Osoba.RC = ' . $params ->addParameter(User::getRC()), $params));
        $filtrPP ->setOptionsSqlFilter("Osoba.RC = '" . $this ->database ->addSlashes(User::getRC()) . "'");

//        $this->filterForm->nastavOsoby(true, Prava::vratPrava('dane_prohlaseni'), DateUtils::incMonth($this->rok, -12), DateUtils::incDay(DateUtils::incMonth($this->rok, 12), -1));
//        //Na prihlasenou osobu
//        $this->filterForm->osobyFiltr = "Osoba.RC = '" . User::getRC() . "'";

    }

    function renderFilter()
    {
        parent::renderFilter();
        echo "<div class='dane_info_seznam'>" . TextManager::_('Mzdy_dane_info_nad_seznamem') . "</div>";
    }

    public function getSQLFilter()
    {
        $params = $this->database->createParameters();
        $result =
            "RocniVyuctovaniDane.RC = (Select RC From PracovniPomer Where ID = " . $params->addParameter($this->filterForm->getFilterValue('osoba')) . ") " .
            " AND {fn_YEAR(RocniVyuctovaniDane.Rok)} >= " . $params->addParameter(RonPortalSettings::getIntValue('dane_pocatecni_rok', 2020));

        $formFilter = $this ->filterForm ->buildSqlFilter($params);
        if ($formFilter) {
            $result .=
                ($result ? ' AND ' : '') .
                $formFilter;
        }

        return new SqlStatement($result, $params);
    }

    public function renderLine($values, $className, $rowNumber, $edit = false)
    {
        parent::renderLine($values, $className, $rowNumber, $edit);

        $rokProhlaseni = date("Y", $this->database->stringToTimestamp($values['col-rok']));
        $aktualniRok = date("Y", time());

        //Pokud nelze vyplnit prohlaseni na portale, tak tuto informaci uzivateli uvidi v poznamce
        if ($aktualniRok == $rokProhlaseni && $values['col-prohlaseni-elektronicky'] == 0) {
            $telefonMzdovyPracovnik = RonPortalSettings::getValue('danekontaktnamzdoveho');

            $kontakt = $telefonMzdovyPracovnik ? sprintf(TextManager::_('Mzdy_kontakt_mzdovy_pracovnik'), "\n" . $telefonMzdovyPracovnik) : '';

            echo "<tr class='$className'>";
            echo '<td></td><td colspan="' . ($this->columnCount - 1) . '">';
            echo '<span class="browserpoznamka">' . TextManager::_('Prohlaseni_poznamka') . TextManager::_('Mzdy_elektronicky_podpis_zrusen') . " " . $kontakt . '</span>';
            echo '</tr>';
        }

        if (($values['col-prohlaseni-portal-zmena-stav'] == 2 || $values['col-prohlaseni-portal-stav'] == 2)) {
            echo "<tr class='$className'>";
            echo '<td></td><td colspan="' . ($this->columnCount - 1) . '">';
            echo '<span class="browserpoznamka font-weight-bold">' . TextManager::_('Prohlaseni_poznamka_mzdoveho_pracovnika') . ": " . '&nbsp;&nbsp;&nbsp;&nbsp;' . StringUtils::htmlTextConvert($values['col-prohlaseni-poznamka']) . '</span>';
            echo '</tr>';
        }

        if ($values['col-zadost-portal-stav'] == 2) {
            echo "<tr class='$className'>";
            echo '<td></td><td colspan="' . ($this->columnCount - 1) . '">';
            echo '<span class="browserpoznamka font-weight-bold">' . TextManager::_('Prohlaseni_poznamka_mzdoveho_pracovnika') . ": " . '&nbsp;&nbsp;&nbsp;&nbsp;' . StringUtils::htmlTextConvert($values['col-zadost-poznamka']) . '</span>';
            echo '</tr>';
        }
    }


}