<?php
/**
 * stanislav.elbl, 25.02.2021
 */

namespace RonPortal\Mzdy;

use Ron\Controls\Browser;
use Ron\DB\Database;
use Ron\DB\SqlStatement;
use Ron\TextManager;
use RonPortal\Controls\RonPortalBrowser;
use RonPortal\PracovniPomer;
use RonPortal\RonPortalSession;

class SrazkyBrowser extends RonPortalBrowser
{
    /** @var PracovniPomer */
//    private $pp;

    /**
     * @param Database $database
     * @param PracovniPomer $pp
     */
    function __construct($database, $pp)
    {
        parent::__construct($database, 'mzdoveudaje');

//        $this ->pp = $pp;

        $this ->title = TextManager::_('Srazky_ze_mzdy');
        $this ->code = 'browsersrazky';

        // definice dotazu a hlavicky
        $this -> sqlFromPart =
            "(NastaveniSrazky LEFT OUTER JOIN ProvedenaSrazka ".
            " ON ProvedenaSrazka.KodSrazky = NastaveniSrazky.KodSrazky ".
            " AND ProvedenaSrazka.SerioveCislo = NastaveniSrazky.SerioveCislo )".
            " INNER JOIN TypSrazky ON NastaveniSrazky.KodSrazky = TypSrazky.Kod ";


        $params = $this ->database ->createParameters();
        $this ->sqlWherePart = new SqlStatement(
            ' NastaveniSrazky.PlatnostOd <= ' . $params ->addDateParameter(time()) .
            ' AND ( NastaveniSrazky.PlatnostDo >= ' . $params ->addDateParameter(time()) . ' OR NastaveniSrazky.PlatnostDo IS NULL ) ' .
            ' AND NastaveniSrazky.RC = ' . $params ->addParameter($pp ->rc) .
            ' AND NastaveniSrazky.OsobniCislo = ' . $params ->addParameter($pp ->osobniCislo) .
            ' AND NastaveniSrazky.DatumNastupu = ' . $params ->addDateParameter($pp ->datumNastupu),
            $params);

        $this -> sqlGroupByPart = true;


        $this ->addField()
            ->setSqlFieldName('TypSrazky.Popis') ->setHeaderText('Typ_srazky')
            ->setCode('col-typsrazky');

        $this -> defaultSort = 'col-typsrazky';

        $this ->addField()
            ->setSqlFieldName('NastaveniSrazky.PlatnostOd') ->setHeaderText('Platnost_od') ->setFormat('date')
            ->setCode('col-platnostod');

        $this ->addField()
            ->setSqlFieldName('NastaveniSrazky.PlatnostDo') ->setHeaderText('Platnost_do') ->setFormat('date')
            ->setCode('col-platnostdo');

        $this ->addField()
            ->setSqlFieldName('NastaveniSrazky.CastkaCelkova') ->setHeaderText('Srazka_celkem') ->setFormat('curr') ->setAlign('right')
            ->setCode('col-castkacelkova');

        $this ->addField()
            ->setSqlFieldName('NastaveniSrazky.CastkaMesicni') ->setHeaderText('Srazka_mesicni') ->setFormat('curr') ->setAlign('right')
            ->setCode('col-castkamesicni');

        $this ->addField()
            ->setSqlFieldName('SUM(ProvedenaSrazka.SrazenaCastka)') ->setHeaderText('Jiz_srazeno') ->setFormat('curr') ->setAlign('right')
            ->setCode('col-srazenacastka')
            ->setExcludeFromGroupBy(true);
    }


    /**
     * Zabrani zobrazeni filtru
     */
    function setFilterForm()
    {
        // nedela nic - filtr se nepouziva
    }

}
