<?php
/**
 * roman.Szablatura 22.11.2016
 */

namespace RonPortal\Napady;

use Exception;
use Ron\Controls\BrowserField;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\SqlStatement;
use Ron\GuiUtils;
use Ron\Http\HttpHeaders;
use Ron\TextManager;
use RonPortal\Controls\FilterFormFilter;
use RonPortal\Controls\FilterFormFilterOsoba;
use RonPortal\Controls\JmenoPrijmeniBrowserField;
use RonPortal\Controls\RonPortalBrowser;
use RonPortal\Controls\RonPortalFilterForm;
use RonPortal\Prava;
use RonPortal\PravaNaOsoby;
use RonPortal\User;


class PrirazeniPilotaKNapadu extends JmenoPrijmeniBrowserField
{
    /** @var  PravaNaOsoby */
    private $jePravoAdmin;

    /**
     * PrirazeniPilotaKNapadu constructor.
     * @param null|Database $database
     */
    function __construct($database)
    {
        parent::__construct($database, 'col-pilot-jmeno', 'col-pilot-prijmeni', 'col-pilot-titul');
        $this->jePravoAdmin = Prava::jsouNejakePrava('napady_admin');
    }

    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $text = parent::getViewText($value, $rowNumber, $recordId, $rowData);
        if (trim($text)) {
            return $text;
        }

        if ($this->jePravoAdmin) {
            return '<a id="odkaz_pilot" href="?menuid=napady&formaction=edit&record=' . $rowData[0] . '">' . TextManager::_('prirazeni_pilota') . '</a>';
        }

        return '';
    }
}


/**
 * Class BarvaNapadu
 * @package RonPortal\Napady
 */
class BarvaNapadu extends BrowserField
{
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        return
            "<i class='inline-glyph fa glyph-napad glyph-napad-{$rowData[3]}'></i>";
    }
}


/**
 * Class PocetDokumentuNapaduField
 * @package RonPortal\Napady
 */
class PocetDokumentuNapaduField extends BrowserField
{
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if ($value) {
            return '<i class="fa inline-glyph glyph-attachment" title="' . TextManager::_('Pocet_priloh') . ': ' . $value . '"></i>';
        }
        return '';
    }
}


/**
 * Class NapadyBrowser
 * @package RonPortal\Napady
 */
class NapadyBrowser extends RonPortalBrowser
{
    /** @var  PravaNaOsoby */
    var $prava;

    /** @var  PravaNaOsoby */
    var $prava_admin;

    /** @var string[] */
    private $colorsCSS = [];


    /**
     * NapadyBrowser constructor.
     * @param Database $database
     * @throws Exception
     */
    function __construct( $database)
    {
        parent::__construct($database, 'napady');

        $this->prava = Prava::vratPrava('napady');
        $this->prava_admin = Prava::vratPrava('napady_admin');

        Prava::nastavObdobiProPrava(time(),time());

        $this->title = TextManager::_('evidence_napadu');
        $this->code = 'browsernapady';
        $this->tableId = 'browsernapady';
        $this->browserUrl = '?menuid=napady';

        $this->sqlSelectPart =
            'Napad.ID, Autor.RC, Pilot.RC,' .       // 0 - 2 (primarni klic pro odkazy, rc pro kontrolu prav)
            ' StavNapadu.Kod, StavNapadu.Barva';    // 3,4   (pro zobrazeni barevnych ikon)

        $this->sqlFromPart =
            'Napad LEFT JOIN Osoba Autor ON (Napad.Autor = Autor.RC)' .
            ' LEFT JOIN Osoba Pilot ON (Napad.Pilot = Pilot.RC)' .
            ' LEFT JOIN StavNapadu ON (Napad.Stav = StavNapadu.Kod)' .
            ' LEFT JOIN OblastNapadu ON (Napad.Oblast = OblastNapadu.Kod)';

        $this->primaryField = 0;

        $this->addField(new BarvaNapadu($this->database))
            ->setCode('col-icon');

        $this->addField()
            ->setSqlFieldName('Napad.Datum') ->setHeaderText('datum_vlozeni')
            ->setCode('col-datum')
            ->setFormat('finedate') ->setOrdering(true) ->setAlign('center');
        // Seradit dle data
        $this->defaultSort = 'col-datum';

        $this->addField()
            ->setSqlFieldName('StavNapadu.Popis') ->setHeaderText('stav_napadu')
            ->setCode('col-stav')
            ->setOrdering(true) ->setLink('?menuid=napady&action=zaznam');

        // Autor
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Autor.Jmeno')
            ->setCode('col-autor-jmeno');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Autor.Prijmeni')
            ->setCode('col-autor-prijmeni');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Autor.Titul')
            ->setCode('col-autor-titul');

        $this->addField(new JmenoPrijmeniBrowserField($this ->database, 'col-autor-jmeno', 'col-autor-prijmeni', 'col-autor-titul'))
            ->setHeaderText('autor')
            ->setOrdering(true)
            ->setSortSQL('Autor.Prijmeni, Autor.Jmeno') ->setSortSQLDesc('Autor.Prijmeni DESC, Autor.Jmeno DESC')
            ->setCode('col-autor');

        // Oblast napadu s kodem
        $this->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Napad.Oblast')
            ->setCode('col-oblast');

        $this->addField()
            ->setSqlFieldName('OblastNapadu.Popis') ->setSqlFieldIndex2('col-oblast')
            ->setCode('col-popisoblasti')
            ->setOrdering(true) ->setHeaderText('oblast_napadu');

        $this->addField()
            ->setSqlFieldName('Nazev') ->setHeaderText('nazev_napadu')
            ->setCode('col-nazev')
            ->setOrdering(true) ->setLink('?menuid=napady&action=zaznam');

        // Pilot
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Pilot.Jmeno')
            ->setCode('col-pilot-jmeno');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Pilot.Prijmeni')
            ->setCode('col-pilot-prijmeni');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Pilot.Titul')
            ->setCode('col-pilot-titul');

        $this ->addField(new PrirazeniPilotaKNapadu($this ->database))
            ->setCode('col-pilot')
            ->setHeaderText('pilot')
            ->setOrdering(true)
            ->setSortSQL('Pilot.Prijmeni, Pilot.Jmeno') ->setSortSQLDesc('Pilot.Prijmeni DESC, Pilot.Jmeno DESC');

//        $this->addField(new PrirazeniPilotaKNapadu($this->database, $this->prava_admin))
//            ->setSqlFieldName("Coalesce (Pilot.Prijmeni, '') {fn_concat()} ' ' {fn_concat()} Coalesce (Pilot.Jmeno, '') {fn_concat()} Coalesce (', ' {fn_concat()}  Pilot.Titul,'') Pilot")
//            ->setSortSQL('Pilot.Prijmeni, Pilot.Jmeno') ->setSortSQLDesc('Pilot.Prijmeni DESC, Pilot.Jmeno DESC')
//            ->setOrdering(true) ->setHeaderText('pilot')
//            ->setCode('col-pilot');

        // Priorita
        $this->addField()
            ->setSqlFieldName('Priorita') ->setHeaderText('priorita') ->setOrdering(true)
            ->setCode('col-priorita');

        $this->addField()
            ->setSqlFieldName('Napad.Rozpocet') ->setHeaderText('rozpocet')
            ->setCode('col-rozpocet')
            ->setFormat('number_2') ->setOrdering(true) ->setAlign('right');

        $this->addField()
            ->setSqlFieldName('OdmenaCastka') ->setHeaderText('odmena')
            ->setCode('col-odmena')
            ->setFormat('number_2') ->setOrdering(true) ->setAlign('right');

        $this->addField()
            ->setSqlFieldName('OdmenaVyplacenaVObdobi') ->setHeaderText('obdobi_vyplaceni_odmeny')
            ->setCode('col-obdobiodmeny')
            ->setFormat('finedate') ->setOrdering(true) ->setAlign('center');

        $this->addField(new PocetDokumentuNapaduField())
            ->setSqlFieldName('(SELECT COUNT(*) FROM DokumentNapadu WHERE DokumentNapadu.IDNapadu = Napad.ID) AS PocetDokumentu')
            ->setCode('col-dokumenty')
            ->setSortSQL('PocetDokumentu')
            ->setAlign('center');

        $this->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Napad.Update_UserName')
            ->setCode('col-update_username');

        $this->addField() //new DatumPosledniAktualizaceField($this->database));
            ->setSqlFieldName('Napad.Update_Timestamp') ->setSqlFieldIndex2('col-update_username') ->setHeaderText('datum_editace')
            ->setCode('col-aktualizace')
            ->setFormat('date') ->setOrdering(true) ->setAlign('center');

//        //Pokud vchazim do Browseru mazu seznam ID napadu v session aby se vzdy vytvoril aktualni seznam pri kazde zmene filtru
//        unset($_SESSION['pocetnapadufilru']);
//        unset($_SESSION['seznamnapadu']);
    }


    function setFilterForm()
    {
        $this->filterForm = new RonPortalFilterForm($this->database, 'napady', true);

        $this ->filterForm ->addFilter(
            (new FilterFormFilter('stav', FilterFormFilter::TYPE_OPTIONS))
                ->addItem('', TextManager::_('_vse_'))
                ->setSessionName('filter_stavnapadu')
                ->setCaption(TextManager::_('stav_napadu'))
                ->setOptionsSql(
                    'SELECT Kod, Popis FROM StavNapadu WHERE %FILTER% ORDER BY %ORDER%',
                    '1 = 1', 'Kod')
                ->setSqlFilter('Napad.Stav = ')
        );

        $this ->filterForm ->addFilter(
            (new FilterFormFilter('oblast', FilterFormFilter::TYPE_OPTIONS))
                ->addItem('', TextManager::_('_vse_'))
                ->setSessionName('filter_oblastnapadu')
                ->setCaption(TextManager::_('oblast_napadu'))
                ->setShowCode(true)
                ->setOptionsSql(
                    'SELECT Kod, Popis FROM OblastNapadu WHERE %FILTER% ORDER BY %ORDER%',
                    '1 = 1', 'Popis')
                ->setSqlFilter('Napad.Oblast = ')
        );

        $this ->filterForm ->addFilter(
            (new FilterFormFilterOsoba($this ->filterForm, 'autor'))
                ->addItem('', TextManager::_('_vsichni_'))
                ->setSessionName('filter_autornapadu')
                ->setCaption(TextManager::_('autor'))
                ->setOptionsSqlFilter('EXISTS(SELECT * FROM Napad WHERE Napad.Autor = Osoba.RC)')
                ->setPrava(Prava::vratPrava('napady'))
                ->setDefaultValue('')
        );

        $this ->filterForm ->addFilter(
            (new FilterFormFilterOsoba($this ->filterForm, 'pilot'))
                ->addItem('', TextManager::_('_vsichni_'))
                ->addItem('-', TextManager::_('pilot_neni_urcen'))
                ->setSessionName('filter_pilotnapadu')
                ->setCaption(TextManager::_('pilot'))
                ->setOptionsSqlFilter('EXISTS(SELECT * FROM Napad WHERE Napad.Pilot = Osoba.RC)')
                ->setPrava(Prava::vratPrava('napady'))
                ->setDefaultValue('')
        );

        $this ->filterForm ->addFilter(
            (new FilterFormFilter('priorita', FilterFormFilter::TYPE_OPTIONS))
                ->addItem(null, TextManager::_('_vse_'))
                ->setOptionsSql(
                    'SELECT DISTINCT Napad.Priorita, Napad.Priorita' .
                    ' FROM Napad ' .
                    ' WHERE Napad.Priorita IS NOT NULL AND %FILTER%' .
                    ' ORDER BY %ORDER%', '1 = 1', 'Napad.Priorita')
                ->setSessionName('filter_prioritanapadu')
                ->setCaption(TextManager::_('priorita'))
                ->setSqlFilter('Napad.Priorita = ')
        );
    }


    /**
     * @param array $values
     * @return bool
     * @throws Exception
     */
    function enableLine($values)
    {
        //Zobrazi pokud ma na autora pravo nebo je pilotem napadu
        if ($this->prava->lzeZobrazitRC($values[1]) || $this->prava_admin->lzeZobrazitRC($values[1]) || User::hasRC($values[2])) {
            return true;
        }

        return false;
    }


    /**
     * @return SqlStatement|string
     * @throws DBException
     */
    public function getSQLFilter()
    {
        $result = '';

        $params = $this->database->createParameters();
        $result = $this ->filterForm ->buildSqlFilter($params);

//        if (isset($this ->filterForm ->stavNapadu)){
//            $result .=
//                ($result ? ' AND ' : '') .
//                'StavNapadu.Kod = ' . $params->addParameter($this ->filterForm ->stavNapadu);
//        }

        if ($this ->filterForm ->getFilterValue('autor')) {
            $result .=
                ($result ? ' AND ' : '') .
                ' Napad.Autor = ' . $params->addParameter($this ->filterForm ->getFilterValue('autor', 'rc'));
        }

//        if ($this ->filterForm ->oblastNapadu){
//            $result .=
//                ($result ? ' AND ' : '') .
//                ' Napad.Oblast = ' . $params->addParameter($this ->filterForm ->oblastNapadu);
//        }

        $pilot = $this ->filterForm ->getFilterValue('pilot');
        if ($pilot){
            if ($pilot === '-') {
                $result .=
                    ($result ? ' AND ' : '') .
                    ' Napad.Pilot IS NULL';
            } else {
                $result .=
                    ($result ? ' AND ' : '') .
                    ' Napad.Pilot = ' . $params->addParameter($this ->filterForm ->getFilterValue('pilot', 'rc'));
            }
        }

        return $result
            ? new SqlStatement($result, $params)
            : null;
    }

    public function renderLine($values, $className, $rowNumber, $edit = false)
    {
        parent::renderLine($values, $className, $rowNumber, $edit);

        if (!isset($this ->colorsCSS[$values[3]])  &&  isset($values[4])) {
            $this ->colorsCSS[$values[3]] = $values[4];
        }
    }

    public function renderData()
    {
        echo
            '<div class="margin-top-2">' .
            '<a href="?menuid=napady&formaction=new">' . TextManager::_('zapis_noveho_napadu') . '</a>' .
            '</div>';

        parent::renderData();
    }


    public function render()
    {
        $this ->colorsCSS = [];

        parent::render();

        // styl pro obarveni ikon
        if ($this ->colorsCSS) {
            $nonce = HttpHeaders::getNonce();
            echo "\n<style nonce='$nonce'>";

            foreach ($this ->colorsCSS as $key => $value) {
                echo " .glyph-napad-$key {color:" . GuiUtils::getHtmlColor($value) . ';}';
            }

            echo "</style>\n";
        }
    }


}