<?php
/**
 * roman.Szablatura 21.11.2016
 */

namespace RonPortal\Napady;

use Exception;
use Ron\Controls\ComboBox;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\DBUtils;
use Ron\Http\Csrf;
use Ron\Http\HtmlHeaders;
use Ron\MessageManager;
use Ron\MVC\InternalRedirect;
use Ron\MVC\JsonController;
use Ron\MVC\JsonOutputObject;
use Ron\MVC\OutputObject;
use Ron\MVC\ViewObject;
use Ron\MVC\BaseViewObject;
use Ron\MVC\Model;
use Ron\MVC\Redirect;
use Ron\TextManager;
use RonPortal\Dokumenty\DokumentOutputObject;
use RonPortal\Dokumenty\DokumentUploadForm;
use RonPortal\Dokumenty\DokumentyUtils;
use RonPortal\NeniPravoException;
use RonPortal\Prava;
use RonPortal\PravaNaOsoby;
use RonPortal\RonPortalDBRecord;


/**
 * Class NapadController
 */
class NapadyController extends JsonController
{
    /** @var  Database */
    private $database;

    /** @var  PravaNaOsoby */
    private $prava;

    /** @var  PravaNaOsoby */
    private $prava_admin;


    function __construct(Database $database)
    {
        $this->database = $database;

        $this->prava = Prava::vratPrava('napady');
        $this->prava_admin = Prava::vratPrava('napady_admin');

        Prava::nastavObdobiProPrava(time(), time());
    }


    /**
     * @return Redirect|NapadForm
     * @throws Exception
     */
    function napadForm()
    {
        $form = new NapadForm($this->database);

        try {
            if ($form->handleAction()) {
                return $this->redirectToView($form->id);
            }
        } catch (Exception $e) {
            MessageManager::addErrorMessage($e);
        }

        return $form;
    }


    private function redirectToView($id)
    {
        return new Redirect(null, '?menuid=napady&action=zaznam&record=' . $id );
    }

    /**
     * @param $id
     * @param $spoluautor
     * @return Redirect
     * @throws Exception
     */
    function novySpoluautor($id, $spoluautor)
    {
        if ($spoluautor) {
            $this->jePravoNaNapad($id);

            $params = $this->database->createParameters();
            $osobaRC = DBUtils::getSqlValue($this->database,
                'SELECT RC FROM PracovniPomer WHERE ID = ' . $params->addParameter($spoluautor),
                $params);

            $rec = new RonPortalDBRecord($this->database, 'OsobaNapadu', $this);
            $rec->addIntField('IDNapadu', false, (int)$id);
            $rec->addStringField('RC', false, 15, $osobaRC);

            $rec->insertRecord();
        }

        return $this->redirectToView($id);
    }


    /**
     * @param int $idNapadu
     * @param int $idDokumentu
     * @throws Exception
     */
    private function jePravoNaDokument($idNapadu, $idDokumentu)
    {
        $this->jePravoNaNapad($idNapadu);

        $params = $this->database->createParameters();

        //Kontrola, zdali existuje u dotycneho napadu prohlizeny dokument
        $sql =
            'SELECT DokumentNapadu.ID ' .
            'FROM DokumentNapadu LEFT JOIN Napad ON (DokumentNapadu.IDNapadu = Napad.ID) ' .
            "WHERE DokumentNapadu.IDNapadu = " . $params->addParameter($idNapadu) . " AND DokumentNapadu.ID = " . $params->addParameter($idDokumentu);

        $existujeDokument = DBUtils::getSqlValue($this->database, $sql, $params);

        if (!$existujeDokument) {
            throw new NeniPravoException();
        }
    }


    /**
     * Zapise uzivatelem nahrany dokument do databaze
     * @param JsonOutputObject $output
     * @throws DBException
     * @throws NeniPravoException
     */
    function uploadDokument(JsonOutputObject $output)
    {
        if (!($this->prava->jsouNejakePrava() || $this->prava_admin->jsouNejakePrava())) {
            throw new NeniPravoException();
        }

        $idNapadu = $_POST['IDNapadu'];
        Csrf::validateCSRFPostParam('dokument_napadu_' . $idNapadu);

        // Vlozi dokument do databaze
        $id = DokumentyUtils::vlozDokumentJson($this ->database, 'soubor', (int)$_POST['typ'], $_POST['popis'], $output, $this);

        // Vlozi id dokumentu k zaznamu DokumentNapadu
        $rec = new RonPortalDBRecord($this ->database, 'DokumentNapadu', $this);
        $rec->addField('ID', 'int', true, 0, $id);
        $rec->addField('IDNapadu', 'int', true, 0, (int)$idNapadu);

        $rec->insertRecord();
    }


    /**
     * @param int $idNapadu
     * @param int $idDokumentu
     * @return DokumentOutputObject
     * @throws Exception
     */
    function downloadDokument($idNapadu, $idDokumentu)
    {
        $this->jePravoNaDokument($idNapadu, $idDokumentu);

        return new DokumentOutputObject($this->database,$idDokumentu);
    }


    function nactiSpoluautory($idNapadu)
    {
        $params = $this ->database ->createParameters();
        $cursor = $this ->database ->getSqlResult(
            'SELECT Spoluautor.Jmeno, Spoluautor.Prijmeni, Spoluautor.Titul, Spoluautor.RC, ' .     // 0 - 3
            ' (SELECT MAX(ID) FROM PracovniPomer WHERE PracovniPomer.RC = Spoluautor.RC) IDOsoby' . // 4
            ' FROM OsobaNapadu JOIN Osoba Spoluautor ON (OsobaNapadu.RC = Spoluautor.RC)' .
            ' WHERE OsobaNapadu.IDNapadu = ' . $params ->addParameter($idNapadu),
            $params
        );

        $result = [];

        while ($row = $cursor ->fetch()) {
            $result[] = [
                'jmeno' => $row[0],
                'prijmeni' => $row[1],
                'titul' => $row[2],
                'rc' => $row[3],
                'id' => $row[4]
            ];
        }

        $cursor ->close();

        return $result;
    }

    /**
     * @param int $idNapadu
     * @return BaseViewObject
     * @throws Exception
     */
    private function zobrazDetailNapadu($idNapadu)
    {
        if (!$idNapadu){
            throw new Exception(TextManager::_('napad_neexistuje'));
        }

        $napad = Napad::nactiNapad($this ->database, $idNapadu);
        $this->jePravoNaNapad($napad);

        // dokument upload form
        $dokumentUploadForm = new DokumentUploadForm($this->database,
            '?menuid=napady&action=dokumentupload',
            ['data-update-browser' => '#Dokumenty']
        );
        $dokumentUploadForm ->redirectUrl = '?menuid=napady&action=zaznam&record=' . $idNapadu;
        $dokumentUploadForm ->hiddenFields = [
            'IDNapadu' => $idNapadu,
        ];
        $dokumentUploadForm ->csrfRequestId = 'dokument_napadu_' . $idNapadu;


        //DokumentyNapaduBrowser
        $dokumentBrowserNapadu = new DokumentyNapaduBrowser($this->database,(int)$idNapadu);
        $dokumentBrowserNapadu ->showMessages = false;
        $dokumentBrowserNapadu->load();

        Model::addData('spoluautori', $this ->nactiSpoluautory($idNapadu));

        //comboProSpoluautory
        $comboSpoluautori = $this->comboPridaniSpoluautora($this->database, $idNapadu);

        Model::addData('napad', $napad);
        Model::addData('pravaNapadu', $this->prava);
        Model::addData('pravaNapaduAdmin', $this->prava_admin);
        Model::addData('dokumentUploadForm', $dokumentUploadForm);
        Model::addData('dokumentBrowserNapadu', $dokumentBrowserNapadu);
        Model::addData('comboSpoluautori', $comboSpoluautori);

        return new BaseViewObject('ronportal/napady/view_napad.php');
    }


    /**
     * @param Database $database
     * @param $record
     * @return ComboBox
     * @throws Exception
     */
    function comboPridaniSpoluautora($database, $record)
    {
        $params = $database->createParameters();

        $sql =
            'SELECT ' .
            ' (SELECT MAX(ID) FROM PracovniPomer WHERE PracovniPomer.RC = Spoluautor.RC ' .
            '  AND PracovniPomer.DatumNastupu <= ' . $params->addDateParameter(time()) .
            '  AND (PracovniPomer.DatumUkonceni >= ' . $params->addDateParameter(time()) . ' OR PracovniPomer.DatumUkonceni IS NULL)), ' .
            " COALESCE (Spoluautor.Prijmeni, '') {fn_concat()} ' ' {fn_concat()} COALESCE (Spoluautor.Jmeno, '') {fn_concat()} COALESCE (', ' {fn_concat()}  Spoluautor.Titul,'') Spoluautor\n" .

            " FROM Osoba Spoluautor WHERE (NOT EXISTS (select * from OsobaNapadu where (OsobaNapadu.rc = Spoluautor.rc) and (OsobaNapadu.IDNapadu = " . $params->addParameter($record ) . ")))" .
            " AND (NOT EXISTS (select * from Napad where (Napad.Autor = Spoluautor.rc) and (Napad.ID = " . $params->addParameter($record) . ")))" .
            " AND (NOT EXISTS (select * from Napad where (Napad.Pilot = Spoluautor.rc) and (Napad.ID = " . $params->addParameter($record) . ")))" .
            ' AND EXISTS(SELECT P.RC FROM PracovniPomer P WHERE (P.RC = Spoluautor.RC) AND (P.DatumNastupu <= ' . $params->addDateParameter(time()) . ' ) ' .
            ' AND ((P.DatumUkonceni IS NULL) OR (P.DatumUkonceni >= ' . $params->addDateParameter(time()) . ')) AND (P.Zamestnanec > 0))' .

            ' ORDER BY Spoluautor.Prijmeni, Spoluautor.Jmeno';

        $comboSpoluautori = new ComboBox('comboSpoluautori');
        $comboSpoluautori
            ->setEmptyRow(' ')
            ->setClassName('form-control ronselect')
            ->addHtmlParam('data-columns','text');

        $comboSpoluautori->loadData($database, $sql, null, $params);

        return $comboSpoluautori;
    }

    /**
     * @param int $idNapadu
     * @param int $idDokumentu
     * @return Redirect
     * @throws Exception
     */
    private function smazPripojenyDokument($idNapadu, $idDokumentu)
    {
        $this->jePravoNaDokument($idNapadu, $idDokumentu);

        try {
            //$this->smazDokument($idDokumentu);
            $params = $this ->database ->createParameters();

            DokumentyUtils::smazDokument($this ->database, (int)$idDokumentu, $this, 'DokumentNapadu',
                'IdNapadu = ' . $params ->addParameter($idNapadu), $params);

            MessageManager::addInfoMessage(TextManager::_('napad_odstraneni_dokumentu_ok'));
        } catch (Exception $e) {
            MessageManager::addErrorMessage($e->getMessage());
        }

        return $this->redirectToView($idNapadu);
    }


    /**
     * @param $idNapadu
     * @param $idOsoby
     * @throws DBException
     * @throws Exception
     */
    private function smazSpoluautora($idNapadu, $idOsoby)
    {
        $this->jePravoNaNapad($idNapadu);

        $params = $this->database->createParameters();
        $osobaRC = DBUtils::getSqlValue($this->database,
            'SELECT RC FROM PracovniPomer WHERE ID = ' . $params->addParameter($idOsoby),
            $params);

        $recSpoluautor = new RonPortalDBRecord($this->database, 'OsobaNapadu', $this);
        $recSpoluautor->addIntField('IDNapadu', true);
        $recSpoluautor->addStringField('RC', true);

        $params = $this ->database ->createParameters();
        $recSpoluautor->setWhere(
            'RC = ' . $params ->addParameter($osobaRC) .
            ' AND IDNapadu = ' . $params ->addParameter((int)$idNapadu),
            $params);

        if ($recSpoluautor ->loadRecord()) {
            $recSpoluautor->deleteRecord();
        }
    }


    private function odebratSpoluautora($idNapadu, $idOsoby)
    {
        try {
            $this->smazSpoluautora($idNapadu, $idOsoby);
            MessageManager::addInfoMessage(TextManager::_('odebrani_spoluautora_ok'));
        } catch (Exception $e) {
            MessageManager::addErrorMessage($e->getMessage());
        }

        return $this->redirectToView($idNapadu);
    }

    /**
     * @param int|Napad $napad
     * @throws Exception
     */
    function jePravoNaNapad($napad)
    {
        if (!$napad instanceof Napad) {
            $napad = Napad::nactiNapad($this ->database, (int)$napad);
        }

        if (!$napad ->jePravoNaNapad()) {
            throw new NeniPravoException();
        }
    }

    function handleJsonAction($action, $output)
    {
        switch ($_GET['action'] ?? null) {
            case 'dokumentupload':
                $this->uploadDokument($output);
                break;

            default :
                throw new Exception('Chyba v NapadyController::handleJsonAction - neznama akce');
        }
    }


    /**
     * @param string $action
     * @return BaseViewObject|InternalRedirect|OutputObject|Redirect|ViewObject|DokumentOutputObject|ViewObject|string
     * @throws DBException
     * @throws Exception
     */
    public function handleAction($action)
    {
        if (!($this->prava->jsouNejakePrava() || $this->prava_admin->jsouNejakePrava())){
            throw new NeniPravoException();
        }

        TextManager::getInstance()->loadStrings('napady.ini');
        HtmlHeaders::getInstance()->setTitle(TextManager::_('RON_Portal') . ' - ' . TextManager::_('napady'));


        if (isset($_POST['formaction'])  ||  isset($_GET['formaction'])) {
            // formular pro editaci nebo vlozeni napadu
            return $this ->napadForm();
        }

        switch ($_GET['action'] ?? null) {

            case 'zaznam':
                return $this->zobrazDetailNapadu((int)$_GET['record']);

            case 'dokumentupload':
                return parent::handleAction($action);

            case 'dokumentdownload':
                return $this->downloadDokument((int)$_GET['idnapadu'], (int)$_GET['record']);

            case 'dokumentdelete' :
                return $this->smazPripojenyDokument((int)$_GET['idnapadu'], (int)$_GET['record']);

            case 'novyspoluautor':
                return $this->novySpoluautor((int)$_POST['id_napadu'], (int)$_POST['comboSpoluautori']);

            case 'spoluautordelete' :
                return $this->odebratSpoluautora((int)$_GET['idnapadu'], (int)$_GET['idosoby']);

            default:
                $b = new NapadyBrowser($this->database);
                $b->load();
                return $b;
        }
    }
}
