<?php
/**
 * stanislav.elbl, 29.03.2021
 */

namespace RonPortal\Personalistika;

use Ron\DateUtils;
use Ron\TextManager;

/**
 * Definice objektu vraceneho funkci analyzujRC
 * Class RodneCislo
 * @package RonPortal\Personalistika
 */
class RodneCislo {
    /** @var string     Puvodni analyzovane RC */
    public $rc;

    /** @var int */
    public $datumNarozeni;

    /** @var int        O=neurceno, 1=muz, 2=zena */
    public $pohlavi = 0;

    /** @var bool       Priznak, ze se jedna o ECP (zahranicni zamestnanec) */
    public $ecp = false;

    /** @var string     Text pripadne chyby */
    public $error = null;
}


class PersonUtils
{
    /**
     * Provede analyzu RC podle https://www.cssz.cz/standardni-kontrola-rodneho-cisla-a-evidencniho-cisla-pojistence
     * @param $rc
     * @return RodneCislo
     */
    public static function analyzujRC($rc)
    {
        $result = new RodneCislo();
        $result ->rc = $rc;

        //$rcNumbers = str_replace(['-', '/', ' '], '', $rc); //odstraneni mezer, lomitek, pomlcek

        if (!ctype_digit($rc)) {
            $result ->error = TextManager::_('Rodne_cislo_obsahuje_nepovolene_znaky');
            // jeste zkusi zjistit datum narozeni
        }

        if (strlen($rc) < 6) {
            $result ->error = TextManager::_('Rodne_cislo_je_prilis_kratke');
            return $result;
        }

        $datumStr = substr($rc, 0, 6);
        $koncovka = substr($rc, 6);

        if (!ctype_digit($rc)) {
            $result ->error = TextManager::_('Rodne_cislo_obsahuje_nepovolene_znaky');
            // dale neni co analyzovat
            return $result;
        }

        $rok = (int)substr($datumStr, 0, 2);
        $mesic = (int)substr($datumStr, 2, 2);
        $den = (int)substr($datumStr, 4, 2);

        $len = strlen($rc);

        if ($mesic > 50) {
            $result ->pohlavi = 2;
            $mesic -= 50;
        } else {
            $result ->pohlavi = 1;
        }

        if ($mesic > 20) {
            // v RC muze byt mesic zvyseny o 20. V ECP nesmi.
            $mesic -= 20;
            $pouzeRC = true;
        } else {
            $pouzeRC = false;
        }

        if ($den > 40) {
            $den -= 40;
            $result ->ecp = true;
        }

        if ($len === 9) {
            $rok = (($rok > 53) ? 1800 : 1900) + $rok;
        } else {
            $rok = (($rok > 53) ? 1900 : 2000) + $rok;
        }

        $timestamp = mktime(0, 0, 0, $mesic, $den, $rok);

        // dodatecne zkontroluje, jestli byly zadane spravne hodnoty,
        if ((int)date('d', $timestamp) !== $den  ||  (int)date('m', $timestamp) !== $mesic) {
            $result ->error = TextManager::_('Rodne_cislo_nelze_prevest_na_datum');
            return $result;
        }

        $result ->datumNarozeni = $timestamp;

        if ($len > 10) {
            $result ->error = TextManager::_('Rodne_cislo_je_prilis_dlouhe');
            return $result;
        }

        if ($len < 9) {
            $result ->error = TextManager::_('Rodne_cislo_je_kratke');
            return $result;
        }

        if ($koncovka === '000') {
            $result ->error = TextManager::_('Koncovka_rc_000_nepripustna');
            return $result;
        }

        if ($result ->ecp  &&  $pouzeRC) {
            $result ->error = TextManager::_('Nevalidni_ECP_mesic_plus_20');  // ECP nesmi mit mesic + 20
            return $result;
        }

        if ($result ->ecp  &&  $len === 9  &&  (int)$koncovka < 600) {
            $result ->error = TextManager::_('Nepripustna_koncovka_ecp');
            return $result;
        }

        if ($result ->ecp  &&  $len === 10  &&  (int)$koncovka < 6000) {
            $result ->error = TextManager::_('Nepripustna_koncovka_ecp');
            return $result;
        }

        $rem = (int)substr($rc, 0, -6) % 11 + (int)substr($rc, -6) % 11;
        $rem %= 11;

        if ($rem) {
            global $development;
            $result ->error = TextManager::_('RC_neni_delitelne_11') . ($development ? " (zbytek je $rem)" : '');
            return $result;
        }

        return $result;
    }
}
