<?php

namespace RonPortal;

/*
  PravoNaPP - polozka seznamu pracovnich pomeru zamestnancu
  SeznamPPProPrava
    - obsahuje seznam PP (objekty PravoNaPP) pro konkretni typ rozdeleni a rozsah datumu
    - bud se jedna o podrizena rozdeleni nebo o vlastni

  PPUzivatele
    - pracovni pomer aktualniho uzivatele
    - obsahuje seznamy pracovnich pomeru pro prava
      (kazdy pracovni pomer muze mit jine rozdeleni a jine prava)  

*/


use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\Parameters;
use Ron\DB\DBException;
use Ron\DB\SqlStatement;
use Ron\DB\SQLUtils;
use Ron\DebugException;
use Ron\MessageManager;
use Ron\Profiler;


/**
 * Urcuje pravo uzivatele na konkretni pracovni pomer.
 * Class PravoNaPP
 */
class PravoNaPP
{
    var $idPP;
    var $platnostOd;
    var $platnostDo;

    var $rc; // pro agendy, ktere jsou napojene primo na osobu, napr. ukoly


    function __construct($idpp, $rc, $platnostOd, $platnostDo)
    {
        $this -> idPP = $idpp;
        $this -> rc = $rc;
        $this -> platnostOd = DateUtils::startOfTheDay($platnostOd);
        $this -> platnostDo = $platnostDo;
    }
}


/**
 * Pomocny zaznam pro nacteni seznamu osob na rozdelenich
 * Class RozdeleniOsobyProPrava
 */
class RozdeleniUzivateleProPrava
{
    var $kod;
    var $platnostOd;
    var $platnostDo;
}


/**
 * Seznam pracovnich pomeru, pro vyhledavani prav.
 *   - obsahuje seznam PP (objekty PravoNaPP) pro konkretni typ rozdeleni a rozsah datumu
 *   - bud se jedna o podrizena rozdeleni nebo o vlastni
 */
class SeznamPPProPrava
{
    /** @var int    Typ rozdeleni, pro ktere je nacten seznam p.pomeru */
    var $typRozdeleni;


    /** @var  int   Priznak, jestli se jedna o podrizena rozdeleni (1), primo podrizena (2) nebo vlastni (0) */
    var $podrizena;

    /** @var int    Pocatek obdobi, pro ktere je nacten seznam p.pomeru */
    var $obdobiOd;
    /** @var int    Konec obdobi, pro ktere je nacten seznam p.pomeru */
    var $obdobiDo;


    /** @var bool   Priznak, ze je nacten seznam povolenych osob */
    var $seznamNacten;

    /* @var PravoNaPP[] $list pole nactenych PravoNaPP. Pozor! Polozka muze byt i pole! */
    var $list;


    // Nastavene obdobi pro kontrolu prav
    static $obdobiPravaOd;
    static $obdobiPravaDo;


    /**
     * SeznamPPProPrava constructor.
     * @param int $typrozdeleni
     * @param int $podrizena
     */
    function __construct($typrozdeleni, $podrizena)
    {
        $this->list = [];
        $this->seznamNacten = false;

        $this->typRozdeleni = $typrozdeleni;
        $this->podrizena = $podrizena;
    }


    /**
     * @param PravoNaPP $pravo
     */
    private function pridejPravoNaPP($pravo)
    {
        if (isset($this->list[$pravo ->idPP])) {
            // Polozka uz existuje - vyrobi z ni pole. Podrizeny muze byt v seznamu vicekrat, pokud v prubehu obdobi
            // zmeni rozdeleni a uzivatel na nej ma porad pravo (napr. mam pravo na B+podrizene a uzivatel prejde z B1 do B2)
            if (is_array($this->list[$pravo ->idPP])) {
                $this->list[$pravo ->idPP][] = $pravo;
            } else {
                $this->list[$pravo ->idPP] = [$this->list[$pravo ->idPP], $pravo];
            }

            // "Optimalizace" seznamu prav (zmenseni poctu zaznamu) - lze resit primo v datech skriptem
//                    if (is_array($this->list[$idpp])) {
//                        // uz je to pole - pridam dalsi prvek
//                        $list = $this->list[$idpp];
//                        if (!self::spojPrava($list[count($list) - 1], $pravo)) {
//                            $this->list[$idpp][] = $pravo;
//                        }
//                    } else {
//                        // jeste to neni pole - vyrobim ho
//                        if (!self::spojPrava($this ->list[$idpp], $pravo)) {
//                            $this->list[$idpp] = [$this->list[$idpp], $pravo];
//                        }
//                    }
        } else {
            $this->list[$pravo ->idPP] = $pravo;
        }
    }


    /**
     * @param Database $database
     * @param string $sql
     * @param Parameters $params
     * @param PPUzivatele|null $ppUzivatele  POZOR! Muze byt null!
     * @throws \Exception
     */
    private function nactiSQLPrimo($database, $sql, $params, $ppUzivatele)
    {
        $this->list = [];

        $platnostUzivateleDo = null;  $platnostUzivateleDoStr = null; // obvykle casto se opakujici - zapamatuji si ho
        $platnostUzivateleOdStr = null;

        $cursor = $database ->select($sql, $params);

        while ($row = $cursor ->fetch()) {
            $idpp = (int)$row[0];
            // kontrola datumu
            // - musi byt prunik (platnost PP uzivatele i zamestnance, totez pro rozdeleni)
            $platnostDo = null;
            $platnostOd = $database ->stringToTimestamp($row[2]); // PP.DatumNastupu

            if ($row[3]) {
                $pom = $database ->stringToTimestamp($row[3]); // PP.DatumUkonceni
                if (!$platnostDo || $pom < $platnostDo) {
                    $platnostDo = $pom;
                }
            }

            if ($row[4]) {
                $pom = $database ->stringToTimestamp($row[4]); // RZamestnance.PlatnostOd
                if ($pom > $platnostOd) {
                    $platnostOd = $pom;
                }
            }

            if ($row[5]) {
                $pom = $database ->stringToTimestamp($row[5]); // RZamestnance.PlatnostDo
                if (!$platnostDo || $pom < $platnostDo) {
                    $platnostDo = $pom;
                }
            }

            if ($row[6]) { // RUzivatele.PlatnostOd
                if ($row[6] != $platnostUzivateleOdStr) {
                    $platnostUzivateleOdStr = $row[6];
                    $platnostUzivateleOd = $database->stringToTimestamp($row[6]);
                }
                if ($platnostUzivateleOd > $platnostOd) {
                    $platnostOd = $platnostUzivateleOd;
                }
            }

            if ($row[7]) { // RUzivatele.PlatnostDo
                if ($row[7] != $platnostUzivateleDoStr) {
                    $platnostUzivateleDoStr = $row[7];
                    $platnostUzivateleDo = $database ->stringToTimestamp($row[7]);
                }
                if (!$platnostDo || $platnostUzivateleDo < $platnostDo) {
                    $platnostDo = $platnostUzivateleDo;
                }
            }


//            if ($ppUzivatele  &&  $ppUzivatele ->datumNastupu > $platnostOd) {   // od 01/2024 nekontroluje datum nastupu, ridi se to platnosti rozdeleni - napr. aby mohl novy vedouci schvalovat dochazku zpracovat zaznamy za predchozi mesic atd.
//                $platnostOd = $ppUzivatele ->datumNastupu;
//            }

//            if ($row[9]) {   //od 01/2019 - ukonceni pracovniho pomeru se nekontroluje. Ridi se to pouze platnosti rozdeleni, aby se napriklad daly schvalovat budouci absence
//                $pom = $database ->stringToTimestamp($row[9]); // PPUzivatele.DatumUkonceni
//                if (!$platnostDo || $pom < $platnostDo) $platnostDo = $pom;
//            }

            if ($platnostOd <= $platnostDo || !$platnostDo) {
                // prida zaznam do seznamu
                $pravo = new PravoNaPP($idpp, $row[1], $platnostOd, $platnostDo); // id,rc
                $this ->pridejPravoNaPP($pravo);
            } else {
            //    echo "$idpp - nepouzito! ";
            }
        }

        $cursor ->close();

        $this->seznamNacten = true;
    }


    /**
     * Nacte seznam pracovnich pomeru na vlastnich / podrizenych rozdelenich pro zadany pracovni pomer uzivatele, typ rozdeleni a obdobi.
     * Kontroluje a zapisuje i data platnosti
     * @param Database $database
     * @param PPUzivatele $pp
     * @throws \Exception
     */
    private function nactiPrimoSeznamPP($database, $pp)
    {
        $params = $database ->createParameters(); // od 27.6.2019 - testovano i se starym ODBC driverem a je to funkcni

        $sql =
            'SELECT DISTINCT PP.Id, PP.RC, ' . // 0,1
            '       PP.DatumNastupu, PP.DatumUkonceni, ' . // 2,3
            '       RZamestnance.PlatnostOd, RZamestnance.PlatnostDo, ' . // 4,5
            '       RUzivatele.PlatnostOd, RUzivatele.PlatnostDo '; // 6,7

        if ($this->podrizena === 1) {
            // podrizena rozdeleni
            $sql .=
                ' FROM RozdeleniOsob RUzivatele, HierarchieRozdeleni H, RozdeleniOsob RZamestnance, ' .
                ' PracovniPomer PP, PracovniPomer PPUzivatele ' .
                "\n" .
                ' WHERE RUzivatele.KodTypuRozdeleni = ' . $params ->addParameter($this->typRozdeleni) .
                '   AND RZamestnance.KodTypuRozdeleni = ' . $params ->addParameter($this->typRozdeleni) .
                '   AND H.TypNadrazene = RUzivatele.KodTypuRozdeleni ' .
                '   AND H.KodNadrazene = RUzivatele.Rozdeleni ' .
                '   AND H.TypPodrizene = RZamestnance.KodTypuRozdeleni ' .
                '   AND H.KodPodrizene = RZamestnance.Rozdeleni ' .

                // pouze podrizena
                // (vlastni budou zvlast)
                ' AND RUzivatele.Rozdeleni <> RZamestnance.Rozdeleni ';
        } else if ($this->podrizena === 2) {
            // primo podrizena rozdeleni
            $sql .=
                ' FROM RozdeleniOsob RUzivatele, RozdeleniOsob RZamestnance, Rozdeleni R_Podrizene, ' .
                ' PracovniPomer PP, PracovniPomer PPUzivatele ' .
                "\n" .
                ' WHERE RUzivatele.KodTypuRozdeleni = ' . $params ->addParameter($this->typRozdeleni) .
                '   AND RZamestnance.KodTypuRozdeleni = ' . $params ->addParameter($this->typRozdeleni) .
                '   AND R_Podrizene.RozdeleniRodice = RUzivatele.KodTypuRozdeleni ' .
                '   AND R_Podrizene.Rodic = RUzivatele.Rozdeleni ' .
                '   AND R_Podrizene.KodTypuRozdeleni = RZamestnance.KodTypuRozdeleni ' .
                '   AND R_Podrizene.Kod = RZamestnance.Rozdeleni ';
        } else {
            // vlastni rozdeleni
            $sql .=
                ' FROM RozdeleniOsob RUzivatele, RozdeleniOsob RZamestnance, PracovniPomer PP, PracovniPomer PPUzivatele ' .
                "\n" .
                ' WHERE RUzivatele.KodTypuRozdeleni = ' . $params ->addParameter($this->typRozdeleni) .
                '   AND RZamestnance.KodTypuRozdeleni = ' . $params ->addParameter($this->typRozdeleni) .
                '   AND RUzivatele.Rozdeleni = RZamestnance.Rozdeleni ';
        }

        $sql .=
            "\n" .
            '   AND PPUzivatele.Id = ' . $params ->addParameter((int)$pp ->idPP) .
            '   AND RUzivatele.RC = PPUzivatele.RC ' .
            '   AND RUzivatele.OsobniCislo = PPUzivatele.OsobniCislo ' .
            '   AND RUzivatele.DatumNastupu = PPUzivatele.DatumNastupu ' .

            // PP uzivatele se kontrolovat nemusi - kontrolu provedl uz volajici kod a nacita se seznam jen pro jeden PP uzivatele
            //'   AND PPUzivatele.DatumNastupu <= ' . $params ->addParameter($database ->dateToParam($this->obdobiDo)) .
            //'   AND (PPUzivatele.DatumUkonceni >= ' . $params ->addParameter($database ->dateToParam($this->obdobiOd < time() ? $this->obdobiOd : time())) . '  OR PPUzivatele.DatumUkonceni IS NULL) ' . // pri "pohledu do budoucnosti" kontroluju aktualni cas - aby se napriklad daly schvalovat budouci absence i kdyz vedoucimu konci PP

            '   AND RUzivatele.PlatnostOd <= ' . $params ->addDateParameter($this->obdobiDo) .
            '   AND (RUzivatele.PlatnostDo >= ' . $params ->addDateParameter($this->obdobiOd) .
            '  OR RUzivatele.PlatnostDo IS NULL) ' .

            '   AND RZamestnance.PlatnostOd <= ' . $params ->addDateParameter($this->obdobiDo) .
            '   AND (RZamestnance.PlatnostDo >= ' . $params ->addDateParameter($this->obdobiOd) .
            '  OR RZamestnance.PlatnostDo IS NULL) ' .

            // kontroluje i platnost PP - jinak by zobrazoval i osoby, ktere mely starsi
            // pracovni pomer s prirazenim na povolene stredisko
            "\n" .
            '   AND PP.RC = RZamestnance.RC ' .
            '   AND PP.OsobniCislo = RZamestnance.OsobniCislo ' .
            '   AND PP.DatumNastupu = RZamestnance.DatumNastupu ' .
            '   AND PP.DatumNastupu <= ' . $params ->addDateParameter($this->obdobiDo) .
            '   AND (PP.DatumUkonceni >= ' . $params ->addDateParameter($this->obdobiOd) .
            '  OR PP.DatumUkonceni IS NULL) ';

        $sql .=
            "\n" .
            ' ORDER BY PP.Id, RZamestnance.PlatnostOd '; // optimalizace

        $this->nactiSQLPrimo($database, $sql, $params, $pp);
    }

    /**
     * Nacte seznam vlastnich / (primo) podrizenych rozdeleni pro zadany pracovni pomer uzivatele, typ rozdeleni a obdobi.
     * Kontroluje a zapisuje i data platnosti
     * @param Database $database
     * @param int $idpp
     * @return RozdeleniUzivateleProPrava[][]
     * @throws \Exception
     */
    function nactiRozdeleniOsoby($database, $idpp)
    {
        $params = $database ->createParameters();

        if ($this ->podrizena === 1) {
            // podrizena rozdeleni
            $sql =
                'SELECT DISTINCT H.KodPodrizene AS KodRozdeleni, RUzivatele.PlatnostOd, RUzivatele.PlatnostDo' .
                " FROM RozdeleniOsob RUzivatele, HierarchieRozdeleni H, PracovniPomer PPUzivatele\n" .
                ' WHERE RUzivatele.KodTypuRozdeleni = ' . $params ->addParameter($this->typRozdeleni) .
                '   AND H.TypNadrazene = RUzivatele.KodTypuRozdeleni ' .
                '   AND H.KodNadrazene = RUzivatele.Rozdeleni ' .
                // pouze podrizena (vlastni budou zvlast)
                '   AND H.KodNadrazene <> H.KodPodrizene';
        } else if ($this ->podrizena === 2) {
            // primo podrizena rozdeleni
            $sql =
                ' SELECT DISTINCT R.Kod AS KodRozdeleni, RUzivatele.PlatnostOd, RUzivatele.PlatnostDo' .
                " FROM RozdeleniOsob RUzivatele, PracovniPomer PPUzivatele, Rozdeleni R\n" .
                ' WHERE RUzivatele.KodTypuRozdeleni = ' . $params ->addParameter($this->typRozdeleni) .
                '   AND R.RozdeleniRodice = RUzivatele.KodTypuRozdeleni ' .
                '   AND R.Rodic = RUzivatele.Rozdeleni ';
        } else {
            // vlastni rozdeleni
            $sql =
                ' SELECT DISTINCT RUzivatele.Rozdeleni AS KodRozdeleni, RUzivatele.PlatnostOd, RUzivatele.PlatnostDo' .
                " FROM RozdeleniOsob RUzivatele, PracovniPomer PPUzivatele\n" .
                ' WHERE RUzivatele.KodTypuRozdeleni = ' . $params ->addParameter($this->typRozdeleni);
        }

        $sql .=
            "\n" .
            ' AND PPUzivatele.Id = ' . $params ->addParameter((int)$idpp) .
            ' AND RUzivatele.RC = PPUzivatele.RC ' .
            ' AND RUzivatele.OsobniCislo = PPUzivatele.OsobniCislo ' .
            ' AND RUzivatele.DatumNastupu = PPUzivatele.DatumNastupu ' .
            "\n" .
            ' AND RUzivatele.PlatnostOd <= ' . $params ->addDateParameter($this->obdobiDo) .
            ' AND (RUzivatele.PlatnostDo >= ' . $params ->addDateParameter($this->obdobiOd) .
            '  OR RUzivatele.PlatnostDo IS NULL) ';

        $sql .=
            ' ORDER BY KodRozdeleni';

        $cursor = $database ->getSqlResult($sql, $params);

        $result = [];

        while ($row = $cursor ->fetch()) {
            $item = new RozdeleniUzivateleProPrava();
            $item ->kod = $row[0];
            $item ->platnostOd = $database ->stringToTimestamp($row[1]);
            $item ->platnostDo = $database ->stringToTimestamp($row[2]);

            if (isset($result[$item ->kod])) {
                $result[$item ->kod][] = $item;
            } else {
                $result[$item ->kod] = [$item];
            }
        }

        $cursor ->close();

        return $result;
    }


    /**
     * @param Database $database
     * @param string $sql
     * @param Parameters $params
     * @param PPUzivatele $ppUzivatele POZOR! Muze byt null!
     * @param RozdeleniUzivateleProPrava[][] $seznamRozdeleni
     * @throws \Exception
     */
    private function nactiSQLPodleRozdeleni($database, $sql, $params, $ppUzivatele, $seznamRozdeleni)
    {
        //'SELECT DISTINCT PP.Id, PP.RC,' . // 0,1
        //' PP.DatumNastupu, PP.DatumUkonceni,' . // 2,3
        //' RZamestnance.PlatnostOd, RZamestnance.PlatnostDo, RZamestnance.Rozdeleni' . // 4..6
        $this->list = [];

        $cursor = $database ->select($sql, $params);

        while ($row = $cursor ->fetch()) {
            $idpp = (int)$row[0];
            // kontrola datumu
            // - musi byt prunik (platnost PP uzivatele i zamestnance, totez pro rozdeleni)
            $platnostDo = null;
            $platnostOd = $database ->stringToTimestamp($row[2]); // PP.DatumNastupu

            if ($row[3]) {
                $pom = $database ->stringToTimestamp($row[3]); // PP.DatumUkonceni
                if (!$platnostDo || $pom < $platnostDo) {
                    $platnostDo = $pom;
                }
            }

            if ($row[4]) {
                $pom = $database ->stringToTimestamp($row[4]); // RZamestnance.PlatnostOd
                if ($pom > $platnostOd) {
                    $platnostOd = $pom;
                }
            }

            if ($row[5]) {
                $pom = $database ->stringToTimestamp($row[5]); // RZamestnance.PlatnostDo
                if (!$platnostDo || $pom < $platnostDo) {
                    $platnostDo = $pom;
                }
            }

//            if ($ppUzivatele  &&  $ppUzivatele ->datumNastupu > $platnostOd) {      OD 01/2024 nekontroluje datum nastupu uzivatele - rozhoduje jen platnost prirazeni na rozdeleni
//                $platnostOd = $ppUzivatele ->datumNastupu;
//            }

            if (!$platnostDo  ||  $platnostDo >= $platnostOd) {
                // jeste zkontroluje platnost rozdeleni uzivatele
                $itemsRozdeleni = $seznamRozdeleni[$row[6]];
                foreach ($itemsRozdeleni as $rozdeleniUzivatele) {
                    $od = $platnostOd;
                    if ($rozdeleniUzivatele ->platnostOd > $od) {
                        $od = $rozdeleniUzivatele ->platnostOd;
                    }
                    $do = $platnostDo;
                    if (!$do  ||  ($rozdeleniUzivatele->platnostDo  &&   $rozdeleniUzivatele->platnostDo < $do)) {
                        $do = $rozdeleniUzivatele->platnostDo;
                    }

                    if ($od <= $do || !$do) {
                        // prida zaznam do seznamu
                        $pravo = new PravoNaPP($idpp, $row[1], $od, $do); // id,rc
                        $this->pridejPravoNaPP($pravo);
                    } else {
                        //    echo "$idpp - nepouzito! ";
                    }
                }
            }
        }

        $cursor ->close();

        $this->seznamNacten = true;
    }


    /**
     * Nacte seznam pracovnich pomeru na vlastnich / podrizenych rozdelenich pro zadany pracovni pomer uzivatele, typ rozdeleni a obdobi.
     * Kontroluje a zapisuje i data platnosti
     * @param Database $database
     * @param PPUzivatele $ppUzivatele
     * @throws \Exception
     */
    private function nactiSeznamPPPodleRozdeleni($database, $ppUzivatele)
    {
        // nejdrive nacte seznam rozdeleni, do kterych patri uzivatel
        $seznamRozdeleni = $this ->nactiRozdeleniOsoby($database, $ppUzivatele ->idPP);

        if (!$seznamRozdeleni) {
            return; // zadne rozdeleni
        }

        $params = $database ->createParameters();

        $sql =
            'SELECT DISTINCT PP.Id, PP.RC,' . // 0,1
            ' PP.DatumNastupu, PP.DatumUkonceni,' . // 2,3
            ' RZamestnance.PlatnostOd, RZamestnance.PlatnostDo, RZamestnance.Rozdeleni' . // 4..6
            ' FROM RozdeleniOsob RZamestnance, PracovniPomer PP ' .
            "\n" .
            ' WHERE RZamestnance.KodTypuRozdeleni = ' . $params ->addParameter($this->typRozdeleni) .
            '   AND RZamestnance.PlatnostOd <= ' . $params ->addDateParameter($this->obdobiDo) .
            '   AND (RZamestnance.PlatnostDo >= ' . $params ->addDateParameter($this->obdobiOd) .
            '  OR RZamestnance.PlatnostDo IS NULL) ' .

            // kontroluje i platnost PP - jinak by zobrazoval i osoby, ktere mely starsi
            // pracovni pomer s prirazenim na povolene stredisko
            '   AND PP.RC = RZamestnance.RC ' .
            '   AND PP.OsobniCislo = RZamestnance.OsobniCislo ' .
            '   AND PP.DatumNastupu = RZamestnance.DatumNastupu ' .
            '   AND PP.DatumNastupu <= ' . $params ->addDateParameter($this->obdobiDo) .
            '   AND (PP.DatumUkonceni >= ' . $params ->addDateParameter($this->obdobiOd) .
            '  OR PP.DatumUkonceni IS NULL) ' .
            "\n" .
            '   AND RZamestnance.Rozdeleni IN (';

        $first = true;
        foreach ($seznamRozdeleni as $kod => $itemsRozdeleni) {
            if (!$first) {
                $sql .= ', ';
            } else {
                $first = false;
            }

            $sql .= $params ->addParameter("$kod"); // Pozor! musi byt predano jako string, jinak nefunguje pri rozdeleni s ciselnym kodem!
        }

        $sql .= ")\n"; // konec IN


        $sql .=
            ' ORDER BY PP.Id, RZamestnance.PlatnostOd '; // optimalizace

        $this ->nactiSQLPodleRozdeleni($database, $sql, $params, $ppUzivatele, $seznamRozdeleni);
    }

    /**
     * Nacte seznam pracovnich pomeru na vlastnich / podrizenych rozdelenich pro zadany pracovni pomer uzivatele, typ rozdeleni a obdobi.
     * Kontroluje a zapisuje i data platnosti
     * @param Database $database
     * @param PPUzivatele $ppUzivatele
     * @param int $typRozdeleni
     * @param int $podrizene
     * @throws \Exception
     */
    function nactiSeznamPPProPrava($database, $ppUzivatele, $typRozdeleni, $podrizene)
    {
        Profiler::addItem('');
        $item = Profiler::addItem("nactiSeznamPPProPrava TYP=$typRozdeleni, podrizene=$podrizene", "PRAVA");

        $this->typRozdeleni = (int)$typRozdeleni;
        $this->podrizena = $podrizene;

        $this->obdobiOd = self::$obdobiPravaOd;
        $this->obdobiDo = self::$obdobiPravaDo;

        // kontrola, jestli uz PP zacal ...
        if ($ppUzivatele ->datumNastupu <= self::$obdobiPravaDo) {                                                        // TODO: upravit (nebo odstranit) kontrolu DATUMU NASTUPU - aby bylo mozne zpracovavat zaznamy pred nastupem
            // ... a kontrola jestli uz neni ukoncen (pro obdobi v budoucnosti se overuje aktualni cas, aby sly napriklad schvalovat budouci absence)
            if (!$ppUzivatele ->datumUkonceni  ||  $ppUzivatele ->datumUkonceni >= ($this->obdobiOd < time() ? $this->obdobiOd : time()) ) {
                global $_prava_nacitat_postupne;
                if ($_prava_nacitat_postupne) {
                    $this ->nactiSeznamPPPodleRozdeleni($database, $ppUzivatele);
                } else {
                    $this->nactiPrimoSeznamPP($database, $ppUzivatele);
                }
            }
        }

        Profiler::endItem($item);
    }


    /**
     * Nacte seznam pracovnich pomeru na vybranych pristupovych skupinach
     * @param Database $database
     * @param string $skupiny
     * @param string $osoby
     */
    function nactiVybraneSkupiny($database, $skupiny, $osoby)
    {
        //$params = $database ->createParameters();

        $this->obdobiOd = self::$obdobiPravaOd;
        $this->obdobiDo = self::$obdobiPravaDo;

        $params = $database ->createParameters();

        $sql =
            'SELECT DISTINCT PP.Id, PP.RC, ' . // 0,1
            '       PP.DatumNastupu, PP.DatumUkonceni, ' . // 2,3
            '       PrirazeniOsobDoPristSkup.PlatnostOd, PrirazeniOsobDoPristSkup.PlatnostDo, ' . // 4,5
            "       null, null\n" . // 6,7
            '  FROM PracovniPomer PP LEFT OUTER JOIN PrirazeniOsobDoPristSkup' .
            '    ON PP.RC = PrirazeniOsobDoPristSkup.RC ' .
            '   AND PP.OsobniCislo = PrirazeniOsobDoPristSkup.OsobniCislo ' .
            '   AND PP.DatumNastupu = PrirazeniOsobDoPristSkup.DatumNastupu ';
        // odfiltruje osoby, ktere jsou primo zadane, protoze jinak se muze stat, ze se nezobrazi
        // (kdyz maji prirazenou jinou pristupovou skupinu, jsou pak odfiltrovany ve where)
        if ($osoby) {
            $pomArray = explode(';', $osoby);
            foreach ($pomArray as $osoba) {
                if ($osoba) {
                    $sql .= '   AND PP.RC <> ' . $params ->addParameter($osoba);
                }
            }
        }

        $sql .=
            // jen platne PP
            ' WHERE PP.DatumNastupu <= ' . $params ->addDateParameter($this->obdobiDo) .
            '   AND (PP.DatumUkonceni >= ' . $params ->addDateParameter($this->obdobiOd) .
            '  OR PP.DatumUkonceni IS NULL) ' .

            ' AND ( ( 0 = 1 ) '; // --- start


        // filtr na osoby z vybranych pristupovych skupin
        if (($skupiny ?? '') != '') {
            // Filtr pro zadane pristupove skupiny
            $sql .=
                ' OR ( ' .  // --- skupiny
                // kontrola casu prirazeni do skupiny
                '   PrirazeniOsobDoPristSkup.PlatnostOd <= ' . $params ->addDateParameter($this->obdobiDo) .
                '   AND (PrirazeniOsobDoPristSkup.PlatnostDo >= ' . $params ->addDateParameter($this->obdobiOd) .
                '  OR PrirazeniOsobDoPristSkup.PlatnostDo IS NULL) ';

            // povolene skupiny
            $pomArray = explode(';', $skupiny);

            $sql .= ' AND ( 0 = 1';
            foreach ($pomArray as $skupina) {
                if ($skupina !== '') {
                    $sql = $sql . ' OR Kod = ' . $params ->addParameter($skupina);
                }
            }
            $sql .= ')';

            $sql .= ')'; // --- skupiny konec
        }

        if ($osoby) {
            // RC vybranych osob
            $pomArray = explode(';', $osoby);
            foreach ($pomArray as $osoba) {
                if ($osoba) {
                    $sql .= ' OR PP.RC = ' . $params ->addParameter($osoba);
                }
            }
            //foreach ($pomArray as $osoba) if ($osoba) $sql .= ' OR PP.RC = ' . $params ->addParameter($osoba);
        }

        $sql .= ')'; // --- stop

        $sql .=
            ' ORDER BY PP.Id '; // dulezite !

        $this->nactiSQLPrimo($database, $sql, $params, null);
    }

    /**
     * @param PravoNaPP|PravoNaPP[] $polozka
     * @param int $obdobiOd
     * @param int $obdobiDo
     * @return bool
     */
    private function lzeZobrazitPolozku($polozka, $obdobiOd, $obdobiDo)
    {
        if (is_array($polozka)) {
            foreach ($polozka as $pravo1) {
                if ((!$pravo1->platnostDo || $obdobiOd <= $pravo1->platnostDo)
                    && $obdobiDo >= $pravo1->platnostOd
                ) {
                    return true;
                }
            }
        } else {
            if ((!$polozka->platnostDo || $obdobiOd <= $polozka->platnostDo)
                && $obdobiDo >= $polozka->platnostOd
            ) {
                return true;
            }
        }

        return false;
    }

    /**
     * Vraci true, pokud je mozne zobrazit zadany PP v zadanem obdobi
     * Pokud neni obdobi zadano, tak staci pravo kdykoliv v nactenem rozsahu!
     * Seznam uz musi byt nacten!
     * @param int $idpp Id PP podrizeneho
     * @param int $obdobiOd
     * @param int $obdobiDo
     * @return bool
     */
    function lzeZobrazitPP($idpp, $obdobiOd, $obdobiDo)
    {
        // TODO: Predelat pro zmenu rozdeleni!
        $pravo = $this->list[(int)$idpp] ?? null;
        if (!$pravo) {
            return false;
        }

        if (!$obdobiOd) {
            // obdobi nekontroluje
            return true;
        }

        $obdobiOd = DateUtils::startOfTheDay($obdobiOd);

        return $this ->lzeZobrazitPolozku($pravo, $obdobiOd, $obdobiDo);
    }


    /**
     * Vraci true, pokud je mozne zobrazit zadanou osobu (RC) v zadanem obdobi.
     * (pro agendy, ktere jsou napojene primo na osobu, napr. ukoly)
     * Pokud neni obdobi zadano, tak staci pravo kdykoliv v nactenem rozsahu!
     * Seznam uz musi byt nacten!
     * @param string $rc
     * @param int $obdobiOd
     * @param int $obdobiDo
     * @return bool
     */
    function lzeZobrazitRC($rc, $obdobiOd, $obdobiDo)
    {
        foreach ($this->list as $pravo) {
            if (is_array($pravo)) {
                if ($pravo[0] ->rc == $rc) {
                    if (!$obdobiOd) {
                        // obdobi nekontroluje
                        return true;
                    }

                    return $this ->lzeZobrazitPolozku($pravo, $obdobiOd, $obdobiDo);
                }
            } else {
                if ($pravo->rc == $rc) {
                    if (!$obdobiOd) {
                        // obdobi nekontroluje
                        return true;
                    }

                    return $this ->lzeZobrazitPolozku($pravo, $obdobiOd, $obdobiDo);
                }
            }
            // Nutno projit az do konce! Neni serazeno podle RC!
        }

        return false;
    }


    /**
     * @return bool Vraci true, pokud je seznam
     */
    function jeSeznamNacten()
    {
        if (!$this->seznamNacten) {
            return false;
        }

        if ($this->obdobiOd > self::$obdobiPravaOd || $this->obdobiDo < self::$obdobiPravaDo) {
            return false;
        }

        return true;
    }


//    private function dumpPolozka($pravo) {
//        echo ' ' . $pravo ->idPP . ' (' . date('j.n.Y', $pravo ->platnostOd) . ' - ' . ($pravo ->platnostDo ? date('j.n.Y', $pravo ->platnostDo) : 'x') . '),';
//    }


//    function dump()
//    {
//        echo "<b>SeznamPPProPrava: typ rozdeleni=" . $this ->typRozdeleni . ($this ->podrizena ? ' (podrizena)' : '') . "</b>, " . date('j.n.Y', $this ->obdobiOd) . ' - ' . ($this ->obdobiDo ? date('j.n.Y', $this ->obdobiDo) : 'x');
//        foreach ($this ->list as $pravo) {
//            if (is_array($pravo)) {
//                foreach($pravo as $polozka) $this ->dumpPolozka($polozka);
//            } else {
//                $this ->dumpPolozka($pravo);
//            }
//        }
//    }

} // class SeznamPPProPrava



/**
 * Pracovni pomer uzivatele - udrzuje seznam podrizenych pracovnich pomeru pro kontrolu prav
 * Class PPUzivatele
 */
class PPUzivatele
{
    var $idPP;
    var $datumNastupu;
    var $datumUkonceni;

    /** @var SeznamPPProPrava[]  seznamy "podrizenych" pracovnich pomeru (pole objektu SeznamPPProPrava) */
    var $seznamyPP;


    function __construct($idpp, $datumNastupu, $datumUkonceni)
    {
        $this->idPP = $idpp;
        $this->datumNastupu = $datumNastupu;
        $this->datumUkonceni = $datumUkonceni;

        $this->seznamyPP = [];
    }


    /**
     * @param int $typRozdeleni
     * @param int $podrizene
     * @return SeznamPPProPrava
     */
    function najdiSeznamPP($typRozdeleni, $podrizene)
    {
        /** @var SeznamPPProPrava $seznampp */
        foreach ($this->seznamyPP as $seznampp) {
            if ($seznampp->typRozdeleni == $typRozdeleni && $seznampp->podrizena === $podrizene) {
                // nasel pozadovany seznam pracovnich pomeru
                return $seznampp;
            }
        }

        // nenasel prislusny seznam PP - vytvori novy
        $seznampp = new SeznamPPProPrava($typRozdeleni, $podrizene);
        $this->seznamyPP[] = $seznampp;

        return $seznampp;
    }


    /**
     * Vraci seznam pracovnich pomeru ve vlastnim/podrizenem rozdeleni uzivatele (zadany typ)
     * @param Database $database
     * @param int $typRozdeleni
     * @param int $podrizene priznak, jestli se jedna o podrizena rozdeleni (1), primo podrizena (2) nebo vlastni (0)
     * @param bool $nacistPP
     * @return SeznamPPProPrava
     * @throws \Exception
     */
    function vratSeznamPP($database, $typRozdeleni, $podrizene, $nacistPP)
    {
        $seznampp = $this->najdiSeznamPP($typRozdeleni, $podrizene);

        if ($nacistPP && !$seznampp->jeSeznamNacten()) {
            // nenasel prislusny seznam PP - vytvori novy
            $seznampp->nactiSeznamPPProPrava($database, $this, $typRozdeleni, $podrizene);
        }

        return $seznampp;
    }


    /**
     * Vraci true, pokud je zadany pracovni pomer ($idpp) v zadanem obdobi (kdykoliv behem obdobi)
     * na stejnem rozdeleni jako tento pracovni pomer uzivatele
     * @param Database $database
     * @param int $idpp
     * @param int $typRozdeleni
     * @param int $obdobiOd
     * @param int $obdobiDo
     * @return bool
     * @throws \Exception
     */
    function jeStejneRozdeleni($database, $idpp, $typRozdeleni, $obdobiOd, $obdobiDo)
    {
//        if ($obdobiDo  &&  $this ->datumNastupu > $obdobiDo) {    //- zruseno 23.2.2024 - kontroluje se jen prirazeni na rozdeleni (obvykle je stejne jako datum nastupu)
//            return false;
//        }

        if ($obdobiOd < time()) {
            // Ukonceni PP uzivatele nekontroluje pro budoucnost (napr. pro schvalovani absenci)
            if ($obdobiOd && $this->datumUkonceni && $this->datumUkonceni < $obdobiOd) {
                return false;
            }
        }

        $seznampp = $this->vratSeznamPP($database, $typRozdeleni, 0, true); // podrizene = 0 (vlastni), nacistPP = true
        return $seznampp->lzeZobrazitPP($idpp, $obdobiOd, $obdobiDo);
    }


    /**
     * Vraci true, pokud je zadany pracovni pomer ($idpp) v zadanem obdobi (kdykoliv behem obdobi)
     * na podrizenem rozdeleni jako tento pracovni pomer uzivatele
     * @param Database $database
     * @param int $idpp
     * @param int $typRozdeleni
     * @param int $obdobiOd
     * @param int $obdobiDo
     * @return bool
     * @throws \Exception
     */
    function jePodrizeneRozdeleni($database, $idpp, $typRozdeleni, $obdobiOd, $obdobiDo)
    {
        $seznampp = $this->vratSeznamPP($database, $typRozdeleni, 1, true); // podrizene = 1 = podrizene, nacistPP = true
        return $seznampp->lzeZobrazitPP($idpp, $obdobiOd, $obdobiDo);
    }

    /**
     * Vraci true, pokud je zadany pracovni pomer ($idpp) v zadanem obdobi (kdykoliv behem obdobi)
     * na primo podrizenem rozdeleni jako tento pracovni pomer uzivatele
     * @param Database $database
     * @param int $idPP
     * @param int $typRozdeleni
     * @param int $obdobiOd
     * @param int $obdobiDo
     * @return bool
     * @throws \Exception
     */
    function jePrimoPodrizeneRozdeleni($database, $idPP, $typRozdeleni, $obdobiOd, $obdobiDo)
    {
        $seznampp = $this->vratSeznamPP($database, $typRozdeleni, 2, true); // podrizene = 2 = primo podrizene, nacistPP = true
        return $seznampp->lzeZobrazitPP($idPP, $obdobiOd, $obdobiDo);
    }

    /**
     * Vraci true, pokud je zadana osoba ($rc) v zadanem obdobi (kdykoliv behem obdobi)
     * na stejnem rozdeleni jako tento pracovni pomer uzivatele
     * @param Database $database
     * @param string $rc
     * @param int $typRozdeleni
     * @param int $obdobiOd
     * @param int $obdobiDo
     * @return bool
     * @throws \Exception
     */
    function rcJeStejneRozdeleni($database, $rc, $typRozdeleni, $obdobiOd, $obdobiDo)
    {
        if ($obdobiDo  &&  $this ->datumNastupu > $obdobiDo) {
            return false;
        }
        if ($obdobiOd  &&  $this ->datumUkonceni  &&  $this ->datumUkonceni < $obdobiOd) {
            return false;
        }

        $seznampp = $this->vratSeznamPP($database, $typRozdeleni, 0, true); // podrizene = 0 = vlastni, nacistPP = true
        return $seznampp->lzeZobrazitRC($rc, $obdobiOd, $obdobiDo);
    }



    /**
     * vraci true, pokud je zadana osoba ($rc) v zadanem obdobi (kdykoliv behem obdobi)
     *  na stejnem rozdeleni jako tento pracovni pomer uzivatele
     * @param Database $database
     * @param string $rc
     * @param int $typRozdeleni
     * @param int $obdobiOd
     * @param int $obdobiDo
     * @return bool
     * @throws \Exception
     */
    function rcJePodrizeneRozdeleni($database, $rc, $typRozdeleni, $obdobiOd, $obdobiDo)
    {
        $seznampp = $this->vratSeznamPP($database, $typRozdeleni, 1, true); // podrizene = 1 = podrizene, nacistPP = true
        return $seznampp->lzeZobrazitRC($rc, $obdobiOd, $obdobiDo);
    }

    /**
     * Vraci true, pokud je zadana osoba ($rc) v zadanem obdobi (kdykoliv behem obdobi)
     * na primo podrizenem rozdeleni jako tento pracovni pomer uzivatele
     * @param Database $database
     * @param string $rc
     * @param int $typRozdeleni
     * @param int $obdobiOd
     * @param int $obdobiDo
     * @return bool
     * @throws \Exception
     */
    function rcJePrimoPodrizeneRozdeleni($database, $rc, $typRozdeleni, $obdobiOd, $obdobiDo)
    {
        $seznampp = $this->vratSeznamPP($database, $typRozdeleni, 2, true); // podrizene = 2 = primo podrizene, nacistPP = true
        return $seznampp->lzeZobrazitRC($rc, $obdobiOd, $obdobiDo);
    }
}


class Prava
{
    const PRAVA_ZAKAZAT = 0;
    const PRAVA_POUZE_VLASTNI = 1;
    const PRAVA_VYBRANE_OSOBY_A_SKUPINY = 2;
    const PRAVA_VSE = 3;
    const PRAVA_VYBRANE_KROME_SEBE = 4;

    const VLASTNI_ROZDELENI = 10;
    const VLASTNI_ROZDELENI_KROME_SEBE = 11;
    const VLASTNI_A_PODRIZENA_ROZDELENI = 12;
    const VLASTNI_A_PODRIZENA_ROZDELENI_KROME_SEBE = 13;
    const PODRIZENA_ROZDELENI = 14;
    const PODRIZENA_ROZDELENI_A_SEBE = 15;
    const VLASTNI_A_PRIMO_PODRIZENA_ROZDELENI = 22;
    const VLASTNI_A_PRIMO_PODRIZENA_ROZDELENI_KROME_SEBE = 23;
    const PRIMO_PODRIZENA_ROZDELENI = 24;
    const PRIMO_PODRIZENA_ROZDELENI_A_SEBE = 25;
    const PRIMO_PODRIZENA_ROZDELENI_KROME_SEBE = 26;

    /** @var  PPUzivatele[] Seznam pracovnich pomeru uzivatele */
    private static $seznamPP;

    /** @var PravaNaOsoby[] Seznam prav aktualniho uzivatele */
    private static $pravaUzivatele;

    /** @var Database */
    static $database;


    /**
     * Vraci true, pokud existuje zadany typ prava (napr. uzivatelsky definovany).
     * POZOR! Uz musi byt nacteny seznam vsech prav!
     * @param string $kod
     * @return bool
     *
     */
    public static function existujePravo($kod)
    {
        self::overPravaJsouNactena();
        return isset($_SESSION['prava'][$kod]);
    }


    /**
     * Nastavi promenne obdobiOd a obdobiDo pro prava - tj. obdobi pro ktere se nacitaji prava uzivatele (prirazeni do rozdeleni)
     * @param int $obdobiOd
     * @param int $obdobiDo
     */
    public static function nastavObdobiProPrava($obdobiOd, $obdobiDo = null)
    {
        $obdobiDo = ($obdobiDo ?? $obdobiOd);
        if ($obdobiDo < time()) {
            $obdobiDo = time(); // 11-2023 - vzdy nacita minimalne do aktualniho data - napr. kvuli kopirovani starsich SC
        }

        $obdobiOd = DateUtils::firstDayOfMonth($obdobiOd); // vzdy nacita minimalne cely mesic
        $obdobiDo = DateUtils::lastDayOfMonth($obdobiDo ?: $obdobiOd);

        SeznamPPProPrava::$obdobiPravaOd = $obdobiOd;
        SeznamPPProPrava::$obdobiPravaDo = $obdobiDo;
    }


    /**
     * @throws DBException
     */
    private static function nactiPPUzivateleZDB()
    {
        self::$seznamPP = [];

        $params = self::$database ->createParameters();
        $sql =
            'SELECT Id, DatumNastupu, DatumUkonceni FROM PracovniPomer ' .
            ' WHERE RC = ' . $params ->addParameter(User::getRC()) .
            ' ORDER BY DatumNastupu DESC ';

        $cursor = self::$database ->getSqlResult($sql, $params);

        while ($row = $cursor ->fetch()) {
            $datumUkonceni = $row[2]
                ? $cursor ->stringToTimestamp($row[2])
                : null;

            $pp = new PPUzivatele((int)$row[0], $cursor ->stringToTimestamp($row[1]), $datumUkonceni);
            self::$seznamPP[] = $pp;
        }

        $cursor ->close();
    }


    /**
     * @throws DBException
     */
    private static function nactiPPUzivatele()
    {
        if (self::$seznamPP) {
            return;
        }

        global $_allwaysloadrights;

        if ($_allwaysloadrights  ||  !isset($_SESSION['PPUzivatele'])) {
            self::nactiPPUzivateleZDB();
            $_SESSION['PPUzivatele'] = Prava::$seznamPP;
        } else {
            self::$seznamPP = $_SESSION['PPUzivatele'];
        }
    }


    public static function getSeznamIdPP()
    {
        if (!self::$seznamPP) {
            self::nactiPPUzivatele();
        }

        $result = [];

        if (self::$seznamPP) {
            foreach (self::$seznamPP as $ppUzivatele) {
                $result[] = $ppUzivatele->idPP;
            }
        }

        return $result;
    }


    /**
     * @param string $sql
     * @param Parameters $params
     * @param string $typPrava
     * @return PravaNaOsoby
     * @throws DBException
     */
    private static function nactiPravaUzivateleSQL($sql, $params, $typPrava)
    {
        $result = null;

        self::$pravaUzivatele = [];

        $cursor = self::$database ->getSqlResult($sql, $params);
        global $restrict_rights; // Umozni nacist jen vybrana prava uzivatele. Ostatni nastavi na zakazat. (napr. pro Kiosek)
        global $profilUzivatele;

        if ($restrict_rights  &&  is_string($restrict_rights)) {
            $restrict_rights = explode(';', $restrict_rights);
        }

        $item = Profiler::addItem('Prava::nactiPravaUzivateleSQL - zpracovavam nactena data', 'PRAVA');

        $i = 0;
        while ($row = $cursor ->fetch()) {
            $i++;

            if ($restrict_rights  &&  !in_array($row[0], $restrict_rights)) {
                // Prava jsou globalne omezena (napr. v _appdbset) a toto pravo neni povoleno
                $pravo = new PravaNaOsoby(0, null, null, 0);
            } else if ($row[2] || $profilUzivatele || !User::getSkupinaUzivatele()) {
                // uzivatel ma nastavenou vlastni hodnotu prava nebo nema prirazenou skupinu, nebo se ma natvrdo pouzit globalne nastaveny profil
                $pravo = new PravaNaOsoby((int)$row[2], $row[3], $row[4], $row[5], (int)$row[1]);
                // $prava, $osoby, $skupiny, $priznak, $typRozdeleni
                $pravo ->priznak2 = (int)$row[6];
                $pravo ->priznak3 = $row[7];
            } else {
                // vychozi nastaveni podle skupiny uzivatele
                $pravo = new PravaNaOsoby((int)$row[9], $row[10], $row[11], $row[12], (int)$row[1]);
                $pravo ->priznak2 = (int)$row[13];
                $pravo ->priznak3 = $row[14];
            }

            $pravo ->kod = $row[0];
            $pravo ->zpusobKontrolyObdobi = (int)$row[8];

            if (!$typPrava) {
                self::$pravaUzivatele[$row[0]] = $pravo;
                $_SESSION['prava'][$row[0]] = $pravo;
            }
            // pokud nacitam jen konkretni pravo, tak jej nezapisuji do session!

            if ($typPrava) {
                $result = $pravo;
            }
        }

        $cursor ->close();
        Profiler::endItem($item, "Celkem nacteno $i zaznamu prav");

        return $result;
    }


    /**
     * @param null $typPrava
     * @return PravaNaOsoby
     * @throws DBException
     */
    private static function nactiPravaZastupuZDB($typPrava = null)
    {
        $params = self::$database ->createParameters();

        $sql =
            'SELECT TypPravaProIntraweb.Kod, TypPravaProIntraweb.TypRozdeleni,' . // 0, 1
            ' PravaProIntraweb.Hodnota, ' . self::$database ->castWideMemo('PravaProIntraweb.Osoby') . ', ' . // 2, 3
            self::$database ->castWideMemo('PravaProIntraweb.Skupiny') . ',' .// 4
            ' PravaProIntraweb.Priznak, PravaProIntraweb.Priznak2, PravaProIntraweb.Priznak3,' . // 5..7
            ' TypPravaProIntraweb.ZpusobKontrolyObdobi'; // 8 - doplneno 01/2024

        // nastaveni podle skupiny, pokud ma uzivatel skupinu prirazenou
        if (User::getSkupinaUzivatele()) {
            $sql .=
                ',' .
                ' PravaSkupinIW.Hodnota, ' .
                self::$database->castWideMemo('PravaSkupinIW.Osoby') . ', ' . // 9,10
                self::$database->castWideMemo('PravaSkupinIW.Skupiny') .    // 11
                ', PravaSkupinIW.Priznak, PravaSkupinIW.Priznak2, PravaSkupinIW.Priznak3 '; // 12..14
        }

        $sql .=
            ' FROM ZastupUzivatele INNER JOIN Osoba' .
            '  ON Osoba.RC = ZastupUzivatele.Zastupovany' .
            ' INNER JOIN VybraneRoleProZastup' .
            '  ON VybraneRoleProZastup.IdZastupu = ZastupUzivatele.Id' .
            ' INNER JOIN PravaProZastupUzivatele' .
            '  ON PravaProZastupUzivatele.KodRole = VybraneRoleProZastup.KodRole' .
            ' INNER JOIN TypPravaProIntraweb' .
            '  ON TypPravaProIntraweb.Kod = PravaProZastupUzivatele.TypPrava' .
            ' LEFT OUTER JOIN PravaProIntraweb' .
            '  ON PravaProIntraweb.TypPrava = TypPravaProIntraweb.Kod' .
            '  AND PravaProIntraweb.RC = ZastupUzivatele.Zastupovany' .
            ' LEFT OUTER JOIN PravaSkupinIW' .
            '  ON PravaSkupinIW.TypPrava = TypPravaProIntraweb.Kod' .
            '  AND PravaSkupinIW.Skupina = Osoba.SkupinaIW';

        // jen aktualne platne zastupy pro aktualniho a zastupovaneho uzivatele
        $sql .=
            ' WHERE ZastupUzivatele.Zastupujici = ' . $params ->addParameter(User::getRealUserRC()) .
            ' AND ZastupUzivatele.Zastupovany = ' . $params ->addParameter(User::getRC()) .
            ' AND ZastupUzivatele.Potvrzeni <= 1' . // jen stav "nepotvrzeno" nebo "potvrzeno" (vynecha zamitnuto a zruseno)
            ' AND ZastupUzivatele.PlatnostOd <= ' . $params ->addDateParameter(time()) .
            ' AND (ZastupUzivatele.PlatnostDo >= ' . $params ->addDateParameter(time()) . ' OR ZastupUzivatele.PlatnostDo IS NULL)';

        if ($typPrava) {
            $sql .=
                ' AND TypPravaProIntraweb.Kod = ' . $params ->addParameter($typPrava);
        }

        return self::nactiPravaUzivateleSQL($sql, $params, $typPrava);
    }

    /**
     * @param string $typPrava
     * @return PravaNaOsoby
     * @throws DBException
     */
    private static function nactiPravaUzivateleZDB($typPrava = null)
    {
        $params = self::$database ->createParameters();

        global $profilUzivatele; // zajisti prihlaseni osoby vzdy pod stejnym profilem - napriklad pro kiosek
        if ($profilUzivatele) {
            $sql =
                'SELECT TypPravaProIntraweb.Kod, TypPravaProIntraweb.TypRozdeleni,' . // 0, 1
                ' PravaSkupinIW.Hodnota, ' . self::$database->castWideMemo('PravaSkupinIW.Osoby') . ', ' . // 2,3
                self::$database->castWideMemo('PravaSkupinIW.Skupiny') . // 4
                ', PravaSkupinIW.Priznak, PravaSkupinIW.Priznak2, PravaSkupinIW.Priznak3,' . // 5..7
                ' TypPravaProIntraweb.ZpusobKontrolyObdobi' . // 8 - doplneno 01/2024
                ' FROM TypPravaProIntraweb, PravaSkupinIW ' .
                ' WHERE PravaSkupinIW.TypPrava = TypPravaProIntraweb.Kod ' .
                ' AND PravaSkupinIW.Skupina = ' . $params->addParameter($profilUzivatele);
        } else {
            $sql =
                'SELECT TypPravaProIntraweb.Kod, TypPravaProIntraweb.TypRozdeleni, ' . // 0, 1
                ' PravaProIntraweb.Hodnota, ' . self::$database->castWideMemo('PravaProIntraweb.Osoby') . ', ' . // 2, 3
                self::$database->castWideMemo('PravaProIntraweb.Skupiny') . // 4
                ', PravaProIntraweb.Priznak, PravaProIntraweb.Priznak2, PravaProIntraweb.Priznak3,' . // 5..7
                ' TypPravaProIntraweb.ZpusobKontrolyObdobi'; // 8 - doplneno 01/2024
            // vlastni nastaveni osoby

            // nastaveni podle skupiny, pokud ma uzivatel skupinu prirazenou
            if (User::getSkupinaUzivatele()) {
                $sql .=
                    ', ' .
                    ' PravaSkupinIW.Hodnota, ' . self::$database->castWideMemo('PravaSkupinIW.Osoby') . ', ' . // 9,10
                    self::$database->castWideMemo('PravaSkupinIW.Skupiny') . // 11
                    ', PravaSkupinIW.Priznak, PravaSkupinIW.Priznak2, PravaSkupinIW.Priznak3 '; // 12..14
            }

            $sql .=
                ' FROM TypPravaProIntraweb INNER JOIN Osoba ' .
                '  ON 1 = 1' .
                ' LEFT OUTER JOIN PravaProIntraweb ' .
                '  ON PravaProIntraweb.TypPrava = TypPravaProIntraweb.Kod ' .
                '  AND PravaProIntraweb.RC = Osoba.RC'; //. self::$database ->addSlashes(User::getRC()) . "' ";

            if (User::getSkupinaUzivatele()) // pripoji prava podle skupiny uzivatele
                $sql .=
                    ' LEFT OUTER JOIN PravaSkupinIW ' .
                    ' ON PravaSkupinIW.TypPrava = TypPravaProIntraweb.Kod ' .
                    ' AND PravaSkupinIW.Skupina = Osoba.SkupinaIW'; // " . self::$database ->addSlashes(User::getSkupinaUzivatele()) . "' ";

            $sql .= ' WHERE Osoba.RC = ' . $params->addParameter(User::getRC());
        }

        if ($typPrava) {
            $sql .= ' AND TypPravaProIntraweb.Kod = ' . $params ->addParameter($typPrava);
        }

        return self::nactiPravaUzivateleSQL($sql, $params, $typPrava);
    }


    /**
     * Vraci true, pokud patri zadany pracovni pomer aktualnimu uzivateli
     * @param int $idpp
     * @return bool
     */
    public static function jeVlastniPP($idpp)
    {
        if (!self::$seznamPP) {
            self::nactiPPUzivatele();
        }

        foreach (self::$seznamPP as $pp) {
            if ($pp ->idPP == $idpp) {
                return true;
            }
        }

        return false;
    }


    /**
     * Vraci true, v pripade ze zadany pracovni pomer je nekdy v prubehu zadaneho obdobi (od-do) ve stejnem rozdeleni jako aktualni uzivatel
     * @param int $idpp         Id pracovniho pomeru
     * @param int $typRozdeleni Kod typu rozdeleni
     * @param int $obdobiOd     Timestamp pocatku obdobi
     * @param int $obdobiDo     Timestamp konce obdobi
     * @return bool
     * @throws \Exception
     */
    public static function ppJeVeVlastnimRozdeleni($idpp, $typRozdeleni, $obdobiOd, $obdobiDo)
    {
        if (!self::$seznamPP) {
            self::nactiPPUzivatele();
        }

        foreach (self::$seznamPP as $pp) {
            if ($pp->jeStejneRozdeleni(self::$database, $idpp, $typRozdeleni, $obdobiOd, $obdobiDo)) {
                return true;
            }
        }

        return false;
    }


    /**
     * Vraci true, v pripade ze zadany pracovni pomer je nekdy v prubehu zadaneho obdobi (od-do) v podrizenem rozdeleni aktualniho uzivatele
     * @param int $idpp         Id pracovniho pomeru
     * @param int $typRozdeleni Kod typu rozdeleni
     * @param int $obdobiOd     Timestamp pocatku obdobi
     * @param int $obdobiDo     Timestamp pocatku obdobi
     * @return bool
     * @throws \Exception
     */
    public static function ppJeVPodrizenemRozdeleni($idpp, $typRozdeleni, $obdobiOd, $obdobiDo)
    {
        if (!self::$seznamPP) {
            self::nactiPPUzivatele();
        }
        foreach (self::$seznamPP as $pp) {
            if ($pp->jePodrizeneRozdeleni(self::$database, $idpp, $typRozdeleni, $obdobiOd, $obdobiDo)) {
                return true;
            }
        }

        return false;
    }

    /**
     * Vraci true, v pripade ze zadany pracovni pomer je nekdy v prubehu zadaneho obdobi (od-do) v primo podrizenem rozdeleni (tj. o jednu uroven ve stromu nize) aktualniho uzivatele
     * @param int $idpp         Id pracovniho pomeru
     * @param int $typRozdeleni Kod typu rozdeleni
     * @param int $obdobiOd     Timestamp pocatku obdobi
     * @param int $obdobiDo     Timestamp pocatku obdobi
     * @return bool
     * @throws \Exception
     */
    public static function ppJeVPrimoPodrizenemRozdeleni($idpp, $typRozdeleni, $obdobiOd, $obdobiDo)
    {
        if (!self::$seznamPP) {
            self::nactiPPUzivatele();
        }

        foreach (self::$seznamPP as $pp) {
            if ($pp->jePrimoPodrizeneRozdeleni(self::$database, $idpp, $typRozdeleni, $obdobiOd, $obdobiDo)) {
                return true;
            }
        }

        return false;
    }

    /**
     * Vraci true, v pripade ze zadany zamestnanec je nekdy v prubehu zadaneho obdobi (od-do) ve stejnem rozdeleni jako aktualni uzivatel
     * @param string $rc
     * @param int $typRozdeleni
     * @param int $obdobiOd
     * @param int $obdobiDo
     * @return bool
     * @throws \Exception
     */
    public static function rcJeVeVlastnimRozdeleni($rc, $typRozdeleni, $obdobiOd, $obdobiDo)
    {
        if (!self::$seznamPP) {
            self::nactiPPUzivatele();
        }

        foreach (self::$seznamPP as $pp) {
            if ($pp->rcJeStejneRozdeleni(self::$database, $rc, $typRozdeleni, $obdobiOd, $obdobiDo)) {
                return true;
            }
        }

        return false;
    }


    /**
     * Vraci true, v pripade ze zadany zamestnanec je nekdy v prubehu zadaneho obdobi (od-do) v podrizenem rozdeleni jako aktualnihe uzivatele
     * @param string $rc
     * @param int $typRozdeleni
     * @param int $obdobiOd
     * @param int $obdobiDo
     * @return bool
     * @throws \Exception
     */
    public static function rcJeVPodrizenemRozdeleni($rc, $typRozdeleni, $obdobiOd, $obdobiDo)
    {
        if (!self::$seznamPP) {
            self::nactiPPUzivatele();
        }

        foreach (self::$seznamPP as $pp) {
            if ($pp->rcJePodrizeneRozdeleni(self::$database, $rc, $typRozdeleni, $obdobiOd, $obdobiDo)) {
                return true;
            }
        }

        return false;
    }

    /**
     * Vraci true, v pripade ze zadany zamestnanec je nekdy v prubehu zadaneho obdobi (od-do) v primo podrizenem rozdeleni (tj. o jednu uroven ve stromu nize) aktualniho uzivatele
     * @param string $rc
     * @param int $typRozdeleni
     * @param int $obdobiOd
     * @param int $obdobiDo
     * @return bool
     * @throws \Exception
     */
    public static function rcJeVPrimoPodrizenemRozdeleni($rc, $typRozdeleni, $obdobiOd, $obdobiDo)
    {
        if (!self::$seznamPP) {
            self::nactiPPUzivatele();
        }

        foreach (Prava::$seznamPP as $pp) {
            if ($pp->rcJePrimoPodrizeneRozdeleni(self::$database, $rc, $typRozdeleni, $obdobiOd, $obdobiDo)) {
                return true;
            }
        }

        return false;
    }


//    /**
//     * Ladici dump
//     */
//    public static function dump()
//    {
//        echo 'DUMP PRAVA >>> <br/>';
//        echo '- prava od ' . date('j.n.Y', SeznamPPProPrava::$obdobiPravaOd) . ' - ' . date('j.n.Y', SeznamPPProPrava::$obdobiPravaDo) . '<br/>';
//        echo ' - seznam PP: <br/>';
//
//        if (Prava::$seznamPP) {
//            foreach (Prava::$seznamPP as $ppUzivatele) {
//                $ppUzivatele ->dump();
//            }
//        }
//        echo '<br/>';
//    }


    /**
     * Vraci objekt PravaNaOsoby pro zadany kod
     * @param string|PravaNaOsoby $kod
     * @param string|null $kod2  Pokud je zadan $kod2 a existuje takove pravo, tak vraci objekt pro $kod2, jinak pro $kod
     * @return PravaNaOsoby
     */
    public static function vratPrava($kod, $kod2 = null)
    {
        self::overPravaJsouNactena();

        if (isset($kod2)  &&  self::existujePravo($kod2) ) {
            $result = $_SESSION['prava'][$kod2] ?? null;
        } else {
            if ($kod instanceof PravaNaOsoby) {
                return $kod;
            }

            $result = $_SESSION['prava'][$kod] ?? null;
        }

        if (!$result) {
            return new PravaNaOsoby(0, '', '', 0, 0);
        }

        return $result;
    }


    /**
     * Vraci hodnotu prava uzivatele pro zadany kod
     * @param string $kod
     * @param string $kod2  Pokud je zadan $kod2 a existuje takove pravo, tak vraci objekt pro $kod2, jinak pro $kod
     * @return int
     */
    public static function vratHodnotuPrava($kod, $kod2 = null)
    {
        $prava = self::vratPrava($kod, $kod2);
        if (!$prava  ||  $prava ->prava == 100) {
            return 0;
        }

        return $prava ->prava;
    }


    /**
     * Vraci true, pokud ma uzivatel prirazena nejaka prava (tj. nema "zakazat")
     * @param PravaNaOsoby[]|PravaNaOsoby|string $prava   Jeden konkretni objekt PravaNaOsoby nebo pole objektu nebo kod prava
     * @return bool
     */
    public static function jsouNejakePrava($prava)
    {
        if (is_string($prava)) {
            $pravaStrArray = explode(';', $prava);
            foreach ($pravaStrArray as $kodPrava) {
                if (self::vratPrava($kodPrava) ->jsouNejakePrava()) {
                    return true;
                }
            }

            return false;
        }

        if (is_array($prava)) {
            /** @var PravaNaOsoby $pravo */
            foreach ($prava as $pravo)
                if ($pravo->jsouNejakePrava()) {
                    return true;
                }

            return false;
//        } else if (is_string($prava)) {
//            $pravo = static::vratPrava($prava);
//            return  $pravo ? $pravo ->jsouNejakePrava() : false;
        } else if ($prava) {
            return $prava ->jsouNejakePrava();
        }

        return false;
    }


    /**
     * Vraci true, pokud ma uzivatel pravo na zadany pracovni pomer
     * @param PravaNaOsoby[]|PravaNaOsoby|string $prava Jeden konkretni objekt PravaNaOsoby nebo pole objektu nebo kod prava
     * @param int $idpp     Id pracovniho pomeru
     * @param int $datumOd  Zacatek obdobi, ve kterem se ma pravo zkontrolovat
     * @param int $datumDo  Konec obdobi, ve kterem se ma pravo zkontrolovat
     * @return bool
     * @throws DebugException
     */
    public static function lzeZobrazitPP($prava, $idpp, $datumOd = 0, $datumDo = 0)
    {
        if (!$prava) {
            return false;
        }

        if (is_array($prava)) {
            /** @var PravaNaOsoby $pravo */
            foreach ($prava as $pravo) {
                if ($pravo->lzeZobrazitPP($idpp, $datumOd, $datumDo)) {
                    return true;
                }
            }

            return false;
        }

        if (is_string($prava)) {
            $pravo = static::vratPrava($prava);
            return $pravo && $pravo->lzeZobrazitPP($idpp, $datumOd, $datumDo);
        }

        return $prava ->lzeZobrazitPP($idpp, $datumOd, $datumDo);
    }


    /**
     * Vraci true, pokud ma uzivatel prava na vice osob
     * Ale pozor! Skutecny pocet osob nezkouma! Staci, kdyz je nastaveno napr. "vlastni rozdeleni".
     * @param PravaNaOsoby[]|PravaNaOsoby|string $prava   Jeden konkretni objekt PravaNaOsoby nebo pole objektu, nebo kod typu prava
     * @return bool
     */
    public static function pravaNaViceOsob($prava)
    {
        if ($prava === null) {
            return false;
        }

        if (is_array($prava)) {
            /** @var PravaNaOsoby $pravo */
            foreach ($prava as $pravo) {
                if ($pravo->jsouNejakePrava() && !$pravo->pouzeSebe()) {
                    return true;
                }
            }

            return false;
        }

        if (is_string($prava)) {
            $prava = self::vratPrava($prava);
        }
        return $prava ->jsouNejakePrava() && !$prava ->pouzeSebe();
    }


    /**
     * Vraci true, pokud ma uzivatel prava "... vcetne sebe"
     * @param PravaNaOsoby[]|PravaNaOsoby $prava   Jeden konkretni objekt PravaNaOsoby nebo pole objektu
     * @return bool
     */
    public static function pravaVcetneSebe($prava)
    {
        if (is_array($prava)) {
            /** @var PravaNaOsoby $pravo */
            foreach ($prava as $pravo) {
                if ($pravo->vcetneSebe()) {
                    return true;
                }
            }

            return false;
        }

        return $prava ->vcetneSebe();
    }

    /**
     * @param PravaNaOsoby|[PravaNaOsoby] $prava
     * @param string $tabulka
     * @param int $datumOd
     * @param int $datumDo
     * @param bool $vynechatVcetneSebe Priznak, jestli se ma vynechat u prav ("vcetne sebe") cast dotazu, ktere vybira PP uzivatele (kvuli rychlosti)
     *                                  V tom pripade to musi zajistit volajici kod (napr. pomoci UNION)
     * @param Parameters $params
     * @return string
     * @throws \Exception
     */
    public static function sqlFiltrPodlePrav($prava, $tabulka, $datumOd = 0, $datumDo = 0, $vynechatVcetneSebe = false, $params = null)
    {
        if (!$prava) {
            throw new \Exception('Chyba ve funkci SQLFiltrPodlePrav - nejsou zadana prava!');
        }

        if (is_array($prava)) {
            $result = '';

            /* @var PravaNaOsoby $pravo */
            foreach ($prava as $pravo) {
                $filtr = $pravo ->sqlFiltr($tabulka, $datumOd, $datumDo, $params, $vynechatVcetneSebe);

                $result .=
                    ($result ? ' OR ' : '') .
                    '(' . $filtr .')';
            }

            if ($result) {
                $result = '(' . $result . ')';
            }

            return $result;
        }

        return $prava ->sqlFiltr($tabulka, $datumOd, $datumDo, $params, $vynechatVcetneSebe);
    }


    /**
     * @param PravaNaOsoby[]|PravaNaOsoby|string $prava
     * @return string
     */
    public static function getPopisPrav($prava)
    {
        if (is_string($prava)) {
            return $prava;
        }

        if (is_array($prava)) {
            $result = '';
            foreach ($prava as $item) {
                $result .= ($result ? ';' : '') . $item ->kod;
            }
            return $result;
        }

        if ($prava instanceof PravaNaOsoby) {
            return $prava ->kod;
        }

        return '';
    }

    private static function overPravaJsouNactena()
    {
        if (!User::getRC()) {
            return false;
        }

        self::nactiPPUzivatele();

        if (self::$pravaUzivatele) {
            return true;
        }

//        if ($typPrava) {
//            if (isset($_SESSION['prava'][$typPrava])) {
//                return $_SESSION['prava'][$typPrava];
//            }
//
//            if (User::jeZastup()) {
//                return self::nactiPravaZastupuZDB($typPrava);
//            }
//
//            return self::nactiPravaUzivateleZDB($typPrava);
//        }

        global $_allwaysloadrights; // z _appdbset
        if ($_allwaysloadrights  ||  !isset($_SESSION['prava'])) {
            if (User::jeZastup()) {
                self::nactiPravaZastupuZDB();
            } else {
                self::nactiPravaUzivateleZDB();
            }
        }

        return true;
    }


    /**
     * Nacte prava aktualne prihlaseneho uzivatele. Pozor! Parametr $RC uz se nepouziva!
     * @param Database $database
     * @param null $unusedParam Nepouzivany parametr - nezadavat, a v pripade pouziti parametru $typPrava zadat null!
     * @param string $typPrava Pokud neni zadan, nactou se vsechna prava, jinak se nacita jen zadany typ
     * @return bool|null|PravaNaOsoby
     * @throws \Exception
     * @deprecated      Neni potreba volat. Prava se nactou automaticky. Pokud je potreba konkretni typ prava, tak je lepsi pouzit metodu vratPrava()
     */
    public static function nactiPravaUzivatele($database, $unusedParam = null, $typPrava = null)
    {
        self::$database = $database;

        if ($typPrava) {
            return self::vratPrava($typPrava);
        }

        return self::overPravaJsouNactena();

//        if (!User::getRC()) {
//            return false;
//        }
//
//        self::nactiPPUzivatele();
//
//        if ($typPrava) {
//            if (isset($_SESSION['prava'][$typPrava])) {
//                return $_SESSION['prava'][$typPrava];
//            }
//
//            if (User::jeZastup()) {
//                return self::nactiPravaZastupuZDB($typPrava);
//            }
//
//            return self::nactiPravaUzivateleZDB($typPrava);
//        }
//
//        global $_allwaysloadrights; // z _appdbset
//        if ($_allwaysloadrights  ||  !isset($_SESSION['prava'])) {
//            if (User::jeZastup()) {
//                self::nactiPravaZastupuZDB($typPrava);
//            } else {
//                self::nactiPravaUzivateleZDB($typPrava);
//            }
//        }
//
//        return true;
    }


    public static function clear()
    {
        unset($_SESSION['prava']);
        unset($_SESSION['PPUzivatele']);
    }

}


/**
 * Trida PravaNaOsoby reprezentuje nastaveni jednoho konkretniho typu prava aktualniho uzivatele.
 * Class PravaNaOsoby
 */
class PravaNaOsoby
{
    /** @var string */
    public $kod;

    /**
     * @var int
     * 0 = zakazano
     * 1 = pouze vlastni
     * 2 = vybrane osoby a skupiny
     * 3 = vse
     * 4 = vybrane osoby a skupiny krome sebe
     * 10 = vlastni rozdeleni
     * 11 = vlastni rozdeleni krome sebe
     * 12 = vlastni a podrizena rozdeleni
     * 13 = vlastni a podrizena rozdeleni krome sebe
     * 14 = podrizena rozdeleni
     * 15 = podrizena rozdeleni a sebe
     * 22 = vlastni a primo podrizena rozdeleni
     * 23 = vlastni a primo podrizena rozdeleni krome sebe
     * 24 = primo podrizena rozdeleni
     * 25 = primo podrizena rozdeleni a sebe
     * 26 = primo podrizena rozdeleni krome sebe
     */
    public $prava;

    /** @var int  kod typu rozdeleni - pro nastaveni 10 - 25 */
    public $typRozdeleni;

    const OBDOBI_PODLE_AGENDY = 0;
    const OBDOBI_PODLE_AKTUALNIHO_DNE = 1;

    /** @var int    0 = kontroluje se podle agendy, 1 = vzdy se kontroluje podle aktualniho dne */
    public $zpusobKontrolyObdobi = 0;

    /** @var string  pro nastaveni 2 a 4 - retezce nactene z databaze */
    var $osoby;
    /** @var string  pro nastaveni 2 a 4 - retezce nactene z databaze */
    var $skupiny;
    /** @var SeznamPPProPrava  pro nastaveni 2 a 4 (vybrane osoby a skupiny) - SeznamPPProPrava */
    var $seznamPP;

    /** @var bool   checkbox */
    public $priznak;
    /** @var int    Integer priznak */
    public $priznak2 = 0;
    /** @var string Textovy priznak */
    public $priznak3 = null;


    function __construct($prava, $osoby, $skupiny, $priznak, $typRozdeleni = 0)
    {
        $this ->prava = $prava;
        $this ->typRozdeleni = (int)$typRozdeleni;
        $this ->osoby = $osoby;
        $this ->skupiny = $skupiny;
        $this ->priznak = $priznak;
    }


    /**
     * Vraci false pokud uzivatel nema zadne prava
     * @return bool
     */
    function jsouNejakePrava()
    {
        // pokud neni 0 nebo 100, tak nejake prava ma
        return $this ->prava != 100  &&  $this ->prava;
    }


    /**
     * Vraci true, pokud ma uzivatel pravo jen na sebe
     * @return bool
     */
    function pouzeSebe()
    {
        return $this ->prava == 1;
    }


    /**
     * Vraci true, pokud ma uzivatel pravo i na sebe (tj. nektere z prav "... a sebe"). Pozor! Pro "pouze sebe" vraci false!
     * @return bool
     */
    function vcetneSebe()
    {
        return $this ->prava == 15 || $this ->prava == 25;
    }


    /**
     * Vraci false, pokud nema smysl zobrazit vyber rozdeleni - tj. pri nastaveni prav "pouze vlastni" a "zakazat"
     * "vlastni rozdeleni" a "vlastni rozdeleni krome sebe" - vraci true, protoze muze byt ve vice rozdelenich
     * @return bool
     */
    function zobrazitVyberRozdeleni()
    {
        if (!$this ->jsouNejakePrava()) {
            return false;
        }

        if ($this ->pouzeSebe()) {
            return false;
        }

        return true;
    }


    /**
     * @param int $idpp
     * @param int $datumOd
     * @param int $datumDo
     * @return bool
     * @throws \Exception
     */
    private function lzeZobrazit_VybraneSkupiny($idpp, $datumOd = 0, $datumDo = 0)
    {
        if (!$this ->seznamPP) {
            // parametry nepodstatne
            $this ->seznamPP = new SeznamPPProPrava(0, false);
        }

        if (!$this ->seznamPP ->jeSeznamNacten()) {
            $this ->seznamPP ->nactiVybraneSkupiny(Prava::$database, $this ->skupiny, $this ->osoby);
        }

        return $this ->seznamPP ->lzeZobrazitPP($idpp, $datumOd, $datumDo);
    }


    /**
     * @param string $rc
     * @param int $datumOd
     * @param int $datumDo
     * @return bool
     * @throws \Exception
     */
    private function lzeZobrazitRC_VybraneSkupiny($rc, $datumOd = 0, $datumDo = 0)
    {
        if (!$this ->seznamPP) {
            // parametry nepodstatne
            $this ->seznamPP = new SeznamPPProPrava(0, false);
        }

        if (!$this ->seznamPP ->jeSeznamNacten()) {
            $this ->seznamPP ->nactiVybraneSkupiny(Prava::$database, $this ->skupiny, $this ->osoby);
        }

        return $this ->seznamPP ->lzeZobrazitRC($rc, $datumOd, $datumDo);
    }


    /**
     * Priznak 3 se casto pouziva pro vyjmenovani povolenych kodu (operaci atd.).
     * Tato funkce vraci true, pokud priznak 3 obsahuje zadany kod.
     * Pokud neni priznak3 vyplnen, tak vraci $defaultValue
     * @param $kod
     * @param bool|true $defaultValue
     * @return bool
     */
    public function priznak3ObsahujeKod($kod, $defaultValue = true)
    {
        if (!$this ->priznak3) {
            return $defaultValue;
        }

        $kody = explode(';', $this ->priznak3);
        return in_array($kod, $kody);
    }


    /**
     * @param $idpp
     * @param int $datumOd
     * @param int $datumDo
     * @return bool
     * @throws DebugException
     */
    function lzeZobrazitPP($idpp, $datumOd = 0, $datumDo = 0)
    {
        if ($this ->prava == 3) {
            // muze zobrazit vsechny
            return true;
        }

        if ($this ->prava == 0  ||  $this ->prava == 100) {
            // nic
            return false;
        }

        if ($this ->prava == 1) {
            // jen sebe
            return Prava::jeVlastniPP($idpp);
        }

        if ($this ->prava == 4 || $this ->prava == 11 || $this ->prava == 13 || $this->prava == 23  ||  $this->prava == 26) {
            // ... krome sebe
            if (Prava::jeVlastniPP($idpp)) {
                // jinak pokracuje dale
                return false;
            }
        }

        if (($this ->prava == 15 || $this->prava == 25)  &&  Prava::jeVlastniPP($idpp)) {
            // podrizena a sebe, 25 primo podrizena a sebe
            return true;
        }

        if ($this ->zpusobKontrolyObdobi) {
            // prava se kontroluji vzdy k aktualnimu datu (ne podle agendy)
            $datumOd = time();
            $datumDo = time();
        } else if ($datumOd  &&  !$datumDo)  {
            $datumDo = $datumOd;
        }

        // 10 = vlastni rozdeleni, 11 = ... krome sebe
        // 12 = vlastni a podrizena rozdeleni, 13 = ... krome sebe
        // 14 = podrizena rozdeleni
        // 15 = podrizena rozdeleni a sebe
        if ($this ->prava >= 10 && $this ->prava <= 13) {
            if (Prava::ppJeVeVlastnimRozdeleni($idpp, $this ->typRozdeleni, $datumOd, $datumDo)) {
                return true;
            }

            if ($this ->prava == 10 || $this ->prava == 11) {
                return false;
            }
            // pro "vlastni a podrizena" pokracuje dale
        }

        // podrizena rozdeleni
        if ($this ->prava >= 12 && $this ->prava <= 15) {
            return Prava::ppJeVPodrizenemRozdeleni($idpp, $this ->typRozdeleni, $datumOd, $datumDo);
        }

        // 22 = vlastni a primo podrizena rozdeleni, 23 = ... krome sebe
        if ($this ->prava >= 22 && $this ->prava <= 23) {
            if (Prava::ppJeVeVlastnimRozdeleni($idpp, $this ->typRozdeleni, $datumOd, $datumDo)) {
                return true;
            }
            // pro "vlastni a primo podrizena" pokracuje dale
        }

        // primo podrizena rozdeleni
        // 24 = primo podrizena rozdeleni, 25 = ... a sebe, 26 = ... krome sebe
        if ($this ->prava >= 22 && $this ->prava <= 26) {
            return Prava::ppJeVPrimoPodrizenemRozdeleni($idpp, $this ->typRozdeleni, $datumOd, $datumDo);
        }

        if ($this ->prava == 2 || $this ->prava == 4) // vybrane osoby a skupiny ("krome sebe" vyreseno jiz drive)
        {
            return $this ->lzeZobrazit_VybraneSkupiny($idpp, $datumOd, $datumDo);
        }

        throw new DebugException('Chyba pri kontrole prav ve funkci LzeZobrazitPP - NENI IMPLEMENTOVANO! (' . $this ->prava . ')');
    }


    /**
     * @param string $rc
     * @param int $datumOd
     * @param int $datumDo
     * @return bool
     * @throws \Exception
     */
    function lzeZobrazitRC($rc, $datumOd = 0, $datumDo = 0)
    {
        if ($this ->prava == 3) {
            // muze zobrazit vsechny
            return true;
        }

        if ($this ->prava == 0  ||  $this ->prava == 100) {
            // nic
            return false;
        }

        if (User::hasRC($rc)) {
            switch ($this ->prava) {
                case 1 :    // jen sebe
                case 10 :   // vlastni rozdeleni
                case 12 :   // vlastni a podrizena
                case 15 :   // podrizena a sebe
                case 25 :   // primo podrizena a sebe
                    return true;

                case 4 :    // krome sebe ...
                case 11 :
                case 13 :
                case 23 :
                case 26 :
                    return false;
            }
        } else if ($this ->prava == 1) { // jen sebe
            return false;
        }

        if ($this ->zpusobKontrolyObdobi) {
            // prava se kontroluji vzdy k aktualnimu datu (ne podle agendy)
            $datumOd = time();
            $datumDo = time();
        }

        // 10 = vlastni rozdeleni, 11 = ... krome sebe
        // 12 = vlastni a podrizena rozdeleni, 13 = ... krome sebe
        // 14 = podrizena rozdeleni, 15 = ... a sebe
        if ($this ->prava >= 10 && $this ->prava <= 13) {
            if (Prava::rcJeVeVlastnimRozdeleni($rc, $this ->typRozdeleni, $datumOd, $datumDo)) {
                return true;
            }

            if ($this ->prava == 10 || $this ->prava == 11) return false;
            // pro "vlastni a podrizena" pokracuje dale
        }

        if ($this ->prava >= 12 && $this ->prava <= 15) {
            return Prava::rcJeVPodrizenemRozdeleni($rc, $this ->typRozdeleni, $datumOd, $datumDo);
        }

        // 22 = vlastni a primo podrizena rozdeleni, 23 = ... krome sebe
        if ($this ->prava >= 22 && $this ->prava <= 23) {
            if (Prava::rcJeVeVlastnimRozdeleni($rc, $this ->typRozdeleni, $datumOd, $datumDo)) {
                return true;
            }

            // pro "vlastni a primo podrizena" pokracuje dale
        }

        // 24 = primo podrizena rozdeleni, 25 = ... a sebe, 26 = ... krome sebe
        if ($this ->prava >= 22 && $this ->prava <= 26) {
            return Prava::rcJeVPrimoPodrizenemRozdeleni($rc, $this ->typRozdeleni, $datumOd, $datumDo);
        }

        if ($this ->prava == 2 || $this ->prava == 4) // vybrane osoby a skupiny ("krome sebe" vyreseno jiz drive)
        {
            return $this ->lzeZobrazitRC_VybraneSkupiny($rc, $datumOd, $datumDo);
        }

        throw new \Exception('Chyba pri kontrole prav ve funkci LzeZobrazitRC - NENI IMPLEMENTOVANO! (' . $this ->prava . ')');
    }


    /**
     * Vraci true, pokud ma uzivatel pravo na zobrazeni daneho rozdeleni
     * (pouziva se napr. v planovani kapacit, aby se urcilo, jestli muze videt cizi lidi, nebo v terminalech)
     * @param $typRozdeleni
     * @param $kod
     * @param int $datumOd
     * @param int $datumDo
     * @return bool
     * @throws DBException
     */
    function jePravoNaRozdeleni($typRozdeleni, $kod, $datumOd = 0, $datumDo = 0)
    {
        if ($this ->prava == 3) {
            // muze zobrazit vsechny
            return true;
        }

        if ($this ->prava == 0  ||  $this ->prava == 100  ||  $this ->prava == 1) {
            // nic nebo jen sebe
            return false;
        }

        if ($this ->prava == 2  ||  $this ->prava == 4) {
            // vybrane skupiny (krome sebe)
            return false;
        }

        if (!isset($kod)  ||  !isset($typRozdeleni)) {
            return false;
        }

        if ($this ->prava == 10 || $this ->prava == 11) {
            $vcetneNadrazenych = 0; // vlastni rozdeleni
        }
        else if ($this ->prava == 12 || $this ->prava == 13) {
            $vcetneNadrazenych = 1; // vlastni a podrizena
        }
        else if ($this ->prava == 14 || $this ->prava == 15) {
            $vcetneNadrazenych = 2; // podrizena
        }
        else if ($this->prava == 22 || $this->prava == 23) {
            $vcetneNadrazenych = 4; // vlastni a primo podrizena
        }
        else if ($this->prava == 24 || $this->prava == 25 || $this->prava == 26) {
            $vcetneNadrazenych = 5; // primo podrizena
        }
        else {
            return false;
        }

        if ($this ->zpusobKontrolyObdobi) {
            // prava se kontroluji vzdy k aktualnimu datu (ne podle agendy)
            $datumOd = time();
            $datumDo = time();
        } else {
            if (!$datumOd) {
                $datumOd = SeznamPPProPrava::$obdobiPravaOd;
            }
            if (!$datumDo) {
                $datumDo = SeznamPPProPrava::$obdobiPravaDo;
            }
        }

        return Rozdeleni::jeVRozdeleni(Prava::$database, User::getRC(), null, $typRozdeleni, $kod, $vcetneNadrazenych, $datumOd, $datumDo);
    }


    /**
     * Vraci SQL filtr pro odfiltrovani zaznamu zadane tabulky (ta musi obsahovat pole RC, OsobniCislo, DatumNastupu) podle tohoto prava ($this).
     * @param string $jmenoTabulky      Jmeno tabulky, jejiz zaznamy se filtruji (napr. PracovniPomer)
     * @param int|string    $datumOd    Pocatek obdobi, ve kterem se prava kontroluji. Pokud je zadan string, tak se primo pouzije v SQL dotazu (napr. Absence.DatumOd).
     * @param int|string    $datumDo    Konec obdobi, ve kterem se prava kontroluji. Stejne jako u $datumOd muze byt zadan string.
     * @param Parameters    $params     Pokud jsou zadany parametry, tak se pouziji pro vkladani promennych. Jinak se vytvari dotaz s Ad Hoc parametry
     * @param bool  $vynechatVcetneSebe     Pro prava "... vcetne sebe" nepripojuje podminku na vlastni RC (protoze je to na SQL serveru pomale)
     * @return string
     */
    function sqlFiltr($jmenoTabulky, $datumOd, $datumDo, $params = null, $vynechatVcetneSebe = false, $jenRC = false)
    {
        if ($this ->prava == 3) {
            // vse
            return ' (1 = 1) ';
        }

        if ($this ->prava == 0  ||  $this ->prava == 100) {
            // nic
            return ' (0 = 1) ';
        }

        // jen sebe
        if ($this ->prava == 1) return $params
            ? " ($jmenoTabulky.RC = " . $params ->addParameter(User::getRC()). ") "
            : " ($jmenoTabulky.RC = '" . Prava::$database ->addSlashes(User::getRC()) . "') ";

        // krome sebe
        $filtrRC = ($this ->prava == 11 || $this ->prava == 13 || $this ->prava == 23 || $this ->prava == 26)
            ? "$jmenoTabulky.RC <> " . ($params
                ? $params ->addParameter(User::getRC())
                : "'" . Prava::$database ->addSlashes(User::getRC()) . "'")
            : null;

        if (!$datumOd  ||  $this ->zpusobKontrolyObdobi) {
            $datumOd = time();
        }

        if (!$datumDo  ||  $this ->zpusobKontrolyObdobi) {
            $datumDo = $datumOd;
        }

        // vlastni rozdeleni [krome sebe]
        if ($this ->prava == 10 || $this ->prava == 11) {
            $filtr =
                ' EXISTS (' .
                'SELECT RO_Ja.Rozdeleni FROM RozdeleniOsob RO_Ja, RozdeleniOsob RO_On ' .
                " WHERE RO_On.RC = $jmenoTabulky.RC " .
                (!$jenRC
                    ? " AND RO_On.OsobniCislo = $jmenoTabulky.OsobniCislo " .
                      " AND RO_On.DatumNastupu = $jmenoTabulky.DatumNastupu "
                    : '') .
                '   AND RO_Ja.RC = ' . ($params ? $params ->addParameter(User::getRC()): "'" . Prava::$database ->addSlashes(User::getRC()) . "'") .
                '   AND RO_Ja.KodTypuRozdeleni = ' . (int)$this ->typRozdeleni .
                '   AND RO_On.KodTypuRozdeleni = RO_Ja.KodTypuRozdeleni ' .
                '   AND RO_On.Rozdeleni = RO_Ja.Rozdeleni ' .
                '   AND RO_On.PlatnostOd <= ' . (is_string($datumDo) ? $datumDo : ($params ? $params ->addDateParameter($datumDo) : Prava::$database ->dateToString($datumDo))) .
                '   AND (RO_On.PlatnostDo IS NULL  OR  RO_On.PlatnostDo >= ' . (is_string($datumOd) ? $datumOd : ($params ? $params ->addDateParameter($datumOd) : Prava::$database ->dateToString($datumOd))) . ')' .
                '   AND RO_Ja.PlatnostOd <= ' . (is_string($datumDo) ? $datumDo : ($params ? $params ->addDateParameter($datumDo) : Prava::$database ->dateToString($datumDo))) .
                '   AND (RO_Ja.PlatnostDo IS NULL  OR  RO_Ja.PlatnostDo >= ' . (is_string($datumOd) ? $datumOd : ($params ? $params ->addDateParameter($datumOd) : Prava::$database ->dateToString($datumOd))) . ')' .
                ') ';

            return ' (' . SQLUtils::concatenateWherePartsOfSql( [$filtrRC, $filtr] ) . ') ';
        }

        // [vlastni a] podrizena rozdeleni [krome/a sebe]
        if ($this ->prava == 12 || $this ->prava == 13 || $this ->prava == 14 || $this ->prava == 15) {
            $filtr =
                ' EXISTS (' .
                'SELECT RO_Ja.Rozdeleni FROM RozdeleniOsob RO_Ja, RozdeleniOsob RO_On, HierarchieRozdeleni H ' .
                " WHERE RO_On.RC = $jmenoTabulky.RC " .
                (!$jenRC
                    ? " AND RO_On.OsobniCislo = $jmenoTabulky.OsobniCislo " .
                      " AND RO_On.DatumNastupu = $jmenoTabulky.DatumNastupu "
                    : '') .
                '   AND RO_Ja.RC = ' . ($params ? $params ->addParameter(User::getRC()) : "'" . Prava::$database ->addSlashes(User::getRC()). "'")  .

                '   AND RO_Ja.KodTypuRozdeleni = ' . (int)$this ->typRozdeleni .

                '   AND H.TypNadrazene = RO_Ja.KodTypuRozdeleni ' .
                '   AND H.KodNadrazene = RO_Ja.Rozdeleni ' .
                '   AND H.TypPodrizene = RO_On.KodTypuRozdeleni ' .
                '   AND H.KodPodrizene = RO_On.Rozdeleni ' .

                '   AND RO_On.PlatnostOd <= ' . (is_string($datumDo) ? $datumDo : ($params ? $params ->addDateParameter($datumDo) : Prava::$database ->dateToString($datumDo))) .
                '   AND (RO_On.PlatnostDo IS NULL  OR  RO_On.PlatnostDo >= ' . (is_string($datumOd) ? $datumOd : ($params ? $params ->addDateParameter($datumOd) : Prava::$database ->dateToString($datumOd))) . ')' .

                '   AND RO_Ja.PlatnostOd <= ' . (is_string($datumDo) ? $datumDo : ($params ? $params ->addDateParameter($datumDo) : Prava::$database ->dateToString($datumDo))) .
                '   AND (RO_Ja.PlatnostDo IS NULL  OR  RO_Ja.PlatnostDo >= ' . (is_string($datumOd) ? $datumOd : ($params ? $params ->addDateParameter($datumOd) : Prava::$database ->dateToString($datumOd))) . ')';

            if ($this ->prava == 14 || $this ->prava == 15) {
                // jen podrizene (bez vlastnich)
                $filtr .= ' AND RO_Ja.Rozdeleni <> RO_On.Rozdeleni ';
            }

            if (!$vynechatVcetneSebe  &&  $this ->prava == 15) {
                // podrizena a sebe
                // Optimalizace. Puvodni OR byl prilis pomaly.
                $filtr .= " UNION SELECT RC FROM Osoba O2 WHERE O2.RC = $jmenoTabulky.RC AND O2.RC = " . ($params ? $params ->addParameter(User::getRC()) : "'" . Prava::$database ->addSlashes(User::getRC()) . "'");
            }

            $filtr .=
                ') ';


//            if ($this ->prava == 15) {
//                // podrizena a sebe - predelano na union v EXISTS
//                return " ( ($jmenoTabulky.RC = '" . Prava::$database ->addSlashes(User::getRC()) . "') OR $filtr ) ";
//            }

            return ' (' . SQLUtils::concatenateWherePartsOfSql( [$filtrRC, $filtr] ) . ') ';
        }

        // [vlastni a] primo podrizena rozdeleni [krome/a sebe]
        if ($this ->prava == 22 || $this ->prava == 23 || $this ->prava == 24 || $this ->prava == 25 || $this ->prava == 26) {
            $filtr =
                ' EXISTS (' .
                'SELECT RO_Ja.RC FROM RozdeleniOsob RO_Ja, RozdeleniOsob RO_On, Rozdeleni R_Pod ' .
                " WHERE RO_On.RC = $jmenoTabulky.RC " .
                (!$jenRC
                    ? " AND RO_On.OsobniCislo = $jmenoTabulky.OsobniCislo " .
                      " AND RO_On.DatumNastupu = $jmenoTabulky.DatumNastupu "
                    : '') .
                '   AND RO_Ja.RC = ' . ($params ? $params ->addParameter(User::getRC()) : "'" . Prava::$database ->addSlashes(User::getRC()). "'")  .

                '   AND RO_Ja.KodTypuRozdeleni = ' . (int)$this ->typRozdeleni .
                '   AND RO_On.KodTypuRozdeleni = RO_Ja.KodTypuRozdeleni' .

                '   AND R_Pod.KodTypuRozdeleni = RO_On.KodTypuRozdeleni ' .
                '   AND R_Pod.Kod = RO_On.Rozdeleni ';

            if ($this ->prava == 22 || $this ->prava == 23) {
                //vlastni a primo podrizena, vazba muze byt na podrizene, nebo primo na rozdeleni
                $filtr .=
                    ' AND ((R_Pod.KodTypuRozdeleni = RO_Ja.KodTypuRozdeleni ' .
                    '           AND R_Pod.Kod = RO_Ja.Rozdeleni) ' .
                    '   OR (R_Pod.RozdeleniRodice = RO_Ja.KodTypuRozdeleni ' .
                    '           AND R_Pod.Rodic = RO_Ja.Rozdeleni)) ';
            }
            else {
                //primo podrizena rozdeleni
                $filtr .=
                    ' AND (R_Pod.RozdeleniRodice = RO_Ja.KodTypuRozdeleni ' .
                    '           AND R_Pod.Rodic = RO_Ja.Rozdeleni) ';
            }

            $filtr .=
                '   AND RO_On.PlatnostOd <= ' . (is_string($datumDo) ? $datumDo : ($params ? $params ->addDateParameter($datumDo) : Prava::$database ->dateToString($datumDo))) .
                '   AND (RO_On.PlatnostDo IS NULL  OR  RO_On.PlatnostDo >= ' . (is_string($datumOd) ? $datumOd : ($params ? $params ->addDateParameter($datumOd) : Prava::$database ->dateToString($datumOd))) . ')' .

                '   AND RO_Ja.PlatnostOd <= ' . (is_string($datumDo) ? $datumDo : ($params ? $params ->addDateParameter($datumDo) : Prava::$database ->dateToString($datumDo))) .
                '   AND (RO_Ja.PlatnostDo IS NULL  OR  RO_Ja.PlatnostDo >= ' . (is_string($datumOd) ? $datumOd : ($params ? $params ->addDateParameter($datumOd) : Prava::$database ->dateToString($datumOd))) . ')';

            if (!$vynechatVcetneSebe  &&  $this ->prava == 25) {
                // primo podrizena a sebe
                // Optimalizace. Puvodni OR byl prilis pomaly.
                $filtr .= " UNION SELECT RC FROM Osoba O2 WHERE O2.RC = $jmenoTabulky.RC AND O2.RC = " . ($params ? $params ->addParameter(User::getRC()) : "'" . Prava::$database ->addSlashes(User::getRC()) . "'");
            }

            $filtr .=
                ') ';

            return ' (' . SQLUtils::concatenateWherePartsOfSql( [$filtrRC, $filtr] ) . ') ';
        }

        if ($this ->prava == Prava::PRAVA_VYBRANE_OSOBY_A_SKUPINY|| $this ->prava == Prava::PRAVA_VYBRANE_KROME_SEBE) {
            // vybrane osoby a skupiny ( 4 = krome sebe )

            $filtrOsoby = '';

            $pomArray = explode(';', $this ->osoby ?? '');
            foreach ($pomArray as $osoba) {
                if (trim($osoba) != '') {
                    $filtrOsoby .=
                        ($filtrOsoby ? ' OR ' : '') .
                        "$jmenoTabulky.RC = " . ($params ? $params ->addParameter($osoba) : "'" . Prava::$database ->addSlashes($osoba) . "'");
                }
            }

            $filtrSkupiny = '';
            $pomArray = explode(';', $this ->skupiny ?? '');
            foreach ($pomArray as $skupina) {
                if (trim($skupina) != '') {
                    $filtrSkupiny .=
                        ($filtrSkupiny ? ' OR ' : '') .
                        'PrirazeniOsobDoPristSkup.Kod = ' . ($params ? $params ->addParameter($skupina) : "'" . Prava::$database ->addSlashes($skupina) . "'");
                }
            }

            if ($filtrSkupiny) {
                $filtrSkupiny =
                    ' EXISTS (' .
                    'SELECT PrirazeniOsobDoPristSkup.Kod FROM PrirazeniOsobDoPristSkup' .
                    ' WHERE (' . $filtrSkupiny . ')' .
                    "   AND PrirazeniOsobDoPristSkup.RC = $jmenoTabulky.RC" .
                    (!$jenRC
                        ? " AND PrirazeniOsobDoPristSkup.OsobniCislo = $jmenoTabulky.OsobniCislo" .
                          " AND PrirazeniOsobDoPristSkup.DatumNastupu = $jmenoTabulky.DatumNastupu"
                        : '') .
                    '   AND PrirazeniOsobDoPristSkup.PlatnostOd <= ' . (is_string($datumDo)
                        ? $datumDo
                        : ($params ? $params ->addDateParameter($datumDo) : Prava::$database ->dateToString($datumDo))) .
                    '   AND (PrirazeniOsobDoPristSkup.PlatnostDo IS NULL  OR  PrirazeniOsobDoPristSkup.PlatnostDo >= ' . (is_string($datumOd)
                        ? $datumOd
                        : ($params ? $params ->addDateParameter($datumOd) : Prava::$database ->dateToString($datumOd))) .
                    '))';
            }

            if (!$filtrOsoby && !$filtrSkupiny) {
                return '(0 = 1)';
            }

            if ($filtrOsoby != '') {
                if ($filtrSkupiny) {
                    $filtr = " (($filtrOsoby)  OR  ($filtrSkupiny)) ";
                } else {
                    $filtr = '(' . $filtrOsoby . ')';
                }

            } else {
                $filtr = $filtrSkupiny;
            }

            if ($this ->prava == Prava::PRAVA_VYBRANE_KROME_SEBE) {
                $filtr = " ( $jmenoTabulky.RC <> '" . Prava::$database ->addSlashes(User::getRC()) . "'  AND  ($filtr)) ";
            }

            return $filtr;
        }

        MessageManager::addErrorMessage("Chyba ve funkci PravaNaOsoby::SQLFiltr - pro typ prav {$this ->prava} neni funkce implementovana!");
        return ' (0 = 1) ';
    }


    /**
     * @param int|string $date
     * @param Parameters|null $params
     * @return string
     */
    private function getDateParam($date, $params)
    {
        if ($this ->zpusobKontrolyObdobi === self::OBDOBI_PODLE_AKTUALNIHO_DNE) {
            // obdobi se kontroluje podle aktualniho dne - vzdy pouzije aktualni cas
            return $params
                ? $params ->addDateParameter(time())
                : Prava::$database ->dateToString(time());
        }

        // obdobi se kontroluje podle agendy - pouzije zadany cas
        if (is_string($date)) {
            //  je zadany string - napriklad nazev jineho pole z databaze - vraci primo tento string
            return $date;
        }

        return $params
            ? $params ->addDateParameter($date)
            : Prava::$database ->dateToString($date);
    }

    /**
     * @param int $datumOd
     * @param int $datumDo
     * @param Parameters $params    Parametry - pro kompatibilitu se starym kodem je defaultni hodnota null, ale jinak je zadani parametru povinne
     * @param int $omezitVyber  Urcuje, co ma udelat v pripade, ze nelze filtrovat podle rozdleeni - 0 - vypise chybu a nezobrazi data, 1 - nezobrazi data, 2 - neomezi SQL dotaz
     * @return string
     */
    function sqlfiltrRozdeleni($datumOd, $datumDo, $params = null, $omezitVyber = 0)
    {
        if (!$params instanceof Parameters) {
            $omezitVyber = $params;  // pro kompatibilitu se starym kodem
            $params = null;
        }

        if (!$params  &&  ($GLOBALS['development'] ?? false)) {
            MessageManager::addWarning('DEV: V metode Prava::sqlfiltrRozdeleni nejsou zadany parametry!');
        }

        $result = '';

        $datumOd = $datumOd ?: time();
        $datumDo = $datumDo ?: $datumOd;

        switch ($this->prava) {

            case 3  :
                $result = ' 1 = 1';
                break;
            case Prava::PRAVA_POUZE_VLASTNI : // 1
            case PRAVA::VLASTNI_ROZDELENI : // 10
            case PRAVA::VLASTNI_ROZDELENI_KROME_SEBE : // 11
            case PRAVA::VLASTNI_A_PODRIZENA_ROZDELENI : // 12
            case PRAVA::VLASTNI_A_PODRIZENA_ROZDELENI_KROME_SEBE : // 13
            case PRAVA::PODRIZENA_ROZDELENI : // 14
            case PRAVA::PODRIZENA_ROZDELENI_A_SEBE : // 15
            case PRAVA::VLASTNI_A_PRIMO_PODRIZENA_ROZDELENI : // 22
            case PRAVA::VLASTNI_A_PRIMO_PODRIZENA_ROZDELENI_KROME_SEBE : // 23
            case PRAVA::PRIMO_PODRIZENA_ROZDELENI : // 24
            case PRAVA::PRIMO_PODRIZENA_ROZDELENI_A_SEBE : // 25
            case PRAVA::PRIMO_PODRIZENA_ROZDELENI_KROME_SEBE : // 26
                if ($this->prava == 1 || $this->prava == 10 || $this->prava == 11 || $this->prava == 12 || $this->prava == 13
                    || $this->prava == 22 || $this->prava == 23) {
                    // vlastni [a ...]
                    $result =
                        ' EXISTS(SELECT ROUzivatele.RC ' .
                        "\n   FROM RozdeleniOsob ROUzivatele JOIN PracovniPomer PPUzivatele ON ((PPUzivatele.RC = ROUzivatele.RC) " .
                        '      AND (PPUzivatele.OsobniCislo = ROUzivatele.OsobniCislo) AND (PPUzivatele.DatumNastupu = ROUzivatele.DatumNastupu))' .
                        "\n   WHERE ROUzivatele.RC = " . ($params ? $params ->addParameter(User::getRC()) : "'" . Prava::$database->addSlashes(User::getRC()) . "'") .
                        '       AND ROUzivatele.KodTypuRozdeleni = ' . (int)$this->typRozdeleni .
                        "       AND ROUzivatele.Rozdeleni = Rozdeleni.Kod" .
                        '       AND ROUzivatele.KodTypuRozdeleni = Rozdeleni.KodTypuRozdeleni ' .
                        '       AND ROUzivatele.PlatnostOd <= ' . $this ->getDateParam($datumDo, $params) .
                        '       AND (ROUzivatele.PlatnostDo >= ' . $this ->getDateParam($datumOd, $params) . ' OR ROUzivatele.PlatnostDo IS NULL)' .
                        //'       AND PPUzivatele.DatumNastupu <= ' . Prava::$database->dateToString($datumDo) .            zruseno 02-2024 - kontroluje se jen prirazeni na rozdeleni
                        '       AND (PPUzivatele.DatumUkonceni >= ' . $this ->getDateParam($datumOd, $params) . ' OR PPUzivatele.DatumUkonceni IS NULL)' .
                        "\n )\n";
                }
                if ($this->prava == 12 || $this->prava == 13 || $this->prava == 22 || $this->prava == 23) {
                    $result .= "\n OR\n";
                }
                if ($this->prava == 12 || $this->prava == 13 || $this->prava == 14 || $this->prava == 15) {
                    // [vlastni a] podrizena
                    $result .=
                        ' EXISTS(SELECT ROUzivatele.RC ' .
                        "\n    FROM RozdeleniOsob ROUzivatele JOIN PracovniPomer PPUzivatele ON ((PPUzivatele.RC = ROUzivatele.RC) " .
                        '       AND (PPUzivatele.OsobniCislo = ROUzivatele.OsobniCislo) AND (PPUzivatele.DatumNastupu = ROUzivatele.DatumNastupu))' .
                        '     JOIN HierarchieRozdeleni ON ((HierarchieRozdeleni.typNadrazene = ROUzivatele.KodTypuRozdeleni) ' .
                        '       AND (HierarchieRozdeleni.kodNadrazene = ROUzivatele.Rozdeleni)) ' .
                        "\n    WHERE ROUzivatele.RC = " . ($params ? $params ->addParameter(User::getRC()) : "'" . Prava::$database->addSlashes(User::getRC()) . "'") .
                        '       AND ROUzivatele.KodTypuRozdeleni = ' . (int)$this->typRozdeleni .
                        '       AND ROUzivatele.PlatnostOd <= ' . $this ->getDateParam($datumDo, $params) .
                        '       AND (ROUzivatele.PlatnostDo >= ' . $this ->getDateParam($datumOd, $params) . ' OR ROUzivatele.PlatnostDo IS NULL)' .
                        '       AND HierarchieRozdeleni.typPodrizene = ROUzivatele.KodTypuRozdeleni ' .
                        '       AND (HierarchieRozdeleni.typPodrizene = Rozdeleni.KodTypuRozdeleni AND HierarchieRozdeleni.kodPodrizene = Rozdeleni.Kod)' .
                        '       AND HierarchieRozdeleni.kodPodrizene <> HierarchieRozdeleni.kodNadrazene' .
                        //'       AND PPUzivatele.DatumNastupu <= ' . Prava::$database->dateToString($datumDo) .            zruseno 02-2024 - kontroluje se jen prirazeni na rozdeleni
                        '       AND (PPUzivatele.DatumUkonceni >= ' . $this ->getDateParam($datumOd, $params) . ' OR PPUzivatele.DatumUkonceni IS NULL)' .
                        "\n)\n";
                }
                if ($this->prava == 22 || $this->prava == 23 || $this->prava == 24 || $this->prava == 25 || $this->prava == 26) {
                    // [vlastni a] primo podrizena
                    $result .=
                        ' EXISTS(SELECT ROUzivatele.RC ' .
                        "\n   FROM RozdeleniOsob ROUzivatele JOIN PracovniPomer PPUzivatele ON ((PPUzivatele.RC = ROUzivatele.RC) " .
                        '      AND (PPUzivatele.OsobniCislo = ROUzivatele.OsobniCislo) AND (PPUzivatele.DatumNastupu = ROUzivatele.DatumNastupu))' .
                        '     JOIN Rozdeleni R_Pod ON ((R_Pod.RozdeleniRodice = ROUzivatele.KodTypuRozdeleni) AND ' .
                        '       (R_Pod.Rodic = ROUzivatele.Rozdeleni)) ' .
                        "\n    WHERE ROUzivatele.RC = " . ($params ? $params ->addParameter(User::getRC()) : "'" . Prava::$database->addSlashes(User::getRC()) . "'") .
                        '       AND ROUzivatele.KodTypuRozdeleni = ' . (int)$this->typRozdeleni .
                        "       AND R_Pod.Kod = Rozdeleni.Kod" .
                        '       AND R_Pod.KodTypuRozdeleni = Rozdeleni.KodTypuRozdeleni ' .
                        '       AND ROUzivatele.PlatnostOd <= ' . $this ->getDateParam($datumDo, $params) .
                        '       AND (ROUzivatele.PlatnostDo >= ' . $this ->getDateParam($datumOd, $params) . ' OR ROUzivatele.PlatnostDo IS NULL)' .

                        //'       AND PPUzivatele.DatumNastupu <= ' . Prava::$database->dateToString($datumDo) .            zruseno 02-2024 - kontroluje se jen prirazeni na rozdeleni
                        '       AND (PPUzivatele.DatumUkonceni >= ' . $this ->getDateParam($datumOd, $params) . ' OR PPUzivatele.DatumUkonceni IS NULL)' .
                        ')';
                }
                break;

            case Prava::PRAVA_ZAKAZAT :
                $result = '0 = 1';
                break;

            default :
                $result = ($omezitVyber == 0 || $omezitVyber == 1)
                    ? '0 = 1'
                    : '1 = 1';

                if ($omezitVyber == 0) {
                    MessageManager::addDebugLog("Hodnota prava: " . $this->prava);
                    MessageManager::addErrorMessage('Chybne nastavene pravo na rozdeleni.');
                }

                break;
        }

        return $result;
    }

    /**
     * Vraci SQL dotaz pro vyber rozdeleni zadaneho typu tak, ze odfiltruje pouze rozdeleni osob, na ktere mam pravo a nadrazene (aby mohl filtrovat hromadne)
     * @param Database $database
     * @param string $selectPart    Seznam poli pro vyber (z tabulky Rozdeleni)
     * @param int $typRozdeleni     Kod typu rozdeleni
     * @param int $datumOd          Pocatek obdobi v nemz musi byt platne prirazeni na rozdeleni
     * @param int $datumDo          Konec obdobi, v nemz musi byt platne prirazeni na rozdeleni
     * @param bool $zobrazitNadrazene   Urcuje, jestli se maji nacist i nadrazena rozdeleni
     * @return SqlStatement|string
     */
    function sqlRozdeleni(Database $database, $selectPart, $typRozdeleni, $datumOd, $datumDo, $zobrazitNadrazene = true)
    {
        if ($this ->prava == Prava::PRAVA_ZAKAZAT  ||  $this ->prava == 100) {
            // nic
            return "SELECT DISTINCT $selectPart FROM Rozdeleni WHERE 0 = 1";
        }

        // pokud se maji prava kontrolavat vzdy podle aktualniho datumu, tak upravi datum
        $datumOd = (!$this ->zpusobKontrolyObdobi  &&  $datumOd)
            ? $datumOd
            : time();
        $datumDo = (!$this ->zpusobKontrolyObdobi  &&  $datumDo)
            ? $datumDo
            : $datumOd;

        // POZOR! Nekontroluje se, jestli na danem rozdeleni nejsou jen osoby, ktere nemaji zaskrtnuto "zpracovavat dochazku"
        if ($this ->prava == Prava::PRAVA_POUZE_VLASTNI) {
            // jen sebe - neni co resit, vybere jen uzivatelovo a nadrazene
            $params = $database ->createParameters();

            if ($zobrazitNadrazene) {
                return new SqlStatement(
                    'SELECT DISTINCT ' . $selectPart .
                    ' FROM Rozdeleni, RozdeleniOsob, HierarchieRozdeleni ' .
                    ' WHERE RozdeleniOsob.RC = ' . $params->addParameter(User::getRC()) .  // uzivatelovo platne RozdeleniOsob
                    '   AND RozdeleniOsob.KodTypuRozdeleni = ' . (int)$typRozdeleni .
                    '   AND RozdeleniOsob.PlatnostOd <= ' . $params->addDateParameter($datumDo) .
                    '   AND (RozdeleniOsob.PlatnostDo >= ' . $params->addDateParameter($datumOd) . ' OR RozdeleniOsob.PlatnostDo IS NULL) ' .
                    '   AND HierarchieRozdeleni.TypPodrizene = RozdeleniOsob.KodTypuRozdeleni ' . // pres hierarchii hledam vsechny nadrazene
                    '   AND HierarchieRozdeleni.KodPodrizene = RozdeleniOsob.Rozdeleni ' .
                    '   AND HierarchieRozdeleni.TypNadrazene = Rozdeleni.KodTypuRozdeleni ' .
                    '   AND HierarchieRozdeleni.KodNadrazene = Rozdeleni.Kod ' .
                    '   AND (Rozdeleni.DatumZruseni >= ' . $params->addDateParameter($datumOd) . ' OR Rozdeleni.DatumZruseni IS NULL)',
                    $params);
            } else {
                return new SqlStatement(
                    'SELECT DISTINCT ' . $selectPart .
                    ' FROM Rozdeleni, RozdeleniOsob ' .
                    ' WHERE RozdeleniOsob.RC = ' . $params->addParameter(User::getRC()) .  // uzivatelovo platne RozdeleniOsob
                    '   AND RozdeleniOsob.KodTypuRozdeleni = ' . (int)$typRozdeleni .
                    '   AND RozdeleniOsob.PlatnostOd <= ' . $params->addDateParameter($datumDo) .
                    '   AND (RozdeleniOsob.PlatnostDo >= ' . $params->addDateParameter($datumOd) . ' OR RozdeleniOsob.PlatnostDo IS NULL) ' .
                    '   AND RozdeleniOsob.KodTypuRozdeleni = Rozdeleni.KodTypuRozdeleni ' .
                    '   AND RozdeleniOsob.Rozdeleni = Rozdeleni.Kod ' .
                    '   AND (Rozdeleni.DatumZruseni >= ' . $params->addDateParameter($datumOd) . ' OR Rozdeleni.DatumZruseni IS NULL)',
                    $params);
            }
        }

        if ( $this ->prava == Prava::VLASTNI_ROZDELENI || $this ->prava == Prava::VLASTNI_ROZDELENI_KROME_SEBE ) {
            // vlastni rozdeleni [krome sebe]
            $params = $database ->createParameters();

            return new SqlStatement(
                'SELECT DISTINCT ' . $selectPart .
                ' FROM Rozdeleni, RozdeleniOsob ROPrava_Ja, RozdeleniOsob ROPrava_On, RozdeleniOsob RO_Vyber' . ($zobrazitNadrazene ? ', HierarchieRozdeleni H' : '') .
                // uzivatelovo platne RozdeleniOsob (typ rozdeleni podle prav)
                ' WHERE ROPrava_Ja.RC = ' . $params ->addParameter(User::getRC()) .
                '   AND ROPrava_Ja.KodTypuRozdeleni = ' . (int)$this ->typRozdeleni .
                '   AND ROPrava_Ja.PlatnostOd <= ' . $params ->addDateParameter($datumDo) .
                '   AND (ROPrava_Ja.PlatnostDo >= ' . $params ->addDateParameter($datumOd) . ' OR ROPrava_Ja.PlatnostDo IS NULL) ' .
                // uzivatele, na ktere mam pravo (jsou na stejnem rozdeleni - typ podle prav)
                '   AND ROPrava_On.KodTypuRozdeleni = ROPrava_Ja.KodTypuRozdeleni ' .
                '   AND ROPrava_On.Rozdeleni = ROPrava_Ja.Rozdeleni ' .
                '   AND ROPrava_On.PlatnostOd <= ' . $params ->addDateParameter($datumDo) .
                '   AND (ROPrava_On.PlatnostDo >= ' . $params ->addDateParameter($datumOd) . ' OR ROPrava_On.PlatnostDo IS NULL) ' .
                // jejich rozdeleni (zadany typ)
                '   AND RO_Vyber.RC = ROPrava_On.RC ' .
                '   AND RO_Vyber.OsobniCislo = ROPrava_On.OsobniCislo ' .
                '   AND RO_Vyber.DatumNastupu = ROPrava_On.DatumNastupu ' .
                '   AND RO_Vyber.KodTypuRozdeleni = ' . (int)$typRozdeleni .
                '   AND RO_Vyber.PlatnostOd <= ' . $params ->addDateParameter($datumDo) .
                '   AND (RO_Vyber.PlatnostDo >= ' . $params ->addDateParameter($datumOd) . ' OR RO_Vyber.PlatnostDo IS NULL) ' .
                ($zobrazitNadrazene
                    // pres hierarchii vybere nadrazene
                    ?
                        '   AND H.TypPodrizene = RO_Vyber.KodTypuRozdeleni ' .
                        '   AND H.KodPodrizene = RO_Vyber.Rozdeleni ' .
                        // vysledna tabulka
                        '   AND Rozdeleni.KodTypuRozdeleni = H.TypNadrazene ' .
                        '   AND Rozdeleni.Kod = H.KodNadrazene '
                    :
                        '   AND Rozdeleni.KodTypuRozdeleni = RO_Vyber.KodTypuRozdeleni ' .
                        '   AND Rozdeleni.Kod = RO_Vyber.Rozdeleni '
                ) .
                '   AND (Rozdeleni.DatumZruseni >= ' . $params ->addDateParameter($datumOd) . ' OR Rozdeleni.DatumZruseni IS NULL)',
                $params);
        }

        if ($this ->prava == 12 || $this ->prava == 13 || $this ->prava == 14 || $this ->prava == 15) { // 12=vlastni a podr, 13=...krome sebe, 14=podrizena, 15=... a sebe
            // [vlastni a] podrizena rozdeleni [krome/a sebe]
            $params = $database ->createParameters();

            return new SqlStatement(
                'SELECT DISTINCT ' . $selectPart .
                ' FROM Rozdeleni, RozdeleniOsob ROPrava_Ja, HierarchieRozdeleni HPrava, RozdeleniOsob ROPrava_On, RozdeleniOsob RO_Vyber' .
                ($zobrazitNadrazene ? ', HierarchieRozdeleni H' : '') . "\n" .
                // uzivatelovo platne RozdeleniOsob (typ rozdeleni podle prav)
                ' WHERE ROPrava_Ja.RC = ' . $params ->addParameter(User::getRC()) .
                '   AND ROPrava_Ja.KodTypuRozdeleni = ' . (int)$this ->typRozdeleni .
                '   AND ROPrava_Ja.PlatnostOd <= ' . $params ->addDateParameter($datumDo) .
                '   AND (ROPrava_Ja.PlatnostDo >= ' . $params ->addDateParameter($datumOd) . ' OR ROPrava_Ja.PlatnostDo IS NULL) ' .
                "\n" .
                // hierarchie
                '   AND HPrava.TypNadrazene = ROPrava_Ja.KodTypuRozdeleni ' .
                '   AND HPrava.KodNadrazene = ROPrava_Ja.Rozdeleni ' .
                "\n" .
                // uzivatele, na ktere mam pravo (jsou na stejnem nebo podrizenem rozdeleni - typ podle prav)
                '   AND ROPrava_On.KodTypuRozdeleni = HPrava.TypPodrizene ' .
                '   AND ROPrava_On.Rozdeleni = HPrava.KodPodrizene ' .
                '   AND ROPrava_On.PlatnostOd <= ' . $params ->addDateParameter($datumDo) .
                '   AND (ROPrava_On.PlatnostDo >= ' . $params ->addDateParameter($datumOd) . ' OR ROPrava_On.PlatnostDo IS NULL) ' .
                //( ($this ->prava == 14 || $this ->prava == 15) ? ' AND ROPrava_Ja.Rozdeleni <> ROPrava_On.Rozdeleni ' : '' ) . // jen podrizene - doplneno i pro 15, protoze jinak to zobrazovalo i jine vetve, na ktere uzivatel nema narok
                ( ($this ->typRozdeleni != $typRozdeleni) // 02-2024 - pridal jsem tuto podminku, jinak to vubec nezobrazovalo vlastni rozdeleni
                    ? ' AND ROPrava_Ja.Rozdeleni <> ROPrava_On.Rozdeleni ' // jen podrizene - doplneno i pro 15 ... a 12 a 13, protoze jinak to zobrazovalo i jine vetve, na ktere uzivatel nema narok
                    : ''
                ) .
                "\n" .
                // jejich rozdeleni (zadany typ)
                '   AND RO_Vyber.RC = ROPrava_On.RC ' .
                '   AND RO_Vyber.OsobniCislo = ROPrava_On.OsobniCislo ' .
                '   AND RO_Vyber.DatumNastupu = ROPrava_On.DatumNastupu ' .
                '   AND RO_Vyber.KodTypuRozdeleni = ' . (int)$typRozdeleni .
                '   AND RO_Vyber.PlatnostOd <= ' . $params ->addDateParameter($datumDo) .
                '   AND (RO_Vyber.PlatnostDo >= ' . $params ->addDateParameter($datumOd) . ' OR RO_Vyber.PlatnostDo IS NULL) ' .
                "\n" .
                ($zobrazitNadrazene
                    // pres hierarchii vybere nadrazene
                    ?
                    '   AND H.TypPodrizene = RO_Vyber.KodTypuRozdeleni ' .
                    '   AND H.KodPodrizene = RO_Vyber.Rozdeleni ' .
                    // vysledna tabulka
                    '   AND Rozdeleni.KodTypuRozdeleni = H.TypNadrazene ' .
                    '   AND Rozdeleni.Kod = H.KodNadrazene '
                    :
                    '   AND Rozdeleni.KodTypuRozdeleni = RO_Vyber.KodTypuRozdeleni ' .
                    '   AND Rozdeleni.Kod = RO_Vyber.Rozdeleni '
                ) .
                "\n" .
                '   AND (Rozdeleni.DatumZruseni >= ' . $params ->addDateParameter($datumOd) . ' OR Rozdeleni.DatumZruseni IS NULL)' .

                //( (!$zobrazitNadrazene  &&  $this ->prava == 15)
                ( (!$zobrazitNadrazene  &&  $this ->prava !== 14  &&  ($this ->typRozdeleni == $typRozdeleni  ||  $this ->prava == 15))  // 15=pokud je podrizena a sebe - staci kdyz prida rozdeleni uzivatele
                    ?   // jeste prida moje vlastni, protoze vyse je to odfiltrovane - tuto verzi pouzije, pokud jsou prava podle stejneho typu rozdeleni
                        "\nUNION\n" .
                        ' SELECT DISTINCT ' . $selectPart .
                        ' FROM Rozdeleni, RozdeleniOsob RO_Ja' .
                        ' WHERE RO_Ja.RC = ' . $params ->addParameter(User::getRC()) .
                        '   AND RO_Ja.KodTypuRozdeleni = ' . (int)$typRozdeleni .
                        '   AND RO_Ja.PlatnostOd <= ' . $params ->addDateParameter($datumDo) .
                        '   AND (RO_Ja.PlatnostDo >= ' . $params ->addDateParameter($datumOd) . ' OR RO_Ja.PlatnostDo IS NULL) ' .
                        '   AND Rozdeleni.KodTypuRozdeleni = RO_Ja.KodTypuRozdeleni ' .
                        '   AND Rozdeleni.Kod = RO_Ja.Rozdeleni '
                    :
                        ''
                ) .

                ( ($this ->prava < 14  &&  $this ->typRozdeleni != $typRozdeleni)
                    ?   // jeste prida moje vlastni, protoze vyse je to odfiltrovane - tuto verzi pouzije, pokud jsou prava podle jineho typu rozdeleni
                        "\nUNION\n" .
                        ' SELECT DISTINCT ' . $selectPart .
                        ' FROM Rozdeleni, RozdeleniOsob ROPrava_Ja, RozdeleniOsob ROPrava_On, RozdeleniOsob RO_Vyber' .
                        ($zobrazitNadrazene ? ', HierarchieRozdeleni H' : '') . "\n" .
                        // uzivatelovo platne RozdeleniOsob (typ rozdeleni podle prav)
                        ' WHERE ROPrava_Ja.RC = ' . $params ->addParameter(User::getRC()) .
                        '   AND ROPrava_Ja.KodTypuRozdeleni = ' . (int)$this ->typRozdeleni .
                        '   AND ROPrava_Ja.PlatnostOd <= ' . $params ->addDateParameter($datumDo) .
                        '   AND (ROPrava_Ja.PlatnostDo >= ' . $params ->addDateParameter($datumOd) . ' OR ROPrava_Ja.PlatnostDo IS NULL) ' .
                        "\n" .
                        // uzivatele, na ktere mam pravo (jsou na stejnem nebo podrizenem rozdeleni - typ podle prav)
                        '   AND ROPrava_On.KodTypuRozdeleni = ROPrava_Ja.KodTypuRozdeleni' .
                        '   AND ROPrava_On.Rozdeleni = ROPrava_Ja.Rozdeleni' .
                        '   AND ROPrava_On.PlatnostOd <= ' . $params ->addDateParameter($datumDo) .
                        '   AND (ROPrava_On.PlatnostDo >= ' . $params ->addDateParameter($datumOd) . ' OR ROPrava_On.PlatnostDo IS NULL) ' .
                        ' AND ROPrava_Ja.Rozdeleni = ROPrava_On.Rozdeleni ' .
                        "\n" .
                        // jejich rozdeleni (zadany typ)
                        '   AND RO_Vyber.RC = ROPrava_On.RC ' .
                        '   AND RO_Vyber.OsobniCislo = ROPrava_On.OsobniCislo ' .
                        '   AND RO_Vyber.DatumNastupu = ROPrava_On.DatumNastupu ' .
                        '   AND RO_Vyber.KodTypuRozdeleni = ' . (int)$typRozdeleni .
                        '   AND RO_Vyber.PlatnostOd <= ' . $params ->addDateParameter($datumDo) .
                        ($zobrazitNadrazene
                            // pres hierarchii vybere nadrazene
                            ?
                            '   AND H.TypPodrizene = RO_Vyber.KodTypuRozdeleni ' .
                            '   AND H.KodPodrizene = RO_Vyber.Rozdeleni ' .
                            // vysledna tabulka
                            '   AND Rozdeleni.KodTypuRozdeleni = H.TypNadrazene ' .
                            '   AND Rozdeleni.Kod = H.KodNadrazene '
                            :
                            '   AND Rozdeleni.KodTypuRozdeleni = RO_Vyber.KodTypuRozdeleni ' .
                            '   AND Rozdeleni.Kod = RO_Vyber.Rozdeleni '
                        ) .
                        "\n" .
                        '   AND (RO_Vyber.PlatnostDo >= ' . $params ->addDateParameter($datumOd) . ' OR RO_Vyber.PlatnostDo IS NULL) '
                    :
                        ''
                ) .

                '',

                $params);

        }

        if ($this ->prava == 22 || $this ->prava == 23 || $this ->prava == 24 || $this ->prava == 25 || $this->prava == 26) {
            // [vlastni a] primo podrizena rozdeleni [krome/a sebe]
            $params = $database ->createParameters();

            $sql =
                'SELECT DISTINCT ' . $selectPart . "\n" .
                ' FROM Rozdeleni, RozdeleniOsob ROPrava_Ja, Rozdeleni RPrava, RozdeleniOsob ROPrava_On, RozdeleniOsob RO_Vyber' . ($zobrazitNadrazene ? ', HierarchieRozdeleni H' : '') . "\n" .
                // uzivatelovo platne RozdeleniOsob (typ rozdeleni podle prav)
                ' WHERE ROPrava_Ja.RC = ' . $params ->addParameter(User::getRC()) .
                '   AND ROPrava_Ja.KodTypuRozdeleni = ' . (int)$this ->typRozdeleni .
                '   AND ROPrava_Ja.PlatnostOd <= ' . $params ->addDateParameter($datumDo) .
                '   AND (ROPrava_Ja.PlatnostDo >= ' . $params ->addDateParameter($datumOd) . ' OR ROPrava_Ja.PlatnostDo IS NULL) ' .
                // uzivatel je na nadrazenem rozdeleni - v RPrava.Kod jsou primo podrizena
                '   AND RPrava.RozdeleniRodice = ROPrava_Ja.KodTypuRozdeleni ' .
                '   AND RPrava.Rodic = ROPrava_Ja.Rozdeleni ' .
                // uzivatele, na ktere mam pravo (jsou na stejnem nebo primo podrizenem rozdeleni - typ podle prav)
                '   AND ' .
                (($this->prava == 22 || $this->prava == 23) ? ' (' : '') . // vlastni, bude se pres OR pripojovat RO_PravaOn
                '    (ROPrava_On.KodTypuRozdeleni = RPrava.KodTypuRozdeleni ' .
                '   AND ROPrava_On.Rozdeleni = RPrava.Kod) ' .
                (($this->prava == 22 || $this->prava == 23) ? ' OR (ROPrava_On.KodTypuRozdeleni = RPrava.RozdeleniRodice ' .
                    '   AND ROPrava_On.Rozdeleni = RPrava.Rodic))' : '') . // pro vlastni, muze byt RO_PravaOn i na nadrazenem rozdeleni

                '   AND ROPrava_On.PlatnostOd <= ' . $params ->addDateParameter($datumDo) .
                '   AND (ROPrava_On.PlatnostDo >= ' . $params ->addDateParameter($datumOd) . ' OR ROPrava_On.PlatnostDo IS NULL) ' .

                // jejich rozdeleni (zadany typ)
                '   AND RO_Vyber.RC = ROPrava_On.RC ' .
                '   AND RO_Vyber.OsobniCislo = ROPrava_On.OsobniCislo ' .
                '   AND RO_Vyber.DatumNastupu = ROPrava_On.DatumNastupu ' .
                '   AND RO_Vyber.KodTypuRozdeleni = ' . (int)$typRozdeleni .
                '   AND RO_Vyber.PlatnostOd <= ' . $params ->addDateParameter($datumDo) .
                '   AND (RO_Vyber.PlatnostDo >= ' . $params ->addDateParameter($datumOd) . ' OR RO_Vyber.PlatnostDo IS NULL) ' .
                ($zobrazitNadrazene
                    // pres hierarchii vybere nadrazene
                    ?
                        '   AND H.TypPodrizene = RO_Vyber.KodTypuRozdeleni ' .
                        '   AND H.KodPodrizene = RO_Vyber.Rozdeleni ' .
                        // vysledna tabulka
                        '   AND Rozdeleni.KodTypuRozdeleni = H.TypNadrazene ' .
                        '   AND Rozdeleni.Kod = H.KodNadrazene '
                    :
                        '   AND Rozdeleni.KodTypuRozdeleni = RO_Vyber.KodTypuRozdeleni ' .
                        '   AND Rozdeleni.Kod = RO_Vyber.Rozdeleni '
                ) .
                '   AND (Rozdeleni.DatumZruseni >= ' . $params ->addDateParameter($datumOd) . ' OR Rozdeleni.DatumZruseni IS NULL)';

            if ($this->prava == 25) {
                // UNIONEM prida jeste vlastni rozdeleni (protoze puvodni verze s OR je pomala)
                $sql .= "\nUNION\n" .

                'SELECT DISTINCT ' . $selectPart . "\n" .
                ' FROM Rozdeleni, RozdeleniOsob RO_Vyber' . ($zobrazitNadrazene ? ', HierarchieRozdeleni H' : '') . "\n" .
                // rozdeleni (zadany typ)
                ' WHERE RO_Vyber.RC = ' . $params ->addParameter(User::getRC()) .
                //'   AND RO_Vyber.OsobniCislo = ROPrava_On.OsobniCislo ' .
                //'   AND RO_Vyber.DatumNastupu = ROPrava_On.DatumNastupu ' .
                '   AND RO_Vyber.KodTypuRozdeleni = ' . (int)$typRozdeleni .
                '   AND RO_Vyber.PlatnostOd <= ' . $params ->addDateParameter($datumDo) .
                '   AND (RO_Vyber.PlatnostDo >= ' . $params ->addDateParameter($datumOd) . ' OR RO_Vyber.PlatnostDo IS NULL) ' .
                ($zobrazitNadrazene
                    // pres hierarchii vybere nadrazene
                    ?
                        '   AND H.TypPodrizene = RO_Vyber.KodTypuRozdeleni ' .
                        '   AND H.KodPodrizene = RO_Vyber.Rozdeleni ' .
                        // vysledna tabulka
                        '   AND Rozdeleni.KodTypuRozdeleni = H.TypNadrazene ' .
                        '   AND Rozdeleni.Kod = H.KodNadrazene '
                    :
                        '   AND Rozdeleni.KodTypuRozdeleni = RO_Vyber.KodTypuRozdeleni ' .
                        '   AND Rozdeleni.Kod = RO_Vyber.Rozdeleni '
                ) .
                '   AND (Rozdeleni.DatumZruseni >= ' . $params ->addDateParameter($datumOd) . ' OR Rozdeleni.DatumZruseni IS NULL)';
            }

            return new SqlStatement($sql, $params);
        }

        if ($this ->prava == 2  ||  $this ->prava == 4) { // vybrane osoby a skupiny / krome sebe
            $filtr = $this ->sqlFiltr('PracovniPomer', $datumOd, $datumDo);
            if ($filtr) {
                // vybrane osoby a skupiny - zobrazuje i nadrazena rozdeleni

                // filtr obsahuje subselect -> nepouzivam params
                return
                    'SELECT DISTINCT ' . $selectPart .
                    ' FROM Rozdeleni, RozdeleniOsob, PracovniPomer ' . ($zobrazitNadrazene ? ', HierarchieRozdeleni' : '') .
                    ' WHERE Rozdeleni.KodTypuRozdeleni = ' . (int)$typRozdeleni .
                    '   AND (Rozdeleni.DatumZruseni >= ' . Prava::$database ->dateToString($datumOd) . ' OR Rozdeleni.DatumZruseni IS NULL)' .
                    ($zobrazitNadrazene
                        ?
                            ' AND HierarchieRozdeleni.TypNadrazene = Rozdeleni.KodTypuRozdeleni ' .
                            ' AND HierarchieRozdeleni.KodNadrazene = Rozdeleni.Kod ' .
                            ' AND RozdeleniOsob.KodTypuRozdeleni = HierarchieRozdeleni.TypPodrizene' .
                            ' AND RozdeleniOsob.Rozdeleni = HierarchieRozdeleni.KodPodrizene'
                        :
                            ' AND RozdeleniOsob.KodTypuRozdeleni = Rozdeleni.KodTypuRozdeleni' .
                            ' AND RozdeleniOsob.Rozdeleni = Rozdeleni.Kod'
                    ) .
                    '   AND RozdeleniOsob.PlatnostOd <= ' . Prava::$database ->dateToString($datumDo) .
                    '   AND (RozdeleniOsob.PlatnostDo >= ' . Prava::$database ->dateToString($datumOd) . ' OR RozdeleniOsob.PlatnostDo IS NULL) ' .
                    '   AND PracovniPomer.RC = RozdeleniOsob.RC ' .
                    '   AND PracovniPomer.OsobniCislo = RozdeleniOsob.OsobniCislo ' .
                    '   AND PracovniPomer.DatumNastupu = RozdeleniOsob.DatumNastupu ' .
                    '   AND (' . $filtr . ')';
            }

        }

        // vse (prava = 3) a ostatni pripady
        $params = $database ->createParameters();
        return new SqlStatement(
            "SELECT DISTINCT $selectPart FROM Rozdeleni" .
            ' WHERE KodTypuRozdeleni = ' . (int)$typRozdeleni .
            ' AND (Rozdeleni.DatumZruseni >= ' . $params ->addDateParameter($datumOd) . ' OR Rozdeleni.DatumZruseni IS NULL)',
            $params);
    }

}
