<?php
/**
 * stanislav.elbl, 21.02.2021
 */

namespace RonPortal;


use Exception;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\Parameters;
use Ron\DB\SqlStatement;
use Ron\DB\SQLUtils;

class Rozdeleni
{
    /** @var int    Kod typu rozdeleni */
    var $typ;
    /** @var string Kod rozdeleni */
    var $kod;
    /** @var string Popis rozdeleni */
    var $popis;

    /** @var string Kod nadrazeneho rozdeleni */
    var $rodic;
    /**
     * @var int     Jen pro kompatibilitu! Nepouzivat
     * @deprecated
     */
    var $f_typ;
    /**
     * @var string  Jen pro kompatibilitu! Nepouzivat
     * @deprecated
     */
    var $f_kod;

    var $cssClass;

    var $datumVytvoreni;
    var $datumZruseni;
    var $stat;

    var $eMail;
    var $vedouciRC;

    /** @var string Kod prirazeneho kalendare */
    var $kalendar;

    private static function getSelectPart()
    {
        return
            'Rozdeleni.KodTypuRozdeleni, Rozdeleni.Kod, Rozdeleni.Popis, Rozdeleni.cssClass, ' . // 0..3
            'Rozdeleni.DatumVytvoreni, Rozdeleni.DatumZruseni, ' . // 4,5
            'Rozdeleni.Stat, Rozdeleni.eMail, Rozdeleni.Rodic, Rozdeleni.Kalendar, Rozdeleni.VedouciRC'; // 6..10
    }

    /**
     * @param Database $database
     * @param array $row
     */
    private function loadRow($database, $row)
    {
        $this ->typ = (int)$row[0];
        $this ->kod = $row[1];
        $this ->popis = $row[2];
        $this ->cssClass = $row[3];
        $this ->datumVytvoreni = $database ->stringToTimestamp($row[4]);
        $this ->datumZruseni = $database ->stringToTimestamp($row[5]);
        $this ->stat = $row[6];
        $this ->eMail = $row[7];
        $this ->rodic = $row[8];
        $this ->kalendar = $row[9];
        $this ->vedouciRC = $row[10];

        $this ->f_typ = $this ->typ;    // 12/2017 - pouze pro kompatibilitu pripadneho uzivatelskeho kodu
        $this ->f_kod = $this ->kod;
        //$this ->f_popis = $this ->popis;
    }

    /**
     * @param Database $database
     * @param int $typRozdeleni
     * @param string $kod
     * @throws DBException
     * @throws Exception
     */
    public function load($database, $typRozdeleni, $kod)
    {
        $params = $database ->createParameters();
        $sql =
            'SELECT ' . self::getSelectPart() .
            '  FROM Rozdeleni '.
            ' WHERE KodTypuRozdeleni = ' . $params ->addParameter((int)$typRozdeleni) .
            '   AND Kod = ' . $params ->addParameter($kod);

        $cursor = $database ->getSqlResult($sql, $params);
        $row = $cursor ->fetch();
        $cursor ->close();
        if (!$row) {
            throw new Exception('Zadane rozdeleni nebylo nalezeno!');
        }

        $this ->loadRow($database, $row);
    }


    /**
     * @param Database $database
     * @param int    $typRozdeleni
     * @param string $kod
     * @return Rozdeleni
     * @throws Exception
     */
    public static function loadRozdeleni($database, $typRozdeleni, $kod)
    {
        $result = new self();
        $result ->load($database, $typRozdeleni, $kod);
        return $result;
    }


    /**
     * @param Database $database
     * @param int $typRozdeleni
     * @param string $filter
     * @return Rozdeleni[]
     * @throws DBException
     */
    public static function loadSeznamRozdeleni($database, $typRozdeleni, $filter = null, $orderBy = 'Popis')
    {
        $params = $database ->createParameters();
        $sql =
            'SELECT ' . self::getSelectPart() .
            ' FROM Rozdeleni '.
            ' WHERE KodTypuRozdeleni = ' . $params ->addParameter((int)$typRozdeleni) .
            ($filter ? ' AND ' . $filter : '') .
            ' ORDER BY ' . $orderBy;

        $cursor = $database ->getSqlResult($sql, $params);

        $result = [];
        while ($row = $cursor ->fetch()) {
            $item = new self();
            $item ->loadRow($database, $row);
            $result[] = $item;
        }
        $cursor ->close();

        return $result;
    }


    /**
     * Nacte rozdeleni zadaneho typu pro zadany pracovni pomer.
     * @param Database $database
     * @param int|int[] $idPP
     * @param int $typRozdeleni
     * @param int $datumOd Obdobi, ve kterem ma byt osoba na danem rozdeleni. Pokud neni zadano, pouzije se time()
     * @param int $datumDo ... pokud neni zadano, nastavi se na stejnou hodnotu jako $datumOd
     * @param int $vcetnePodrizenych  0=jen prirazene, 1=vcetne podrizenych, 2=jen podrizene
     * @return Rozdeleni[]
     * @throws DBException
     */
    public static function loadRozdeleniOsoby($database, $idPP, $typRozdeleni, $datumOd = null, $datumDo = null, $vcetnePodrizenych = 0)
    {
        if (!$datumOd) {
            $datumOd = time();
        }
        if (!$datumDo) {
            $datumDo = $datumOd;
        }
        $params = $database ->createParameters();

        $sql =
            'SELECT ' . self::getSelectPart();

        if ($vcetnePodrizenych) {
            $sql .=
                ' FROM RozdeleniOsob, Rozdeleni, PracovniPomer, HierarchieRozdeleni' .
                ' WHERE HierarchieRozdeleni.TypPodrizene = Rozdeleni.KodTypuRozdeleni' .
                ' AND HierarchieRozdeleni.KodPodrizene = Rozdeleni.Kod' .
                ' AND RozdeleniOsob.KodTypuRozdeleni = HierarchieRozdeleni.TypNadrazene' .
                ' AND RozdeleniOsob.Rozdeleni = HierarchieRozdeleni.KodNadrazene';

            if ($vcetnePodrizenych === 2) {
                // jen podrizena
                $sql .=
                    ' AND HierarchieRozdeleni.KodNadrazene <> HierarchieRozdeleni.KodPodrizene';
            }
        } else {
            $sql .=
                ' FROM RozdeleniOsob, Rozdeleni, PracovniPomer' .
                ' WHERE RozdeleniOsob.KodTypuRozdeleni = Rozdeleni.KodTypuRozdeleni' .
                ' AND RozdeleniOsob.Rozdeleni = Rozdeleni.Kod';
        }

        $sql .=
            // typ rozdeleni
            ' AND Rozdeleni.KodTypuRozdeleni = ' . $params ->addParameter((int)$typRozdeleni) .

            // PP
            ' AND RozdeleniOsob.RC = PracovniPomer.RC' .
            ' AND RozdeleniOsob.OsobniCislo = PracovniPomer.OsobniCislo' .
            ' AND RozdeleniOsob.DatumNastupu = PracovniPomer.DatumNastupu';

        if (is_array($idPP)) {
            $sql .= ' AND PracovniPomer.Id IN (';
            $first = true;
            foreach ($idPP as $id) {
                $sql .= ($first ? '' : ', ') . $params ->addParameter((int)$id);
                $first = false;
            }
            $sql .= ')';
        } else {
            $sql .= ' AND PracovniPomer.Id = ' . $params ->addParameter((int)$idPP);
        }

        $sql .=
            // platnost prirazeni a platnost rozdeleni
            ' AND RozdeleniOsob.PlatnostOd <= ' . $params ->addParameter($database ->dateToParam($datumDo)) .
            ' AND (RozdeleniOsob.PlatnostDo >= ' . $params ->addParameter($database ->dateToParam($datumOd)) . ' OR RozdeleniOsob.PlatnostDo IS NULL)' .
            ' AND Rozdeleni.DatumVytvoreni <= ' . $params ->addParameter($database ->dateToParam($datumDo)) .
            ' AND (Rozdeleni.DatumZruseni >= ' . $params ->addParameter($database ->dateToParam($datumOd)) . ' OR Rozdeleni.DatumZruseni IS NULL)';

        $sql .=
            ' ORDER BY RozdeleniOsob.PlatnostOd, Rozdeleni.Kod';

        $cursor = $database ->getSqlResult($sql, $params);

        $result = [];
        while ($row = $cursor ->fetch()) {
            $rozdeleni = new self();
            $rozdeleni ->loadRow($database, $row);
            $result[] = $rozdeleni;
        }

        $cursor ->close();

        return $result;
    }



    /**
     * Nacte rozdeleni zadaneho typu pro zadany pracovni pomer.
     * @param Database $database
     * @param int $idPP
     * @param int $typRozdeleni
     * @param int $datumOd Obdobi, ve kterem ma byt osoba na danem rozdeleni. Pokud neni zadano, pouzije se time()
     * @param int $datumDo ... pokud neni zadano, nastavi se na stejnou hodnotu jako $datumOd
     * @param int $vcetnePodrizenych  0=jen prirazene, 1=vcetne podrizenych, 2=jen podrizene
     * @return string|null
     * @throws DBException
     */
    public static function vratRozdeleniOsoby($database, $idPP, $typRozdeleni, $datumOd = null, $datumDo = null, $vcetnePodrizenych = 0)
    {
        $seznamRozdeleni = self::loadRozdeleniOsoby($database, $idPP, $typRozdeleni, $datumOd, $datumDo, $vcetnePodrizenych);
        if (!count($seznamRozdeleni)) {
            return null;
        }

        return $seznamRozdeleni[0] -> kod;
    }

    /**
     * @param Database $database
     * @param string $tabulka
     * @param int $typRozdeleni
     * @param string $kod
     * @param int $datumOd
     * @param int $datumDo
     * @param int $vcetnePodrizenych   0=jen vybrane, 1=vcetne podrizenych, 2=jen podrizne, 3=jen primo podrizene
     * @param null|Parameters $params
     * @return string
     * @deprecated  Presunuto do RozdeleniUtils, metoda sqlFilterRozdeleniPP
     */
    public static function sqlFilterRozdeleni($database, $tabulka, $typRozdeleni, $kod, $datumOd, $datumDo, $vcetnePodrizenych, $params = null)
    {
        return RozdeleniUtils::sqlFilterRozdeleniPP($database, $tabulka, $typRozdeleni, $kod, $datumOd, $datumDo, $vcetnePodrizenych, $params);
    }


    /**
     * Vraci true, pokud je RC/idPP v danem obdobi v zadanem rozdeleni nebo v nadrazenem/podrizenem rozdeleni (viz priznak $vcetneNadrazenych)
     * @param Database $database
     * @param string|null $RC Pokud je nastaveno, hleda podle RC
     * @param int|null $idPP Pokud je nastaveno, hleda podle pracovniho pomeru
     * @param int $typRozdeleni
     * @param string|string[] $kodRozdeleni Kod rozdeleni nebo seznam kodu (POZOR! seznam nefunguje pro vcetneNadrazenych = 2)
     * @param int $vcetneNadrazenych 0 - pouze vlastni;  1/true - vcetne nadrazenych,  2 - jen nadrazene, 3 - vcetne podrizenych,
     *                  4 - vlastni a primo podrizene, 5 - primo podrizene
     * @param int $obdobiOd
     * @param int $obdobiDo
     * @return bool
     * @throws DBException
     */
    public static function jeVRozdeleni($database, $RC, $idPP, $typRozdeleni, $kodRozdeleni, $vcetneNadrazenych, $obdobiOd, $obdobiDo)
    {
        if (!isset($kodRozdeleni)) {
            return true;
        }

        $typRozdeleni = (int)$typRozdeleni;

        $params = $database ->createParameters();

        if ($vcetneNadrazenych === 4) {
            //vlastni a primo podrizene
            $sql =
                'SELECT DISTINCT R_Ja.RC ' .
                '  FROM RozdeleniOsob R_JA, Rozdeleni R_Nad, Rozdeleni R_Pod ' . ( isset($idPP) ? ', PracovniPomer PP' : '') .
                ' WHERE R_Nad.KodTypuRozdeleni = ' .$params ->addParameter($typRozdeleni) .
                '   AND (' . SQLUtils::getINCondition($database, 'R_Nad.Kod', $kodRozdeleni, $params) .
                '       OR ' . SQLUtils::getINCondition($database, 'R_Pod.Kod', $kodRozdeleni, $params) . ' ) ' .
                '   AND (R_Nad.KodTypuRozdeleni = R_Ja.KodTypuRozdeleni AND R_Nad.Kod = R_Ja.Rozdeleni) ' .
                '   AND (R_Pod.RozdeleniRodice = R_Nad.KodTypuRozdeleni AND R_Pod.Rodic = R_Nad.Kod) ';
        } else if ($vcetneNadrazenych === 5) {
            //primo podrizene
            $sql =
                'SELECT DISTINCT R_Ja.RC ' .
                '  FROM RozdeleniOsob R_JA, Rozdeleni R_Nad, Rozdeleni R_Pod ' . ( isset($idPP) ? ', PracovniPomer PP' : '') .
                ' WHERE R_Nad.KodTypuRozdeleni = ' . $params ->addParameter($typRozdeleni) .
                '   AND ' . SQLUtils::getINCondition($database, 'R_Pod.Kod', $kodRozdeleni, $params) .
                '   AND (R_Nad.KodTypuRozdeleni = R_Ja.KodTypuRozdeleni AND R_Nad.Kod = R_Ja.Rozdeleni) ' .
                '   AND (R_Pod.RozdeleniRodice = R_Nad.KodTypuRozdeleni AND R_Pod.Rodic = R_Nad.Kod) ';
        } else if ($vcetneNadrazenych === 3) {
            // vcetne podrizenych rozdeleni
            $sql =
                'SELECT DISTINCT R_Ja.RC ' .
                '  FROM RozdeleniOsob R_JA, HierarchieRozdeleni H ' . ( isset($idPP) ? ', PracovniPomer PP' : '') .
                ' WHERE H.TypNadrazene = ' . $params ->addParameter($typRozdeleni) .
                '   AND ' . SQLUtils::getINCondition($database, 'H.KodNadrazene', $kodRozdeleni, $params) .
                '   AND H.TypPodrizene = R_Ja.KodTypuRozdeleni ' .
                '   AND H.KodPodrizene = R_Ja.Rozdeleni ';
        } else if ($vcetneNadrazenych) {
            $sql =
                'SELECT DISTINCT R_Ja.RC ' .
                '  FROM RozdeleniOsob R_JA, HierarchieRozdeleni H ' . ( isset($idPP) ? ', PracovniPomer PP' : '') .
                ' WHERE H.TypPodrizene =  ' . $params ->addParameter($typRozdeleni) .
                //"   AND H.KodPodrizene = '" . $database ->addSlashes($kodRozdeleni) . "'" .
                '   AND ' . SQLUtils::getINCondition($database, 'H.KodPodrizene', $kodRozdeleni, $params) .
                '   AND H.TypNadrazene = R_Ja.KodTypuRozdeleni ' .
                '   AND H.KodNadrazene = R_Ja.Rozdeleni ';

            if ($vcetneNadrazenych === 2) { // hledam jen v nadrazenych
                $sql .=
                    ' AND H.KodNadrazene <> ' . $params->addParameter($kodRozdeleni);
            }
        } else {
            $sql =
                'SELECT DISTINCT R_Ja.RC ' .
                ' FROM RozdeleniOsob R_JA ' . ( isset($idPP) ? ', PracovniPomer PP' : '') .
                ' WHERE R_Ja.KodTypuRozdeleni = ' . $params ->addParameter($typRozdeleni) .
                //"   AND R_Ja.Rozdeleni = '" . $database ->addSlashes($kodRozdeleni) . "'";
                '   AND ' . SQLUtils::getINCondition($database, 'R_Ja.Rozdeleni', $kodRozdeleni, $params);
        }

        $sql .=
            '   AND R_Ja.PlatnostOd <= ' . $params ->addParameter($database ->dateToParam($obdobiDo)) .
            '   AND (R_Ja.PlatnostDo >= ' . $params ->addParameter($database ->dateToParam($obdobiOd)) . '  OR  R_Ja.PlatnostDo IS NULL)';

        if (isset($idPP)) {
            $sql .=
                ' AND PP.RC = R_Ja.RC ' .
                ' AND PP.OsobniCislo = R_Ja.OsobniCislo ' .
                ' AND PP.DatumNastupu = R_Ja.DatumNastupu ' .
                ' AND PP.ID = ' . $params ->addParameter((int)$idPP) .
                ' AND PP.DatumNastupu <= ' . $params ->addParameter($database ->dateToParam($obdobiDo)) .
                ' AND (PP.DatumUkonceni >= ' . $params ->addParameter($database ->dateToParam($obdobiOd)) . '  OR  PP.DatumUkonceni IS NULL)';
        } else {
            $sql .=
                ' AND R_Ja.RC = ' . $params ->addParameter($RC);
        }

        $cursor = $database ->getSqlResult($sql, $params);
        $row = $cursor ->fetch();
        $cursor ->close();

        return (bool)$row;
    }


    /**
     * Vraci stejnou hodnotu jako funkce jeVRozdeleni. Rozdil je ve zpusobu zadani rozdeleni
     * @param Database $database
     * @param string $RC
     * @param int $idPP
     * @param string $rozdeleniStr retezec ve formatu "typRozdeleni/kod1;kod2;kod3;..."
     * @param int $vcetneNadrazenych
     * @param int $obdobiOd
     * @param int $obdobiDo
     * @return bool
     * @throws Exception
     */
    public static function jeVRozdeleniStr($database, $RC, $idPP, $rozdeleniStr, $vcetneNadrazenych, $obdobiOd, $obdobiDo)
    {
        $pom = explode('/', $rozdeleniStr, 2);
        $typRozdeleni = (int)$pom[0];

        $kody = explode(';', $pom[1]);
        return self::jeVRozdeleni($database, $RC, $idPP, $typRozdeleni, $kody, $vcetneNadrazenych, $obdobiOd, $obdobiDo);
    }

}
