<?php
/**
 * stanislav.elbl, 21.02.2021
 */

namespace RonPortal;

use Ron\DB\Database;
use Ron\DB\Parameters;
use Ron\DB\SqlStatement;

class RozdeleniUtils
{

    /**
     * Vraci SQL pro odfiltrovani zaznamu pro zadane rozdeleni (pripadne podrizenych a podobne)
     * Zadana tabulka musi mit pole KodTypuRozdeleni, Rozdeleni
     * @param Database $database
     * @param string $tabulka
     * @param int $typRozdeleni
     * @param string $kod
     * @param int $vcetnePodrizenych    0=jen vybrane, 1=vcetne podrizenych, 2=jen podrizene, 3=jen primo podrizene
     * @param null|Parameters $params
     * @param string[] $fieldNames      Nepovinne pole s jmeny poli pro typ rozdeleni a kod rozdeleni
     * @return string
     */
    public static function sqlFilterRozdeleni($database, $tabulka, $typRozdeleni, $kod, $vcetnePodrizenych, $params = null, $fieldNames = null)
    {
        $fieldNameTyp = $fieldNames ? $fieldNames[0] : 'KodTypuRozdeleni';
        $fieldNameKod = $fieldNames ? $fieldNames[1] : 'Rozdeleni';

        if ($vcetnePodrizenych === true) {
            $vcetnePodrizenych = 1;
        } else if ($vcetnePodrizenych === false) {
            $vcetnePodrizenych = 0;
        }

        switch ($vcetnePodrizenych) {
            case 1 : // vcetne podrizenych
            case 2 : // jen podrizene
                return
                    'EXISTS (' .
                    '  SELECT * FROM HierarchieRozdeleni ' .
                    '  WHERE HierarchieRozdeleni.TypNadrazene = ' . (is_null($params) ? (int)$typRozdeleni : $params->addParameter($typRozdeleni)) .
                    '    AND HierarchieRozdeleni.KodNadrazene = ' . (is_null($params) ? "'" . $database ->addSlashes($kod) . "'" : $params->addParameter($kod)) .

                    (($vcetnePodrizenych == 2) ? ' AND HierarchieRozdeleni.KodNadrazene <> HierarchieRozdeleni.KodPodrizene' : '') . // filtr pro jen podrizene

                    "    AND $tabulka.$fieldNameTyp = HierarchieRozdeleni.typPodrizene" .
                    "    AND $tabulka.$fieldNameKod = HierarchieRozdeleni.kodPodrizene" .
                    ')';

            case 3 :
                // jen primo podrizene
                return
                    'EXISTS (' .
                    '  SELECT * FROM Rozdeleni ' .
                    '  WHERE Rozdeleni.RozdeleniRodice = ' . (is_null($params) ? (int)$typRozdeleni : $params->addParameter($typRozdeleni)) .
                    '    AND Rozdeleni.Rodic = ' . (is_null($params) ? "'" . $database ->addSlashes($kod) . "'" : $params->addParameter($kod)) .
                    "    AND $tabulka.$fieldNameTyp = Rozdeleni.KodTypuRozdeleni" .
                    "    AND $tabulka.$fieldNameKod = Rozdeleni.Kod" .
                    ')';
            default :
                // jen vybrane
                return
                    "$tabulka.$fieldNameTyp = " . (is_null($params) ? (int)$typRozdeleni : $params->addParameter($typRozdeleni)) .
                    " AND $tabulka.$fieldNameKod = " . (is_null($params) ? "'" . $database ->addSlashes($kod) . "'" : $params->addParameter($kod));
        }
    }




    /**
     * Vraci SQL (EXISTS) pro odfiltrovani zaznamu pro pracovni pomery ze zadaneho rozdeleni (pripadne podrizenych a podobne)
     * Zadana tabulka musi mit pole RC, OsobniCislo, DatumNastupu
     * @param Database $database
     * @param string $tabulka
     * @param int $typRozdeleni
     * @param string $kod
     * @param int|string $datumOd       Pokud je string, vlozi se do SQL dotazu primo zadany text, jinak se prevadi timestamp na SQL konstantu
     * @param int|string $datumDo       -//-
     * @param int $vcetnePodrizenych    0=jen vybrane, 1=vcetne podrizenych, 2=jen podrizne, 3=jen primo podrizene
     * @param null|Parameters $params
     * @return string
     */
    public static function sqlFilterRozdeleniPP($database, $tabulka, $typRozdeleni, $kod, $datumOd, $datumDo, $vcetnePodrizenych, $params = null, $jenRC = false)
    {
        if ($vcetnePodrizenych === true) {
            $vcetnePodrizenych = 1;
        } else if ($vcetnePodrizenych === false) {
            $vcetnePodrizenych = 0;
        }

        switch ($vcetnePodrizenych) {
            case 1 : // vcetne podrizenych
            case 2 : // jen podrizene
                return
                    'EXISTS (' .
                    '  SELECT * FROM RozdeleniOsob, HierarchieRozdeleni ' .
                    '  WHERE HierarchieRozdeleni.TypNadrazene = ' . (is_null($params) ? (int)$typRozdeleni : $params->addParameter($typRozdeleni)) .
                    "    AND HierarchieRozdeleni.KodNadrazene = " . (is_null($params) ? "'" . $database ->addSlashes($kod) . "'" : $params->addParameter($kod)) .

                    (($vcetnePodrizenych == 2)
                        ? ' AND HierarchieRozdeleni.KodNadrazene <> HierarchieRozdeleni.KodPodrizene'
                        : ''
                    ) . // filtr pro jen podrizene

                    '    AND RozdeleniOsob.KodTypuRozdeleni = HierarchieRozdeleni.typPodrizene' .
                    '    AND RozdeleniOsob.Rozdeleni = HierarchieRozdeleni.kodPodrizene' .

                    '    AND RozdeleniOsob.RC = ' . $tabulka . '.RC ' .
                    ($jenRC
                        ? ''
                        : '    AND RozdeleniOsob.OsobniCislo = ' . $tabulka . '.OsobniCislo ' .
                        '    AND RozdeleniOsob.DatumNastupu = ' . $tabulka . '.DatumNastupu '
                    ) .
                    '    AND RozdeleniOsob.PlatnostOd <= ' . (is_string($datumDo)
                            ? $datumDo
                            : (is_null($params) ? $database ->timestampToString($datumDo) : $params->addParameter($database->timestampToParam($datumDo)))) .
                    '    AND (RozdeleniOsob.PlatnostDo >= ' . (is_string($datumOd)
                            ? $datumOd
                            : (is_null($params) ? $database ->timestampToString($datumOd) : $params->addParameter($database->timestampToParam($datumOd)))) .
                            '  OR  RozdeleniOsob.PlatnostDo IS NULL)' .
                    ')';

            case 3 :
                // jen primo podrizene
                return
                    'EXISTS (' .
                    '  SELECT * FROM RozdeleniOsob, Rozdeleni ' .
                    '  WHERE Rozdeleni.RozdeleniRodice = ' . (is_null($params) ? (int)$typRozdeleni : $params->addParameter($typRozdeleni)) .
                    "    AND Rozdeleni.Rodic = " . (is_null($params) ? "'" . $database ->addSlashes($kod) . "'" : $params->addParameter($kod)) .
                    '    AND RozdeleniOsob.KodTypuRozdeleni = Rozdeleni.KodTypuRozdeleni' .
                    '    AND RozdeleniOsob.Rozdeleni = Rozdeleni.Kod' .
                    '    AND RozdeleniOsob.RC = ' . $tabulka . '.RC ' .
                    ($jenRC
                        ? ''
                        : '    AND RozdeleniOsob.OsobniCislo = ' . $tabulka . '.OsobniCislo ' .
                        '    AND RozdeleniOsob.DatumNastupu = ' . $tabulka . '.DatumNastupu '
                    ).
                    '    AND RozdeleniOsob.PlatnostOd <= ' . (is_string($datumDo)
                            ? $datumDo
                            : (is_null($params) ? $database ->timestampToString($datumDo) : $params->addParameter($database->timestampToParam($datumDo)))) .
                    '    AND (RozdeleniOsob.PlatnostDo >= ' . (is_string($datumOd)
                            ? $datumOd
                            : (is_null($params) ? $database ->timestampToString($datumOd) : $params->addParameter($database->timestampToParam($datumOd))))
                            . '  OR  RozdeleniOsob.PlatnostDo IS NULL)' .
                    ')';

            default :
                // jen vybrane
                return
                    'EXISTS (' .
                    '  SELECT * FROM RozdeleniOsob ' .
                    '  WHERE RozdeleniOsob.KodTypuRozdeleni = ' . (is_null($params) ? (int)$typRozdeleni : $params->addParameter($typRozdeleni)) .
                    "    AND RozdeleniOsob.Rozdeleni = " . (is_null($params) ? "'" . $database ->addSlashes($kod) . "'" : $params->addParameter($kod)) .
                    '    AND RozdeleniOsob.RC = ' . $tabulka . '.RC ' .
                    ($jenRC
                        ? ''
                        : '    AND RozdeleniOsob.OsobniCislo = ' . $tabulka . '.OsobniCislo ' .
                        '    AND RozdeleniOsob.DatumNastupu = ' . $tabulka . '.DatumNastupu '
                    ).
                    '    AND RozdeleniOsob.PlatnostOd <= ' . (is_string($datumDo)
                            ? $datumDo
                            : (is_null($params) ? $database ->timestampToString($datumDo) : $params->addParameter($database->timestampToParam($datumDo)))) .
                    '    AND (RozdeleniOsob.PlatnostDo >= ' . (is_string($datumOd)
                            ? $datumOd
                            : (is_null($params) ? $database ->timestampToString($datumOd) : $params->addParameter($database->timestampToParam($datumOd))))
                            . '  OR  RozdeleniOsob.PlatnostDo IS NULL)' .
                    ')';
        }
    }


    /**
     * Vraci true, pokud je RCpodrizeny v danem obdobi v nejakem podrizenem rozdeleni daneho typu
     * Pokud je vcetneVlastnich true, tak muze byt i ve stejnem rozdeleni
     * Pokud neni nastaveno $OCpodrizeny, $nastupPodrizeny, kontroluje se jen RC
     *
     * @param Database $database
     * @param string $RCNadrizeny
     * @param string $RCPodrizeny
     * @param string $OCPodrizeny
     * @param int $nastupPodrizeny
     * @param int $typRozdeleni
     * @param int $vcetneVlastnich
     * @param int $obdobiOd
     * @param int $obdobiDo
     * @return bool
     */
    public static function jePodrizeny($database, $RCNadrizeny, $RCPodrizeny, $OCPodrizeny, $nastupPodrizeny, $typRozdeleni, $vcetneVlastnich, $obdobiOd, $obdobiDo)
    {
        $params = $database ->createParameters();

        $sql =
            'SELECT R_Ja.RC ' .
            '  FROM RozdeleniOsob R_JA, HierarchieRozdeleni H, RozdeleniOsob R_Hodnoceny ' .
            '  WHERE R_Ja.RC = ' . $params->addParameter($RCNadrizeny) .
            '    AND R_JA.KodTypuRozdeleni = ' . $params ->addParameter((int)$typRozdeleni) .
            '    AND R_Ja.PlatnostOd <= ' . $params ->addParameter($database ->dateToParam($obdobiDo)) .
            '    AND (R_Ja.PlatnostDo >= ' . $params ->addParameter($database ->dateToParam($obdobiOd)) . '  OR  R_Ja.PlatnostDo IS NULL)' .
            '    AND R_Hodnoceny.KodTypuRozdeleni = R_JA.KodTypuRozdeleni ' .
            '    AND H.TypNadrazene = R_Ja.KodTypuRozdeleni ' .
            '    AND H.KodNadrazene = R_Ja.Rozdeleni ' .
            '    AND H.TypPodrizene = R_Hodnoceny.KodTypuRozdeleni ' .
            '    AND H.KodPodrizene = R_Hodnoceny.Rozdeleni ' .
            '    AND R_Hodnoceny.PlatnostOd <= ' . $params ->addParameter($database ->dateToParam($obdobiDo)) .
            '    AND (R_Hodnoceny.PlatnostDo >= ' . $params ->addParameter($database ->dateToParam($obdobiOd)) . '  OR  R_Hodnoceny.PlatnostDo IS NULL)' .
            (!$vcetneVlastnich ? ' AND H.KodNadrazene <> H.KodPodrizene ' : '') .
            '    AND R_Hodnoceny.RC = ' . $params->addParameter($RCPodrizeny);

        if ($OCPodrizeny && $nastupPodrizeny) {
            // kontroluje se i osobni cislo a datum nastupu
            $sql .=
                '  AND R_Hodnoceny.OsobniCislo = ' . $params ->addParameter($OCPodrizeny) .
                '  AND R_Hodnoceny.DatumNastupu = ' . $params ->addParameter($database ->dateToParam($nastupPodrizeny));
        }

        $cursor = $database ->getSqlResult($sql, $params);
        $row = $cursor ->fetch();
        $cursor ->close();

        return (bool)$row;
    }


    /**
     * @param Database $database
     * @param PracovniPomer|string $osoba Pokud je string, hleda se jen podle RC, jinak podle celeho pracovniho pomeru
     * @param int $kodTypuRozdeleni
     * @param int $obdobiOd
     * @param int $obdobiDo
     * @param int $pouzeVlastni 0=vse, 1=pouze vlastni, 2=vlastni a podrizene, 3=jen podrizene, 4=jen primo podrizene
     * @param string $selectPart
     * @return SqlStatement
     */
    public static function sqlSeznamRozdeleniOsoby($database, $osoba, $kodTypuRozdeleni, $obdobiOd = null, $obdobiDo = null, $pouzeVlastni = 0, $selectPart = 'Rozdeleni.Kod, Rozdeleni.Popis')
    {
        // SE - kod typu rozdeleni je Adhoc, aby mohl pouzit ruzne exekucni plany pro ruzne typy rozdeleni (protoze struktura zaznamu se muze lisit)
        $params = $database ->createParameters();

        if (!isset($obdobiOd)) {
            $obdobiOd = time();
        }

        if (!isset($obdobiDo)) {
            $obdobiDo = $obdobiOd;
        }

        if (is_string($osoba)) {
            $rc = $osoba;
            $osobniCislo = null;
            $datumNastupu = null;
        } else {
            $rc = $osoba ->rc;
            $osobniCislo = $osoba ->osobniCislo;
            $datumNastupu = $osoba ->datumNastupu;
        }

        if ($pouzeVlastni === 1) {
            // pouze vlastni
            $sql =
                'SELECT DISTINCT ' . $selectPart .
                ' FROM Rozdeleni, RozdeleniOsob ' .
                ' WHERE Rozdeleni.KodTypuRozdeleni = ' . (int)$kodTypuRozdeleni .
                ' AND RozdeleniOsob.KodTypuRozdeleni = Rozdeleni.KodTypuRozdeleni ' .
                ' AND RozdeleniOsob.Rozdeleni = Rozdeleni.Kod ' .
                ' AND RozdeleniOsob.RC = ' . $params ->addParameter($rc) .
                (isset($osobniCislo) ? ' AND RozdeleniOsob.OsobniCislo = ' . $params ->addParameter($osobniCislo) : '') .
                (isset($datumNastupu) ? ' AND RozdeleniOsob.DatumNastupu = ' . $params ->addDateParameter($datumNastupu) : '') .
                ' AND RozdeleniOsob.PlatnostOd <= ' . $params ->addDateParameter($obdobiDo) .
                ' AND (RozdeleniOsob.PlatnostDo >= ' . $params ->addDateParameter($obdobiOd) . '  OR  RozdeleniOsob.PlatnostDo IS NULL)';
        } else if ($pouzeVlastni === 2) {
            // vlastni a podrizene
            $sql =
                'SELECT DISTINCT ' . $selectPart .
                ' FROM Rozdeleni, RozdeleniOsob R_JA, HierarchieRozdeleni H ' .
                ' WHERE Rozdeleni.KodTypuRozdeleni = ' . (int)$kodTypuRozdeleni .
                ' AND R_Ja.RC = ' . $params ->addParameter($rc) .
                (isset($osobniCislo) ? ' AND R_Ja.OsobniCislo = ' . $params ->addParameter($osobniCislo) : '') .
                (isset($datumNastupu) ? ' AND R_Ja.DatumNastupu = ' . $params ->addDateParameter($datumNastupu) : '') .
                ' AND H.TypNadrazene = R_Ja.KodTypuRozdeleni ' .
                ' AND H.KodNadrazene = R_Ja.Rozdeleni ' .
                ' AND H.TypPodrizene = Rozdeleni.KodTypuRozdeleni ' .
                ' AND H.KodPodrizene = Rozdeleni.Kod ' .
                ' AND R_Ja.KodTypuRozdeleni = Rozdeleni.KodTypuRozdeleni' .
                ' AND R_Ja.PlatnostOd <= ' . $params ->addDateParameter($obdobiDo) .
                ' AND (R_Ja.PlatnostDo >= ' . $params ->addDateParameter($obdobiOd) . '  OR  R_Ja.PlatnostDo IS NULL)';
        } else if ($pouzeVlastni === 3) {
            // podrizene
            $sql =
                'SELECT DISTINCT ' . $selectPart .
                ' FROM Rozdeleni, RozdeleniOsob R_JA, HierarchieRozdeleni H ' .
                ' WHERE Rozdeleni.KodTypuRozdeleni = ' . (int)$kodTypuRozdeleni .
                ' AND R_Ja.RC = ' . $params ->addParameter($rc) .
                (isset($osobniCislo) ? ' AND R_Ja.OsobniCislo = ' . $params ->addParameter($osobniCislo) : '') .
                (isset($datumNastupu) ? ' AND R_Ja.DatumNastupu = ' . $params ->addDateParameter($datumNastupu) : '') .
                ' AND H.TypNadrazene = R_Ja.KodTypuRozdeleni ' .
                ' AND H.KodNadrazene = R_Ja.Rozdeleni ' .
                ' AND H.TypPodrizene = Rozdeleni.KodTypuRozdeleni ' .
                ' AND H.KodPodrizene = Rozdeleni.Kod ' .
                ' AND R_Ja.KodTypuRozdeleni = Rozdeleni.KodTypuRozdeleni' .
                ' AND R_Ja.PlatnostOd <= ' . $params ->addDateParameter($obdobiDo) .
                ' AND (R_Ja.PlatnostDo >= ' . $params ->addDateParameter($obdobiOd) . '  OR  R_Ja.PlatnostDo IS NULL)' .
                ' AND H.KodNadrazene <> H.KodPodrizene';
        } else if ($pouzeVlastni === 4) {
            // primo podrizene
            $sql =
                'SELECT DISTINCT ' . $selectPart .
                ' FROM Rozdeleni, RozdeleniOsob R_JA' .
                ' WHERE Rozdeleni.KodTypuRozdeleni = ' . (int)$kodTypuRozdeleni .
                ' AND R_JA.KodTypuRozdeleni = Rozdeleni.KodTypuRozdeleni ' .
                ' AND R_Ja.RC = ' . $params ->addParameter($rc) .
                (isset($osobniCislo) ? ' AND R_Ja.OsobniCislo = ' . $params ->addParameter($osobniCislo) : '') .
                (isset($datumNastupu) ? ' AND R_Ja.DatumNastupu = ' . $params ->addDateParameter($datumNastupu) : '') .
                ' AND Rozdeleni.RozdeleniRodice = R_Ja.KodTypuRozdeleni ' .
                ' AND Rozdeleni.Rodic = R_Ja.Rozdeleni ' .
                ' AND R_Ja.PlatnostOd <= ' . $params ->addDateParameter($obdobiDo) .
                ' AND (R_Ja.PlatnostDo >= ' . $params ->addDateParameter($obdobiOd) . '  OR  R_Ja.PlatnostDo IS NULL)';
        } else {
            // vse
            $sql =
                'SELECT DISTINCT ' . $selectPart .
                ' FROM Rozdeleni ' .
                ' WHERE Rozdeleni.KodTypuRozdeleni = ' . (int)$kodTypuRozdeleni;
        }

        if ($obdobiOd) {
            $sql .=
                ' AND (Rozdeleni.DatumZruseni >= ' . $params ->addDateParameter($obdobiOd) .
                ' OR Rozdeleni.DatumZruseni IS NULL) ';
        }

        if ($obdobiDo) {
            $sql .=
                ' AND Rozdeleni.DatumVytvoreni <= ' . $params ->addDateParameter($obdobiDo);
        }

        return new SqlStatement($sql, $params);
    }


//    /**
//     * @param Database $database
//     * @param PravaNaOsoby $typPrava Typ prava, podle ktereho se seznam rozdeleni odfiltruje
//     * @param int $kodTypuRozdeleni
//     * @param int $obdobiOd
//     * @param int $obdobiDo
//     * @param int $pouzeVlastni 0=vse, 1=pouze vlastni, 2=vlastni a podrizene, 3=jen podrizene, 4=jen primo podrizene
//     * @param string $selectPart
//     * @return SqlStatement
//     */
//    public static function sqlSeznamRozdeleniUzivatele($database, $typPrava, $kodTypuRozdeleni, $obdobiOd = null, $obdobiDo = null, $selectPart = 'Rozdeleni.Kod, Rozdeleni.Popis')
//    {
//        $pouzeVlastni = 0;
//
//        if ($typPrava) {
//            switch ($typPrava ->prava) {
//                case 1 :  // pouze vlastni udaje
//                case 10 : // vlastni rozdeleni
//                case 11 : // -//- krome sebe
//                    $pouzeVlastni = 1;
//                    break;
//
//                case 12 : // vlastni a podrizena
//                case 13 : // -//- krome sebe
//                    $pouzeVlastni = 2; // vlastni a podrizene
//                    break;
//
//                case 14 : // podrizena
//                case 15 : // podrizena a sebe
//                case 22 : // vlastni a primo podrizena ! ! ! POZOR - zobrazuje i neprimo podrizena !
//                case 23 : // -//- krome sebe  ! ! ! POZOR - zobrazuje i neprimo podrizena !
//                case 25 : // primo podrizena a sebe ! ! ! POZOR - zobrazuje i neprimo podrizena !
//                    $pouzeVlastni = 3; // podrizene
//                    break;
//
//                case 24 : // primo podrizena
//                case 26 : // primo podrizena krome sebe
//                    $pouzeVlastni = 4; // primo podrizene
//                    break;
//
//                case 100 :
//                    return
//                        'SELECT DISTINCT ' . $selectPart .
//                        ' FROM Rozdeleni ' .
//                        ' WHERE 0 = 1';
//
//                //case 2 : // vybrane osoby a skupiny
//                //case 3 : // vse
//                //case 4 : // vybrane krome sebe
//            }
//        }
//
//        return self::sqlSeznamRozdeleniOsoby($database, User::getRC(), $kodTypuRozdeleni, $obdobiOd, $obdobiDo, $pouzeVlastni, $selectPart);
//    }

}
