<?php


namespace RonPortal\Sklad;

use Exception;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\Http\App;
use Ron\MessageManager;
use Ron\StringUtils;
use RonPortal\Dokumenty\DokumentOutputObject;
use RonPortal\Dokumenty\DokumentUploadForm;
use RonPortal\Dokumenty\DokumentyBrowser;
use RonPortal\Dokumenty\DokumentyUtils;
use RonPortal\NeniPravoException;
use RonPortal\RonPortalDBRecord;
use function Ron\ronText;


class NovyDokumentVydaneObjednavkyForm extends DokumentUploadForm
{
    var $cisloObjednavky = null;

    /**
     * @param Database $database
     * @param int $cisloObjednavky
     */
    function __construct($database, $cisloObjednavky)
    {
        parent::__construct($database, '?menuid=objednavky&action=dokumentupload&cisloobjednavky=' . $cisloObjednavky);

        $this ->database = $database;
        $this ->cisloObjednavky = $cisloObjednavky;
//        $this->kategorieDokumentu = 203; // TODO> dokument vydane objednavky ?

    }

}



class DokumentyVydaneObjednavkyBrowser extends DokumentyBrowser
{
    public $cisloObjednavky;

    function __construct($database, $cisloObjednavky) {
        parent::__construct($database, "?menuid=objednavky&action=download&cisloobjednavky=$cisloObjednavky", 1, "?menuid=objednavky&action=zrusitdokument&cisloobjednavky=$cisloObjednavky");

        $this ->cisloObjednavky = $cisloObjednavky;

        $this->sqlFromPart = 'Dokument JOIN TypDokumentu ON (TypDokumentu.Kod = Dokument.Typ) JOIN DokumentObjednavkyVydane ON (Dokument.Id = DokumentObjednavkyVydane.Id)';
        $this->sqlWherePart =
            " DokumentObjednavkyVydane.cisloVydaneObjednavky = '" . $this->database->addSlashes($cisloObjednavky) . "'";

        $this->showMessages = false;
        $this->title = null;
        $this->tableId = "DokumentyVydaneObjednavky";
    }


    /**
     * @param Database $database
     * @param int $cisloObjednavky
     * @param bool $lzeEditovat
     * @throws Exception
     */
    static function zobrazDokumenty($database, $cisloObjednavky, $lzeEditovat)
    {
        echo '<div class="row" id="DokumentyObjednavkyView">';
        echo    "<div class='col-12'>";
        echo    "<A id='dokumenty'></A>\n";

        // Seznam dokumentu
        $b = new DokumentyVydaneObjednavkyBrowser($database, $cisloObjednavky);
        $b ->load();
        $b ->render();

        echo    "</div>";
        echo "</div>";

        // Pokud lze editovat, zobrazi odkaz "vlozit novy"
        if ($lzeEditovat) {
            echo "<div class='row'><div class='col-12'><a id='vlozit_dokument_vydane_objednavky' href='#' data-cisloobjednavky='$cisloObjednavky'>" . ronText('Vlozit_novy_dokument') . "</a></div></div>\n";

            $form = new NovyDokumentVydaneObjednavkyForm($database, $cisloObjednavky);
            $form ->render();
        }
    }

    /**
     * @param Database $database
     * @param int $cisloObjednavky
     * @param int $idDokumentu
     * @throws DBException
     */
    static function zrusDokumentVydaneObjednavky($database, $cisloObjednavky, $idDokumentu)
    {
        // smaze vazbu na dokument
        $rec = new RonPortalDBRecord($database, 'DokumentObjednavkyVydane');
        $rec ->addField('Id', 'int');
        $rec ->addField('cisloVydaneObjednavky', '');

        $params = $database ->createParameters();
        $rec ->deleteRecord(
            'Id = ' . $params ->addParameter((int)$idDokumentu) .
            ' AND cisloVydaneObjednavky = ' . $params ->addParameter($cisloObjednavky),
            $params);

        // zjisti jestli je pripojen jeste k dalsi objednavce (pomoci funkce "pripojit starsi dokument")
        $params = $database ->createParameters();
        $cursor = $database ->getSqlResult('SELECT COUNT(*) FROM DokumentObjednavkyVydane WHERE Id = ' . $params ->addParameter((int)$idDokumentu), $params);
        $row = $cursor->fetch();
        $cursor ->close();
        if (!$row[0]) {
            // dokument neni pripojen k jine sluzebni ceste => smaze i dokument
            try {
                DokumentyUtils::smazDokument($database, (int)$idDokumentu, StringUtils::fromUTF8('Vydaná objednávka'));
            } catch (Exception $e) {
                // chybu ignoruje - zobrazi ji jen v debug rezimu - dokument totiz muze byt pripojen jeste v jine agende
                if (App::getDebugMode()) {
                    MessageManager::addErrorMessage($e);
                }
            }
        }
    }


    /**
     * @param Database $database
     * @param int $cisloObjednavky
     * @param int $idDokumentu
     * @return DokumentOutputObject
     * @throws Exception
     */
    static function downloadDokument($database, $cisloObjednavky, $idDokumentu)
    {
        $params = $database->createParameters();
        $sql =
            'SELECT ObjednavkaVydana.vystavil, ObjednavkaVydana.datumVystaveni ' .
            '  FROM Dokument JOIN DokumentObjednavkyVydane ON (Dokument.Id = DokumentObjednavkyVydane.Id) JOIN ObjednavkaVydana ON (ObjednavkaVydana.cisloVydaneObjednavky = DokumentObjednavkyVydane.cisloVydaneObjednavky)' .
            ' WHERE ObjednavkaVydana.cisloVydaneObjednavky = ' . $params->addParameter($cisloObjednavky) .
            '   AND DokumentObjednavkyVydane.Id = ' . $params->addParameter($idDokumentu);

        if (!$cursor = $database ->select($sql, $params)) {
            throw new DBException($database, $sql);
        }

        if (!$row = $cursor ->fetch()) {
            throw new Exception('Pozadovany dokument vydane objednavky nebyl nalezen!');
        }

        if (!VydanaObjednavka::jePravoNaObjednavku($database, $row[0], $database ->stringToTimestamp($row[1]))) {
            throw new NeniPravoException();
        }

        return new DokumentOutputObject($database, $idDokumentu);
    }
}
