<?php

namespace RonPortal\Sklad;

use Exception;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\StringUtils;

class SkladUtils
{
    /**
     * @param Database $database
     * @param int $datum
     * @return int|null
     * @throws DBException
     * @throws Exception
     */
    public static function getObdobiProDatum($database, $datum)
    {
        $params = $database->createParameters();

        $sql =
            "SELECT PocatekObdobi FROM Obdobi WHERE Aplikace = " . $params->addParameter('FAOS') .
            " AND PocatekObdobi <= " . $params->addParameter($database->dateToParam($datum)) . " AND KonecObdobi >= " . $params->addParameter($database->dateToParam($datum));

        $cursor = $database->getSqlResult($sql, $params);

        $pocetZaznamu = 0;
        $result = 0;

        while ($row = $cursor->fetch()) {
            $result = $database->stringToTimestamp($row[0]);
            $pocetZaznamu++;
        }

        $cursor->close();

        if ($pocetZaznamu > 1)
            throw new Exception(StringUtils::fromUTF8('CHYBA: Pro zadané datum bylo nalezeno více období!'));

        if ($result == 0)
            throw new Exception(StringUtils::fromUTF8('CHYBA: Nenalezeno období pro zadané datum! Zkontrolujte vytvořená období v aplikaci.'));

        return $result;
    }

    /**
     * @param Database $database
     * @param string $typRady
     * @param string $stredisko
     * @param int $obdobi
     * @return mixed
     * @throws DBException
     * @throws Exception
     */
    public static function getNastaveniCiselneRady($database, $typRady, $stredisko, $obdobi)
    {
        $params = $database->createParameters();

        $sql =
            'SELECT CiselnaRada.Maska, CiselnaRada.CisloMin, CiselnaRada.CisloMax ' .
            '  FROM CiselnaRada, CisRadyStrediskaProObdobi ' .
            ' WHERE (CisRadyStrediskaProObdobi.KodStrediska = ' . $params->addParameter($stredisko) . ')' .
            '   AND (CisRadyStrediskaProObdobi.Aplikace = ' . $params->addParameter('FAOS') . ')' .
            '   AND (CisRadyStrediskaProObdobi.PocatekObdobi = ' . $params->addParameter($database->dateToParam($obdobi)) . ')' .
            '   AND CiselnaRada.Kod = CisRadyStrediskaProObdobi.' . $typRady .
            ' ORDER BY CiselnaRada.Kod';

        $cursor = $database->getSqlResult($sql, $params);

        $row = $cursor->fetch();

        $cursor->close();

        if ($row) {
            $nastaveniRady['maska'] = $row[0];
            $nastaveniRady['min'] = $row[1];
            $nastaveniRady['max'] = $row[2];
            return $nastaveniRady;
        }

        throw new Exception(StringUtils::fromUTF8('CHYBA: Nenalezena číselná řada pro středisko s kódem: ' . StringUtils::htmlTextConvert($stredisko)));
    }
}
