<?php

namespace RonPortal\Sklad;

use Exception;
use Ron\DateUtils;
use Ron\DB\DBException;
use Ron\DB\DBRecordField;
use Ron\EMail;
use Ron\MailManager;
use Ron\MessageManager;
use RonPortal\Dokumenty\DokumentyUtils;
use RonPortal\Log;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\PravaNaOsoby;
use RonPortal\System\Report;
use RonPortal\System\ReportOutputObject;
use RonPortal\RonPortalDBRecord;
use RonPortal\RonPortalSettings;
use RonPortal\RonPortalUtils;
use RonPortal\RonPortalXMLEvents;
use RonPortal\User;
use Ron\StringUtils;
use function Ron\ronText;


class VydanaObjednavkaRecord extends RonPortalDBRecord
{
    public $cisloObjednavky;

    /** @var DBRecordField */
    public $RC_Schvalovatele1;
    /** @var DBRecordField */
    public $schvaleni1_jmeno;
    /** @var DBRecordField */
    public $schvaleni1_DT;
    /** @var DBRecordField */
    public $schvaleni1_stav;
    /** @var DBRecordField */
    public $RC_Schvalovatele2;
    /** @var DBRecordField */
    public $schvaleni2_jmeno;
    /** @var DBRecordField */
    public $schvaleni2_DT;
    /** @var DBRecordField */
    public $schvaleni2_stav;
    /** @var DBRecordField */
    public $RC_Schvalovatele3;
    /** @var DBRecordField */
    public $schvaleni3_jmeno;
    /** @var DBRecordField */
    public $schvaleni3_DT;
    /** @var DBRecordField */
    public $schvaleni3_stav;

    public $historieField;

    public function __construct($database, $cisloObjednavky = null)
    {
        parent::__construct($database, 'ObjednavkaVydana');
        $this->logDescription = StringUtils::fromUTF8('Vydaná objednávka');

        if (!is_null($cisloObjednavky)) {
            $this->cisloObjednavky = $cisloObjednavky;

            $params = $database->createParameters();
            $where = 'cisloVydaneObjednavky = ' . $params->addParameter($cisloObjednavky);
            $this->setWhere($where, $params);
        }

        /* 8.4.2021 RON-PaRu after post se vola az v instanci, protoze vkladani probiha v transakci (neni mozne cist zapsany udaj) */
        $this->setXmlEvents(RonPortalXMLEvents::VYD_OBJEDNAVKA_BEFORE_POST, null);

        $this->addField('cisloVydaneObjednavky', 'string', true, 20);
        $this->addField('stredisko', 'string', true, 30);
        $this->addField('datumVystaveni', 'timestamp', true);
        $this->addField('terminDodani', 'timestamp', true);
        $this->addField('Popis', 'string', true, 80);
        $this->addField('dodavatel', 'string', true, 10);
        $this->addField('vystavil', 'string', true, 15);
        $this->addField('odeslano', 'int', true);
        $this->addField('vyrizeno', 'int', true);
        $this->addField('datumCasOdeslani', 'timestamp', true)->updateLog = true;

        $this->addField('cenaCelkem', 'float', true);
        $this->addField('predpokladanaCena', 'float', true);
        $this->addField('mena', 'string', true, 10);
        $this->addField('poznamka', 'memo', true);

        $this->addField('Uzivatelska1', 'string', true, 100);
        $this->addField('Uzivatelska2', 'string', true, 100);
        $this->addField('Uzivatelska3', 'string', true, 100);
        $this->addField('Uzivatelska4', 'string', true, 100);
        $this->addField('Uzivatelska5', 'string', true, 100);

        $this->addField('schvaleno', 'int', true);
        $this->addField('odeslatMailem', 'int', true);
        $this->addField('vyzadovatSchvaleni', 'int', true);

        $this->addField('KodTypuRozdeleni', 'int', true);
        $this->addField('Rozdeleni', 'string', true, 30);
        $this->addField('KodTypuRozdeleni2', 'int', true);
        $this->addField('Rozdeleni2', 'string', true, 30);
        $this->addField('KodTypuRozdeleni3', 'int', true);
        $this->addField('Rozdeleni3', 'string', true, 30);
        $this->addField('KodTypuRozdeleni4', 'int', true);
        $this->addField('Rozdeleni4', 'string', true, 30);
        $this->addField('KodTypuRozdeleni5', 'int', true);
        $this->addField('Rozdeleni5', 'string', true, 30);

        $this->addField('sablona', 'int', true);

        $this->addField('DM_Castka', 'float', true, 0, 0);
        $this->addField('DM_KodMeny', 'string', true, 10);

        $this->RC_Schvalovatele1 = $this->addField('RC_Schvalovatele1', 'string', true, 15);
        $this->schvaleni1_jmeno = $this->addField('schvaleni1_jmeno', 'string', true, 30);
        $this->schvaleni1_DT = $this->addField('schvaleni1_DT', 'timestamp', true);
        $this->schvaleni1_stav = $this->addField('schvaleni1_stav', 'int', true, 0, 0);
        $this->RC_Schvalovatele2 = $this->addField('RC_Schvalovatele2', 'string', true, 15);
        $this->schvaleni2_jmeno = $this->addField('schvaleni2_jmeno', 'string', true, 30);
        $this->schvaleni2_DT = $this->addField('schvaleni2_DT', 'timestamp', true);
        $this->schvaleni2_stav = $this->addField('schvaleni2_stav', 'int', true, 0, 0);
        $this->RC_Schvalovatele3 = $this->addField('RC_Schvalovatele3', 'string', true, 15);
        $this->schvaleni3_jmeno = $this->addField('schvaleni3_jmeno', 'string', true, 30);
        $this->schvaleni3_DT = $this->addField('schvaleni3_DT', 'timestamp', true);
        $this->schvaleni3_stav = $this->addField('schvaleni3_stav', 'int', true, 0, 0);

        $this->historieField = $this->addField('HistorieSchvalovani', 'memo', true);
    }

    /**
     * @param PravaNaOsoby $prava
     * @param bool $vkladani
     * @param int $uroven
     * @return bool
     * @throws DBException
     * @throws Exception
     */
    public function jePravo($prava, $vkladani = false, $uroven = null)
    {
        $this->loadRecord();

        Prava::nastavObdobiProPrava($this->findFieldByName('datumVystaveni')->value);

        if (!is_null($uroven)) {
            if ($uroven == 3) {
                //pro stav schvaleni nebo zamitnuto nelze znovu zapsat
                if (($this->schvaleni3_DT->value) && (($this->schvaleni3_stav->value->value == 1) || ($this->schvaleni3_stav->value == 3))) {
                    throw new Exception(ronText("Schvaleni_uroven_jiz_zapsana"));
                }
            } elseif ($uroven == 2) {
                if ($this->schvaleni3_DT->value && (($this->schvaleni3_stav->value == 1) || ($this->schvaleni3_stav->value == 3))) {
                    throw new Exception(ronText("Schvaleni_uroven_vyssi"));
                }
                //pro stav schvaleni nebo zamitnuto nelze znovu zapsat
                if (($this->schvaleni2_DT->value) && (($this->schvaleni2_stav->value == 1) || ($this->schvaleni2_stav->value == 3))) {
                    throw new Exception(ronText("Schvaleni_uroven_jiz_zapsana"));
                }
            } elseif ($uroven == 1) {
                if (($this->schvaleni3_DT->value &&  (($this->schvaleni3_stav->value == 1) || ($this->schvaleni3_stav->value == 3))) || ($this->schvaleni2_DT->value &&  (($this->schvaleni2_stav->value == 1) || ($this->schvaleni2_stav->value == 3)))) {
                    throw new Exception(ronText("Schvaleni_uroven_vyssi"));
                }
                //pro stav schvaleni nebo zamitnuto nelze znovu zapsat
                if (($this->schvaleni1_DT->value) && (($this->schvaleni1_stav->value == 1) || ($this->schvaleni1_stav->value == 3))) {
                    throw new Exception(ronText("Schvaleni_uroven_jiz_zapsana"));
                }
            }
        }

        if ($prava->lzeZobrazitRC($this->findFieldByName('vystavil')->value)) {
            //u prava na vkladani musi byt zaroven povolena editace
            if (!$vkladani || $prava->priznak)
                return true;
        }

        //pokud je uzivatel prirazen jako schvalovatel ma pravo
        if (($this->RC_Schvalovatele1->value == User::getRC()) || ($this->RC_Schvalovatele2->value == User::getRC()) || ($this->RC_Schvalovatele3->value == User::getRC()))
            return true;

        return false;
    }

    /** Vraci true, pokud je schvaleno libovolnou urovni.
     * @return bool
     */
    public function jeSchvalenaObjednavka()
    {
        if ($this->schvaleni1_DT->value)
            return true;
        if ($this->schvaleni2_DT->value)
            return true;
        if ($this->schvaleni3_DT->value)
            return true;

        return false;
    }

    /**
     * @param string $eventName
     * @param string $xmlContext
     * @return bool
     */
    public function xmlEvents($eventName, $xmlContext)
    {
        return $this->executeXmlEvents($eventName, $xmlContext, ['vydanaobjednavka' => $this->cisloObjednavky]);
    }

    /**
     * @param int $uroven
     * @param bool $vyzadovatSchvaleni
     * @param string|null $RC
     * @return bool
     * @throws DBException
     * @throws Exception
     */
    public function zapisSchvaleni($uroven, $vyzadovatSchvaleni, $RC = null)
    {
        $this->addField('PoznamkaKSchvaleni', ''); // a poznamku

        $this->findFieldByName('vyzadovatSchvaleni')->setNewValue($vyzadovatSchvaleni);

        if (!$this->loadRecord()) {
            throw new Exception('Chyba - zadana objednavka nebyla nalezena!');
        }

        switch ($uroven) {
            case 1 :
                if (empty($this->RC_Schvalovatele1->value) && is_null($RC))
                    throw new Exception(ronText('Schvaleni_neni_schvalovatel'));
                if (!empty($this->RC_Schvalovatele1->value) && $this->RC_Schvalovatele1->value !== $RC)
                    throw new Exception(ronText('Schvaleni_je_prirazen_jiny_schvalovatel'));
                if (empty($this->RC_Schvalovatele1->value))
                    $this->RC_Schvalovatele1->setNewValue($RC);

                $this->schvaleni1_DT->setNewValue(DateUtils::now());
                $this->schvaleni1_jmeno->setNewValue(RonPortalUtils::getLogUserName());
                $this->schvaleni1_stav->setNewValue(1);
                break;
            case 2 :
                if (empty($this->RC_Schvalovatele2->value) && is_null($RC))
                    throw new Exception(ronText('Schvaleni_neni_schvalovatel'));
                if (!empty($this->RC_Schvalovatele2->value) && $this->RC_Schvalovatele2->value !== $RC)
                    throw new Exception(ronText('Schvaleni_je_prirazen_jiny_schvalovatel'));
                if (empty($this->RC_Schvalovatele2->value))
                    $this->RC_Schvalovatele2->setNewValue($RC);

                $this->schvaleni2_DT->setNewValue(DateUtils::now());
                $this->schvaleni2_jmeno->setNewValue(RonPortalUtils::getLogUserName());
                $this->schvaleni2_stav->setNewValue(1);
                break;
            case 3 :
                if (empty($this->RC_Schvalovatele3->value) && is_null($RC))
                    throw new Exception(ronText('Schvaleni_neni_schvalovatel'));
                if (!empty($this->RC_Schvalovatele3->value) && $this->RC_Schvalovatele3->value !== $RC)
                    throw new Exception(ronText('Schvaleni_je_prirazen_jiny_schvalovatel'));
                if (empty($this->RC_Schvalovatele3->value))
                    $this->RC_Schvalovatele3->setNewValue($RC);

                $this->schvaleni3_DT->setNewValue(DateUtils::now());
                $this->schvaleni3_jmeno->setNewValue(RonPortalUtils::getLogUserName());
                $this->schvaleni3_stav->setNewValue(1);
                break;
        }

        $this->historieField->setNewValue(
            "<I>" . date('j.n.Y G:i') . ', <B>' . StringUtils::htmlTextConvert(User::getUserName()) . '</B></I>: ' . sprintf(ronText('Schvaleni_objednavky_log'), '' . $uroven) . "\n" .
            $this->historieField->originalValue);


        $this->logText = StringUtils::fromUTF8('Schválení vydané objednávky');

        if (!$this->xmlEvents(RonPortalXMLEvents::VYD_OBJEDNAVKA_SCHVALENI, 'schvaleni' . $uroven . '_before')) return false;

        if (!$this->updateRecord()) return false;

        if ($this->findFieldByName('schvaleno')->value && $this->findFieldByName('schvaleno')->isChanged() && $this->findFieldByName('odeslatMailem')->value && !$this->findFieldByName('odeslano')->value)
            return $this->odesliObjednavkuMailem();

        return $this->xmlEvents(RonPortalXMLEvents::VYD_OBJEDNAVKA_SCHVALENI, 'schvaleni' . $uroven . '_after');
    }

    /**
     * @return bool
     * @throws DBException
     * @throws Exception
     */
    public function predejKeSchvaleni()
    {
        $this->addField('PoznamkaKSchvaleni', ''); // poznamku

        if (!$this->loadRecord()) {
            throw new Exception('Chyba - zadana objednavka nebyla nalezena!');
        }

        if (isset($RC) && $RC !== $this->findFieldByName('vystavil')->value)
            RonPortalUtils::neniPravoException();

        $this->schvaleni1_stav->setNewValue(4);
        $this->schvaleni1_DT->setNewValue(null);
        $this->schvaleni1_jmeno->setNewValue(null);

        if ($this->schvaleni2_stav->value != 0) {
            $this->schvaleni2_stav->setNewValue(0);
            $this->schvaleni2_DT->setNewValue(null);
            $this->schvaleni2_jmeno->setNewValue(null);
        }
        if ($this->schvaleni3_stav->value != 0) {
            $this->schvaleni3_stav->setNewValue(0);
            $this->schvaleni3_DT->setNewValue(null);
            $this->schvaleni3_jmeno->setNewValue(null);
        }

        $this->historieField->setNewValue(
            "<I>" . date('j.n.Y G:i') . ', <B>' . StringUtils::htmlTextConvert(User::getUserName()) . '</B></I>: ' . ronText('Predani_objednavky_log') . "\n" .
            $this->historieField->originalValue);


        $this->logText = StringUtils::fromUTF8('Předání vydané objednávky ke schválení');

        if (!$this->xmlEvents(RonPortalXMLEvents::VYD_OBJEDNAVKA_SCHVALENI, 'predanischvaleni_before')) return false;

        if (!$this->updateRecord()) return false;

        return $this->xmlEvents(RonPortalXMLEvents::VYD_OBJEDNAVKA_SCHVALENI, 'predanischvaleni_after');
    }

    /**
     * @param $uroven
     * @param $poznamka
     * @return bool
     * @throws DBException
     * @throws Exception
     */
    function zapisVraceni($uroven, $poznamka)
    {
        $this ->addField('PoznamkaKSchvaleni', '', false, 100, $poznamka);

        if (!$this ->loadRecord()) {
            throw new Exception('Chyba pri vraceni schvaleni - zadana objednavka nebyla nalezena!');
        }

        if ($uroven == 1) {
            $this->schvaleni1_DT->setNewValue(DateUtils::now());
            $this->schvaleni1_jmeno->setNewValue(RonPortalUtils::getLogUserName());
            $this->schvaleni1_stav->setNewValue(2);
        }
        if (($uroven == 2) || ($uroven == 3)) {
            $this->schvaleni1_DT->setNewValue(DateUtils::now());
            $this->schvaleni1_jmeno->setNewValue(RonPortalUtils::getLogUserName());
            $this->schvaleni1_stav->setNewValue(5);
            $this->schvaleni2_DT->setNewValue(DateUtils::now());
            $this->schvaleni2_jmeno->setNewValue(RonPortalUtils::getLogUserName());
            $this->schvaleni2_stav->setNewValue(2);
        }
        if ($uroven == 3) {
            $this->schvaleni3_DT->setNewValue(DateUtils::now());
            $this->schvaleni3_jmeno->setNewValue(RonPortalUtils::getLogUserName());
            $this->schvaleni3_stav->setNewValue(2);
        }

        $this->historieField->setNewValue(
            "<I>" . date('j.n.Y G:i') . ", <B>" . StringUtils::htmlTextConvert(User::getUserName()) . "</B></I>: " . sprintf(ronText('Vraceni_objednavky_log'), '' . $uroven) .
            "\n> " . $poznamka . "\n" .
            $this->historieField->originalValue
        );

        $this ->logText = StringUtils::fromUTF8('Vráceno schválení vydané objednávky');

        if (!$this->xmlEvents(RonPortalXMLEvents::VYD_OBJEDNAVKA_SCHVALENI, 'vratitschvaleni' . $uroven . '_before')) return false;
        if (!$this->updateRecord()) return false;
        return $this->xmlEvents(RonPortalXMLEvents::VYD_OBJEDNAVKA_SCHVALENI, 'vratitschvaleni' . $uroven . '_after');
    }

    /**
     * @param $uroven
     * @param $poznamka
     * @return bool
     * @throws DBException
     * @throws Exception
     */
    function zapisZamitnuti($uroven, $poznamka)
    {
        $this ->addField('PoznamkaKSchvaleni', '', false, 100, $poznamka);

        if (!$this ->loadRecord()) {
            throw new Exception('Chyba pri zamitnuti schvaleni - zadana objednavka nebyla nalezena!');
        }

        switch ($uroven) {
            case 1 :
                $this->schvaleni1_DT->setNewValue(DateUtils::now());
                $this->schvaleni1_jmeno->setNewValue(RonPortalUtils::getLogUserName());
                $this->schvaleni1_stav->setNewValue(3);
                break;

            case 2 :
                $this->schvaleni2_DT->setNewValue(DateUtils::now());
                $this->schvaleni2_jmeno->setNewValue(RonPortalUtils::getLogUserName());
                $this->schvaleni2_stav->setNewValue(3);
                break;

            case 3 :
                $this->schvaleni3_DT->setNewValue(DateUtils::now());
                $this->schvaleni3_jmeno->setNewValue(RonPortalUtils::getLogUserName());
                $this->schvaleni3_stav->setNewValue(3);
                break;

            default :
                throw new Exception('Chyba pri zamitnuti schvaleni objednavky - spatne zadana uroven schvaleni!');
        }

        $this->historieField->setNewValue(
            "<I>" . date('j.n.Y G:i') . ", <B>" . StringUtils::htmlTextConvert(User::getUserName()) . "</B></I>: " . sprintf(ronText('Zamitnuti_objednavky_log'), '' . $uroven) .
            "\n> " . $poznamka . "\n" .
            $this->historieField->originalValue
        );

        $this ->logText = StringUtils::fromUTF8('Zamítnutí schválení vydané objednávky');

        if (!$this->xmlEvents(RonPortalXMLEvents::VYD_OBJEDNAVKA_SCHVALENI, 'zamitnoutschvaleni' . $uroven . '_before')) return false;
        if (!$this->updateRecord()) return false;
        return $this->xmlEvents(RonPortalXMLEvents::VYD_OBJEDNAVKA_SCHVALENI, 'zamitnoutschvaleni' . $uroven . '_after');
    }

    /** Zruší jednotlivé úrovně schválení + schválení celé objednávky.
     * @return bool
     * @throws DBException
     * @throws Exception
     */
    function zrusSchvaleni()
    {
        $this->addField('PoznamkaKSchvaleni', ''); // a poznamku

        if (!$this->loadRecord()) {
            throw new Exception('Chyba - zadana objednavka nebyla nalezena!');
        }

        if ($this->findFieldByName('vyrizeno')->value) {
            throw new Exception(ronText('Objednavka_jiz_uzavrena'));
        }

        if ($this->schvaleni1_stav->value == 1) {
            $this->schvaleni1_DT->setNewValue(null);
            $this->schvaleni1_jmeno->setNewValue(null);
            $this->schvaleni1_stav->setNewValue(0);
        }
        if ($this->schvaleni2_stav->value == 1) {
            $this->schvaleni2_DT->setNewValue(null);
            $this->schvaleni2_jmeno->setNewValue(null);
            $this->schvaleni2_stav->setNewValue(0);
        }
        if ($this->schvaleni3_stav->value == 1) {
            $this->schvaleni3_DT->setNewValue(null);
            $this->schvaleni3_jmeno->setNewValue(null);
            $this->schvaleni3_stav->setNewValue(0);
        }
        $this->findFieldByName('schvaleno')->setNewValue(0);

        $this->historieField->setNewValue(
            "<I>" . date('j.n.Y G:i') . ', <B>' . StringUtils::htmlTextConvert(User::getUserName()) . '</B></I>: ' . ronText('Vynulovani_schvaleni_log') . "\n" .
            $this->historieField->originalValue);


        $this->logText = StringUtils::fromUTF8('Zrušení schválení vydané objednávky');

        if (!$this->xmlEvents(RonPortalXMLEvents::VYD_OBJEDNAVKA_SCHVALENI, 'zrusenischvaleni_before')) return false;

        if (!$this->updateRecord()) return false;

        return $this->xmlEvents(RonPortalXMLEvents::VYD_OBJEDNAVKA_SCHVALENI, 'zrusenischvaleni_after');
    }

    /** Nastavi zvolenou osobu k dane urovni schvaleni.
     * @param $uroven
     * @param $RC
     * @return bool
     * @throws DBException
     * @throws Exception
     */
    function nastavSchvalovatele($uroven, $RC)
    {
        if (!$this ->loadRecord()) {
            throw new Exception('Chyba pri nastaveni schvalovatele - zadana objednavka nebyla nalezena!');
        }

        switch ($uroven) {
            case 1 :
                $this->RC_Schvalovatele1->setNewValue($RC);
                break;

            case 2 :
                $this->RC_Schvalovatele2->setNewValue($RC);
                break;

            case 3 :
                $this->RC_Schvalovatele3->setNewValue($RC);
                break;

            default :
                throw new Exception('Chyba pri nastaveni schvalovatele objednavky - spatne zadana uroven schvaleni!');
        }

        $this ->logText = StringUtils::fromUTF8('Nastavení schvalovatele vydané objednávky');

        if (!$this->xmlEvents(RonPortalXMLEvents::VYD_OBJEDNAVKA_SCHVALENI, 'nastavenischvalovatele' . $uroven . '_before')) return false;
        if (!$this->updateRecord()) return false;
        return $this->xmlEvents(RonPortalXMLEvents::VYD_OBJEDNAVKA_SCHVALENI, 'nastavenischvalovatele' . $uroven . '_after');
    }

    /** Odesle mailem dotaz zadavateli objednavky a vlozi jej do historie schvalovani. Pote zrusi schvaleni (ponecha historii).
     * @param $prava
     * @param $dotaz
     * @return bool
     * @throws DBException
     * @throws Exception
     */
    function odesliDotazZadavateli($prava, $dotaz)
    {
        if (!$this ->loadRecord()) {
            throw new Exception('Chyba pri odeslani dotazu zadavateli - zadana objednavka nebyla nalezena!');
        }

        if (!$this->jePravo($prava))
            RonPortalUtils::neniPravoException();

        $prijemce = PracovniPomer::emailOsoby($this->database, $this->findFieldByName('vystavil')->value);
        $odesilatel = PracovniPomer::emailOsoby($this->database, User::getRC());

        if (!$prijemce) {
            // neni zadana zadna adresa pro odeslani
            Log::addRecordToDBLog($this ->database, 'ObjednavkaVydana', 'UPDATE', $this->logDescription, StringUtils::fromUTF8('Odeslání emailu zadavateli - není zadána adresa příjemce'));

            return $dotaz;
        }

        $predmet = sprintf(ronText('Dotaz_zadavateli_predmet'), $this->cisloObjednavky);

        $text = $dotaz;

        $logstr = sprintf(StringUtils::fromUTF8(
            "Odeslání emailu \n\n" .
            "Příjemce: %s\n" .
            "Odesílatel: %s\n" .
            "Předmět: %s\n" .
            "Text: - - - - - - - -\n%s\n- - - - - - - - - - -\n"),
            $prijemce, $odesilatel, $predmet, $text);

        $eMail = new EMail();
        $eMail ->from = $odesilatel;
        $eMail ->to = $prijemce;
        $eMail ->subject = $predmet;
        $eMail ->text = $text;

        try {
            MailManager::sendMail($eMail);
            Log::addRecordToDBLog($this ->database, 'ObjednavkaVydana', 'UPDATE', $this->logDescription, $logstr);
        } catch (Exception $e) {
            $logstr .= StringUtils::fromUTF8("   ... při odesílání emailu došlo k chybě!\n") . $e ->getMessage();
            Log::addRecordToDBLog($this ->database, 'ObjednavkaVydana', 'UPDATE', $this->logDescription, $logstr);

            throw $e;
        }

        $this->historieField->setNewValue(
            "<I>" . date('j.n.Y G:i') . ", <B>" . StringUtils::htmlTextConvert(User::getUserName()) . "</B></I>: " . ronText('Odeslani_dotazu_log') .
            "\n> " . $dotaz . "\n" .
            $this->historieField->originalValue
        );


        $this ->logText = StringUtils::fromUTF8('Odeslání dotazu zadavateli vydané objednávky');

        if (!$this->xmlEvents(RonPortalXMLEvents::VYD_OBJEDNAVKA_SCHVALENI, 'dotazzadavateli')) return false;
        if (!$this->updateRecord()) return false;
        return $this->xmlEvents(RonPortalXMLEvents::VYD_OBJEDNAVKA_SCHVALENI, 'dotazzadavateli');
    }

    /** Vyhleda e-mail dle nastaveni v rozdeleni firmy nebo v kontaktnich udajich firmy a odesle e-mail s objednavkou, do prilohy vlozi sestavu.
     * @param null|PravaNaOsoby $prava
     * @return bool|int
     * @throws DBException
     * @throws Exception
     */
    function odesliObjednavkuMailem($prava = null)
    {
        if (!$this ->loadRecord()) {
            throw new Exception('Chyba pri odeslani objednavky dodavateli - zadana objednavka nebyla nalezena!');
        }

        if (!is_null($prava) && !$this->jePravo($prava))
            RonPortalUtils::neniPravoException();

        if (!$this->findFieldByName('schvaleno')->value)
            throw new Exception(ronText('Neschvalena_objednavka'));

        $kodSestavy = RonPortalSettings::getValue('vydanaobjednavka_sestava');
        $order = null;
        if (!empty($kodSestavy)) {
            $report = Report::loadReport($this->database, $kodSestavy);
            $order = ReportOutputObject::downloadReport($report, null, null, ['CisloObjednavky' => $this->cisloObjednavky],
                ['PocatekZvolenehoObdobi' => StringUtils::format('date', DateUtils::today()), 'KonecZvolenehoObdobi' => StringUtils::format('date', DateUtils::today())], null);

            if (!$order) {
                MessageManager::addErrorMessage('Pri generovani objednavky doslo k chybe!');
            }
        }

        $prijemce = Firma::vratEmailDodavatele($this->database, $this->findFieldByName('dodavatel')->value, $this->findFieldByName('stredisko')->value);
        $odesilatel = RonPortalSettings::getValue('vydanaobjednavka_odesilatel');

        // Pokud neni zadan odesilatel, tak nacte emailovou adresu schvalovatele (3-2-1) nebo aktualniho uzivatele
        if (!$odesilatel) {
            if ($this->schvaleni3_stav->value == 1) {
                $rc = $this->RC_Schvalovatele3->value;
            } elseif ($this->schvaleni2_stav->value == 1) {
                $rc = $this->RC_Schvalovatele2->value;
            } elseif ($this->schvaleni1_stav->value == 1) {
                $rc = $this->RC_Schvalovatele1->value;
            } else {
                $rc = User::getRC();
            }
            $odesilatel = PracovniPomer::emailOsoby($this ->database, $rc);
        }

        $kopie = PracovniPomer::emailOsoby($this->database, $this->findFieldByName('vystavil')->value);

        if (!$prijemce) {
            // neni zadana zadna adresa pro odeslani
            Log::addRecordToDBLog($this ->database, 'ObjednavkaVydana', 'UPDATE', $this->logDescription, StringUtils::fromUTF8('Odeslání emailu zadavateli - není zadána adresa příjemce'));
            return false;
        }

        $predmet = RonPortalSettings::getValue('vydanaobjednavka_predmet');

        $text = RonPortalSettings::getBvarSettings($this->database, 'vydanaobjednavka_text');

        $predmet = $this->nahradParametry($predmet);
        $text = $this->nahradParametry($text);

        global $sklad_mail_objednavky;
        if (isset($sklad_mail_objednavky)) {
            $text .= "\n" . StringUtils::fromUTF8('Načtené adresy dodavatele: ') . $prijemce;
            $text .= "\n" . StringUtils::fromUTF8('Kopie: ') . $kopie;

            $prijemce = $sklad_mail_objednavky;
            $kopie = null;
        }

        $logstr = sprintf(StringUtils::fromUTF8(
            "Odeslání emailu \n\n" .
            "Příjemce: %s\n" .
            "Kopie: %s\n" .
            "Odesílatel: %s\n" .
            "Předmět: %s\n" .
            "Text: - - - - - - - -\n%s\n- - - - - - - - - - -\n"),
            $prijemce, $kopie, $odesilatel, $predmet, $text);

        $eMail = new EMail();
        $eMail ->from = $odesilatel;
        $eMail ->to = $prijemce;
        $eMail ->copyTo = $kopie;
        $eMail ->subject = $predmet;
        $eMail ->text = $text;

        if (!is_null($order)) {
            $eMail->addAttachment("$this->cisloObjednavky.pdf", $order);
        }

        try {
            MailManager::sendMail($eMail);

            if (RonPortalSettings::getIntValue('vydanaobjednavka_typdokumentu_log_zapsat')) {
                $typDokumentu = RonPortalSettings::getIntValue('vydanaobjednavka_typdokumentu_log');
                $this->zapisEmailDoDB($eMail, $typDokumentu);
            }
            Log::addRecordToDBLog($this ->database, 'ObjednavkaVydana', 'UPDATE', $this->logDescription, $logstr);
        } catch (Exception $e) {
            $logstr .= StringUtils::fromUTF8("   ... při odesílání emailu došlo k chybě!\n") . $e ->getMessage();
            Log::addRecordToDBLog($this ->database, 'ObjednavkaVydana', 'UPDATE', $this->logDescription, $logstr);

            throw $e;
        }

        $this->historieField->setNewValue(
            "<I>" . date('j.n.Y G:i') . ", <B>" . StringUtils::htmlTextConvert(User::getUserName()) . "</B></I>: " . ronText('Odeslani_dodavateli_log') . "\n" .
            $this->historieField->originalValue
        );

        $odeslano = $this->findFieldByName('odeslano')->setNewValue(1);

        if ($odeslano->isChanged()) {
            $this->findFieldByName('datumCasOdeslani')->setNewValue(DateUtils::now());
        }

        $this ->logText = StringUtils::fromUTF8('Odeslání objednávky dodavateli');

        return $this->updateRecord();
    }

    /**
     * @param EMail $eMail
     * @param int $typDokumentu
     * @throws DBException
     */
    function zapisEmailDoDB(EMail $eMail, $typDokumentu)
    {
        $rec = new RonPortalDBRecord($this->database, 'EMail');
        $rec->logDescription = StringUtils::fromUTF8('Odeslání objednávky dodavateli');

        $id = $rec->addField('ID', 'serial', true);
        $rec->addTimestampField('DatumCasOdeslani', true, DateUtils::now());
        $rec->addStringField('Odesilatel', true, '160', $eMail->from);
        $rec->addField('Komu', 'memo', true, 0, $eMail->to);
        $rec->addStringField('Kopie', true, '160', $eMail->copyTo);
        $rec->addStringField('Predmet', true, '160', $eMail->subject);
        $rec->addField('Popis', 'memo', true, 0, $eMail->text);
        $rec->addField('PripojeneSoubory', 'memo', true, 0, $eMail->attachments[0]->fileName);
        $rec->addIntField('Odeslano', true, 1);
        $rec->addIntField('PocetPriloh', true, 1);
        $rec->addStringField('Insert_UserName', true, 25, RonPortalUtils::getLogUserName());
        $rec->addTimestampField('Insert_Timestamp', true, DateUtils::now());

        $rec->insertRecord();
        $idEmailu = $id->value;

        $idDokumentu = DokumentyUtils::vytvorDokument($this->database, $eMail->attachments[0]->content, $eMail->attachments[0]->fileName, $typDokumentu, $eMail->attachments[0]->fileName);

        $rec = new RonPortalDBRecord($this->database, 'DokumentEMailu');
        $rec->logDescription = StringUtils::fromUTF8('Odeslání objednávky dodavateli');

        $rec->addIntField('ID', true, $idDokumentu);
        $rec->addIntField('ID_Email', true, $idEmailu);

        $rec->insertRecord();
    }

    function nahradParametry($text)
    {
        $text = str_replace('%CISLOOBJEDNAVKY%', $this->cisloObjednavky, $text);

        $vystavilPP = PracovniPomer::nactiPPOsoby($this->database, $this->findFieldByName('vystavil')->value);
        $vystavilPP = $vystavilPP[0];
        $text = str_replace('%VYSTAVIL%', $vystavilPP->jmenoPrijmeni, $text);
        $text = str_replace('%EMAIL%', $vystavilPP->eMail, $text);
        $text = str_replace('%TELEFON%', $vystavilPP->telefon, $text);
        $text = str_replace('%MOBIL%', $vystavilPP->mobil, $text);

        return $text;
    }

    function loadAndUpdateRecord($filtr = '')
    {
        if ($this->loadRecord($filtr)) {
            if ($this->findFieldByName('datumVystaveni')->isChanged()) {
                //pred vlozenim se k datu pricte aktualni cas, aby bylo mozne radit, nastaveno zde, aby se datum neaktulizovalo pri kazde aktualizaci
                $this->findFieldByName('datumVystaveni')->setNewValue($this->findFieldByName('datumVystaveni')->value + (DateUtils::now() - DateUtils::startOfTheDay(DateUtils::now())));
            }

            return $this->updateRecord($filtr);
        } else {
            throw new Exception('Chyba pri editaci zaznamu - puvodni zaznam nebyl nalezen!');
        }
    }

}
