<?php
/**
 * stanislav.elbl, 08.07.2022 14:18
 */

namespace RonPortal\Skoleni;
use Ron\DB\Database;
use Ron\TextManager;

/**
 * Class Skoleni
 * @package RonPortal\Skoleni
 * Reprezentuje zaznam z tabulky Skoleni
 */
class Skoleni
{
    const STAV_PRIPRAVUJE_SE = 0;
    const STAV_PROBIHA = 1;
    const STAV_PROVEDENO = 2;
    const STAV_ZRUSENO = 3;

    /** @var string     Cislo (kod) - primarni klic v tabulce Skoleni*/
    public $cisloSkoleni;

    /** @var string */
    public $popis;

    /** @var string */
    public $druhSkoleni;

    /** @var int */
    public $datumPocatkuSkoleni;

    /** @var int */
    public $datumUkonceniSkoleni;
    /** @var int */
    public $casPocatkuSkoleni;
    /** @var int */
    public $casUkonceniSkoleni;

    /** @var int */
    public $kapacita;

    /** @var int    Viz konstanty STAV_... */
    public $stavSkoleni;

    /** @var int */
    public $pozadovatSchvaleniPrihl;

    /** @var string Vazba do tabulky PlanovaneSkoleni - nastavuje se jen v pripade, ze je v tabulce nastaven priznak UcastniciDlePlanSkoleni */
    public $cisloPlanovanehoSkoleni;

    /** @var int    Priznak, jestli se jedna o elearningovy kurz - nacten z tabulky druhSkoleni */
    public $elearning;

    /** @var string */
    public $skolitel;
    /** @var string */
    public $hodnotici;
    /** @var string     Kod skolici firmy*/
    public $skoliciFirma;
    /** @var string     Nazev skolici firmy*/
    public $nazevSkoliciFirmy;


    /**
     * @param Database $database
     * @param array $row
     */
    private function loadRow($database, $row)
    {
        $this ->cisloSkoleni = $row[0];
        $this ->druhSkoleni = $row[1];
        $this ->popis = $row[2];
        $this ->datumPocatkuSkoleni = $database ->stringToTimestamp($row[3]);
        $this ->stavSkoleni = (int)$row[4];
        $this ->kapacita = (int)$row[5];

        $this ->cisloPlanovanehoSkoleni = (int)$row[6] ? $row[7] : null;

        $this ->elearning = (int)$row[8];

        $this ->pozadovatSchvaleniPrihl = (int)$row[9];

        $this ->datumUkonceniSkoleni = $database ->stringToTimestamp($row[10]);
        $this ->casPocatkuSkoleni = $database ->stringToTimeOfDay($row[11]);
        $this ->casUkonceniSkoleni = $database ->stringToTimeOfDay($row[12]);

        $this ->skolitel = $row[13];
        $this ->hodnotici = $row[14];
        $this ->skoliciFirma = $row[15];
        $this ->nazevSkoliciFirmy = $row[16];
    }

    /**
     * @param Database  $database
     * @param string    $cisloSkoleni
     * @return static
     * @throws \Exception
     */
    public static function nactiSkoleni($database, $cisloSkoleni)
    {
        $params = $database->createParameters();

        $sql =
            'SELECT CisloSkoleni, DruhSkoleni, Skoleni.Popis, DatumPocatkuSkoleni, StavSkoleni, Kapacita,' . // 0..5
            '  UcastniciDlePlanSkoleni, CisloPlanovanehoSkoleni, DS.Elearning, PozadovatSchvaleniPrihl,' . // 6..9
            '  DatumUkonceniSkoleni, CasPocatkuSkoleni, CasUkonceniSkoleni,' .    // 10..12
            '  Skolitel, Hodnotici, SkoliciFirma, Firma.Nazev1' . // 13..15
            ' FROM Skoleni' .
            ' JOIN DruhSkoleni DS ON Skoleni.DruhSkoleni = DS.Kod' .
            ' LEFT OUTER JOIN Firma ON Firma.KodFirmy = Skoleni.SkoliciFirma' .
            ' WHERE CisloSkoleni = ' . $params->addParameter($cisloSkoleni);

        $cursor = $database->select($sql, $params);
        $row = $cursor->fetch();
        $cursor ->close();

        if (!$row) {
            throw new \Exception(TextManager::_('SkoleniNenalezeno'));
        }

        $result = new static();
        $result ->loadRow($database, $row);
        return $result;
    }


    /**
     * Zkontroluje jestli ma skoleni volna mista
     * @param Database  $database
     * @return bool     Vraci true pokud je pocet prihlasenych < kapacita skoleni
     */
    public function jeVolnaKapacitaSkoleni($database)
    {
        $params = $database->createParameters();

        $sql =
            "SELECT COUNT(RC) FROM GenerovaneSkoleni" .
            " WHERE CisloSkoleni = " . $params->addParameter($this ->cisloSkoleni);

        $cursor = $database->select($sql, $params);
        $row = $cursor->fetch();
        $cursor ->close();

        return (int)$row[0] < $this ->kapacita;
    }
}
