<?php
/**
 * stanislav.elbl, 22.02.2016
 */

namespace RonPortal\Skoleni;

use Exception;
use Ron\CalendarItem;
use Ron\DB\Database;
use function Ron\ronText;


class SkoleniCalendarItem extends CalendarItem
{
    function __construct()
    {
        parent::__construct();
    }

    /**
     * @throws Exception
     */
    function computeUid()
    {
        if (!$this ->dateTimeFrom) {
            throw new Exception('Chyba v SkoleniCalendarItem::computeUid! Nejdrive musi byt nastaven datum!');
        }

        $this ->uid .= "." . date("Y-m-d", $this ->dateTimeFrom);
    }

    /**
     * @param Database $database
     * @param array $row
     */
    function loadRow($database, $row)
    {
        $this ->dateTimeFrom = $database ->stringToTimestamp($row[1]) + $database->stringToSecondsOfDay($row[2]);

        $this ->dateTimeTo = $row[3]
            ? $database ->stringToTimestamp($row[3])
            : $database ->stringToTimestamp($row[1]);

        $this->dateTimeTo += $database->stringToSecondsOfDay($row[4]);

        $this ->uid = "ronportal.kurz.{$row[0]}";

        $this->summary = $row[5];

        $desc = '';

        if ($row[7]) {
            $desc .= ronText('Skoleni_Adresa') . ': ' . $row[7] .
                ($row[8] ? '\, ' . $row[8] : '') .
                ($row[9] ? '\, ' . $row[9] : '') .
                ($row[10] ? '\, ' . ronText('Skoleni_Patro') . ': ' . $row[10] : '');
        }

        $desc .= '\n' . ronText('Skoleni_obsah_kurzu') . ": " . $row[6];

        if ($row[13]) {
            $desc .= '\n' . $row[13];
        }

        if ($row[12]) {
            $desc .= '\n' . ronText('Skoleni_poznamka') . ": " . $row[12];
        }

        $this->description =  $desc;
    }


    /**
     * @param Database $database
     * @param $cisloSkoleni
     * @return bool|SkoleniCalendarItem|SkoleniCalendarItem[]
     * @throws Exception
     */
    static function loadKurz($database, $cisloSkoleni)
    {
        $params = $database ->createParameters();

        $sql =
            'SELECT Skoleni.CisloSkoleni, Skoleni.DatumPocatkuSkoleni, Skoleni.CasPocatkuSkoleni, ' . // 0..2
            '  Skoleni.DatumUkonceniSkoleni, Skoleni.CasUkonceniSkoleni,  Skoleni.Popis, '. $database ->castWideMemo('Skoleni.ObsahSkoleni') . ', ' . // 3..6
            '  Umisteni.Popis, Umisteni.Ulice, Umisteni.Mesto, Umisteni.Patro, Skoleni.DlouhodobyKurz, ' . $database ->castWideMemo('Skoleni.HodnoceniSkoleni') . // 7..12
            '  FROM Skoleni ' .
            ' LEFT OUTER JOIN Umisteni ' .
            '    ON Umisteni.Kod = Skoleni.MistoSkoleni ';

        if ($cisloSkoleni) {
            // jedna konkretni sluzebni cesta
            $sql .= ' WHERE CisloSkoleni = ' . $params ->addParameter($cisloSkoleni);

            $cursor = $database ->getSqlResult($sql, $params);

            if (!$row = $cursor ->fetch()) {
                return false;
            }

            //dlouhodoby kurz
            if ($row[11]) {
                $params = $database ->createParameters();

                $sql =
                    'SELECT TerminSkoleni.CisloSkoleni, TerminSkoleni.Datum, TerminSkoleni.Cas, TerminSkoleni.Poznamka ' . // 0..3
                    '  FROM TerminSkoleni ';

                if ($cisloSkoleni) {
                    // jedna konkretni sluzebni cesta
                    $sql .= ' WHERE CisloSkoleni = ' . $params ->addParameter($cisloSkoleni);

                    $cursor = $database ->getSqlResult($sql, $params);

                    $result = [];

                    /** @noinspection PhpAssignmentInConditionInspection */
                    while ($rowTermin = $cursor ->fetch()) {
                        $rowEvent = $row;
                        $rowEvent[1] = $rowTermin[1];
                        $rowEvent[2] = $rowTermin[2];
                        $rowEvent[3] = $rowTermin[1];
                        $rowEvent[13] = $rowTermin[3];

                        $event = new SkoleniCalendarItem();
                        $event->loadRow($database, $rowEvent);
                        $event->computeUid();
                        $result[] = $event;
                    }

                    return  $result;
                }

            } else {
                $result = new SkoleniCalendarItem();
                $result->loadRow($database, $row);
                return $result;
            }
        }

        return false;
    }
}
