<?php
/**
 * stanislav.elbl, 05.07.2022 22:55
 */

namespace RonPortal\Skoleni;

use Ron\DateUtils;
use Ron\DB\Database;
use Ron\MessageManager;
use Ron\MVC\JsonController;
use Ron\MVC\JsonOutputObject;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\NeniPravoException;
use RonPortal\Prava;

class SkoleniJsController extends JsonController
{
    /** @var Database */
    private $database;


    /**
     * SkoleniJsController constructor.
     * @param Database $database
     */
    public function __construct(Database $database)
    {
        $this->database = $database;
    }


    /**
     * Vraci true, pokud ma uzivatel pravo editovat terminy a zaznamenavat ucast na terminech pro zadane skoleni
     * (pouziva se pro dlouhodobe skoleni)
     * @param $cisloSkoleni
     * @return bool
     */
    public static function jePravoNaZapisUcasti($cisloSkoleni)
    {
        $prava = Prava::vratPrava('skoleni_lektor');

//        if (!Prava::jsouNejakePrava('skoleni_absolvovani')) {
        if (!$prava ->jsouNejakePrava()) {
            return false;
        }

        if (trim($prava ->priznak3)) {
            // uzivatel ma omezena cisla skoleni, u kterych muze evidovat ucast
            if (!StringUtils::isInList($cisloSkoleni, $prava ->priznak3)) {
                return false;
            }
        }

        return true;
    }


    public static function zkontrolujPravoNaZapisUcasti($cisloSkoleni)
    {
        if (!self::jePravoNaZapisUcasti($cisloSkoleni)) {
            throw new NeniPravoException();
        }
    }


    /**
     * @param JsonOutputObject $output
     * @throws NeniPravoException
     */
    private function terminSkoleni($output)
    {
//        if (!Prava::jsouNejakePrava('skoleni_absolvovani')) {
//            throw new NeniPravoException();
//        }
        self::zkontrolujPravoNaZapisUcasti($_GET['cisloskoleni']);

        $f = new TerminSkoleniFormular($this ->database);

        $output ->addData('data', $f ->getJsonData());
    }


    /**
     * @param JsonOutputObject $output
     * @throws NeniPravoException
     */
    private function zapisTerminSkoleni($output)
    {
        self::zkontrolujPravoNaZapisUcasti($_POST['CisloSkoleni']);

        $f = new TerminSkoleniFormular($this ->database);
        if (!$f ->handleAction()) {
            $output ->addMessages(MessageManager::getInstance());
        }
    }


    /**
     * Nacte data pro dialog "Ucast na skoleni"
     * @param JsonOutputObject $output
     * @throws NeniPravoException
     */
    private function ucastNaSkoleni($output)
    {
        self::zkontrolujPravoNaZapisUcasti($_GET['cisloskoleni']);
//        if (!Prava::jsouNejakePrava('skoleni_absolvovani')) {
//            throw new NeniPravoException();
//        }

        $f = new UcastNaSkoleniFormular($this ->database,
            $_GET['cisloskoleni'],
            DateUtils::urlStringToDate($_GET['datum'])
        );

        $output ->addData('data', $f ->getJsonData());
    }


    /**
     * Nacte data pro dialog "Ucast na skoleni"
     * @param JsonOutputObject $output
     * @throws NeniPravoException
     */
    private function zapisUcastNaSkoleni($output)
    {
        self::zkontrolujPravoNaZapisUcasti($_POST['CisloSkoleni']);

        $f = new UcastNaSkoleniFormular($this ->database,
            $_POST['CisloSkoleni'], DateUtils::urlStringToDate($_POST['Datum']));
        if (!$f ->handleAction()) {
            $output ->addMessages(MessageManager::getInstance());
        }
    }


    /**
     * @param JsonOutputObject $output
     */
    private function kapacitaSkoleni($output)
    {
        $params = $this->database->createParameters();

        $sql =
            'SELECT Skoleni.Kapacita, Skoleni.DatumPocatkuSkoleni, COUNT(*)' .
            ' FROM Skoleni LEFT JOIN GenerovaneSkoleni GS' .
            '  ON (Skoleni.CisloSkoleni = GS.CisloSkoleni) AND (GS.Potvrzeni < 5)' .
            ' WHERE Skoleni.CisloSkoleni = ' . $params->addParameter($_GET['cisloskoleni'] ?? '') .
            ' GROUP BY GS.DruhSkoleni, GS.CisloSkoleni, Skoleni.Kapacita, Skoleni.DatumPocatkuSkoleni';

        $cursor = $this->database->getSqlResult($sql, $params);
        $row = $cursor->fetch();
        $cursor ->close();

        if ($row) {
            $output ->addData('kapacita', (int)$row[0]);
            $output ->addData('datumPocatkuSkoleni', StringUtils::format('date', $this ->database ->stringToTimestamp($row[1])));

            $output ->addData('pocetPrihlasenych', (int)$row[2]);
        }
    }


    /**
     * @param JsonOutputObject $output
     */
    private function detailSkoleni($output)
    {
        $form = new DetailSkoleniDialog($this ->database);
        $output ->addData('data', $form ->getJsonData());
    }


    /**
     * @param string $action
     * @param JsonOutputObject $output
     */
    function handleJsonAction($action, $output)
    {
        TextManager::getInstance() ->loadStrings('skoleni.ini');
        Prava::nastavObdobiProPrava(time(), time());

        switch ($_GET['action'] ?? null) {
            case 'terminskoleni' :
                $this ->terminSkoleni($output);
                break;

            case 'zapistermin' :
                $this ->zapisTerminSkoleni($output);
                break;

            case 'ucastnaskoleni' :
                $this ->ucastNaSkoleni($output);
                break;
            case 'zapisucast' :
                $this ->zapisUcastNaSkoleni($output);
                break;

            case 'kapacitaskoleni' :
                $this ->kapacitaSkoleni($output);
                break;

            case 'detailskoleni' :
                $this ->detailSkoleni($output);
                break;
        }
    }
}
