<?php
/**
 * stanislav.elbl, 26.05.2021 0:01
 */

namespace RonPortal\Skoleni;

use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\MessageManager;

class SkoleniUtils
{
    /** 
     * Vraci true, pokud pro zadanou osobu a druh skoleni existuje vygenerovany navrh bez zadaneho absolvovani.
     * @param Database $database
     * @param string $osobaRC
     * @param string $druhSkoleni
     * @return bool
     * @throws DBException
     */
    public static function existujeNavrhTypuSkoleni($database, $osobaRC, $druhSkoleni)
    {
        $params = $database->createParameters();
        $sql =
            'SELECT COUNT(RC) FROM GenerovaneSkoleni LEFT JOIN Skoleni' .
            '  ON (Skoleni.CisloSkoleni = GenerovaneSkoleni.CisloSkoleni)' .
            ' WHERE GenerovaneSkoleni.RC = ' . $params->addParameter($osobaRC) .
            '  AND GenerovaneSkoleni.DruhSkoleni = ' . $params->addParameter($druhSkoleni) .
            '  AND (GenerovaneSkoleni.Potvrzeni IN (0,5,6)' .
            '    OR (GenerovaneSkoleni.Potvrzeni = 1 AND (Skoleni.StavSkoleni IS NULL OR Skoleni.StavSkoleni <> 2)))';

        MessageManager::addDebugMessage($sql);

        $cursor = $database->getSqlResult($sql, $params);
        $row = $cursor ->fetch();
        $cursor ->close();

        return $row  &&  $row[0] > 0;
    }


    /**
     *
     * @param Database $database
     * @param string $cisloSkoleni kod skoleni
     * @param string $rc RC osoby
     * @return bool     Vraci true, pokud je osoba prihlasena na planu skoleni
     */
    public static function jeOsobaNaPlanovanemSkoleni($database, $cisloSkoleni, $rc)
    {
        $params = $database->createParameters();

        $sql =
            "SELECT RC FROM PlanovaneSkoleniOsob" .
            " WHERE CisloSkoleni = " . $params->addParameter($cisloSkoleni) .
            "  AND RC = " . $params->addParameter($rc) .
            "  AND Potvrzeni = 1";

        $cursor = $database->select($sql, $params);
        $row = $cursor->fetch();
        $cursor ->close();

        return (bool)$row;
    }

    /**
     *
     * @param Database $database
     * @param string $cisloSkoleni kod skoleni
     * @param string $rc RC osoby
     * @return bool Vraci true, pokud je osoba prihlasena na skoleni
     */
    public static function jeOsobaNaSkoleni($database, $cisloSkoleni, $rc)
    {
        $params = $database->createParameters();

        $sql =
            "SELECT DISTINCT RC FROM GenerovaneSkoleni" .
            " WHERE CisloSkoleni = " . $params->addParameter($cisloSkoleni) .
            "  AND RC = " . $params->addParameter($rc) .
            "  AND SchvaleniPrihlasky <> 2";

        $cursor = $database->select($sql, $params);
        $row = $cursor->fetch();
        $cursor ->close();

        return (bool)$row;
    }


    /**
     * @param Database $database
     * @param string $skoleni   Kod skoleni
     * @param string $rc        RC osoby
     * @return bool     Vraci true, pokud ma osoba zaznamenanou (ne)ucast na nekterem z terminu skoleni
     */
    public static function jeOsobaNaTerminuSkoleni($database, $skoleni, $rc)
    {
        $params = $database->createParameters();

        $sql =
            "SELECT DISTINCT RC FROM UcastNaTerminuSkoleni" .
            " WHERE CisloSkoleni = " . $params->addParameter($skoleni) .
            " AND RC = " . $params->addParameter($rc);

        $cursor = $database->select($sql, $params);
        $row = $cursor->fetch();
        $cursor ->close();

        return (bool)$row;
    }


    /**
     * @param Database $database
     * @param string $skoleni kod skoleni
     * @param string $rc RC osoby
     * @return mixed
     */
    public static function pocetNeabsolvovanychEKurzu($database, $skoleni, $rc)
    {
        $params = $database->createParameters();

        $sql =
            'SELECT COUNT(ID)' .
            ' FROM GenerovaneSkoleni JOIN EKurzOsoby' .
            '  ON ((GenerovaneSkoleni.RC = EKurzOsoby.RC_Skoleni) AND (GenerovaneSkoleni.DruhSkoleni = EKurzOsoby.DruhSkoleni)' .
            '   AND (GenerovaneSkoleni.DatumVznikuPovinnosti = EKurzOsoby.DatumVznikuPovinnosti))' .
            ' WHERE GenerovaneSkoleni.CisloSkoleni = ' . $params->addParameter($skoleni) .
            '  AND GenerovaneSkoleni.RC = ' . $params->addParameter($rc) .
            '  AND EKurzOsoby.Stav NOT IN (3, 4)'; // neni absolvovan uspesne ani neuspesne

        $cursor = $database->getSqlResult($sql, $params);
        $row = $cursor->fetch();
        $cursor ->close();

        return (int)$row[0];
    }



    /**
     * Zjisti volnou kapacitu planovaneho skoleni
     * @param Database $database
     * @param string $cisloSkoleni
     * @return int
     */
    public static function volnaKapacitaPlanovanehoSkoleni($database, $cisloSkoleni)
    {
        $params = $database->createParameters();
        $sql =
            'SELECT PS.PocetOsob, (SELECT COUNT(RC) FROM PlanovaneSkoleniOsob PSO Where PSO.CisloSkoleni = PS.CisloSkoleni AND PSO.Potvrzeni = 1)' .
            ' FROM PlanovaneSkoleni PS' .
            ' WHERE PS.CisloSkoleni = ' . $params->addParameter($cisloSkoleni);

        $cursor = $database->getSqlResult($sql, $params);
        $row = $cursor->fetch();
        $cursor ->close();

        return (int)$row[0] - (int)$row[1];
    }



    /**
     * Zkontroluje jestli ma mistnost, ve stejnem case volna mista. Pokud neni kapacita zadana, bere se ze neni omezeno.
     * @param Database $database
     * @param string $cisloSkoleni Kod skoleni
     * @param string $rc RC prihlasovane osoby (vynecha se z poctu prihlasenych - pro pripady kdy prihlasuji do jedne mistnosti na vice kurzu a dodatecne doplni prihlaseni)
     * @return bool Vraci true pokud je pocet prihlasenych < kapacita mistnosti nebo kapacita neni zadana
     */
    public static function jeVolnaKapacitaMistnosti($database, $cisloSkoleni, $rc)
    {
        // nejdrive zjisti kapacitu
        $paramsKapacita = $database->createParameters();

        $sql =
            'SELECT Umisteni.KapacitaMistnosti' .
            ' FROM Skoleni JOIN Umisteni ON Skoleni.MistoSkoleni = Umisteni.Kod' .
            ' WHERE CisloSkoleni = ' . $paramsKapacita->addParameter($cisloSkoleni);

        $cursorKapacita = $database->select($sql, $paramsKapacita);
        $rowKapacita = $cursorKapacita->fetch();
        $cursorKapacita->close();

        $kapacita = $rowKapacita ? $rowKapacita[0] : null;

        if (is_null($kapacita)) {   
            return true;
        }

        $params = $database->createParameters();

        $sql =
            'SELECT COUNT(DISTINCT RC)' .
            ' FROM GenerovaneSkoleni JOIN Skoleni ON (Skoleni.CisloSkoleni = GenerovaneSkoleni.CisloSkoleni)' .
            ' WHERE EXISTS(' .
            '   SELECT RC FROM Skoleni S' .
            '   WHERE S.CisloSkoleni = ' . $params->addParameter($cisloSkoleni) .
            '     AND S.MistoSkoleni = Skoleni.MistoSkoleni' .
            '     AND S.CasPocatkuSkoleni IS NOT NULL' .
            '     AND S.DatumUkonceniSkoleni IS NOT NULL' .
            '     AND S.CasUkonceniSkoleni IS NOT NULL' .
            '     AND (' .
            '       ((S.DatumPocatkuSkoleni <= Skoleni.DatumUkonceniSkoleni AND S.DatumUkonceniSkoleni >= Skoleni.DatumPocatkuSkoleni)' .
            '         AND (S.CasPocatkuSkoleni < Skoleni.CasUkonceniSkoleni AND S.CasUkonceniSkoleni > Skoleni.CasPocatkuSkoleni))' .
            '       OR' .
            '       ((Skoleni.DatumPocatkuSkoleni <= S.DatumUkonceniSkoleni AND Skoleni.DatumUkonceniSkoleni >= S.DatumPocatkuSKoleni)' .
            '         AND (Skoleni.CasPocatkuSkoleni < S.CasUkonceniSkoleni AND Skoleni.CasUkonceniSkoleni > S.CasPocatkuSkoleni))' .
            '     )' .
            ' )' .
            ' AND GenerovaneSkoleni.RC <> ' .  $params->addParameter($rc);

        $cursor = $database->select($sql, $params);
        $row = $cursor->fetch();
        $cursor ->close();

        $prihlasenych = $row ? (int)$row[0] : 0;

        //kapacita neni zadana
        return $prihlasenych < $kapacita;
    }
    
    /**
     * Kontroluje jestli je typ dokumentu nastaven jako verejny
     * @param Database $database
     * @param integer $idDokumentu id dokumentu
     * @return boolean true pokud je typ dokumentu verejny
     * @throws \Exception
     */
    public static function jeDokumentSkoleniVerejny($database, $idDokumentu)
    {        
        $params = $database->createParameters();

        $sql =
            'SELECT Verejny FROM Dokument JOIN TypDokumentu ON (TypDokumentu.kod = Dokument.typ)' .
            ' WHERE Dokument.id = ' . $params->addParameter($idDokumentu);
        
        $cursor  = $database->select($sql, $params);
        $row = $cursor->fetch();
        $cursor ->close();

        return $row  &&  (int)$row[0] == 1;
    }


}
