<?php
/**
 * stanislav.elbl, 28.09.2021 22:36
 */

namespace RonPortal\SluzebniCesty;

use Ron\DB\Database;
use Ron\DB\DBRecordField;
use RonPortal\RonPortalDBRecord;
use RonPortal\RonPortalSettings;
use RonPortal\RonPortalXMLEvents;

class RadekUctovaniSCDBRecord extends RonPortalDBRecord
{
    /** @var  DBRecordField */
    public $fCisloRadku;
    /** @var  DBRecordField */
    public $fDen;
    /** @var  DBRecordField */
    public $fPopis;
    /** @var  DBRecordField */
    public $fKodMeny;
    /** @var  DBRecordField */
    public $fTyp;
    /** @var  DBRecordField */
    public $fZpusobPlatby;
    /** @var  DBRecordField */
    public $fCastka;

    /** @var  DBRecordField */
    public $fCastkaDPH;
    /** @var  DBRecordField */
    public $fNVV_SpotrebaVozidla;
    /** @var  DBRecordField */
    public $fNVV_PocetUjetychKM;
    /** @var  DBRecordField */
    public $fNVV_NepocitatZaklSazbu;
    /** @var  DBRecordField */
    public $fNVV_NeprokazanaCenaPHM;
    /** @var  DBRecordField */
    public $fNVV_DruhPHM;
    /** @var  DBRecordField */
    public $fNVV_CenaPHM;
    /** @var  DBRecordField */
    public $fNVV_Vozidlo;

    /** @var  DBRecordField */
    public $fDM_VlastniKurz;
    /** @var  DBRecordField */
    public $fDM_Kurz_Mnozstvi;
    /** @var  DBRecordField */
    public $fDM_Kurz;

    /** @var DBRecordField */
    public $fKodTypuRozdeleni;
    /** @var DBRecordField */
    public $fRozdeleni;
    /** @var DBRecordField */
    public $fKodTypuRozdeleni2;
    /** @var DBRecordField */
    public $fRozdeleni2;

    /**
     * @param Database $database
     * @param int $idSC
     * @param int $cisloRadku
     * @param object|string $logDescription
     */
    function __construct($database, $idSC, $cisloRadku = null, $logDescription = 'Sluzebni cesty')
    {
        parent::__construct($database, 'RadekUctovaniSCesty', $logDescription);

        $this ->beforePostEvent = RonPortalXMLEvents::SC_UCTOVANI_BEFORE_POST;
        $this ->afterPostEvent = RonPortalXMLEvents::SC_UCTOVANI_AFTER_POST;

        $this ->addField('SluzebniCesta_ID', 'int', true, 0, $idSC); // readonly = true
        $this ->fCisloRadku = $this ->addField('CisloRadku', 'int', true, 0, $cisloRadku);

        $this ->fDen = $this ->addField('Den', 'date', true);
        $this ->fPopis = $this ->addField('Popis', '', true, 160);
        $this ->fKodMeny = $this ->addField('KodMeny', '', true, 10);
        $this ->fTyp = $this ->addField('Typ', 'int', true);
        $this ->fZpusobPlatby = $this ->addField('ZpusobPlatby', 'int', true, 0, 0);

        $this ->fNVV_Vozidlo = $this ->addField('NVV_Vozidlo', '', true, 30);

        $this ->fCastka = $this ->addField('Castka', 'float', true, 0, 0);
        if (RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_ZADAVAT_DPH)) {
            $this ->fCastkaDPH = $this ->addField('CastkaDPH', 'float', true, 0, 0);
        }

        // nahrada
        $this ->fNVV_SpotrebaVozidla = $this ->addField('NVV_SpotrebaVozidla', 'float', true);
        $this ->fNVV_PocetUjetychKM = $this ->addField('NVV_PocetUjetychKM', 'int', true);
        $this ->fNVV_NepocitatZaklSazbu = $this ->addField('NVV_NepocitatZaklSazbu', 'int', true, 0, RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_NEPOCITAT_ZAKLADNI_SAZBU));

        $this ->fNVV_NeprokazanaCenaPHM = $this ->addField('NVV_NeprokazanaCenaPHM', 'int', true, 0, 0);
        $this ->fNVV_DruhPHM = $this ->addField('NVV_DruhPHM', '', true, 10);
        $this ->fNVV_CenaPHM = $this ->addField('NVV_CenaPHM', 'float', true);

        if (RonPortalSettings::getIntValue('vlastnikurzradkusc')) {
            $this ->fDM_VlastniKurz = $this ->addField('DM_VlastniKurz', 'int', true, 0, 0);
            $this ->fDM_Kurz_Mnozstvi = $this ->addField('DM_Kurz_Mnozstvi', 'float', true);
            $this ->fDM_Kurz = $this ->addField('DM_Kurz', 'float', true);
        }

        if (RonPortalSettings::getIntValue('typrozdeleniusc')) {
            $this->fKodTypuRozdeleni = $this ->addField('KodTypuRozdeleni', 'int', true);
            $this->fRozdeleni = $this->addField('Rozdeleni', '', true, 30);
        }
        if (RonPortalSettings::getIntValue('typrozdeleniusc2')) {
            $this->fKodTypuRozdeleni2 = $this ->addField('KodTypuRozdeleni2', 'int', true);
            $this->fRozdeleni2 = $this->addField('Rozdeleni2', '', true, 30);
        }

        $params = $database ->createParameters();
        $where =
            ' SluzebniCesta_ID = ' . $params ->addParameter((int)$idSC);

        if ($cisloRadku) {
            $where .= ' AND CisloRadku = ' . $params ->addParameter($cisloRadku);
        }

        $this ->setWhere($where, $params);
    }

}
