<?php
/**
 * stanislav.elbl, 24.03.2021
 */

namespace RonPortal\SluzebniCesty;

use Ron\CalendarItem;
use Ron\DB\Database;

class SluzebniCestaCalItem extends CalendarItem
{
    public $idPP;

    public $zahranicni;


    function __construct()
    {
        parent::__construct();
    }

    /**
     * @param Database $database
     * @param array $row
     */
    protected function loadRow($database, $row)
    {
        $this ->idPP = $row[0];

        $this ->dateTimeFrom = $database ->stringToTimestamp($row[1]);
        $this ->dateTimeTo = $database ->stringToTimestamp($row[2]);

        $this ->uid = "ronportal.sluzebni.cesta.{$row[3]}";

        $this->summary = $row[8] ?: \Ron\ronText('Sluzebni_cesta');

        $desc =
            ($row[6] ? \Ron\ronText('Cil_sc') . ': ' . $row[6] : '') .
            ($row[5] ? ($row[6] ? ', ' : \Ron\ronText('Cil_sc') . ': ') . $row[5] : '');

        $this->description =
            $desc .
            ($desc ? "\n" : '') .
            $row[9]; // pripoji poznamku

        $this ->zahranicni = (int)$row[10];
    }


    /**
     * @param Database $database
     * @param $idSlCesty
     * @param null $login
     * @return bool|SluzebniCestaCalItem|SluzebniCestaCalItem[]
     * @throws \Ron\DB\DBException
     */
    static function loadSluzebniCesty($database, $idSlCesty, $login = null)
    {
        $params = $database ->createParameters();

        $sql =
            'SELECT PracovniPomer.Id, SluzebniCesta.DatumCasOd, SluzebniCesta.DatumCasDo, ' . // 0..2
            '  SluzebniCesta.Id, SluzebniCesta.Cil, SluzebniCesta.PopisCile, MistoSluzebniCesty.Popis AS PopisCile2, ' . // 3..6
            '  SluzebniCesta.Pres, SluzebniCesta.Popis, '. $database ->castWideMemo('SluzebniCesta.Poznamka') . ', ' . // 7..9
            '  SluzebniCesta.Zahranicni ' . // 10
            //'  SluzebniCesta.Uzivatelska1, SluzebniCesta.Uzivatelska2, SluzebniCesta.Uzivatelska3, SluzebniCesta.Uzivatelska4, SluzebniCesta.Uzivatelska5 ' . // 11..15
            '  FROM SluzebniCesta INNER JOIN PracovniPomer ' .
            '    ON PracovniPomer.RC = SluzebniCesta.RC ' .
            '   AND PracovniPomer.OsobniCislo = SluzebniCesta.OsobniCislo ' .
            '   AND PracovniPomer.DatumNastupu = SluzebniCesta.DatumNastupu ' .
            '  INNER JOIN Osoba' .
            '    ON PracovniPomer.RC = Osoba.RC ' .
            ' LEFT OUTER JOIN MistoSluzebniCesty ' .
            '    ON MistoSluzebniCesty.Kod = SluzebniCesta.Cil ';

        if ($idSlCesty) {
            // jedna konkretni sluzebni cesta
            $sql .= ' WHERE SluzebniCesta.Id = ' . $params ->addParameter((int)$idSlCesty);

            $cursor = $database ->getSqlResult($sql, $params);

            if (!$row = $cursor ->fetch()) {
                return false;
            }

            $cursor ->close();

            $result = new SluzebniCestaCalItem();
            $result ->loadRow($database, $row);
            return $result;
        }

        $result = [];

        // sync - seznam vsech budoucich SC uzivatele
        $sql .=
            ' WHERE Osoba.IntrawebLogin = ' . $params->addParameter($login) .
            ' AND SluzebniCesta.DatumCasDo >= ' . $params ->addParameter($database ->dateToParam(time()));

        $cursor = $database ->getSqlResult($sql, $params);

        while ($row = $cursor ->fetch()) {
            $cal = new SluzebniCestaCalItem();
            $cal ->loadRow($database, $row);
            $result[] = $cal;
        }

        $cursor ->close();

        return $result;
    }
}
