<?php
/**
 * stanislav.elbl, 23.02.2021
 */

namespace RonPortal\System;

use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\Parameters;
use RonPortal\Prava;

class Report
{
    const FORMATY = ['PDF', 'XLS', 'ODS', 'PDFA', 'XLSX'];

    /** @var string */
    public $kod;
    /** @var string */
    public $nazev;
    /** @var string */
    public $prava;
    /** @var int        Bitova maska 1=PDF, 2=XLS, 4=ODS, 8=PDFA, 16=XLSX */
    public $formaty;
    /** @var string[]   Pole formatu - naplni se podle bitove masky */
    public $nazvyFormatu;
    /** @var string     Formular, ve kterem se ma sestava zobrazit */
    public $formular;

    /** @var string     Rozlišení v seznamu tiskových sestav */
    public $kategorie;

    /** @var string */
    public $applicationId;
    /** @var int */
    public $managerNumber;
    /** @var string     Jmeno sestavy z manazera sestav */
    public $appReportName;
    /** @var string     Jmeno kategorie v manazeru sestav */
    public $appCategoryName;


    // zobrazene panely - doplni se dodatecne podle informaci z reportserveru
    /** @var bool */
    public $zobrazitPanelObdobi;
    /** @var bool */
    public $zobrazitPanelRozdeleni;
    /** @var bool */
    public $zobrazitPanelRozdeleni2;
    /** @var bool */
    public $zobrazitPanelOsoba;
    /** @var bool */
    public $zobrazitPanelKalendar;
    /** @var bool */
    public $zobrazitPanelRazeni;

    /** @var string[]   Asociativni pole sloupcu pro razeni (nazev sloupce => titulek) */
    public $orderFieldsList = [];


    public function setFormaty($formaty)
    {
        $this ->formaty = $formaty;

        $this ->nazvyFormatu = [];
        $bit = 0x01;
        foreach (self::FORMATY as $format) {
            if ($this ->formaty & $bit) {
                $this ->nazvyFormatu[] = $format;
            }
            $bit <<= 1;
        }
    }


    /**
     * @param Database $database
     * @param string $kod
     * @return Report
     * @throws \Exception
     * @throws DBException
     */
    public static function loadReport($database, $kod)
    {
        $params = $database ->createParameters();
        $cursor = $database ->getSqlResult(
            'SELECT SestavyProIntraweb.Kod, SestavyProIntraweb.Nazev, SestavyProIntraweb.Prava,' . // 0..2
            ' SestavyProIntraweb.Formaty, SestavyProIntraweb.Formular, SestavyProIntraweb.Kategorie,' . // 3..5
            ' SYS_RMRE.applicationId, SYS_RMRE.managerNumber, SYS_RMRE.ReportName, SYS_RMRE.CategoryName' . // 6..9
            ' FROM SestavyProIntraweb LEFT OUTER JOIN SYS_RMRE' .
            ' ON SestavyProIntraweb.applicationId = SYS_RMRE.applicationId' .
            '  AND SestavyProIntraweb.managerNumber = SYS_RMRE.managerNumber' .
            '  AND SestavyProIntraweb.ReportName = SYS_RMRE.ReportName' .
            ' WHERE Kod = ' . $params ->addParameter($kod), $params);
        $row = $cursor ->fetch();
        $cursor ->close();

        if (!$row) {
            throw new \Exception('Chyba pri nacitani reportu - sestava se zadanym kodem nebyla nalezena!');
        }

        $result = new Report();
        $result ->kod = $row[0];
        $result ->nazev = $row[1];
        $result ->prava = $row[2];
        $result ->setFormaty($row[3]);
        $result ->formular = $row[4];
        $result ->kategorie = $row[5];
        $result ->applicationId = $row[6];
        $result ->managerNumber = $row[7];
        $result ->appReportName = $row[8];
        $result ->appCategoryName = $row[9];

        return $result;
    }


    /**
     * @param Database $database
     * @param string $sqlFilter
     * @param Parameters $sqlParams
     * @return Report[]
     * @throws DBException
     */
    public static function loadSQL($database, $sqlFilter, $sqlParams = null)
    {
        $sql =
            'SELECT Kod, Nazev, Formaty, ReportName, Kategorie, Prava FROM SestavyProIntraweb ' .
            ' WHERE Povolit = 1' .
            ($sqlFilter ? ' AND (' . $sqlFilter . ')' : '') .
            ' ORDER BY Kategorie, Kod';

        $cursor = $database->getSqlResult($sql, $sqlParams);

        $result = [];

        while ($row = $cursor->fetch()) {
            if (!$row[5] || Prava::jsouNejakePrava($row[5])) {
                $report = new static();
                $report->kod = $row[0];
                $report->nazev = $row[1];
                $report->setFormaty($row[2]);
                $report->appReportName = $row[3];

                $report->kategorie = $row[4];
                $report->prava = $row[5];


                $result[] = $report;
            }
        }

        return $result;
    }


    /**
     * Vraci seznam reportu pro zadany formular (tj. reporty, jejich kod zacina zadanym jmenem formulare)
     *
     * @param Database $database
     * @param string|string[] $formName
     * @param @deprecated string $deprecated_NameStart   NEPOUZIVAT - JEN PRO KOMPATIBILITU!
     * @return Report[]
     * @throws DBException
     */
    public static function loadFormReports($database, $formName, $deprecated_NameStart = null)
    {
        $params = $database ->createParameters();

        $sqlFilter = '';

        if (is_array($formName)) {
            foreach ($formName as $name) {
                if ($sqlFilter) {
                    $sqlFilter .= ' OR ';
                }
                $sqlFilter .= 'formular = ' . $params ->addParameter($name);
            }
        } else {
            $sqlFilter = 'formular = ' . $params->addParameter($formName);
        }

        if ($deprecated_NameStart) {
            $sqlFilter .= ($sqlFilter ? ' OR ' : '') . 'Kod LIKE ' . $params->addParameter($deprecated_NameStart . '%');
        }

        return self::loadSQL($database, $sqlFilter, $params);
    }

    /**
     * Vraci seznam reportu pro zadany formular (tj. reporty, jejich kod zacina zadanym jmenem formulare)
     * @param Database $database
     * @param string $reportNameStart
     * @return Report[]
     * @deprecated
     */
    public static function loadFormReportsByName($database, $reportNameStart)
    {
        $params = $database ->createParameters();
        return self::loadSQL($database, 'Kod LIKE ' . $params->addParameter($reportNameStart . '%'), $params);
    }
}
