<?php
/**
 * stanislav.elbl, 23.02.2021
 */

namespace RonPortal\System;

use RonPortal\RonPortalSession;

class TaskManager
{
    private static $instance;
    private static $lastId = 0;

    /** @var TaskManagerTask[] */
    public $tasks;

    private function __construct()
    {
        $this ->tasks = [];
    }

    public static function getInststance()
    {
        if (!self::$instance) {
            self::$instance = new static();
            self::$instance ->loadFromSession();
        }

        return self::$instance;
    }


    private function storeToSession()
    {
        $storeArray = [
            'lastId' => self::$lastId
        ];

        foreach ($this ->tasks as $task) {
            $storeArray[$task ->id] = $task ->storeToArray();
        }

        $_SESSION['ronportal.taskManager'] = $storeArray;

        return $this;
    }


    private function loadFromSession()
    {
        $array = RonPortalSession::getValue('ronportal.taskManager');

        if ($array) {
            foreach ($array as $key => $taskInfo) {
                if ($key == 'lastId') {
                    self::$lastId = $taskInfo;
                } else {
                    $task = TaskManagerTask::createFromArray($taskInfo, $key);
                    $this->tasks[$key] = $task;
                }
            }
        }

        return $this;
    }


    /**
     * @param TaskManagerTask $task
     * @return $this
     */
    public function addTask($task)
    {
        self::$lastId++;
        $task ->id = sprintf('%x-%04x', time(), self::$lastId);

        $this ->tasks = [$task ->id => $task] + $this->tasks;
        $this ->storeToSession();

        return $this;
    }


    /**
     * @param TaskManagerTask $task
     */
    public function updateTask(/** @noinspection PhpUnusedParameterInspection */$task)
    {
        if ($task  &&  $task ->isRunning) {
            $manager = new AgentManager();
            $manager->getTaskStatus($task);

            $task ->executorParams['updated'] = time();
        }

        $this ->storeToSession();
    }


    public function findTask($taskId)
    {
        return $this ->tasks[$taskId];
    }
}
