<?php

/**
 * Pavel Rudol, RON Software
 *  10.2.2015
 */

namespace RonPortal\Terminaly;

use Exception;
use Ron\DB\Database;
use Ron\DB\DBUtils;
use function Ron\ronText;



/**
 * Objekt s detaily terminalu
 */
class Terminal
{
    public $kod;
    public $popis;
    public $poznamka;
    
    public $urceni;
    public $umisteni;
    public $rozdeleni;
    public $typrozdeleni;
    
    public $driver;
    public $typterminalu;
    public $typkomunikacekod;
    public $typkomunikacepopis;
    public $hostitel;
    public $port;
    public $comport;
    public $parita;
    public $baudrate;
    
    public $stav;
    
    public $casuspesnehonacteni;
    public $casuspesnehonastaveni;
    public $casuspesnehospojeni;
    public $pocetneuspesnychnacteni;
    public $pocetneuspesnychnastaveni;
    public $pocetneuspesnychspojeni;
    
    public $verzefirmware;
    public $seriovecislo;
    public $macadresa;


    public function __construct($kodTerminalu)
    {
        $this->kod = $kodTerminalu;
    }

    /**
     * @param Database $database
     * @throws Exception
     */
    public function nactiTerminal($database)
    {
        $row = $this ->nactiDetailTerminalu($database);

        $this->popis = $row[1];
        $this->poznamka = $row[7];
        
        $this->urceni = $row[3];
        $this->umisteni = $row[5];
        $this->rozdeleni = $row[6];
        $this->typrozdeleni = $row[8];
        
        $this->driver = $row[9];
        $this->typterminalu = $row[10];
        $this->typkomunikacekod = (int)$row[11];
        $this->typkomunikacepopis = DBUtils::getSqlValue($database, 'SELECT Popis FROM S_CommType WHERE Kod = ' . (int)$row[11] );
        $this->hostitel = $row[12];
        $this->port = $row[13];
        
        $this->comport = $row[14];
        $this->baudrate = DBUtils::getSqlValue($database, 'SELECT Popis FROM S_BaudRate WHERE Kod = ' . (int)$row[15] );
        $this->parita = DBUtils::getSqlValue($database, 'SELECT Popis FROM S_Parity WHERE Kod = ' . (int)$row[16] );
    }

    /**
     * @param Database $database
     * @return bool
     */
    public function nactiStavTerminalu($database)
    {
        $rowStav = $this ->nactiStavTerminaluDB($database);
        if (!$rowStav) {
            $this->stav = false;
            return false;
        }
        
        $this->casuspesnehonacteni = $database ->stringToTimestamp($rowStav[1]);
        $this->casuspesnehonastaveni = $database ->stringToTimestamp($rowStav[2]);
        $this->casuspesnehospojeni = $database ->stringToTimestamp($rowStav[3]);
        
        $this->pocetneuspesnychnacteni = (int)$rowStav[4];
        $this->pocetneuspesnychnastaveni = (int)$rowStav[5];
        $this->pocetneuspesnychspojeni = (int)$rowStav[6];
        
        $this->verzefirmware = $rowStav[7];
        $this->seriovecislo = $rowStav[8];
        $this->macadresa = $rowStav[9];

        $this->stav = true;

        return true;
    }

    /**
     * @param Database $database
     * @return array|bool
     * @throws Exception
     */
    private function nactiDetailTerminalu($database)
    {
        if (!$this->kod) {
            return false;
        }

        $params = $database->createParameters();

        $sql =
            'SELECT Terminal.Kod, Terminal.Popis, Terminal.KodUrceni, S_UrceniTerminalu.Popis, '. // 0 - 3            
            'Terminal.KodUmisteni, Umisteni.Popis, Rozdeleni.Popis, ' . $database->castWideMemo('Terminal.Poznamka') . ', '. // 4 - 7
            'TypRozdeleni.Popis, Terminal.Driver, Terminal.C_TerminalType, Terminal.C_CommType, '. //8 - 11
            'Terminal.C_Host, Terminal.C_Port, Terminal.C_ComPort, Terminal.C_BaudRate, Terminal.C_Parity '. //12 - 16
                
            ' FROM Terminal JOIN S_UrceniTerminalu ON (S_UrceniTerminalu.Kod = Terminal.KodUrceni) '.
                'LEFT JOIN Umisteni ON (Umisteni.Kod = Terminal.KodUmisteni) '.
                'LEFT JOIN Rozdeleni ON (Rozdeleni.KodTypuRozdeleni = Terminal.KodTypuRozdeleni AND Rozdeleni.Kod = Terminal.KodRozdeleni) ' .
                'LEFT JOIN TypRozdeleni ON (TypRozdeleni.Kod = Terminal.KodTypuRozdeleni) '.
            " WHERE Terminal.kod = " . $params->addParameter($this->kod);

        $cursor = $database->select($sql, $params);
        $row = $cursor ->fetch();
        $cursor ->close();

        if (!$row) {
            throw new Exception('Terminal se zadanym kodem neexistuje!');
        }

        return $row;
    }

    /**
     * @param Database $database
     * @return array|bool
     */
    private function nactiStavTerminaluDB($database)
    {
        if (!$this->kod) {
            return false;
        }

        $params = $database->createParameters();

        $sql =
            'SELECT  KodTerminalu, CasPoslednihoNacteni, CasPoslednihoNastaveni, CasPoslednihoTestuSpojeni,'.   //0 - 3
            ' NeuspesnychNacteni, NeuspesnychNastaveni, NeuspesnychTestuSpojeni, INF_VerzeFirmware, INF_SerioveCislo,'. //4 - 8
            ' INF_MAC_Adresa, INF_PocetOperaci, INF_PocetOsob, INF_PocetSalda, INF_PocetDatovychZaznamu'.       //9 - 13
            ' FROM StavTerminalu'.
            ' WHERE KodTerminalu = ' . $params->addParameter($this->kod);

        $cursor = $database->select($sql, $params);
        $row = $cursor ->fetch();
        $cursor ->close();

        return $row;
    }
}
