<?php

/**
 * Pavel Rudol, RON Software
 *  10.2.2015
 */

namespace RonPortal\Terminaly;

use Exception;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\MVC\BaseViewObject;
use Ron\MVC\HttpController;
use Ron\MVC\InternalRedirect;
use Ron\MVC\JsonOutputObject;
use Ron\MVC\Model;
use Ron\MVC\OutputObject;
use Ron\MVC\Redirect;
use Ron\MVC\TextViewObject;
use Ron\MVC\TranslationsOutputObject;
use Ron\MVC\ViewObject;
use Ron\StringUtils;
use Ron\TextManager;
use Ron\Http\HtmlHeaders;
use Ron\MessageManager;
use function Ron\ronText;
use RonPortal\NeniPravoException;
use RonPortal\Prava;
use RonPortal\PravaNaOsoby;
use RonPortal\System\AgentManager;


class TerminalyController extends HttpController
{
    /** @var $database Database */
    private $database;
    
    public function __construct($database) 
    {
        $this->database = $database;
        
        HtmlHeaders::getInstance()->addScript('script/terminaly.js');
        TextManager::getInstance() ->loadStrings('terminaly.ini');
        $this ->addTranslationsScript('terminaly');
    }

    /**
     *
     * @param PravaNaOsoby $prava pravo na zobrazeni terminalu
     * @param string $terminal kod terminalu
     * @return true pokud ma uzivatel pravo na zvoleny terminal
     * @throws DBException
     * @throws Exception
     */
    function jsouPravaNaTerminal($prava, $terminal)
    {
        $params = $this ->database ->createParameters();
        $sql =
            'SELECT Terminal.KodTypuRozdeleni, Terminal.KodRozdeleni FROM Terminal' .
            ' WHERE Kod = ' . $params ->addParameter($terminal);

        $cursor = $this->database->select($sql, $params);
        $row = $cursor->fetch();
        $cursor ->close();

        return $row  &&  $prava ->jePravoNaRozdeleni($row[0], $row[1]);
    }

    /**
     * @param string $ukodTerminal kod terminalu
     * @return bool|ViewObject
     * @throws DBException
     * @throws Exception
     */
    function detailTerminalu($ukodTerminal)
    {
        $d = new Terminal($ukodTerminal);
        $d->nactiTerminal($this ->database);
        $d->nactiStavTerminalu($this ->database);

        $kodAplikace = $this ->zjistiAplikaci();

        Model::addData('terminal',$d);
        Model::addData('kodAplikace', $kodAplikace);
        Model::addViewObject(new BaseViewObject('ronportal/terminaly/view_detail_terminalu.php'));

        if ('skupiny' == ($_GET['action'] ?? 'detail')) {
            $bSkupiny = new SkupinyTerminaluBrowser($this->database, $ukodTerminal);
            $bSkupiny->load();

            return $bSkupiny;
        }


        if ('osoby' == ($_GET['action'] ?? 'detail')) {
            $b = new OsobyTerminaluBrowser($this->database, $ukodTerminal);
            $b->load();

            return $b;
        }

        return true;
    }

    /**
     * Spusti agenta terminalu
     * @param string $kodTerminalu kod terminalu
     * @param boolean $lzeGenerovat je mozne (je pravo) provest generovani
     * @return JsonOutputObject|Redirect|ViewObject|string
     * @throws DBException
     * @throws Exception
     */
    function provedAkciTerminalu($kodTerminalu, $lzeGenerovat)
    {
        if (isset($_POST['terminal_kod']))
        {   
            //$_SESSION['terminalUspesnyTest'] = false;
            
            $d = new Terminal($kodTerminalu);
            if (isset($_POST['test'])) {
                if ($this->spustAgentaTerminalu('test', $d ->kod)) {
                    MessageManager::addSuccessMessage(ronText('UspesnyTestTerminalu'));
                }

                return new Redirect(null,'?menuid=terminaly&record='.$kodTerminalu);
            }
            elseif (isset($_POST['nastaveni']) && $lzeGenerovat) {
                return $this->provedGenerovaniTerminalu($d);
            }
        }

        if (isset($_GET['kontrolastavu'])) {
            return $this->vratStavTerminalu($kodTerminalu);
        }

        return $this->detailTerminalu($kodTerminalu);
    }

    /**
     * Spusti agenta pro generovani, pri pozadavku na JSON vrati i aktualni stav terminalu.
     * Jinak pri uspesnem generovani provede redirect na detail terminalu.
     * @param Terminal $terminal vybrany terminal
     * @return JsonOutputObject|string
     * @throws DBException
     * @throws Exception
     */
    function provedGenerovaniTerminalu($terminal)
    {
        $vysledekAgenta = $this->spustAgentaTerminalu('nastaveni', $terminal->kod);
        if ($vysledekAgenta) {
            if (isset($_POST['json'])) {
                return $this->vratVysledeGenerovaniJSON($terminal, $vysledekAgenta);
            }   
            else {
                return new Redirect(null,'?menuid=terminaly&record='.$terminal->kod);
            }
        }
        else {
            if (isset($_POST['json'])) {
                return $this->vratVysledeGenerovaniJSON($terminal, $vysledekAgenta);
            }   
            else {
                return $this->detailTerminalu($terminal->kod);
            }
        }
    }

    /**
     * @param Terminal $terminal
     * @param bool $vysledekAgenta
     * @return JsonOutputObject
     * @throws Exception
     */
    function vratVysledeGenerovaniJSON($terminal, $vysledekAgenta)
    {
        $terminal->nactiStavTerminalu($this ->database);

        $json = new JsonOutputObject(['posledniNastaveni' => $terminal->casuspesnehonastaveni, 'posledniNacteni' => $terminal->casuspesnehonacteni,
            'posledniNastaveniStr' => StringUtils::jsonText(DateUtils::fineDateString($terminal->casuspesnehonastaveni, TRUE)),
            'posledniNacteniStr' => StringUtils::jsonText(DateUtils::fineDateString($terminal->casuspesnehonacteni, TRUE)),
            'pocetNastaveni' => $terminal->pocetneuspesnychnastaveni, 'pocetNacteni' => $terminal->pocetneuspesnychnacteni,
            'vysledekAgenta' => $vysledekAgenta,
            'result' => JsonOutputObject::RESULT_OK]);
        $json->addMessages(MessageManager::getInstance());
        return $json;
    }

    /**
     * Vraci JSON objekt s aktualnim stavem terminalu nactenym z databaze.
     * @param string $terminal kod terminalu
     * @return JsonOutputObject
     * @throws Exception
     */
    function vratStavTerminalu($terminal)
    {
        $t = new Terminal($terminal);
        $t->nactiStavTerminalu($this ->database);
        
        return new JsonOutputObject([
            'posledniNastaveni' => $t->casuspesnehonastaveni,
            'posledniNacteni' => $t->casuspesnehonacteni,
            'posledniNastaveniStr' => StringUtils::jsonText(DateUtils::fineDateString($t->casuspesnehonastaveni, TRUE)),
            'posledniNacteniStr' => StringUtils::jsonText(DateUtils::fineDateString($t->casuspesnehonacteni, TRUE)),
            'pocetNastaveni' => $t->pocetneuspesnychnastaveni,
            'pocetNacteni' => $t->pocetneuspesnychnacteni,
            'result' => JsonOutputObject::RESULT_OK]);
    }


    /**
     * Spusti agenta terminalu
     * @param string $action    Typ ulohy agenta terminalu ktery se ma provest
     * @param string $kodTerminalu
     * @return bool
     * @throws Exception
     */
    public function spustAgentaTerminalu($action, $kodTerminalu)
    {
        switch ($action) {
            case 'nastaveni':
                $uloha = 1;
                break;
            case 'test':
                $uloha = 4;
                break;
            default:
                throw new Exception('Chyba v metode TerminalyController:spustAgentaTerminalu - pro hodnotu ' . StringUtils::htmlTextConvert($action) . ' neni definovana uloha agenta.');
        }

        $kodAplikace = $this ->zjistiAplikaci();

        if ($kodAplikace) {
            //return \ExecuteAgent_Internal($this->kodAplikace.'_Agent_Terminal.exe', $uloha.' -terminals:"' . $this->kod . '"');
            try {
                AgentManager::executeADS($kodAplikace . '_Agent_Terminal.exe', $uloha . ' -terminals:"' . $kodTerminalu . '"');
                return true;
            } catch (Exception $e) {
                MessageManager::addErrorMessage($e ->getMessage());
                return false;
            }
        }
        else {
            return FALSE;
        }
    }


    /**
     * @throws Exception
     */
    private function zjistiAplikaci()
    {
        $sql =
            'SELECT applicationID FROM SYS_DDEF' .
            " WHERE applicationID = 'ADS' OR applicationID = 'APS' OR applicationID = 'ZAKAZKY'" .
            ' ORDER BY applicationID';

        $cursor = $this->database->select($sql);
        $row = $cursor ->fetch();
        $cursor ->close();

        return $row ? $row[0] : null;
    }


    private function idMediaBrowser()
    {
        $b = new IdMediaBrowser($this ->database);
        $b ->load();
        return $b;
    }

    /**
     * @param string $action
     * @return InternalRedirect|JsonOutputObject|OutputObject|Redirect|TranslationsOutputObject|ViewObject|TerminalyBrowser|string
     * @throws DBException
     * @throws Exception
     */
    public function handleAction($action)
    {
        Prava::nastavObdobiProPrava(time(), time());

        $action = $_GET['action'] ?? null;

        if ($action === 'translations') {
            return $this->translations();
        }

        $pravaProhlizet = Prava::vratPrava('prohl_terminaly');

        if (!$pravaProhlizet->jsouNejakePrava()) {
            throw new NeniPravoException();
        }

        if ($action === 'media') {
            return $this ->idMediaBrowser();
        }
        
        $terminal = $_GET['record'] ?? null;
        
        HtmlHeaders::getInstance() ->setTitle(TextManager::_('RON_Portal') . ' - ' . TextManager::_('Terminaly'));
        
        //je vybrany konkretni terminal
        if ($terminal) {
            if ($this->jsouPravaNaTerminal($pravaProhlizet, $terminal)) {
                Model::addData('prava', $pravaProhlizet);
                
                return $this->provedAkciTerminalu($terminal, $pravaProhlizet->priznak);
            } else {
                throw new NeniPravoException();
            }
        } else {
            $b = new TerminalyBrowser($this->database);
            $b->load();            
            return $b;            
        }       
    }
}
