<?php
/**
 * Trida reprezentuje jednu otazku v testu, ankete nebo v hodnoceni
 */

namespace RonPortal\Testy;

use \Exception;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\SqlStatement;
use Ron\MessageManager;
use Ron\StringUtils;
use RonPortal\RonPortalDBRecord;
use RonPortal\System\RecordNotFoundException;
use function Ron\ronText;



class OtazkaDotazniku
{
    var $temaCislo; // nadrazeny dotaznik
    var $druhHodnoceni;
    var $f_id; // id dotazniku osoby ("odpovedni" formular) - pokud se zobrazuje jen sablona (nahled), tak neni zadano!

    /** @var string     Kod kriteria */
    var $kod;
    var $poradi;

    var $povinnaOdpoved;

    var $f_popis;
    var $f_upresnujicipopis;
    var $f_popisproweb;

    var $f_typrozsahu;
    var $zpusobHodnoceni; // z TypRozsahuHodnoceni
    /** @var VolbaDotazniku[] */
    var $volby = [];    // mozne volby z RozsahHodnoceni nebo VycetOdpovedi

    // spravna odpoved a zpusob vypoctu bodu
    var $spravnaHodnota;
    var $spravnaSlovniOdpoved;
    var $vycetSpravnychOdpovedi = []; // pro otazky s vice spravnymi volbami
    var $hodnoceniSpravneOdpovedi;
    var $hodnoceniSpatneOdpovedi;
    var $povolitZaporneHodnoceni;

    var $zpusobZobrazeni; // 0 = pod sebou, 1 = vedle sebe
    var $barvaPozadi;
    var $sirkaSloupce;

    // vlastni otazka
    var $definovatVlastniOtazku;    //uzivatel si muze definovat vlastni otazku (pomoci mema - nebere se v uvahu TypRozsahuHodnoceni)
    var $zobrazitVlastniOtazku;     //misto otazky z ciselniku se zobrazi otazka, kterou si definoval uzivatel

    /** @var integer */
    var $bodovaVaha;

    var $vlastniOtazka;
    var $vlastniVaha;
    var $vahaStr;  // text pro zobrazeni vahy - pouzije se vychozi bodovaVaha, vlastni vaha, nebo POST

    var $maPodotazky;
    /** @var PodrizenaOtazkaDotazniku[] */
    var $podotazky;

    // odpoved
    var $f_hodnota;
    var $f_slovniodpoved;
    var $f_memoodpoved;
    var $f_datumodpoved;
    /** @var string */
    var $datumStr;
    /** @var  float korekce vysledneho bodoveho hodnoceni */
    var $bodyKlasifikace;
    /** @var  string pro korekci nejnizsi mozna hodnota korekce */
    var $popisNejnizsiHodnoty;
    /** @var  string pro korekci nejvyssi mozna hodnota korekce */
    var $popisNejvyssiHodnoty;

    // vice ruznych typu odpovedi - kazda ma vlastni kod kriteria
    var $viceTypuOdpovedi;
    /** @var OtazkaDotazniku[] */
    var $dalsiOdpovedi;

    var $f_error; // priznak, ze je v odpovedi chyba
    var $odpovedVyplnena; // priznak vyplnene odpovedi

    /** @var int 0 - viditelne editovatelne, 1 - viditelne needitovatelne, 2 - skryte */
    public $zobrazitOtazku;

    /**
     * @param int $id
     * @param int $druhHodnoceni
     * @param int $temaCislo
     */
    function __construct($id, $druhHodnoceni, $temaCislo)
    {
        $this ->temaCislo = $temaCislo; // nadrazeny dotaznik (TemaHodnoceni)
        $this ->druhHodnoceni = $druhHodnoceni;

        $this ->f_id = $id; // nadrazene PripraveneTemaHodnoceni (odpovedi)

        $this ->odpovedVyplnena = false;
    }


    /**
     * Vraci SQL dotaz pro nacteni otazky dotazniku vcetne odpovedi (tabulka Hodnoceni).
     * Pokud je zadan $kodKriteria, tak vysledny dotaz odfiltruje jen jednu zadanou otazku, jinak vraci vsechny otazky dotazniku
     * @param Database $database
     * @param int $idDotazniku
     * @param int $druhHodnoceni
     * @param int $temaCislo
     * @param string $kodKriteria
     * @param bool   $podrizenaOtazka
     * @return SqlStatement
     */
    private static function getSql($database, $idDotazniku, $druhHodnoceni, $temaCislo, $kodKriteria = null, $podrizenaOtazka = false)
    {
        $params = $database->createParameters();

        if ($idDotazniku) {
            // pokud je zadan id tak vyhledava podle PripraveneTemaHodnoceni a nacita i odpoved
            $sql =
                'SELECT HodnoticiKriterium.TypRozsahu, HodnoticiKriterium.Popis, ' .
                $database ->castWideMemo('HodnoticiKriterium.UpresnujiciPopis') . ', ' .  // 2
                $database ->castWideMemo('HodnoticiKriterium.UpresnujiciPopisProWeb') . ', ' .  // 3
                ' HodnoticiKriterium.KodKriteria, TypRozsahuHodnoceni.ZpusobHodnoceni, ' .  // 4, 5
                ' Hodnoceni.Hodnota, Hodnoceni.SlovniOdpoved, ' .          // 6, 7
                ' TypRozsahuHodnoceni.ZpusobZobrazeni, ' .                 // 8
                ' HodnoticiKriterium.BarvaPozadi, HodnoticiKriterium.SirkaSloupce, TemaHodnoceni.SirkaSloupce, HodnoticiKriterium.Poradi, ' .  // 9..12
                ' COALESCE(ParametryKriteria.PovinnaOdpoved, HodnoticiKriterium.PovinnaOdpoved), ' . $database ->castWideMemo('Hodnoceni.MemoOdpoved') . ', Hodnoceni.DatumOdpoved, ' . // 13..15
                ' HodnoticiKriterium.DefinovatVlastniText, ' . $database ->castWideMemo('Hodnoceni.VlastniTextOtazky') . ', Hodnoceni.VlastniBodovaVaha, ' . // 16..18
                ' Hodnoceni.ZobrazitVlastniText, HodnoticiKriterium.BodovaVaha, ' . // 19, 20
                ' HodnoticiKriterium.SpravnaHodnota, HodnoticiKriterium.SpravnaSlovniOdpoved, ' . // 21,22
                ' HodnoticiKriterium.HodnoceniSpravneOdpovedi, HodnoticiKriterium.HodnoceniSpatneOdpovedi, HodnoticiKriterium.PovolitZaporneHodnoceni, ' . // 23..25
                ' (SELECT MAX(1) FROM HodnoticiKriterium HK WHERE HK.DruhHodnoceniHlavni = HodnoticiKriterium.DruhHodnoceni '.            // 26
                '    AND HK.TemaCisloHlavni = HodnoticiKriterium.TemaCislo AND HK.KodKriteriaHlavni = HodnoticiKriterium.KodKriteria), ' . // 26
                ' Hodnoceni.BodyKlasifikace, TypRozsahuHodnoceni.PopisNejnizsiHodnoty, TypRozsahuHodnoceni.PopisNejvyssiHodnoty, ' . // 27..29
                ' COALESCE(ParametryKriteria.Stav, HodnoticiKriterium.Stav) ' . // 30
                '  FROM ( ( ( HodnoticiKriterium INNER JOIN TypRozsahuHodnoceni ON HodnoticiKriterium.TypRozsahu = TypRozsahuHodnoceni.Kod ' .
                '           )' .
                '           INNER JOIN TemaHodnoceni ON TemaHodnoceni.DruhHodnoceni = HodnoticiKriterium.DruhHodnoceni AND TemaHodnoceni.TemaCislo = HodnoticiKriterium.TemaCislo ' .
                '         )' .
                '         INNER JOIN PripraveneTemaHodnoceni ON PripraveneTemaHodnoceni.TemaCislo = TemaHodnoceni.TemaCislo AND PripraveneTemaHodnoceni.DruhHodnoceni = TemaHodnoceni.DruhHodnoceni     ' .
                '         LEFT OUTER JOIN HodnoceniOsoby ON HodnoceniOsoby.ID = PripraveneTemaHodnoceni.ID     ' .
                '         LEFT OUTER JOIN ParametryKriteria ON HodnoticiKriterium.DruhHodnoceni = ParametryKriteria.DruhHodnoceni AND HodnoticiKriterium.TemaCislo = ParametryKriteria.TemaCislo AND HodnoticiKriterium.KodKriteria = ParametryKriteria.KodKriteria ' .
                '                AND ParametryKriteria.KodProcesu = HodnoceniOsoby.KodProcesu AND ParametryKriteria.PocatecniUroven = HodnoceniOsoby.PocatecniUroven AND ParametryKriteria.Poradi = HodnoceniOsoby.PoradiPredavani ' .
                '       )' .
                '       LEFT OUTER JOIN Hodnoceni ' .
                '       ON Hodnoceni.Id = PripraveneTemaHodnoceni.ID ' . // vybere jiz ulozene odpovedi
                '          AND Hodnoceni.DruhHodnoceni = PripraveneTemaHodnoceni.DruhHodnoceni ' .
                '          AND Hodnoceni.TemaCislo = PripraveneTemaHodnoceni.TemaCislo ' .
                '          AND Hodnoceni.KodKriteria = HodnoticiKriterium.KodKriteria ' .
                ' WHERE PripraveneTemaHodnoceni.ID = ' . $params->addParameter($idDotazniku);

        } else {
            // jinak nacita podle druhHodnoceni a temaCislo (jen sablona) a nenacita odpovedi (slouzi pro "nahled")
            $sql =
                'SELECT HodnoticiKriterium.TypRozsahu, HodnoticiKriterium.Popis, ' .
                $database ->castWideMemo('HodnoticiKriterium.UpresnujiciPopis') . ', ' .  // 2
                $database ->castWideMemo('HodnoticiKriterium.UpresnujiciPopisProWeb') . ', ' .  // 3
                ' HodnoticiKriterium.KodKriteria, TypRozsahuHodnoceni.ZpusobHodnoceni, ' .  // 4, 5
                ' NULL AS Hodnota, NULL AS SlovniOdpoved, ' .          // 6, 7
                ' TypRozsahuHodnoceni.ZpusobZobrazeni, ' .                 // 8
                ' HodnoticiKriterium.BarvaPozadi, HodnoticiKriterium.SirkaSloupce, TemaHodnoceni.SirkaSloupce, HodnoticiKriterium.Poradi, ' .  // 9..12
                ' HodnoticiKriterium.PovinnaOdpoved, NULL AS MemoOdpoved, NULL AS DatumOdpoved, ' . // 13..15
                ' HodnoticiKriterium.DefinovatVlastniText, NULL AS VlastniTextOtazky, NULL AS VlastniBodovaVaha, ' . // 16..18
                ' NULL AS ZobrazitVlastniText, HodnoticiKriterium.BodovaVaha, ' . // 19, 20
                ' HodnoticiKriterium.SpravnaHodnota, HodnoticiKriterium.SpravnaSlovniOdpoved, ' . // 21,22
                ' HodnoticiKriterium.HodnoceniSpravneOdpovedi, HodnoticiKriterium.HodnoceniSpatneOdpovedi, HodnoticiKriterium.PovolitZaporneHodnoceni, ' . // 23..25

                ' (SELECT MAX(1) FROM HodnoticiKriterium HK WHERE HK.DruhHodnoceniHlavni = HodnoticiKriterium.DruhHodnoceni '.            // 26
                '    AND HK.TemaCisloHlavni = HodnoticiKriterium.TemaCislo AND HK.KodKriteriaHlavni = HodnoticiKriterium.KodKriteria), ' . // 26
                ' 0, TypRozsahuHodnoceni.PopisNejnizsiHodnoty, TypRozsahuHodnoceni.PopisNejvyssiHodnoty, ' . // 27..29
                ' HodnoticiKriterium.Stav ' . // 30
                '  FROM ( HodnoticiKriterium INNER JOIN TypRozsahuHodnoceni ' .
                '         ON HodnoticiKriterium.TypRozsahu = TypRozsahuHodnoceni.Kod ' .
                '       )' .
                '       INNER JOIN TemaHodnoceni ' .
                '       ON TemaHodnoceni.DruhHodnoceni = HodnoticiKriterium.DruhHodnoceni ' .
                '          AND TemaHodnoceni.TemaCislo = HodnoticiKriterium.TemaCislo ' .
                ' WHERE HodnoticiKriterium.DruhHodnoceni = ' . $params->addParameter($druhHodnoceni) .
                '   AND HodnoticiKriterium.TemaCislo = ' . $params->addParameter($temaCislo);

        }

        if (!$podrizenaOtazka) {
            $sql .= '   AND HodnoticiKriterium.KodKriteriaHlavni IS NULL';
        }

        // filtr na jednu konkretni otazku
        if (isset($kodKriteria)) {
            $sql .= " AND HodnoticiKriterium.KodKriteria = " . $params->addParameter($kodKriteria);
        }

        $sql .=
            ' ORDER BY HodnoticiKriterium.Poradi';

        return new SqlStatement($sql, $params);
    }


    /**
     * Nacte seznam moznych odpovedi - z RozsahHodnoceni nebo VycetOdpovedi
     * @param Database $database
     * @param bool $vzdyJenVlastni Priznak, ze ma nacist jen vlastni volby (pouziva se pri editaci testu)
     * @throws DBException
     */
    private function nactiVolby($database, $vzdyJenVlastni)
    {
        $zpusobHodnoceni = $vzdyJenVlastni ? TypOdpovedi::ZPUSOB_ODPOVEDI_VYBER1 : $this ->zpusobHodnoceni;

        switch ($zpusobHodnoceni) {
            case TypOdpovedi::ZPUSOB_ODPOVEDI_VYBER1 :
            case TypOdpovedi::ZPUSOB_ODPOVEDI_VYBER_VICE :
                $this ->volby = VolbaDotazniku::nactiVolbyKOtazce($database, $this ->druhHodnoceni, $this ->temaCislo, $this ->kod, $this ->f_id);
                break;

            case TypOdpovedi::ZPUSOB_ODPOVEDI_VYBER1_DEF :
            case TypOdpovedi::ZPUSOB_ODPOVEDI_VYBER_VICE_DEF :
            case TypOdpovedi::ZPUSOB_ODPOVEDI_STUPNICE :
                $this ->volby = VolbaDotazniku::nactiPreddefinovaneVolby($database, $this ->f_typrozsahu, $this ->kod, $this ->f_id);
                break;
        }

    }


    /**
     * @param Database $database
     * @param array $row
     * @param bool $nacistJenVlastniOdpovedi Priznak, ze se maji nacist jen vlastni odpovedi k otazce (i kdyz pouziva preddefinovane) - pouziva se pri editaci
     * @throws Exception
     */
    private function nactiRadek($database, $row, $nacistJenVlastniOdpovedi)
    {
        $this ->kod = $row[4];
        $this ->poradi = (int)$row[12];

        $this ->povinnaOdpoved = $row[13];

        $this ->f_popis = $row[1];
        $this ->f_upresnujicipopis = $row[2];
        $this ->f_popisproweb = $row[3];

        $this ->f_typrozsahu = $row[0];
        $this ->zpusobHodnoceni = (int)$row[5];
        $this ->zpusobZobrazeni = (int)$row[8]; // 1 = vedle sebe
        $this ->barvaPozadi = $row[9];

        $this ->sirkaSloupce = $row[10] ? $row[10] : $row[11]; // sirka sloupce primo u otazky, nebo podle hlavicky dotazniku

        $this ->nactiVolby($database, $nacistJenVlastniOdpovedi);

        $this ->definovatVlastniOtazku = $row[16];
        $this ->zobrazitVlastniOtazku = $row[19];
        $this ->vlastniOtazka = $row[17];
        $this ->vlastniVaha = $row[18];
        $this ->bodovaVaha = $row[20];
        $this ->vahaStr = $this ->vlastniVaha ? $this ->vlastniVaha : $this ->bodovaVaha;

        //pokud uzivatel muze definovat vlastni otazku a v popisu pro web jsou pouzity tagy pro definici struktury otazky
        if ($this ->definovatVlastniOtazku && (strpos($this ->f_popisproweb, StringUtils::fromUTF8('«')) !== false)) {
            $this->maPodotazky = true;
            if ($this ->vlastniOtazka) {
                $this ->nactiPodOtazky($this ->vlastniOtazka);
            }
            else {
                $this ->nactiPodOtazky($this ->f_popisproweb);
            }
        }

        $this ->f_hodnota = $row[6];
        $this ->f_slovniodpoved = $row[7];
        $this ->f_memoodpoved = $row[14];
        $this ->f_datumodpoved = $database ->stringToTimestamp($row[15]);
        $this ->datumStr = $this ->f_datumodpoved ? date('j.n.Y', $this ->f_datumodpoved) : '';

        $this ->spravnaHodnota = $row[21];
        $this ->spravnaSlovniOdpoved = $row[22];
        $this ->hodnoceniSpravneOdpovedi = $row[23];
        $this ->hodnoceniSpatneOdpovedi = $row[24];
        $this ->povolitZaporneHodnoceni = $row[25];

        $this ->viceTypuOdpovedi = (int)$row[26];
        if ($this ->viceTypuOdpovedi) {
            $this ->nactiDalsiOdpovedi($database);
        }

        $this->bodyKlasifikace = $row[27];
        $this->popisNejnizsiHodnoty = $row[28];
        $this->popisNejvyssiHodnoty = $row[29];

        $this->zobrazitOtazku = (int)$row[30];

        $this ->nastavPriznakVyplnena();
    }


    /**
     * @throws Exception
     */
    private function nastavPriznakVyplnena()
    {
        if ($this->definovatVlastniOtazku && !$this->zobrazitVlastniOtazku) {
            // vlastni definice otazky
            if ($this->maPodotazky) {
                //uzivatel muze otazku slozit z vice podotazek, zobrazi se pole pro zadani otazek dle preddefinovaneho retezce v Popisu pro web u otazky
                $this ->odpovedVyplnena = true;

                foreach ($this->podotazky as $podotazka) {
                    $this ->odpovedVyplnena &= $podotazka ->odpovedVyplnena;
                }
            } else {
                $this ->odpovedVyplnena = $this ->f_memoodpoved != '';
            }
        } else {
            switch ($this->zpusobHodnoceni) {
                case TypOdpovedi::ZPUSOB_ODPOVEDI_VYBER1_DEF :  // vyber 1 moznosti
                case TypOdpovedi::ZPUSOB_ODPOVEDI_STUPNICE :    // stupnice
                case TypOdpovedi::ZPUSOB_ODPOVEDI_VYBER1 :      // vyber koeficientu (jedna moznost z tabulky VycetOdpovedi)
                    $this->odpovedVyplnena = isset($this->f_hodnota);
                    break;

                case TypOdpovedi::ZPUSOB_ODPOVEDI_VYBER_VICE_DEF :  // vyber vice moznosti
                case TypOdpovedi::ZPUSOB_ODPOVEDI_VYBER_VICE :      // vyber vice koeficientu
                    $this->odpovedVyplnena = true; // povoleno vice voleb (nebo zadna) => otazka je z principu vyplnena vzdy
                    break;

                case TypOdpovedi::ZPUSOB_ODPOVEDI_VLASTNI : // vlastni odpoved
                    $this->odpovedVyplnena = $this->f_slovniodpoved != '';
                    break;

                case TypOdpovedi::ZPUSOB_ODPOVEDI_VLASTNI_MEMO : // vlastni odpoved - memo
                    $this->odpovedVyplnena = $this->f_memoodpoved != '';
                    break;

                case TypOdpovedi::ZPUSOB_ODPOVEDI_DATUM : // odpoved - datum
                    $this->odpovedVyplnena = !$this->f_error  &&  $this ->f_datumodpoved != null;
                    break;

                case TypOdpovedi::ZPUSOB_ODPOVEDI_HODNOTA : // integer
                    $this->odpovedVyplnena = isset($this->f_hodnota);
                    break;

                case TypOdpovedi::ZPUSOB_ODPOVEDI_KOREKCE : // integer + memo
                    $this->povinnaOdpoved = false; // korekce neni nikdy povinna - nemusi byt vyplnena
                    $this->odpovedVyplnena = isset($this->bodyKlasifikace);
                    break;

                default :
                    throw new Exception('Chyba v metode OtazkaDotazniku::nastavPriznakVyplnena - neznamy zpusob odpovedi! ' . $this ->zpusobHodnoceni );
            }
        }
    }


    /**
     * @param Database $database
     * @param int $idDotazniku
     * @param int $druhHodnoceni
     * @param int $temaCislo
     * @param bool $nacistJenVlastniOdpovedi
     * @return OtazkaDotazniku[]
     * @throws DBException
     */
    public static function nactiOtazky($database, $idDotazniku, $druhHodnoceni, $temaCislo, $nacistJenVlastniOdpovedi)
    {
        $sql = self::getSql($database, $idDotazniku, $druhHodnoceni, $temaCislo);
        $cursor = $database ->getSqlResult($sql);

        $result = [];

        while ($row = $cursor ->fetch()) {
            $otazka = new OtazkaDotazniku($idDotazniku, $druhHodnoceni,  $temaCislo);
            $otazka ->nactiRadek($database, $row, $nacistJenVlastniOdpovedi);

            $result [] = $otazka;
        }

        return $result;
    }


    /**
     * @param Database $database
     * @param string $kodKriteria
     * @param bool $podrizena
     * @throws DBException|RecordNotFoundException
     */
    function nactiOtazku($database, $kodKriteria, $podrizena = false)
    {
        $sql = OtazkaDotazniku::getSql($database, $this ->f_id, $this ->druhHodnoceni, $this ->temaCislo, $kodKriteria, $podrizena);

        $cursor = $database ->getSqlResult($sql);

        $row = $cursor ->fetch();
        if (!$row) {
            throw new RecordNotFoundException('Chyba - otazka s kodem ' . $kodKriteria . ' nebyla nalezena!');
        }

        $this ->nactiRadek($database, $row, false); // jen vlastni odpovedi = false => podle zpusobu odpovedi nacita vlastni nebo preddefinovane volby
    }


    /**
     * @param Database $database
     * @throws DBException
     */
    function nactiSpravneOdpovedi($database)
    {
        $this ->vycetSpravnychOdpovedi = [];

        $params = $database ->createParameters();
        $sql =
            'SELECT Hodnota FROM VycetSpravnychOdpovedi' .
            ' WHERE DruhHodnoceni = ' . $params ->addParameter((int)$this ->druhHodnoceni) .
            ' AND TemaCislo = ' . $params ->addParameter((int)$this ->temaCislo) .
            ' AND KodKriteria = ' . $params->addParameter($this ->kod);

        $cursor = $database ->getSqlResult($sql, $params);

        while ($row = $cursor ->fetch()) {
            $this ->vycetSpravnychOdpovedi[] = $row[0];
        }
        $cursor ->close();
    }


    function nactiPodOtazky($text)
    {
        /** @var PodrizenaOtazkaDotazniku[] */
        $this->podotazky = [];

        $startPos = StringUtils::strpos($text, StringUtils::fromUTF8('«'));
        $i = 0;

        while  ($startPos !== false) {
            $nadpis = '';
            $cssid = '';
            $maxznaku = 160;

            $i++;

            $endPos = StringUtils::strpos($text, StringUtils::fromUTF8('»'), $startPos);

            $aktualniOtazka = StringUtils::substr($text, $startPos+1, $endPos  - ($startPos + 1));

            $pole = explode(StringUtils::fromUTF8('|'), $aktualniOtazka);

            $typotazky = $pole[0];

            switch ($typotazky)
            {
                case 'LONGTEXT' :
                case 'DATUM' :
                case 'CISLO' :
                    $nadpis = $pole[1];
                    if (isset($pole[2])) {
                        $cssid = $pole[2];
                    }
                    break;
                case 'TEXT' :
                    if (is_numeric($pole[1])) {
                        $maxznaku = (int)$pole[1];
                    }
                    else {
                        $nadpis = $pole[1];
                    }


                    if ($maxznaku > 160) {
                        $maxznaku = 160;
                    }
                    if ($nadpis == '') {
                        $nadpis = $pole[2];

                        if (isset($pole[3])) {
                            $cssid = $pole[3];
                        }
                    }
                    elseif (isset($pole[2])) {
                        $cssid = $pole[2];
                    }

                    break;

            }

            $startPos = StringUtils::strpos($text, StringUtils::fromUTF8('«'), $startPos+1);

            if ($startPos !== false) {
                $odpoved = StringUtils::substr($text, $endPos + 1, $startPos - ($endPos + 1));
            }
            else {
                $odpoved = StringUtils::substr($text, $endPos + 1);
            }

            $podotazka = new PodrizenaOtazkaDotazniku($this->kod . StringUtils::fromUTF8('#') . $i, StringUtils::fromUTF8('«') . $aktualniOtazka . StringUtils::fromUTF8('»'), $typotazky, $nadpis, $odpoved, $cssid, $maxznaku);

            $this->podotazky[] = $podotazka;
        }

    }


    /**
     * @param Database $database
     * @return bool
     * @throws Exception
     * @throws DBException
     */
    function nactiDalsiOdpovedi($database)
    {
        $this ->dalsiOdpovedi = [];

        $params = $database->createParameters();

        $sql =
            'SELECT DruhHodnoceni, TemaCislo, KodKriteria FROM HodnoticiKriterium ' .
            ' WHERE HodnoticiKriterium.DruhHodnoceniHlavni = ' . $params->addParameter($this ->druhHodnoceni) .
            '   AND HodnoticiKriterium.TemaCisloHlavni = ' . $params->addParameter($this ->temaCislo) .
            '   AND HodnoticiKriterium.KodKriteriaHlavni = ' . $params->addParameter($this ->kod) .
            ' ORDER BY Poradi';

        $cursor = $database ->getSqlResult($sql, $params);

        while ($row = $cursor ->fetch()) {
            $dalsiodpoved = new OtazkaDotazniku($this ->f_id, $this ->druhHodnoceni, $this ->temaCislo);
            $dalsiodpoved ->nactiOtazku($database, $row[2], true);
            $this ->dalsiOdpovedi[] = $dalsiodpoved;
        }

        return true;
    }


    function getJson()
    {
        $objOtazka = [
            'kodKriteria' => StringUtils::jsonText($this->kod),
            'poradi' => $this->poradi,
            'typRozsahu' => StringUtils::jsonText($this->f_typrozsahu),
            'zpusobHodnoceni' => (int)$this->zpusobHodnoceni,
            'popis' => StringUtils::jsonText($this->f_popis),
            'upresnujiciPopis' => StringUtils::jsonText($this->f_upresnujicipopis),
            'upresnujiciPopisProWeb' => StringUtils::jsonText($this->f_popisproweb),
            'bodovaVaha' => $this ->bodovaVaha,
            'povinnaOdpoved' => $this ->povinnaOdpoved ? 1 : 0,
            'volby' => [],
            'spravnaHodnota' => $this ->spravnaHodnota,
            'spravnaSlovniOdpoved' => StringUtils::jsonText($this ->spravnaSlovniOdpoved),
            'vycetSpravnychOdpovedi' => $this ->vycetSpravnychOdpovedi
        ];

        foreach ($this ->volby as $volba) {
            $objOtazka['volby'][] = $volba ->getJson();
        }

        return $objOtazka;
    }


    private function loadPostVolby()
    {
        foreach ($this ->volby as $odpoved) {
            // povoleno vice hodnot - zobrazuji se checkboxy
            $elementName = $this->kod . '#' . $odpoved->hodnota;
            $odpoved->vybrano = isset($_POST[$elementName])  &&  $_POST[$elementName];
        }
    }


    /**
     * Nacte hodnoty z POST. V pripade chyby vraci false.
     * @return bool
     * @throws Exception
     */
    function loadPost()
    {
        $this ->f_error = false;
        $this ->odpovedVyplnena = false;
        // pokud otazka nema vyplnenou odpoved, tak se dale priznak nastavi na false

        //pokud uzivatel muze definovat vlastni otazku a hodnoceni na urovni, kde na ni odpovida, nacte odpovedi na podotazky
        if ($this->definovatVlastniOtazku && !$this->zobrazitVlastniOtazku) {
            $result = true;

            $this ->vahaStr = trim($_POST[$this ->kod . '_vaha']);
            if ( !StringUtils::isInteger($this ->vahaStr) ) {
                $result = false;
                MessageManager::addErrorMessage( ronText('Bad_number_format') );
            }

            if ($this->maPodotazky) {
                //uzivatel muze otazku slozit z vice podotazek, zobrazi se pole pro zadani otazek dle preddefinovaneho retezce v Popisu pro web u otazky
                $this ->odpovedVyplnena = true;

                foreach ($this->podotazky as $podotazka) {
                    $result &= $podotazka->loadPost();
                    $this ->odpovedVyplnena &= $podotazka ->odpovedVyplnena;
                }
            } else {
                //uzivatel definuje jen jednu otazku (memo)
                $this ->f_memoodpoved = $_POST[$this ->kod];
                $this ->odpovedVyplnena = $this ->f_memoodpoved != '';
            }
            return $result;
        }

        switch ($this ->zpusobHodnoceni)
        {
            case TypOdpovedi::ZPUSOB_ODPOVEDI_VYBER1_DEF :  // vyber 1 moznosti
            case TypOdpovedi::ZPUSOB_ODPOVEDI_STUPNICE :    // stupnice
            case TypOdpovedi::ZPUSOB_ODPOVEDI_VYBER1 :      // vyber koeficientu (jedna moznost z tabulky VycetOdpovedi)
                $this ->f_hodnota = $_POST[$this->kod] ?? null;
                $this ->odpovedVyplnena = isset($this ->f_hodnota);

                break;
            case TypOdpovedi::ZPUSOB_ODPOVEDI_HODNOTA :
                $this ->f_hodnota = $_POST[$this->kod] ?? null;
                $this ->odpovedVyplnena = isset($this ->f_hodnota);

                $this ->f_hodnota = $_POST[$this->kod] ?? null;

                $mimoRozsah = false;
                if (StringUtils::isInteger($this->popisNejnizsiHodnoty)) {
                    if ($this->f_hodnota < (int)$this->popisNejnizsiHodnoty) $mimoRozsah = true;
                }
                if (StringUtils::isInteger($this->popisNejvyssiHodnoty)) {
                    if ($this->f_hodnota > (int)$this->popisNejvyssiHodnoty) $mimoRozsah = true;
                }

                if ($mimoRozsah) {
                    $this->f_error = true;
                    MessageManager::addErrorMessage(sprintf(ronText('Hodnota_mimo_roszah'), $this->popisNejnizsiHodnoty, $this->popisNejvyssiHodnoty));
                } else {
                    $this ->odpovedVyplnena = isset($this ->f_hodnota);
                }

                break;

            case TypOdpovedi::ZPUSOB_ODPOVEDI_VYBER_VICE_DEF :  // vyber vice moznosti
            case TypOdpovedi::ZPUSOB_ODPOVEDI_VYBER_VICE :      // vyber vice koeficientu
                $this ->loadPostVolby();
                $this ->odpovedVyplnena = true; // povoleno vice voleb (nebo zadna) => otazka je z principu vyplnena vzdy
                break;

            case TypOdpovedi::ZPUSOB_ODPOVEDI_VLASTNI : // vlastni odpoved
                $this ->f_slovniodpoved = $_POST[$this ->kod] ?? '';
                $this ->odpovedVyplnena = $this ->f_slovniodpoved != '';
                break;
            case TypOdpovedi::ZPUSOB_ODPOVEDI_VLASTNI_MEMO : // vlastni odpoved - memo
                $this ->f_memoodpoved = $_POST[$this ->kod];
                $this ->odpovedVyplnena = $this ->f_memoodpoved != '';
                break;

            case TypOdpovedi::ZPUSOB_ODPOVEDI_DATUM : // odpoved - datum
                $this ->datumStr = trim($_POST[$this ->kod]);
                if ($this ->datumStr != '') {
                    try {
                        $this ->f_datumodpoved = DateUtils::dateStringToTimestamp($this ->datumStr);
                    } catch (Exception $e) {
                        $this->f_error = true;
                        MessageManager::addErrorMessage($e ->getMessage());
                    }
                }

                $this ->odpovedVyplnena = !$this ->f_error  &&  $this ->datumStr != '';
                break;

            case TypOdpovedi::ZPUSOB_ODPOVEDI_KOREKCE : // korekce - body klasifikace + memo
                $this ->f_memoodpoved = $_POST[$this ->kod . '_text'];

                $value = $_POST[$this ->kod . '_hodnota'];
                if (($value != '') && (!StringUtils::isFloat($value))) {
                    $this->f_error = true;
                    MessageManager::addErrorMessage(ronText('Bad_number_format'));
                }

                $this ->bodyKlasifikace = StringUtils::stringToFloat($value);

                $mimoRozsah = false;
                if (StringUtils::isInteger($this->popisNejnizsiHodnoty)) {
                    if ($this->bodyKlasifikace < (int)$this->popisNejnizsiHodnoty) $mimoRozsah = true;
                }
                if (StringUtils::isInteger($this->popisNejvyssiHodnoty)) {
                    if ($this->bodyKlasifikace > (int)$this->popisNejvyssiHodnoty) $mimoRozsah = true;
                }

                if ($mimoRozsah) {
                    $this->f_error = true;
                    MessageManager::addErrorMessage(sprintf(ronText('Korekce_mimo_roszah'), $this->popisNejnizsiHodnoty, $this->popisNejvyssiHodnoty));
                }

                $this->povinnaOdpoved = false; // korekce neni nikdy povinna - nemusi byt vyplnena
                $this ->odpovedVyplnena = isset($this->bodyKlasifikace);
                break;

            default :
                throw new Exception('Chyba v metode OtazkaDotazniku::loadPost - neznamy zpusob odpovedi! ' . $this ->zpusobHodnoceni);
        }

        return !$this ->f_error;
    }


    /**
     * @param bool $povolitVice
     * @param bool $editace
     * @param int $oznacitSpravne urcuje, jestli se maji oznacit spravne a spatne odpovedi (0=ne, 1=jen zaskrtnute, 2=vsechny)
     * @param int $povolitHTML  0=ne, 1=ano a nahradit \n, 2=ano
     * @return string
     */
    function renderOdpovedi($povolitVice, $editace, $oznacitSpravne = 0, $povolitHTML = 0)
    {
        $result = '';

        if ($this->zobrazitOtazku != 0) {
            if ($povolitVice) {
                foreach ($this ->volby as $odpoved) {
                    if ($odpoved->vybrano) {
                        $elementName = StringUtils::htmlTextConvert( str_replace(' ', '_', $this ->kod) ) . '#' . $odpoved ->hodnota;
                        $result .= "<input type='hidden' name='{$elementName}' value='1'>";
                    }
                }
            } else {
                $elementName = StringUtils::htmlTextConvert($this ->kod);
                $result = "<input type='hidden' name='{$elementName}' value='{$this->f_hodnota}'>";
            }

            if ($result != '') {
                $result = '<div class="vycetodpovedi list-style-type-none">' . $result . '</div>';
            }

            if ($this->zobrazitOtazku == 2) {
                return $result;
            }
        }

        if ($this ->zpusobZobrazeni == 1) $result .= '<div class="row vycetodpovedi list-style-type-none">'; // vedle sebe
        else $result .= '<UL class="vycetodpovedi list-style-type-none">';

        foreach ($this ->volby as $odpoved) {
            if ($this ->zpusobZobrazeni == 1) {
                // vedle sebe
                $sirkastr = ($this ->sirkaSloupce) ? "style='width:{$this ->sirkaSloupce}px'" : '';

                $result .= "<div class='col' $sirkastr>";
            } else {
                $result .= '<LI>';
            }

            if ($povolitVice) {
                // ----- checkboxy ---------------------------------
                // povoleno vice hodnot - zobrazi checkboxy

                $elementName = StringUtils::htmlTextConvert( str_replace(' ', '_', $this ->kod) ) . '#' . $odpoved ->hodnota;
                if ($odpoved ->vybrano) {
                    // je zaskrtnuty
                    $checked = 'checked="checked"';
                    $classname = 'dotaznik_volba hodnocenichecked';
                } else {
                    // neni zaskrtnuty
                    $checked = '';
                    $classname = 'dotaznik_volba';
                }

                if ($oznacitSpravne) {
                    if (in_array($odpoved ->hodnota, $this ->vycetSpravnychOdpovedi)) {
                        if ($odpoved ->vybrano || $oznacitSpravne == 2) {
                            // odpoved je zaskrtnuta nebo se maji oznacit i spravne
                            $classname .= ' spravna_odpoved';
                        }
                    } else if ($odpoved ->vybrano) {
                        $classname .= ' spatna_odpoved';
                    }
                    //var_dump($this ->vycetSpravnychOdpovedi);
                }

                $disabledStr = !$editace ? 'disabled="disabled"' : '';

                //doplnkovy popis po najeti na checkbox
                $dodatecnyPopis = '';
                if ($odpoved ->doplnujiciPopis !== '') {
                    $dodatecnyPopis = 'title="' . StringUtils::htmlTextConvert($odpoved ->doplnujiciPopis) . '"';
                }

                if ($odpoved ->popis !== '') {
                    // pokud je text prazdny, tak buttonek nezobrazi
                    $result .=
                        "<label class='$classname' $dodatecnyPopis><input type='checkbox' name='$elementName' $checked $disabledStr /> " .
                        "<span>" .
                        ($povolitHTML ? $odpoved ->popis : StringUtils::htmlTextConvert($odpoved ->popis)) .
                        "</span></label>";
                }

            } else {
                // ----- radiobuttonky -----------------------------
                // povolen jen jeden - zobrazi radiobuttony
                $ischecked = isset($this ->f_hodnota) && $this ->f_hodnota == $odpoved ->hodnota;

                if ($ischecked) {
                    $checked = 'checked="checked"';
                    $classname = 'dotaznik_volba hodnocenichecked';

                    //$jeVybrano = true; // nastavi priznak, ze je nektera z odpovedi vybrana
                } else {
                    $checked = '';
                    $classname = 'dotaznik_volba';
                }

                if ($oznacitSpravne) {
                    if ($odpoved->hodnota == $this->spravnaHodnota) {
                        if ($ischecked || $oznacitSpravne == 2) {
                            // odpoved je zaskrtnuta nebo se maji oznacit i spravne
                            $classname .= ' spravna_odpoved';
                        }
                    } else if ($ischecked) {
                        $classname .= ' spatna_odpoved';
                    }
                }

                $elementName = StringUtils::htmlTextConvert($this ->kod);

                //doplnkovy popis po najeti na checkbox
                $dodatecnyPopis = '';
                if ($odpoved ->doplnujiciPopis !== '') {
                    $dodatecnyPopis = 'title="' . StringUtils::htmlTextConvert($odpoved ->doplnujiciPopis) . '"';
                }

                if ($odpoved ->popis !== '') {
                    // (pokud neni zadan text, tak radiobuttonek nezobrazi)
                    $result .= "<label class='$classname' $dodatecnyPopis>";
                    //if ($editace || $checked) {
                    // v pripade editace nebo pokud je volba vybrana zobrazi radiobutonek
                    $disabledStr = (!$editace) ? 'disabled="disabled"' : '';

                    $result .= "<input type='radio' name='$elementName' value='{$odpoved ->hodnota}' $checked $disabledStr /> ";
                    //}

                    $result .=
                        '<span>' .
                        ($povolitHTML ? $odpoved ->popis : StringUtils::htmlTextConvert($odpoved ->popis)) .
                        '</span>';

                    $result .= '</label>';
                }

            }

            $result .= ($this ->zpusobZobrazeni == 1)
                ? "</div>\n" // vedle sebe
                : "</LI>\n";
        }

        $result .= ($this ->zpusobZobrazeni == 1)
            ? "</div>\n"
            : "</UL>";

        //$this ->odpovedVyplnena = $povolitVice || $jeVybrano; - PRESUNUTO do nacitani

        return $result;
    }


    /**
     * @param bool $editace
     * @param int  $oznacitSpravne  0=ne, 1=oznacit, 2=vypsat i spravnou
     * @return string
     */
    function HTMLOdpoved_Edit($editace, $oznacitSpravne)
    {
        $spravnaOdpoved = strtolower($this ->f_slovniodpoved ?? '') == strtolower($this ->spravnaSlovniOdpoved ?? '');

        $classErrStr = !$this->odpovedVyplnena && $this->povinnaOdpoved ? 'error' : '';

        if ($editace) {
            return
                "<input type='text' class='form-control $classErrStr' name='{$this ->kod}' maxlength='160' value='" . StringUtils::htmlTextConvert($this ->f_slovniodpoved) . "' />";
        } else {
            $result =
                "<div class='$classErrStr'>" .
                "<input type='hidden' name='{$this ->kod}' value='" . StringUtils::htmlTextConvert($this ->f_slovniodpoved) . "' />";

            $classStr = '';
            if ($oznacitSpravne) {
                $classStr = ($spravnaOdpoved) ? 'spravna_odpoved' : 'spatna_odpoved';
            }

            //pro skryte nastaveni nezobrazovat odpoved
            if ($this->zobrazitOtazku !== 2) {
                $result .= "<div class='textovaodpoved $classStr'>" . StringUtils::htmlTextConvert($this->f_slovniodpoved) . "</div>";
            }

            $result .=
                "</div>\n";

            if ($oznacitSpravne == 2  &&  !$spravnaOdpoved) {
                // jeste vypise spravnou odpoved
                $result .=
                    '<div class="pozadovana-odpoved">' . ronText('Spravna_odpoved') . ': <span class="spravna_odpoved">' .
                    StringUtils::htmlTextConvert($this ->spravnaSlovniOdpoved) .
                    '</span></div>';
            }

            return $result;
        }
    }



    /**
     * @param bool $editace
     * @param int  $oznacitSpravne  0=ne, 1=oznacit, 2=vypsat i spravnou
     * @return string
     */
    function HTMLOdpoved_Hodnota($editace, $oznacitSpravne)
    {
        $classStr = '';
        $spravnaOdpoved = $this ->f_hodnota == $this ->spravnaHodnota;

        if ($oznacitSpravne) {
            $classStr = ($spravnaOdpoved) ? 'spravna_odpoved' : 'spatna_odpoved';
        }

        $classErrStr = $this ->f_error  ||  (!$this->odpovedVyplnena && $this->povinnaOdpoved)
            ? 'error'
            : '';

        if ($editace) {
            return
                "<input type='number' class='form-control inline-form-control inputInt $classErrStr' name='{$this ->kod}' maxlength='10' min='{$this->popisNejnizsiHodnoty}' max='{$this->popisNejvyssiHodnoty}' value='" . ($this->f_hodnota ?? '') . "' />";
        } else {
            $result =
                ($classErrStr ? "<div class='$classErrStr'>" : '') .
                "<input type='hidden' name='{$this ->kod}' value='" . ($this->f_hodnota ?? '') . "' />" .
                ($this->zobrazitOtazku !== 2 //pro skryte nastaveni nezobrazovat odpoved
                    ? "<div class='textovaodpoved $classStr'>" . ($this->f_hodnota ?? '') . "</div>"
                    : ''
                ) .
                ($classErrStr ? '</div>' : '') .
                "\n";

            if ($oznacitSpravne == 2  &&  !$spravnaOdpoved) {
                // jeste vypise spravnou odpoved
                $result .=
                    '<div class="pozadovana-odpoved">' . ronText('Spravna_odpoved') . ': <span class="spravna_odpoved">' . $this ->spravnaHodnota . '</span></div>';
            }

            return $result;
        }
    }



    function HTMLOdpoved_Datum($editace)
    {
        $classErrStr = !$this->odpovedVyplnena && $this->povinnaOdpoved ? 'error' : '';

        if ($editace) {
            return
                "<input class='form-control inline-form-control inputDate $classErrStr' type='text' name='{$this ->kod}' maxlength='10' size='10' value='" . StringUtils::htmlTextConvert($this ->datumStr) . "' />" .
                "<i class='showcalendar inline-glyph glyph-button fa glyph-calendar' data-roncalendar-inputname='{$this ->kod}'></i>";
        } else {
            return
                "<input type='hidden' name='{$this ->kod}' value='" . StringUtils::htmlTextConvert($this ->datumStr) . "' />" .
                ($this->zobrazitOtazku !== 2  //pro skryte nastaveni nezobrazovat odpoved
                    ? "<div class='textovaodpoved'>" . StringUtils::htmlTextConvert($this->datumStr) . "</div>"
                    : ''
                );
        }
    }


    function HTMLOdpoved_Memo($editace)
    {
        $classErrStr =
            (!$this->odpovedVyplnena && $this->povinnaOdpoved ? ' error' : '');

        if ($editace) {
            return
                "<textarea class='form-control $classErrStr' name='{$this ->kod}' rows='8'>" .
                StringUtils::htmlTextConvert($this ->f_memoodpoved) .
                "</textarea>";
        } else {
            return
                "<input type='hidden' name='{$this ->kod}' value='" . StringUtils::htmlTextConvert($this ->f_memoodpoved) . "' />" .

                ($this->zobrazitOtazku !== 2 //pro skryte nastaveni nezobrazovat odpoved
                    ? "<div class='textovaodpoved'>" . StringUtils::longTextToHtml($this->f_memoodpoved, true) . "</div>"
                    : ''
                );
        }
    }


    function HTMLVaha_Edit($editace)
    {
        //pokud je zadana hodnota pouzije se z postu, jinak v pripade, ze je zadana vlastnivaha pouzije se vlastni vaha
        //pokud neni zadana vlastni vaha, pouzije se vychozi
        //$hodnota = $postValues ? $_POST[$this ->kod . '_vaha'] : ($this ->vlastniVaha ? $this ->vlastniVaha : $this ->vychoziVaha);

        if ($editace) {
            $className =
                'inputInt' .
                ( ($this ->vahaStr != ''  &&  !StringUtils::isInteger($this ->vahaStr)) ? ' app_input_error' : '' );
            return
                '<DIV class="odpovedhodnoceni_vaha">' . ronText('Vlastni_bodova_vaha') . ' ' .
                "<INPUT type='text' class='$className' name='{$this ->kod}_vaha' size='3' maxlength='3' value='" . StringUtils::htmlTextConvert($this ->vahaStr) . "' />" .
                "</DIV>\n";
        } else {
            $result =
                '<DIV class="odpovedhodnoceni_vaha">' . ronText('Vlastni_bodova_vaha') . ' ' .
                "<INPUT type='hidden' name='{$this ->kod}_vaha' value='" . StringUtils::htmlTextConvert($this ->vahaStr) . "' />";
            //pro skryte nastaveni nezobrazovat odpoved
            if ($this->zobrazitOtazku !== 2)
                $result .= "<DIV class='textovaodpoved'>" . StringUtils::htmlTextConvert($this ->vahaStr) . "</DIV>";

            $result .=
                "</DIV>\n";

            return $result;
        }
    }

    function HTMLOdpoved_Korekce($editace)
    {
        $classStr = '';

        if ($editace) {
            return
                "<input type='text' class='form-control inputFloat' name='{$this ->kod}_hodnota' maxlength='10' value='" . ($this ->bodyKlasifikace ?: '') . "' />" .
                "<textarea class='form-control' name='{$this ->kod}_text' rows='8'>" . StringUtils::htmlTextConvert($this ->f_memoodpoved) . "</textarea>";
        }

        $result =
            "<INPUT type='hidden' name='{$this ->kod}_hodnota' value='" . ($this ->bodyKlasifikace ? $this ->bodyKlasifikace : '') . "' />";

        //pro skryte nastaveni nezobrazovat odpoved
        if ($this->zobrazitOtazku !== 2) {
            $result .= "<div class='textovaodpoved $classStr'>" . ($this->bodyKlasifikace ? $this->bodyKlasifikace : '') . "</div>";
        }

        // memo
        $result .=
            "<INPUT type='hidden' name='{$this ->kod}_text' value='" . StringUtils::htmlTextConvert($this ->f_memoodpoved) . "' />";

        //pro skryte nastaveni nezobrazovat odpoved
        if ($this->zobrazitOtazku !== 2) {
            $result .= "<DIV class='textovaodpoved'>" . StringUtils::longTextToHtml($this->f_memoodpoved, true) . "</DIV>";
        }

        return $result;
    }


    /**
     * @param bool $editace
     * @param int  $oznacitSpravne  urcuje, jestli se maji oznacit spravne a spatne odpovedi (0=ne, 1=jen zaskrtnute, 2=vsechny)
     * @param int $povolitHTML  0=ne, 1=ano a nahradit \n, 2=ano
     * @return string
     */
    function vratHTML_Odpovedi($editace, $oznacitSpravne = 0, $povolitHTML = 0)
    {
        // $this ->odpovedVyplnena = false; - PRESUNUTO do nacitani
        // pokud otazka nema vyplnenou odpoved, tak se dale priznak nastavi na false

        //pokud uzivatel muze definovat vlastni otazku a hodnoceni na urovni, kde na ni odpovida, zobrazi se pole pro zadani otazky
        if ($this->definovatVlastniOtazku && !$this->zobrazitVlastniOtazku) {
            if ($this->maPodotazky) {
                //uzivatel muze otazku slozit z vice podotazek, zobrazi se pole pro zadani otazek dle preddefinovaneho retezce v Popisu pro web u otazky
                $result = '';

                foreach ($this->podotazky as $podotazka) {
                    /** @var $podotazka PodrizenaOtazkaDotazniku */
                    $result .= $podotazka->HTML_Otazka($editace, ($this->zobrazitOtazku !== 2));
                }

                return $result . $this->HTMLVaha_Edit($editace);
            } else {
                //uzivatel definuje jen jednu otazku
                return $this->HTMLOdpoved_Memo($editace) . $this->HTMLVaha_Edit($editace);
            }
        }

        switch ($this->zpusobHodnoceni)  {
            case TypOdpovedi::ZPUSOB_ODPOVEDI_VYBER1_DEF : // vyber 1 moznosti
            case TypOdpovedi::ZPUSOB_ODPOVEDI_STUPNICE : // stupnice
                return $this->renderOdpovedi(false, $editace, $oznacitSpravne, $povolitHTML); // povolit vice = false
                break;

            case TypOdpovedi::ZPUSOB_ODPOVEDI_VYBER_VICE_DEF : // vyber vice moznosti
                return $this->renderOdpovedi(true, $editace, $oznacitSpravne, $povolitHTML); // povolit vice = true
                break;

            case TypOdpovedi::ZPUSOB_ODPOVEDI_VLASTNI : // vlastni odpoved
                return $this->HTMLOdpoved_Edit($editace, $oznacitSpravne);
                break;

            case TypOdpovedi::ZPUSOB_ODPOVEDI_HODNOTA : // vlastni odpoved hodnotou
                return $this->HTMLOdpoved_Hodnota($editace, $oznacitSpravne);
                break;

            case TypOdpovedi::ZPUSOB_ODPOVEDI_VYBER1 : // vyber koeficientu (jedna moznost z tabulky VycetOdpovedi)
                return $this->renderOdpovedi(false, $editace, $oznacitSpravne, $povolitHTML); // povolit vice = false
                break;

            case TypOdpovedi::ZPUSOB_ODPOVEDI_VYBER_VICE : // vyber vice koeficientu
                return $this->renderOdpovedi(true, $editace, $oznacitSpravne, $povolitHTML); // povolit vice = true
                break;

            case TypOdpovedi::ZPUSOB_ODPOVEDI_VLASTNI_MEMO : // vlastni odpoved - memo
                return $this->HTMLOdpoved_Memo($editace);
                break;

            case TypOdpovedi::ZPUSOB_ODPOVEDI_DATUM : // odpoved - datum
                return $this->HTMLOdpoved_Datum($editace);
                break;

            case TypOdpovedi::ZPUSOB_ODPOVEDI_KOREKCE : //korekce - cislo + memo pole
                return $this->HTMLOdpoved_Korekce($editace);
                break;
        }

        return '';
    }


    /**
     * Nova metoda pro zobrazeni HTML odpovedi, slouzi pro zretezeni vice typu odpovedi u jedne otazky.
     *
     * @param bool $editace priznak ze je hodnoceni v editacnim rezimu
     * @param int  $oznacitSpravne  urcuje, jestli se maji oznacit spravne a spatne odpovedi (0=ne, 1=jen zaskrtnute, 2=vsechny)
     * @param int $povolitHTML  0=ne, 1=ano a nahradit \n, 2=ano
     * @return string HTML struktura odpovedi
     */
    function vratHTML_Odpoved($editace, $oznacitSpravne = 0, $povolitHTML = 0)
    {
        //dalsi zpusoby odpovedi se nezobrazi pokud se jedna o definici vlastni otazky
        if ($this->viceTypuOdpovedi && !($this->definovatVlastniOtazku && !$this->zobrazitVlastniOtazku)) {

            $result =
                '<div class="odpovedhodnoceni">' .
                '<div class="row vicenasobnaodpoved">' .
                '<div class="col">' .
                $this->vratHTML_Odpovedi($editace, $oznacitSpravne, $povolitHTML) .
                '</div>';

            foreach ($this->dalsiOdpovedi as $otazka) {
                $result .=
                    '<div class="col">' .
                    $otazka->vratHTML_Odpovedi($editace, $oznacitSpravne, $povolitHTML) .
                    '</div>';
            }

            $result .=
                "</div></div>\n";

            return $result;
        } else {
            return
                '<div class="odpovedhodnoceni">' .
                $this->vratHTML_Odpovedi($editace, $oznacitSpravne, $povolitHTML) .
                '</div>';
        }
    }


    /**
     * @param int $povolitHTML  0=ne, 1=ano a nahradit /n, 2=ano
     * @return string
     */
    function vratHTML_Otazka($povolitHTML = 0)
    {
        //pokud uzivatel muze definovat vlastni otazku a hodnoceni je v predavani na urovni, kde se ma zobrazit vlastni otazka, zobrazi se otazka, kterou si definoval,
        //jinak se zobrazi otazka z ciselniku
        $result = '<DIV class="textotazky">';

        if ($this->definovatVlastniOtazku && $this->zobrazitVlastniOtazku) {
            if ($this->maPodotazky) {
                foreach ($this->podotazky as $podotazka) {
                    $result .= $podotazka->HTML_Odpoved();
                }
            } else {
                $result .= $this->vlastniOtazka;
            }
            //pokud je definovana vlastni vaha, zobrazi se jeji hodnota spolu s vlastni otazkou
            if ($this->vlastniVaha) {
                $result .= '<DIV class="textotazky">' . ronText('Vlastni_bodova_vaha') . ' ' . $this->vlastniVaha . '</DIV>';
            }
        } else {
            $result .= $this->f_popis;
        }

        $result .= '</DIV>';

        //pokud se nejedna o vlastni otazku, ale otazku z ciselniku zobrazi se detailni popis
        if (!$this ->maPodotazky) {
            // text otazky (pokud neni zadan text pro web, zobrazi vychozi text)
            $text = $this ->f_popisproweb ?: $this ->f_upresnujicipopis;

            if ($text) {
                $result .=
                    '<div class="podrobnostiotazky poznamka">' .
                    (($povolitHTML == 2) ? $text : StringUtils::longTextToHtml($text, !$povolitHTML)) .
                    "</div>\n";
            }
        }

        return $result;
    }


    /**
     * @param bool $editace
     * @param int $oznacitSpravne urcuje, jestli se maji oznacit spravne a spatne odpovedi (0=ne, 1=jen zaskrtnute, 2=vsechny)
     * @param int $povolitHTML  0=ne, 1=ano a nahradit konce radku, 2=ano
     */
    function zobrazOtazku($editace, $oznacitSpravne = 0, $povolitHTML = 0)
    {
        //zobrazi se jen otazky, ktere nejsou definovany jako skryte (definovano, bud pro konkretni uroven predavani, nebo primo pro otazku)
        if ($this->zobrazitOtazku !== 2)
            echo $this->vratHTML_Otazka($povolitHTML);

        if (($this->zobrazitOtazku == 1) || ($this->zobrazitOtazku == 2))
            $editace = false;

        echo $this ->vratHTML_Odpoved($editace, $oznacitSpravne, $povolitHTML);
    }


    // EDITACE DAT ------------------------------------------------------------
    /**
     * @param Database $database
     * @param bool $vlastniOdpovedi   urcuje jestli ma otazka vlastni seznam odpovedi (typ "vyber vice koeficientu")
     * @throws Exception
     */
    private function ulozVicenasobnouOdpoved($database, $idDotazniku, $vlastniOdpovedi)
    {
        // nejdrive smaze puvodni odpovedi
        $params = $database ->createParameters();
        $sql =
            'DELETE FROM VycetOdpovediHodnoceni ' .
            ' WHERE DruhHodnoceni = ' . $params ->addParameter((int)$this ->druhHodnoceni) .
            ' AND TemaCislo = ' . $params ->addParameter((int)$this ->temaCislo) .
            ' AND KodKriteria = ' . $params->addParameter($this ->kod) .
            ' AND ID = ' . $params->addParameter($idDotazniku);

        $database ->sql($sql, $params);

        // nacte vsechny mozne odpovedi
        $params = $database ->createParameters();
        if ($vlastniOdpovedi) {
            $sql =
                'SELECT Hodnota ' .
                '  FROM VycetOdpovedi ' .
                ' WHERE DruhHodnoceni = ' . $params ->addParameter((int)$this ->druhHodnoceni) .
                '  AND TemaCislo = ' . $params ->addParameter((int)$this ->temaCislo) .
                '  AND KodKriteria = ' . $params->addParameter($this ->kod);
        } else {
            $sql =
                'SELECT Hodnota ' .
                '  FROM RozsahHodnoceni ' .
                ' WHERE TypRozsahu = ' . $params->addParameter($this ->f_typrozsahu);
        }

        $cursor = $database ->getSqlResult($sql, $params);

        while ($volba_row = $cursor ->fetch()) {
            $elementName = str_replace(' ', '_', $this ->kod) . '#' . $volba_row[0];
            if ($_POST[$elementName] ?? false) {
                // checkbox je zaskrtnuty - ulozi hodnotu
                $insertParams = $database ->createParameters();
                $sql =
                    'INSERT INTO VycetOdpovediHodnoceni ' .
                    ' (Id, DruhHodnoceni, TemaCislo, KodKriteria, DatumHodnoceni, Hodnota ) ' .
                    ' VALUES ( ' .
                    $insertParams ->addParameter($idDotazniku) . ', ' .
                    $insertParams ->addParameter((int)$this ->druhHodnoceni) . ', ' .
                    $insertParams ->addParameter((int)$this ->temaCislo) . ', ' .
                    $insertParams ->addParameter($this ->kod) . ', ' .
                    $insertParams ->addParameter($database ->timestampToParam(time())) . ', ' .
                    $insertParams ->addParameter((int)$volba_row[0]) .
                    ')';

                $database ->sql($sql, $insertParams);
            }
        }
        $cursor ->close();
    }


    /**
     * @param Database $database
     * @param int $idDotazniku  Id dotazniku je potreba zadat, aby fungoval i insert (napr. pri prvnim vyplnovani ankety)
     * @throws DBException
     */
    function ulozOdpoved($database, $idDotazniku)
    {
        $rec = new RonPortalDBRecord($database, 'Hodnoceni', $this);

        $rec ->addField('Id', 'int', true, 0, $idDotazniku); // readonly        $this ->f_id - predelano na parametr, aby fungoval insert (u anket)
        $rec ->addField('DruhHodnoceni', 'int', true, 0, $this ->druhHodnoceni); // readonly
        $rec ->addField('TemaCislo', 'int', true, 0, $this ->temaCislo); // readonly
        $rec ->addField('KodKriteria', '', true, 10, $this ->kod);

        if (!($this ->definovatVlastniOtazku && !$this ->zobrazitVlastniOtazku)) {
            switch ($this ->zpusobHodnoceni) {
                case 1 : // vice moznych odpovedi - jsou ulozeny az v podrizene tabulce
                case 6 :
                    $rec ->addField('Hodnota', 'int', false, 0, null);
                    $rec ->addField('DatumOdpoved', 'date', false, 0, null);
                    $rec ->addField('SlovniOdpoved', '', false, 160, null);
                    $rec ->addField('MemoOdpoved', 'memo', false, 0, null);

                    $this ->odpovedVyplnena = true; // z principu je vyplnena vzdy
                    break;

                case 2 : // vlastni odpoved - text
                    $rec ->addField('Hodnota', 'int', false, 0, null);
                    $rec ->addField('DatumOdpoved', 'date', false, 0, null);
                    $rec ->addField('SlovniOdpoved', '', false, 160, ($_POST[$this ->kod] ?? null) ?: null);
                    $rec ->addField('MemoOdpoved', 'memo', false, 0, null);

                    $this ->odpovedVyplnena = isset($_POST[$this ->kod])  &&   $_POST[$this ->kod] != '';
                    break;

                case 7 : // vlastni odpoved - memo
                    $rec ->addField('Hodnota', 'int', false, 0, null);
                    $rec ->addField('DatumOdpoved', 'date', false, 0, null);
                    $rec ->addField('SlovniOdpoved', '', false, 160, null);
                    $rec ->addField('MemoOdpoved', 'memo', false, 0, ($_POST[$this ->kod] ?? null) ?: null);

                    $this ->odpovedVyplnena = ($_POST[$this ->kod] ?? '') != '';
                    break;

                case 8 : // datum
                    $this ->odpovedVyplnena = false;
                    $pomts = null;

                    if ($_POST[$this ->kod]) {
                        try {
                            $pomts = DateUtils::datetimeStringToTimestamp($_POST[$this->kod]);
                            $this ->odpovedVyplnena = true;
                        } catch (\Exception $e) {
                            $this ->f_error = true;
                            MessageManager::addErrorMessage($e ->getMessage() );
                            //throw new \Exception( \Ron\ronText('Bad_date_format') );
                        }
                    }

                    $rec ->addField('Hodnota', 'int', false, 0, null);
                    $rec ->addField('DatumOdpoved', 'date', false, 0, $pomts);
                    $rec ->addField('SlovniOdpoved', '', false, 160, null);
                    $rec ->addField('MemoOdpoved', 'memo', false, 0, null);

                    break;

                case 9 : // korekce - body + memo
                    $rec ->addField('Hodnota', 'int', false, 0, null);
                    $rec ->addField('DatumOdpoved', 'date', false, 0, null);
                    $rec ->addField('SlovniOdpoved', '', false, 160, null);

                    $value = $_POST[$this ->kod . '_hodnota'];
                    if (($value != '') && (!StringUtils::isFloat($value))) throw new Exception(ronText('Bad_number_format'));

                    $rec ->addField('BodyKlasifikace', 'float', false, 0, $value ? StringUtils::stringToFloat($value) : null);
                    $rec ->addField('MemoOdpoved', 'memo', false, 0, $_POST[$this ->kod . '_text'] ? $_POST[$this ->kod . '_text'] : null);

                    $this ->odpovedVyplnena = ($_POST[$this ->kod] ?? '') != '';
                    break;

                default : // 0 - vyber, 3 - stupnice
                    $rec ->addField('Hodnota', 'int', false, 0, isset($_POST[$this ->kod]) ? (int)$_POST[$this ->kod] : null);
                    $rec ->addField('DatumOdpoved', 'date', false, 0, null);
                    $rec ->addField('SlovniOdpoved', '', false, 160, null);
                    $rec ->addField('MemoOdpoved', 'memo', false, 0, null);

                    $this ->odpovedVyplnena = isset($_POST[$this ->kod]);
                    break;
            }
        }

        if ($this ->definovatVlastniOtazku  &&  !$this ->zobrazitVlastniOtazku) {
            $rec ->addField('Hodnota', 'int', false, 0, null);
            $rec ->addField('DatumOdpoved', 'date', false, 0, null);
            $rec ->addField('SlovniOdpoved', '', false, 160, null);
            $rec ->addField('MemoOdpoved', 'memo', false, 0, null);
            $rec ->addField('VlastniBodovaVaha', 'int', false, 0, $_POST[$this ->kod . '_vaha'] ?: null);

            if ($this ->maPodotazky) {
                $odpoved = '';

                $this->odpovedVyplnena = true;

                foreach ($this->podotazky as $otazka) {
                    $odpoved .= $otazka->retezecNastaveni . ($_POST[$otazka->kod] ? $_POST[$otazka->kod] : '');
                    $this->odpovedVyplnena &= !!$_POST[$otazka->kod];
                }

                $rec->addField('VlastniTextOtazky', 'memo', false, 0, $odpoved);

            } else {
                $rec->addField('VlastniTextOtazky', 'memo', false, 0, $_POST[$this->kod] ?: null);

                $this->odpovedVyplnena = !!$_POST[$this->kod];
            }
        }

        $params = $database ->createParameters();
        $rec ->setWhere(
            ' Id = ' . $params ->addParameter($idDotazniku) .   // (int)$this ->f_id - predelano na parametr $idDotazniku, aby fungoval insert (u anket)
            ' AND DruhHodnoceni = ' . $params ->addParameter((int)$this ->druhHodnoceni) .
            ' AND TemaCislo = ' . $params ->addParameter((int)$this ->temaCislo) .
            ' AND KodKriteria = ' . $params->addParameter($this ->kod),
            $params);

        $rec ->updateOrInsert();

        switch ($this ->zpusobHodnoceni) {
            case 1 :
                $this ->ulozVicenasobnouOdpoved($database, $idDotazniku, false); // vlastni odpovedi = false (vyber vice moznosti)
                break;

            case 6 :
                $this ->ulozVicenasobnouOdpoved($database, $idDotazniku, true); // vlastniOdpovedi = true (otazka ma vlastni vycet moznosti = vyber vice koeficientu)
                break;
        }
    }

}
