<?php

namespace RonPortal\Testy;

use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\StringUtils;

class TypOdpovedi
{
    const ZPUSOB_ODPOVEDI_VYBER1_DEF = 0; // vyber z preddefinovanych hodnot
    const ZPUSOB_ODPOVEDI_VYBER_VICE_DEF = 1;
    const ZPUSOB_ODPOVEDI_VLASTNI = 2;
    const ZPUSOB_ODPOVEDI_STUPNICE = 3;   // (min-max)
    const ZPUSOB_ODPOVEDI_HODNOTA = 4;
    const ZPUSOB_ODPOVEDI_VYBER1 = 5;     // otazka ma vlastni odpovedi
    const ZPUSOB_ODPOVEDI_VYBER_VICE = 6;
    const ZPUSOB_ODPOVEDI_VLASTNI_MEMO = 7;
    const ZPUSOB_ODPOVEDI_DATUM = 8;
    const ZPUSOB_ODPOVEDI_KOREKCE = 9;

    /** @var string */
    var $kod;
    /** @var string */
    var $popis;
    /** @var integer */
    var $zpusobHodnoceni;

    /** @var string */
    var $popisNejnizsiHodnoty;  // pro ZPUSOB_ODPOVEDI_STUPNICE
    /** @var string */
    var $popisNejvyssiHodnoty;
    /** @var int */
    var $zpusobZobrazeni; // 0=pod sebou, 1=vedle sebe

    /** @var VolbaDotazniku[] */
    var $volby = [];

    /**
     * Nacte seznam vsech moznych typu odpovedi vcetne preddefinovanych voleb
     * @param Database $database
     * @param int $typVyuziti
     * @return TypOdpovedi[]
     * @throws DBException
     */
    public static function nactiTypyOdpovedi($database, $typVyuziti = 1)
    {
        $result = [];

        $params = $database->createParameters();

        $sql =
            'SELECT TypRozsahuHodnoceni.Kod, TypRozsahuHodnoceni.Popis, TypRozsahuHodnoceni.ZpusobHodnoceni, ' . // 0..2
            '       TypRozsahuHodnoceni.PopisNejnizsiHodnoty, TypRozsahuHodnoceni.PopisNejvyssiHodnoty, TypRozsahuHodnoceni.ZpusobZobrazeni, ' . // 3..5
            '       RozsahHodnoceni.Hodnota, RozsahHodnoceni.Popis, RozsahHodnoceni.DoplnujiciPopis ' . // 6..8
            '  FROM TypRozsahuHodnoceni LEFT OUTER JOIN RozsahHodnoceni ' .
            '       ON RozsahHodnoceni.TypRozsahu = TypRozsahuHodnoceni.Kod ' .
            ' WHERE TypRozsahuHodnoceni.TypVyuziti = ' . $params->addParameter($typVyuziti) .
            ' ORDER BY TypRozsahuHodnoceni.Kod';

        $cursor = $database ->getSqlResult($sql, $params);

        /** @var TypOdpovedi $typOdpovedi*/
        $typOdpovedi = null;

        while ($row = $cursor ->fetch()) {
            if ($typOdpovedi  &&  $typOdpovedi ->kod !== $row[0]) {
                $result[] = $typOdpovedi;
                $typOdpovedi = null;
            }

            if (!$typOdpovedi) {
                $typOdpovedi = new TypOdpovedi();
                $typOdpovedi ->kod = $row[0];
                $typOdpovedi ->popis = $row[1];
                $typOdpovedi ->zpusobHodnoceni = (int)$row[2];
                $typOdpovedi ->popisNejnizsiHodnoty = $row[3];
                $typOdpovedi ->popisNejvyssiHodnoty = $row[4];
                $typOdpovedi ->zpusobZobrazeni = (int)$row[5];
            }

            if ($row[6] ||  $row[7]) {
                $volba = new VolbaDotazniku();

                $volba ->hodnota = (int)$row[6];
                $volba ->popis = $row[7];
                $volba ->doplnujiciPopis = $row[8];

                $typOdpovedi ->volby[] = $volba;
            }
        }

        if ($typOdpovedi) {
            $result[] = $typOdpovedi;
        }

        return $result;
    }



    public function getJson()
    {
        $result = [
            'kod' => StringUtils::jsonText($this ->kod),
            'popis' => StringUtils::jsonText($this ->popis),
            'zpusobHodnoceni' => $this ->zpusobHodnoceni,

            'popisNejnizsiHodnoty' => StringUtils::jsonText($this ->popisNejnizsiHodnoty),
            'popisNejvyssiHodnoty' => StringUtils::jsonText($this ->popisNejvyssiHodnoty),

            'zpusobZobrazeni' => $this ->zpusobZobrazeni,
            'volby' => []
        ];

        foreach ($this ->volby as $volba) {
            $result['volby'][] = $volba ->getJson();
        }

        return $result;
    }


    /**
     * @param TypOdpovedi[] $typyOdpovedi
     * @return array
     */
    public static function getJsonList($typyOdpovedi)
    {
        $result = [];
        foreach ($typyOdpovedi as $typOdpovedi) {
            $result[] = $typOdpovedi ->getJson();
        }

        return $result;
    }
}
