<?php

namespace RonPortal\Testy;

use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\SqlStatement;
use Ron\MVC\Model;
use Ron\MVC\ViewObject;
use Ron\StringUtils;
use Tracy\Debugger;


class StatistikaOdpovedi
{
    /** @var array  Asociativni pole hodnota => pocet */
    public $pocetOdpovedi = [];

    public $texty = [];
}



class VyhodnoceniDotaznikuView implements ViewObject
{
    /** @var Dotaznik */
    private $dotaznik = null;

    /** @var StatistikaOdpovedi[]   Asociativni pole KodKriteria => StatistikaOdpovedi */
    private $statistikaOdpovedi = [];


    public function loadData(Database $database, int $druhHodnoceni, int $temaCislo)
    {
        Model::getInstance()
            ->addScript('script/chart.js', 'chart-js')
            ->addScript('script/chartjs-plugin-datalabels.min.js', 'chartjs-plugin-datalabels-js');

        $this ->dotaznik = new Dotaznik($database);
        $this ->dotaznik ->nactiSablonu($druhHodnoceni, $temaCislo, false);
        $this ->loadOdpovedi($database, '');
    }


    /**
     * Nacte odpovedi, ktere se vybiraji z voleb (varianta pro jednu i vicenasobnou volbu)
     * @param Database $database
     * @param string $tableName Urcuje tabulku, ze ktere se nacitaji hodnoty - podle toho, jestli se jedna o vicenasobnou volbu
     * @param OtazkaDotazniku $otazkaDotazniku
     * @param string $filtr     Filtr pro vyber jen nekterych odpovedi - napriklad pro jednu konkretni ankety (pouzije se v pripade, ze je jedne dotaznik prirazen k vice anketam)
     * @return void
     * @throws DBException
     */
    private function loadOdpovediVyber(Database $database, string $tableName, OtazkaDotazniku $otazkaDotazniku, $filtr)
    {
        $params = $database->createParameters();

        $sql = new SqlStatement(
            "SELECT $tableName.Hodnota, COUNT(*)" .
            "  FROM $tableName, PripraveneTemaHodnoceni" .
            " WHERE $tableName.DruhHodnoceni = " . $params ->addParameter($otazkaDotazniku ->druhHodnoceni) .
            "   AND $tableName.TemaCislo = " . $params ->addParameter($otazkaDotazniku ->temaCislo) .
            "   AND $tableName.KodKriteria = " . $params ->addParameter($otazkaDotazniku ->kod) .
            "   AND PripraveneTemaHodnoceni.Id = $tableName.Id",
            $params);


        if ($filtr) {
            $sql ->append($filtr);
        }

        $sql ->append(
            " GROUP BY $tableName.Hodnota" .
            " ORDER BY $tableName.Hodnota"
        );

        $cursor = $database ->select($sql);

        $statistika = new StatistikaOdpovedi();

        while ($row = $cursor ->fetch()) {
            $statistika ->pocetOdpovedi[$row[0]] = $row[1];
        }

        $this ->statistikaOdpovedi[$otazkaDotazniku ->kod] = $statistika;
    }


    private function loadOdpovediVyber1(Database $database, OtazkaDotazniku $otazkaDotazniku, $filtr)
    {
        $this ->loadOdpovediVyber($database, 'Hodnoceni', $otazkaDotazniku, $filtr);
    }

    private function loadOdpovediVyberVice(Database $database, OtazkaDotazniku $otazkaDotazniku, $filtr)
    {
        $this ->loadOdpovediVyber($database, 'VycetOdpovediHodnoceni', $otazkaDotazniku, $filtr);
    }


    private function loadTextoveOdpovedi(Database $database, OtazkaDotazniku $otazkaDotazniku, $filtr)
    {
        $params = $database->createParameters();

        $sql = new SqlStatement(
            "SELECT SlovniOdpoved, COUNT(*)" .
            "  FROM Hodnoceni, PripraveneTemaHodnoceni" .
            " WHERE Hodnoceni.DruhHodnoceni = " . $params ->addParameter($otazkaDotazniku ->druhHodnoceni) .
            "   AND Hodnoceni.TemaCislo = " . $params ->addParameter($otazkaDotazniku ->temaCislo) .
            "   AND Hodnoceni.KodKriteria = " . $params ->addParameter($otazkaDotazniku ->kod) .
            "   AND PripraveneTemaHodnoceni.Id = Hodnoceni.Id",
            $params);


        if ($filtr) {
            $sql ->append($filtr);
        }

        $sql ->append(
            " GROUP BY SlovniOdpoved" .
            " ORDER BY COUNT(*) DESC, SlovniOdpoved"
        );

        $cursor = $database ->select($sql);

        $statistika = new StatistikaOdpovedi();

        while ($row = $cursor ->fetch()) {
            $statistika ->texty[$row[0]] = $row[1];
        }

        $this ->statistikaOdpovedi[$otazkaDotazniku ->kod] = $statistika;
    }


    /**
     * @param Database $database
     * @param SqlStatement|string $filtr    Umozni vybrat jen nektere odpovedi - napr. v jedne konkretni ankete (protoze dotaznik muze byt pro vice anket)
     * @return void
     */
    public function loadOdpovedi(Database $database, $filtr)
    {
        foreach ($this ->dotaznik ->otazky as $otazkaDotazniku) {
            switch ($otazkaDotazniku ->zpusobHodnoceni) {
                case TypOdpovedi::ZPUSOB_ODPOVEDI_VYBER1 :
                case TypOdpovedi::ZPUSOB_ODPOVEDI_VYBER1_DEF :
                case TypOdpovedi::ZPUSOB_ODPOVEDI_STUPNICE :
                    $this ->loadOdpovediVyber1($database, $otazkaDotazniku, $filtr);
                    break;

                case TypOdpovedi::ZPUSOB_ODPOVEDI_VYBER_VICE :
                    $this ->loadOdpovediVyberVice($database, $otazkaDotazniku, $filtr);
                    break;

                case TypOdpovedi::ZPUSOB_ODPOVEDI_VLASTNI :
                    $this ->loadTextoveOdpovedi($database, $otazkaDotazniku, $filtr);
                    break;
//
//                case TypOdpovedi::ZPUSOB_ODPOVEDI_VYBER1_DEF :
//                case TypOdpovedi::ZPUSOB_ODPOVEDI_VYBER_VICE_DEF :
//                case TypOdpovedi::ZPUSOB_ODPOVEDI_STUPNICE :
//                    $this ->volby = VolbaDotazniku::nactiPreddefinovaneVolby($database, $this ->f_typrozsahu, $this ->kod, $this ->f_id);
//                    break;

            }
        }
    }


    /**
     * @param OtazkaDotazniku $otazkaDotazniku
     * @return void
     */
    public function renderOdpovedi($otazkaDotazniku)
    {
        $typGrafu = 'doughnut';

        switch ($otazkaDotazniku ->zpusobHodnoceni) {
            case TypOdpovedi::ZPUSOB_ODPOVEDI_VYBER_VICE :
            case TypOdpovedi::ZPUSOB_ODPOVEDI_VYBER_VICE_DEF :
                $typGrafu = 'bar';
                break;

            case TypOdpovedi::ZPUSOB_ODPOVEDI_VLASTNI :
                $typGrafu = '';
        }

        $graphSourceClass = $typGrafu ? 'graph-source' : '';
        echo "<div class='padding-left-3 oddEvenChild $graphSourceClass' data-graph-type='$typGrafu' data-graph-target='#graph_target_{$otazkaDotazniku ->kod}'>\n";

        foreach ($otazkaDotazniku ->volby as $volba) {
            $popis = StringUtils::htmlTextConvert(strip_tags($volba ->popis));
            $hodnota = $this ->statistikaOdpovedi[$otazkaDotazniku ->kod] ->pocetOdpovedi[$volba ->hodnota] ?? 0;

            echo
                "<div class='row graph-value padding-all' data-graph-label='$popis'  data-graph-value='$hodnota'>" .
                '<div class="col-11">' . $volba ->popis . '</div>' .
                '<div class="col-1">' . $hodnota . '</div>' .
                "</div>\n";
        }

        if ($otazkaDotazniku ->zpusobHodnoceni === TypOdpovedi::ZPUSOB_ODPOVEDI_VLASTNI) {
            foreach ($this ->statistikaOdpovedi[$otazkaDotazniku ->kod] ->texty ?? [] as $text => $count) {
                echo
                    "<div class='row padding-all'>" .
                    '<div class="col-11">' . StringUtils::htmlTextConvert($text) . '</div>' .
                    '<div class="col-1">' . $count . '</div>' .
                    "</div>\n";
            }
        }

        echo "</div>\n";

        //Debugger::dump($this ->statistikaOdpovedi[$otazkaDotazniku ->kod]);
    }


    public function render()
    {
        foreach ($this ->dotaznik ->otazky as $otazkaDotazniku) {
            echo
                '<div class="form large-form">' .
                '<div class="form-content row vyhodnoceni-otazky flex-align-items-center">' .
                '<div class="col-12 col-sm-7">' .
                '<div>' . $otazkaDotazniku ->vratHTML_Otazka() . "</div>\n";

            $this ->renderOdpovedi($otazkaDotazniku);

            echo '</div>'; // col

            echo "<div class='col-12 col-sm-5' id='graph_target_{$otazkaDotazniku ->kod}'></div>\n";

            echo
                "</div>\n" . // form-content row
                "</div>\n";  // form
        }
    }

}

