<?php
/**
 * stanislav.elbl, 14.01.2016
 */

namespace RonPortal\Uchazeci;

use Ron\DB\Database;
use Ron\DB\SQLUtils;
use Ron\Http\HtmlHeaders;
use Ron\MessageManager;
use Ron\MVC\BaseViewObject;
use Ron\MVC\HttpController;
use Ron\MVC\Model;
use Ron\MVC\OutputObject;
use Ron\MVC\Redirect;
use Ron\MVC\ViewObject;
use Ron\TextManager;
use RonPortal\Dokumenty\DokumentOutputObject;
use RonPortal\NeniPravoException;
use RonPortal\Prava;
use RonPortal\RonPortalDBRecord;
use RonPortal\RonPortalSettings;
use RonPortal\RonPortalUtils;
use RonPortal\System\FotoOutputObject;
use RonPortal\User;



class UchazeciController extends HttpController
{
    /**
     * @var Database
     */
    private $database;
    /**
     * @var \RonPortal\PravaNaOsoby
     */
    private $prava;
    /**
     * @var \RonPortal\PravaNaOsoby
     */
    private $pravaKomunikace;

    /**
     * UchazeciController constructor.
     * @param Database $database databaze
     */
    public function __construct(Database $database)
    {
        $this->database = $database;
        $this->pravaKomunikace = Prava::vratPrava('uchazeci_komunikace');
        $this->prava = Prava::vratPrava('uchazeci_prohl');
        Prava::nastavObdobiProPrava(time(),time());
    }


    /**
     * @param $rc
     * @return BaseViewObject BaseViewObject
     * @throws \Exception
     */
    public function uchazecView($rc)
    {
        $uchazec = Uchazec::nactiUchazece($this->database,$rc);

        Model::addData('uchazec', $uchazec);
        Model::addData('pravaKomunikace', $this->pravaKomunikace);

        return new BaseViewObject('ronportal/uchazeci/view_uchazec.php');
    }


    /**
     * @return KomunikaceUchazeceForm
     * @throws \Exception
     */
    public function uchazeciKomunikaceForm()
    {

        if (!($this->pravaKomunikace->jsouNejakePrava())) {
            throw new NeniPravoException();
        }

        $formAction = isset($_POST['formaction'])
            ? $_POST['formaction']
            : $_GET['formaction'];

        switch ($formAction) {
            case 'new' :
                $uchazec = Uchazec::nactiUchazece($this->database,$_GET['uchazec']);
                break;

            case 'edit' :
                $komunikace = KomunikaceUchazece::nactiZaznamKomunikace($this ->database, (int)$_GET['record']);
                if (User::getRC() != $komunikace ->personalistaRC) {
                    throw new NeniPravoException();
                }
                $uchazec = Uchazec::nactiUchazece($this->database, $komunikace ->rcUchazece);
                break;

            case 'insert' :
                $this->vytvorPersonalistu(); // vytvori personalistu z prihlasene osoby
                $uchazec = Uchazec::nactiUchazece($this->database, $_POST['RCUchazece']);
                break;

            case 'save' :
                $komunikace = KomunikaceUchazece::nactiZaznamKomunikace($this ->database, (int)$_POST['IdKomunikace']);
                if (User::getRC() != $komunikace ->personalistaRC) {
                    throw new NeniPravoException();
                }
                $uchazec = Uchazec::nactiUchazece($this->database, $komunikace ->rcUchazece);
                break;

            default:
                throw new \Exception('Chyba v UchazeciController::uchazeciKomunikaceForm!');
        }

        $f = new KomunikaceUchazeceForm($this ->database, $uchazec);

        if ($f ->handleAction()) {
            return new Redirect(null, '?menuid=uchazeci&action=zobrazuchazece&record='.$uchazec->rc);
        }
        return $f;
    }

    /**
     * @param $rc
     * @param $id
     * @return DokumentOutputObject
     * @throws \Exception
     */

    public function downloadDocument($rc, $id)
    {
        // zkontroluje, ze je dokument opravdu pripojen k uchazeci
        $params = $this->database->createParameters();

        $sql =
            'SELECT ID' .
            ' FROM DokumentUchazece ' .
            ' WHERE DokumentUchazece.ID = ' . $params->addParameter((int)$id) .
            ' AND IDUchazece = ' . $params->addParameter($rc);

        MessageManager::addDebugLog($sql);

        $cursor = $this->database->getSqlResult($sql, $params);
        $row = $cursor->fetch();
        $cursor ->close();

        if (!$row){
            throw new NeniPravoException();
        }

        return new DokumentOutputObject($this->database,$id);
    }

    /**
     * @param string $rc
     * @return FotoOutputObject
     */
    public function downloadFoto($rc)
    {
        $result = new FotoOutputObject($this->database);
        $result->nactiIdFotkyOsoby($rc,'P_Uchazec');
        return $result;
    }

    private function zapisZobrazeniUchazece($rc)
    {
        $recHistorie = new RonPortalDBRecord($this->database,'HistorieUchazece', $this);

        $recHistorie->addField('ID','int',false,0,SQLUtils::newSerialNumber($this->database, 'HistorieUchazece', 'ID'));
        $recHistorie->addField('RCUchazece', '', false,15,$rc);
        $recHistorie->addField('Cas', 'timestamp', false, 0, time());
        $recHistorie->addField('Popis', 'memo', false, 0,  sprintf(\Ron\ronText('Uchazec_zobrazeni'),User::getUserName()));
        $recHistorie->addField('Insert_UserName', '', false, 25, RonPortalUtils::getLogUserName());
        $recHistorie->addField('Insert_Timestamp','timestamp',false,0,time());

        $recHistorie->insertRecord();
    }



    private function vytvorPersonalistu()
    {
        $params = $this->database->createParameters();
        $sql =
            'SELECT Jmeno, Prijmeni, Titul, TitulZa' .
            ' FROM Osoba WHERE RC = ' . $params->addParameter(User::getRC());

        $cursor = $this->database->getSqlResult($sql,$params);

        if (!$row = $cursor->fetch()) {
            throw new \Exception( TextManager::_('Prihlaseny_uzivatel_nenalezen') );
        }
        $cursor ->close();

        $recPersonalista = new RonPortalDBRecord($this->database,'P_Osoba', $this);
        $recPersonalista->addStringField('RC', false, 15, User::getRC());
        $recPersonalista->addStringField('Jmeno', false, 30, $row[0]);
        $recPersonalista->addStringField('Prijmeni', false, 30, $row[1]);
        $recPersonalista->addStringField('Titul', false, 20, $row[2]);
        $recPersonalista->addStringField('TitulZa', false, 20, $row[3]);
        $recPersonalista->addStringField('Insert_UserName', false, 25, RonPortalUtils::getLogUserName());
        $recPersonalista->addTimestampField('Insert_Timestamp', false, time());

        $params = $this ->database ->createParameters();
        $recPersonalista->setWhere(
            'RC = ' . $params ->addParameter(User::getRC()),
            $params);;

        if (!$recPersonalista->loadRecord()) {
            $recPersonalista->insertRecord();
        }
    }


    /**
     * @param string $action
     * @return ViewObject|OutputObject|Redirect
     * @throws \Exception
     */
    public function handleAction($action)
    {
    	if (!(($this->prava->jsouNejakePrava() || $this->pravaKomunikace->jsouNejakePrava()) && RonPortalSettings::getIntValue(RonPortalSettings::POVOLIT_UCHAZECE))) {
    		throw new NeniPravoException();
    	}

        HtmlHeaders::getInstance()->addCss('./css/person.css');
        TextManager::getInstance() ->loadStrings('person.ini');
        HtmlHeaders::getInstance() ->setTitle(TextManager::_('RON_Portal') . ' - ' . TextManager::_('Uchazeci'));

        if (isset($_GET['action'])) {
            switch ($_GET['action']) {
                case 'download':
                    return $this->downloadDocument($_GET['record'], $_GET['dokument']);

                case 'downloadfoto':
                    return $this->downloadFoto($_GET['record']);

                case 'zobrazuchazece':
                    $this->zapisZobrazeniUchazece($_GET['record']);
                    return $this->uchazecView($_GET['record']);

                case 'komunikace':
                    return $this->uchazeciKomunikaceForm();
            }
        }

        $b = new UchazeciBrowser($this ->database);
        $b ->load();
        return $b;
    }
}
