<?php
/**
 * stanislav.elbl, 21.02.2021
 */

namespace RonPortal\Vyroba;

use Ron\DateUtils;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Dochazka\DochazkaBrowser;
use RonPortal\Kalendar;
use RonPortal\RonPortalUtils;

class PuvodniVyrobaOsoby extends DochazkaBrowser
{
    protected $format = '%CAS% - %POPIS%';
    protected $titleFormat = '';

    function __construct($database)
    {
        parent::__construct($database, 'puvodnivyroba');

        $this->title = TextManager::_('Puvodni_vyroba');

        $this->cssClass = 'browser grid browserdenni puvodnivyroba mobile-table-rows';
        $this->code = 'browserpuvodnivyroba';

        $this->tableHtmlParams = [
            'data-variables' => '%CAS%,%POPIS%,%KOD%,%TERMINAL%,%TERMINAL_POPIS%,%VYROBEK%,%ZAKAZKA%,%VYROBNICISLO%,%MNOZSTVIDOBRE%,%MNOZSTVISPATNE%,%HODINY%,%ROZDELENI%,%POZNAMKA%'
        ];

        $this ->filterFormRights = 'vyroba';
        $this ->zobrazitPoznamky = false;
    }


    /**
     * Nacteni konfiguracniho parametru z XML - umoznuje navic nastavit formaty pro zobrazeni
     * @param $key
     * @param $value
     * @param $valueText
     * @return void
     */
    public function loadConfigurationItem($key, $value, $valueText)
    {
        switch ($key) {
            case 'format' :
            case 'titleFormat' :
                $this ->{$key} = $valueText;
                break;

            default :
                parent::loadConfigurationItem($key, $value, $valueText);
        }
    }


    function nactiData()
    {
        if ($this ->filterForm ->getFilterValue('osoba') === null) { // kvuli firebirdu
            return null;
        }

        $params = $this ->database ->createParameters();
        $filter = $this ->filterForm ->buildSqlFilter($params);

        $sql =
            'SELECT PuvodniVyroba.DatumCas, VyrobniCinnost.Popis, PuvodniVyroba.KodVyrobniCinnosti, PuvodniVyroba.Terminal, Terminal.Popis,'. // 0..4
            ' PuvodniVyroba.Vyrobek, PuvodniVyroba.VyrobniZakazka, PuvodniVyroba.VyrobniCislo,' . // 5..7
            ' PuvodniVyroba.MnozstviDobre, PuvodniVyroba.MnozstviSpatne, PuvodniVyroba.Hodiny, PuvodniVyroba.Rozdeleni, PuvodniVyroba.Poznamka' . // 8..12
            "\nFROM PuvodniVyroba INNER JOIN VyrobniCinnost".
            '    ON PuvodniVyroba.KodVyrobniCinnosti = VyrobniCinnost.Kod' .
            "\n LEFT OUTER JOIN Terminal" .
            '    ON Terminal.Kod = PuvodniVyroba.Terminal' .
            "\nWHERE " . ($filter ? $filter . ' AND ' : '') .
            "   RC = " . $params ->addParameter($this ->filterForm ->getFilterValue('osoba', 'rc')) .
            '   AND OsobniCislo = ' . $params ->addParameter($this ->filterForm ->getFilterValue('osoba', 'osobniCislo')) .
            '   AND DatumNastupu = ' . $params ->addDateParameter($this ->filterForm ->getFilterValue('osoba', 'datumNastupu')) .
            '   AND DatumCas >= ' . $params ->addDateParameter($this ->filterForm ->getFilterValue('obdobi')) .
            '   AND DatumCas < ' . $params ->addDateParameter(DateUtils::incDay($this ->filterForm ->getFilterValue('obdobi', 'datumDo'), 1)) .
            ' ORDER BY DatumCas';

        return $this ->database ->getSqlResult($sql, $params);
    }


    private function formatData($zaznam, $format)
    {
        return str_replace(
            [
                '%CAS%', '%POPIS%', '%KOD%', '%TERMINAL%', '%TERMINAL_POPIS%', '%VYROBEK%', '%ZAKAZKA%', '%VYROBNICISLO%',
                '%MNOZSTVIDOBRE%', '%MNOZSTVISPATNE%', '%HODINY%', '%ROZDELENI%', '%POZNAMKA%'
            ],
            [
                date('G:i', $this ->database ->stringToTimestamp($zaznam[0])),
                StringUtils::htmlTextConvert($zaznam[1]),
                StringUtils::htmlTextConvert($zaznam[2]),
                StringUtils::htmlTextConvert($zaznam[3]),
                StringUtils::htmlTextConvert($zaznam[4]),
                StringUtils::htmlTextConvert($zaznam[5]),
                StringUtils::htmlTextConvert($zaznam[6]),
                StringUtils::htmlTextConvert($zaznam[7]),
                StringUtils::format('number_1', $zaznam[8]),
                StringUtils::format('number_1', $zaznam[9]),
                StringUtils::format('hodiny', $zaznam[10]),
                StringUtils::htmlTextConvert($zaznam[11]),
                StringUtils::htmlTextConvert($zaznam[12])
            ],
            $format
        );
    }

    function vratTextKZaznamu($zaznam, $posledni)
    {
        return $this ->formatData($zaznam, $this ->format);
    }

    function vratTitleKZaznamu($zaznam)
    {
        return $this ->formatData($zaznam, $this ->titleFormat);
    }

    public function getTableAttributes()
    {
        $result = parent::getTableAttributes() ?? [];
        $result['data-format'] = $this ->format;
        $result['data-title-format'] = $this ->titleFormat;

        return $result;
    }
}
