<?php
/**
 * User: pavel.rudol
 * Date: 31.1.2019
 * Time: 11:31
 */

namespace RonPortal\Vyroba;

use Ron\DB\Database;
use Ron\DB\Parameters;
use Ron\MessageManager;
use RonPortal\PravaNaOsoby;


class VysledekVyroby
{
    public $id;
    public $vyroba_serial;

    public $cisloZakazky;
    public $datumCasOd;
    public $datumCasDo;
    public $hodiny;

    /**
     * @param Database $database
     * @param string $sql
     * @param Parameters $params
     * @return array
     * @throws \Exception
     */
    private static function loadSql($database, $sql, $params)
    {
        $cursor = $database->getSqlResult($sql, $params);

        $vysledkyVyroby = [];
        $vysledky = [];
        $vyrobaSerial = 0;

        while ($row = $cursor->fetch()) {
            if ($vyrobaSerial && $vyrobaSerial != $row[0]) {
                $vysledkyVyroby[$vyrobaSerial] = $vysledky;
                $vysledky = [];
            }

            $vysledek = new VysledekVyroby();
            $vysledek->id = $row[1];
            $vysledek->vyroba_serial = $row[0];
            $vysledek->cisloZakazky = $row[2];
            $vysledek->datumCasOd = $database->stringToTimestamp($row[3]);
            $vysledek->datumCasDo = $database->stringToTimestamp($row[4]);
            $vysledek->hodiny = (float)$row[5];

            $vysledky[] = $vysledek;

            $vyrobaSerial = $row[0];
        }

        if (!empty($vysledky))
            $vysledkyVyroby[$vyrobaSerial] = $vysledky;

        return $vysledkyVyroby;
    }

    /**
     * @param Database $database
     * @param int $den
     * @param PravaNaOsoby $prava
     * @return array
     * @throws \Exception
     */
    public static function nactiVysledkyVyrobyProDenniEditaci($database, $den, $prava, $filterForm)
    {
        $params = $database->createParameters();

        $sql =
            "SELECT Vyroba.serial, VysledekVyroby.ID, VysledekVyroby.cisloZakazky, VysledekVyroby.DatumCasOd, VysledekVyroby.DatumCasDo, VysledekVyroby.hodiny ". // 0 .. 5

            "FROM PracovniPomer " .
            " INNER JOIN Osoba ON (Osoba.RC = PracovniPomer.RC) ".
            " INNER JOIN Vyroba ON (PracovniPomer.RC = Vyroba.RC AND PracovniPomer.OsobniCislo = Vyroba.OsobniCislo AND PracovniPomer.DatumNastupu = Vyroba.DatumNastupu " .
            "   AND den = " . $params->addParameter($database->dateToParam($den)) . ") " .
            " INNER JOIN VysledekVyroby ON (VysledekVyroby.vyroba_serial = Vyroba.serial)" .

            " WHERE 1 = 1" .

            " AND " . ZaznamEditaceVyroby::sqlFiltrOsoby($database, $params, $prava, $filterForm, $den);


        $sql .=
            " ORDER BY PracovniPomer.ID, Vyroba.serial";

        MessageManager::addDebugLog('Vysledky vyroby: ' . $sql);

        return self::loadSql($database, $sql, $params);
    }
}