<?php
/**
 * stanislav.elbl, 28.01.2019
 */

namespace RonPortal\XML\Classes;

use Exception;
use Ron\DB\DBException;
use Ron\RonPortal\XML\XMLForm;
use Ron\StringUtils;
use RonPortal\Controls\FilterFormFilterDny;
use RonPortal\Controls\FilterFormFilterObdobi;
use RonPortal\Controls\FilterFormFilterPP;
use RonPortal\Controls\FilterFormFilterRozdeleni;
use RonPortal\Controls\RonPortalFilterForm;
use RonPortal\Prava;
use RonPortal\XML\XMLFormElement;

class XMLRonPortalFilterForm extends RonPortalFilterForm
{
    /** @var XMLForm */
    private $xmlForm;

    /**
     * @param XMLForm $xmlForm
     */
    public function __construct(XMLForm $xmlForm)
    {
        parent::__construct($xmlForm ->database, ['menuid' => 'xml', 'record' => $xmlForm->idSablony],true);
        $this->xmlForm = $xmlForm;
    }


//    public function buildSqlPodrobnostiDnu()
//    {
//        $result = $this ->sqlPodrobnostiDnu;
//        if ($result) {
//            $result = $this ->xmlForm ->nahradPromenne($result, 'sql');
//        }
//
//        return $result;
//    }


    protected function afterLoadFilter($filter)
    {
        parent::afterLoadFilter($filter);

        if ($filter instanceof FilterFormFilterObdobi) {
//            echo " NASTAVUJI OBDOBI {$filter->obdobiAplikace} " . $filter ->getValue();
//            var_dump($filter ->getValue());
            $this->xmlForm->nastavObdobiAplikace('OBDOBI', $filter->obdobiAplikace, $filter ->getValue());
        } else if ($filter instanceof FilterFormFilterRozdeleni) {
            $rozdeleniFilterName = strtoupper($filter ->name);
            $this->xmlForm->nastavRozdeleni($rozdeleniFilterName, $filter->getTypRozdeleni(), $filter->getValue());

            if ($filter ->getShowVcetnePodrizenych()) {
                $this->xmlForm ->setVariable($rozdeleniFilterName . '_vcetnepodrizenych', $filter->getValue('vcetnePodrizenych'));
            }
        } else if ($filter instanceof FilterFormFilterPP) {
            $this->xmlForm->nastavPracovniPomer('OSOBA', $filter ->getValue());
//        } else if ($filter === 'osoba') {  - ODSTRANENO - filtr osoba uz je predelan na FilterFormFilter (tj. predchozi varianta)
//            $this->xmlForm->nastavPracovniPomer('OSOBA', (int)$this ->vybranaOsoba);
        } else {
//            var_dump($filter);
            $this->xmlForm->setVariable(strtoupper($filter ->name), $filter ->getValue());
        }



//        if ($zobrazitDnyObdobi) {
//            $this ->setVariableValue('DATUM', $this ->filterForm ->getFilterValue('datum'));
//        }
    }


}



class FilterFormFilterDnyXml extends FilterFormFilterDny
{
    /** @var XMLForm */
    private $xmlForm;

    public function __construct(RonPortalFilterForm $filterForm, XMLForm $xmlForm)
    {
        parent::__construct($filterForm, 'datum');
        $this ->xmlForm = $xmlForm;
    }


    protected function buildSqlPodrobnostiDnu($database)
    {
        $result = parent::buildSqlPodrobnostiDnu($database);


        if ($result) {
            $result = $this ->xmlForm ->nahradPromenne($result, 'sql');
        }

        return $result;
    }
}


/**
 * implementace tagu <adsfilterform>
 * Class XMLElementADSFilterForm
 * @package Ron\RonPortal\XML
 */
class XMLElementADSFilterForm extends XMLFormElement
{
    /** @var RonPortalFilterForm */
    public $filterForm;


    /**
     * @throws DBException
     * @throws Exception
     */
    private function createFilterForm()
    {
        $this->filterForm = new XMLRonPortalFilterForm($this ->xmlForm,
            ['menuid' => 'xml', 'record' => $this->xmlForm->idSablony],
            true
        );
//        if ($this ->getBoolAttribute('useget')) {      - upraveno 10-2023 - vzdy pouzije GET
//            $this ->filterForm ->setUseGet(true, ['menuid' => 'xml', 'record' => $this->xmlForm->idSablony]);
//        }

        $autoSubmit = $this->getAttributeValue('autosubmit');
        $this->filterForm->autoSubmit = (!isset($autoSubmit) || $autoSubmit === '')
            ? 1
            : self::getBoolValue($autoSubmit);

        // vnorene tagy adsfilterform_obdobi atd. vyhledaji tento element a nastavi filterForm pro zobrazeni
        $this->getElementsHTMLOutput();

        $zobrazitDnyObdobi = $this ->getBoolAttribute('zobrazitdnyobdobi');
        if ($zobrazitDnyObdobi) {
            $filterDny = new FilterFormFilterDnyXml($this ->filterForm, $this ->xmlForm);

            if ($varName = $this ->getAttributeValue('sqlvariablepodrobnostidnu')) {
                $filterDny ->setSqlPodrobnostiDnu($this ->xmlForm ->getVariableValue($varName));
            } else {
                $filterDny ->setSqlPodrobnostiDnu($this ->getAttributeValue('sqlpodrobnostidnu'));
            }
            $filterDny ->setNadpisPodrobnostiDnu($this ->nahradPromenne($this ->getAttributeValue('nadpispodrobnostidnu')));

            $this ->filterForm ->addFilter($filterDny);
        }
//        $this ->filterForm ->zobrazitDnyObdobi = $zobrazitDnyObdobi;
//        if ($varName = $this ->getAttributeValue('sqlvariablepodrobnostidnu')) {
//            $this ->filterForm ->sqlPodrobnostiDnu = $this ->xmlForm ->getVariableValue($varName);
//        } else {
//            $this ->filterForm ->sqlPodrobnostiDnu = $this ->getAttributeValue('sqlpodrobnostidnu');
//        }
//        $this ->filterForm ->nadpisPodrobnostiDnu = $this ->nahradPromenne($this ->getAttributeValue('nadpispodrobnostidnu'));

        $this->filterForm->loadData();

//        // doplni promenne do XML formulare
//        $filtrObdobi = $this ->filterForm ->getFilter('obdobi');
//        if ($filtrObdobi instanceof FilterFormFilterObdobi  &&  $filtrObdobi ->enabled) {
//            $this->xmlForm->nastavObdobiAplikace('OBDOBI', $filtrObdobi->obdobiAplikace, $this ->filterForm ->vybraneObdobiOd);
//
//            if ($zobrazitDnyObdobi) {
//                $this ->setVariableValue('DATUM', $this ->filterForm ->getFilterValue('datum'));
//            }
//        }

//        foreach ($this ->filterForm->getRozdeleniFilters() as $rozdeleniFilter) {
//            $rozdeleniFilterName = strtoupper($rozdeleniFilter ->name);
//            $this->xmlForm->nastavRozdeleni($rozdeleniFilterName, $rozdeleniFilter->getTypRozdeleni(), $rozdeleniFilter->getValue());
//
//            if ($rozdeleniFilter ->getShowVcetnePodrizenych()) {
//                $this->setVariableValue($rozdeleniFilterName . '_vcetnepodrizenych', $rozdeleniFilter->getValue('vcetnePodrizenych'));
//            }
//
//        }

//        if ($this->filterForm->showOsoby) {
//            $this->xmlForm->nastavPracovniPomer('OSOBA',
//                $this ->filterForm ->vsechnyOsoby ?
//                    null :
//                    $this->filterForm ->vybranaOsoba);
//        }

        $this->setVariableValue('adsfilterformsubmit', isset($_POST['adsfilterform']) ? (int)$_POST['adsfilterform'] : 0);
    }


    /**
     * @throws DBException
     */
    public function getHTMLOutput()
    {
        $this ->createFilterForm();

        // zobrazeni filtru zajisti formular - vyrenderuje az zaroven s formularem
        $this ->xmlForm ->ronPortalFilterForm = $this ->filterForm;

        return '';
    }


    /**
     * Provadi se i v rezimu process, aby se spravne nastavily promenne
     * @throws DBException
     */
    public function process()
    {
        $this ->createFilterForm();
    }
}


/**
 * Implementace tagu <adsfilterform_...> pro nastaveni formulare (podrizene tagy <adsfilterform>)
 * Class XMLElementADSFilterFormPart
 * @package RonPortal\XML\Classes
 */
class XMLElementADSFilterFormPart extends XMLFormElement
{
    /** @var string Filtr pro seznam rozdeleni */
    private $sqlFiltr;

    public function loadXMLElement($xmlReader)
    {
        parent::loadXMLElement($xmlReader);

        switch ($this ->elementName) {
            case 'adsfilterform_rozdeleni' :
            case 'adsfilterform_rozdeleni2' :
            case 'adsfilterform_osoby' :
            case 'adsfilterform_obdobi' :
                $this ->sqlFiltr = StringUtils::fromUTF8($this ->loadXMLValue($xmlReader) );
                break;
        }
    }


    private function addFilterRozdeleni($filterForm)
    {
        $name = $this ->nahradPromenne($this ->getAttributeValue('name'));
        $typRozdeleni = (int)$this ->nahradPromenne($this ->getAttributeValue('typrozdeleni'));

        /** @var FilterFormFilterRozdeleni $rozdeleniFilter */
        if ($typRozdeleni || $name) {
            $rozdeleniFilter = $filterForm->addFilterRozdeleni(
                (new FilterFormFilterRozdeleni($filterForm, $name ?: ($this ->elementName === 'adsfilterform_rozdeleni2' ? 'rozdeleni2' : 'rozdeleni')))
                    ->setTypRozdeleni($typRozdeleni));
        } else if ($this ->elementName === 'adsfilterform_rozdeleni2') {
            $rozdeleniFilter = $filterForm ->addFilterRozdeleni2();
        } else {
            $rozdeleniFilter = $filterForm ->addFilterRozdeleni1();
        }

        if (!$rozdeleniFilter) {
            return;
        }

        $jenVlastniStr = $this ->getAttributeValue('pouzevlastni');
        if ($jenVlastniStr !== null  &&  $jenVlastniStr !== '') {
            $rozdeleniFilter ->setJenVlastniRozdeleni((int)$jenVlastniStr);
        }

        $typPrava = $this ->nahradPromenne($this ->getAttributeValue('prava'));
        if ($typPrava) {
            $rozdeleniFilter->setPrava(Prava::vratPrava($typPrava));
        }

        $caption = $this ->nahradPromenne($this ->getAttributeValue('caption'));
        if ($caption) {
            $rozdeleniFilter ->setCaption($caption);
        } else {
            $rozdeleniFilter ->setPouzitPopisTypuRozdeleni($this ->getBoolAttribute('pouzitpopistypurozdeleni', true ));
        }

        $rozdeleniFilter
            ->setShowVcetnePodrizenych($this ->getBoolAttribute('showvcetnepodrizenych'))
            ->setOptionsSqlFilter(trim($this ->nahradPromenneSQL($this ->sqlFiltr)))
            ->setCaption2($this ->nahradPromenne($this ->getAttributeValue('caption2')));
    }


    public function process()
    {
        // nastavi nejblizsi nadrazeny adsfilterform
        /* @var  XMLElementADSFilterForm $filterFormElement*/
        $filterFormElement = $this ->findParentByName('adsfilterform');
        if (!$filterFormElement) {
            return;
        }

        $sqlFiltr = trim($this ->nahradPromenneSQL($this ->sqlFiltr));

        switch ($this ->elementName) {
            case 'adsfilterform_obdobi' :
                $filterFormElement ->filterForm ->addFilter(
                    (new FilterFormFilterObdobi(
                        $filterFormElement ->filterForm,
                        $this ->nahradPromenne($this ->getAttributeValue('aplikace') ?? 'ADS')))
                    ->setJenOtevreneObdobi(self::getBoolValue($this ->getAttributeValue('jenotevrene')))
                    ->setOptionsSqlFilter($sqlFiltr)
                    ->setRequired(true),
                    true
                );
//                $filterFormElement ->filterForm ->nastavObdobi(true,
//                    $this ->nahradPromenne($this ->getAttributeValue('aplikace')),
//                    self::getBoolValue($this ->getAttributeValue('jenotevrene'))
//                );
//
//                $filterFormElement ->filterForm ->getFilter('obdobi') ->setOptionsSqlFilter($sqlFiltr);

                break;

            case 'adsfilterform_rozdeleni' :
            case 'adsfilterform_rozdeleni2' :
                $this ->addFilterRozdeleni($filterFormElement ->filterForm);
                break;
//                $filterFormElement ->filterForm ->nastavRozdeleni(true,
//                    $typRozdeleni, (int)$this ->getAttributeValue('pouzevlastni'),
//                    $sqlFiltr,
//                    $this ->getBoolAttribute('pouzitpopistypurozdeleni'));
//                $typPrava = $this ->nahradPromenne($this ->getAttributeValue('prava'));
//                if ($typPrava) {
//                    $filterFormElement ->filterForm ->f_pravarozdeleni = Prava::vratPrava($typPrava);
//                }
//                $filterFormElement->filterForm->showVcetnePodrizenychRozdeleni = self::getBoolValue($this ->getAttributeValue('showvcetnepodrizenych'));
//                break;
//            case 'adsfilterform_rozdeleni2' :
//                $typRozdeleni = (int)$this ->nahradPromenne($this ->getAttributeValue('typrozdeleni'));
//                $filterFormElement ->filterForm ->nastavRozdeleni2(true,
//                    $typRozdeleni, self::getBoolValue($this ->getAttributeValue('pouzevlastni')),
//                    $sqlFiltr, $this ->getBoolAttribute('pouzitpopistypurozdeleni'));
//                $filterFormElement->filterForm->showVcetnePodrizenychRozdeleni2 = self::getBoolValue($this ->getAttributeValue('showvcetnepodrizenych'));
//                break;

            case 'adsfilterform_osoby' :
                $prava = $this ->nahradPromenne($this ->getAttributeValue('prava'));

//                $datumOd = $this->getAttributeValue('datumod');
//                $datumDo = $this->getAttributeValue('datumdo');

//                $datumOd = (int)$this->nahradPromenne($datumOd);
//                $datumDo = (int)$this->nahradPromenne($datumDo);

                if ($prava) {
                    $prava = Prava::vratPrava($prava);
                    //$filterFormElement ->filterForm ->nastavOsoby(true, $prava, $datumOd, $datumDo);
                    $filterFormElement ->filterForm ->addFilterPP(self::getBoolValue($this ->getAttributeValue('vsechnyosoby')), $prava);
                } else {
                    $predavaciProces = $this ->getAttributeValue('predavaciproces');
                    if ( !$predavaciProces ) {
                        $predavaciProces = $this ->xmlForm -> predavaciProces;
                    }
                    $filterFormElement ->filterForm ->addFilter(
                        (new FilterFormFilterPP($filterFormElement ->filterForm))
                            ->setPredavaciProces($predavaciProces, $this ->xmlForm -> typRozdeleniProcesu)
                    );
                    //$filterFormElement ->filterForm ->nastavOsobyDleProcesu(true, $predavaciProces, $this ->xmlForm -> typRozdeleniProcesu);
                }

                //$filterFormElement->filterForm->osobyFiltr = $sqlFiltr;
                $filterFormElement->filterForm->getFilter('osoba') ->setOptionsSqlFilter($sqlFiltr);
//
//                if (self::getBoolValue($this ->getAttributeValue('vsechnyosoby'))) {
//                    $filterFormElement ->filterForm ->povolitVsechyOsoby = true;
//                }
        }
    }


    public function getHTMLOutput()
    {
        $this ->process();

        return '';
    }
}

/*
//DOC
PAGE adsfilterform
<pre>
PRE
  <adsfilterform autosubmit="1">
      <adsfilterform_obdobi aplikace="ADS" jenotevrene="0"/>
      <adsfilterform_rozdeleni typrozdeleni="kodtypu" pouzevlastni="0/1/2" prava="typprav" pouzitpopistypurozdeleni="0" showvcetnepodrizenych="0">
         nepovinny_sql_filtr_pro_vyber_rozdeleni
      </adsfilterform_rozdeleni>
      <adsfilterform_rozdeleni2 typrozdeleni="kodtypu" pouzevlastni="0/1/2" pouzitpopistypurozdeleni="0" >
         nepovinny_sql_filtr_pro_vyber_rozdeleni
      </adsfilterform_rozdeleni2>
      <adsfilterform_osoby prava="prohlizeni" vsechnyosoby="1" datumod="" datumdo=""/>
  </adsfilterform>
PREEND
</pre>
//END
*/