<?php

namespace RonPortal\XML\Classes;

use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\XML\XMLFormElementInput;

/**
 * Implementace tagu <edit>
 * Class XMLFormElementEdit
 * @package RonPortal\XML\Classes
 */
class XMLElementEdit extends XMLFormElementInput
{
    protected $format;
    protected $defaultValue;
    protected $size;


    public function loadXML($xmlReader)
    {
        parent::loadXML($xmlReader); // TODO: Change the autogenerated stub

        $this->format = $this->getAttributeValue('format');
        $this->size = $this->getAttributeValue('size');

        //$this ->f_defaultvalue = $xmlReader ->hasValue ? iconv('utf-8', 'windows-1250', $xmlReader ->value) : '';
        //$this ->f_defaultvalue = iconv('utf-8', 'windows-1250', $xmlReader ->value);
        $this->defaultValue = $this->getTextContent(false);
    }


    public function getHTMLOutput()
    {
        $name = $this->nahradPromenne($this->name);
        $editing = $this ->nahradPromenne($this->getAttributeValue('editing')); // yes/no/append/nolog

        if ($this ->xmlForm ->dataMode()  &&  ($editing === 'no' ||  $editing == '0')  &&  $this ->variableValue($name) !== null) {
            // datove XML, needitovatelne pole, uz vyplnene
            return '<span class="dotaznik_odpoved">' . StringUtils::htmlTextConvert($this ->variableValue($name)) . '</span>';
        }

        $attrs = [
            'type' => 'text',
            'name' => $name
        ];

        $htmlParams = $this->nahradPromenne($this->htmlParams);
        $htmlConvert = true;

        if ($this->xmlForm->usePostValues && isset($_POST[$name])) {
            $value = $_POST[$name];

            if ( $this->format === 'datetime'  &&  $_POST[$name . '_time'] !== '') {
                $value .= ' ' . $_POST[$name . '_time'];
            }
        } else {
            $value = null;

            if ($this ->xmlForm ->dataMode()) {
                $value = $this ->variableValue($name);
            }

            if (!isset($value)) {
                $value = $this->defaultValue;
                // nahradi promenne - aby bylo mozno napr. prednastavit aktualni cas
                $value = $this->nahradPromenne($value, 'html'); // bez toho se neformatuji promenne (napr. number_2)
                $htmlConvert = false;
            }
        }

        $attrs['value'] = $htmlConvert ? StringUtils::htmlTextConvert($value) : $value;

        //questform_nastavpromennou($name, $this -> f_format, $value);
        $this->setVariableValueWithFormat($name, $this->format, $value);

        if ($this->xmlForm->readOnly && strpos($htmlParams, 'readonly') === false) {
            $attrs['readonly'] = 'readonly';
        }

        $cssClass = 'form-control';
        switch ($this->format) {
            case 'float' :
                $cssClass .= ' inputFloat';
                break;

            case 'int' :
            case 'integer' :
            case 'positive' :
                $cssClass .= ' inputInt';
                break;

            case 'time' :
                $attrs['size'] = '5';
                $cssClass = 'inputTime';
                break;

            case 'date' :
                $attrs['size'] = '10';
                $attrs['maxlength'] = '10';
                $cssClass = ' inputDate';
                break;

            case 'datetime' :
                $cssClass = '';
                break;
        }

        $attrs['class'] = $cssClass .
            ($this->xmlForm->isError($name)
                ? ' error'
                : ''
            );

        if ($this->size && (int)$this->size > 0) {
            $attrs['size'] = (int)$this ->nahradPromenne($this->size);
        }

        // doplni atributy oznacene prefixem html:
        $attrs = $this->setHTMLAttributes($attrs);
        $attrs_str = $this->getHTMLAttributes($attrs);

        switch ($this->format) {
            case 'date' :
                return
                    '<span>' .
                    "<input $attrs_str $htmlParams/>" .
                    '<i class="showcalendar inline-glyph glyph-button fa glyph-calendar" data-roncalendar-inputname="' . $name . '"></i>' .
                    '</span>';

            case 'datetime' :
                $poma = explode(' ', $attrs['value']);
                $readonlystr = isset($attrs['readonly']) ? 'readonly="' . $attrs['readonly'] . '"' : '';

                return
                    '<span>' .
                    '<input type="text" name="' . $attrs['name'] . '" class="' . $attrs['class'] . ' inputDate" maxlength="10"' .
                      ' size="10" value="' . ($poma ? $poma[0] : '') . '" ' . $htmlParams . '/>' .
                    '<i class="showcalendar inline-glyph glyph-button fa glyph-calendar" data-roncalendar-inputname="' . $name . '"></i>' .
                    '<input type="text" name="' . $attrs['name'] . '_time" class="' . $attrs['class'] . ' inputTime" ' . $readonlystr .
                      ' size="5" value="' . (isset($poma[1]) ? $poma[1] : '') . '" ' . $htmlParams . '/>' .
                    '</span>';

            default :
                return "<input $attrs_str $htmlParams/>";
        }
    }


    protected function getPostValue($name)
    {
        if ($this->format === 'datetime') {
            $value = '';

            if ($_POST[$name] != '' && $_POST[$name . '_time'] == '') {
                $value = $_POST[$name] . ' 00:00'; // neni zadan cas - doplni 0
            }
            else if ($_POST[$name] == '' && $_POST[$name . '_time'] != '') {
                $this->xmlForm->addErrorMsg(TextManager::_('Bad_date_format'), $name); // neni zadan datum
            }
            else if ($_POST[$name] != '') {
                $value = $_POST[$name] . ' ' . $_POST[$name . '_time'];
            }
        } else {
            $value = isset($_POST[$name]) ? $_POST[$name] : '';
        }

        if ($value === '') {
            if ($this->required) {
                $this->xmlForm->addErrorMsg( TextManager::_('Chybi_povinna_hodnota'), $name);
            }
        }

        return $value;
    }


    public function writeXMLOutput($xmlWriter)
    {
        $name = $this->nahradPromenne($this->name);
        $editing = $this ->nahradPromenne($this->getAttributeValue('editing')); // yes/no/append/nolog

        $value = $this ->getPostValue($name);

        $newDefaultValue = $this->defaultValue;
        $showedit = true;
        $logvalue = true;

        // nastavi hodnotu promenne
        //questform_nastavpromennou($name, $this -> f_format, $value);
        $this->setVariableValueWithFormat($name, $this->format, $value);

        if ($editing === 'append') {
            // pridava se dalsi text, vychozi hodnota je porad stejna jako v sablone
            if ($value == '') {
                $logvalue = false;
            }
            else if (!strcmp($value, $this->defaultValue)) {
                $logvalue = false;
            }
            // neni co logovat (byla zadana prazdna nebo vychozi hodnota)

            $newDefaultValue = $this->defaultValue;
        } else if ($editing === 'yes'  ||  $editing == '1') {
            $logvalue = (strcmp($value, $this->defaultValue)); // loguje v pripade zmeny
            $newDefaultValue = $value;
        } else if ($editing === 'nolog') {
            // editace je povolena a zadana hodnota se neloguje
            $logvalue = false;
            $newDefaultValue = $value;
        } else {
            $showedit = false;
            $logvalue = false;
            // needitovatelne pole netreba logovat
        }

        if ($logvalue) {
            $this->posledniOdpovedXML($xmlWriter, StringUtils::xmlTextConvert($value));
        }

        if ($showedit) {
            $xmlWriter->startElement(($this->prefix ? $this->prefix . ':' : '') . $this->elementName);
            $this->writeXMLAttributes($xmlWriter);
            $xmlWriter->text( StringUtils::toUTF8($newDefaultValue) );
            $xmlWriter->endElement();
        } else {
            if ($this->format === 'date' || $this->format === 'datetime' || $this->format === 'time') {
                $this ->writeXmlVariable($xmlWriter, $name . '_TIMESTAMP', $this->variableValue($name . '_TIMESTAMP'));
            }

            $this ->writeXmlVariable($xmlWriter, $name, $value);

            $xmlWriter->startElement('div');
            $xmlWriter->writeAttribute('class', 'dotaznik_odpoved');
            $xmlWriter->text( StringUtils::toUTF8($value) );
            $xmlWriter->endElement();
        }
    }


    public function process()
    {
        $name = $this->nahradPromenne($this->name);
        $editing = $this ->nahradPromenne($this->getAttributeValue('editing')); // yes/no/append/nolog

        if ($editing !== 'append'  &&  $editing !== 'yes'  &&  $editing != '1'  &&  $editing !== 'nolog') {
            if ($this ->variableValue($name) != null) {
                // neni editovatelny a uz ma vyplnenou hodnotu
                return;
            }
        }

        $value = $this ->getPostValue($name);
        $oldValue = $this ->variableValue($name);

        $this->setVariableValueWithFormat($name, $this->format, $value);
        if ($this ->dataVariable) {
            $this->xmlForm->addDataVariable($name);
        }

        if ($value != $oldValue) {
            // zaloguje zmenu
            $this ->xmlForm ->addLogData($name, $value);
        }
    }
}

/*
//DOC
PAGE edit
<pre>
PRE
  <edit name="jmeno" format="" required="0" editing="yes" size="60" datavariable="1">vychozi obsah</edit>
PREEND
</pre>

//END
*/