<?php
/**
 * stanislav.elbl, 04.10.2018
 */

namespace RonPortal\XML\Classes;

use Ron\TextManager;
use Ron\XmlLoader;
use RonPortal\Editace\EditaceMesicnichHromadna;
use RonPortal\XML\XMLElementAttribute;
use RonPortal\XML\XMLFormElement;

class XMLElementEditaceMesicnich extends XMLFormElement
{
    /** @var EditaceMesicnichHromadna */
    var $editace;

    var $settingsArray;


    function loadXML($xmlReader)
    {
        TextManager::getInstance()->loadStrings('dochazka.ini'); // obsahuje chybove hlasky

        $loader = new XmlLoader();
        $loader->convertFromUtf8 = true;
        $this->settingsArray = $loader->readXmlElement($xmlReader);

        $this->attributes = [];    // doplni standardni pole atributu

        if ($this->settingsArray['#attr']) {
            foreach ($this->settingsArray['#attr'] as $name => $value) {
                $this->attributes[] = new XMLElementAttribute(null, $name, $value);
            }
        }

        // TODO: vyresit parametr content!
    }


    private function getTypRozdeleni()
    {
        $typStr = $this ->getAttributeValue('typrozdeleni');
        if ($typStr == null) {
            return $this ->xmlForm ->kodTypuRozdeleni;
        }

        return (int)$this ->nahradPromenne($typStr);
    }


    private function getKodRozdeleni()
    {
        $attr = $this ->getAttributeValue('rozdeleni');
        $result = isset($attr)
            ? $this ->nahradPromenne($attr)
            : $this ->xmlForm ->rozdeleni;

        return $result === ''
            ? null
            : $result;
    }


    private function getEditace()
    {
        if ($this ->editace) {
            return $this ->editace;
        }

        // nahrazeni promennych v atributech hlavniho tagu
        if ($this ->settingsArray['#attr']) {
            foreach ($this->settingsArray['#attr'] as $name => $attr) {
                $this->settingsArray['#attr'][$name] = $this ->nahradPromenne($this->settingsArray['#attr'][$name]);
            }
        }
        // nahrazeni promennych v atributech sloupcu
        if ($this ->settingsArray['#children']) {
            foreach ($this ->settingsArray['#children'] as $index => $sloupec) { // => puvodne bylo by reference - to ale nefunguje pro ProxyArray
                if ($sloupec['element']['#attr']) {
                    foreach ($sloupec['element']['#attr'] as $name => $attr) {
                        $sloupec['element']['#attr'][$name] = $this ->nahradPromenne($sloupec['element']['#attr'][$name]);
                    }
                }

                $this ->settingsArray['#children'][$index] = $sloupec; // nahrada za puvodni kod by reference
            }
        }

        $this ->editace = new EditaceMesicnichHromadna($this -> xmlForm -> database,
            $this ->getTypRozdeleni(), $this ->getKodRozdeleni()
        );

        $this ->editace ->f_readonly = $this ->xmlForm->readOnly;

        $this ->editace ->loadSettings($this ->settingsArray);

        $typRozdeleni2 = (int)$this ->nahradPromenne($this ->getAttributeValue('typrozdeleni2') ?? '');
        if ($typRozdeleni2) {
            $rozdeleni2 = $this ->nahradPromenne($this ->getAttributeValue('rozdeleni2'));

            if ($rozdeleni2 != '') {
                $this ->editace ->typRozdeleni2 = $typRozdeleni2;
                $this ->editace ->rozdeleni2 = $rozdeleni2;
            }
        }

        return $this ->editace;
    }

    /**
     * Pouziva se jeste ve starych formularich bez nastaveneho mode
     * @deprecated
     */
    function zapisData()
    {
        $editace = $this ->getEditace();
        $editace->nactiSeznamOsobPodlePost($this ->xmlForm->pocatekObdobi, !$editace ->kontrolovatPrava);

        $editace->nactiLimityRozdeleni();

        if (!$editace->jeChyba) {
            if ($editace->kontrolaObdobi()) {
                if ($editace->zkontrolujLimity()) {
                    $editace->zapisData();
                    if ($editace->jeChyba) $this->xmlForm ->addErrorMsg('&nbsp;');
                } else {
                    $this->xmlForm->addErrorMsg(\Ron\ronText('Prekrocen_limit_MP'));
                }
            } else {
                $this ->xmlForm->addErrorMsg( \Ron\ronText('Obdobi_uzavreno') );
            }
        } else {
            $this ->xmlForm->addErrorMsg( \Ron\ronText('Spatna_hodnota') );
        }
    }


    /**
     * Nastavi promenne formulare podle nacteneho seznamu osob
     * @param EditaceMesicnichHromadna $editace
     */
    function nastavPromenne($editace)
    {
        $name = $this ->getAttributeValue('name');
        if (!$name) return;

        $osoby = [];
        if ($editace ->osoby) {
            foreach ($editace ->osoby as $osoba) {
                $osoby[] = $osoba ->idPP;
            }
        }

        $delimiter = $this ->getAttributeValue('delimiter', ',');

        $this ->setVariableValue($name, implode($delimiter, $osoby)); // retezec - seznam idpp oddelenych strednikem
        $this ->setVariableValue($name . '_pole', $osoby);
    }


    /**
     * @return string
     * @throws \Exception
     */
    function getHTMLOutput()
    {
        $editace = $this ->getEditace();
        $editace->filtrOsoby = $this ->xmlForm->nahradPromenne($editace->filtrOsoby, 'sql');

        if ($this ->xmlForm->neukladat == 2  &&  $this ->xmlForm ->usePostValues) {
            // stary neukladaci formular
            $this ->zapisData();
        } else if ($this ->xmlForm->usePostValues) {
            // fill, process (pokud nastane chyba)
            $editace->nactiSeznamOsobXML($this ->getTypRozdeleni(), $this ->getKodRozdeleni(), $this ->xmlForm->pocatekObdobi);
            $editace->nactiPost();
            //$this->editace->nactiSeznamOsobPodlePost($this ->xmlForm->pocatekObdobi, true); // povolitvse = true - lze editovat vsechny osoby

            $editace->zkontrolujLimity(); // aby se oznacily prekrocene hodnoty

            $editace->nactiLimityRozdeleni();
        } else {
            //$this ->editace -> f_vcetnepodrizenych = $vcetnepodrizenych;
            $editace->nactiSeznamOsobXML($this ->getTypRozdeleni(), $this ->getKodRozdeleni(), $this ->xmlForm->pocatekObdobi);

            $editace->zkontrolujLimity(); // aby se oznacily prekrocene hodnoty
        }

        $this ->nastavPromenne($editace);

        $result = $editace->vratObsahFormulareH(false); // tlacitkoulozit = false

        $exportVariableName = $this ->nahradPromenne($this ->getAttributeValue('exportvariablename'));
        if ($exportVariableName) {
            // nastavi oddelovac radku
            $delimiter = $this ->nahradPromenne($this ->getAttributeValue('exportrowdelimiter'));
            if (isset($delimiter)  &&  $delimiter !== '') {
                $editace ->exportRowDelimiter = $delimiter;
            }
            // nastavi oddelovac sloupcu
            $delimiter = $this ->nahradPromenne($this ->getAttributeValue('exportcoldelimiter'));
            if (isset($delimiter)  &&  $delimiter !== '') {
                $editace ->exportColDelimiter = $delimiter;
            }

            $exportData = $editace ->vratObsahFormulareT();
            $this ->setVariableValue($exportVariableName, $exportData);
        }

        return
            '<div>' . $result . '</div>';
    }


    function writeXMLOutput($xmlWriter)
    {
        throw new \Exception('Chyba - transformace tagu editacemesicnich neni implementovana!');
    }


    function process()
    {
        $editace = $this ->getEditace();
        $editace->filtrOsoby = $this ->xmlForm->nahradPromenne($editace->filtrOsoby, 'sql');

        $editace->nactiSeznamOsobXML($this ->getTypRozdeleni(), $this ->getKodRozdeleni(), $this ->xmlForm->pocatekObdobi);
        $editace->nactiPost();

        $this ->nastavPromenne($editace);

        if (!$editace->jeChyba) {
            if ($editace->kontrolaObdobi()) {
                if ($editace->zkontrolujLimity()) {
                    try {
                        $editace->zapisData();
                        if ($editace->jeChyba) {
                            $this->xmlForm->addErrorMsg('');
                        }
                    } catch (\Exception $e) {
                        $this ->xmlForm ->addErrorMsg($e);
                    }
                } else {
                    $this->xmlForm->addErrorMsg(\Ron\ronText('Prekrocen_limit_MP'));
                }
            } else {
                $this ->xmlForm->addErrorMsg( \Ron\ronText('Obdobi_uzavreno') );
            }
        } else {
            $this ->xmlForm->addErrorMsg( \Ron\ronText('Spatna_hodnota') );
        }
    }
}
