<?php
/**
 * stanislav.elbl, 16.05.2017
 */

namespace RonPortal\XML\Classes;

use Exception;
use Ron\DB\DBException;
use Ron\DB\DBUtils;
use Ron\DB\Parameters;
use Ron\EMail;
use Ron\MailManager;
use Ron\StringUtils;
use RonPortal\Log;
use RonPortal\User;
use RonPortal\XML\XMLFormElementCommand;
use RonPortal\ZastupyUtils;


class XMLEmailReceiver {
    public $rc;
    public $address;

    /**
     * XMLEmailReceiver constructor.
     * @param $rc
     * @param $address
     */
    public function __construct($address, $rc = null)
    {
        $this->rc = $rc;
        $this->address = $address;
    }
}

/**
 * Implementace tagu <email>
 * Class XMLFormElementEmail
 * @package RonPortal\XML\Classes
 */
class XMLElementEmail extends XMLFormElementCommand
{
    protected $to;
    /** @var string email zjisti z osoby */
    protected $toRC;
    /** @var int email zjisti z osoby pro dany PP */
    protected $toIdPP;
    protected $toTypRozdeleni;
    /** @var string email zjisti z rozdeleni ( bud je zadano typ/kod nebo zjisti typ z toTypRozdeleni ) */
    protected $toRozdeleni;
    /** @var string email zjisti z osob na dalsi urovni predavaciho procesu (v parametru bude zadano osoba/PP/rozdeleni pro rozliseni zda vyhledavat dle osoby, PP nebo rozdeleni formulare) */
    protected $toDalsiUroven;

    protected $from;
    protected $fromRC;
    protected $fromIdPP;

    protected $replyTo;
    protected $replyToRC;
    protected $replyToIdPP;

    protected $copyTo;
    protected $copyToRC;
    protected $copyToIdPP;

    protected $subject;
    protected $text;

    /** @var  int       hodnota 0/1/2/3  1=jen chyby, 2=vzdy, 3 = vzdy+SQL */
    protected $log;
    /** @var  string   promenna pro odeslani vysledku - nastavi se na 1 v pripade OK, 0 pri chybe */
    protected $variableName;
    protected $logSql;


    public function loadXML($xmlReader)
    {
        parent::loadXML($xmlReader);

        $this->to = StringUtils::fromUTF8($this->getAttributeValue('to'));
        $this->toRC = StringUtils::fromUTF8($this->getAttributeValue('torc'));
        $this->toIdPP = $this->getAttributeValue('toidpp');
        $this->toTypRozdeleni = $this->getAttributeValue('totyprozdeleni');
        $this->toRozdeleni = StringUtils::fromUTF8($this->getAttributeValue('torozdeleni'));
        $this->toDalsiUroven = StringUtils::fromUTF8($this->getAttributeValue('todalsiuroven'));

        $this->from = StringUtils::fromUTF8($this->getAttributeValue('from'));
        $this->fromRC = StringUtils::fromUTF8($this->getAttributeValue('fromrc'));
        $this->fromIdPP = $this->getAttributeValue('fromidpp');

        $this->replyTo = StringUtils::fromUTF8($this->getAttributeValue('replyto'));
        $this->replyToRC = StringUtils::fromUTF8($this->getAttributeValue('replytorc'));
        $this->replyToIdPP = $this->getAttributeValue('replytoidpp');

        $this->copyTo = StringUtils::fromUTF8($this->getAttributeValue('copyto'));
        $this->copyToRC = StringUtils::fromUTF8($this->getAttributeValue('copytorc'));
        $this->copyToIdPP = $this->getAttributeValue('copytoidpp');

        $this->subject = StringUtils::fromUTF8($this->getAttributeValue('subject'));

        $this->variableName = $this->getAttributeValue('variablename');

        $logAttr = $this->getAttributeValue('log'); // 0/1/2
        $this->log = (isset($logAttr) && $logAttr !== '')
            ? (int)$logAttr
            : 1;    // implicitne loguje v pripade chyby

        $this->text = $this->getTextContent(false);
    }


    /**
     * @param string $sql
     * @param Parameters $params
     * @return XMLEmailReceiver|null
     * @throws DBException
     */
    private function getReceiverFromSQL($sql, $params)
    {
        $row = DBUtils::getSqlValue($this->xmlForm->database, $sql, $params, true); // nacte cely radek

        if ($row  &&  $row[0]) {
            $this->logSql .= ($this->logSql ? "\n" : '') . $sql . "\n" . $params->getLogText();

            return new XMLEmailReceiver($row[0], $row[1]);
        }

        return null;
    }

    /**
     * Vraci email zadane Osoby
     * @param $rc
     * @return XMLEmailReceiver
     * @throws DBException
     */
    private function getEmailProRC($rc)
    {
        if (!$rc) {
            return null;
        }

        $params = $this->xmlForm->database->createParameters();
        $sql = 'SELECT eMail, RC FROM Osoba WHERE RC = ' . $params->addParameter($rc);
        return $this ->getReceiverFromSQL($sql, $params);
    }


    /**
     * Vraci email osoby se zadanym pracovnim pomerem
     * @param $idpp
     * @return null|XMLEmailReceiver
     * @throws DBException
     */
    private function getEmailProIdPP($idpp)
    {
        if (!$idpp) {
            return null;
        }

        $params = $this->xmlForm->database->createParameters();
        $sql =
            'SELECT DISTINCT Osoba.eMail, Osoba.RC FROM Osoba, PracovniPomer ' .
            ' WHERE Osoba.RC = PracovniPomer.RC AND PracovniPomer.Id = ' . $params->addParameter((int)$idpp);

        return $this ->getReceiverFromSQL($sql, $params);
    }


    /**
     * Vraci emailovou adresu (respektive objekt XMLEmailReceiver) pro zadane parametry
     * @param string $address Natvrdo zadana emailova adresa nebo #USER
     * @param string $rc RC osoby, pro kterou se vyhleda email (Osoba.Email)
     * @param string $idpp Id pracovniho pomeru, pro ktery se vyhleda email
     * @return null|XMLEmailReceiver
     * @throws DBException
     */
    private function getReceiver($address, $rc, $idpp)
    {
        $address = $this->nahradPromenne($address);
        if ($address === '#USER') {
            $receiver = $this ->getEmailProRC(User::getRC());
            if ($receiver) {
                return $receiver;
            }
        } else if ($address) {
            return new XMLEmailReceiver($address); // bez RC
        }

        // neni zadana natvrdo adresa - zkusi RC
        if ($rc) {
            $receiver = $this ->getEmailProRC($this->nahradPromenne($rc));
            if ($receiver) {
                return $receiver;
            }
        }

        // ... zkusi PP
        if ($idpp) {
            $receiver = $this ->getEmailProIdPP($this->nahradPromenne($idpp));
            if ($receiver) {
                return $receiver;
            }
        }

        return null;
    }


    /**
     * Vraci email zadaneho rozdeleni. Do pole RC nastavi vedouciho tohoto rozdeleni.
     * @param $rozdeleni
     * @param null $typRozdeleni
     * @return null|XMLEmailReceiver
     * @throws DBException
     */
    private function getReceiverRozdeleni($rozdeleni, $typRozdeleni = null)
    {
        $rozdeleni = $this->nahradPromenne($rozdeleni);

        if ($typRozdeleni) {
            $kodRozdeleni = $rozdeleni;
        } else {
            $poma = explode('/', $rozdeleni, 2);
            $typRozdeleni = (int)$poma[0];
            $kodRozdeleni = $poma[1];
        }

        if ($typRozdeleni) {
            $params = $this->xmlForm->database->createParameters();
            $sql =
                'SELECT eMail, VedouciRC FROM Rozdeleni ' .
                ' WHERE KodTypuRozdeleni = ' . $params->addParameter($typRozdeleni) .
                '   AND Kod = ' . $params->addParameter($kodRozdeleni);

            return $this ->getReceiverFromSQL($sql, $params);
       }

        return null;
    }

    /**
     * Vraci emailovou adresu vedouciho zadaneho rozdeleni
     * @param string $kodRozdeleni
     * @param int $typRozdeleni
     * @return XMLEmailReceiver
     * @throws DBException
     */
    private function getReceiverVedoucihoRozdeleni($kodRozdeleni, $typRozdeleni)
    {
        $kodRozdeleni = $this->nahradPromenne($kodRozdeleni);

        $params = $this->xmlForm->database->createParameters();
        $sql =
            'SELECT Osoba.eMail, Osoba.RC FROM Rozdeleni, Osoba ' .
            ' WHERE Rozdeleni.VedouciRC = Osoba.RC' .
            ' AND Rozdeleni.KodTypuRozdeleni = ' . $params->addParameter($typRozdeleni) .
            ' AND Rozdeleni.Kod = ' . $params->addParameter($kodRozdeleni);

        return $this ->getReceiverFromSQL($sql, $params);
    }


    /**
     * Vraci email rozdeleni zadane osoby
     * @param int $typRozdeleni
     * @param int $idPP
     * @param bool $podleVedouciho Pokud je true, tak vraci email vedouciho rozdeleni
     * @return null|XMLEmailReceiver
     * @throws DBException
     */
    private function getReceiverRozdeleniOsoby($typRozdeleni, $idPP, $podleVedouciho)
    {
        $params = $this->xmlForm->database->createParameters();
        $sql =
            "SELECT Rozdeleni.eMail, Rozdeleni.VedouciRC, Osoba.Email\n" .
            ' FROM PracovniPomer  INNER JOIN RozdeleniOsob' .
            '   ON RozdeleniOsob.RC = PracovniPomer.RC' .
            '   AND RozdeleniOsob.OsobniCislo = PracovniPomer.OsobniCislo' .
            "   AND RozdeleniOsob.DatumNastupu = PracovniPomer.DatumNastupu\n" .
            ' INNER JOIN Rozdeleni' .
            '   ON Rozdeleni.KodTypuRozdeleni = RozdeleniOsob.KodTypuRozdeleni' .
            "   AND Rozdeleni.Kod = RozdeleniOsob.Rozdeleni\n" .
            ' LEFT OUTER JOIN Osoba' .
            "   ON Osoba.RC = Rozdeleni.VedouciRC\n" .
            ' WHERE PracovniPomer.Id = ' . $params->addParameter($idPP) .
            '   AND RozdeleniOsob.PlatnostOd <= ' . $params->addParameter($this->xmlForm->database->dateToParam(time())) .
            '   AND (RozdeleniOsob.PlatnostDo IS NULL  OR  RozdeleniOsob.PlatnostDo >= ' . $params->addParameter($this->xmlForm->database->dateToParam(time())) . ')' .
            '   AND RozdeleniOsob.KodTypuRozdeleni = ' . $params->addParameter((int)$typRozdeleni);

        $row = DBUtils::getSqlValue($this->xmlForm->database, $sql, $params, true);
        $result = null;

        if ($row) {
            if ($podleVedouciho) {
                if ($row[2]) {
                    // email podle vedouciho rozdeleni
                    $result = new XMLEmailReceiver($row[2], $row[1]);
                }
            } else {
                if ($row[0]) {
                    // email podle rozdeleni
                    $result = new XMLEmailReceiver($row[0], $row[1]);
                }
            }
        }

        if ($result) {
            $this->logSql .= ($this->logSql ? "\n" : '') . $sql . "\n(" . $params->getLogText() . ')';
        }

        return $result;
    }


    /**
     * Vraci adresu podle rozdeleni (bud primo zadane rozdeleni, nebo rozdeleni osoby)
     * @return null|XMLEmailReceiver
     * @throws DBException
     */
    private function getReceiverPodleRozdeleni()
    {
        $typRozdeleni = $this->toTypRozdeleni ? (int)$this->nahradPromenne($this->toTypRozdeleni) : null;

        if ($this->toRozdeleni) {
            $receiver = $this ->getReceiverRozdeleni($this->toRozdeleni, $typRozdeleni);
            if ($receiver) {
                return $receiver;
            }
        }

        $toRozdeleniOsoby = $this->getAttributeValue('torozdeleniosoby');
        if ($toRozdeleniOsoby && $typRozdeleni) {
            $receiver = $this ->getReceiverRozdeleniOsoby($typRozdeleni, (int)$this->nahradPromenne($toRozdeleniOsoby), false);
            if ($receiver) {
                return $receiver;
            }
        }

        $toVedouciOsoby = $this->getAttributeValue('tovedouciosoby');
        if ($toVedouciOsoby && $typRozdeleni) {
            $receiver = $this ->getReceiverRozdeleniOsoby($typRozdeleni, (int)$this->nahradPromenne($toVedouciOsoby), true);
            if ($receiver) {
                return $receiver;
            }
        }

        $toVedouciRozdeleni = $this ->getAttributeValue('tovedoucirozdeleni');
        if ($toVedouciRozdeleni && $typRozdeleni) {
            $receiver = $this ->getReceiverVedoucihoRozdeleni($this ->nahradPromenne($toVedouciRozdeleni), $typRozdeleni);

            if ($receiver) {
                return $receiver;
            }
        }

        return null;
    }


    /**
     * @return null|XMLEmailReceiver
     * @throws DBException
     */
    private function getEmailAddressPodleProcesu()
    {
        $params = $this->xmlForm->database->createParameters();

        $sql =
            'SELECT O.eMail, O.RC FROM Osoba O ' .
            '   JOIN PracovniPomer PPOsoby ON (PPOsoby.RC = O.RC) ' .
            '   JOIN RozdeleniOsob PMOsoby ON (PMOsoby.RC = PPOsoby.RC AND PMOsoby.OsobniCislo = PPOsoby.OsobniCislo AND PMOsoby.DatumNastupu = PPOsoby.DatumNastupu) ' .
            '   JOIN PMProUrovne PMU ON (PMU.KodTypuRozdeleni = PMOsoby.KodTypuRozdeleni AND PMU.KodRozdeleni = PMOsoby.Rozdeleni) ' .
            '   JOIN PoradiPredavani ON (PoradiPredavani.KodUrovne = PMU.KodUrovne) ' .
            '   JOIN PredavaciProces ON (PredavaciProces.Kod = PoradiPredavani.KodProcesu) ' .
            'WHERE PMOsoby.PlatnostOd <= ' . $params->addParameter($this->xmlForm->database->dateToParam(time())) .
            '   AND (PMOsoby.PlatnostDo >= ' . $params->addParameter($this->xmlForm->database->dateToParam(time())) . '  OR   PMOsoby.PlatnostDo IS NULL) ' .

            '   AND PPOsoby.DatumNastupu <= ' . $params->addParameter($this->xmlForm->database->dateToParam(time())) .
            '   AND (PPOsoby.DatumUkonceni >= ' . $params->addParameter($this->xmlForm->database->dateToParam(time())) . ' OR PPOsoby.DatumUkonceni IS NULL) ' .

            '   AND PMU.KodUrovne = ' . $params->addParameter($this->xmlForm->kodUrovne) .
            '   AND PoradiPredavani.KodProcesu = ' . $params->addParameter($this->xmlForm->predavaciProces) .
            '   AND PoradiPredavani.PocatecniUroven = ' . $params->addParameter($this->xmlForm->pocatecniUroven) .
            '   AND PoradiPredavani.Poradi = ' . $params->addParameter($this->xmlForm->poradiPredavani) .

            '   AND ((PoradiPredavani.NekontrolovatHierarchii = 1) OR ' .
            '        (EXISTS(SELECT RZamestnance.RC ' .
            '               FROM RozdeleniOsob RZamestnance ' .
            '               JOIN HierarchieRozdeleni ON (HierarchieRozdeleni.typPodrizene = RZamestnance.KodTypuRozdeleni AND HierarchieRozdeleni.kodPodrizene = RZamestnance.Rozdeleni) ' .
            '               JOIN RozdeleniOsob ROsoby ON (ROsoby.KodTypuRozdeleni = HierarchieRozdeleni.typNadrazene AND ROsoby.Rozdeleni = HierarchieRozdeleni.kodNadrazene) ' .
            '               JOIN PracovniPomer PPZamestnance ON (PPZamestnance.RC = RZamestnance.RC AND PPZamestnance.OsobniCislo = RZamestnance.OsobniCislo AND PPZamestnance.DatumNastupu = RZamestnance.DatumNastupu)' .
            '               WHERE HierarchieRozdeleni.typNadrazene = PredavaciProces.KodTypuRozdeleni ';


        switch ($this->toDalsiUroven) {
            case 'osoba' :
                $sql .=
                    '       AND RZamestnance.RC = ' . $params->addParameter($this->xmlForm->rc) .
                    '       AND PPZamestnance.DatumNastupu <= ' . $params->addParameter($this->xmlForm->database->dateToParam(time())).
                    '       AND (PPZamestnance.DatumUkonceni >= ' . $params->addParameter($this->xmlForm->database->dateToParam(time())) . ' OR PPZamestnance.DatumUkonceni IS NULL)';
                break;
            case 'PP' :
                $sql .=
                    '       AND RZamestnance.RC = ' . $params->addParameter($this->xmlForm->pp_RC) .
                    '       AND RZamestnance.OsobniCislo = ' . $params->addParameter($this->xmlForm->osobniCislo) .
                    '       AND RZamestnance.DatumNastupu = ' . $params->addParameter($this->xmlForm->database->dateToParam($this->xmlForm->datumNastupu));
                break;
            case 'rozdeleni' :
                $sql .=
                    '       AND RZamestnance.KodTypuRozdeleni = ' . $params->addParameter($this->xmlForm->kodTypuRozdeleni) .
                    '       AND RZamestnance.Rozdeleni = ' . $params->addParameter($this->xmlForm->rozdeleni);
                break;
            default :
                return null;
        }

        $sql .=
            '                   AND RZamestnance.PlatnostOd <= ' . $params->addParameter($this->xmlForm->database->dateToParam(time())) .
            '                   AND (RZamestnance.PlatnostDo >= ' . $params->addParameter($this->xmlForm->database->dateToParam(time())) . ' OR RZamestnance.PlatnostDo IS NULL) ' .

            '                   AND ROsoby.RC = PPOsoby.RC ' .
            '                   AND ROsoby.OsobniCislo = PPOsoby.OsobniCislo ' .
            '                   AND ROsoby.DatumNastupu = PPOsoby.DatumNastupu ' .
            '                   AND ROsoby.PlatnostOd <= ' . $params->addParameter($this->xmlForm->database->dateToParam(time())) .
            '                   AND (ROsoby.PlatnostDo >= ' . $params->addParameter($this->xmlForm->database->dateToParam(time())) . ' OR ROsoby.PlatnostDo IS NULL) ' .
            '               ) ' .
            '        )' .
            '       ) ' .

            '   AND O.eMail IS NOT NULL ' .
            'GROUP BY O.Prijmeni, O.eMail, O.RC';

        $this->logSql = $sql . "\n" . $params->getLogText();

        $cursor = $this->xmlForm->database->getSqlResult($sql, $params);
        $address = '';
        $rc = null;

        while ($row = $cursor->fetch()) {
            if (!$rc) {
                $rc = $row[1];
            }
            $address .= ($address ? ';' : '') . $row[0];
        }

        if ($address) {
            return new XMLEmailReceiver($address, $rc);
        }

        return null;
    }


    private function getEmailAddresZastup($rc)
    {
        if (!$rc) {
            return null;
        }

        $pravoProZastup = $this ->nahradPromenne(StringUtils::fromUTF8($this ->getAttributeValue('pravoprozastup')));

        if (!$pravoProZastup) {
            return null;
        }

        return ZastupyUtils::vratEmaiZastupu($this ->xmlForm ->database, $rc, $pravoProZastup);
    }


    /**
     * @throws DBException
     * @throws Exception
     */
    private function sendEmail()
    {
        $to = $this->getReceiver($this->to, $this->toRC, $this->toIdPP);
        if (!$to) {
            $to = $this ->getReceiverPodleRozdeleni();
        }
        if (!$to) {
            $to = $this->getEmailAddressPodleProcesu();
        }

        $from = $this->getReceiver($this->from, $this->fromRC, $this->fromIdPP);
        $replyTo = $this->getReceiver($this->replyTo, $this->replyToRC, $this->replyToIdPP);
        $copyTo = $this->getReceiver($this->copyTo, $this->copyToRC, $this->copyToIdPP);

        $emailZastup = ($to &&  $to ->rc)
            ? $this ->getEmailAddresZastup($to ->rc)
            : null;

        $text = $this->nahradPromenne($this->text);
        $subject = $this->nahradPromenne($this->subject);

        $logText = sprintf(StringUtils::fromUTF8(
            "Příjemce: %s\n" .
            "Předmět: %s\n" .
            "Odesílatel: %s\n" .
            "Odpověď: %s\n" .
            "Kopie: %s\n" .
            "Zástup (kopie): %s\n" .
            "Text:\n%s"),
            $to ? $to ->address : '',
            $subject,
            $from ? $from ->address : '', $replyTo ? $replyTo ->address : '', $copyTo ? $copyTo ->address : '',
            $emailZastup ?: '',
            $text);

        // pokud neni adresa v to, tak pouzije copyTo
        if ((!$to  ||  !$to ->address)  &&  $copyTo) {
            $to = $copyTo;
            $copyTo = null;
        }

        // prida do copyTo adresu pripadneho zastupu
        $copyToAddr = ($copyTo && $copyTo ->address ? $copyTo ->address : '');
        if ($emailZastup) {
            $copyToAddr .= ($copyToAddr ? ';' : '') . $emailZastup;
        }

        $hodnota = 0;

        if ($to  &&  $to ->address) {
            $eMail = new EMail();
            $eMail->to = $to ->address;
            $eMail->subject = $subject;
            $eMail->text = $text;
            $eMail->from = $from ? $from ->address : null;
            $eMail->copyTo = $copyToAddr ?: null;
            $eMail->replyTo = $replyTo ? $replyTo ->address : null;

            foreach ($this ->elements as $element) {
                if ($element instanceof XmlElementEmailAttachment) {
                    $content = $element ->getContent();
                    if ($content) {
                        $fileName = $element ->getFileName();
                        $eMail ->addAttachment($fileName ?: 'attachment', $content);
                    }
                }
            }

            $OK = true;
            $errorMessage = null;

            try {
                MailManager::sendMail($eMail);
            } catch (Exception $e) {
                if (!$this->variableName || $this->getBoolAttribute("generateerror")) {
                    // chybu generuje jen v pripade, ze neni zadano variableName, nebo je to explicitne povoleno atributem generateerror
                    $this ->xmlForm ->addErrorMsg($e);
                }

                $errorMessage = $e->getMessage();
                $OK = false;
            }

            if ($OK) {
                $hodnota = 1;
            }

            if ($this->log === 2 || $this->log === 3 || ($this->log && !$OK)) {
                if ($this->log === 3) {
                    $logText .= "\n" . $this->logSql;
                }

                Log::addRecordToLog($this->xmlForm->database,
                    StringUtils::fromUTF8($OK ? 'Odeslání emailu proběhlo úspěšně' : 'Chyba při odesílání emailu'),
                    StringUtils::fromUTF8('XML formuláře'),
                    ($OK ? '' : "Chyba: $errorMessage\n\n") .
                    $logText
                );

                $this->xmlForm->addLogData(null, 'Email => ' . $to ->address . ': ' . $subject . ($OK ? ' - OK' : ' - chyba: ' . $errorMessage));
            }
        } else if ($this->log) {
            Log::addRecordToLog($this->xmlForm->database, StringUtils::fromUTF8('Chyba při odesílání emailu'),
                StringUtils::fromUTF8('XML formuláře'),
                StringUtils::fromUTF8("Není zadána adresa příjemce\n\n") . $logText);
            $this->xmlForm->addLogData(null, StringUtils::fromUTF8('Chyba při odesílání emailu - není zadána adresa příjemce'));
        }

        if ($this->variableName) {
            $this->setVariableValue($this->variableName, $hodnota);
        }
    }


    /**
     * @throws DBException
     */
    public function doCommand()
    {
        $this->sendEmail();
    }

}

/*
//DOC
PAGE email
<pre>
PRE
  <email subject="Email z XML formulare" variablename="jmenopromenne"
      to="emailova_adresa" torc="rc_osoby" toidpp="id_pracovniho_pomeru"
      totyprozdeleni="kod_typu_rozdeleni" torozdeleni="kod_rozdeleni" tovedoucirozdeleni="kod_rozdeleni"
      torozdeleniosoby="id_pracovniho_pomeru" tovedouciosoby="id_pracovniho_pomeru"
      todalsiuroven=""

      from="emailova_adresa_odesilatele" fromrc="rc_odesilatele" fromidpp="id_pp_odesilatele"
      copyto="emailova_adresa" copytorc="rc_dalsiho_prijemce" copytoidpp="id_pp_dalsiho_prijemce"
      replyto="emailova_adresa" replytorc="rc_odesilatele_pro_odpoved" replytoidpp="id_pp_odesilatele_pro_odpoved"
      pravoprozastup="kod_typu_prava"
      log="0" generateerror="1"
  >
  Text emailu ...
    <emailattachment filename="priloha.pdf">obsah prilohy ...</emailattachment>
  </email>
PREEND
</pre>
//END
*/
