<?php
/**
 * User: stanislav.elbl
 */

namespace RonPortal\XML\Classes;

use Ron\MessageManager;
use Ron\StringUtils;
use RonPortal\XML\XMLFormElement;

/**
 * Implementace xml tagu <forsql>
 * Class XMLFormElementForSql
 * @package RonPortal\XML\Classes
 */
class XMLElementForSql extends XMLFormElement
{
    protected $name;

    protected $expand;
    protected $sqlText;
    protected $sqlTextVariable;


    function loadXMLElement($xmlReader)
    {
        parent::loadXMLElement($xmlReader);

        $this->name = $this->getAttributeValue('name');
        $this->expand = self::getBoolValue($this->getAttributeValue('expand'));

        $this->sqlText = StringUtils::fromUTF8($this->getAttributeValue('sqltext'));
        $this->sqlTextVariable = $this->getAttributeValue('sqltextvariable');
    }


    private function getCursor()
    {
        $sql = $this->sqlTextVariable
            ? $this->variableValue($this->sqlTextVariable)
            : $this->sqlText;

        $sql = $this->nahradPromenneSQL($sql);

        MessageManager::addDebugLog("XMLDebug> forsql: $sql");

        return $this->xmlForm->database->getSqlResult($sql);
    }


    function getHTMLOutput()
    {
        $cursor = $this->getCursor();
        $result = '';

        $index = 0;

        while ($row = $cursor->fetch()) {
            // nastavi promenne
            foreach ($row as $fieldindex => $value) {
                if ($this->xmlForm->verze > 1) {
                    $this->setVariableValue($this->name . '_index', $index);
                    $this->setVariableValue($this->name, $value, $fieldindex);
                } else {
                    $this->setVariableValue("{$this->name}[$fieldindex]", $value);
                }
            }

            $result .= $this->getElementsHTMLOutput();

            $index++;
        }

        $cursor ->close();

        return $result;
    }


    function event()
    {
        $cursor = $this->getCursor();

        $index = 0;

        while ($row = $cursor->fetch()) {
            // nastavi promenne
            foreach ($row as $fieldIndex => $value) {
                if ($this->xmlForm->verze > 1) {
                    $this->setVariableValue($this->name . '_index', $index);
                    $this->setVariableValue($this->name, $value, $fieldIndex);
                } else {
                    $this->setVariableValue("{$this->name}[$fieldIndex]", $value);
                }
            }

            $this->elementsEvent();

            $index++;
        }

        $cursor ->close();
    }


    function writeXMLOutput($xmlWriter)
    {
        if (!$this ->expand) {
            $this->copyXML($xmlWriter);

            // neni expand - provede se jen kdyz je nastaveno processonfill
            if (!self::getBoolValue($this ->getAttributeValue('processonfill'))) return;
        }

        $cursor = $this->getCursor();
        while ($row = $cursor->fetch()) {
            // nastavi promenne
            foreach ($row as $fieldindex => $value) {
                if ($this->xmlForm->verze > 1) {
                    $this->setVariableValue($this->name, $value, $fieldindex);
                    if ($this ->expand) {
                        $xmlWriter ->startElement('variable');
                        $xmlWriter ->writeAttribute('name', $this ->name);
                        $xmlWriter ->writeAttribute('index', $fieldindex);
                        $xmlWriter ->writeAttribute('value', StringUtils::toUTF8($value));
                        $xmlWriter ->endElement();
                    }
                } else {
                    $this->setVariableValue("{$this->name}[$fieldindex]", $value);
                    if ($this ->expand) {
                        $xmlWriter->startElement('variable');
                        $xmlWriter->writeAttribute('name', "{$this->name}[$fieldindex]");
                        $xmlWriter->writeAttribute('value', StringUtils::toUTF8($value));
                        $xmlWriter->endElement();
                    }
                }
            }

            // provede vnorene prikazy
            if ($this ->expand) {
                $this ->writeXMLElements($xmlWriter);
            } else {
                parent::process();
            }
        }

        $cursor ->close();
    }


    function process()
    {
        if ($this ->expand) {
            throw new \Exception('XMLFormElementForSQL::process - parametr expand neni implementovan!');
        }

        $index = 0;

        $cursor = $this->getCursor();
        while ($row = $cursor->fetch()) {
            // nastavi promenne
            foreach ($row as $fieldindex => $value) {
                if ($this->xmlForm->verze > 1) {
                    $this->setVariableValue($this->name . '_index', $index);
                    $this->setVariableValue($this->name, $value, $fieldindex);
                } else {
                    $this->setVariableValue("{$this->name}[$fieldindex]", $value);
                }
            }

            $this ->processElements();
            //parent::process();

            $index++;
        }

        $cursor ->close();
    }
}


/*
//DOC
PAGE forsql
<pre>
PRE
  <forsql name="jmeno_promennych" sqltext="SELECT ... " sqltextvariable="sql_promenna">
    ... obsah ...
  </forsql>
PREEND
</pre>

//END
*/
