<?php
/**
 * User: stanislav.elbl
 */

namespace RonPortal\XML\Classes;

use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\XML\XMLFormElement;

class XMLElementVyberObdobi extends XMLFormElement
{
    /** @var string */
    protected $name;
    /** @var bool */
    protected $required;
    /** @var string */
    protected $aplikace;
    /** @var bool */
    protected $jenNeuzavrene;
    /** @var bool */
    protected $jenUkoncene;
    /** @var bool */
    protected $variabilni;
    /** @var string */
    protected $sqlFilter;
    /** @var bool */
    protected $editing;
    /** @var int */
    protected $value;


    protected function loadXMLElement($xmlReader)
    {
        parent::loadXMLElement($xmlReader);

        $this ->name = $this ->getAttributeValue('name');
        $this ->required = self::getBoolValue($this ->getAttributeValue('required'), true); // defaultne je true
        $this ->aplikace = $this ->getAttributeValue('aplikace');
        $this ->jenNeuzavrene = self::getBoolValue($this ->getAttributeValue('jenneuzavrene'));
        $this ->jenUkoncene = self::getBoolValue($this ->getAttributeValue('jenukoncene'));
        $this ->variabilni = self::getBoolValue($this ->getAttributeValue('variabilni'));
        $this ->editing = self::getBoolValue($this ->getAttributeValue('editing'));
        $this ->value = $this ->getAttributeValue('value');

        $this ->sqlFilter = StringUtils::fromUTF8($this ->loadXMLValue($xmlReader) );
    }


    function getHTMLOutput()
    {
        $params = $this ->xmlForm ->database ->createParameters();

        $sqlFilter = trim($this->nahradPromenneSQL($this->sqlFilter));

        if ($this ->variabilni) {
            $filter = '';
            if ($this ->jenUkoncene) {
                $filter = 'KonecObdobi < ' . $params ->addParameter($this ->xmlForm ->database ->dateToParam(time()));
            }
            if ($sqlFilter) {
                $filter .=
                    ($filter ? ' AND ' : '') .
                    " ({$sqlFilter}) ";
            }

            $sql =
                'SELECT PocatekObdobi, PopisObdobi, KonecObdobi FROM VariantniObdobi ' .
                ($filter ? ' WHERE ' . $filter : '') .
                ' ORDER BY PocatekObdobi DESC ';
        } else {
            $sql =
                'SELECT PocatekObdobi, Popis, KonecObdobi, Uzavreno FROM Obdobi ' .
                ' WHERE Aplikace = ' . $params ->addParameter($this ->aplikace) .
                ($this ->jenNeuzavrene
                    ? ' AND Uzavreno = 0 '
                    : ''
                ) .
                ($this ->jenUkoncene
                    ? ' AND KonecObdobi < ' . $params ->addParameter($this ->xmlForm ->database ->dateToParam(time()))
                    : ''
                ) .
                ($sqlFilter
                    ? " AND  ({$sqlFilter}) "
                    : ''
                ) .
                ' ORDER BY PocatekObdobi DESC ';
        }

        if ($this ->xmlForm ->usePostValues) {
            // hodnota z POST
            $value = $_POST[$this->name];
        } else {
            // defaultni hodnota
            $value = null;

            if ($this ->xmlForm ->dataMode()) {
                // ... nebo hodnota zadana uz drive
                $value = $this ->variableValue($this ->name);
            }

            if (!isset($value)) {
                $value = $this ->nahradPromenne(StringUtils::fromUTF8($this->value)); // umoznuje zadat defaultni hodnotu
            }
        }

        $this ->xmlForm ->addDebugMessage('Vyberobdobi, sql: ' . $sql, true);

        $cursor = $this ->xmlForm ->database ->getSqlResult($sql, $params);

        $attrs = ['name' => $this ->name];
        $attrs = $this->setHTMLAttributes($attrs);
        if ($this ->xmlForm ->isError($this ->name)) {
            if (isset($attrs['class'])) {
                $attrs['class'] .= ' dotaznik_errinput';
            } else {
                $attrs['class'] = 'dotaznik_errinput';
            }
        }

        $attrs_str = $this->getHTMLAttributes($attrs);

        $result = '<SELECT ' . $attrs_str . '>';

        if (!$value  ||  !$this ->required) { // pokud jeste nebyla zadana hodnota, prida prazdnou hodnotu  -  pridat prazdny radek, kdyz pole neni povinne? ? ?
            $selectedStr = $value ? '' : 'selected="selected"';
            $result .= "<OPTION value='' $selectedStr></OPTION> ";

            $this ->xmlForm ->nastavObdobi($this ->name, null, null, null, null);
        }

        while ($row = $cursor ->fetch()) {
            $pocatek = $cursor ->stringToTimestamp($row[0]);

            $popis = $row[1];
            if (!$this ->variabilni) {
                $popisT = TextManager::_('db_obdobi_popis_' . date('n', $pocatek), '');
                if ($popisT) {
                    $popis = sprintf($popisT, date('Y', $pocatek));
                }
            }

            if ($pocatek == $value) {
                $selectedStr = 'selected="selected"';
                // nastavi prislusne promenne
                if ($this ->variabilni) {
                    $this ->xmlForm ->nastavObdobi($this ->name, $pocatek, $cursor->stringToTimestamp($row[2]), $popis);
                } else {
                    $this ->xmlForm ->nastavObdobi($this ->name, $pocatek, $cursor->stringToTimestamp($row[2]), $popis, (int)$row[3]);
                }
            } else {
                $selectedStr = '';
            }

            $result .= "<OPTION value='{$pocatek}' $selectedStr>" . StringUtils::htmlTextConvert($popis) . "</OPTION>";
        }

        $cursor ->close();

        $result .= '</SELECT>';

        return $result;
    }


    function writeXMLOutput($xmlWriter)
    {
        throw new \Exception('XMLElementVyberObdobi::writeXMLOutput neni implementovano!');
        //parent::writeXMLOutput($xmlWriter);
    }


    function process()
    {
        if ( $this ->required  &&  !$_POST[$this ->name] ) {
            $this ->xmlForm -> addErrorMsg( \Ron\ronText('Chybi_povinna_hodnota'), $this ->name );
        }

        $oldValue = $this ->xmlForm ->getVariableValue($this ->name);
        if (!isset($oldValue)) {
            $oldValue = $this ->value;
        }

        $this ->xmlForm -> nastavObdobiAplikace($this ->name, $this ->aplikace, $_POST[$this ->name], $this ->variabilni);

        if ($oldValue != $_POST[$this ->name]) {
            $this ->xmlForm ->addLogData($this ->name, $_POST[$this ->name] . ' (' . $this ->variableValue($this ->name . '_popis') . ')');
        }

        $this ->xmlForm ->addDataVariable($this ->name);
        $this ->xmlForm ->addDataVariable($this ->name . '_pocatek');
        $this ->xmlForm ->addDataVariable($this ->name . '_pocatekstr');
        $this ->xmlForm ->addDataVariable($this ->name . '_konec');
        $this ->xmlForm ->addDataVariable($this ->name . '_konecstr');
        $this ->xmlForm ->addDataVariable($this ->name . '_popis');
    }
}

/*
//DOC
PAGE vyberobdobi
<pre>
PRE
  <vyberobdobi name="OBDOBI" aplikace="ADS" variabilni="0" required="0" jenneuzavrene="0" jenukoncene="0" editing="0" value="prednastavena_hodnota">SQL filtr</vyberobdobi>
PREEND
</pre>
//END
*/
