<?php
/**
 * stanislav.elbl, 02.06.2021 16:24
 */

namespace RonPortal\XML;

use Ron\DB\Database;
use Ron\MessageManager;
use Ron\MVC\BaseViewObject;
use Ron\MVC\Model;
use Ron\MVC\ViewObject;
use RonPortal\Controller\PasswordProtectionController;
use RonPortal\RonPortalSession;


class XMLFormViewObject implements ViewObject
{
    /** @var  RonPortalXMLForm */
    protected $xmlForm;
    protected $html;


    /**
     * XMLFormViewObject constructor.
     * @param RonPortalXMLForm $xmlForm
     */
    public function __construct($xmlForm) // , $xmlId, $zeSablony = true
    {
//        $this->database = $database;
//        $this->xmlId = $xmlId;
//        $this->zeSablony = $zeSablony;
        $this ->xmlForm = $xmlForm;
    }


    public function loadHtml()
    {
        $this ->html = $this ->xmlForm ->getHtml();
    }


    public static function zobrazXml($database, $id, $zeSablony)
    {
        $xmlForm = new RonPortalXMLForm($database);

        global $xmlform;
        $xmlform = $xmlForm;

        $xmlForm ->nactiXMLFormular($id, $zeSablony); // zeSablony = true

        return static::showXmlForm($xmlForm);
    }

    public static function zobrazXmlSablonu($database, $id)
    {
        return self::zobrazXml($database, $id, true);
    }


    public static function zobrazVyplnenyXml($database, $id)
    {
        return self::zobrazXml($database, $id, false);
    }


    /**
     * @param RonPortalXMLForm $xmlForm
     * @return XMLFormViewObject|BaseViewObject|string
     * @throws \Exception
     */
    public static function showXmlForm($xmlForm)
    {
        $result = new static($xmlForm);
        $result ->loadHtml();

        if ($xmlForm ->forbiddenMode()) {
            throw new \Exception('Tento formular neni mozne samostatne zobrazit (mod ' . $xmlForm ->mode . ')');
        }

        if ($xmlForm ->passwordProtected  &&  !RonPortalSession::enablePasswordProtected()) {
            // formular je chraneny heslem
            return PasswordProtectionController::passwordDialog();
        }

        if ($result ->xmlForm ->outputObject) {
            // xml formular ma nastaven outputObject (napriklad tagem <download>)
            return $result ->xmlForm ->outputObject;
        }

        if ($result ->xmlForm ->viewFileName) {
            Model::addViewObject($result);
            return $result ->xmlForm ->viewFileName;
        }

        return $result;
    }



    public function render()
    {
        if ($this ->xmlForm ->showHeader) {
            echo "<h1>{$this ->xmlForm ->popis}</h1>"; // ne htmlTextConvert!
        }

        if ($this ->xmlForm ->ronPortalFilterForm) {
            $this ->xmlForm ->ronPortalFilterForm ->render();
        }

        if ($this ->xmlForm ->showErrorMessages) {
            MessageManager::getInstance() ->showMessages();
        }

        if ($this ->xmlForm ->viewObjects) {
            // vypise vystup vcetne viewobjectu. Misto viewobjectu je ve vystupu placeholder (defaultne $VIEW$)
            $lastPos = 0;

            foreach ($this ->xmlForm ->viewObjects as $viewObject) {
                $pos = strpos($this ->html, $this ->xmlForm ->viewObjectPlaceholder, $lastPos);
                if ($pos !== false) {
                    // nasel misto, kde se ma viewobject zobrazit

                    // vypise text pred viewobjectem
                    echo substr($this ->html, $lastPos, $pos - $lastPos);
                    $viewObject ->render();

                    $lastPos = $pos + strlen($this ->xmlForm ->viewObjectPlaceholder);
                }
            }

            echo substr($this ->html, $lastPos);
        } else {
            echo $this->html;
        }
    }

}
