<?php
/**
 * stanislav.elbl, 11.01.2018
 */
namespace Ron\RonPortal;

use Ron\MessageManager;
use Ron\MVC\Model;
use Ron\StringUtils;
use RonPortal\Admin\LDAPTestController;

function getValue($valueName, $defaultValue)
{
    return isset($_POST[$valueName])
        ? $_POST[$valueName]
        : (isset($_SESSION['ldaptest'])
            ? $_SESSION['ldaptest'][$valueName]
            : $defaultValue);
}


global $ldap_server;
global $ldap_nameprefix;
global $ldap_namesuffix;
global $ldap_options;
global $ldap_tls;

$ldapEntry = Model::getInstance() ->getData('ldapEntry');
$ldapData = Model::getInstance() ->getData('ldapData');

?>
<h1>Test LDAP</h1>
<form method="POST">
    <div><?php MessageManager::getInstance() ->showMessages(); ?></div>

    <div>Server</div>
    <div><input name="server" size="60" value="<?= StringUtils::htmlTextConvert(getValue('server', $ldap_server)) ?>"></div>
    <div class="poznamka">IP adresa nebo jmeno LDAP serveru.</div>
    <div><input type="checkbox" name="tls" <?= getValue('tls', $ldap_tls) ? 'checked="checked"' : '' ?>> TLS</div>
    <br/>

    <div>Uzivatelske jmeno - prefix</div>
    <div><input name="prefix" size="60" value="<?= StringUtils::htmlTextConvert(getValue('prefix', $ldap_nameprefix)) ?>"></div>
    <div>Uzivatelske jmeno</div>
    <div><input name="username" size="60" value="<?= StringUtils::htmlTextConvert(getValue('username', '')) ?>"></div>
    <div>Uzivatelske jmeno - sufix</div>
    <div><input name="suffix" size="60" value="<?= StringUtils::htmlTextConvert(getValue('suffix', $ldap_namesuffix)) ?>"></div>
    <div class="poznamka">Prefix, jmeno a suffix se spoji do jednoho retezce.</div>
    <br/>

    <div>Testovaci Distinguished Name k vyhledani</div>
    <div><input name="dn" size="60" value="<?= StringUtils::htmlTextConvert(getValue('dn', '')) ?>"></div>
    <div class="poznamka">Pokusi se vyhledat informace k zadane polozce, napriklad</div>
    <div class="poznamka">ou=ron,dc=ron,dc=local</div>
    <div class="poznamka">POZOR! V AD muze byt cn (common name) jine nez prihlasovaci jmeno!</div>
    <div>Vyhledavaci filtr</div>
    <div><input name="filter" size="60" value="<?= StringUtils::htmlTextConvert(getValue('filter', '')) ?>"></div>
    <div class="poznamka">Vyzkousejte napriklad <tt>(objectclass=*)</tt> nebo <tt>(sAMAccountName=user.login.name)</tt></div>
    <div>Atributy k vyhledani</div>
    <div><input name="attributes" size="60" value="<?= StringUtils::htmlTextConvert(getValue('attributes', '')) ?>"></div>
    <div class="poznamka">Vyzkousejte napriklad <tt>cn,samaccountname</tt></div>
    <br/>

    <div>Heslo</div>
    <div><input name="password" type="password" size="10" value="<?= StringUtils::htmlTextConvert(getValue('password', '')) ?>"></div>
    <div class="poznamka">Zadani hesla neni povinne. Pokud nebude zadano, otestuje se jen pripojeni k serveru.</div>
    <br/>

    <?php if ($ldap_options) { ?>
        <div>LDAP Options</div>
        <div>
            <?php foreach ($ldap_options as $optionName => $optionValue) { ?>
                <div><?= StringUtils::htmlTextConvert(LDAPTestController::getOptionName($optionName)) ?> = <?= StringUtils::htmlTextConvert($optionValue) ?></div>
            <?php } ?>
        </div>
    <?php } ?>

    <div><input type="submit" class="submitbutton" value="Test"></div>
</form>

<?php if ($ldapData) { ?>
    <table class="browser oddEvenRows fixedCols">
    <?php //if (count($ldapData['items']) > count($ldapData['data'])) { ?>
        <?php foreach ($ldapData['items'] as $item) { ?>
            <tr>
                <td><?= StringUtils::htmlTextConvert($item) ?></td>
            <?php foreach ($ldapData['data'] as $object) { ?>
                <td><?php var_dump($object[$item]) ?></td>
            <?php } ?>
            </tr>
        <?php } ?>
    <?php //} else { ?>
    <?php //} ?>
    </table>
<?php } ?>

<?php if ($ldapEntry) { ?>
    <pre>
    <?php var_dump($ldapEntry); ?>
    </pre>
<?php } ?>